// ============================================================================
// Project:	Deductive Database
// Filename:	argtype.h
// Purpose:	Argument types for predicates (int, atom, string)
// Last Change:	16.11.2015
// Language:	C++
// EMail:	brass@informatik.uni-halle.de
// WWW:		http://www.informatik.uni-halle.de/~brass/
// Address:	Feldschloesschen 15, D-06120 Halle (Saale), GERMANY
// Copyright:	(c) 1998-2015 by Stefan Brass
// License:	See file "LICENSE" for copying conditions.
// Note:	There is no warranty at all - this code may contain bugs.
// ============================================================================


//=============================================================================
// Include File Frame:
//=============================================================================

#ifndef	ARGTYPE_INCLUDED
#define ARGTYPE_INCLUDED

//=============================================================================
// Used Types and Macros:
//=============================================================================

#ifndef VER_INCLUDED
#include "ver.h"
#endif

#ifndef STR_INCLUDED
#include "str.h"
#endif

#ifndef CHECK_INCLUDED
#include "check.h"
#endif

//=============================================================================
// Exported Types:
//=============================================================================

//-----------------------------------------------------------------------------
// argtype_t: Argument types for predicates.
//-----------------------------------------------------------------------------

typedef enum {
	ARGTYPE_NULL,
	ARGTYPE_INT,
	ARGTYPE_ATOM,
	ARGTYPE_STR,
	ARGTYPE_ATOM_STR
} argtype_t;

//=============================================================================
// Exported Functions:
//=============================================================================

//-----------------------------------------------------------------------------
// argtype_valid: Is the parameter a valid element of the enumeration type?
//-----------------------------------------------------------------------------

#ifdef	VER_DEBUG
bool	argtype_valid(argtype_t argtype);
#endif

//-----------------------------------------------------------------------------
// argtype_name: String representation of this binding pattern.
//-----------------------------------------------------------------------------

str_t	argtype_name(argtype_t argtype);

//=============================================================================
// End of File:
//=============================================================================

#endif

