// ============================================================================
// Project:	Deductive Database
// Filename:	bench_dblp_1.h
// Purpose:	Bottom-Up Evaluation for DBLP Example
// Last Change:	10.09.2015
// Language:	C++
// EMail:	brass@informatik.uni-halle.de
// WWW:		http://www.informatik.uni-halle.de/~brass/
// Address:	Feldschloesschen 15, D-06120 Halle (Saale), GERMANY
// Copyright:	(c) 2015 by Stefan Brass
// License:	See file "LICENSE" for copying conditions.
// Note:	There is no warranty at all - this code may contain bugs.
// ============================================================================


//=============================================================================
// Include File Frame:
//=============================================================================

#ifndef	BENCH_DBLP_1_INCLUDED
#define BENCH_DBLP_1_INCLUDED

//=============================================================================
// Used Types and Macros:
//=============================================================================

#ifndef VER_INCLUDED
#include "ver.h"
#endif

#ifndef STR_INCLUDED
#include "str.h"
#endif

#ifndef CHECK_INCLUDED
#include "check.h"
#endif

#ifndef PERF_INCLUDED
#include "perf.h"
#endif

//=============================================================================
// Bottom-Up Evalutation (DBLP Benchmark) - Class With Only Static Members:
//=============================================================================

class bench_dblp_1_c {
	public:

//-----------------------------------------------------------------------------
// Static Methods:
//-----------------------------------------------------------------------------

	// run: Load data file, do bottom-up evaluation.
	static long run(str_t filename, perf_t timer_load, perf_t timer_eval);

	// name: Name of this benchmark.
	static str_t name() {
		return
		"DBLP, Ver. 1: Relations in local vars, hand-crafted loader";
	}

	// file: List of data files.
	static str_t file(int i);

	// result: List of correct results of eval method.
	static long result(int i);


//-----------------------------------------------------------------------------
// This class has no objects:
//-----------------------------------------------------------------------------

	private:

	// Constructor:
	bench_dblp_1_c();	// Not implemented

	// Destructor:
	~bench_dblp_1_c();	// Not implemented

//-----------------------------------------------------------------------------
// Copy-constructor and assignment operator are not supported for this class:
//-----------------------------------------------------------------------------

	private:

	bench_dblp_1_c(const bench_dblp_1_c& obj);	// Not implemented
	bench_dblp_1_c& operator=(const bench_dblp_1_c& obj);// Not implemented


//=============================================================================
// End of Class:
//=============================================================================

};


//=============================================================================
// End of Include File:
//=============================================================================

#endif

