// ============================================================================
// Project:	Deductive Database
// Filename:	bench_dblp_3.h
// Purpose:	Bottom-Up Evaluation for DBLP Example, Ver 3: Separate Objects
// Last Change:	11.06.2016
// Language:	C++
// EMail:	brass@informatik.uni-halle.de
// WWW:		http://www.informatik.uni-halle.de/~brass/
// Address:	Feldschloesschen 15, D-06120 Halle (Saale), GERMANY
// Copyright:	(c) 2015-2016 by Stefan Brass
// License:	See file "LICENSE" for copying conditions.
// Note:	There is no warranty at all - this code may contain bugs.
// ============================================================================


//=============================================================================
// Include File Frame:
//=============================================================================

#ifndef	BENCH_DBLP_3_INCLUDED
#define BENCH_DBLP_3_INCLUDED

//=============================================================================
// Used Types and Macros:
//=============================================================================

#ifndef VER_INCLUDED
#include "ver.h"
#endif

#ifndef STR_INCLUDED
#include "str.h"
#endif

#ifndef CHECK_INCLUDED
#include "check.h"
#endif

#ifndef DB_DBLP_INCLUDED
#include "db_dblp.h"
#endif

#ifndef EVAL_DBLP_INCLUDED
#include "eval_dblp.h"
#endif

#ifndef BENCH_INCLUDED
#include "bench.h"
#endif

//=============================================================================
// Private Constants:
//=============================================================================

//-----------------------------------------------------------------------------
// BENCH_DBLP_3_MAGIC: Magic number (identifies objects of this class).
//-----------------------------------------------------------------------------

static const long BENCH_DBLP_3_MAGIC = 0x4450330AL;	// 'DP3\n'


//=============================================================================
// Bottom-Up Evalutation (DBLP Benchmark) - Ver. 3: Separate Objects
//=============================================================================

class bench_dblp_3_c : public bench_c {
	public:

//-----------------------------------------------------------------------------
// Constructor, Destructor:
//-----------------------------------------------------------------------------

	public:

	// Constructor:
	bench_dblp_3_c();

	// Destructor:
	~bench_dblp_3_c();

//-----------------------------------------------------------------------------
// (Object) Methods:
//-----------------------------------------------------------------------------

	// load: Load data file with EDB facts.
	bool load(str_t filename);

	// eval: Do bottom-up evaluation.
	long eval();

	// name: Name of this benchmark.
	str_t name() const {
		return "DBLP Bench., Ver.2: Rel/Var in Attrs, Cursor Interface";
	}

	// file: List of data files.
	str_t file(int i) const;

	// result: List of correct results of eval method.
	long result(int i) const;


//-----------------------------------------------------------------------------
// Copy-constructor and assignment operator are not supported for this class:
//-----------------------------------------------------------------------------

	private:

	// These methods are not implemented:
	bench_dblp_3_c(const bench_dblp_3_c& obj);
	bench_dblp_3_c& operator=(const bench_dblp_3_c& obj);


//-----------------------------------------------------------------------------
// Debugging Support:
//-----------------------------------------------------------------------------

#ifdef	VER_DEBUG
	// Integrity check:
	public:
	str_t	check() const;

	// Magic number (identifies objects of this class for debugging).
	private:
	long	Magic;		// Must be "BENCH_DBLP_3_MAGIC".
#endif


#ifdef	VER_DUMP
	public:
	// Display data structure:
	void dump(str_t headline = STR_NULL) const;
#endif

//-----------------------------------------------------------------------------
// Private Object Members:
//-----------------------------------------------------------------------------

	private:

//-----------------------------------------------------------------------------
// Database for EDB Predicates:
//-----------------------------------------------------------------------------

	db_dblp_c DB;

//-----------------------------------------------------------------------------
// Evaluator with Cursor Interface for Query Result:
//-----------------------------------------------------------------------------

	eval_dblp_c Query;

//=============================================================================
// End of Class:
//=============================================================================

};


//=============================================================================
// End of Include File:
//=============================================================================

#endif

