// ============================================================================
// Project:	Deductive Database
// Filename:	bench_j1axy_4.h
// Purpose:	Bottom-Up Evaluation for Join1, a(X,Y), V4: with duplicate test
// Last Change:	18.05.2016
// Language:	C++
// EMail:	brass@informatik.uni-halle.de
// WWW:		http://www.informatik.uni-halle.de/~brass/
// Address:	Feldschloesschen 15, D-06120 Halle (Saale), GERMANY
// Copyright:	(c) 2015-2016 by Stefan Brass
// License:	See file "LICENSE" for copying conditions.
// Note:	There is no warranty at all - this code may contain bugs.
// ============================================================================

// Note: Duplicate test only for b1, b2, c1

//=============================================================================
// Include File Frame:
//=============================================================================

#ifndef	BENCH_J1AXY_4_INCLUDED
#define BENCH_J1AXY_4_INCLUDED

//=============================================================================
// Used Types and Macros:
//=============================================================================

#ifndef VER_INCLUDED
#include "ver.h"
#endif

#ifndef STR_INCLUDED
#include "str.h"
#endif

#ifndef CHECK_INCLUDED
#include "check.h"
#endif

#ifndef STRTAB_INCLUDED
#include "strtab.h"
#endif

#ifndef LIST_1_INCLUDED
#include "list_1.h"
#endif

#ifndef LIST_2_INCLUDED
#include "list_2.h"
#endif

#ifndef SET_1_INCLUDED
#include "set_1.h"
#endif

#ifndef REL_N_N_INCLUDED
#include "rel_n_n.h"
#endif

#ifndef BENCH_INCLUDED
#include "bench.h"
#endif


//=============================================================================
// Private Constants:
//=============================================================================

//-----------------------------------------------------------------------------
// BENCH_J1AXY_4_MAGIC: Magic number (identifies objects of this class).
//-----------------------------------------------------------------------------

static const long BENCH_J1AXY_4_MAGIC = 0x4A31410AL;	// 'J1A\n'


//=============================================================================
// Bottom-Up Evaluation (Join 1 Benchmark: a(X,Y), with duplicate test):
//=============================================================================

class bench_j1axy_4_c : public bench_c {
	public:


//-----------------------------------------------------------------------------
// Constructor, Destructor:
//-----------------------------------------------------------------------------

	public:

	// Constructor:
	bench_j1axy_4_c();

	// Destructor:
	~bench_j1axy_4_c();

//-----------------------------------------------------------------------------
// (Object) Methods:
//-----------------------------------------------------------------------------

	// load: Load data file with EDB facts.
	bool load(str_t filename);

	// Do bottom-up evaluation:
	long eval();
	// Note: This returns number of results in millions
	// (otherwise 32bit int overflow)


	// name: Name of this benchmark.
	str_t name() const {
		return
		"Join 1, a(X,Y), Ver. 4: Rel.Attrs, dupcheck for b1,b2,c1";
	}

	// file: List of data files.
	str_t file(int i) const;

	// result: List of correct results of eval method.
	long result(int i) const;

//-----------------------------------------------------------------------------
// Copy-constructor and assignment operator are not supported for this class:
//-----------------------------------------------------------------------------

	private:

	// These methods are not implemented:
	bench_j1axy_4_c(const bench_j1axy_4_c& obj);
	bench_j1axy_4_c& operator=(const bench_j1axy_4_c& obj);


//-----------------------------------------------------------------------------
// Debugging Support:
//-----------------------------------------------------------------------------

#ifdef	VER_DEBUG
	// Integrity check:
	public:
	str_t	check() const;

	// Magic number (identifies objects of this class for debugging).
	private:
	long	Magic;		// Must be "BENCH_J1AXY_4_MAGIC".
#endif


#ifdef	VER_DUMP
	public:
	// Display data structure:
	void dump(str_t headline = STR_NULL) const;
#endif

//-----------------------------------------------------------------------------
// Private Object Members:
//-----------------------------------------------------------------------------

	private:

//-----------------------------------------------------------------------------
// Data storage for EDB relations:
//-----------------------------------------------------------------------------

	rel_n_n_c c2_bf;
	list_2_c c3_ff;
	rel_n_n_c c4_bf;
	list_2_c d1_ff;
	rel_n_n_c d2_bf;

//=============================================================================
// End of Class:
//=============================================================================

};


//=============================================================================
// End of Include File:
//=============================================================================

#endif

