// ============================================================================
// Project:	Deductive Database
// Filename:	bench_j1axy_8.h
// Purpose:	Benchmark Join1, a(X,Y), Ver. 8: functions, bitmap dupcheck
// Last Change:	27.08.2016
// Language:	C++
// EMail:	brass@informatik.uni-halle.de
// WWW:		http://www.informatik.uni-halle.de/~brass/
// Address:	Feldschloesschen 15, D-06120 Halle (Saale), GERMANY
// Copyright:	(c) 2015-2016 by Stefan Brass
// License:	See file "LICENSE" for copying conditions.
// Note:	There is no warranty at all - this code may contain bugs.
// ============================================================================


//=============================================================================
// Include File Frame:
//=============================================================================

#ifndef	BENCH_J1AXY_8_INCLUDED
#define BENCH_J1AXY_8_INCLUDED

//=============================================================================
// Used Types and Macros:
//=============================================================================

#ifndef VER_INCLUDED
#include "ver.h"
#endif

#ifndef STR_INCLUDED
#include "str.h"
#endif

#ifndef CHECK_INCLUDED
#include "check.h"
#endif

#ifndef STRTAB_INCLUDED
#include "strtab.h"
#endif

#ifndef LIST_1_INCLUDED
#include "list_1.h"
#endif

#ifndef LIST_2_INCLUDED
#include "list_2.h"
#endif

#ifndef SET_TT_INCLUDED
#include "set_tt.h"
#endif

#ifndef REL_N_N_INCLUDED
#include "rel_n_n.h"
#endif

#ifndef OUT_INCLUDED
#include "out.h"
#endif

#ifndef BENCH_INCLUDED
#include "bench.h"
#endif


//=============================================================================
// Private Constants:
//=============================================================================

//-----------------------------------------------------------------------------
// BENCH_J1AXY_8_MAGIC: Magic number (identifies objects of this class).
//-----------------------------------------------------------------------------

static const long BENCH_J1AXY_8_MAGIC = 0x4A31410AL;	// 'J1A\n'


//=============================================================================
// Benchmark Join1, a(X,Y), Ver. 7: goto, duplicate check, cursor interface
//=============================================================================

class bench_j1axy_8_c : public bench_c {
	public:


//-----------------------------------------------------------------------------
// Constructor, Destructor:
//-----------------------------------------------------------------------------

	public:

	// Constructor:
	bench_j1axy_8_c();

	// Destructor:
	~bench_j1axy_8_c();

//-----------------------------------------------------------------------------
// (Object) Methods:
//-----------------------------------------------------------------------------

	// load: Load data file with EDB facts.
	bool load(str_t filename);

	// eval: Do bottom-up evaluation.
	long eval();

	// name: Name of this benchmark.
	str_t name() const {
		return "Join 1, a(X,Y), Ver. 8: functions, bitmap dupcheck";
	}

	// file: List of data files.
	str_t file(int i) const;

	// result: List of correct results of the eval method.
	long result(int i) const;

	// Open cursor (note: load() must be called before this method).
	void open();

	// fetch: Get next tuple, returns false if there is no further tuple.
	inline bool fetch() {
		return result_cursor.fetch();
	}


	// c1: Return value for column 1 of current row in cursor.
	inline int c1() {
		return result_cursor.c1();
	}

	// c2: Return value for column 2 of current row in cursor.
	inline int c2() {
		return result_cursor.c2();
	}

	// error: Did an error occur?
	inline bool error() {
		// EDB relations:
		if(c2_bf.mem_err())
			return true;
		if(c3_ff.mem_err())
			return true;
		if(c4_bf.mem_err())
			return true;
		if(d1_ff.mem_err())
			return true;
		if(d2_bf.mem_err())
			return true;

		// Derived b1 tuples (for rule with two IDB lit):
		if(b1_fb.mem_err())
			return true;

		// Sets for duplicate checks:
		if(b1_bb.mem_err())
			return true;
		if(b2_bb.mem_err())
			return true;
		if(c1_bb.mem_err())
			return true;
		if(a_bb.mem_err())
			return true;

		// Error with backtrack stacks:
		return Error;
	}

	// close: Close cursor.
	inline void close() {
		result_cursor.close();
	}

//-----------------------------------------------------------------------------
// Copy-constructor and assignment operator are not supported for this class:
//-----------------------------------------------------------------------------

	private:

	// These methods are not implemented:
	bench_j1axy_8_c(const bench_j1axy_8_c& obj);
	bench_j1axy_8_c& operator=(const bench_j1axy_8_c& obj);


//-----------------------------------------------------------------------------
// Debugging Support:
//-----------------------------------------------------------------------------

#ifdef	VER_DEBUG
	// Integrity check:
	public:
	str_t	check() const;

	// Magic number (identifies objects of this class for debugging).
	private:
	long	Magic;		// Must be "BENCH_J1AXY_8_MAGIC".
#endif


#ifdef	VER_DUMP
	public:
	// Display data structure:
	void dump(str_t headline = STR_NULL) const;
#endif

//-----------------------------------------------------------------------------
// Private Object Members:
//-----------------------------------------------------------------------------

	private:

//-----------------------------------------------------------------------------
// Data storage for EDB relations:
//-----------------------------------------------------------------------------

	rel_n_n_c c2_bf;
	list_2_c c3_ff;
	rel_n_n_c c4_bf;
	list_2_c d1_ff;
	rel_n_n_c d2_bf;

//-----------------------------------------------------------------------------
// Relation for storing derived b1 tuples (for rule with two IDB lit):
//-----------------------------------------------------------------------------

	rel_n_n_c b1_fb;
		// Note that columns 1 and 2 are exchanged,
		// because c1 is always input to the mapping, and c2 output.

//-----------------------------------------------------------------------------
// Sets for duplicate checks:
//-----------------------------------------------------------------------------

	set_tt_c b1_bb;
	set_tt_c b2_bb;
	set_tt_c c1_bb;
	set_tt_c a_bb;

//-----------------------------------------------------------------------------
// Cursors for loops over relations:
//-----------------------------------------------------------------------------

	cur_2_c c4_1_ff;
	cur_n_n_c c4_2_bf;
	cur_n_n_c c3_2_bf;
	cur_2_c c5_1_ff;
	cur_n_n_c c5_2_bf;
	cur_n_n_c c2_1_fb;


//-----------------------------------------------------------------------------
// Result list and cursor:
//-----------------------------------------------------------------------------

	list_2_c result_list;
	cur_2_c  result_cursor;

//-----------------------------------------------------------------------------
// Variable declarations:
//-----------------------------------------------------------------------------

//	int v_2_1;
//	int v_3_2;
//	int v_4_1;
//	int v_4_2;
//	int v_5_1;
//	int v_5_2;

//-----------------------------------------------------------------------------
// Call Depth for Trace Output:
//-----------------------------------------------------------------------------

	OUT_TRACE_DEPTH_DECL;

//-----------------------------------------------------------------------------
// Did an error occur?
//-----------------------------------------------------------------------------

	bool Error;

//-----------------------------------------------------------------------------
// Auxiliary functions:
//-----------------------------------------------------------------------------

	private:

//-----------------------------------------------------------------------------
// start_rule_4: Compute c1 facts for rule with only EDB body literals.
//-----------------------------------------------------------------------------

inline void start_rule_4()
{
	OUT_TRACE_DEPTH;

	// Start Rule 4: c1(A,B) :- d1(A,C), d2(C,B).
	OUT_TRACE_RULE_APP_0("4: c1(A,B) :- d1(A,C), d2(C,B).",
				"start");

	// Loop over d1(A, C):
	c4_1_ff.open();
	while (c4_1_ff.fetch()) {
		int A = c4_1_ff.c1();
		int C = c4_1_ff.c2();

		// Loop over d2(C, B):
		c4_2_bf.open(C);
		while (c4_2_bf.fetch()) {
			int B = c4_2_bf.c2();
			// Derived: c1(A, B)

			// Duplicate check:
			if(!c1_bb.insert(A, B))
				continue;

			// Use derived fact:
			OUT_TRACE_CALL_2("c1", A, B);
			derived_c1(A, B);
		}
	}
}

//-----------------------------------------------------------------------------
// derived_c1: Function to call when a c1-tuple was derived.
//-----------------------------------------------------------------------------

inline void derived_c1(int A, int C)
{
	OUT_TRACE_DEPTH;

	// 3: b1(A,B) :- c1(A,C), c2(C,B).
	OUT_TRACE_RULE_APP_2("3: b1(A,B) :- c1(A,C), c2(C,B).",
				"c1", A, C);

	// Cursor over c2(C,B):
	c3_2_bf.open(C);
	while (c3_2_bf.fetch()) {
		int B = c3_2_bf.c2();

		// b1(A, B) is derived. Duplicate check:
		if(!b1_bb.insert(A, B))
			continue;

		OUT_TRACE_CALL_2("b1", A, B);
		derived_b1(A, B);
		// This stores the derived fact in a temporary relation
		// because it is used in a rule with two IDB body literals.
		// Of course we could immediately do this here,
		// but the code looks more uniform if we call a function
		// for that. We trust on the inline declaration.
	}
}


//-----------------------------------------------------------------------------
// derived_b1: Function to call when a b1-tuple was derived.
//-----------------------------------------------------------------------------

inline void derived_b1(int A, int B)
{
	OUT_TRACE_DEPTH;

	OUT_TRACE_RULE_APP_2("bf_fb += b1(A, B)", "b1", A, B);

	// Store in temporary relation
	// (used in rule 2 with two IDB body literals):
	if(!b1_fb.insert(B, A))
		Error = true;
		// Note: The columns are exchanged here, because the
		// bound value needs to come first, but the binding pattern
		// is "fb". So "A" is free, and "B" is bound.
}


//-----------------------------------------------------------------------------
// start_rule_5: Compute b2 facts for rule with only EDB body literals.
//-----------------------------------------------------------------------------

inline void start_rule_5()
{
	OUT_TRACE_DEPTH;

	// Start rule 5: b2(A,B) :- c3(A,C), c4(C,B).
	OUT_TRACE_RULE_APP_0("5: b2(A,B) :- c3(A,C), c4(C,B).", "start");

	// Loop over c3(A, C):
	c5_1_ff.open();
	while (c5_1_ff.fetch()) {
		int A = c5_1_ff.c1();
		int C = c5_1_ff.c2();

		// Loop over c4(C, B):
		c5_2_bf.open(C);
		while (c5_2_bf.fetch()) {
			int B = c5_2_bf.c2();
			// Derived: b2(A, B)

			// Duplicate check:
			if(!b2_bb.insert(A, B))
				continue;

			// Use derived fact:
			OUT_TRACE_CALL_2("b2", A, B);
			derived_b2(A, B);
		}
	}
}

//-----------------------------------------------------------------------------
// derived_b2: Function to call when a b2-tuple was derived.
//-----------------------------------------------------------------------------

inline void derived_b2(int C, int B)
{
	OUT_TRACE_DEPTH;

	// 2: a(A,B) :- b1(A,C), b2(C,B).
	OUT_TRACE_RULE_APP_2("2: a(A,B) :- b1(A,C), b2(C,B).",
				"b2", C, B);

	// Loop over b1(A, C):
	c2_1_fb.open(C);
	while (c2_1_fb.fetch()) {
		int A = c2_1_fb.c2();
			// Note: This is really column 1.
			// The columns are inversed because of the binding
			// pattern fb.

		// Derived: a(A, B)

		// Duplicate check:
		if(!a_bb.insert(A, B))
			continue;

		// Use derived fact:
		OUT_TRACE_CALL_2("a", A, B);
		derived_a(A, B);
	}
}


//-----------------------------------------------------------------------------
// derived_a: Function to call when an a-tuple was derived.
//-----------------------------------------------------------------------------

inline void derived_a(int A, int B)
{
	OUT_TRACE_DEPTH;

	// 1: answer(A,B) :- a(A,B).
	OUT_TRACE_RULE_APP_2("1: answer(A,B) :- a(A,B).",
			"a", A, B);

	// The tuple is stored in the result set:
	if(!result_list.insert(A, B))
		Error = true;
}



//=============================================================================
// End of Class:
//=============================================================================

};


//=============================================================================
// End of Include File:
//=============================================================================

#endif

