// ============================================================================
// Project:	Deductive Database
// Filename:	bench_j1axy_9.h
// Purpose:	Benchmark Join1, a(X,Y), Ver. 9: functions without args, bitmap
// Last Change:	29.08.2016
// Language:	C++
// EMail:	brass@informatik.uni-halle.de
// WWW:		http://www.informatik.uni-halle.de/~brass/
// Address:	Feldschloesschen 15, D-06120 Halle (Saale), GERMANY
// Copyright:	(c) 2015-2016 by Stefan Brass
// License:	See file "LICENSE" for copying conditions.
// Note:	There is no warranty at all - this code may contain bugs.
// ============================================================================


//=============================================================================
// Include File Frame:
//=============================================================================

#ifndef	BENCH_J1AXY_9_INCLUDED
#define BENCH_J1AXY_9_INCLUDED

//=============================================================================
// Used Types and Macros:
//=============================================================================

#ifndef VER_INCLUDED
#include "ver.h"
#endif

#ifndef STR_INCLUDED
#include "str.h"
#endif

#ifndef CHECK_INCLUDED
#include "check.h"
#endif

#ifndef STRTAB_INCLUDED
#include "strtab.h"
#endif

#ifndef LIST_1_INCLUDED
#include "list_1.h"
#endif

#ifndef LIST_2_INCLUDED
#include "list_2.h"
#endif

#ifndef SET_TT_INCLUDED
#include "set_tt.h"
#endif

#ifndef REL_N_N_INCLUDED
#include "rel_n_n.h"
#endif

#ifndef OUT_INCLUDED
#include "out.h"
#endif

#ifndef BENCH_INCLUDED
#include "bench.h"
#endif


//=============================================================================
// Private Constants:
//=============================================================================

//-----------------------------------------------------------------------------
// BENCH_J1AXY_9_MAGIC: Magic number (identifies objects of this class).
//-----------------------------------------------------------------------------

static const long BENCH_J1AXY_9_MAGIC = 0x4A31410AL;	// 'J1A\n'


//=============================================================================
// Benchmark Join1, a(X,Y), Ver. 9: functions without args, bitmap dupcheck
//=============================================================================

class bench_j1axy_9_c : public bench_c {
	public:


//-----------------------------------------------------------------------------
// Constructor, Destructor:
//-----------------------------------------------------------------------------

	public:

	// Constructor:
	bench_j1axy_9_c();

	// Destructor:
	~bench_j1axy_9_c();

//-----------------------------------------------------------------------------
// (Object) Methods:
//-----------------------------------------------------------------------------

	// load: Load data file with EDB facts.
	bool load(str_t filename);

	// eval: Do bottom-up evaluation.
	long eval();

	// name: Name of this benchmark.
	str_t name() const {
		return "Join 1, a(X,Y), Ver. 9: functions without args, bitmap";
	}

	// file: List of data files.
	str_t file(int i) const;

	// result: List of correct results of the eval method.
	long result(int i) const;

	// Open cursor (note: load() must be called before this method).
	void open();

	// fetch: Get next tuple, returns false if there is no further tuple.
	inline bool fetch() {
		return result_cursor.fetch();
	}


	// c1: Return value for column 1 of current row in cursor.
	inline int c1() {
		return result_cursor.c1();
	}

	// c2: Return value for column 2 of current row in cursor.
	inline int c2() {
		return result_cursor.c2();
	}

	// error: Did an error occur?
	inline bool error() {
		// EDB relations:
		if(c2_bf.mem_err())
			return true;
		if(c3_ff.mem_err())
			return true;
		if(c4_bf.mem_err())
			return true;
		if(d1_ff.mem_err())
			return true;
		if(d2_bf.mem_err())
			return true;

		// Derived b1 tuples (for rule with two IDB lit):
		if(b1_fb.mem_err())
			return true;

		// Sets for duplicate checks:
		if(b1_bb.mem_err())
			return true;
		if(b2_bb.mem_err())
			return true;
		if(c1_bb.mem_err())
			return true;
		if(a_bb.mem_err())
			return true;

		// Error with backtrack stacks:
		return Error;
	}

	// close: Close cursor.
	inline void close() {
		result_cursor.close();
	}

//-----------------------------------------------------------------------------
// Copy-constructor and assignment operator are not supported for this class:
//-----------------------------------------------------------------------------

	private:

	// These methods are not implemented:
	bench_j1axy_9_c(const bench_j1axy_9_c& obj);
	bench_j1axy_9_c& operator=(const bench_j1axy_9_c& obj);


//-----------------------------------------------------------------------------
// Debugging Support:
//-----------------------------------------------------------------------------

#ifdef	VER_DEBUG
	// Integrity check:
	public:
	str_t	check() const;

	// Magic number (identifies objects of this class for debugging).
	private:
	long	Magic;		// Must be "BENCH_J1AXY_9_MAGIC".
#endif


#ifdef	VER_DUMP
	public:
	// Display data structure:
	void dump(str_t headline = STR_NULL) const;
#endif

//-----------------------------------------------------------------------------
// Private Object Members:
//-----------------------------------------------------------------------------

	private:

//-----------------------------------------------------------------------------
// Data storage for EDB relations:
//-----------------------------------------------------------------------------

	rel_n_n_c c2_bf;
	list_2_c c3_ff;
	rel_n_n_c c4_bf;
	list_2_c d1_ff;
	rel_n_n_c d2_bf;

//-----------------------------------------------------------------------------
// Relation for storing derived b1 tuples (for rule with two IDB lit):
//-----------------------------------------------------------------------------

	rel_n_n_c b1_fb;
		// Note that columns 1 and 2 are exchanged,
		// because c1 is always input to the mapping, and c2 output.

//-----------------------------------------------------------------------------
// Sets for duplicate checks:
//-----------------------------------------------------------------------------

	set_tt_c b1_bb;
	set_tt_c b2_bb;
	set_tt_c c1_bb;
	set_tt_c a_bb;

//-----------------------------------------------------------------------------
// Cursors for loops over relations:
//-----------------------------------------------------------------------------

	cur_2_c c4_1_ff;
	cur_n_n_c c4_2_bf;
	cur_n_n_c c3_2_bf;
	cur_2_c c5_1_ff;
	cur_n_n_c c5_2_bf;
	cur_n_n_c c2_1_fb;


//-----------------------------------------------------------------------------
// Result list and cursor:
//-----------------------------------------------------------------------------

	list_2_c result_list;
	cur_2_c  result_cursor;

//-----------------------------------------------------------------------------
// Variable declarations:
//-----------------------------------------------------------------------------

	int v_2_1;
	int v_3_2;
	int v_4_1;
	int v_4_2;
	int v_5_1;
	int v_5_2;

//-----------------------------------------------------------------------------
// Call Depth for Trace Output:
//-----------------------------------------------------------------------------

	OUT_TRACE_DEPTH_DECL;

//-----------------------------------------------------------------------------
// Did an error occur?
//-----------------------------------------------------------------------------

	bool Error;

//-----------------------------------------------------------------------------
// Auxiliary functions:
//-----------------------------------------------------------------------------

	private:

	// start_rule_4: Compute c1 facts for rule with only EDB body literals.
	inline void start_rule_4();

	// derived_c1: Function to call when c1(v_4_1,v_4_2) was derived.
	inline void derived_c1();

	// derived_b1: Function to call when b1(v_4_1,v_3_2) was derived.
	inline void derived_b1();

	// start_rule_5: Compute b2 facts for rule with only EDB body literals.
	inline void start_rule_5();

	// derived_b2: Function to call when b2(v_5_1,v_5_2) was derived.
	inline void derived_b2();

	// derived_a: Function to call when a(v_2_1,v_5_2) was derived.
	inline void derived_a();

//=============================================================================
// End of Class:
//=============================================================================

};


//=============================================================================
// End of Include File:
//=============================================================================

#endif

