// ============================================================================
// Project:	Deductive Database
// Filename:	bench_tcff_2.h
// Purpose:	Bottom-Up Evaluation for tc(X,Y), Ver 2: Cursor interface
// Last Change:	26.05.2016
// Language:	C++
// EMail:	brass@informatik.uni-halle.de
// WWW:		http://www.informatik.uni-halle.de/~brass/
// Address:	Feldschloesschen 15, D-06120 Halle (Saale), GERMANY
// Copyright:	(c) 2015-2016 by Stefan Brass
// License:	See file "LICENSE" for copying conditions.
// Note:	There is no warranty at all - this code may contain bugs.
// ============================================================================


//=============================================================================
// Include File Frame:
//=============================================================================

#ifndef	BENCH_TCFF_2_INCLUDED
#define BENCH_TCFF_2_INCLUDED

//=============================================================================
// Used Types and Macros:
//=============================================================================

#ifndef VER_INCLUDED
#include "ver.h"
#endif

#ifndef STR_INCLUDED
#include "str.h"
#endif

#ifndef CHECK_INCLUDED
#include "check.h"
#endif

#ifndef STACK_INCLUDED
#include "stack.h"
#endif

#ifndef LIST_2_INCLUDED
#include "list_2.h"
#endif

#ifndef CUR_2_INCLUDED
#include "cur_2.h"
#endif

#ifndef DUP_2I_INCLUDED
#include "dup_2i.h"
#endif

#ifndef REL_N_N_INCLUDED
#include "rel_n_n.h"
#endif

#ifndef CUR_N_N_INCLUDED
#include "cur_n_n.h"
#endif

#ifndef BENCH_INCLUDED
#include "bench.h"
#endif

//=============================================================================
// Private Constants:
//=============================================================================

//-----------------------------------------------------------------------------
// BENCH_TCFF_2_MAGIC: Magic number (identifies objects of this class).
//-----------------------------------------------------------------------------

static const long BENCH_TCFF_2_MAGIC = 0x5443460AL;	// 'TCF\n'


//=============================================================================
// Bottom-Up Evalutation (DBLP Benchmark) - Ver. 2: Attrib., Cursor Interface.
//=============================================================================

class bench_tcff_2_c : public bench_c {
	public:

//-----------------------------------------------------------------------------
// Constructor, Destructor:
//-----------------------------------------------------------------------------

	public:

	// Constructor:
	bench_tcff_2_c();

	// Destructor:
	~bench_tcff_2_c();

//-----------------------------------------------------------------------------
// (Object) Methods:
//-----------------------------------------------------------------------------

	// load: Load data file with EDB facts.
	bool load(str_t filename);

	// eval: Do bottom-up evaluation.
	long eval();

	// name: Name of this benchmark.
	str_t name() const {
		return
		"tc(X,Y) Bench., Ver.2: Rel/Var->Attrs, Cursor Interface";
	}

	// file: List of data files.
	str_t file(int i) const;

	// result: List of correct results of eval method.
	long result(int i) const;

	// Open cursor (note: load() must be called before this method).
	void open();

	// fetch: Get next tuple, returns false if there is no further tuple.
	bool fetch();

	// There are two exit points:
	// answer(v2_1,v2_2):
	// answer(v3_1,v2_2):

	// c1: Return value for column 1 of current row in cursor.
	inline int c1() {
		if(first_exit_point)
			return v2_1;
		else
			return v3_1;
	}

	// c2: Return value for column 2 of current row in cursor.
	inline int c2() {
		return v2_2;
	}

	// error: Did an error occur?
	inline bool error() {
		if(par_ff.mem_err())
			return true;
		if(par_fb.mem_err())
			return true;
		return Error;
	}

	// close: Close cursor.
	inline void close() {
	}

//-----------------------------------------------------------------------------
// Copy-constructor and assignment operator are not supported for this class:
//-----------------------------------------------------------------------------

	private:

	// These methods are not implemented:
	bench_tcff_2_c(const bench_tcff_2_c& obj);
	bench_tcff_2_c& operator=(const bench_tcff_2_c& obj);


//-----------------------------------------------------------------------------
// Debugging Support:
//-----------------------------------------------------------------------------

#ifdef	VER_DEBUG
	// Integrity check:
	public:
	str_t	check() const;

	// Magic number (identifies objects of this class for debugging).
	private:
	long	Magic;		// Must be "BENCH_TCFF_2_MAGIC".
#endif


#ifdef	VER_DUMP
	public:
	// Display data structure:
	void dump(str_t headline = STR_NULL) const;
#endif

//-----------------------------------------------------------------------------
// Private Object Members:
//-----------------------------------------------------------------------------

	private:

//-----------------------------------------------------------------------------
// Relations for EDB body literals:
//-----------------------------------------------------------------------------

	list_2_c  par_ff;
	rel_n_n_c par_fb;

//-----------------------------------------------------------------------------
// Relations for duplicate check (IDB predicates):
//-----------------------------------------------------------------------------

	dup_2i_c tc_bb;

//-----------------------------------------------------------------------------
// Variables to hold values during the bottom-up evaluation:
//-----------------------------------------------------------------------------

	int v2_1;
	int v2_2;
	int v3_1;

//-----------------------------------------------------------------------------
// Exit point (at which label was an answer-fact derived)?
//-----------------------------------------------------------------------------

	bool first_exit_point;

//-----------------------------------------------------------------------------
// Cursors for loops during the bottom-up evaluation:
//-----------------------------------------------------------------------------

	cur_2_c   c2_1_ff;
	cur_n_n_c c3_1_fb;

//-----------------------------------------------------------------------------
// Stacks for the bottom-up evaluation:
//-----------------------------------------------------------------------------

	stack_c<int> backtrack_stack;
	stack_c<int> int_value_stack;

//-----------------------------------------------------------------------------
// Did an error occur?
//-----------------------------------------------------------------------------

	bool Error;


//=============================================================================
// End of Class:
//=============================================================================

};


//=============================================================================
// End of Include File:
//=============================================================================

#endif

