// ============================================================================
// Project:	Deductive Database
// Filename:	bench_tcff_4.h
// Purpose:	Bottom-Up Evaluation for tc(X,Y), Ver 4: Functions
// Last Change:	29.08.2016
// Language:	C++
// EMail:	brass@informatik.uni-halle.de
// WWW:		http://www.informatik.uni-halle.de/~brass/
// Address:	Feldschloesschen 15, D-06120 Halle (Saale), GERMANY
// Copyright:	(c) 2015-2016 by Stefan Brass
// License:	See file "LICENSE" for copying conditions.
// Note:	There is no warranty at all - this code may contain bugs.
// ============================================================================


//=============================================================================
// Include File Frame:
//=============================================================================

#ifndef	BENCH_TCFF_4_INCLUDED
#define BENCH_TCFF_4_INCLUDED

//=============================================================================
// Used Types and Macros:
//=============================================================================

#ifndef VER_INCLUDED
#include "ver.h"
#endif

#ifndef STR_INCLUDED
#include "str.h"
#endif

#ifndef CHECK_INCLUDED
#include "check.h"
#endif

#ifndef LIST_2_INCLUDED
#include "list_2.h"
#endif

#ifndef CUR_2_INCLUDED
#include "cur_2.h"
#endif

#ifndef DUP_2I_INCLUDED
#include "dup_2i.h"
#endif

#ifndef REL_N_N_INCLUDED
#include "rel_n_n.h"
#endif

#ifndef BENCH_INCLUDED
#include "bench.h"
#endif

//=============================================================================
// Private Constants:
//=============================================================================

//-----------------------------------------------------------------------------
// BENCH_TCFF_4_MAGIC: Magic number (identifies objects of this class).
//-----------------------------------------------------------------------------

static const long BENCH_TCFF_4_MAGIC = 0x5443460AL;	// 'TCF\n'


//=============================================================================
// Bottom-Up Evalutation (DBLP Benchmark) - Ver. 4: Functions
//=============================================================================

class bench_tcff_4_c : public bench_c {
	public:

//-----------------------------------------------------------------------------
// Constructor, Destructor:
//-----------------------------------------------------------------------------

	public:

	// Constructor:
	bench_tcff_4_c();

	// Destructor:
	~bench_tcff_4_c();

//-----------------------------------------------------------------------------
// (Object) Methods:
//-----------------------------------------------------------------------------

	// load: Load data file with EDB facts.
	bool load(str_t filename);

	// eval: Do bottom-up evaluation and read result over cursor interface.
	long eval();

	// name: Name of this benchmark.
	str_t name() const {
		return
		"tc(X,Y) Bench., Ver.4: Functions, Cursor Interface";
	}

	// file: List of data files.
	str_t file(int i) const;

	// result: List of correct results of eval method.
	long result(int i) const;

	// Open cursor (note: load() must be called before this method).
	void open();

	// fetch: Get next tuple, returns false if there is no further tuple.
	inline bool fetch() {
		return result_cursor.fetch();
	}

	// c1: Return value for column 1 of current row in cursor.
	inline int c1() {
		return result_cursor.c1();
	}

	// c2: Return value for column 2 of current row in cursor.
	inline int c2() {
		return result_cursor.c2();
	}

	// error: Did an error occur?
	inline bool error() {
		if(par_ff.mem_err())
			return true;
		if(par_fb.mem_err())
			return true;
		if(result_list.mem_err())
			return true;
		return Error;
	}

	// close: Close cursor.
	inline void close() {
		result_cursor.close();
	}

//-----------------------------------------------------------------------------
// Copy-constructor and assignment operator are not supported for this class:
//-----------------------------------------------------------------------------

	private:

	// These methods are not implemented:
	bench_tcff_4_c(const bench_tcff_4_c& obj);
	bench_tcff_4_c& operator=(const bench_tcff_4_c& obj);


//-----------------------------------------------------------------------------
// Debugging Support:
//-----------------------------------------------------------------------------

#ifdef	VER_DEBUG
	// Integrity check:
	public:
	str_t	check() const;

	// Magic number (identifies objects of this class for debugging).
	private:
	long	Magic;		// Must be "BENCH_TCFF_4_MAGIC".
#endif


#ifdef	VER_DUMP
	public:
	// Display data structure:
	void dump(str_t headline = STR_NULL) const;
#endif

//-----------------------------------------------------------------------------
// Private Object Members:
//-----------------------------------------------------------------------------

	private:

//-----------------------------------------------------------------------------
// Relations for EDB body literals:
//-----------------------------------------------------------------------------

	list_2_c  par_ff;
	rel_n_n_c par_fb;

//-----------------------------------------------------------------------------
// Relations for duplicate check (IDB predicates):
//-----------------------------------------------------------------------------

	dup_2i_c tc_bb;

//-----------------------------------------------------------------------------
// Result list and cursor:
//-----------------------------------------------------------------------------

	list_2_c result_list;
	cur_2_c  result_cursor;

//-----------------------------------------------------------------------------
// Call Depth for Trace Output:
//-----------------------------------------------------------------------------

	OUT_TRACE_DEPTH_DECL;

//-----------------------------------------------------------------------------
// Did an error occur?
//-----------------------------------------------------------------------------

	bool Error;

//-----------------------------------------------------------------------------
// Auxiliary functions:
//-----------------------------------------------------------------------------

	private:

	// start_rule_2: Compute tc facts for rule with only EDB body literals.
	inline void start_rule_2();

	// derived_tc: Function to call when a tc-tuple was derived.
	inline void derived_tc(int A, int B);

	// derived_answer: Function to call when an answer-tuple was derived.
	inline void derived_answer(int A, int B);


//=============================================================================
// End of Class:
//=============================================================================

};


//=============================================================================
// End of Include File:
//=============================================================================

#endif

