// ============================================================================
// Project:	Deductive Database
// Filename:	bench_wine.h
// Purpose:	Bottom-Up Evaluation for Wine Ontology Example
// Last Change:	19.05.2015
// Language:	C++
// EMail:	brass@informatik.uni-halle.de
// WWW:		http://www.informatik.uni-halle.de/~brass/
// Address:	Feldschloesschen 15, D-06120 Halle (Saale), GERMANY
// Copyright:	(c) 2015 by Stefan Brass
// License:	See file "LICENSE" for copying conditions.
// Note:	There is no warranty at all - this code may contain bugs.
// ============================================================================

//=============================================================================
// Include File Frame:
//=============================================================================

#ifndef	BENCH_WINE_INCLUDED
#define BENCH_WINE_INCLUDED

//=============================================================================
// Used Types and Macros:
//=============================================================================

#ifndef VER_INCLUDED
#include "ver.h"
#endif

#ifdef VER_WINE_ENABLED

#ifndef STR_INCLUDED
#include "str.h"
#endif

#ifndef CHECK_INCLUDED
#include "check.h"
#endif

#ifndef STRTAB_INCLUDED
#include "strtab.h"
#endif

#ifndef LIST_1_INCLUDED
#include "list_1.h"
#endif

#ifndef LIST_2_INCLUDED
#include "list_2.h"
#endif

#ifndef SET_1_INCLUDED
#include "set_1.h"
#endif

#ifndef BENCH_INCLUDED
#include "bench.h"
#endif


//=============================================================================
// Private Constants:
//=============================================================================

//-----------------------------------------------------------------------------
// BENCH_WINE_MAGIC: Magic number (identifies objects of this class).
//-----------------------------------------------------------------------------

static const long BENCH_WINE_MAGIC = 0x4557490AL;	// 'EWI\n'


//=============================================================================
// Bottom-Up Evaluation (Wine Ontology Benchmark):
//=============================================================================

class bench_wine_c : public bench_c {
	public:


//-----------------------------------------------------------------------------
// Constructor, Destructor:
//-----------------------------------------------------------------------------

	public:

	// Constructor:
	bench_wine_c();

	// Destructor:
	~bench_wine_c();


//-----------------------------------------------------------------------------
// (Object) Methods:
//-----------------------------------------------------------------------------

	// load: Load data file with EDB facts.
	bool load(str_t filename);

	// Do bottom-up evaluation for wine ontology:
	long eval();

	// name: Name of this benchmark.
	str_t name() const {
		return
		"Wine Ontology Benchmark";
	}

	// file: List of data files.
	str_t file(int i) const;

	// result: List of correct results of eval method.
	long result(int i) const;

//-----------------------------------------------------------------------------
// Copy-constructor and assignment operator are not supported for this class:
//-----------------------------------------------------------------------------

	private:

	bench_wine_c(const bench_wine_c& obj);		// Not implemented
	bench_wine_c& operator=(const bench_wine_c& obj); // Not implemented


//-----------------------------------------------------------------------------
// Debugging Support:
//-----------------------------------------------------------------------------

#ifdef	VER_DEBUG
	// Integrity check:
	public:
	str_t	check() const;

	// Magic number (identifies objects of this class for debugging).
	private:
	long	Magic;		// Must be "BENCH_WINE_MAGIC".
#endif


#ifdef	VER_DUMP
	public:
	// Display data structure:
	void dump(str_t headline = STR_NULL) const;
#endif

//-----------------------------------------------------------------------------
// Private Object Members:
//-----------------------------------------------------------------------------

	private:

//-----------------------------------------------------------------------------
// Domain for all constants:
//-----------------------------------------------------------------------------

	strtab_c dom;

//-----------------------------------------------------------------------------
// Data storage for EDB relations:
//-----------------------------------------------------------------------------

	list_1_c kaon2namedobjects_f;
	list_2_c hasflavor_aux_ff;
	set_1_c kaon2namedobjects_b;
	list_1_c cheninblanc_aux_f;
	list_1_c beaujolais_aux_f;
	list_2_c hassugar_aux_ff;
	list_2_c hasvintageyear_aux_ff;
	list_2_c hascolor_aux_ff;
	list_1_c meritage_aux_f;
	list_1_c ot____nom8_aux_f;
	list_2_c madefromgrape_aux_ff;
	list_1_c riesling_aux_f;
	list_1_c dryriesling_aux_f;
	list_1_c chianti_aux_f;
	list_2_c hasmaker_aux_ff;
	list_1_c chardonnay_aux_f;
	list_1_c whiteburgundy_aux_f;
	list_1_c meursault_aux_f;
	list_2_c hasbody_aux_ff;
	list_1_c muscadet_aux_f;
	list_1_c margaux_aux_f;
	list_2_c locatedin_aux_ff;
	list_1_c sancerre_aux_f;
	list_1_c cotesdor_aux_f;
	list_1_c pauillac_aux_f;
	list_1_c anjou_aux_f;
	list_1_c port_aux_f;
	list_1_c sauternes_aux_f;
	list_1_c stemilion_aux_f;
	list_1_c ot____nom36_aux_f;
	list_1_c cabernetsauvignon_aux_f;
	list_1_c icewine_aux_f;
	list_1_c merlot_aux_f;
	list_1_c sweetriesling_aux_f;
	list_1_c whitewine_aux_f;
	list_1_c ot____nom23_aux_f;
	list_1_c pinotnoir_aux_f;
	list_1_c cabernetfranc_aux_f;
	list_1_c semillon_aux_f;
	list_1_c sauvignonblanc_aux_f;
	list_1_c ot____nom6_aux_f;
	list_1_c ot____nom19_aux_f;
	list_1_c petitesyrah_aux_f;
	list_1_c zinfandel_aux_f;
	list_1_c redtablewine_aux_f;
	list_1_c dessertwine_aux_f;
	list_1_c ot____nom3_aux_f;
	list_1_c ot____nom43_aux_f;
	list_1_c ot____nom24_aux_f;
	list_1_c ot____nom11_aux_f;
	list_1_c ot____nom34_aux_f;
	list_1_c ot____nom37_aux_f;
	list_1_c ot____nom20_aux_f;
	list_1_c ot____nom33_aux_f;
	list_1_c ot____nom39_aux_f;
	list_1_c ot____nom27_aux_f;
	list_1_c ot____nom25_aux_f;
	list_1_c ot____nom21_aux_f;
	list_1_c ot____nom29_aux_f;
	list_1_c ot____nom7_aux_f;
	list_1_c ot____nom28_aux_f;
	list_1_c ot____nom45_aux_f;
	list_1_c ot____nom1_aux_f;
	list_1_c ot____nom32_aux_f;
	list_1_c ot____nom5_aux_f;
	list_1_c ot____nom12_aux_f;
	list_1_c ot____nom15_aux_f;
	list_1_c ot____nom2_aux_f;
	list_1_c ot____nom44_aux_f;
	list_1_c ot____nom10_aux_f;
	list_1_c ot____nom31_aux_f;
	list_1_c ot____nom16_aux_f;
	list_1_c ot____nom41_aux_f;
	list_1_c ot____nom38_aux_f;
	list_1_c ot____nom26_aux_f;
	list_1_c ot____nom4_aux_f;
	list_1_c ot____nom17_aux_f;
	list_1_c ot____nom14_aux_f;
	list_1_c ot____nom46_aux_f;
	list_1_c ot____nom13_aux_f;
	list_1_c ot____nom47_aux_f;
	list_1_c ot____nom42_aux_f;
	list_1_c ot____nom30_aux_f;
	list_1_c ot____nom22_aux_f;

//=============================================================================
// End of Class:
//=============================================================================

};


//=============================================================================
// End of Include File:
//=============================================================================

// End of #ifdef VER_WINE_ENABLED:
#endif

// End of #ifndef BENCH_WINE_INCLUDED:
#endif

