// ============================================================================
// Project:	Deductive Database
// Filename:	bind.cpp
// Purpose:	Binding patterns for predicates
// Last Change:	10.11.2015
// Language:	C++
// EMail:	brass@informatik.uni-halle.de
// WWW:		http://www.informatik.uni-halle.de/~brass/
// Address:	Feldschloesschen 15, D-06120 Halle (Saale), GERMANY
// Copyright:	(c) 1998-2015 by Stefan Brass
// License:	See file "LICENSE" for copying conditions.
// Note:	There is no warranty at all - this code may contain bugs.
// ============================================================================


//=============================================================================
// Interface:
//=============================================================================

#include "ver.h"
#include "check.h"
#include "bind.h"

//=============================================================================
// Exported Functions:
//=============================================================================

//-----------------------------------------------------------------------------
// bind_valid: Is the parameter a valid element of the enumeration type?
//-----------------------------------------------------------------------------

#ifdef	VER_DEBUG

bool	bind_valid(bind_t bind)
{
	return( bind == BIND_NULL ||
		bind == BIND_EMPTY ||
		bind == BIND_B ||
		bind == BIND_F ||
		bind == BIND_BB ||
		bind == BIND_BF ||
		bind == BIND_FB ||
		bind == BIND_FF
	);
}

#endif


//-----------------------------------------------------------------------------
// bind_name: String representation of this binding pattern.
//-----------------------------------------------------------------------------

str_t	bind_name(bind_t bind)
{
	str_t name;
	
	switch(bind) {
		case BIND_NULL:
			name = "NULL";
			break;
		case BIND_EMPTY:
			name = "- (empty)";
			break;
		case BIND_B:
			name = "b";
			break;
		case BIND_F:
			name = "f";
			break;
		case BIND_BB:
			name = "bb";
			break;
		case BIND_BF:
			name = "bf";
			break;
		case BIND_FB:
			name = "fb";
			break;
		case BIND_FF:
			name = "ff";
			break;
		default:
			CHECK_IMPOSSIBLE("bind_name: unknown binding pattern");
			name = "? (unknown/illegal)";
			break;
	}
	
	return name;
}

//-----------------------------------------------------------------------------
// bind_arity: Length of the binding pattern (number of predicate arguments).
//-----------------------------------------------------------------------------

int	bind_arity(bind_t bind)
{
	int arity;
	
	switch(bind) {
		case BIND_NULL:
		case BIND_EMPTY:
			arity = 0;
			break;
		case BIND_B:
		case BIND_F:
			arity = 1;
			break;
		case BIND_BB:
		case BIND_BF:
		case BIND_FB:
		case BIND_FF:
			arity = 2;
			break;
		default:
			CHECK_IMPOSSIBLE(
				"bind_arity: unknown binding pattern");
			arity = 0;
			break;
	}
	
	return arity;
}

