// ============================================================================
// Project:	Deductive Database
// Filename:	bind.h
// Purpose:	Binding patterns for predicates
// Last Change:	14.11.2015
// Language:	C++
// EMail:	brass@informatik.uni-halle.de
// WWW:		http://www.informatik.uni-halle.de/~brass/
// Address:	Feldschloesschen 15, D-06120 Halle (Saale), GERMANY
// Copyright:	(c) 1998-2015 by Stefan Brass
// License:	See file "LICENSE" for copying conditions.
// Note:	There is no warranty at all - this code may contain bugs.
// ============================================================================


//=============================================================================
// Include File Frame:
//=============================================================================

#ifndef	BIND_INCLUDED
#define BIND_INCLUDED

//=============================================================================
// Used Types and Macros:
//=============================================================================

#ifndef VER_INCLUDED
#include "ver.h"
#endif

#ifndef STR_INCLUDED
#include "str.h"
#endif

#ifndef CHECK_INCLUDED
#include "check.h"
#endif

//=============================================================================
// Exported Types:
//=============================================================================

//-----------------------------------------------------------------------------
// bind_t: Binding patterns for predicates.
//-----------------------------------------------------------------------------

typedef enum {
	BIND_NULL,
	BIND_EMPTY,
	BIND_B,
	BIND_F,
	BIND_BB,
	BIND_BF,
	BIND_FB,
	BIND_FF
} bind_t;

//=============================================================================
// Exported Functions:
//=============================================================================

//-----------------------------------------------------------------------------
// bind_valid: Is the parameter a valid element of the enumeration type?
//-----------------------------------------------------------------------------

#ifdef	VER_DEBUG
bool	bind_valid(bind_t bind);
#endif

//-----------------------------------------------------------------------------
// bind_name: String representation of this binding pattern.
//-----------------------------------------------------------------------------

str_t	bind_name(bind_t bind);

//-----------------------------------------------------------------------------
// bind_arity: Length of the binding pattern (number of predicate arguments).
//-----------------------------------------------------------------------------

int	bind_arity(bind_t bind);

//=============================================================================
// End of File:
//=============================================================================

#endif

