// ============================================================================
// Project:	Deductive Database
// Filename:	cur_1.h
// Purpose:	Cursor for list of rows with one int column
// Last Change:	07.11.2015
// Language:	C++
// EMail:	brass@informatik.uni-halle.de
// WWW:		http://www.informatik.uni-halle.de/~brass/
// Address:	Feldschloesschen 15, D-06120 Halle (Saale), GERMANY
// Copyright:	(c) 2010-2015 by Stefan Brass
// License:	See file "LICENSE" for copying conditions.
// Note:	There is no warranty at all - this code may contain bugs.
// ============================================================================


//=============================================================================
// Include File Frame:
//=============================================================================

#ifndef	CUR_1_INCLUDED
#define CUR_1_INCLUDED

//=============================================================================
// Used Types and Macros:
//=============================================================================

#ifndef VER_INCLUDED
#include "ver.h"
#endif

#ifndef STR_INCLUDED
#include "str.h"
#endif

#ifndef CHECK_INCLUDED
#include "check.h"
#endif

#ifndef ROW_1_INCLUDED
#include "row_1.h"
#endif

#ifndef LIST_1_INCLUDED
#include "list_1.h"
#endif

#ifndef CUR_LIST_INCLUDED
#include "cur_list.h"
#endif


//=============================================================================
// Cursor over List of Rows with one int column:
//=============================================================================

class cur_1_c: public cur_list_c<row_1_c> {

	public:

//-----------------------------------------------------------------------------
// Constructor, Destructor:
//-----------------------------------------------------------------------------

	// Constructor:
	cur_1_c(const list_1_c *list);

	// Destructor:
	~cur_1_c();

//-----------------------------------------------------------------------------
// Copy-constructor and assignment operator are not supported for this class:
//-----------------------------------------------------------------------------

	private:

	cur_1_c(const cur_1_c& obj);		// Not implemented
	cur_1_c& operator=(const cur_1_c& obj);	// Not implemented


//=============================================================================
// (Object) Methods:
//=============================================================================

	public:

//-----------------------------------------------------------------------------
// c1: Return value for column 1 of current row in cursor.
//-----------------------------------------------------------------------------

	inline int c1() {
		return cur_list_c<row_1_c>::row()->c1();
	}

//=============================================================================
// End of Class Template:
//=============================================================================

};


//-----------------------------------------------------------------------------
// Define pointer type:
//-----------------------------------------------------------------------------

typedef cur_1_c *cur_1_t;

//-----------------------------------------------------------------------------
// Define null pointer:
//-----------------------------------------------------------------------------

#define CUR_1_NULL (static_cast<cur_1_t>(0))


//=============================================================================
// End of Include File:
//=============================================================================

#endif

