// ============================================================================
// Project:	Deductive Database
// Filename:	db_dblp.h
// Purpose:	Bottom-Up Evaluation for DBLP Example, Ver 3: Database
// Last Change:	11.06.2016
// Language:	C++
// EMail:	brass@informatik.uni-halle.de
// WWW:		http://www.informatik.uni-halle.de/~brass/
// Address:	Feldschloesschen 15, D-06120 Halle (Saale), GERMANY
// Copyright:	(c) 2015-2016 by Stefan Brass
// License:	See file "LICENSE" for copying conditions.
// Note:	There is no warranty at all - this code may contain bugs.
// ============================================================================


//=============================================================================
// Include File Frame:
//=============================================================================

#ifndef	DB_DBLP_INCLUDED
#define DB_DBLP_INCLUDED

//=============================================================================
// Used Types and Macros:
//=============================================================================

#ifndef VER_INCLUDED
#include "ver.h"
#endif

#ifndef STR_INCLUDED
#include "str.h"
#endif

#ifndef CHECK_INCLUDED
#include "check.h"
#endif

#ifndef ATOMTAB_INCLUDED
#include "atomtab.h"
#endif

#ifndef STRTAB_INCLUDED
#include "strtab.h"
#endif

#ifndef LIST_2_INCLUDED
#include "list_2.h"
#endif

#ifndef REL_N_N_INCLUDED
#include "rel_n_n.h"
#endif

//=============================================================================
// Private Constants:
//=============================================================================

//-----------------------------------------------------------------------------
// DB_DBLP_MAGIC: Magic number (identifies objects of this class).
//-----------------------------------------------------------------------------

static const long DB_DBLP_MAGIC = 0x4444500AL;	// 'DPP\n'


//=============================================================================
// Bottom-Up Evalutation (DBLP Benchmark) - Ver. 3: Database
//=============================================================================

class db_dblp_c {
	public:

//-----------------------------------------------------------------------------
// Constructor, Destructor:
//-----------------------------------------------------------------------------

	public:

	// Constructor:
	db_dblp_c();

	// Destructor:
	~db_dblp_c();

//-----------------------------------------------------------------------------
// (Object) Methods:
//-----------------------------------------------------------------------------

	// load: Load data file with EDB facts.
	bool load(str_t filename);

//-----------------------------------------------------------------------------
// Domains:
//-----------------------------------------------------------------------------

	strtab_c doc_ids;	// String table for first argument of att.
	atomtab_c attrs;	// Atom table for second argument of att.
	strtab_c str_values;	// String table for third argument of att.

//-----------------------------------------------------------------------------
// Atoms which are relevant for selection condition:
//-----------------------------------------------------------------------------

	int title;
	int year;
	int author;
	int month;

//-----------------------------------------------------------------------------
// Relations for EDB body literals:
//-----------------------------------------------------------------------------

	list_2_c  att_title;
	rel_n_n_c att_year;
	rel_n_n_c att_author;
	rel_n_n_c att_month;

//-----------------------------------------------------------------------------
// Copy-constructor and assignment operator are not supported for this class:
//-----------------------------------------------------------------------------

	private:

	// These methods are not implemented:
	db_dblp_c(const db_dblp_c& obj);
	db_dblp_c& operator=(const db_dblp_c& obj);

//-----------------------------------------------------------------------------
// Debugging Support:
//-----------------------------------------------------------------------------

#ifdef	VER_DEBUG
	// Integrity check:
	public:
	str_t	check() const;

	// Magic number (identifies objects of this class for debugging).
	private:
	long	Magic;		// Must be "DB_DBLP_MAGIC".
#endif


#ifdef	VER_DUMP
	public:
	// Display data structure:
	void dump(str_t headline = STR_NULL) const;
#endif

//-----------------------------------------------------------------------------
// Private Object Members:
//-----------------------------------------------------------------------------

	private:

	// None: Domains, Constants and Relations are public in this class.

//=============================================================================
// End of Class:
//=============================================================================

};

//-----------------------------------------------------------------------------
// Define pointer type:
//-----------------------------------------------------------------------------

typedef db_dblp_c *db_dblp_t;

//-----------------------------------------------------------------------------
// Define null pointer:
//-----------------------------------------------------------------------------

#define DB_DBLP_NULL (static_cast<db_dblp_t>(0))


//=============================================================================
// End of Include File:
//=============================================================================

#endif

