// ============================================================================
// Project:	Deductive Database
// Filename:	eval_dblp.h
// Purpose:	Bottom-Up Evaluation for DBLP Example, Ver 3: Evaluator
// Last Change:	11.06.2016
// Language:	C++
// EMail:	brass@informatik.uni-halle.de
// WWW:		http://www.informatik.uni-halle.de/~brass/
// Address:	Feldschloesschen 15, D-06120 Halle (Saale), GERMANY
// Copyright:	(c) 2015-2016 by Stefan Brass
// License:	See file "LICENSE" for copying conditions.
// Note:	There is no warranty at all - this code may contain bugs.
// ============================================================================


//=============================================================================
// Include File Frame:
//=============================================================================

#ifndef	EVAL_DBLP_INCLUDED
#define EVAL_DBLP_INCLUDED

//=============================================================================
// Used Types and Macros:
//=============================================================================

#ifndef VER_INCLUDED
#include "ver.h"
#endif

#ifndef STR_INCLUDED
#include "str.h"
#endif

#ifndef CHECK_INCLUDED
#include "check.h"
#endif

#ifndef STACK_INCLUDED
#include "stack.h"
#endif

#ifndef CUR_2_INCLUDED
#include "cur_2.h"
#endif

#ifndef CUR_N_N_INCLUDED
#include "cur_n_n.h"
#endif

#ifndef DB_DBLP_INCLUDED
#include "db_dblp.h"
#endif

//=============================================================================
// Private Constants:
//=============================================================================

//-----------------------------------------------------------------------------
// EVAL_DBLP_MAGIC: Magic number (identifies objects of this class).
//-----------------------------------------------------------------------------

static const long EVAL_DBLP_MAGIC = 0x4544500AL;	// 'EDP\n'


//=============================================================================
// Bottom-Up Evalutation (DBLP Benchmark) - Ver. 3: Evaluator
//=============================================================================

class eval_dblp_c {
	public:

//-----------------------------------------------------------------------------
// Constructor, Destructor:
//-----------------------------------------------------------------------------

	public:

	// Constructor:
	eval_dblp_c(db_dblp_t db);

	// Destructor:
	~eval_dblp_c();

//-----------------------------------------------------------------------------
// (Object) Methods:
//-----------------------------------------------------------------------------

	// Open cursor.
	void open();

	// fetch: Get next tuple, returns false if there is no further tuple.
	bool fetch();

	// answer(v2_1,v2_2,v2_3,v2_4,v2_5)

	// c1: Return value for column 1 of current row in cursor.
	inline str_t c1() {
		if(v2_1 < 0)
			return STR_NULL;
		return DB->doc_ids.get(v2_1);
	}

	// c2: Return value for column 2 of current row in cursor.
	inline str_t c2() {
		if(v2_2 < 0)
			return STR_NULL;
		return DB->str_values.get(v2_2);
	}

	// c3: Return value for column 3 of current row in cursor.
	inline str_t c3() {
		if(v2_3 < 0)
			return STR_NULL;
		return DB->str_values.get(v2_3);
	}

	// c4: Return value for column 4 of current row in cursor.
	inline str_t c4() {
		if(v2_4 < 0)
			return STR_NULL;
		return DB->str_values.get(v2_4);
	}

	// c5: Return value for column 5 of current row in cursor.
	inline str_t c5() {
		if(v2_5 < 0)
			return STR_NULL;
		return DB->str_values.get(v2_5);
	}

	// close: Close cursor.
	inline void close() {
	}

//-----------------------------------------------------------------------------
// Copy-constructor and assignment operator are not supported for this class:
//-----------------------------------------------------------------------------

	private:

	// These methods are not implemented:
	eval_dblp_c(const eval_dblp_c& obj);
	eval_dblp_c& operator=(const eval_dblp_c& obj);


//-----------------------------------------------------------------------------
// Debugging Support:
//-----------------------------------------------------------------------------

#ifdef	VER_DEBUG
	// Integrity check:
	public:
	str_t	check() const;

	// Magic number (identifies objects of this class for debugging).
	private:
	long	Magic;		// Must be "EVAL_DBLP_MAGIC".
#endif


#ifdef	VER_DUMP
	public:
	// Display data structure:
	void dump(str_t headline = STR_NULL) const;
#endif

//-----------------------------------------------------------------------------
// Private Object Members:
//-----------------------------------------------------------------------------

	private:

//-----------------------------------------------------------------------------
// Database:
//-----------------------------------------------------------------------------

	db_dblp_t DB;	// Object with domain and EDB relation data.

//-----------------------------------------------------------------------------
// Variables to hold values during the bottom-up evaluation:
//-----------------------------------------------------------------------------

	int v2_1;
	int v2_2;
	int v2_3;
	int v2_4;
	int v2_5;

//-----------------------------------------------------------------------------
// Cursors for loops during the bottom-up evaluation:
//-----------------------------------------------------------------------------

	cur_2_c   c2_1_fbf;	// att(A,title,B),
	cur_n_n_c c2_2_bbf;	// att(A,year,C),
	cur_n_n_c c2_3_bbf;	// att(A,author,D),
	cur_n_n_c c2_4_bbf;	// att(A,month,E).

//-----------------------------------------------------------------------------
// Backtrack stack for the bottom-up evaluation:
//-----------------------------------------------------------------------------

	stack_c<int> backtrack_stack;


//=============================================================================
// End of Class:
//=============================================================================

};


//-----------------------------------------------------------------------------
// Define pointer type:
//-----------------------------------------------------------------------------

typedef eval_dblp_c *eval_dblp_t;

//-----------------------------------------------------------------------------
// Define null pointer:
//-----------------------------------------------------------------------------

#define EVAL_DBLP_NULL (static_cast<eval_dblp_t>(0))



//=============================================================================
// End of Include File:
//=============================================================================

#endif

