// ============================================================================
// Project:	Deductive Database
// Filename:	idiv.h
// Purpose:	Macros for rounded integer division and percent
// Last Change:	31.10.2016
// Language:	C++
// EMail:	brass@informatik.uni-halle.de
// WWW:		http://www.informatik.uni-halle.de/~brass/
// Address:	Feldschloesschen 15, D-06120 Halle (Saale), GERMANY
// Copyright:	(c) 2015-2016 by Stefan Brass
// License:	See file "LICENSE" for copying conditions.
// Note:	There is no warranty at all - this code may contain bugs.
// ============================================================================


//=============================================================================
// Include File Frame:
//=============================================================================

#ifndef	IDIV_INCLUDED
#define IDIV_INCLUDED


//=============================================================================
// Used Types and Macros:
//=============================================================================

#ifndef VER_INCLUDED
#include "ver.h"
#endif

#ifndef CHECK_INCLUDED
#include "check.h"
#endif


//=============================================================================
// Defined Types and Macros:
//=============================================================================

// Note that all macros work only for non-negative numbers.
// For negative numbers, C and C++ do not completely prescribe the result
// of integer division. It would be more complicated to get a portable result.

//-----------------------------------------------------------------------------
// Integer division, round result (works only for non-negative numbers):
//-----------------------------------------------------------------------------

#define IDIV_ROUND(X,Y)	(((X)+(Y)/2)/(Y))

//-----------------------------------------------------------------------------
// Integer division, round up (works only for non-negative numbers):
//-----------------------------------------------------------------------------

#define IDIV_CEILING(X,Y)	(((X)+(Y)-1)/(Y))

//-----------------------------------------------------------------------------
// Integer division with one decimal place, whole part:
//-----------------------------------------------------------------------------

#define IDIV_WHOLE(X,Y)	((((X)*10+(Y)/2)/(Y))/10)

//-----------------------------------------------------------------------------
// Integer division with one decimal place, tenth:
//-----------------------------------------------------------------------------

//#define IDIV_TENTH(X,Y)	((((X)*10+(Y)/2)/(Y))%10)
#define IDIV_TENTH(X,Y)	(((((X)-IDIV_WHOLE((X),(Y))*(Y)) * 10 + (Y)/2)) / (Y))
	// In this way, we do not have to multiply X by 10,
	// which is important if X is large (near the integer limit).

//-----------------------------------------------------------------------------
// Compute Percentage, round result:
//-----------------------------------------------------------------------------

#define IDIV_PERCENT(X,Y)	(((X)*100+((Y)/2))/(Y))


//=============================================================================
// End of Include File:
//=============================================================================

#endif

