// ============================================================================
// Project:	Deductive Database
// Filename:	in.h
// Purpose:	Abstract Input Source (e.g. File, CGI, Keyboard)
// Last Change:	25.08.2015
// Language:	C++
// EMail:	brass@informatik.uni-halle.de
// WWW:		http://www.informatik.uni-halle.de/~brass/
// Address:	Feldschloesschen 15, D-06120 Halle (Saale), GERMANY
// Copyright:	(c) 1998-2015 by Stefan Brass
// License:	See file "LICENSE" for copying conditions.
// Note:	There is no warranty at all - this code may contain bugs.
// ============================================================================


//=============================================================================
// Include File Frame:
//=============================================================================

#ifndef	IN_INCLUDED
#define IN_INCLUDED

//=============================================================================
// Used Types and Macros:
//=============================================================================

#ifndef STR_INCLUDED
#include "str.h"
#endif

#ifndef CHECK_INCLUDED
#include "check.h"
#endif

//=============================================================================
// Class for Input Source:
//=============================================================================

class in_c {
	public:

//-----------------------------------------------------------------------------
// Destructor:
//-----------------------------------------------------------------------------

	virtual inline ~in_c()
		{};

//-----------------------------------------------------------------------------
// read: Read characters from input file into buffer.
//-----------------------------------------------------------------------------

	virtual int	read(char *buf, int size) = 0;

//-----------------------------------------------------------------------------
// is_open: This is really an open input stream (i.e. opening was successful).
//-----------------------------------------------------------------------------

	virtual bool	is_open() = 0;

//-----------------------------------------------------------------------------
// close: Close input file explicitly.
//-----------------------------------------------------------------------------

	virtual bool	close() = 0;

//-----------------------------------------------------------------------------
// filename: Name of this file (if it is a file).
//-----------------------------------------------------------------------------

	virtual str_t	filename() = 0;

//=============================================================================
// End of Class:
//=============================================================================

};

//-----------------------------------------------------------------------------
// Define pointer type:
//-----------------------------------------------------------------------------

typedef in_c *in_t;

//-----------------------------------------------------------------------------
// Define null pointer:
//-----------------------------------------------------------------------------

#define IN_NULL (static_cast<in_t>(0))

//=============================================================================
// End of File:
//=============================================================================

#endif

