// ============================================================================
// Project:	Deductive Database
// Filename:	in_cgi.h
// Purpose:	Get Input Via Common Gateway Interface (CGI) - Method POST
// Last Change:	27.08.2015
// Language:	C++
// EMail:	brass@informatik.uni-halle.de
// WWW:		http://www.informatik.uni-halle.de/~brass/
// Address:	Feldschloesschen 15, D-06120 Halle (Saale), GERMANY
// Copyright:	(c) 1998-2015 by Stefan Brass
// License:	See file "LICENSE" for copying conditions.
// Note:	There is no warranty at all - this code may contain bugs.
// ============================================================================


//=============================================================================
// Include File Frame:
//=============================================================================

#ifndef	IN_CGI_INCLUDED
#define IN_CGI_INCLUDED

//=============================================================================
// Used Types and Macros:
//=============================================================================

#ifndef STR_INCLUDED
#include "str.h"
#endif

#ifndef CHECK_INCLUDED
#include "check.h"
#endif

#ifndef IN_INCLUDED
#include "in.h"
#endif

//=============================================================================
// Class for Input File:
//=============================================================================

class in_cgi_c : public in_c {

	public:

//-----------------------------------------------------------------------------
// Constructor:
//-----------------------------------------------------------------------------

	in_cgi_c();

//-----------------------------------------------------------------------------
// Destructor:
//-----------------------------------------------------------------------------

	~in_cgi_c();

//-----------------------------------------------------------------------------
// close: Close input explicitly.
//-----------------------------------------------------------------------------

	bool	close();

//-----------------------------------------------------------------------------
// read: Read characters from input file into buffer.
//-----------------------------------------------------------------------------

	int	read(char *buf, int size);

//-----------------------------------------------------------------------------
// is_open: Is this input stream currently open?
//-----------------------------------------------------------------------------

	inline bool	is_open()
		{ return(BufPtr != 0); }

//-----------------------------------------------------------------------------
// filename: Name of this "file" (None).
//-----------------------------------------------------------------------------

	inline str_t	filename()
		{ return STR_NULL; }

//=============================================================================
// Private Class Members:
//=============================================================================

	private:

//-----------------------------------------------------------------------------
// Buf: Input buffer.
//-----------------------------------------------------------------------------

	char*	Buf;

//-----------------------------------------------------------------------------
// BufSize: Size of input buffer.
//-----------------------------------------------------------------------------

	int	BufSize;

//-----------------------------------------------------------------------------
// BufPtr: Pointer to character in buffer which will be returned on next read.
//-----------------------------------------------------------------------------

	char*	BufPtr;

//=============================================================================
// Auxillary Functions:
//=============================================================================

	private:

//-----------------------------------------------------------------------------
// input_size: Check the relevant environment variables, return input size.
//-----------------------------------------------------------------------------

	int input_size();

//-----------------------------------------------------------------------------
// decode: Decode a HTTP-string.
//-----------------------------------------------------------------------------

	bool decode(char* buf);

//-----------------------------------------------------------------------------
// hex: Return value of hexadecimal digit (-1000 if not valid).
//-----------------------------------------------------------------------------

	int hex(char c);

//=============================================================================
// End of Class:
//=============================================================================

};

//-----------------------------------------------------------------------------
// Define pointer type:
//-----------------------------------------------------------------------------

typedef in_cgi_c *in_cgi_t;

//-----------------------------------------------------------------------------
// Define null pointer:
//-----------------------------------------------------------------------------

#define IN_CGI_NULL (static_cast<in_cgi_t>(0))


//=============================================================================
// End of File:
//=============================================================================

#endif

