// ============================================================================
// Project:	Deductive Database
// Filename:	in_kbd.h
// Purpose:	Get Input Interactively (From Keyboard)
// Last Change:	27.08.2015
// Language:	C++
// EMail:	brass@informatik.uni-halle.de
// WWW:		http://www.informatik.uni-halle.de/~brass/
// Address:	Feldschloesschen 15, D-06120 Halle (Saale), GERMANY
// Copyright:	(c) 1997-2015 by Stefan Brass
// License:	See file "LICENSE" for copying conditions.
// Note:	There is no warranty at all - this code may contain bugs.
// ============================================================================


//=============================================================================
// Include File Frame:
//=============================================================================

#ifndef	IN_KBD_INCLUDED
#define IN_KBD_INCLUDED

//=============================================================================
// Used Types and Macros:
//=============================================================================

#ifndef STR_INCLUDED
#include "str.h"
#endif

#ifndef CHECK_INCLUDED
#include "check.h"
#endif

#ifndef IN_INCLUDED
#include "in.h"
#endif

//=============================================================================
// Class for Input File:
//=============================================================================

class in_kbd_c : public in_c {

	public:

//-----------------------------------------------------------------------------
// Methods:
//-----------------------------------------------------------------------------


	// Constructor:
	in_kbd_c();

	// Destructor:
	~in_kbd_c();

	// close: Close input explicitly.
	bool	close();

	// read: Read characters from input file into buffer.
	int	read(char *buf, int size);

	// is_open: Is this input stream currently open?
	inline bool	is_open()
		{ return Open; }

	// filename: Name of this "file".
	inline str_t	filename()
		{ return STR_NULL; }

//-----------------------------------------------------------------------------
// Private Members:
//-----------------------------------------------------------------------------

	private:

	// Open: Remember whether input was closed.
	bool	Open;

	// AtEOF: Was an end of file seen already?
	bool	AtEOF;

//=============================================================================
// End of Class:
//=============================================================================

};

//-----------------------------------------------------------------------------
// Define pointer type:
//-----------------------------------------------------------------------------

typedef in_kbd_c *in_kbd_t;

//-----------------------------------------------------------------------------
// Define null pointer:
//-----------------------------------------------------------------------------

#define IN_KBD_NULL (static_cast<in_kbd_t>(0))

//=============================================================================
// End of File:
//=============================================================================

#endif

