// ============================================================================
// Project:	Deductive Database
// Filename:	load_dblp.h
// Purpose:	Parse and load data from file "dblp.data" (Datalog facts).
// Last Change:	25.05.2016
// Language:	C++
// EMail:	brass@informatik.uni-halle.de
// WWW:		http://www.informatik.uni-halle.de/~brass/
// Address:	Feldschloesschen 15, D-06120 Halle (Saale), GERMANY
// Copyright:	(c) 2015-2016 by Stefan Brass
// License:	See file "LICENSE" for copying conditions.
// Note:	There is no warranty at all - this code may contain bugs.
// ============================================================================


//=============================================================================
// Include File Frame:
//=============================================================================

#ifndef	LOAD_DBLP_INCLUDED
#define LOAD_DBLP_INCLUDED

//=============================================================================
// Used Types and Macros:
//=============================================================================

#ifndef VER_INCLUDED
#include "ver.h"
#endif

#ifndef STR_INCLUDED
#include "str.h"
#endif

#ifndef CHECK_INCLUDED
#include "check.h"
#endif

#ifndef	ATOMTAB_INCLUDED
#include "atomtab.h"
#endif

#ifndef	STRTAB_INCLUDED
#include "strtab.h"
#endif

#ifndef	LIST_2_INCLUDED
#include "list_2.h"
#endif

#ifndef	REL_N_N_INCLUDED
#include "rel_n_n.h"
#endif


//=============================================================================
// Load Data, Version for "dblp.data" - Class With Only Static Members:
//=============================================================================

class load_dblp_c {
	public:

//-----------------------------------------------------------------------------
// Static Methods:
//-----------------------------------------------------------------------------

	// Load data:
	static bool load(
			// Name of data file to load:
			str_t filename,
			// Domains:
			strtab_t doc_ids, atomtab_t attrs, strtab_t str_values,
			// Constants:
			int title, int year, int author, int month,
			// Relations for EDB-literals:
			list_2_t  att_title,  rel_n_n_t att_year,
			rel_n_n_t att_author, rel_n_n_t att_month);

//-----------------------------------------------------------------------------
// This class has no objects:
//-----------------------------------------------------------------------------

	private:

	// Constructor:
	load_dblp_c();	// Not implemented

	// Destructor:
	~load_dblp_c();	// Not implemented

//-----------------------------------------------------------------------------
// Copy-constructor and assignment operator are not supported for this class:
//-----------------------------------------------------------------------------

	private:

	load_dblp_c(const load_dblp_c& obj);		// Not implemented
	load_dblp_c& operator=(const load_dblp_c& obj);	// Not implemented


//=============================================================================
// End of Class:
//=============================================================================

};


//=============================================================================
// End of Include File:
//=============================================================================

#endif

