// ============================================================================
// Project:	Deductive Database
// Filename:	page.h
// Purpose:	A memory page
// Last Change:	07.09.2015
// Language:	C++
// EMail:	brass@informatik.uni-halle.de
// WWW:		http://www.informatik.uni-halle.de/~brass/
// Address:	Feldschloesschen 15, D-06120 Halle (Saale), GERMANY
// Copyright:	(c) 2015 by Stefan Brass
// License:	See file "LICENSE" for copying conditions.
// Note:	There is no warranty at all - this code may contain bugs.
// ============================================================================


//=============================================================================
// Include File Frame:
//=============================================================================

#ifndef	PAGE_INCLUDED
#define PAGE_INCLUDED


//=============================================================================
// Used Types and Macros:
//=============================================================================

#ifndef VER_INCLUDED
#include "ver.h"
#endif

#ifndef CHECK_INCLUDED
#include "check.h"
#endif


//=============================================================================
// Basic Types and Macros for Memory Pages:
//=============================================================================

//-----------------------------------------------------------------------------
// Type of memory pages (pointer to start of VER_PAGESIZE bytes):
//-----------------------------------------------------------------------------

typedef void *page_t;

//-----------------------------------------------------------------------------
// Define null pointer:
//-----------------------------------------------------------------------------

#define PAGE_NULL (static_cast<page_t>(0))

//=============================================================================
// For Pages Used for Storing Objects of Only One Type:
//=============================================================================

//-----------------------------------------------------------------------------
// Size of a page in variables of a given type T:
//-----------------------------------------------------------------------------

#define PAGE_SIZE(TYPE)  \
	(VER_PAGESIZE/static_cast<int>(sizeof(TYPE)))

//=============================================================================
// For Pages Used for Storing Objects of Different Types:
//=============================================================================

//-----------------------------------------------------------------------------
// Union with maximal alignment requirements (unit for memory allocation):
//-----------------------------------------------------------------------------

union page_u {
	int i;
	long l;
	double d;
	str_t s;
	void *p;
	VER_INT64 i64;
};

//-----------------------------------------------------------------------------
// Define pointer type:
//-----------------------------------------------------------------------------

typedef union page_u *page_u_t;

//-----------------------------------------------------------------------------
// Define null pointer:
//-----------------------------------------------------------------------------

#define PAGE_U_NULL (static_cast<page_u_t>(0))

//-----------------------------------------------------------------------------
// Size of page units:
//-----------------------------------------------------------------------------

#define	PAGE_UNIT_BYTES \
	(static_cast<int>(sizeof(union page_u)))

//-----------------------------------------------------------------------------
// Size of a type in page units:
//-----------------------------------------------------------------------------

#define PAGE_UNITS(TYPE) \
	((static_cast<int>(sizeof(TYPE))+PAGE_UNIT_BYTES-1) / PAGE_UNIT_BYTES)

//-----------------------------------------------------------------------------
// Size of memory page in units:
//-----------------------------------------------------------------------------

#define PAGE_SIZE_UNITS \
	(VER_PAGESIZE / PAGE_UNIT_BYTES)


//=============================================================================
// Other macros:
//=============================================================================

//-----------------------------------------------------------------------------
// Translate number of pages to number of kilobytes:
//-----------------------------------------------------------------------------

#define PAGE_KB(NUM_PAGES) \
	(static_cast<int>( \
		(static_cast<long>(NUM_PAGES) * VER_PAGESIZE) / 1024))

//=============================================================================
// End of Include File:
//=============================================================================

#endif

