// ============================================================================
// Project:	Deductive Database
// Filename:	perf.h
// Purpose:	Performance measurement functions, especially timer.
// Last Change:	18.08.2015
// Language:	C++
// EMail:	brass@informatik.uni-halle.de
// WWW:		http://www.informatik.uni-halle.de/~brass/
// Address:	Feldschloesschen 15, D-06120 Halle (Saale), GERMANY
// Copyright:	(c) 2010-2015 by Stefan Brass
// License:	See file "LICENSE" for copying conditions.
// Note:	There is no warranty at all - this code may contain bugs.
// ============================================================================


//=============================================================================
// Include File Frame:
//=============================================================================

#ifndef	PERF_INCLUDED
#define PERF_INCLUDED

//=============================================================================
// Used Types and Macros:
//=============================================================================

#ifndef VER_INCLUDED
#include "ver.h"
#endif

#ifndef CHECK_INCLUDED
#include "check.h"
#endif


//=============================================================================
// Private Constants:
//=============================================================================

//-----------------------------------------------------------------------------
// PERF_MAGIC: Magic number (identifies objects of this class).
//-----------------------------------------------------------------------------

static const long PERF_MAGIC = 0x5045520AL;	// 'PER\n'


//=============================================================================
// Class for Time Measurements:
//=============================================================================

class perf_c {
	public:


//-----------------------------------------------------------------------------
// Constructor, Destructor:
//-----------------------------------------------------------------------------

	// Constructor:
	perf_c();

	// Destructor:
	~perf_c();

//-----------------------------------------------------------------------------
// (Object) Methods:
//-----------------------------------------------------------------------------

	// start: Start time measurement.
	void start();

	// stop: Stop time measurement.
	void stop();

	// realtime_msec: Time between last start() and stop() in milliseconds.
	long int realtime_msec();

	// cputime_msec: CPU time between last start() and stop() in millisecs.
	long int cputime_msec();


//-----------------------------------------------------------------------------
// Debugging Support:
//-----------------------------------------------------------------------------

#ifdef	VER_DEBUG
	// Integrity check:
	public:
	str_t	check() const;

	// Magic number (identifies objects of this class for debugging).
	private:
	long	Magic;		// Must be "PERF_MAGIC".
#endif

//-----------------------------------------------------------------------------
// Copy-constructor and assignment operator are not supported for this class:
//-----------------------------------------------------------------------------

	private:

	perf_c(const perf_c& perf);		// Not implemented
	perf_c& operator=(const perf_c& perf);	// Not implemented

//-----------------------------------------------------------------------------
// Private Object Members:
//-----------------------------------------------------------------------------

	private:

	// ID: Index in OS-dependent table of timer data (-1 if none free).
	int ID;


//=============================================================================
// End of Class:
//=============================================================================

};

//-----------------------------------------------------------------------------
// Define pointer type:
//-----------------------------------------------------------------------------

typedef perf_c *perf_t;

//-----------------------------------------------------------------------------
// Define null pointer:
//-----------------------------------------------------------------------------

#define PERF_NULL (static_cast<perf_t>(0))

//=============================================================================
// End of Include File:
//=============================================================================

#endif

