// ============================================================================
// Project:	Deductive Database
// Filename:	rel_ii.h
// Purpose:	Relation with two integer columns, no index.
// Last Change:	15.09.2015
// Language:	C++
// EMail:	brass@informatik.uni-halle.de
// WWW:		http://www.informatik.uni-halle.de/~brass/
// Address:	Feldschloesschen 15, D-06120 Halle (Saale), GERMANY
// Copyright:	(c) 2010-2015 by Stefan Brass
// License:	See file "LICENSE" for copying conditions.
// Note:	There is no warranty at all - this code may contain bugs.
// ============================================================================


//=============================================================================
// Include File Frame:
//=============================================================================

#ifndef	REL_II_INCLUDED
#define REL_II_INCLUDED

//=============================================================================
// Used Types and Macros:
//=============================================================================

#ifndef VER_INCLUDED
#include "ver.h"
#endif

#ifndef STR_INCLUDED
#include "str.h"
#endif

#ifndef CHECK_INCLUDED
#include "check.h"
#endif

#ifndef MPOOL_INCLUDED
#include "mpool.h"
#endif

#ifndef FLEXARR_INCLUDED
#include "flexarr.h"
#endif


//=============================================================================
// Private Constants:
//=============================================================================

//-----------------------------------------------------------------------------
// REL_II_MAGIC: Magic number (identifies objects of this class).
//-----------------------------------------------------------------------------

static const long REL_II_MAGIC = 0x5249490AL;	// 'RII\n'


//=============================================================================
// Class for Relation with two integer columns, no index:
//=============================================================================

class rel_ii_c {
	public:

//-----------------------------------------------------------------------------
// Constructor, Destructor:
//-----------------------------------------------------------------------------

	// Constructor:
	rel_ii_c(str_t rel_name);

	// Destructor:
	~rel_ii_c();

//-----------------------------------------------------------------------------
// (Object) Methods:
//-----------------------------------------------------------------------------

	// insert: Insert tuple into relation.
	bool insert(int c1, int c2);

	// name: Get name of this relation.
	str_t name() {
		CHECK_VALID("rel_ii_c::name");
		return Name;
	}

	// num_rows: Get number of rows in this relation.
	int num_rows() {
		CHECK_VALID("rel_ii_c::num_rows");
		return NumRows;
	}

	// num_pages: Get number of memory pages used for this relation.
	int num_pages() {
		CHECK_VALID("rel_ii_c::num_pages");
		return MemPool.num_alloc_pages();
	}

//-----------------------------------------------------------------------------
// Debugging Support:
//-----------------------------------------------------------------------------

#ifdef	VER_DEBUG
	public:
	// Integrity check:
	str_t	check() const;

	private:
	// Magic number (identifies objects of this class for debugging).
	long	Magic;		// Must be "REL_II_MAGIC".
#endif

#ifdef	VER_DUMP
	public:
	// Display data structure:
	void dump(str_t headline = STR_NULL) const;
#endif

//-----------------------------------------------------------------------------
// Copy-constructor and assignment operator are not supported for this class:
//-----------------------------------------------------------------------------

	private:

	rel_ii_c(const rel_ii_c& obj);			// Not implemented
	rel_ii_c& operator=(const rel_ii_c& obj);	// Not implemented

//-----------------------------------------------------------------------------
// This class works together with the corresponding cursor class:
//-----------------------------------------------------------------------------

	friend class cur_ii_c;

//-----------------------------------------------------------------------------
// Private Object Members:
//-----------------------------------------------------------------------------

	private:

	// Memory Pool for Allocating Storage Pages:
	mpool_c MemPool;

	// Array which contains the tuples (always two entries for two cols):
	flexarr_c<int> TupleArr;

	// Name of this relation.
	str_t Name;

	// Current number of rows.
	int NumRows;

//=============================================================================
// End of Class:
//=============================================================================

};

//-----------------------------------------------------------------------------
// Define pointer type:
//-----------------------------------------------------------------------------

typedef rel_ii_c *rel_ii_t;

//-----------------------------------------------------------------------------
// Define null pointer:
//-----------------------------------------------------------------------------

#define REL_II_NULL (static_cast<rel_ii_t>(0))

//=============================================================================
// End of Include File:
//=============================================================================

#endif

