// ============================================================================
// Project:	Deductive Database
// Filename:	row_1.h
// Purpose:	Tuple/Table Row with one integer column.
// Last Change:	18.11.2015
// Language:	C++
// EMail:	brass@informatik.uni-halle.de
// WWW:		http://www.informatik.uni-halle.de/~brass/
// Address:	Feldschloesschen 15, D-06120 Halle (Saale), GERMANY
// Copyright:	(c) 2010-2015 by Stefan Brass
// License:	See file "LICENSE" for copying conditions.
// Note:	There is no warranty at all - this code may contain bugs.
// ============================================================================


//=============================================================================
// Include File Frame:
//=============================================================================

#ifndef	ROW_1_INCLUDED
#define ROW_1_INCLUDED

//=============================================================================
// Used Types and Macros:
//=============================================================================

#ifndef VER_INCLUDED
#include "ver.h"
#endif

#ifndef STR_INCLUDED
#include "str.h"
#endif

#ifndef CHECK_INCLUDED
#include "check.h"
#endif


//=============================================================================
// Private Constants:
//=============================================================================

//-----------------------------------------------------------------------------
// ROW_1_MAGIC: Magic number (identifies objects of this class).
//-----------------------------------------------------------------------------

static const long ROW_1_MAGIC = 0x525F310AL;	// 'R_1\n'

//=============================================================================
// Class for Relation with two integer columns, no index:
//=============================================================================

class row_1_c {
	public:

//-----------------------------------------------------------------------------
// Constants:
//-----------------------------------------------------------------------------

	static const int NUM_COLS = 1;

//-----------------------------------------------------------------------------
// Constructor, Destructor:
//-----------------------------------------------------------------------------

	// Constructor:
	row_1_c(int col1) : C1(col1) {

		// Set Magic number:
		CHECK_CODE(Magic = ROW_1_MAGIC);
	}

	// Destructor:
	~row_1_c() {
		// Check this object:
		CHECK_VALID("row_1_c::destructor");

		// Make this object invalid:
		CHECK_CODE(Magic = 0);
	}

	// Copy-constructor:
	row_1_c(const row_1_c& row) : C1(row.C1) {

		// Set Magic number:
		CHECK_CODE(Magic = ROW_1_MAGIC);
	}

//-----------------------------------------------------------------------------
// (Object) Methods:
//-----------------------------------------------------------------------------

	// c1: Get value of first (and only) column.
	int c1() {
		// Check this object:
		CHECK_VALID("row_1_c::c1");

		// Return value of column 1:
		return C1;
	}

	// hash: Compute a hash value.
	int hash() {
		// Check this object:
		CHECK_VALID("row_1_c::hash");

		// There is only a single integer, this is used as hash value:
		if(C1 >= 0)
			return C1;
		else {
			int hash_val = -(C1+1);
			CHECK(hash_val >= 0,
					"row_1_c::hash: Value still negative");
			return hash_val;
		}
	}


	// ==: equality test.
	bool operator==(const row_1_c& other) {
		// Check this object:
		CHECK_VALID("row_1_c::==");

		// Objects are equal if they have the same value in C1:
		return C1 == other.C1;
	}

	// =: Assignment Operator.
	row_1_c& operator=(const row_1_c& other) {
		// Copy data value:
		C1 = other.C1;

		// Set Magic number:
		CHECK_CODE(Magic = ROW_1_MAGIC);

		// Return this object (standard interface of assignment op):
		return *this;
	}

	// Create a string representation in a given buffer:
	void to_string(char *buf, int buf_len);
	
//-----------------------------------------------------------------------------
// Debugging Support:
//-----------------------------------------------------------------------------

#ifdef	VER_DEBUG
	public:
	// Integrity check:
	str_t	check() const;

	private:
	// Magic number (identifies objects of this class for debugging).
	long	Magic;		// Must be "ROW_1_MAGIC".
#endif

#ifdef	VER_DUMP
	public:
	// Display data structure:
	void dump(str_t headline = STR_NULL) const;
#endif


//-----------------------------------------------------------------------------
// Private Object Members:
//-----------------------------------------------------------------------------

	private:

	// Column value:
	int C1;

//=============================================================================
// End of Class:
//=============================================================================

};

//-----------------------------------------------------------------------------
// Define pointer type:
//-----------------------------------------------------------------------------

typedef row_1_c *row_1_t;

//-----------------------------------------------------------------------------
// Define null pointer:
//-----------------------------------------------------------------------------

#define ROW_1_NULL (static_cast<row_1_t>(0))

//=============================================================================
// End of Include File:
//=============================================================================

#endif

