// ============================================================================
// Project:	Deductive Database
// Filename:	row_2.cpp
// Purpose:	Tuple with two integer columns.
// Last Change:	07.10.2015
// Language:	C++
// EMail:	brass@informatik.uni-halle.de
// WWW:		http://www.informatik.uni-halle.de/~brass/
// Address:	Feldschloesschen 15, D-06120 Halle (Saale), GERMANY
// Copyright:	(c) 2010-2015 by Stefan Brass
// License:	See file "LICENSE" for copying conditions.
// Note:	There is no warranty at all - this code may contain bugs.
// ============================================================================


//=============================================================================
// Interface:
//=============================================================================

#include "ver.h"
#include "str.h"
#ifdef	VER_DUMP
#include "err.h"
#endif
#include "check.h"

#include "row_2.h"


//=============================================================================
// Exported Methods:
//=============================================================================

//-----------------------------------------------------------------------------
// to_string: Create a string representation in a given buffer:
//-----------------------------------------------------------------------------

void row_2_c::to_string(char *buf, int buf_len)
{
	if(buf_len > 0) {
		str_cpy(buf, buf_len, "(");
		buf++;
		buf_len--;
	}
	if(buf_len > 0) {
		int chars = str_int(buf, buf_len, C1);
		buf += chars;
		buf_len -= chars;
	}
	if(buf_len > 0) {
		str_cpy(buf, buf_len, ",");
		buf++;
		buf_len--;
	}
	if(buf_len > 0) {
		int chars = str_int(buf, buf_len, C2);
		buf += chars;
		buf_len -= chars;
	}
	if(buf_len > 0) {
		str_cpy(buf, buf_len, ")");
		buf++;
		buf_len--;
	}
}

//=============================================================================
// Debugging Support:
//=============================================================================

//-----------------------------------------------------------------------------
// check: Check the integrity of this object, return error message or NULL.
//-----------------------------------------------------------------------------

#ifdef	VER_DEBUG
str_t	row_2_c::check() const
{
	// Check magic number:
	if(Magic != ROW_2_MAGIC)
		return "wrong magic number";

	// Ok:
	return STR_NULL;
}
#endif


//-----------------------------------------------------------------------------
// dump: Show data structure.
//-----------------------------------------------------------------------------

#ifdef	VER_DUMP
void	row_2_c::dump(str_t headline) const
{
	// Headline:
	if(headline == STR_NULL)
		headline = "Tuple (with two integer columns)";
	err_c::dump_begin(headline);

	// Column Values:
	err_c::dump_str("Column 1:    ");
	err_c::dump_int(C1);
	err_c::dump_nl();
	err_c::dump_str("Column 2:    ");
	err_c::dump_int(C2);
	err_c::dump_nl();

	// Storage information:
	err_c::dump_str("Address:     ");
	err_c::dump_ptr(this);
	err_c::dump_nl();
	err_c::dump_str("Object Size: ");
	err_c::dump_uint(static_cast<unsigned int>(sizeof(row_2_c)));
	err_c::dump_str(" Bytes");
	err_c::dump_nl();
	err_c::dump_nl();

	// Done:
	err_c::dump_end();
}
#endif

