// ============================================================================
// Project:	Deductive Database
// Filename:	set_1e.h
// Purpose:	Set of rows with one int column (binding b) - Extendible Hash
// Last Change:	27.09.2016
// Language:	C++
// EMail:	brass@informatik.uni-halle.de
// WWW:		http://www.informatik.uni-halle.de/~brass/
// Address:	Feldschloesschen 15, D-06120 Halle (Saale), GERMANY
// Copyright:	(c) 2016 by Stefan Brass
// License:	See file "LICENSE" for copying conditions.
// Note:	There is no warranty at all - this code may contain bugs.
// ============================================================================


//=============================================================================
// Include File Frame:
//=============================================================================

#ifndef	SET_1E_INCLUDED
#define SET_1E_INCLUDED

//=============================================================================
// Used Types and Macros:
//=============================================================================

#ifndef VER_INCLUDED
#include "ver.h"
#endif

#ifndef STR_INCLUDED
#include "str.h"
#endif

#ifndef CHECK_INCLUDED
#include "check.h"
#endif

#ifndef ROW_1_INCLUDED
#include "row_1.h"
#endif

#ifndef SET_E_INCLUDED
#include "set_e.h"
#endif


//=============================================================================
// Set of Rows with one int column / Relation with binding pattern b:
//=============================================================================

class set_1e_c: public set_e_c<row_1_c> {

	public:

//-----------------------------------------------------------------------------
// Constructor, Destructor:
//-----------------------------------------------------------------------------

	// Constructor:
	set_1e_c(str_t set_name);

	// Destructor:
	~set_1e_c();

//-----------------------------------------------------------------------------
// Copy-constructor and assignment operator are not supported for this class:
//-----------------------------------------------------------------------------

	private:

	set_1e_c(const set_1e_c& obj);			// Not implemented
	set_1e_c& operator=(const set_1e_c& obj);	// Not implemented


//=============================================================================
// (Object) Methods:
//=============================================================================

	public:

//-----------------------------------------------------------------------------
// insert: Insert an element into the set (returns true if successful).
//-----------------------------------------------------------------------------

	// This method returns true if the row was not already in the set
	// and was successfully inserted (i.e. if it is not a duplicate).
	// It returns false if the row was already in the set (duplicate).
	// It also returns false if it was impossible to insert the element
	// because there was not enough memory.

	inline bool insert(int col1) {
		return set_e_c<row_1_c>::insert(row_1_c(col1));
	}

//-----------------------------------------------------------------------------
// contains: Check whether an element is contained in the set.
//-----------------------------------------------------------------------------

	inline bool contains(int col1) const {
		return set_e_c<row_1_c>::contains(row_1_c(col1));
	}

//=============================================================================
// End of Class Template:
//=============================================================================

};


//-----------------------------------------------------------------------------
// Define pointer type:
//-----------------------------------------------------------------------------

typedef set_1e_c *set_1e_t;

//-----------------------------------------------------------------------------
// Define null pointer:
//-----------------------------------------------------------------------------

#define SET_1E_NULL (static_cast<set_1e_t>(0))


//=============================================================================
// End of Include File:
//=============================================================================

#endif

