// ============================================================================
// Project:	Deductive Database
// Filename:	syn.cpp
// Purpose:	Syntax Error Messages
// Last Change:	06.10.2016
// Language:	C++
// EMail:	brass@informatik.uni-halle.de
// WWW:		http://www.informatik.uni-halle.de/~brass/
// Address:	Feldschloesschen 15, D-06120 Halle (Saale), GERMANY
// Copyright:	(c) 2015-2016 by Stefan Brass
// License:	See file "LICENSE" for copying conditions.
// Note:	There is no warranty at all - this code may contain bugs.
// ============================================================================


//=============================================================================
// Interface:
//=============================================================================

#include "ver.h"
#include "check.h"
#include "syn.h"

//=============================================================================
// Exported Functions:
//=============================================================================

//-----------------------------------------------------------------------------
// syn_valid: Is the parameter a valid element of the enumeration type?
//-----------------------------------------------------------------------------

#ifdef	VER_DEBUG

bool	syn_valid(syn_t syn)
{
	return( syn == SYN_PRED_EXPECTED ||
		syn == SYN_UNKNOWN_PRED ||
		syn == SYN_LPAREN_EXPECTED ||
		syn == SYN_RPAREN_EXPECTED ||
		syn == SYN_COMMA_EXPECTED ||
		syn == SYN_FULLSTOP_EXPECTED ||
		syn == SYN_ARG_MUST_BE_STR ||
		syn == SYN_ARG_MUST_BE_ATOM ||
		syn == SYN_ARG_MUST_BE_ATOM_OR_STR ||
		syn == SYN_ARG_MUST_BE_INT ||
		syn == SYN_ARG_CANNOT_BE_NEGATIVE ||
		syn == SYN_INVALID_ARG_TYPE ||
		syn == SYN_ILLEGAL_CHAR ||
		syn == SYN_MISSINGQ ||
		syn == SYN_NOSPACE ||
		syn == SYN_INT_OVERFLOW ||
		syn == SYN_BAD_ESCAPE_SEQ
	);
}

#endif

//-----------------------------------------------------------------------------
// syn_text: Text of the error message.
//-----------------------------------------------------------------------------

str_t	syn_text(syn_t syn)
{
	str_t text;
	
	switch(syn) {
		case SYN_PRED_EXPECTED:
			text = "predicate expected";
			break;
		case SYN_UNKNOWN_PRED:
			text = "unknown predicate";
			break;
		case SYN_LPAREN_EXPECTED:
			text = "left parenthesis '(' expected";
			break;
		case SYN_RPAREN_EXPECTED:
			text = "right parenthesis ')' expected";
			break;
		case SYN_COMMA_EXPECTED:
			text = "comma ',' expected";
			break;
		case SYN_FULLSTOP_EXPECTED:
			text =
			    "full stop expected ('.' followed by white space)";
			break;
		case SYN_ARG_MUST_BE_STR:
			text = "this predicate argument must be a string";
			break;
		case SYN_ARG_MUST_BE_ATOM:
			text = "this predicate argument must be an atom";
			break;
		case SYN_ARG_MUST_BE_ATOM_OR_STR:
			text =
			"this predicate argument must be an atom or a string";
			break;
		case SYN_ARG_MUST_BE_INT:
			text = "this predicate argument must be an integer";
			break;
		case SYN_ARG_CANNOT_BE_NEGATIVE:
			text = "this predicate argument cannot be negative";
			break;
		case SYN_INVALID_ARG_TYPE:
			text = "this predicate argument has an invalid type";
			break;
		case SYN_ILLEGAL_CHAR:
			text = "illegal character in input file";
			break;
		case SYN_MISSINGQ:
			text = "missing quote in string constant";
			break;
		case SYN_NOSPACE:
			text = "a full stop must be followed by whitespace";
			break;
		case SYN_INT_OVERFLOW:
			text = "integer constant too large (overflow)";
			break;
		case SYN_BAD_ESCAPE_SEQ:
			text = "bad escape sequence, unexpected char after \\";
			break;
		default:
			CHECK_IMPOSSIBLE("syn_text: unknown syntax error");
			text = "unknown error";
			break;
	}
	
	return text;
}

