// ============================================================================
// Project:	Deductive Database
// Filename:	tok.cpp
// Purpose:	Token types
// Last Change:	06.10.2016
// Language:	C++
// EMail:	brass@informatik.uni-halle.de
// WWW:		http://www.informatik.uni-halle.de/~brass/
// Address:	Feldschloesschen 15, D-06120 Halle (Saale), GERMANY
// Copyright:	(c) 1998-2016 by Stefan Brass
// License:	See file "LICENSE" for copying conditions.
// Note:	There is no warranty at all - this code may contain bugs.
// ============================================================================


//=============================================================================
// Interface:
//=============================================================================

#include "ver.h"
#include "str.h"
#include "check.h"
#include "tok.h"

//=============================================================================
// Exported Functions:
//=============================================================================

//-----------------------------------------------------------------------------
// tok_valid: Is the parameter a valid element of the enumeration type?
//-----------------------------------------------------------------------------

#ifdef	VER_DEBUG

bool	tok_valid(tok_t tok)
{
	return( tok == TOK_EOF ||
		tok == TOK_FULLSTOP ||
		tok == TOK_ATOM ||
		tok == TOK_STR ||
		tok == TOK_INT ||
		tok == TOK_VAR ||
		tok == TOK_ANONVAR ||
		tok == TOK_COMMA ||
		tok == TOK_LPAREN ||
		tok == TOK_RPAREN ||
		tok == TOK_IF ||
		tok == TOK_AND ||
		tok == TOK_OR ||
		tok == TOK_NOT ||
		tok == TOK_QUERY ||
		tok == TOK_COLON ||
		tok == TOK_LESS ||
		tok == TOK_GREATER ||
		tok == TOK_MINUS ||
		tok == TOK_ILLEGAL_CHAR ||
		tok == TOK_MISSINGQ ||
		tok == TOK_NOSPACE ||
		tok == TOK_INT_OVERFLOW ||
		tok == TOK_BAD_ESCAPE_SEQ
	);
}

#endif

//-----------------------------------------------------------------------------
// tok_name: Name of token type.
//-----------------------------------------------------------------------------

str_t tok_name(tok_t tok)
{
	str_t name;
	
	switch(tok) {
		case TOK_EOF:
			name = "end of file";
			break;
		case TOK_FULLSTOP:
			name = "`.'";
			break;
		case TOK_ATOM:
			name = "atom";
			break;
		case TOK_STR:
			name = "string";
			break;
		case TOK_INT:
			name = "integer";
			break;
		case TOK_VAR:
			name = "variable";
			break;
		case TOK_ANONVAR:
			name = "the anonymous variable";
			break;
		case TOK_COMMA:
			name = "`,'";
			break;
		case TOK_LPAREN:
			name = "`('";
			break;
		case TOK_RPAREN:
			name = "`)'";
			break;
		case TOK_IF:
			name = "`<-'";
			break;
		case TOK_AND:
			name = "&";
			break;
		case TOK_OR:
			name = "|";
			break;
		case TOK_NOT:
			name = "not";
			break;
		case TOK_QUERY:
			name = "?";
			break;
		case TOK_COLON:
			name = ":";
			break;
		case TOK_LESS:
			name = "`<'";
			break;
		case TOK_GREATER:
			name = "`>'";
			break;
		case TOK_MINUS:
			name = "`-'";
			break;
		case TOK_ILLEGAL_CHAR:
			name = "illegal character";
			break;
		case TOK_MISSINGQ:
			name = "string with missing quote";
			break;
		case TOK_NOSPACE:
			name = "`.' not followed by whitespace";
			break;
		case TOK_INT_OVERFLOW:
			name = "bad integer constant (too large)";
			break;
		case TOK_BAD_ESCAPE_SEQ:
			name = "bad escape sequence in string constant";
			break;
		default:
			CHECK_IMPOSSIBLE("tok_name: unknown token");
			name = "unknown token";
			break;
	}
	
	return name;
}

