// ============================================================================
// Project:	Deductive Database
// Filename:	ver.h
// Purpose:	Version Settings
// Last Change:	06.10.2016
// Language:	C++
// EMail:	brass@informatik.uni-halle.de
// WWW:		http://www.informatik.uni-halle.de/~brass/
// Address:	Feldschloesschen 15, D-06120 Halle (Saale), GERMANY
// Copyright:	(c) 2001-2016 by Stefan Brass
// License:	See file "LICENSE" for copying conditions.
// Note:	There is no warranty at all - this code may contain bugs.
// ============================================================================


//=============================================================================
// Include File Frame:
//=============================================================================

#ifndef	VER_INCLUDED
#define VER_INCLUDED

//=============================================================================
// Basic Parameters:
//=============================================================================

//-----------------------------------------------------------------------------
// VER_PROG_NAME: What is the name of this program?
//-----------------------------------------------------------------------------

#define VER_PROG_NAME "ydb"

//-----------------------------------------------------------------------------
// VER_PROMPT: Prompt for input from keyboard (standard input).
//-----------------------------------------------------------------------------

#define VER_PROMPT "ydb> "

//-----------------------------------------------------------------------------
// VER_CGI: Should this program run as CGI program?
//-----------------------------------------------------------------------------

// #define VER_CGI

//-----------------------------------------------------------------------------
// VER_ALERT_FILE: Name of the file for error messages and warnings:
//-----------------------------------------------------------------------------

#define VER_ALERT_FILE "alert"

//-----------------------------------------------------------------------------
// VER_DUMP: Compile code for data structure dumps (for debugging/testing).
//-----------------------------------------------------------------------------

#define VER_DUMP

//-----------------------------------------------------------------------------
// VER_DEBUG_OUT_ENABLED: Compile code for debug output (other than dumps).
//-----------------------------------------------------------------------------

// #define VER_DEBUG_OUT_ENABLED

// Note that commenting out this symbol does not necessarily switch off
// all debug output. Debug output is also controlled by the debug flag
// in out_c. However, in benchmarks, when checking for this flag costs time,
// this is compiled in only if this symbol is defined.
// So comment out this symbol if doing benchmarks.

//-----------------------------------------------------------------------------
// VER_PAGESIZE: Size of a memory page in bytes.
//-----------------------------------------------------------------------------

#define VER_PAGESIZE 4096
// #define VER_PAGESIZE 8192

//-----------------------------------------------------------------------------
// VER_MAX_TIMERS: Number of timer_c objects existing at the same time.
//-----------------------------------------------------------------------------

#define VER_MAX_TIMERS 5

//-----------------------------------------------------------------------------
// VER_MAX_ATOMSIZE: Maximal length of atoms / identifiers.
//-----------------------------------------------------------------------------

#define VER_MAX_ATOMSIZE 20

//-----------------------------------------------------------------------------
// VER_MAX_PRED_ARGS: Maximal number of arguments of an EDB predicate.
//-----------------------------------------------------------------------------

#define VER_MAX_PRED_ARGS 10

//=============================================================================
// Parameters For Benchmarks:
//=============================================================================

//-----------------------------------------------------------------------------
// VER_WINE_ENABLED: Should the "Wine Ontology Benchmark" be compiled?
//-----------------------------------------------------------------------------

// Please note that this is a very long procedure.
// Compiling with g++ with -O2 is possible (at least on a machine with big
// memory), but takes a long time (about 10 minutes on a fast computer).
// Compiling with "Microsoft Visual Studio Community 2015" on a machine
// with 4 GB memory gives the error:
// "fatal error C1002: compiler is out of heap space in pass 2"

// #define VER_WINE_ENABLED

//-----------------------------------------------------------------------------
// Data File for the DBLP Benchmark:
//-----------------------------------------------------------------------------

#define VER_FILE_DBLP_TEST	"data/dblp_test.data"
// 125.434 KB (~122 MB)
#define VER_FILE_DBLP		"data/dblp.data"
#define VER_FILE_DBLP_BIG	"data/dblp_big.data"

//-----------------------------------------------------------------------------
// Data Files for the Join1 Benchmarks:
//-----------------------------------------------------------------------------

#define VER_FILE_JOIN1_TEST	"data/d1000_test.P"
#define VER_FILE_JOIN1_10K	"data/d1000_relsize10000_xsb_cyc.P"
#define VER_FILE_JOIN1_50K	"data/d1000_relsize50000_xsb_cyc.P"
#define VER_FILE_JOIN1_250K	"data/d1000_relsize250000_xsb_cyc.P"

//-----------------------------------------------------------------------------
// Data Files for the Transitive Closure Benchmarks:
//-----------------------------------------------------------------------------

#define VER_FILE_TC_TEST	"data/tc_test.P"
#define VER_FILE_TC_50K		"data/tc_d1000_parsize50000_xsb_nocyc.P"
#define VER_FILE_TC_50K_CYC	"data/tc_d1000_parsize50000_xsb_cyc.P"
#define VER_FILE_TC_250K	"data/tc_d1000_parsize250000_xsb_nocyc.P"
#define VER_FILE_TC_250K_CYC	"data/tc_d1000_parsize250000_xsb_cyc.P"
#define VER_FILE_TC_500K	"data/tc_d1000_parsize500000_xsb_nocyc.P"
#define VER_FILE_TC_500K_CYC	"data/tc_d1000_parsize500000_xsb_cyc.P"
#define	VER_FILE_TC_500K_D2	"data/tc_d2000_parsize500000_xsb_nocyc.P"
#define VER_FILE_TC_500K_D2_CYC	"data/tc_d2000_parsize500000_xsb_cyc.P"
#define VER_FILE_TC_1M_D2	"data/tc_d2000_parsize1000000_xsb_nocyc.P"
#define VER_FILE_TC_1M_D2_CYC	"data/tc_d2000_parsize1000000_xsb_cyc.P"

//-----------------------------------------------------------------------------
// Data File for the Wine Ontology Benchmark:
//-----------------------------------------------------------------------------

#define VER_FILE_WINE	"data/wine_data.P"

//=============================================================================
// Select an Operating System:
//=============================================================================

//-----------------------------------------------------------------------------
// VER_UNIX/VER_WINDOWS: Select operating system
//-----------------------------------------------------------------------------

// We allow that these constants are already defined on the command line:
#ifndef VER_UNIX
#ifndef VER_WINDOWS

// Select here which version you want if the makefile does not define it:
// #define VER_WINDOWS
// #define VER_UNIX
// If none defined, we choose automatically below.

#endif // VER_WINDOWS
#endif // VER_UNIX

//-----------------------------------------------------------------------------
// VER_UNIX/VER_WINDOWS: Automatic choice of operating system
//-----------------------------------------------------------------------------

// Still not defined? Check for typical Microsoft Symbols, else choose UNIX:
#ifndef VER_UNIX
#ifndef VER_WINDOWS

#if defined(_MSC_VER) || defined(_WINDOWS_) || defined(WIN32)
#define VER_WINDOWS
#else
#define VER_UNIX
#endif

#endif // VER_WINDOWS
#endif // VER_UNIX

//=============================================================================
// Settings for Debugging:
//=============================================================================

//-----------------------------------------------------------------------------
// VER_DEBUG: Check runtime assertions etc. (Debug Version).
//-----------------------------------------------------------------------------

// #define VER_DEBUG

// Automatic choice for Visual C++:
#ifdef _DEBUG
#define VER_DEBUG
#endif

//-----------------------------------------------------------------------------
// VER_FULLCHECK: Do also time-intensive integrity checks of data structures.
//-----------------------------------------------------------------------------

// #define VER_FULLCHECK

	// Note: This has only an effect if VER_DEBUG is also defined.
	// Otherwise all integrity checks are eliminated.

//-----------------------------------------------------------------------------
// VER_SIGNAL: Try to catch signals
//-----------------------------------------------------------------------------

#ifdef VER_UNIX
#define VER_SIGNAL
#endif


//=============================================================================
// Portability Definitions:
//=============================================================================

//-----------------------------------------------------------------------------
// VER_CS_ISO: Allow national characters according to ISO Latin 8859/1 code.
//-----------------------------------------------------------------------------

#ifdef VER_UNIX
#define VER_CS_ISO
#endif

//-----------------------------------------------------------------------------
// VER_CS_MSDOS: Allow national characters according to some MSDOS code.
//-----------------------------------------------------------------------------

// Only one of VER_CS_ISO and VER_CS_MSDOS may be defined.
// If none is defined, only ASCII characters are accepted.

// #define VER_CS_MSDOS

//-----------------------------------------------------------------------------
// VER_IOSTREAM: Use the iostream library (else: low level UNIX functions).
//-----------------------------------------------------------------------------

#ifndef VER_UNIX
#define VER_IOSTREAM
#endif

//-----------------------------------------------------------------------------
// Switch off Warning C4514: unreferenced inline function has been removed
//-----------------------------------------------------------------------------

// Microsoft Visual C++ 6.0 generates this warning in the highest warning
// level 4 when an include file defines an inline function not used in this
// source file. However, it might be used in a different source file which
// includes the same include file.

// #ifdef VER_WINDOWS
// #pragma warning (disable: 4514)
// #endif

//-----------------------------------------------------------------------------
// Mark unused arguments so that the warning is not printed:
//-----------------------------------------------------------------------------

#ifdef __GNUC__
#define VER_UNUSED(ARG)	ARG __attribute__((unused))
#else
#define VER_UNUSED(ARG) ARG
#endif

//-----------------------------------------------------------------------------
// VER_PRESS_RETURN: Do we need to wait at the end before the program ends?
//-----------------------------------------------------------------------------

#ifdef VER_WINDOWS
#define VER_PRESS_RETURN
#endif

//-----------------------------------------------------------------------------
// VER_BYTE: Data type for bytes in data blocks (sizeof must be 1).
//-----------------------------------------------------------------------------

typedef char VER_BYTE;

//-----------------------------------------------------------------------------
// VER_USHORT: Data type for 16 Bit numbers (unsigned), sizeof must be 2.
//-----------------------------------------------------------------------------

typedef unsigned short VER_USHORT;

//-----------------------------------------------------------------------------
// VER_INT: Data type for 32 Bit numbers, sizeof must be 4.
//-----------------------------------------------------------------------------

typedef int VER_INT;

//-----------------------------------------------------------------------------
// VER_UINT: Data type for unsigned 32 Bit numbers, sizeof must be 4.
//-----------------------------------------------------------------------------

typedef unsigned int VER_UINT;

//-----------------------------------------------------------------------------
// VER_UINT_BITS: Number of bits in an unsigned int (used for bitmaps).
//-----------------------------------------------------------------------------

#define VER_UINT_BITS	32

//-----------------------------------------------------------------------------
// VER_INT64: Data type for 64 Bit numbers (if compiled on 64 Bit machine).
//-----------------------------------------------------------------------------

// Must be sufficient to hold a pointer.

// gcc -pedantic complains:
// ver.h:240:14: warning: ISO C++ 1998 does not support ‘long long’

#ifdef __GNUC__
#include <stdint.h>
typedef int64_t VER_INT64;
#else
typedef long long VER_INT64;
#endif

//-----------------------------------------------------------------------------
// VER_UINT64: Data type for 64 Bit numbers (if compiled on 64 Bit machine).
//-----------------------------------------------------------------------------

	// Must be sufficient to hold a size_t value.

//typedef unsigned long long VER_UINT64;

//-----------------------------------------------------------------------------
// VER_SHORTOFFSET: How many shorts are n ints? Answer must be 2*n.
//-----------------------------------------------------------------------------

//#define VER_SHORTOFFSET(N)	((N)*2)
	// The Data part of a block is a union of VER_BYTE, VER_USHORT, VER_INT
	// arrays. Sometimes different types are used in a block, but one must
	// make sure that the same memory location is not used with different
	// types.

//=============================================================================
// End of Include File:
//=============================================================================

#endif

