// ============================================================================
// Project:	Deductive Database
// Filename:	bench_wine.cpp
// Purpose:	Bottom-Up Evaluation for Wine Ontology Example:
// Last Change:	19.05.2016
// Language:	C++
// EMail:	brass@informatik.uni-halle.de
// WWW:		http://www.informatik.uni-halle.de/~brass/
// Address:	Feldschloesschen 15, D-06120 Halle (Saale), GERMANY
// Copyright:	(c) 2015-2016 by Stefan Brass
// License:	See file "LICENSE" for copying conditions.
// Note:	There is no warranty at all - this code may contain bugs.
// ============================================================================

//=============================================================================
// Interface:
//=============================================================================

#include "ver.h"
#ifdef VER_WINE_ENABLED

#include "str.h"
#include "err.h"
#include "check.h"
#include "idiv.h"
#include "perf.h"
#include "stack.h"
#include "list_1.h"
#include "list_2.h"
#include "cur_1.h"
#include "cur_2.h"
#include "set_1.h"
#include "rel_n_n.h"
#include "cur_n_n.h"
#include "pred.h"
#include "load.h"
#include "out.h"

#include "bench_wine.h"


//=============================================================================
// Constructor, Destructor:
//=============================================================================

//-----------------------------------------------------------------------------
// Constructor (initializes lots of subobjects):
//-----------------------------------------------------------------------------

bench_wine_c::bench_wine_c() :

		// Domain for all predicate arguments (data values):
		dom("dom"),

		// Data structures for relations:
		kaon2namedobjects_f("kaon2namedobjects_f"),
		hasflavor_aux_ff("hasflavor_aux_ff"),
		kaon2namedobjects_b("kaon2namedobjects_b"),
		cheninblanc_aux_f("cheninblanc_aux_f"),
		beaujolais_aux_f("beaujolais_aux_f"),
		hassugar_aux_ff("hassugar_aux_ff"),
		hasvintageyear_aux_ff("hasvintageyear_aux_ff"),
		hascolor_aux_ff("hascolor_aux_ff"),
		meritage_aux_f("meritage_aux_f"),
		ot____nom8_aux_f("ot____nom8_aux_f"),
		madefromgrape_aux_ff("madefromgrape_aux_ff"),
		riesling_aux_f("riesling_aux_f"),
		dryriesling_aux_f("dryriesling_aux_f"),
		chianti_aux_f("chianti_aux_f"),
		hasmaker_aux_ff("hasmaker_aux_ff"),
		chardonnay_aux_f("chardonnay_aux_f"),
		whiteburgundy_aux_f("whiteburgundy_aux_f"),
		meursault_aux_f("meursault_aux_f"),
		hasbody_aux_ff("hasbody_aux_ff"),
		muscadet_aux_f("muscadet_aux_f"),
		margaux_aux_f("margaux_aux_f"),
		locatedin_aux_ff("locatedin_aux_ff"),
		sancerre_aux_f("sancerre_aux_f"),
		cotesdor_aux_f("cotesdor_aux_f"),
		pauillac_aux_f("pauillac_aux_f"),
		anjou_aux_f("anjou_aux_f"),
		port_aux_f("port_aux_f"),
		sauternes_aux_f("sauternes_aux_f"),
		stemilion_aux_f("stemilion_aux_f"),
		ot____nom36_aux_f("ot____nom36_aux_f"),
		cabernetsauvignon_aux_f("cabernetsauvignon_aux_f"),
		icewine_aux_f("icewine_aux_f"),
		merlot_aux_f("merlot_aux_f"),
		sweetriesling_aux_f("sweetriesling_aux_f"),
		whitewine_aux_f("whitewine_aux_f"),
		ot____nom23_aux_f("ot____nom23_aux_f"),
		pinotnoir_aux_f("pinotnoir_aux_f"),
		cabernetfranc_aux_f("cabernetfranc_aux_f"),
		semillon_aux_f("semillon_aux_f"),
		sauvignonblanc_aux_f("sauvignonblanc_aux_f"),
		ot____nom6_aux_f("ot____nom6_aux_f"),
		ot____nom19_aux_f("ot____nom19_aux_f"),
		petitesyrah_aux_f("petitesyrah_aux_f"),
		zinfandel_aux_f("zinfandel_aux_f"),
		redtablewine_aux_f("redtablewine_aux_f"),
		dessertwine_aux_f("dessertwine_aux_f"),
		ot____nom3_aux_f("ot____nom3_aux_f"),
		ot____nom43_aux_f("ot____nom43_aux_f"),
		ot____nom24_aux_f("ot____nom24_aux_f"),
		ot____nom11_aux_f("ot____nom11_aux_f"),
		ot____nom34_aux_f("ot____nom34_aux_f"),
		ot____nom37_aux_f("ot____nom37_aux_f"),
		ot____nom20_aux_f("ot____nom20_aux_f"),
		ot____nom33_aux_f("ot____nom33_aux_f"),
		ot____nom39_aux_f("ot____nom39_aux_f"),
		ot____nom27_aux_f("ot____nom27_aux_f"),
		ot____nom25_aux_f("ot____nom25_aux_f"),
		ot____nom21_aux_f("ot____nom21_aux_f"),
		ot____nom29_aux_f("ot____nom29_aux_f"),
		ot____nom7_aux_f("ot____nom7_aux_f"),
		ot____nom28_aux_f("ot____nom28_aux_f"),
		ot____nom45_aux_f("ot____nom45_aux_f"),
		ot____nom1_aux_f("ot____nom1_aux_f"),
		ot____nom32_aux_f("ot____nom32_aux_f"),
		ot____nom5_aux_f("ot____nom5_aux_f"),
		ot____nom12_aux_f("ot____nom12_aux_f"),
		ot____nom15_aux_f("ot____nom15_aux_f"),
		ot____nom2_aux_f("ot____nom2_aux_f"),
		ot____nom44_aux_f("ot____nom44_aux_f"),
		ot____nom10_aux_f("ot____nom10_aux_f"),
		ot____nom31_aux_f("ot____nom31_aux_f"),
		ot____nom16_aux_f("ot____nom16_aux_f"),
		ot____nom41_aux_f("ot____nom41_aux_f"),
		ot____nom38_aux_f("ot____nom38_aux_f"),
		ot____nom26_aux_f("ot____nom26_aux_f"),
		ot____nom4_aux_f("ot____nom4_aux_f"),
		ot____nom17_aux_f("ot____nom17_aux_f"),
		ot____nom14_aux_f("ot____nom14_aux_f"),
		ot____nom46_aux_f("ot____nom46_aux_f"),
		ot____nom13_aux_f("ot____nom13_aux_f"),
		ot____nom47_aux_f("ot____nom47_aux_f"),
		ot____nom42_aux_f("ot____nom42_aux_f"),
		ot____nom30_aux_f("ot____nom30_aux_f"),
		ot____nom22_aux_f("ot____nom22_aux_f")
{
	// Set Magic number:
	CHECK_CODE(Magic = BENCH_WINE_MAGIC);
}

//-----------------------------------------------------------------------------
// Destructor:
//-----------------------------------------------------------------------------

bench_wine_c::~bench_wine_c()
{
	// Check this object:
	CHECK_VALID("bench_wine_c::destructor");

	// Make this object invalid:
	CHECK_CODE(Magic = 0);
}


//=============================================================================
// Public Methods:
//=============================================================================

//-----------------------------------------------------------------------------
// load: Load data file with EDB facts.
//-----------------------------------------------------------------------------


bool bench_wine_c::load(str_t filename) {

	// Check parameter:
	CHECK(filename != STR_NULL, "bench_wine_c::load: filename is null");

	// Check object:
	CHECK_VALID("bench_wine_c::load");

	// Loader object (will contain data about predicates):
	load_c loader;

	// Predicates (input to loader):
	pred_c kaon2namedobjects_p("kaon2namedobjects", &dom);
	pred_c hasflavor_aux_p("hasflavor_aux", &dom, &dom);
	pred_c cheninblanc_aux_p("cheninblanc_aux", &dom);
	pred_c beaujolais_aux_p("beaujolais_aux", &dom);
	pred_c hassugar_aux_p("hassugar_aux", &dom, &dom);
	pred_c hasvintageyear_aux_p("hasvintageyear_aux", &dom, &dom);
	pred_c hascolor_aux_p("hascolor_aux", &dom, &dom);
	pred_c meritage_aux_p("meritage_aux", &dom);
	pred_c ot____nom8_aux_p("ot____nom8_aux", &dom);
	pred_c madefromgrape_aux_p("madefromgrape_aux", &dom, &dom);
	pred_c riesling_aux_p("riesling_aux", &dom);
	pred_c dryriesling_aux_p("dryriesling_aux", &dom);
	pred_c chianti_aux_p("chianti_aux", &dom);
	pred_c hasmaker_aux_p("hasmaker_aux", &dom, &dom);
	pred_c chardonnay_aux_p("chardonnay_aux", &dom);
	pred_c whiteburgundy_aux_p("whiteburgundy_aux", &dom);
	pred_c meursault_aux_p("meursault_aux", &dom);
	pred_c hasbody_aux_p("hasbody_aux", &dom, &dom);
	pred_c muscadet_aux_p("muscadet_aux", &dom);
	pred_c margaux_aux_p("margaux_aux", &dom);
	pred_c locatedin_aux_p("locatedin_aux", &dom, &dom);
	pred_c sancerre_aux_p("sancerre_aux", &dom);
	pred_c cotesdor_aux_p("cotesdor_aux", &dom);
	pred_c pauillac_aux_p("pauillac_aux", &dom);
	pred_c anjou_aux_p("anjou_aux", &dom);
	pred_c port_aux_p("port_aux", &dom);
	pred_c sauternes_aux_p("sauternes_aux", &dom);
	pred_c stemilion_aux_p("stemilion_aux", &dom);
	pred_c ot____nom36_aux_p("ot____nom36_aux", &dom);
	pred_c cabernetsauvignon_aux_p("cabernetsauvignon_aux", &dom);
	pred_c icewine_aux_p("icewine_aux", &dom);
	pred_c merlot_aux_p("merlot_aux", &dom);
	pred_c sweetriesling_aux_p("sweetriesling_aux", &dom);
	pred_c whitewine_aux_p("whitewine_aux", &dom);
	pred_c ot____nom23_aux_p("ot____nom23_aux", &dom);
	pred_c pinotnoir_aux_p("pinotnoir_aux", &dom);
	pred_c cabernetfranc_aux_p("cabernetfranc_aux", &dom);
	pred_c semillon_aux_p("semillon_aux", &dom);
	pred_c sauvignonblanc_aux_p("sauvignonblanc_aux", &dom);
	pred_c ot____nom6_aux_p("ot____nom6_aux", &dom);
	pred_c ot____nom19_aux_p("ot____nom19_aux", &dom);
	pred_c petitesyrah_aux_p("petitesyrah_aux", &dom);
	pred_c zinfandel_aux_p("zinfandel_aux", &dom);
	pred_c redtablewine_aux_p("redtablewine_aux", &dom);
	pred_c dessertwine_aux_p("dessertwine_aux", &dom);
	pred_c ot____nom3_aux_p("ot____nom3_aux", &dom);
	pred_c ot____nom43_aux_p("ot____nom43_aux", &dom);
	pred_c ot____nom24_aux_p("ot____nom24_aux", &dom);
	pred_c ot____nom11_aux_p("ot____nom11_aux", &dom);
	pred_c ot____nom34_aux_p("ot____nom34_aux", &dom);
	pred_c ot____nom37_aux_p("ot____nom37_aux", &dom);
	pred_c ot____nom20_aux_p("ot____nom20_aux", &dom);
	pred_c ot____nom33_aux_p("ot____nom33_aux", &dom);
	pred_c ot____nom39_aux_p("ot____nom39_aux", &dom);
	pred_c ot____nom27_aux_p("ot____nom27_aux", &dom);
	pred_c ot____nom25_aux_p("ot____nom25_aux", &dom);
	pred_c ot____nom21_aux_p("ot____nom21_aux", &dom);
	pred_c ot____nom29_aux_p("ot____nom29_aux", &dom);
	pred_c ot____nom7_aux_p("ot____nom7_aux", &dom);
	pred_c ot____nom28_aux_p("ot____nom28_aux", &dom);
	pred_c ot____nom45_aux_p("ot____nom45_aux", &dom);
	pred_c ot____nom1_aux_p("ot____nom1_aux", &dom);
	pred_c ot____nom32_aux_p("ot____nom32_aux", &dom);
	pred_c ot____nom5_aux_p("ot____nom5_aux", &dom);
	pred_c ot____nom12_aux_p("ot____nom12_aux", &dom);
	pred_c ot____nom15_aux_p("ot____nom15_aux", &dom);
	pred_c ot____nom2_aux_p("ot____nom2_aux", &dom);
	pred_c ot____nom44_aux_p("ot____nom44_aux", &dom);
	pred_c ot____nom10_aux_p("ot____nom10_aux", &dom);
	pred_c ot____nom31_aux_p("ot____nom31_aux", &dom);
	pred_c ot____nom16_aux_p("ot____nom16_aux", &dom);
	pred_c ot____nom41_aux_p("ot____nom41_aux", &dom);
	pred_c ot____nom38_aux_p("ot____nom38_aux", &dom);
	pred_c ot____nom26_aux_p("ot____nom26_aux", &dom);
	pred_c ot____nom4_aux_p("ot____nom4_aux", &dom);
	pred_c ot____nom17_aux_p("ot____nom17_aux", &dom);
	pred_c ot____nom14_aux_p("ot____nom14_aux", &dom);
	pred_c ot____nom46_aux_p("ot____nom46_aux", &dom);
	pred_c ot____nom13_aux_p("ot____nom13_aux", &dom);
	pred_c ot____nom47_aux_p("ot____nom47_aux", &dom);
	pred_c ot____nom42_aux_p("ot____nom42_aux", &dom);
	pred_c ot____nom30_aux_p("ot____nom30_aux", &dom);
	pred_c ot____nom22_aux_p("ot____nom22_aux", &dom);

	// Assign relations to predicates:
	kaon2namedobjects_p.add_rel(&kaon2namedobjects_f);
	hasflavor_aux_p.add_rel(&hasflavor_aux_ff);
	kaon2namedobjects_p.add_rel(&kaon2namedobjects_b);
	cheninblanc_aux_p.add_rel(&cheninblanc_aux_f);
	beaujolais_aux_p.add_rel(&beaujolais_aux_f);
	hassugar_aux_p.add_rel(&hassugar_aux_ff);
	hasvintageyear_aux_p.add_rel(&hasvintageyear_aux_ff);
	hascolor_aux_p.add_rel(&hascolor_aux_ff);
	meritage_aux_p.add_rel(&meritage_aux_f);
	ot____nom8_aux_p.add_rel(&ot____nom8_aux_f);
	madefromgrape_aux_p.add_rel(&madefromgrape_aux_ff);
	riesling_aux_p.add_rel(&riesling_aux_f);
	dryriesling_aux_p.add_rel(&dryriesling_aux_f);
	chianti_aux_p.add_rel(&chianti_aux_f);
	hasmaker_aux_p.add_rel(&hasmaker_aux_ff);
	chardonnay_aux_p.add_rel(&chardonnay_aux_f);
	whiteburgundy_aux_p.add_rel(&whiteburgundy_aux_f);
	meursault_aux_p.add_rel(&meursault_aux_f);
	hasbody_aux_p.add_rel(&hasbody_aux_ff);
	muscadet_aux_p.add_rel(&muscadet_aux_f);
	margaux_aux_p.add_rel(&margaux_aux_f);
	locatedin_aux_p.add_rel(&locatedin_aux_ff);
	sancerre_aux_p.add_rel(&sancerre_aux_f);
	cotesdor_aux_p.add_rel(&cotesdor_aux_f);
	pauillac_aux_p.add_rel(&pauillac_aux_f);
	anjou_aux_p.add_rel(&anjou_aux_f);
	port_aux_p.add_rel(&port_aux_f);
	sauternes_aux_p.add_rel(&sauternes_aux_f);
	stemilion_aux_p.add_rel(&stemilion_aux_f);
	ot____nom36_aux_p.add_rel(&ot____nom36_aux_f);
	cabernetsauvignon_aux_p.add_rel(&cabernetsauvignon_aux_f);
	icewine_aux_p.add_rel(&icewine_aux_f);
	merlot_aux_p.add_rel(&merlot_aux_f);
	sweetriesling_aux_p.add_rel(&sweetriesling_aux_f);
	whitewine_aux_p.add_rel(&whitewine_aux_f);
	ot____nom23_aux_p.add_rel(&ot____nom23_aux_f);
	pinotnoir_aux_p.add_rel(&pinotnoir_aux_f);
	cabernetfranc_aux_p.add_rel(&cabernetfranc_aux_f);
	semillon_aux_p.add_rel(&semillon_aux_f);
	sauvignonblanc_aux_p.add_rel(&sauvignonblanc_aux_f);
	ot____nom6_aux_p.add_rel(&ot____nom6_aux_f);
	ot____nom19_aux_p.add_rel(&ot____nom19_aux_f);
	petitesyrah_aux_p.add_rel(&petitesyrah_aux_f);
	zinfandel_aux_p.add_rel(&zinfandel_aux_f);
	redtablewine_aux_p.add_rel(&redtablewine_aux_f);
	dessertwine_aux_p.add_rel(&dessertwine_aux_f);
	ot____nom3_aux_p.add_rel(&ot____nom3_aux_f);
	ot____nom43_aux_p.add_rel(&ot____nom43_aux_f);
	ot____nom24_aux_p.add_rel(&ot____nom24_aux_f);
	ot____nom11_aux_p.add_rel(&ot____nom11_aux_f);
	ot____nom34_aux_p.add_rel(&ot____nom34_aux_f);
	ot____nom37_aux_p.add_rel(&ot____nom37_aux_f);
	ot____nom20_aux_p.add_rel(&ot____nom20_aux_f);
	ot____nom33_aux_p.add_rel(&ot____nom33_aux_f);
	ot____nom39_aux_p.add_rel(&ot____nom39_aux_f);
	ot____nom27_aux_p.add_rel(&ot____nom27_aux_f);
	ot____nom25_aux_p.add_rel(&ot____nom25_aux_f);
	ot____nom21_aux_p.add_rel(&ot____nom21_aux_f);
	ot____nom29_aux_p.add_rel(&ot____nom29_aux_f);
	ot____nom7_aux_p.add_rel(&ot____nom7_aux_f);
	ot____nom28_aux_p.add_rel(&ot____nom28_aux_f);
	ot____nom45_aux_p.add_rel(&ot____nom45_aux_f);
	ot____nom1_aux_p.add_rel(&ot____nom1_aux_f);
	ot____nom32_aux_p.add_rel(&ot____nom32_aux_f);
	ot____nom5_aux_p.add_rel(&ot____nom5_aux_f);
	ot____nom12_aux_p.add_rel(&ot____nom12_aux_f);
	ot____nom15_aux_p.add_rel(&ot____nom15_aux_f);
	ot____nom2_aux_p.add_rel(&ot____nom2_aux_f);
	ot____nom44_aux_p.add_rel(&ot____nom44_aux_f);
	ot____nom10_aux_p.add_rel(&ot____nom10_aux_f);
	ot____nom31_aux_p.add_rel(&ot____nom31_aux_f);
	ot____nom16_aux_p.add_rel(&ot____nom16_aux_f);
	ot____nom41_aux_p.add_rel(&ot____nom41_aux_f);
	ot____nom38_aux_p.add_rel(&ot____nom38_aux_f);
	ot____nom26_aux_p.add_rel(&ot____nom26_aux_f);
	ot____nom4_aux_p.add_rel(&ot____nom4_aux_f);
	ot____nom17_aux_p.add_rel(&ot____nom17_aux_f);
	ot____nom14_aux_p.add_rel(&ot____nom14_aux_f);
	ot____nom46_aux_p.add_rel(&ot____nom46_aux_f);
	ot____nom13_aux_p.add_rel(&ot____nom13_aux_f);
	ot____nom47_aux_p.add_rel(&ot____nom47_aux_f);
	ot____nom42_aux_p.add_rel(&ot____nom42_aux_f);
	ot____nom30_aux_p.add_rel(&ot____nom30_aux_f);
	ot____nom22_aux_p.add_rel(&ot____nom22_aux_f);

	// Add predicate definitions to loader:
	loader.add(&kaon2namedobjects_p);
	loader.add(&hasflavor_aux_p);
	loader.add(&cheninblanc_aux_p);
	loader.add(&beaujolais_aux_p);
	loader.add(&hassugar_aux_p);
	loader.add(&hasvintageyear_aux_p);
	loader.add(&hascolor_aux_p);
	loader.add(&meritage_aux_p);
	loader.add(&ot____nom8_aux_p);
	loader.add(&madefromgrape_aux_p);
	loader.add(&riesling_aux_p);
	loader.add(&dryriesling_aux_p);
	loader.add(&chianti_aux_p);
	loader.add(&hasmaker_aux_p);
	loader.add(&chardonnay_aux_p);
	loader.add(&whiteburgundy_aux_p);
	loader.add(&meursault_aux_p);
	loader.add(&hasbody_aux_p);
	loader.add(&muscadet_aux_p);
	loader.add(&margaux_aux_p);
	loader.add(&locatedin_aux_p);
	loader.add(&sancerre_aux_p);
	loader.add(&cotesdor_aux_p);
	loader.add(&pauillac_aux_p);
	loader.add(&anjou_aux_p);
	loader.add(&port_aux_p);
	loader.add(&sauternes_aux_p);
	loader.add(&stemilion_aux_p);
	loader.add(&ot____nom36_aux_p);
	loader.add(&cabernetsauvignon_aux_p);
	loader.add(&icewine_aux_p);
	loader.add(&merlot_aux_p);
	loader.add(&sweetriesling_aux_p);
	loader.add(&whitewine_aux_p);
	loader.add(&ot____nom23_aux_p);
	loader.add(&pinotnoir_aux_p);
	loader.add(&cabernetfranc_aux_p);
	loader.add(&semillon_aux_p);
	loader.add(&sauvignonblanc_aux_p);
	loader.add(&ot____nom6_aux_p);
	loader.add(&ot____nom19_aux_p);
	loader.add(&petitesyrah_aux_p);
	loader.add(&zinfandel_aux_p);
	loader.add(&redtablewine_aux_p);
	loader.add(&dessertwine_aux_p);
	loader.add(&ot____nom3_aux_p);
	loader.add(&ot____nom43_aux_p);
	loader.add(&ot____nom24_aux_p);
	loader.add(&ot____nom11_aux_p);
	loader.add(&ot____nom34_aux_p);
	loader.add(&ot____nom37_aux_p);
	loader.add(&ot____nom20_aux_p);
	loader.add(&ot____nom33_aux_p);
	loader.add(&ot____nom39_aux_p);
	loader.add(&ot____nom27_aux_p);
	loader.add(&ot____nom25_aux_p);
	loader.add(&ot____nom21_aux_p);
	loader.add(&ot____nom29_aux_p);
	loader.add(&ot____nom7_aux_p);
	loader.add(&ot____nom28_aux_p);
	loader.add(&ot____nom45_aux_p);
	loader.add(&ot____nom1_aux_p);
	loader.add(&ot____nom32_aux_p);
	loader.add(&ot____nom5_aux_p);
	loader.add(&ot____nom12_aux_p);
	loader.add(&ot____nom15_aux_p);
	loader.add(&ot____nom2_aux_p);
	loader.add(&ot____nom44_aux_p);
	loader.add(&ot____nom10_aux_p);
	loader.add(&ot____nom31_aux_p);
	loader.add(&ot____nom16_aux_p);
	loader.add(&ot____nom41_aux_p);
	loader.add(&ot____nom38_aux_p);
	loader.add(&ot____nom26_aux_p);
	loader.add(&ot____nom4_aux_p);
	loader.add(&ot____nom17_aux_p);
	loader.add(&ot____nom14_aux_p);
	loader.add(&ot____nom46_aux_p);
	loader.add(&ot____nom13_aux_p);
	loader.add(&ot____nom47_aux_p);
	loader.add(&ot____nom42_aux_p);
	loader.add(&ot____nom30_aux_p);
	loader.add(&ot____nom22_aux_p);

	// Predicates which occur in the data file, but are not used:`
	pred_c adjacentregion_aux_p("adjacentregion_aux",
			&dom, &dom);
	loader.add(&adjacentregion_aux_p);
	pred_c ot____nom18_aux_p("ot____nom18_aux", &dom);
	loader.add(&ot____nom18_aux_p);
	pred_c ot____nom35_aux_p("ot____nom35_aux", &dom);
	loader.add(&ot____nom35_aux_p);
	pred_c ot____nom40_aux_p("ot____nom40_aux", &dom);
	loader.add(&ot____nom40_aux_p);
	pred_c ot____nom48_aux_p("ot____nom48_aux", &dom);
	loader.add(&ot____nom48_aux_p);
	pred_c ot____nom49_aux_p("ot____nom49_aux", &dom);
	loader.add(&ot____nom49_aux_p);
	pred_c ot____nom50_aux_p("ot____nom50_aux", &dom);
	loader.add(&ot____nom50_aux_p);
	pred_c ot____nom51_aux_p("ot____nom51_aux", &dom);
	loader.add(&ot____nom51_aux_p);
	pred_c ot____nom52_aux_p("ot____nom52_aux", &dom);
	loader.add(&ot____nom52_aux_p);
	pred_c ot____nom53_aux_p("ot____nom53_aux", &dom);
	loader.add(&ot____nom53_aux_p);
	pred_c ot____nom54_aux_p("ot____nom54_aux", &dom);
	loader.add(&ot____nom54_aux_p);
	pred_c ot____nom55_aux_p("ot____nom55_aux", &dom);
	loader.add(&ot____nom55_aux_p);
	pred_c ot____nom56_aux_p("ot____nom56_aux", &dom);
	loader.add(&ot____nom56_aux_p);
	pred_c ot____nom57_aux_p("ot____nom57_aux", &dom);
	loader.add(&ot____nom57_aux_p);
	pred_c ot____nom58_aux_p("ot____nom58_aux", &dom);
	loader.add(&ot____nom58_aux_p);
	pred_c ot____nom59_aux_p("ot____nom59_aux", &dom);
	loader.add(&ot____nom59_aux_p);
	pred_c ot____nom60_aux_p("ot____nom60_aux", &dom);
	loader.add(&ot____nom60_aux_p);
	pred_c ot____nom61_aux_p("ot____nom61_aux", &dom);
	loader.add(&ot____nom61_aux_p);
	pred_c ot____nom62_aux_p("ot____nom62_aux", &dom);
	loader.add(&ot____nom62_aux_p);
	pred_c ot____nom63_aux_p("ot____nom63_aux", &dom);
	loader.add(&ot____nom63_aux_p);
	pred_c ot____nom64_aux_p("ot____nom64_aux", &dom);
	loader.add(&ot____nom64_aux_p);
	pred_c ot____nom9_aux_p("ot____nom9_aux", &dom);
	loader.add(&ot____nom9_aux_p);
	pred_c wineflavor_aux_p("wineflavor_aux", &dom);
	loader.add(&wineflavor_aux_p);
	pred_c winegrape_aux_p("winegrape_aux", &dom);
	loader.add(&winegrape_aux_p);
	pred_c winesugar_aux_p("winesugar_aux", &dom);
	loader.add(&winesugar_aux_p);
	pred_c winery_aux_p("winery_aux", &dom);
	loader.add(&winery_aux_p);
	pred_c winebody_aux_p("winebody_aux", &dom);
	loader.add(&winebody_aux_p);
	pred_c winecolor_aux_p("winecolor_aux", &dom);
	loader.add(&winecolor_aux_p);
	pred_c region_aux_p("region_aux", &dom);
	loader.add(&region_aux_p);
	pred_c vintageyear_aux_p("vintageyear_aux", &dom);
	loader.add(&vintageyear_aux_p);

	// Now do the load:
	return loader.load(filename);
}


//-----------------------------------------------------------------------------
// eval: Do bottom-up evaluation for the wine ontology.
//-----------------------------------------------------------------------------

long bench_wine_c::eval()
{

// START OF GENERATED CODE
// #ifdef DONT_COMPILE

// Transformed program:
// 1: answer :- californiawine(A).
// 2: californiawine(A) :- q48(A), wine(A).
// 3: q48(A) :- q47(A).
// 4: q47(A) :- californiawine(A).
// 5: californiawine(A) :- californiawine(B), kaon2equal(A,B).
// 6: kaon2equal(A,A) :- kaon2hu(A).
// 7: kaon2hu(A) :- kaon2namedobjects(A).
// 8: kaon2equal(A,B) :- hasflavor(C,A), hasflavor(C,B).
// 9: hasflavor(A,B) :- hasflavor_aux(A,B).
// 10: hasflavor(A,A) :- q4(A), kaon2namedobjects(A).
// 11: q4(A) :- cheninblanc(A).
// 12: cheninblanc(A) :- cheninblanc_aux(A).
// 13: cheninblanc(A) :- q6(A).
// 14: q6(A) :- cheninblanc(A).
// 15: cheninblanc(A) :- cheninblanc(B), kaon2equal(A,B).
// 16: cheninblanc(A) :- wine(A), madefromgrape(A,B), ot____nom34(B).
// 17: wine(A) :- q14(A).
// 18: q14(A) :- gamay(A).
// 19: gamay(A) :- q14(A).
// 20: q14(A) :- beaujolais(A).
// 21: beaujolais(A) :- beaujolais_aux(A).
// 22: beaujolais(A) :- q23(A), wine(A).
// 23: q23(A) :- q22(A).
// 24: q22(A) :- beaujolais(A).
// 25: beaujolais(A) :- beaujolais(B), kaon2equal(A,B).
// 26: kaon2equal(A,B) :- hassugar(C,A), hassugar(C,B).
// 27: hassugar(A,B) :- hassugar_aux(A,B).
// 28: hassugar(A,A) :- wine(A), kaon2namedobjects(A).
// 29: wine(A) :- texaswine(A).
// 30: texaswine(A) :- q74(A), wine(A).
// 31: q74(A) :- q73(A).
// 32: q73(A) :- texaswine(A).
// 33: texaswine(A) :- texaswine(B), kaon2equal(A,B).
// 34: kaon2equal(A,B) :- hasvintageyear(C,A), hasvintageyear(C,B).
// 35: hasvintageyear(A,B) :- hasvintageyear_aux(A,B).
// 36: hasvintageyear(A,A) :- vintage(A), kaon2namedobjects(A).
// 37: vintage(A) :- hasvintageyear(A,B).
// 38: hasvintageyear(A,B) :- hasvintageyear(C,B), kaon2equal(A,C).
// 39: hasvintageyear(A,B) :- hasvintageyear(A,C), kaon2equal(B,C).
// 40: kaon2equal(A,B) :- hascolor(C,A), hascolor(C,B).
// 41: hascolor(A,B) :- hascolor_aux(A,B).
// 42: hascolor(A,A) :- q0(A), kaon2namedobjects(A).
// 43: q0(A) :- meritage(A).
// 44: meritage(A) :- meritage_aux(A).
// 45: meritage(A) :- meritage(B), kaon2equal(A,B).
// 46: meritage(A) :- ot____nom8(A), wine(A), kaon2namedobjects(A).
// 47: ot____nom8(A) :- ot____nom8_aux(A).
// 48: ot____nom8(A) :- meritage(B), madefromgrape(B,A).
// 49: madefromgrape(A,B) :- madefromgrape_aux(A,B).
// 50: madefromgrape(A,B) :- madeintowine(B,A).
// 51: madeintowine(A,B) :- madefromgrape(B,A).
// 52: madefromgrape(A,A) :- riesling(A), kaon2namedobjects(A).
// 53: riesling(A) :- riesling_aux(A).
// 54: riesling(A) :- dryriesling(A).
// 55: dryriesling(A) :- dryriesling_aux(A).
// 56: dryriesling(A) :- q46(A), riesling(A).
// 57: q46(A) :- q31(A).
// 58: q31(A) :- chianti(A).
// 59: chianti(A) :- chianti_aux(A).
// 60: chianti(A) :- chianti(B), kaon2equal(A,B).
// 61: kaon2equal(A,B) :- hasmaker(C,A), hasmaker(C,B).
// 62: hasmaker(A,B) :- hasmaker_aux(A,B).
// 63: hasmaker(A,B) :- produceswine(B,A).
// 64: produceswine(A,B) :- hasmaker(B,A).
// 65: hasmaker(A,A) :- wine(A), kaon2namedobjects(A).
// 66: wine(A) :- q63(A).
// 67: q63(A) :- chardonnay(A).
// 68: chardonnay(A) :- chardonnay_aux(A).
// 69: chardonnay(A) :- q63(A).
// 70: q63(A) :- whiteburgundy(A).
// 71: whiteburgundy(A) :- whiteburgundy_aux(A).
// 72: whiteburgundy(A) :- meursault(A).
// 73: meursault(A) :- meursault_aux(A).
// 74: meursault(A) :- q17(A), whiteburgundy(A).
// 75: q17(A) :- q16(A).
// 76: q16(A) :- meursault(A).
// 77: meursault(A) :- meursault(B), kaon2equal(A,B).
// 78: kaon2equal(A,B) :- hasbody(C,A), hasbody(C,B).
// 79: hasbody(A,B) :- hasbody_aux(A,B).
// 80: hasbody(A,A) :- q45(A), kaon2namedobjects(A).
// 81: q45(A) :- beaujolais(A).
// 82: q45(A) :- muscadet(A).
// 83: muscadet(A) :- muscadet_aux(A).
// 84: muscadet(A) :- q56(A), loire(A).
// 85: q56(A) :- q55(A).
// 86: q55(A) :- muscadet(A).
// 87: muscadet(A) :- muscadet(B), kaon2equal(A,B).
// 88: kaon2equal(A,B) :- q3(C), madefromgrape(C,A), madefromgrape(C,B).
// 89: q3(A) :- margaux(A).
// 90: margaux(A) :- margaux_aux(A).
// 91: margaux(A) :- q19(A), medoc(A).
// 92: q19(A) :- q18(A).
// 93: q18(A) :- margaux(A).
// 94: margaux(A) :- margaux(B), kaon2equal(A,B).
// 95: kaon2equal(A,B) :- kaon2hu(A), kaon2hu(B), kaon2equal(B,A).
// 96: kaon2equal(A,B) :- kaon2equal(A,C), kaon2equal(C,B), kaon2hu(A), kaon2hu(C), kaon2hu(B).
// 97: q18(A) :- locatedin(A,B), q19(B).
// 98: locatedin(A,B) :- locatedin_aux(A,B).
// 99: locatedin(A,A) :- q57(A), kaon2namedobjects(A).
// 100: q57(A) :- loire(A).
// 101: loire(A) :- sancerre(A).
// 102: sancerre(A) :- sancerre_aux(A).
// 103: sancerre(A) :- q40(A), loire(A).
// 104: q40(A) :- q39(A).
// 105: q39(A) :- sancerre(A).
// 106: sancerre(A) :- sancerre(B), kaon2equal(A,B).
// 107: q39(A) :- locatedin(A,B), q40(B).
// 108: locatedin(A,A) :- q37(A), kaon2namedobjects(A).
// 109: q37(A) :- cotesdor(A).
// 110: cotesdor(A) :- cotesdor_aux(A).
// 111: cotesdor(A) :- q38(A), redburgundy(A).
// 112: q38(A) :- q37(A).
// 113: q37(A) :- locatedin(A,B), q38(B).
// 114: locatedin(A,A) :- q61(A), kaon2namedobjects(A).
// 115: q61(A) :- italianwine(A).
// 116: italianwine(A) :- chianti(A).
// 117: italianwine(A) :- q62(A), wine(A).
// 118: q62(A) :- q61(A).
// 119: q61(A) :- locatedin(A,B), q62(B).
// 120: locatedin(A,A) :- q59(A), kaon2namedobjects(A).
// 121: q59(A) :- alsatianwine(A).
// 122: alsatianwine(A) :- q60(A), wine(A).
// 123: q60(A) :- q59(A).
// 124: q59(A) :- locatedin(A,B), q60(B).
// 125: locatedin(A,A) :- q33(A), kaon2namedobjects(A).
// 126: q33(A) :- medoc(A).
// 127: medoc(A) :- pauillac(A).
// 128: pauillac(A) :- pauillac_aux(A).
// 129: pauillac(A) :- q43(A), medoc(A).
// 130: q43(A) :- q42(A).
// 131: q42(A) :- pauillac(A).
// 132: pauillac(A) :- pauillac(B), kaon2equal(A,B).
// 133: q42(A) :- locatedin(A,B), q43(B).
// 134: locatedin(A,A) :- wine(A), kaon2namedobjects(A).
// 135: wine(A) :- q20(A).
// 136: q20(A) :- sancerre(A).
// 137: q20(A) :- muscadet(A).
// 138: q20(A) :- beaujolais(A).
// 139: q20(A) :- dryriesling(A).
// 140: dryriesling(A) :- dryriesling(B), kaon2equal(A,B).
// 141: q20(A) :- anjou(A).
// 142: anjou(A) :- anjou_aux(A).
// 143: anjou(A) :- q67(A), loire(A).
// 144: q67(A) :- q66(A).
// 145: q66(A) :- anjou(A).
// 146: anjou(A) :- anjou(B), kaon2equal(A,B).
// 147: q66(A) :- locatedin(A,B), q67(B).
// 148: locatedin(A,A) :- q18(A), kaon2namedobjects(A).
// 149: q18(A) :- q18(B), kaon2equal(A,B).
// 150: locatedin(A,A) :- q49(A), kaon2namedobjects(A).
// 151: q49(A) :- germanwine(A).
// 152: germanwine(A) :- q50(A), wine(A).
// 153: q50(A) :- q49(A).
// 154: q49(A) :- locatedin(A,B), q50(B).
// 155: locatedin(A,A) :- q42(A), kaon2namedobjects(A).
// 156: q42(A) :- q42(B), kaon2equal(A,B).
// 157: locatedin(A,A) :- q9(A), kaon2namedobjects(A).
// 158: q9(A) :- tours(A).
// 159: tours(A) :- q10(A), loire(A).
// 160: q10(A) :- q9(A).
// 161: q9(A) :- locatedin(A,B), q10(B).
// 162: locatedin(A,A) :- q22(A), kaon2namedobjects(A).
// 163: q22(A) :- locatedin(A,B), q23(B).
// 164: locatedin(A,A) :- chianti(A), kaon2namedobjects(A).
// 165: locatedin(A,A) :- port(A), kaon2namedobjects(A).
// 166: port(A) :- port_aux(A).
// 167: port(A) :- port(B), kaon2equal(A,B).
// 168: locatedin(A,A) :- q26(A), kaon2namedobjects(A).
// 169: q26(A) :- americanwine(A).
// 170: americanwine(A) :- q27(A), wine(A).
// 171: q27(A) :- q26(A).
// 172: q26(A) :- locatedin(A,B), q27(B).
// 173: locatedin(A,A) :- q51(A), kaon2namedobjects(A).
// 174: q51(A) :- frenchwine(A).
// 175: frenchwine(A) :- q52(A), wine(A).
// 176: q52(A) :- q51(A).
// 177: q51(A) :- locatedin(A,B), q52(B).
// 178: locatedin(A,A) :- q55(A), kaon2namedobjects(A).
// 179: q55(A) :- locatedin(A,B), q56(B).
// 180: locatedin(A,A) :- q16(A), kaon2namedobjects(A).
// 181: q16(A) :- locatedin(A,B), q17(B).
// 182: locatedin(A,A) :- q73(A), kaon2namedobjects(A).
// 183: q73(A) :- locatedin(A,B), q74(B).
// 184: locatedin(A,A) :- q29(A), kaon2namedobjects(A).
// 185: q29(A) :- burgundy(A).
// 186: burgundy(A) :- whiteburgundy(A).
// 187: whiteburgundy(A) :- burgundy(A), whitewine(A).
// 188: burgundy(A) :- redburgundy(A).
// 189: redburgundy(A) :- cotesdor(A).
// 190: cotesdor(A) :- cotesdor(B), kaon2equal(A,B).
// 191: redburgundy(A) :- burgundy(A), redwine(A).
// 192: burgundy(A) :- q30(A), wine(A).
// 193: q30(A) :- q29(A).
// 194: q29(A) :- locatedin(A,B), q30(B).
// 195: locatedin(A,A) :- q1(A), kaon2namedobjects(A).
// 196: q1(A) :- bordeaux(A).
// 197: bordeaux(A) :- sauternes(A).
// 198: sauternes(A) :- sauternes_aux(A).
// 199: sauternes(A) :- sauternes(B), kaon2equal(A,B).
// 200: bordeaux(A) :- medoc(A).
// 201: medoc(A) :- margaux(A).
// 202: medoc(A) :- q34(A), bordeaux(A).
// 203: q34(A) :- q33(A).
// 204: q33(A) :- locatedin(A,B), q34(B).
// 205: locatedin(A,A) :- q47(A), kaon2namedobjects(A).
// 206: q47(A) :- locatedin(A,B), q48(B).
// 207: locatedin(A,A) :- q39(A), kaon2namedobjects(A).
// 208: q39(A) :- q39(B), kaon2equal(A,B).
// 209: locatedin(A,A) :- q64(A), kaon2namedobjects(A).
// 210: q64(A) :- stemilion(A).
// 211: stemilion(A) :- stemilion_aux(A).
// 212: stemilion(A) :- q65(A), bordeaux(A).
// 213: q65(A) :- q64(A).
// 214: q64(A) :- locatedin(A,B), q65(B).
// 215: locatedin(A,A) :- sauternes(A), kaon2namedobjects(A).
// 216: locatedin(A,A) :- q66(A), kaon2namedobjects(A).
// 217: q66(A) :- q66(B), kaon2equal(A,B).
// 218: locatedin(A,B) :- locatedin(C,B), kaon2equal(A,C).
// 219: locatedin(A,B) :- locatedin(A,C), kaon2equal(B,C).
// 220: locatedin(A,B) :- locatedin(A,C), locatedin(C,B).
// 221: locatedin(A,B) :- locatedin(A,C), locatedin(C,B), kaon2namedobjects(B), kaon2namedobjects(C), kaon2namedobjects(A).
// 222: q65(A) :- locatedin(A,B), ot____nom36(B).
// 223: ot____nom36(A) :- ot____nom36_aux(A).
// 224: ot____nom36(A) :- q64(A), kaon2namedobjects(A).
// 225: q64(A) :- q64(B), kaon2equal(A,B).
// 226: ot____nom36(A) :- ot____nom36(B), kaon2equal(A,B).
// 227: q65(A) :- q65(B), kaon2equal(A,B).
// 228: bordeaux(A) :- redbordeaux(A).
// 229: redbordeaux(A) :- redwine(A), bordeaux(A).
// 230: redwine(A) :- redburgundy(A).
// 231: redburgundy(A) :- redburgundy(B), kaon2equal(A,B).
// 232: redwine(A) :- redbordeaux(A).
// 233: redbordeaux(A) :- redbordeaux(B), kaon2equal(A,B).
// 234: redwine(A) :- dryredwine(A).
// 235: dryredwine(A) :- drywine(A), redwine(A).
// 236: drywine(A) :- drywhitewine(A).
// 237: drywhitewine(A) :- drywine(A), whitewine(A).
// 238: drywine(A) :- dryredwine(A).
// 239: dryredwine(A) :- dryredwine(B), kaon2equal(A,B).
// 240: drywine(A) :- q46(A), wine(A).
// 241: q46(A) :- hassugar(A,B), ot____nom12(B).
// 242: hassugar(A,A) :- q31(A), kaon2namedobjects(A).
// 243: q31(A) :- cabernetsauvignon(A).
// 244: cabernetsauvignon(A) :- cabernetsauvignon_aux(A).
// 245: cabernetsauvignon(A) :- q69(A).
// 246: q69(A) :- pauillac(A).
// 247: q69(A) :- cabernetsauvignon(A).
// 248: cabernetsauvignon(A) :- cabernetsauvignon(B), kaon2equal(A,B).
// 249: cabernetsauvignon(A) :- wine(A), madefromgrape(A,B), ot____nom29(B).
// 250: wine(A) :- tablewine(A).
// 251: tablewine(A) :- whitetablewine(A).
// 252: whitetablewine(A) :- q21(A), tablewine(A).
// 253: q21(A) :- q12(A).
// 254: q12(A) :- icewine(A).
// 255: icewine(A) :- icewine_aux(A).
// 256: icewine(A) :- icewine(B), kaon2equal(A,B).
// 257: icewine(A) :- q21(A), dessertwine(A), lateharvest(A).
// 258: q21(A) :- hascolor(A,B), ot____nom5(B).
// 259: hascolor(A,A) :- q15(A), kaon2namedobjects(A).
// 260: q15(A) :- anjou(A).
// 261: q15(A) :- rosewine(A).
// 262: rosewine(A) :- q15(A).
// 263: q15(A) :- q15(B), kaon2equal(A,B).
// 264: rosewine(A) :- rosewine(B), kaon2equal(A,B).
// 265: rosewine(A) :- wine(A), hascolor(A,B), ot____nom32(B).
// 266: wine(A) :- haswinedescriptor(A,B).
// 267: haswinedescriptor(A,B) :- hasbody(A,B).
// 268: hasbody(A,A) :- wine(A), kaon2namedobjects(A).
// 269: wine(A) :- q31(A).
// 270: q31(A) :- medoc(A).
// 271: medoc(A) :- medoc(B), kaon2equal(A,B).
// 272: q31(A) :- merlot(A).
// 273: merlot(A) :- merlot_aux(A).
// 274: merlot(A) :- q13(A).
// 275: q13(A) :- margaux(A).
// 276: q13(A) :- merlot(A).
// 277: merlot(A) :- merlot(B), kaon2equal(A,B).
// 278: merlot(A) :- wine(A), madefromgrape(A,B), ot____nom43(B).
// 279: wine(A) :- q11(A).
// 280: q11(A) :- pauillac(A).
// 281: q11(A) :- port(A).
// 282: q11(A) :- stemilion(A).
// 283: stemilion(A) :- stemilion(B), kaon2equal(A,B).
// 284: q11(A) :- q11(B), kaon2equal(A,B).
// 285: wine(A) :- q15(A).
// 286: wine(A) :- earlyharvest(A).
// 287: earlyharvest(A) :- earlyharvest(B), kaon2equal(A,B).
// 288: wine(A) :- sweetwine(A).
// 289: sweetwine(A) :- q71(A), wine(A).
// 290: q71(A) :- q70(A).
// 291: q70(A) :- sweetriesling(A).
// 292: sweetriesling(A) :- sweetriesling_aux(A).
// 293: sweetriesling(A) :- q71(A), riesling(A).
// 294: q71(A) :- hassugar(A,B), ot____nom1(B).
// 295: hassugar(A,A) :- whitewine(A), kaon2namedobjects(A).
// 296: whitewine(A) :- whitewine_aux(A).
// 297: whitewine(A) :- drywhitewine(A).
// 298: drywhitewine(A) :- drywhitewine(B), kaon2equal(A,B).
// 299: whitewine(A) :- whiteburgundy(A).
// 300: whiteburgundy(A) :- whiteburgundy(B), kaon2equal(A,B).
// 301: whitewine(A) :- whitebordeaux(A).
// 302: whitebordeaux(A) :- bordeaux(A), whitewine(A).
// 303: bordeaux(A) :- stemilion(A).
// 304: bordeaux(A) :- whitebordeaux(A).
// 305: whitebordeaux(A) :- whitebordeaux(B), kaon2equal(A,B).
// 306: bordeaux(A) :- q2(A), wine(A).
// 307: q2(A) :- q1(A).
// 308: q1(A) :- locatedin(A,B), q2(B).
// 309: q2(A) :- locatedin(A,B), ot____nom23(B).
// 310: ot____nom23(A) :- ot____nom23_aux(A).
// 311: ot____nom23(A) :- q1(A), kaon2namedobjects(A).
// 312: q1(A) :- q1(B), kaon2equal(A,B).
// 313: ot____nom23(A) :- ot____nom23(B), kaon2equal(A,B).
// 314: q2(A) :- q2(B), kaon2equal(A,B).
// 315: wine(A) :- americanwine(A).
// 316: americanwine(A) :- americanwine(B), kaon2equal(A,B).
// 317: wine(A) :- alsatianwine(A).
// 318: alsatianwine(A) :- alsatianwine(B), kaon2equal(A,B).
// 319: wine(A) :- q0(A).
// 320: q0(A) :- merlot(A).
// 321: q0(A) :- pinotnoir(A).
// 322: pinotnoir(A) :- pinotnoir_aux(A).
// 323: pinotnoir(A) :- q72(A).
// 324: q72(A) :- pinotnoir(A).
// 325: pinotnoir(A) :- pinotnoir(B), kaon2equal(A,B).
// 326: pinotnoir(A) :- wine(A), madefromgrape(A,B), ot____nom39(B).
// 327: wine(A) :- madefromgrape(A,B).
// 328: madefromgrape(A,A) :- wine(A), kaon2namedobjects(A).
// 329: wine(A) :- cabernetfranc(A).
// 330: cabernetfranc(A) :- cabernetfranc_aux(A).
// 331: cabernetfranc(A) :- cabernetfranc(B), kaon2equal(A,B).
// 332: cabernetfranc(A) :- wine(A), madefromgrape(A,B), ot____nom3(B).
// 333: wine(A) :- semillonorsauvignonblanc(A).
// 334: semillonorsauvignonblanc(A) :- semillon(A).
// 335: semillon(A) :- semillon_aux(A).
// 336: semillon(A) :- semillon(B), kaon2equal(A,B).
// 337: semillon(A) :- semillonorsauvignonblanc(A), madefromgrape(A,B), ot____nom27(B).
// 338: semillonorsauvignonblanc(A) :- sauvignonblanc(A).
// 339: sauvignonblanc(A) :- sauvignonblanc_aux(A).
// 340: sauvignonblanc(A) :- q44(A), semillonorsauvignonblanc(A).
// 341: q44(A) :- sancerre(A).
// 342: q44(A) :- sauvignonblanc(A).
// 343: sauvignonblanc(A) :- sauvignonblanc(B), kaon2equal(A,B).
// 344: sauvignonblanc(A) :- semillonorsauvignonblanc(A), madefromgrape(A,B), ot____nom11(B).
// 345: semillonorsauvignonblanc(A) :- q68(A), wine(A).
// 346: q68(A) :- whitebordeaux(A).
// 347: q68(A) :- semillonorsauvignonblanc(A).
// 348: semillonorsauvignonblanc(A) :- semillonorsauvignonblanc(B), kaon2equal(A,B).
// 349: semillonorsauvignonblanc(A) :- ot____nom6(A), wine(A), kaon2namedobjects(A).
// 350: ot____nom6(A) :- ot____nom6_aux(A).
// 351: ot____nom6(A) :- q68(B), madefromgrape(B,A).
// 352: q68(A) :- q68(B), kaon2equal(A,B).
// 353: madefromgrape(A,A) :- q72(A), kaon2namedobjects(A).
// 354: q72(A) :- redburgundy(A).
// 355: q72(A) :- q72(B), kaon2equal(A,B).
// 356: madefromgrape(A,A) :- q5(A), kaon2namedobjects(A).
// 357: q5(A) :- meritage(A).
// 358: q5(A) :- q24(A).
// 359: q24(A) :- pinotblanc(A).
// 360: pinotblanc(A) :- q24(A).
// 361: q24(A) :- muscadet(A).
// 362: q24(A) :- q24(B), kaon2equal(A,B).
// 363: pinotblanc(A) :- pinotblanc(B), kaon2equal(A,B).
// 364: pinotblanc(A) :- wine(A), madefromgrape(A,B), ot____nom21(B).
// 365: wine(A) :- beaujolais(A).
// 366: wine(A) :- fullbodiedwine(A).
// 367: fullbodiedwine(A) :- q35(A).
// 368: q35(A) :- pauillac(A).
// 369: q35(A) :- port(A).
// 370: q35(A) :- sweetriesling(A).
// 371: sweetriesling(A) :- sweetriesling(B), kaon2equal(A,B).
// 372: q35(A) :- meursault(A).
// 373: q35(A) :- fullbodiedwine(A).
// 374: fullbodiedwine(A) :- fullbodiedwine(B), kaon2equal(A,B).
// 375: fullbodiedwine(A) :- wine(A), hasbody(A,B), ot____nom45(B).
// 376: wine(A) :- loire(A).
// 377: loire(A) :- tours(A).
// 378: tours(A) :- tours(B), kaon2equal(A,B).
// 379: loire(A) :- whiteloire(A).
// 380: whiteloire(A) :- loire(A), whitewine(A).
// 381: loire(A) :- anjou(A).
// 382: loire(A) :- muscadet(A).
// 383: loire(A) :- q58(A), wine(A).
// 384: q58(A) :- q57(A).
// 385: q57(A) :- locatedin(A,B), q58(B).
// 386: q58(A) :- locatedin(A,B), ot____nom19(B).
// 387: ot____nom19(A) :- ot____nom19_aux(A).
// 388: ot____nom19(A) :- q57(A), kaon2namedobjects(A).
// 389: q57(A) :- q57(B), kaon2equal(A,B).
// 390: ot____nom19(A) :- ot____nom19(B), kaon2equal(A,B).
// 391: q58(A) :- q58(B), kaon2equal(A,B).
// 392: wine(A) :- petitesyrah(A).
// 393: petitesyrah(A) :- petitesyrah_aux(A).
// 394: petitesyrah(A) :- petitesyrah(B), kaon2equal(A,B).
// 395: petitesyrah(A) :- wine(A), madefromgrape(A,B), ot____nom25(B).
// 396: wine(A) :- q12(A).
// 397: q12(A) :- whitetablewine(A).
// 398: whitetablewine(A) :- whitetablewine(B), kaon2equal(A,B).
// 399: q12(A) :- cheninblanc(A).
// 400: q12(A) :- chardonnay(A).
// 401: chardonnay(A) :- chardonnay(B), kaon2equal(A,B).
// 402: chardonnay(A) :- wine(A), madefromgrape(A,B), ot____nom24(B).
// 403: wine(A) :- q45(A).
// 404: q45(A) :- anjou(A).
// 405: q45(A) :- q45(B), kaon2equal(A,B).
// 406: wine(A) :- q7(A).
// 407: q7(A) :- cabernetfranc(A).
// 408: q7(A) :- sauternes(A).
// 409: q7(A) :- sancerre(A).
// 410: q7(A) :- q7(B), kaon2equal(A,B).
// 411: wine(A) :- pinotnoir(A).
// 412: wine(A) :- q35(A).
// 413: q35(A) :- q35(B), kaon2equal(A,B).
// 414: wine(A) :- q24(A).
// 415: wine(A) :- frenchwine(A).
// 416: frenchwine(A) :- frenchwine(B), kaon2equal(A,B).
// 417: wine(A) :- meritage(A).
// 418: wine(A) :- zinfandel(A).
// 419: zinfandel(A) :- zinfandel_aux(A).
// 420: zinfandel(A) :- zinfandel(B), kaon2equal(A,B).
// 421: zinfandel(A) :- wine(A), madefromgrape(A,B), ot____nom20(B).
// 422: wine(A) :- californiawine(A).
// 423: wine(A) :- pinotblanc(A).
// 424: wine(A) :- q44(A).
// 425: q44(A) :- q44(B), kaon2equal(A,B).
// 426: wine(A) :- bordeaux(A).
// 427: bordeaux(A) :- bordeaux(B), kaon2equal(A,B).
// 428: wine(A) :- riesling(A).
// 429: riesling(A) :- sweetriesling(A).
// 430: riesling(A) :- riesling(B), kaon2equal(A,B).
// 431: riesling(A) :- wine(A), madefromgrape(A,B), ot____nom37(B).
// 432: wine(A) :- hascolor(A,B).
// 433: hascolor(A,A) :- wine(A), kaon2namedobjects(A).
// 434: wine(A) :- redwine(A).
// 435: redwine(A) :- port(A).
// 436: redwine(A) :- q41(A), wine(A).
// 437: q41(A) :- q0(A).
// 438: q0(A) :- beaujolais(A).
// 439: q0(A) :- cabernetfranc(A).
// 440: q0(A) :- zinfandel(A).
// 441: q0(A) :- medoc(A).
// 442: q0(A) :- cabernetsauvignon(A).
// 443: q0(A) :- redtablewine(A).
// 444: redtablewine(A) :- redtablewine_aux(A).
// 445: redtablewine(A) :- q41(A), tablewine(A).
// 446: q41(A) :- hascolor(A,B), ot____nom28(B).
// 447: hascolor(A,A) :- q12(A), kaon2namedobjects(A).
// 448: q12(A) :- pinotblanc(A).
// 449: q12(A) :- whitewine(A).
// 450: whitewine(A) :- whiteloire(A).
// 451: whiteloire(A) :- whiteloire(B), kaon2equal(A,B).
// 452: whitewine(A) :- whitenonsweetwine(A).
// 453: whitenonsweetwine(A) :- whitewine(A), kaon2namedobjects(A).
// 454: whitewine(A) :- q21(A), wine(A).
// 455: q21(A) :- q21(B), kaon2equal(A,B).
// 456: wine(A) :- whitewine(A).
// 457: whitewine(A) :- whitewine(B), kaon2equal(A,B).
// 458: wine(A) :- dessertwine(A).
// 459: dessertwine(A) :- dessertwine_aux(A).
// 460: dessertwine(A) :- sweetriesling(A).
// 461: dessertwine(A) :- icewine(A).
// 462: dessertwine(A) :- dessertwine(B), kaon2equal(A,B).
// 463: wine(A) :- q69(A).
// 464: q69(A) :- stemilion(A).
// 465: q69(A) :- q69(B), kaon2equal(A,B).
// 466: wine(A) :- q6(A).
// 467: q6(A) :- tours(A).
// 468: q6(A) :- q6(B), kaon2equal(A,B).
// 469: wine(A) :- drywine(A).
// 470: drywine(A) :- drywine(B), kaon2equal(A,B).
// 471: wine(A) :- q13(A).
// 472: q13(A) :- q13(B), kaon2equal(A,B).
// 473: wine(A) :- germanwine(A).
// 474: germanwine(A) :- germanwine(B), kaon2equal(A,B).
// 475: wine(A) :- burgundy(A).
// 476: burgundy(A) :- burgundy(B), kaon2equal(A,B).
// 477: wine(A) :- cabernetsauvignon(A).
// 478: wine(A) :- q72(A).
// 479: wine(A) :- q70(A).
// 480: q70(A) :- port(A).
// 481: q70(A) :- sweetwine(A).
// 482: sweetwine(A) :- sweetwine(B), kaon2equal(A,B).
// 483: q70(A) :- lateharvest(A).
// 484: lateharvest(A) :- icewine(A).
// 485: lateharvest(A) :- sauternes(A).
// 486: lateharvest(A) :- lateharvest(B), kaon2equal(A,B).
// 487: q70(A) :- q70(B), kaon2equal(A,B).
// 488: wine(A) :- q4(A).
// 489: q4(A) :- chianti(A).
// 490: q4(A) :- cabernetfranc(A).
// 491: q4(A) :- cotesdor(A).
// 492: q4(A) :- q4(B), kaon2equal(A,B).
// 493: wine(A) :- italianwine(A).
// 494: italianwine(A) :- italianwine(B), kaon2equal(A,B).
// 495: wine(A) :- chardonnay(A).
// 496: wine(A) :- semillon(A).
// 497: wine(A) :- q5(A).
// 498: q5(A) :- q6(A).
// 499: q5(A) :- q13(A).
// 500: q5(A) :- q72(A).
// 501: q5(A) :- q14(A).
// 502: q14(A) :- q14(B), kaon2equal(A,B).
// 503: q5(A) :- q69(A).
// 504: q5(A) :- q63(A).
// 505: q63(A) :- q63(B), kaon2equal(A,B).
// 506: q5(A) :- q44(A), semillonorsauvignonblanc(A).
// 507: q5(A) :- q5(B), kaon2equal(A,B).
// 508: q5(A) :- wine(A), madefromgrape(A,B), ot____nom3(B).
// 509: wine(A) :- rosewine(A).
// 510: wine(A) :- q32(A).
// 511: q32(A) :- sancerre(A).
// 512: q32(A) :- anjou(A).
// 513: q32(A) :- q32(B), kaon2equal(A,B).
// 514: wine(A) :- lateharvest(A).
// 515: wine(A) :- merlot(A).
// 516: wine(A) :- wine(B), kaon2equal(A,B).
// 517: madefromgrape(A,A) :- semillon(A), kaon2namedobjects(A).
// 518: madefromgrape(A,A) :- q14(A), kaon2namedobjects(A).
// 519: madefromgrape(A,A) :- q24(A), kaon2namedobjects(A).
// 520: madefromgrape(A,A) :- zinfandel(A), kaon2namedobjects(A).
// 521: madefromgrape(A,A) :- q6(A), kaon2namedobjects(A).
// 522: madefromgrape(A,A) :- petitesyrah(A), kaon2namedobjects(A).
// 523: madefromgrape(A,A) :- q13(A), kaon2namedobjects(A).
// 524: madefromgrape(A,A) :- chianti(A), kaon2namedobjects(A).
// 525: madefromgrape(A,A) :- cabernetfranc(A), kaon2namedobjects(A).
// 526: madefromgrape(A,A) :- q69(A), kaon2namedobjects(A).
// 527: madefromgrape(A,A) :- q44(A), kaon2namedobjects(A).
// 528: madefromgrape(A,A) :- q63(A), kaon2namedobjects(A).
// 529: madefromgrape(A,B) :- madefromgrape(C,B), kaon2equal(A,C).
// 530: madefromgrape(A,B) :- madefromgrape(A,C), kaon2equal(B,C).
// 531: ot____nom3(A) :- ot____nom3_aux(A).
// 532: ot____nom3(A) :- cabernetfranc(A), kaon2namedobjects(A).
// 533: ot____nom3(A) :- ot____nom3(B), kaon2equal(A,B).
// 534: q5(A) :- wine(A), madefromgrape(A,B), ot____nom43(B).
// 535: ot____nom43(A) :- ot____nom43_aux(A).
// 536: ot____nom43(A) :- q13(A), kaon2namedobjects(A).
// 537: ot____nom43(A) :- ot____nom43(B), kaon2equal(A,B).
// 538: q5(A) :- wine(A), madefromgrape(A,B), ot____nom24(B).
// 539: ot____nom24(A) :- ot____nom24_aux(A).
// 540: ot____nom24(A) :- q63(A), kaon2namedobjects(A).
// 541: ot____nom24(A) :- ot____nom24(B), kaon2equal(A,B).
// 542: q5(A) :- semillonorsauvignonblanc(A), madefromgrape(A,B), ot____nom11(B).
// 543: ot____nom11(A) :- ot____nom11_aux(A).
// 544: ot____nom11(A) :- q44(A), kaon2namedobjects(A).
// 545: ot____nom11(A) :- ot____nom11(B), kaon2equal(A,B).
// 546: q5(A) :- wine(A), madefromgrape(A,B), ot____nom34(B).
// 547: ot____nom34(A) :- ot____nom34_aux(A).
// 548: ot____nom34(A) :- q6(A), kaon2namedobjects(A).
// 549: ot____nom34(A) :- ot____nom34(B), kaon2equal(A,B).
// 550: q5(A) :- wine(A), madefromgrape(A,B), ot____nom37(B).
// 551: ot____nom37(A) :- ot____nom37_aux(A).
// 552: ot____nom37(A) :- riesling(A), kaon2namedobjects(A).
// 553: ot____nom37(A) :- ot____nom37(B), kaon2equal(A,B).
// 554: q5(A) :- wine(A), madefromgrape(A,B), ot____nom20(B).
// 555: ot____nom20(A) :- ot____nom20_aux(A).
// 556: ot____nom20(A) :- zinfandel(A), kaon2namedobjects(A).
// 557: ot____nom20(A) :- ot____nom20(B), kaon2equal(A,B).
// 558: q5(A) :- wine(A), madefromgrape(A,B), ot____nom33(B).
// 559: ot____nom33(A) :- ot____nom33_aux(A).
// 560: ot____nom33(A) :- q14(A), kaon2namedobjects(A).
// 561: ot____nom33(A) :- ot____nom33(B), kaon2equal(A,B).
// 562: q5(A) :- wine(A), madefromgrape(A,B), ot____nom39(B).
// 563: ot____nom39(A) :- ot____nom39_aux(A).
// 564: ot____nom39(A) :- q72(A), kaon2namedobjects(A).
// 565: ot____nom39(A) :- ot____nom39(B), kaon2equal(A,B).
// 566: q5(A) :- semillonorsauvignonblanc(A), madefromgrape(A,B), ot____nom27(B).
// 567: ot____nom27(A) :- ot____nom27_aux(A).
// 568: ot____nom27(A) :- semillon(A), kaon2namedobjects(A).
// 569: ot____nom27(A) :- ot____nom27(B), kaon2equal(A,B).
// 570: q5(A) :- wine(A), madefromgrape(A,B), ot____nom25(B).
// 571: ot____nom25(A) :- ot____nom25_aux(A).
// 572: ot____nom25(A) :- petitesyrah(A), kaon2namedobjects(A).
// 573: ot____nom25(A) :- ot____nom25(B), kaon2equal(A,B).
// 574: q5(A) :- wine(A), madefromgrape(A,B), ot____nom21(B).
// 575: ot____nom21(A) :- ot____nom21_aux(A).
// 576: ot____nom21(A) :- q24(A), kaon2namedobjects(A).
// 577: ot____nom21(A) :- ot____nom21(B), kaon2equal(A,B).
// 578: q5(A) :- wine(A), madefromgrape(A,B), ot____nom29(B).
// 579: ot____nom29(A) :- ot____nom29_aux(A).
// 580: ot____nom29(A) :- q69(A), kaon2namedobjects(A).
// 581: ot____nom29(A) :- ot____nom29(B), kaon2equal(A,B).
// 582: whitenonsweetwine(A) :- q36(A), whitewine(A).
// 583: q36(A) :- whitenonsweetwine(A).
// 584: whitenonsweetwine(A) :- whitenonsweetwine(B), kaon2equal(A,B).
// 585: whitenonsweetwine(A) :- ot____nom7(A), whitewine(A), kaon2namedobjects(A).
// 586: ot____nom7(A) :- ot____nom7_aux(A).
// 587: ot____nom7(A) :- q36(B), hassugar(B,A).
// 588: q36(A) :- earlyharvest(A).
// 589: q36(A) :- cheninblanc(A).
// 590: q36(A) :- q36(B), kaon2equal(A,B).
// 591: hassugar(A,A) :- q70(A), kaon2namedobjects(A).
// 592: hassugar(A,A) :- q32(A), kaon2namedobjects(A).
// 593: hassugar(A,B) :- hassugar(C,B), kaon2equal(A,C).
// 594: hassugar(A,B) :- hassugar(A,C), kaon2equal(B,C).
// 595: ot____nom7(A) :- ot____nom7(B), kaon2equal(A,B).
// 596: q12(A) :- dryriesling(A).
// 597: q12(A) :- sauternes(A).
// 598: q12(A) :- semillonorsauvignonblanc(A).
// 599: q12(A) :- riesling(A).
// 600: q12(A) :- q12(B), kaon2equal(A,B).
// 601: hascolor(A,B) :- hascolor(C,B), kaon2equal(A,C).
// 602: hascolor(A,B) :- hascolor(A,C), kaon2equal(B,C).
// 603: ot____nom28(A) :- ot____nom28_aux(A).
// 604: ot____nom28(A) :- q0(A), kaon2namedobjects(A).
// 605: q0(A) :- petitesyrah(A).
// 606: q0(A) :- redwine(A).
// 607: redwine(A) :- redwine(B), kaon2equal(A,B).
// 608: q0(A) :- chianti(A).
// 609: q0(A) :- stemilion(A).
// 610: q0(A) :- q0(B), kaon2equal(A,B).
// 611: ot____nom28(A) :- ot____nom28(B), kaon2equal(A,B).
// 612: q41(A) :- q41(B), kaon2equal(A,B).
// 613: tablewine(A) :- redtablewine(A).
// 614: redtablewine(A) :- redtablewine(B), kaon2equal(A,B).
// 615: tablewine(A) :- q46(A), wine(A).
// 616: q46(A) :- q46(B), kaon2equal(A,B).
// 617: tablewine(A) :- tablewine(B), kaon2equal(A,B).
// 618: loire(A) :- loire(B), kaon2equal(A,B).
// 619: hasbody(A,A) :- q35(A), kaon2namedobjects(A).
// 620: hasbody(A,A) :- q7(A), kaon2namedobjects(A).
// 621: hasbody(A,B) :- hasbody(C,B), kaon2equal(A,C).
// 622: hasbody(A,B) :- hasbody(A,C), kaon2equal(B,C).
// 623: ot____nom45(A) :- ot____nom45_aux(A).
// 624: ot____nom45(A) :- q35(A), kaon2namedobjects(A).
// 625: ot____nom45(A) :- ot____nom45(B), kaon2equal(A,B).
// 626: ot____nom6(A) :- ot____nom6(B), kaon2equal(A,B).
// 627: ot____nom1(A) :- ot____nom1_aux(A).
// 628: ot____nom1(A) :- q70(A), kaon2namedobjects(A).
// 629: ot____nom1(A) :- ot____nom1(B), kaon2equal(A,B).
// 630: q71(A) :- q71(B), kaon2equal(A,B).
// 631: q31(A) :- muscadet(A).
// 632: q31(A) :- tablewine(A).
// 633: q31(A) :- drywine(A).
// 634: q31(A) :- beaujolais(A).
// 635: q31(A) :- zinfandel(A).
// 636: q31(A) :- cabernetfranc(A).
// 637: q31(A) :- burgundy(A).
// 638: q31(A) :- dryriesling(A).
// 639: q31(A) :- petitesyrah(A).
// 640: q31(A) :- q31(B), kaon2equal(A,B).
// 641: haswinedescriptor(A,B) :- hascolor(A,B).
// 642: haswinedescriptor(A,B) :- hasflavor(A,B).
// 643: hasflavor(A,A) :- q11(A), kaon2namedobjects(A).
// 644: hasflavor(A,A) :- q20(A), kaon2namedobjects(A).
// 645: q20(A) :- margaux(A).
// 646: q20(A) :- q20(B), kaon2equal(A,B).
// 647: hasflavor(A,A) :- wine(A), kaon2namedobjects(A).
// 648: hasflavor(A,B) :- hasflavor(C,B), kaon2equal(A,C).
// 649: hasflavor(A,B) :- hasflavor(A,C), kaon2equal(B,C).
// 650: haswinedescriptor(A,B) :- hassugar(A,B).
// 651: haswinedescriptor(A,B) :- haswinedescriptor(C,B), kaon2equal(A,C).
// 652: haswinedescriptor(A,B) :- haswinedescriptor(A,C), kaon2equal(B,C).
// 653: ot____nom32(A) :- ot____nom32_aux(A).
// 654: ot____nom32(A) :- q15(A), kaon2namedobjects(A).
// 655: ot____nom32(A) :- ot____nom32(B), kaon2equal(A,B).
// 656: ot____nom5(A) :- ot____nom5_aux(A).
// 657: ot____nom5(A) :- q12(A), kaon2namedobjects(A).
// 658: ot____nom5(A) :- ot____nom5(B), kaon2equal(A,B).
// 659: ot____nom12(A) :- ot____nom12_aux(A).
// 660: ot____nom12(A) :- q31(A), kaon2namedobjects(A).
// 661: ot____nom12(A) :- ot____nom12(B), kaon2equal(A,B).
// 662: q48(A) :- locatedin(A,B), ot____nom15(B).
// 663: ot____nom15(A) :- ot____nom15_aux(A).
// 664: ot____nom15(A) :- q47(A), kaon2namedobjects(A).
// 665: q47(A) :- q47(B), kaon2equal(A,B).
// 666: ot____nom15(A) :- ot____nom15(B), kaon2equal(A,B).
// 667: q48(A) :- q48(B), kaon2equal(A,B).
// 668: q34(A) :- locatedin(A,B), ot____nom2(B).
// 669: ot____nom2(A) :- ot____nom2_aux(A).
// 670: ot____nom2(A) :- q33(A), kaon2namedobjects(A).
// 671: q33(A) :- q33(B), kaon2equal(A,B).
// 672: ot____nom2(A) :- ot____nom2(B), kaon2equal(A,B).
// 673: q34(A) :- q34(B), kaon2equal(A,B).
// 674: q30(A) :- locatedin(A,B), ot____nom44(B).
// 675: ot____nom44(A) :- ot____nom44_aux(A).
// 676: ot____nom44(A) :- q29(A), kaon2namedobjects(A).
// 677: q29(A) :- q29(B), kaon2equal(A,B).
// 678: ot____nom44(A) :- ot____nom44(B), kaon2equal(A,B).
// 679: q30(A) :- q30(B), kaon2equal(A,B).
// 680: q74(A) :- locatedin(A,B), ot____nom10(B).
// 681: ot____nom10(A) :- ot____nom10_aux(A).
// 682: ot____nom10(A) :- q73(A), kaon2namedobjects(A).
// 683: q73(A) :- q73(B), kaon2equal(A,B).
// 684: ot____nom10(A) :- ot____nom10(B), kaon2equal(A,B).
// 685: q74(A) :- q74(B), kaon2equal(A,B).
// 686: q17(A) :- locatedin(A,B), ot____nom31(B).
// 687: ot____nom31(A) :- ot____nom31_aux(A).
// 688: ot____nom31(A) :- q16(A), kaon2namedobjects(A).
// 689: q16(A) :- q16(B), kaon2equal(A,B).
// 690: ot____nom31(A) :- ot____nom31(B), kaon2equal(A,B).
// 691: q17(A) :- q17(B), kaon2equal(A,B).
// 692: q56(A) :- locatedin(A,B), ot____nom16(B).
// 693: ot____nom16(A) :- ot____nom16_aux(A).
// 694: ot____nom16(A) :- q55(A), kaon2namedobjects(A).
// 695: q55(A) :- q55(B), kaon2equal(A,B).
// 696: ot____nom16(A) :- ot____nom16(B), kaon2equal(A,B).
// 697: q56(A) :- q56(B), kaon2equal(A,B).
// 698: q52(A) :- locatedin(A,B), ot____nom41(B).
// 699: ot____nom41(A) :- ot____nom41_aux(A).
// 700: ot____nom41(A) :- q51(A), kaon2namedobjects(A).
// 701: q51(A) :- q51(B), kaon2equal(A,B).
// 702: ot____nom41(A) :- ot____nom41(B), kaon2equal(A,B).
// 703: q52(A) :- q52(B), kaon2equal(A,B).
// 704: q27(A) :- locatedin(A,B), ot____nom38(B).
// 705: ot____nom38(A) :- ot____nom38_aux(A).
// 706: ot____nom38(A) :- q26(A), kaon2namedobjects(A).
// 707: q26(A) :- q26(B), kaon2equal(A,B).
// 708: ot____nom38(A) :- ot____nom38(B), kaon2equal(A,B).
// 709: q27(A) :- q27(B), kaon2equal(A,B).
// 710: q23(A) :- locatedin(A,B), ot____nom26(B).
// 711: ot____nom26(A) :- ot____nom26_aux(A).
// 712: ot____nom26(A) :- q22(A), kaon2namedobjects(A).
// 713: q22(A) :- q22(B), kaon2equal(A,B).
// 714: ot____nom26(A) :- ot____nom26(B), kaon2equal(A,B).
// 715: q23(A) :- q23(B), kaon2equal(A,B).
// 716: q10(A) :- locatedin(A,B), ot____nom4(B).
// 717: ot____nom4(A) :- ot____nom4_aux(A).
// 718: ot____nom4(A) :- q9(A), kaon2namedobjects(A).
// 719: q9(A) :- q9(B), kaon2equal(A,B).
// 720: ot____nom4(A) :- ot____nom4(B), kaon2equal(A,B).
// 721: q10(A) :- q10(B), kaon2equal(A,B).
// 722: q50(A) :- locatedin(A,B), ot____nom17(B).
// 723: ot____nom17(A) :- ot____nom17_aux(A).
// 724: ot____nom17(A) :- q49(A), kaon2namedobjects(A).
// 725: q49(A) :- q49(B), kaon2equal(A,B).
// 726: ot____nom17(A) :- ot____nom17(B), kaon2equal(A,B).
// 727: q50(A) :- q50(B), kaon2equal(A,B).
// 728: q67(A) :- locatedin(A,B), ot____nom14(B).
// 729: ot____nom14(A) :- ot____nom14_aux(A).
// 730: ot____nom14(A) :- q66(A), kaon2namedobjects(A).
// 731: ot____nom14(A) :- ot____nom14(B), kaon2equal(A,B).
// 732: q67(A) :- q67(B), kaon2equal(A,B).
// 733: q43(A) :- locatedin(A,B), ot____nom46(B).
// 734: ot____nom46(A) :- ot____nom46_aux(A).
// 735: ot____nom46(A) :- q42(A), kaon2namedobjects(A).
// 736: ot____nom46(A) :- ot____nom46(B), kaon2equal(A,B).
// 737: q43(A) :- q43(B), kaon2equal(A,B).
// 738: q60(A) :- locatedin(A,B), ot____nom13(B).
// 739: ot____nom13(A) :- ot____nom13_aux(A).
// 740: ot____nom13(A) :- q59(A), kaon2namedobjects(A).
// 741: q59(A) :- q59(B), kaon2equal(A,B).
// 742: ot____nom13(A) :- ot____nom13(B), kaon2equal(A,B).
// 743: q60(A) :- q60(B), kaon2equal(A,B).
// 744: q62(A) :- locatedin(A,B), ot____nom47(B).
// 745: ot____nom47(A) :- ot____nom47_aux(A).
// 746: ot____nom47(A) :- q61(A), kaon2namedobjects(A).
// 747: q61(A) :- q61(B), kaon2equal(A,B).
// 748: ot____nom47(A) :- ot____nom47(B), kaon2equal(A,B).
// 749: q62(A) :- q62(B), kaon2equal(A,B).
// 750: q38(A) :- locatedin(A,B), ot____nom42(B).
// 751: ot____nom42(A) :- ot____nom42_aux(A).
// 752: ot____nom42(A) :- q37(A), kaon2namedobjects(A).
// 753: q37(A) :- q37(B), kaon2equal(A,B).
// 754: ot____nom42(A) :- ot____nom42(B), kaon2equal(A,B).
// 755: q38(A) :- q38(B), kaon2equal(A,B).
// 756: q40(A) :- locatedin(A,B), ot____nom30(B).
// 757: ot____nom30(A) :- ot____nom30_aux(A).
// 758: ot____nom30(A) :- q39(A), kaon2namedobjects(A).
// 759: ot____nom30(A) :- ot____nom30(B), kaon2equal(A,B).
// 760: q40(A) :- q40(B), kaon2equal(A,B).
// 761: q19(A) :- locatedin(A,B), ot____nom22(B).
// 762: ot____nom22(A) :- ot____nom22_aux(A).
// 763: ot____nom22(A) :- q18(A), kaon2namedobjects(A).
// 764: ot____nom22(A) :- ot____nom22(B), kaon2equal(A,B).
// 765: q19(A) :- q19(B), kaon2equal(A,B).
// 766: q3(A) :- pauillac(A).
// 767: q3(A) :- zinfandel(A).
// 768: q3(A) :- chardonnay(A).
// 769: q3(A) :- tours(A).
// 770: q3(A) :- riesling(A).
// 771: q3(A) :- semillon(A).
// 772: q3(A) :- petitesyrah(A).
// 773: q3(A) :- whiteburgundy(A).
// 774: q3(A) :- cabernetsauvignon(A).
// 775: q3(A) :- cheninblanc(A).
// 776: q3(A) :- sauvignonblanc(A).
// 777: q3(A) :- cabernetfranc(A).
// 778: q3(A) :- beaujolais(A).
// 779: q3(A) :- stemilion(A).
// 780: q3(A) :- pinotnoir(A).
// 781: q3(A) :- sancerre(A).
// 782: q3(A) :- redburgundy(A).
// 783: q3(A) :- muscadet(A).
// 784: q3(A) :- merlot(A).
// 785: q3(A) :- pinotblanc(A).
// 786: q3(A) :- gamay(A).
// 787: gamay(A) :- gamay(B), kaon2equal(A,B).
// 788: gamay(A) :- wine(A), madefromgrape(A,B), ot____nom33(B).
// 789: q3(A) :- q3(B), kaon2equal(A,B).
// 790: q3(A) :- ot____nom8(A), wine(A), kaon2namedobjects(A).
// 791: ot____nom8(A) :- ot____nom8(B), kaon2equal(A,B).
// 792: hasmaker(A,B) :- hasmaker(C,B), kaon2equal(A,C).
// 793: hasmaker(A,B) :- hasmaker(A,C), kaon2equal(B,C).
// 794: produceswine(A,B) :- produceswine(C,B), kaon2equal(A,C).
// 795: produceswine(A,B) :- produceswine(A,C), kaon2equal(B,C).
// 796: madeintowine(A,B) :- madeintowine(C,B), kaon2equal(A,C).
// 797: madeintowine(A,B) :- madeintowine(A,C), kaon2equal(B,C).
// 798: vintage(A) :- vintage(B), kaon2equal(A,B).


// Counter for number of results:
int num_results = 0;

// Relations for IDB predicates:
set_2_c kaon2equal_set("kaon2equal_set");
set_2_c hasflavor_set("hasflavor_set");
set_2_c haswinedescriptor_set("haswinedescriptor_set");
set_1_c wine_set("wine_set");
set_2_c hassugar_set("hassugar_set");
set_2_c hasmaker_set("hasmaker_set");
set_2_c produceswine_set("produceswine_set");
set_2_c locatedin_set("locatedin_set");
set_2_c hasbody_set("hasbody_set");
set_2_c madeintowine_set("madeintowine_set");
set_2_c madefromgrape_set("madefromgrape_set");
set_2_c hascolor_set("hascolor_set");
set_1_c q4_set("q4_set");
set_1_c q6_set("q6_set");
set_1_c cheninblanc_set("cheninblanc_set");
set_1_c q21_set("q21_set");
set_1_c whitewine_set("whitewine_set");
set_1_c q12_set("q12_set");
set_1_c whitenonsweetwine_set("whitenonsweetwine_set");
set_1_c q36_set("q36_set");
set_1_c ot____nom7_set("ot____nom7_set");
set_1_c ot____nom5_set("ot____nom5_set");
set_1_c q3_set("q3_set");
set_1_c q5_set("q5_set");
set_1_c ot____nom34_set("ot____nom34_set");
set_1_c gamay_set("gamay_set");
set_1_c q14_set("q14_set");
set_1_c ot____nom33_set("ot____nom33_set");
set_1_c q22_set("q22_set");
set_1_c q23_set("q23_set");
set_1_c beaujolais_set("beaujolais_set");
set_1_c q45_set("q45_set");
set_1_c q20_set("q20_set");
set_1_c q0_set("q0_set");
set_1_c q41_set("q41_set");
set_1_c redwine_set("redwine_set");
set_1_c ot____nom28_set("ot____nom28_set");
set_1_c dryredwine_set("dryredwine_set");
set_1_c drywine_set("drywine_set");
set_1_c drywhitewine_set("drywhitewine_set");
set_1_c q31_set("q31_set");
set_1_c q46_set("q46_set");
set_1_c tablewine_set("tablewine_set");
set_1_c ot____nom12_set("ot____nom12_set");
set_1_c whitetablewine_set("whitetablewine_set");
set_1_c redtablewine_set("redtablewine_set");
set_1_c ot____nom26_set("ot____nom26_set");
set_2_c hasvintageyear_set("hasvintageyear_set");
set_1_c vintage_set("vintage_set");
set_1_c meritage_set("meritage_set");
set_1_c ot____nom8_set("ot____nom8_set");
set_1_c riesling_set("riesling_set");
set_1_c ot____nom37_set("ot____nom37_set");
set_1_c dryriesling_set("dryriesling_set");
set_1_c chianti_set("chianti_set");
set_1_c q61_set("q61_set");
set_1_c q62_set("q62_set");
set_1_c italianwine_set("italianwine_set");
set_1_c ot____nom47_set("ot____nom47_set");
set_1_c q63_set("q63_set");
set_1_c chardonnay_set("chardonnay_set");
set_1_c ot____nom24_set("ot____nom24_set");
set_1_c q29_set("q29_set");
set_1_c q30_set("q30_set");
set_1_c burgundy_set("burgundy_set");
set_1_c whiteburgundy_set("whiteburgundy_set");
set_1_c redburgundy_set("redburgundy_set");
set_1_c pinotnoir_set("pinotnoir_set");
set_1_c q72_set("q72_set");
set_1_c ot____nom39_set("ot____nom39_set");
set_1_c ot____nom44_set("ot____nom44_set");
set_1_c q16_set("q16_set");
set_1_c q17_set("q17_set");
set_1_c meursault_set("meursault_set");
set_1_c fullbodiedwine_set("fullbodiedwine_set");
set_1_c q35_set("q35_set");
set_1_c ot____nom45_set("ot____nom45_set");
set_1_c ot____nom31_set("ot____nom31_set");
set_1_c q56_set("q56_set");
set_1_c q55_set("q55_set");
set_1_c ot____nom16_set("ot____nom16_set");
set_1_c muscadet_set("muscadet_set");
set_1_c pinotblanc_set("pinotblanc_set");
set_1_c q24_set("q24_set");
set_1_c ot____nom21_set("ot____nom21_set");
set_1_c q57_set("q57_set");
set_1_c q58_set("q58_set");
set_1_c loire_set("loire_set");
set_1_c whiteloire_set("whiteloire_set");
set_1_c ot____nom19_set("ot____nom19_set");
set_1_c q19_set("q19_set");
set_1_c q18_set("q18_set");
set_1_c ot____nom22_set("ot____nom22_set");
set_1_c margaux_set("margaux_set");
set_1_c q34_set("q34_set");
set_1_c q33_set("q33_set");
set_1_c ot____nom2_set("ot____nom2_set");
set_1_c bordeaux_set("bordeaux_set");
set_1_c q1_set("q1_set");
set_1_c medoc_set("medoc_set");
set_1_c merlot_set("merlot_set");
set_1_c q13_set("q13_set");
set_1_c ot____nom43_set("ot____nom43_set");
set_1_c q2_set("q2_set");
set_1_c whitebordeaux_set("whitebordeaux_set");
set_1_c redbordeaux_set("redbordeaux_set");
set_1_c semillonorsauvignonblanc_set("semillonorsauvignonblanc_set");
set_1_c q68_set("q68_set");
set_1_c ot____nom6_set("ot____nom6_set");
set_1_c ot____nom23_set("ot____nom23_set");
set_1_c q39_set("q39_set");
set_1_c q40_set("q40_set");
set_1_c sancerre_set("sancerre_set");
set_1_c sauvignonblanc_set("sauvignonblanc_set");
set_1_c q44_set("q44_set");
set_1_c ot____nom11_set("ot____nom11_set");
set_1_c q7_set("q7_set");
set_1_c q32_set("q32_set");
set_1_c ot____nom30_set("ot____nom30_set");
set_1_c q37_set("q37_set");
set_1_c q38_set("q38_set");
set_1_c cotesdor_set("cotesdor_set");
set_1_c ot____nom42_set("ot____nom42_set");
set_1_c q42_set("q42_set");
set_1_c q43_set("q43_set");
set_1_c pauillac_set("pauillac_set");
set_1_c cabernetsauvignon_set("cabernetsauvignon_set");
set_1_c q69_set("q69_set");
set_1_c ot____nom29_set("ot____nom29_set");
set_1_c q11_set("q11_set");
set_1_c ot____nom46_set("ot____nom46_set");
set_1_c q66_set("q66_set");
set_1_c q67_set("q67_set");
set_1_c anjou_set("anjou_set");
set_1_c rosewine_set("rosewine_set");
set_1_c q15_set("q15_set");
set_1_c ot____nom32_set("ot____nom32_set");
set_1_c ot____nom14_set("ot____nom14_set");
set_1_c port_set("port_set");
set_1_c sweetwine_set("sweetwine_set");
set_1_c q71_set("q71_set");
set_1_c q70_set("q70_set");
set_1_c ot____nom1_set("ot____nom1_set");
set_1_c sweetriesling_set("sweetriesling_set");
set_1_c dessertwine_set("dessertwine_set");
set_1_c sauternes_set("sauternes_set");
set_1_c lateharvest_set("lateharvest_set");
set_1_c icewine_set("icewine_set");
set_1_c q64_set("q64_set");
set_1_c q65_set("q65_set");
set_1_c stemilion_set("stemilion_set");
set_1_c ot____nom36_set("ot____nom36_set");
set_1_c cabernetfranc_set("cabernetfranc_set");
set_1_c ot____nom3_set("ot____nom3_set");
set_1_c semillon_set("semillon_set");
set_1_c ot____nom27_set("ot____nom27_set");
set_1_c petitesyrah_set("petitesyrah_set");
set_1_c ot____nom25_set("ot____nom25_set");
set_1_c zinfandel_set("zinfandel_set");
set_1_c ot____nom20_set("ot____nom20_set");
set_1_c ot____nom15_set("ot____nom15_set");
set_1_c californiawine_set("californiawine_set");
set_1_c q47_set("q47_set");
set_1_c q48_set("q48_set");
set_1_c ot____nom10_set("ot____nom10_set");
set_1_c texaswine_set("texaswine_set");
set_1_c q73_set("q73_set");
set_1_c q74_set("q74_set");
set_1_c ot____nom41_set("ot____nom41_set");
set_1_c q51_set("q51_set");
set_1_c frenchwine_set("frenchwine_set");
set_1_c q52_set("q52_set");
set_1_c ot____nom38_set("ot____nom38_set");
set_1_c q26_set("q26_set");
set_1_c americanwine_set("americanwine_set");
set_1_c q27_set("q27_set");
set_1_c ot____nom4_set("ot____nom4_set");
set_1_c q9_set("q9_set");
set_1_c tours_set("tours_set");
set_1_c q10_set("q10_set");
set_1_c ot____nom17_set("ot____nom17_set");
set_1_c q49_set("q49_set");
set_1_c germanwine_set("germanwine_set");
set_1_c q50_set("q50_set");
set_1_c ot____nom13_set("ot____nom13_set");
set_1_c q59_set("q59_set");
set_1_c alsatianwine_set("alsatianwine_set");
set_1_c q60_set("q60_set");
rel_n_n_c kaon2equal_bf("kaon2equal_bf");
set_1_c kaon2hu_set("kaon2hu_set");
rel_n_n_c kaon2equal_fb("kaon2equal_fb");
list_1_c kaon2hu_f("kaon2hu_f");
list_2_c kaon2equal_ff("kaon2equal_ff");
rel_n_n_c hasflavor_bf("hasflavor_bf");
rel_n_n_c hassugar_bf("hassugar_bf");
rel_n_n_c hasmaker_bf("hasmaker_bf");
rel_n_n_c hassugar_fb("hassugar_fb");
rel_n_n_c hasflavor_fb("hasflavor_fb");
rel_n_n_c haswinedescriptor_bf("haswinedescriptor_bf");
rel_n_n_c haswinedescriptor_fb("haswinedescriptor_fb");
rel_n_n_c hasmaker_fb("hasmaker_fb");
rel_n_n_c produceswine_bf("produceswine_bf");
rel_n_n_c produceswine_fb("produceswine_fb");
rel_n_n_c locatedin_bf("locatedin_bf");
rel_n_n_c locatedin_fb("locatedin_fb");
rel_n_n_c hasbody_bf("hasbody_bf");
rel_n_n_c hasbody_fb("hasbody_fb");
rel_n_n_c madefromgrape_bf("madefromgrape_bf");
rel_n_n_c hascolor_bf("hascolor_bf");
rel_n_n_c madefromgrape_fb("madefromgrape_fb");
rel_n_n_c hascolor_fb("hascolor_fb");
rel_n_n_c madeintowine_bf("madeintowine_bf");
rel_n_n_c madeintowine_fb("madeintowine_fb");
list_1_c wine_f("wine_f");
rel_n_n_c hasvintageyear_bf("hasvintageyear_bf");
rel_n_n_c hasvintageyear_fb("hasvintageyear_fb");
list_1_c semillonorsauvignonblanc_f("semillonorsauvignonblanc_f");

// Variable declarations:
int v_kaon2hu_1 = -1;
int v_kaon2equal_1 = -1;
int v_kaon2equal_2 = -1;
int v_hasflavor_1 = -1;
int v_hasflavor_2 = -1;
int v_haswinedescriptor_1 = -1;
int v_haswinedescriptor_2 = -1;
int v_wine_1 = -1;
int v_hassugar_1 = -1;
int v_hassugar_2 = -1;
int v_hasmaker_1 = -1;
int v_hasmaker_2 = -1;
int v_produceswine_1 = -1;
int v_produceswine_2 = -1;
int v_locatedin_1 = -1;
int v_locatedin_2 = -1;
int v_hasbody_1 = -1;
int v_hasbody_2 = -1;
int v_madefromgrape_1 = -1;
int v_madefromgrape_2 = -1;
int v_madeintowine_1 = -1;
int v_madeintowine_2 = -1;
int v_hascolor_1 = -1;
int v_hascolor_2 = -1;
int v_cheninblanc_1 = -1;
int v_q4_1 = -1;
int v_q6_1 = -1;
int v_q12_1 = -1;
int v_q21_1 = -1;
int v_whitewine_1 = -1;
int v_whitenonsweetwine_1 = -1;
int v_q36_1 = -1;
int v_ot____nom7_1 = -1;
int v_ot____nom5_1 = -1;
int v_q3_1 = -1;
int v_q5_1 = -1;
int v_ot____nom34_1 = -1;
int v_beaujolais_1 = -1;
int v_q14_1 = -1;
int v_gamay_1 = -1;
int v_ot____nom33_1 = -1;
int v_q22_1 = -1;
int v_q23_1 = -1;
int v_q45_1 = -1;
int v_q20_1 = -1;
int v_q0_1 = -1;
int v_q41_1 = -1;
int v_redwine_1 = -1;
int v_ot____nom28_1 = -1;
int v_q31_1 = -1;
int v_q46_1 = -1;
int v_drywine_1 = -1;
int v_dryredwine_1 = -1;
int v_drywhitewine_1 = -1;
int v_tablewine_1 = -1;
int v_ot____nom12_1 = -1;
int v_whitetablewine_1 = -1;
int v_redtablewine_1 = -1;
int v_ot____nom26_1 = -1;
int v_hasvintageyear_1 = -1;
int v_hasvintageyear_2 = -1;
int v_vintage_1 = -1;
int v_meritage_1 = -1;
int v_ot____nom8_1 = -1;
int v_riesling_1 = -1;
int v_ot____nom37_1 = -1;
int v_dryriesling_1 = -1;
int v_chianti_1 = -1;
int v_italianwine_1 = -1;
int v_q61_1 = -1;
int v_q62_1 = -1;
int v_ot____nom47_1 = -1;
int v_chardonnay_1 = -1;
int v_q63_1 = -1;
int v_ot____nom24_1 = -1;
int v_whiteburgundy_1 = -1;
int v_burgundy_1 = -1;
int v_q29_1 = -1;
int v_q30_1 = -1;
int v_redburgundy_1 = -1;
int v_q72_1 = -1;
int v_pinotnoir_1 = -1;
int v_ot____nom39_1 = -1;
int v_ot____nom44_1 = -1;
int v_meursault_1 = -1;
int v_q16_1 = -1;
int v_q17_1 = -1;
int v_q35_1 = -1;
int v_fullbodiedwine_1 = -1;
int v_ot____nom45_1 = -1;
int v_ot____nom31_1 = -1;
int v_muscadet_1 = -1;
int v_q55_1 = -1;
int v_q56_1 = -1;
int v_ot____nom16_1 = -1;
int v_q24_1 = -1;
int v_pinotblanc_1 = -1;
int v_ot____nom21_1 = -1;
int v_loire_1 = -1;
int v_q57_1 = -1;
int v_q58_1 = -1;
int v_whiteloire_1 = -1;
int v_ot____nom19_1 = -1;
int v_margaux_1 = -1;
int v_q18_1 = -1;
int v_q19_1 = -1;
int v_ot____nom22_1 = -1;
int v_medoc_1 = -1;
int v_q33_1 = -1;
int v_q34_1 = -1;
int v_ot____nom2_1 = -1;
int v_bordeaux_1 = -1;
int v_q1_1 = -1;
int v_q13_1 = -1;
int v_merlot_1 = -1;
int v_ot____nom43_1 = -1;
int v_q2_1 = -1;
int v_whitebordeaux_1 = -1;
int v_redbordeaux_1 = -1;
int v_q68_1 = -1;
int v_semillonorsauvignonblanc_1 = -1;
int v_ot____nom6_1 = -1;
int v_ot____nom23_1 = -1;
int v_sancerre_1 = -1;
int v_q39_1 = -1;
int v_q40_1 = -1;
int v_q44_1 = -1;
int v_sauvignonblanc_1 = -1;
int v_ot____nom11_1 = -1;
int v_q7_1 = -1;
int v_q32_1 = -1;
int v_ot____nom30_1 = -1;
int v_cotesdor_1 = -1;
int v_q37_1 = -1;
int v_q38_1 = -1;
int v_ot____nom42_1 = -1;
int v_pauillac_1 = -1;
int v_q42_1 = -1;
int v_q43_1 = -1;
int v_q69_1 = -1;
int v_cabernetsauvignon_1 = -1;
int v_ot____nom29_1 = -1;
int v_q11_1 = -1;
int v_ot____nom46_1 = -1;
int v_anjou_1 = -1;
int v_q66_1 = -1;
int v_q67_1 = -1;
int v_q15_1 = -1;
int v_rosewine_1 = -1;
int v_ot____nom32_1 = -1;
int v_ot____nom14_1 = -1;
int v_port_1 = -1;
int v_q70_1 = -1;
int v_q71_1 = -1;
int v_sweetwine_1 = -1;
int v_ot____nom1_1 = -1;
int v_sweetriesling_1 = -1;
int v_dessertwine_1 = -1;
int v_sauternes_1 = -1;
int v_lateharvest_1 = -1;
int v_icewine_1 = -1;
int v_stemilion_1 = -1;
int v_q64_1 = -1;
int v_q65_1 = -1;
int v_ot____nom36_1 = -1;
int v_cabernetfranc_1 = -1;
int v_ot____nom3_1 = -1;
int v_semillon_1 = -1;
int v_ot____nom27_1 = -1;
int v_petitesyrah_1 = -1;
int v_ot____nom25_1 = -1;
int v_zinfandel_1 = -1;
int v_ot____nom20_1 = -1;
int v_ot____nom15_1 = -1;
int v_q48_1 = -1;
int v_californiawine_1 = -1;
int v_q47_1 = -1;
int v_ot____nom10_1 = -1;
int v_q74_1 = -1;
int v_texaswine_1 = -1;
int v_q73_1 = -1;
int v_ot____nom41_1 = -1;
int v_q52_1 = -1;
int v_frenchwine_1 = -1;
int v_q51_1 = -1;
int v_ot____nom38_1 = -1;
int v_q27_1 = -1;
int v_americanwine_1 = -1;
int v_q26_1 = -1;
int v_ot____nom4_1 = -1;
int v_q10_1 = -1;
int v_tours_1 = -1;
int v_q9_1 = -1;
int v_ot____nom17_1 = -1;
int v_q50_1 = -1;
int v_germanwine_1 = -1;
int v_q49_1 = -1;
int v_ot____nom13_1 = -1;
int v_q60_1 = -1;
int v_alsatianwine_1 = -1;
int v_q59_1 = -1;

// Cursor declarations:
cur_1_c c7_1_f(&kaon2namedobjects_f);
cur_n_n_c c96_2_bf(&kaon2equal_bf);
cur_n_n_c c96_1_fb(&kaon2equal_fb);
cur_1_c c95_2_f(&kaon2hu_f);
cur_1_c c95_1_f(&kaon2hu_f);
cur_n_n_c c96_1_bf(&kaon2equal_bf);
cur_2_c c96_1_ff(&kaon2equal_ff);
cur_2_c c9_1_ff(&hasflavor_aux_ff);
cur_n_n_c c8_2_bf(&hasflavor_bf);
cur_n_n_c c648_1_bf(&hasflavor_bf);
cur_n_n_c c26_2_bf(&hassugar_bf);
cur_n_n_c c61_2_bf(&hasmaker_bf);
cur_n_n_c c593_1_bf(&hassugar_bf);
cur_n_n_c c593_2_fb(&kaon2equal_fb);
cur_n_n_c c594_2_fb(&kaon2equal_fb);
cur_n_n_c c651_2_fb(&kaon2equal_fb);
cur_n_n_c c652_2_fb(&kaon2equal_fb);
cur_n_n_c c26_1_bf(&hassugar_bf);
cur_n_n_c c594_1_fb(&hassugar_fb);
cur_n_n_c c649_1_fb(&hasflavor_fb);
cur_n_n_c c648_2_fb(&kaon2equal_fb);
cur_n_n_c c649_2_fb(&kaon2equal_fb);
cur_n_n_c c8_1_bf(&hasflavor_bf);
cur_n_n_c c651_1_bf(&haswinedescriptor_bf);
cur_n_n_c c652_1_fb(&haswinedescriptor_fb);
cur_n_n_c c792_1_bf(&hasmaker_bf);
cur_n_n_c c792_2_fb(&kaon2equal_fb);
cur_n_n_c c793_2_fb(&kaon2equal_fb);
cur_n_n_c c61_1_bf(&hasmaker_bf);
cur_n_n_c c793_1_fb(&hasmaker_fb);
cur_n_n_c c794_1_bf(&produceswine_bf);
cur_n_n_c c794_2_fb(&kaon2equal_fb);
cur_n_n_c c795_2_fb(&kaon2equal_fb);
cur_n_n_c c795_1_fb(&produceswine_fb);
cur_n_n_c c218_2_fb(&kaon2equal_fb);
cur_n_n_c c219_2_fb(&kaon2equal_fb);
cur_n_n_c c220_2_bf(&locatedin_bf);
cur_n_n_c c221_2_bf(&locatedin_bf);
cur_n_n_c c220_1_fb(&locatedin_fb);
cur_n_n_c c221_1_fb(&locatedin_fb);
cur_n_n_c c78_2_bf(&hasbody_bf);
cur_n_n_c c218_1_bf(&locatedin_bf);
cur_n_n_c c219_1_fb(&locatedin_fb);
cur_n_n_c c621_1_bf(&hasbody_bf);
cur_n_n_c c621_2_fb(&kaon2equal_fb);
cur_n_n_c c622_2_fb(&kaon2equal_fb);
cur_n_n_c c78_1_bf(&hasbody_bf);
cur_n_n_c c622_1_fb(&hasbody_fb);
cur_n_n_c c16_2_bf(&madefromgrape_bf);
cur_n_n_c c249_2_bf(&madefromgrape_bf);
cur_n_n_c c278_2_bf(&madefromgrape_bf);
cur_n_n_c c326_2_bf(&madefromgrape_bf);
cur_n_n_c c332_2_bf(&madefromgrape_bf);
cur_n_n_c c364_2_bf(&madefromgrape_bf);
cur_n_n_c c375_2_bf(&hasbody_bf);
cur_n_n_c c395_2_bf(&madefromgrape_bf);
cur_n_n_c c402_2_bf(&madefromgrape_bf);
cur_n_n_c c421_2_bf(&madefromgrape_bf);
cur_n_n_c c431_2_bf(&madefromgrape_bf);
cur_n_n_c c40_2_bf(&hascolor_bf);
cur_n_n_c c529_1_bf(&madefromgrape_bf);
cur_n_n_c c529_2_fb(&kaon2equal_fb);
cur_n_n_c c530_2_fb(&kaon2equal_fb);
cur_n_n_c c530_1_fb(&madefromgrape_fb);
cur_n_n_c c601_1_bf(&hascolor_bf);
cur_n_n_c c265_2_bf(&hascolor_bf);
cur_n_n_c c508_2_bf(&madefromgrape_bf);
cur_n_n_c c516_2_fb(&kaon2equal_fb);
cur_n_n_c c534_2_bf(&madefromgrape_bf);
cur_n_n_c c538_2_bf(&madefromgrape_bf);
cur_n_n_c c546_2_bf(&madefromgrape_bf);
cur_n_n_c c550_2_bf(&madefromgrape_bf);
cur_n_n_c c554_2_bf(&madefromgrape_bf);
cur_n_n_c c558_2_bf(&madefromgrape_bf);
cur_n_n_c c562_2_bf(&madefromgrape_bf);
cur_n_n_c c570_2_bf(&madefromgrape_bf);
cur_n_n_c c574_2_bf(&madefromgrape_bf);
cur_n_n_c c578_2_bf(&madefromgrape_bf);
cur_n_n_c c788_2_bf(&madefromgrape_bf);
cur_n_n_c c601_2_fb(&kaon2equal_fb);
cur_n_n_c c602_2_fb(&kaon2equal_fb);
cur_n_n_c c40_1_bf(&hascolor_bf);
cur_n_n_c c602_1_fb(&hascolor_fb);
cur_n_n_c c796_1_bf(&madeintowine_bf);
cur_n_n_c c796_2_fb(&kaon2equal_fb);
cur_n_n_c c797_2_fb(&kaon2equal_fb);
cur_n_n_c c797_1_fb(&madeintowine_fb);
cur_1_c c12_1_f(&cheninblanc_aux_f);
cur_n_n_c c492_2_fb(&kaon2equal_fb);
cur_n_n_c c15_2_fb(&kaon2equal_fb);
cur_n_n_c c457_2_fb(&kaon2equal_fb);
cur_n_n_c c584_2_fb(&kaon2equal_fb);
cur_n_n_c c587_2_bf(&hassugar_bf);
cur_n_n_c c595_2_fb(&kaon2equal_fb);
cur_n_n_c c590_2_fb(&kaon2equal_fb);
cur_n_n_c c600_2_fb(&kaon2equal_fb);
cur_n_n_c c658_2_fb(&kaon2equal_fb);
cur_n_n_c c258_1_fb(&hascolor_fb);
cur_n_n_c c455_2_fb(&kaon2equal_fb);
cur_n_n_c c88_2_bf(&madefromgrape_bf);
cur_n_n_c c88_3_bf(&madefromgrape_bf);
cur_n_n_c c468_2_fb(&kaon2equal_fb);
cur_n_n_c c507_2_fb(&kaon2equal_fb);
cur_n_n_c c789_2_fb(&kaon2equal_fb);
cur_n_n_c c549_2_fb(&kaon2equal_fb);
cur_1_c c16_1_f(&wine_f);
cur_1_c c546_1_f(&wine_f);
cur_1_c c21_1_f(&beaujolais_aux_f);
cur_n_n_c c502_2_fb(&kaon2equal_fb);
cur_n_n_c c561_2_fb(&kaon2equal_fb);
cur_1_c c558_1_f(&wine_f);
cur_1_c c788_1_f(&wine_f);
cur_n_n_c c787_2_fb(&kaon2equal_fb);
cur_n_n_c c25_2_fb(&kaon2equal_fb);
cur_n_n_c c646_2_fb(&kaon2equal_fb);
cur_n_n_c c611_2_fb(&kaon2equal_fb);
cur_n_n_c c446_1_fb(&hascolor_fb);
cur_n_n_c c612_2_fb(&kaon2equal_fb);
cur_n_n_c c610_2_fb(&kaon2equal_fb);
cur_n_n_c c607_2_fb(&kaon2equal_fb);
cur_n_n_c c470_2_fb(&kaon2equal_fb);
cur_n_n_c c617_2_fb(&kaon2equal_fb);
cur_n_n_c c640_2_fb(&kaon2equal_fb);
cur_n_n_c c661_2_fb(&kaon2equal_fb);
cur_n_n_c c241_1_fb(&hassugar_fb);
cur_n_n_c c614_2_fb(&kaon2equal_fb);
cur_n_n_c c398_2_fb(&kaon2equal_fb);
cur_n_n_c c616_2_fb(&kaon2equal_fb);
cur_n_n_c c298_2_fb(&kaon2equal_fb);
cur_n_n_c c239_2_fb(&kaon2equal_fb);
cur_n_n_c c405_2_fb(&kaon2equal_fb);
cur_n_n_c c715_2_fb(&kaon2equal_fb);
cur_n_n_c c163_1_fb(&locatedin_fb);
cur_n_n_c c714_2_fb(&kaon2equal_fb);
cur_n_n_c c710_1_fb(&locatedin_fb);
cur_n_n_c c713_2_fb(&kaon2equal_fb);
cur_2_c c27_1_ff(&hassugar_aux_ff);
cur_2_c c35_1_ff(&hasvintageyear_aux_ff);
cur_n_n_c c34_2_bf(&hasvintageyear_bf);
cur_n_n_c c38_1_bf(&hasvintageyear_bf);
cur_n_n_c c38_2_fb(&kaon2equal_fb);
cur_n_n_c c39_2_fb(&kaon2equal_fb);
cur_n_n_c c34_1_bf(&hasvintageyear_bf);
cur_n_n_c c39_1_fb(&hasvintageyear_fb);
cur_n_n_c c798_2_fb(&kaon2equal_fb);
cur_2_c c41_1_ff(&hascolor_aux_ff);
cur_1_c c44_1_f(&meritage_aux_f);
cur_n_n_c c45_2_fb(&kaon2equal_fb);
cur_n_n_c c48_2_bf(&madefromgrape_bf);
cur_n_n_c c791_2_fb(&kaon2equal_fb);
cur_1_c c47_1_f(&ot____nom8_aux_f);
cur_2_c c49_1_ff(&madefromgrape_aux_ff);
cur_1_c c53_1_f(&riesling_aux_f);
cur_n_n_c c430_2_fb(&kaon2equal_fb);
cur_n_n_c c553_2_fb(&kaon2equal_fb);
cur_1_c c431_1_f(&wine_f);
cur_n_n_c c140_2_fb(&kaon2equal_fb);
cur_1_c c550_1_f(&wine_f);
cur_1_c c55_1_f(&dryriesling_aux_f);
cur_1_c c59_1_f(&chianti_aux_f);
cur_n_n_c c60_2_fb(&kaon2equal_fb);
cur_n_n_c c494_2_fb(&kaon2equal_fb);
cur_n_n_c c749_2_fb(&kaon2equal_fb);
cur_n_n_c c119_1_fb(&locatedin_fb);
cur_n_n_c c748_2_fb(&kaon2equal_fb);
cur_n_n_c c744_1_fb(&locatedin_fb);
cur_n_n_c c747_2_fb(&kaon2equal_fb);
cur_2_c c62_1_ff(&hasmaker_aux_ff);
cur_1_c c68_1_f(&chardonnay_aux_f);
cur_n_n_c c401_2_fb(&kaon2equal_fb);
cur_n_n_c c505_2_fb(&kaon2equal_fb);
cur_n_n_c c541_2_fb(&kaon2equal_fb);
cur_1_c c402_1_f(&wine_f);
cur_1_c c538_1_f(&wine_f);
cur_1_c c71_1_f(&whiteburgundy_aux_f);
cur_n_n_c c300_2_fb(&kaon2equal_fb);
cur_n_n_c c476_2_fb(&kaon2equal_fb);
cur_n_n_c c231_2_fb(&kaon2equal_fb);
cur_n_n_c c355_2_fb(&kaon2equal_fb);
cur_n_n_c c565_2_fb(&kaon2equal_fb);
cur_1_c c326_1_f(&wine_f);
cur_n_n_c c325_2_fb(&kaon2equal_fb);
cur_1_c c562_1_f(&wine_f);
cur_n_n_c c679_2_fb(&kaon2equal_fb);
cur_n_n_c c194_1_fb(&locatedin_fb);
cur_n_n_c c678_2_fb(&kaon2equal_fb);
cur_n_n_c c674_1_fb(&locatedin_fb);
cur_n_n_c c677_2_fb(&kaon2equal_fb);
cur_1_c c73_1_f(&meursault_aux_f);
cur_n_n_c c77_2_fb(&kaon2equal_fb);
cur_n_n_c c413_2_fb(&kaon2equal_fb);
cur_n_n_c c625_2_fb(&kaon2equal_fb);
cur_1_c c375_1_f(&wine_f);
cur_n_n_c c374_2_fb(&kaon2equal_fb);
cur_n_n_c c691_2_fb(&kaon2equal_fb);
cur_n_n_c c181_1_fb(&locatedin_fb);
cur_n_n_c c690_2_fb(&kaon2equal_fb);
cur_n_n_c c686_1_fb(&locatedin_fb);
cur_n_n_c c689_2_fb(&kaon2equal_fb);
cur_2_c c79_1_ff(&hasbody_aux_ff);
cur_1_c c83_1_f(&muscadet_aux_f);
cur_n_n_c c697_2_fb(&kaon2equal_fb);
cur_n_n_c c179_1_fb(&locatedin_fb);
cur_n_n_c c696_2_fb(&kaon2equal_fb);
cur_n_n_c c692_1_fb(&locatedin_fb);
cur_n_n_c c695_2_fb(&kaon2equal_fb);
cur_n_n_c c87_2_fb(&kaon2equal_fb);
cur_n_n_c c362_2_fb(&kaon2equal_fb);
cur_n_n_c c577_2_fb(&kaon2equal_fb);
cur_1_c c364_1_f(&wine_f);
cur_n_n_c c363_2_fb(&kaon2equal_fb);
cur_1_c c574_1_f(&wine_f);
cur_n_n_c c618_2_fb(&kaon2equal_fb);
cur_n_n_c c451_2_fb(&kaon2equal_fb);
cur_n_n_c c391_2_fb(&kaon2equal_fb);
cur_n_n_c c385_1_fb(&locatedin_fb);
cur_n_n_c c390_2_fb(&kaon2equal_fb);
cur_n_n_c c386_1_fb(&locatedin_fb);
cur_n_n_c c389_2_fb(&kaon2equal_fb);
cur_1_c c90_1_f(&margaux_aux_f);
cur_n_n_c c765_2_fb(&kaon2equal_fb);
cur_n_n_c c97_1_fb(&locatedin_fb);
cur_n_n_c c149_2_fb(&kaon2equal_fb);
cur_n_n_c c764_2_fb(&kaon2equal_fb);
cur_n_n_c c761_1_fb(&locatedin_fb);
cur_n_n_c c94_2_fb(&kaon2equal_fb);
cur_n_n_c c673_2_fb(&kaon2equal_fb);
cur_n_n_c c204_1_fb(&locatedin_fb);
cur_n_n_c c672_2_fb(&kaon2equal_fb);
cur_n_n_c c668_1_fb(&locatedin_fb);
cur_n_n_c c671_2_fb(&kaon2equal_fb);
cur_n_n_c c271_2_fb(&kaon2equal_fb);
cur_n_n_c c472_2_fb(&kaon2equal_fb);
cur_n_n_c c537_2_fb(&kaon2equal_fb);
cur_1_c c278_1_f(&wine_f);
cur_n_n_c c277_2_fb(&kaon2equal_fb);
cur_1_c c534_1_f(&wine_f);
cur_n_n_c c427_2_fb(&kaon2equal_fb);
cur_n_n_c c233_2_fb(&kaon2equal_fb);
cur_n_n_c c305_2_fb(&kaon2equal_fb);
cur_n_n_c c337_2_bf(&madefromgrape_bf);
cur_n_n_c c344_2_bf(&madefromgrape_bf);
cur_n_n_c c351_2_bf(&madefromgrape_bf);
cur_n_n_c c348_2_fb(&kaon2equal_fb);
cur_n_n_c c542_2_bf(&madefromgrape_bf);
cur_n_n_c c566_2_bf(&madefromgrape_bf);
cur_n_n_c c626_2_fb(&kaon2equal_fb);
cur_n_n_c c352_2_fb(&kaon2equal_fb);
cur_n_n_c c314_2_fb(&kaon2equal_fb);
cur_n_n_c c308_1_fb(&locatedin_fb);
cur_n_n_c c313_2_fb(&kaon2equal_fb);
cur_n_n_c c309_1_fb(&locatedin_fb);
cur_n_n_c c312_2_fb(&kaon2equal_fb);
cur_2_c c98_1_ff(&locatedin_aux_ff);
cur_1_c c102_1_f(&sancerre_aux_f);
cur_n_n_c c106_2_fb(&kaon2equal_fb);
cur_n_n_c c425_2_fb(&kaon2equal_fb);
cur_n_n_c c545_2_fb(&kaon2equal_fb);
cur_1_c c344_1_f(&semillonorsauvignonblanc_f);
cur_n_n_c c343_2_fb(&kaon2equal_fb);
cur_1_c c542_1_f(&semillonorsauvignonblanc_f);
cur_n_n_c c410_2_fb(&kaon2equal_fb);
cur_n_n_c c513_2_fb(&kaon2equal_fb);
cur_n_n_c c760_2_fb(&kaon2equal_fb);
cur_n_n_c c107_1_fb(&locatedin_fb);
cur_n_n_c c208_2_fb(&kaon2equal_fb);
cur_n_n_c c759_2_fb(&kaon2equal_fb);
cur_n_n_c c756_1_fb(&locatedin_fb);
cur_1_c c110_1_f(&cotesdor_aux_f);
cur_n_n_c c190_2_fb(&kaon2equal_fb);
cur_n_n_c c755_2_fb(&kaon2equal_fb);
cur_n_n_c c113_1_fb(&locatedin_fb);
cur_n_n_c c754_2_fb(&kaon2equal_fb);
cur_n_n_c c750_1_fb(&locatedin_fb);
cur_n_n_c c753_2_fb(&kaon2equal_fb);
cur_1_c c128_1_f(&pauillac_aux_f);
cur_n_n_c c132_2_fb(&kaon2equal_fb);
cur_n_n_c c465_2_fb(&kaon2equal_fb);
cur_n_n_c c581_2_fb(&kaon2equal_fb);
cur_1_c c249_1_f(&wine_f);
cur_n_n_c c248_2_fb(&kaon2equal_fb);
cur_1_c c578_1_f(&wine_f);
cur_n_n_c c284_2_fb(&kaon2equal_fb);
cur_n_n_c c737_2_fb(&kaon2equal_fb);
cur_n_n_c c133_1_fb(&locatedin_fb);
cur_n_n_c c156_2_fb(&kaon2equal_fb);
cur_n_n_c c736_2_fb(&kaon2equal_fb);
cur_n_n_c c733_1_fb(&locatedin_fb);
cur_1_c c142_1_f(&anjou_aux_f);
cur_n_n_c c146_2_fb(&kaon2equal_fb);
cur_n_n_c c263_2_fb(&kaon2equal_fb);
cur_n_n_c c655_2_fb(&kaon2equal_fb);
cur_1_c c265_1_f(&wine_f);
cur_n_n_c c264_2_fb(&kaon2equal_fb);
cur_n_n_c c732_2_fb(&kaon2equal_fb);
cur_n_n_c c147_1_fb(&locatedin_fb);
cur_n_n_c c217_2_fb(&kaon2equal_fb);
cur_n_n_c c731_2_fb(&kaon2equal_fb);
cur_n_n_c c728_1_fb(&locatedin_fb);
cur_1_c c166_1_f(&port_aux_f);
cur_n_n_c c167_2_fb(&kaon2equal_fb);
cur_n_n_c c487_2_fb(&kaon2equal_fb);
cur_n_n_c c629_2_fb(&kaon2equal_fb);
cur_n_n_c c294_1_fb(&hassugar_fb);
cur_n_n_c c371_2_fb(&kaon2equal_fb);
cur_n_n_c c462_2_fb(&kaon2equal_fb);
cur_n_n_c c630_2_fb(&kaon2equal_fb);
cur_n_n_c c482_2_fb(&kaon2equal_fb);
cur_1_c c198_1_f(&sauternes_aux_f);
cur_n_n_c c199_2_fb(&kaon2equal_fb);
cur_n_n_c c486_2_fb(&kaon2equal_fb);
cur_n_n_c c256_2_fb(&kaon2equal_fb);
cur_1_c c211_1_f(&stemilion_aux_f);
cur_n_n_c c283_2_fb(&kaon2equal_fb);
cur_n_n_c c227_2_fb(&kaon2equal_fb);
cur_n_n_c c214_1_fb(&locatedin_fb);
cur_n_n_c c226_2_fb(&kaon2equal_fb);
cur_n_n_c c222_1_fb(&locatedin_fb);
cur_n_n_c c225_2_fb(&kaon2equal_fb);
cur_1_c c223_1_f(&ot____nom36_aux_f);
cur_1_c c244_1_f(&cabernetsauvignon_aux_f);
cur_1_c c255_1_f(&icewine_aux_f);
cur_1_c c273_1_f(&merlot_aux_f);
cur_1_c c292_1_f(&sweetriesling_aux_f);
cur_1_c c296_1_f(&whitewine_aux_f);
cur_1_c c310_1_f(&ot____nom23_aux_f);
cur_1_c c322_1_f(&pinotnoir_aux_f);
cur_1_c c330_1_f(&cabernetfranc_aux_f);
cur_n_n_c c331_2_fb(&kaon2equal_fb);
cur_n_n_c c533_2_fb(&kaon2equal_fb);
cur_1_c c332_1_f(&wine_f);
cur_1_c c508_1_f(&wine_f);
cur_1_c c335_1_f(&semillon_aux_f);
cur_n_n_c c336_2_fb(&kaon2equal_fb);
cur_n_n_c c569_2_fb(&kaon2equal_fb);
cur_1_c c337_1_f(&semillonorsauvignonblanc_f);
cur_1_c c566_1_f(&semillonorsauvignonblanc_f);
cur_1_c c339_1_f(&sauvignonblanc_aux_f);
cur_1_c c350_1_f(&ot____nom6_aux_f);
cur_1_c c387_1_f(&ot____nom19_aux_f);
cur_1_c c393_1_f(&petitesyrah_aux_f);
cur_n_n_c c394_2_fb(&kaon2equal_fb);
cur_n_n_c c573_2_fb(&kaon2equal_fb);
cur_1_c c395_1_f(&wine_f);
cur_1_c c570_1_f(&wine_f);
cur_1_c c419_1_f(&zinfandel_aux_f);
cur_n_n_c c420_2_fb(&kaon2equal_fb);
cur_n_n_c c557_2_fb(&kaon2equal_fb);
cur_1_c c421_1_f(&wine_f);
cur_1_c c554_1_f(&wine_f);
cur_1_c c444_1_f(&redtablewine_aux_f);
cur_1_c c459_1_f(&dessertwine_aux_f);
cur_1_c c531_1_f(&ot____nom3_aux_f);
cur_1_c c535_1_f(&ot____nom43_aux_f);
cur_1_c c539_1_f(&ot____nom24_aux_f);
cur_1_c c543_1_f(&ot____nom11_aux_f);
cur_1_c c547_1_f(&ot____nom34_aux_f);
cur_1_c c551_1_f(&ot____nom37_aux_f);
cur_1_c c555_1_f(&ot____nom20_aux_f);
cur_1_c c559_1_f(&ot____nom33_aux_f);
cur_1_c c563_1_f(&ot____nom39_aux_f);
cur_1_c c567_1_f(&ot____nom27_aux_f);
cur_1_c c571_1_f(&ot____nom25_aux_f);
cur_1_c c575_1_f(&ot____nom21_aux_f);
cur_1_c c579_1_f(&ot____nom29_aux_f);
cur_1_c c586_1_f(&ot____nom7_aux_f);
cur_1_c c603_1_f(&ot____nom28_aux_f);
cur_1_c c623_1_f(&ot____nom45_aux_f);
cur_1_c c627_1_f(&ot____nom1_aux_f);
cur_1_c c653_1_f(&ot____nom32_aux_f);
cur_1_c c656_1_f(&ot____nom5_aux_f);
cur_1_c c659_1_f(&ot____nom12_aux_f);
cur_1_c c663_1_f(&ot____nom15_aux_f);
cur_n_n_c c666_2_fb(&kaon2equal_fb);
cur_n_n_c c662_1_fb(&locatedin_fb);
cur_n_n_c c667_2_fb(&kaon2equal_fb);
cur_n_n_c c206_1_fb(&locatedin_fb);
cur_n_n_c c665_2_fb(&kaon2equal_fb);
cur_n_n_c c5_2_fb(&kaon2equal_fb);
cur_1_c c669_1_f(&ot____nom2_aux_f);
cur_1_c c675_1_f(&ot____nom44_aux_f);
cur_1_c c681_1_f(&ot____nom10_aux_f);
cur_n_n_c c684_2_fb(&kaon2equal_fb);
cur_n_n_c c680_1_fb(&locatedin_fb);
cur_n_n_c c685_2_fb(&kaon2equal_fb);
cur_n_n_c c183_1_fb(&locatedin_fb);
cur_n_n_c c683_2_fb(&kaon2equal_fb);
cur_n_n_c c33_2_fb(&kaon2equal_fb);
cur_1_c c687_1_f(&ot____nom31_aux_f);
cur_1_c c693_1_f(&ot____nom16_aux_f);
cur_1_c c699_1_f(&ot____nom41_aux_f);
cur_n_n_c c702_2_fb(&kaon2equal_fb);
cur_n_n_c c698_1_fb(&locatedin_fb);
cur_n_n_c c703_2_fb(&kaon2equal_fb);
cur_n_n_c c177_1_fb(&locatedin_fb);
cur_n_n_c c701_2_fb(&kaon2equal_fb);
cur_n_n_c c416_2_fb(&kaon2equal_fb);
cur_1_c c705_1_f(&ot____nom38_aux_f);
cur_n_n_c c708_2_fb(&kaon2equal_fb);
cur_n_n_c c704_1_fb(&locatedin_fb);
cur_n_n_c c709_2_fb(&kaon2equal_fb);
cur_n_n_c c172_1_fb(&locatedin_fb);
cur_n_n_c c707_2_fb(&kaon2equal_fb);
cur_n_n_c c316_2_fb(&kaon2equal_fb);
cur_1_c c711_1_f(&ot____nom26_aux_f);
cur_1_c c717_1_f(&ot____nom4_aux_f);
cur_n_n_c c720_2_fb(&kaon2equal_fb);
cur_n_n_c c716_1_fb(&locatedin_fb);
cur_n_n_c c721_2_fb(&kaon2equal_fb);
cur_n_n_c c161_1_fb(&locatedin_fb);
cur_n_n_c c719_2_fb(&kaon2equal_fb);
cur_n_n_c c378_2_fb(&kaon2equal_fb);
cur_1_c c723_1_f(&ot____nom17_aux_f);
cur_n_n_c c726_2_fb(&kaon2equal_fb);
cur_n_n_c c722_1_fb(&locatedin_fb);
cur_n_n_c c727_2_fb(&kaon2equal_fb);
cur_n_n_c c154_1_fb(&locatedin_fb);
cur_n_n_c c725_2_fb(&kaon2equal_fb);
cur_n_n_c c474_2_fb(&kaon2equal_fb);
cur_1_c c729_1_f(&ot____nom14_aux_f);
cur_1_c c734_1_f(&ot____nom46_aux_f);
cur_1_c c739_1_f(&ot____nom13_aux_f);
cur_n_n_c c742_2_fb(&kaon2equal_fb);
cur_n_n_c c738_1_fb(&locatedin_fb);
cur_n_n_c c743_2_fb(&kaon2equal_fb);
cur_n_n_c c124_1_fb(&locatedin_fb);
cur_n_n_c c741_2_fb(&kaon2equal_fb);
cur_n_n_c c318_2_fb(&kaon2equal_fb);
cur_1_c c745_1_f(&ot____nom47_aux_f);
cur_1_c c751_1_f(&ot____nom42_aux_f);
cur_1_c c757_1_f(&ot____nom30_aux_f);
cur_1_c c762_1_f(&ot____nom22_aux_f);

// Stack declarations:
stack_c<int> backtrack_stack("backtrack_stack");
stack_c<int> int_value_stack("int_value_stack");
stack_c<int> cursor_stack("cursor_stack");

// Initialisation Part
if(!backtrack_stack.push(1012)) return -1;
if(!backtrack_stack.push(1011)) return -1;
if(!backtrack_stack.push(1010)) return -1;
if(!backtrack_stack.push(1009)) return -1;
if(!backtrack_stack.push(993)) return -1;
if(!backtrack_stack.push(992)) return -1;
if(!backtrack_stack.push(991)) return -1;
if(!backtrack_stack.push(975)) return -1;
if(!backtrack_stack.push(957)) return -1;
if(!backtrack_stack.push(956)) return -1;
if(!backtrack_stack.push(940)) return -1;
if(!backtrack_stack.push(924)) return -1;
if(!backtrack_stack.push(923)) return -1;
if(!backtrack_stack.push(922)) return -1;
if(!backtrack_stack.push(906)) return -1;
if(!backtrack_stack.push(905)) return -1;
if(!backtrack_stack.push(904)) return -1;
if(!backtrack_stack.push(883)) return -1;
if(!backtrack_stack.push(882)) return -1;
if(!backtrack_stack.push(881)) return -1;
if(!backtrack_stack.push(880)) return -1;
if(!backtrack_stack.push(879)) return -1;
if(!backtrack_stack.push(878)) return -1;
if(!backtrack_stack.push(877)) return -1;
if(!backtrack_stack.push(876)) return -1;
if(!backtrack_stack.push(875)) return -1;
if(!backtrack_stack.push(874)) return -1;
if(!backtrack_stack.push(873)) return -1;
if(!backtrack_stack.push(872)) return -1;
if(!backtrack_stack.push(871)) return -1;
if(!backtrack_stack.push(870)) return -1;
if(!backtrack_stack.push(869)) return -1;
if(!backtrack_stack.push(868)) return -1;
if(!backtrack_stack.push(867)) return -1;
if(!backtrack_stack.push(866)) return -1;
if(!backtrack_stack.push(865)) return -1;
if(!backtrack_stack.push(864)) return -1;
if(!backtrack_stack.push(863)) return -1;
if(!backtrack_stack.push(862)) return -1;
if(!backtrack_stack.push(861)) return -1;
if(!backtrack_stack.push(846)) return -1;
if(!backtrack_stack.push(833)) return -1;
if(!backtrack_stack.push(832)) return -1;
if(!backtrack_stack.push(831)) return -1;
if(!backtrack_stack.push(830)) return -1;
if(!backtrack_stack.push(814)) return -1;
if(!backtrack_stack.push(799)) return -1;
if(!backtrack_stack.push(798)) return -1;
if(!backtrack_stack.push(797)) return -1;
if(!backtrack_stack.push(796)) return -1;
if(!backtrack_stack.push(795)) return -1;
if(!backtrack_stack.push(794)) return -1;
if(!backtrack_stack.push(793)) return -1;
if(!backtrack_stack.push(792)) return -1;
if(!backtrack_stack.push(791)) return -1;
if(!backtrack_stack.push(773)) return -1;
if(!backtrack_stack.push(757)) return -1;
if(!backtrack_stack.push(727)) return -1;
if(!backtrack_stack.push(695)) return -1;
if(!backtrack_stack.push(652)) return -1;
if(!backtrack_stack.push(637)) return -1;
if(!backtrack_stack.push(593)) return -1;
if(!backtrack_stack.push(590)) return -1;
if(!backtrack_stack.push(499)) return -1;
if(!backtrack_stack.push(443)) return -1;
if(!backtrack_stack.push(442)) return -1;
if(!backtrack_stack.push(412)) return -1;
if(!backtrack_stack.push(365)) return -1;
if(!backtrack_stack.push(346)) return -1;
if(!backtrack_stack.push(345)) return -1;
if(!backtrack_stack.push(318)) return -1;
if(!backtrack_stack.push(317)) return -1;
if(!backtrack_stack.push(301)) return -1;
if(!backtrack_stack.push(299)) return -1;
if(!backtrack_stack.push(298)) return -1;
if(!backtrack_stack.push(287)) return -1;
if(!backtrack_stack.push(285)) return -1;
if(!backtrack_stack.push(251)) return -1;
if(!backtrack_stack.push(250)) return -1;
if(!backtrack_stack.push(155)) return -1;
if(!backtrack_stack.push(86)) return -1;
if(!backtrack_stack.push(11)) return -1;
if(!backtrack_stack.push(1)) return -1;

// Loop Part
while(!backtrack_stack.is_empty()) {
	int test_label = backtrack_stack.pop();
	//cout << test_label << " [" << backtrack_stack.num_elems() << "]\n";
	switch(test_label) {
	case 1:
	// applying rule 
	// 7: kaon2hu(A) :- kaon2namedobjects(A).
	// resulting in symbolic fact kaon2hu(v_kaon2hu_1)
		c7_1_f.open();
	case 1014:
	// Continuation for label 1
		while (c7_1_f.fetch()) {
			v_kaon2hu_1 = c7_1_f.c1();
			kaon2hu_set.insert(v_kaon2hu_1);
			if(!kaon2hu_f.insert(v_kaon2hu_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1014)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(10)) return -1;
			if(!backtrack_stack.push(9)) return -1;
			if(!backtrack_stack.push(8)) return -1;
			if(!backtrack_stack.push(7)) return -1;
			if(!backtrack_stack.push(6)) return -1;
			goto label_2;
		}
		break;
	case 2: label_2: 
	// using  symbolic fact kaon2hu(v_kaon2hu_1) with literal 1 in rule 
	// 6: kaon2equal(A,A) :- kaon2hu(A).
	// resulting in symbolic fact kaon2equal(v_kaon2equal_1,v_kaon2equal_2)

		v_kaon2equal_1 = v_kaon2hu_1;
		v_kaon2equal_2 = v_kaon2hu_1;
		if(kaon2equal_set.insert(v_kaon2equal_1, v_kaon2equal_2)) {
		if(!kaon2equal_bf.insert(v_kaon2equal_1, v_kaon2equal_2)) return -1;
		if(!kaon2equal_fb.insert(v_kaon2equal_2, v_kaon2equal_1)) return -1;
		if(!kaon2equal_ff.insert(v_kaon2equal_1, v_kaon2equal_2)) return -1;
		// Labels for next rules
		if(!backtrack_stack.push(283)) return -1;
		if(!backtrack_stack.push(282)) return -1;
		if(!backtrack_stack.push(281)) return -1;
		if(!backtrack_stack.push(280)) return -1;
		if(!backtrack_stack.push(279)) return -1;
		if(!backtrack_stack.push(278)) return -1;
		if(!backtrack_stack.push(277)) return -1;
		if(!backtrack_stack.push(276)) return -1;
		if(!backtrack_stack.push(275)) return -1;
		if(!backtrack_stack.push(274)) return -1;
		if(!backtrack_stack.push(273)) return -1;
		if(!backtrack_stack.push(271)) return -1;
		if(!backtrack_stack.push(270)) return -1;
		if(!backtrack_stack.push(269)) return -1;
		if(!backtrack_stack.push(268)) return -1;
		if(!backtrack_stack.push(267)) return -1;
		if(!backtrack_stack.push(266)) return -1;
		if(!backtrack_stack.push(265)) return -1;
		if(!backtrack_stack.push(264)) return -1;
		if(!backtrack_stack.push(263)) return -1;
		if(!backtrack_stack.push(262)) return -1;
		if(!backtrack_stack.push(261)) return -1;
		if(!backtrack_stack.push(260)) return -1;
		if(!backtrack_stack.push(254)) return -1;
		if(!backtrack_stack.push(253)) return -1;
		if(!backtrack_stack.push(152)) return -1;
		if(!backtrack_stack.push(142)) return -1;
		if(!backtrack_stack.push(141)) return -1;
		if(!backtrack_stack.push(140)) return -1;
		if(!backtrack_stack.push(139)) return -1;
		if(!backtrack_stack.push(138)) return -1;
		if(!backtrack_stack.push(137)) return -1;
		if(!backtrack_stack.push(134)) return -1;
		if(!backtrack_stack.push(133)) return -1;
		if(!backtrack_stack.push(127)) return -1;
		if(!backtrack_stack.push(126)) return -1;
		if(!backtrack_stack.push(125)) return -1;
		if(!backtrack_stack.push(124)) return -1;
		if(!backtrack_stack.push(85)) return -1;
		if(!backtrack_stack.push(82)) return -1;
		if(!backtrack_stack.push(81)) return -1;
		if(!backtrack_stack.push(73)) return -1;
		if(!backtrack_stack.push(72)) return -1;
		if(!backtrack_stack.push(69)) return -1;
		if(!backtrack_stack.push(62)) return -1;
		if(!backtrack_stack.push(57)) return -1;
		if(!backtrack_stack.push(56)) return -1;
		if(!backtrack_stack.push(55)) return -1;
		if(!backtrack_stack.push(45)) return -1;
		if(!backtrack_stack.push(42)) return -1;
		if(!backtrack_stack.push(41)) return -1;
		if(!backtrack_stack.push(35)) return -1;
		if(!backtrack_stack.push(34)) return -1;
		if(!backtrack_stack.push(33)) return -1;
		if(!backtrack_stack.push(29)) return -1;
		if(!backtrack_stack.push(28)) return -1;
		if(!backtrack_stack.push(21)) return -1;
		if(!backtrack_stack.push(18)) return -1;
		if(!backtrack_stack.push(13)) return -1;
		if(!backtrack_stack.push(5)) return -1;
		if(!backtrack_stack.push(4)) return -1;
		goto label_3;
		}
		break;
	case 3: label_3: 
	// using  symbolic fact kaon2equal(v_kaon2equal_1,v_kaon2equal_2) with literal 1 in rule 
	// 96: kaon2equal(A,B) :- kaon2equal(A,C), kaon2equal(C,B), kaon2hu(A), kaon2hu(C), kaon2hu(B).
	// resulting in symbolic fact kaon2equal(v_kaon2equal_1,v_kaon2equal_2)

		//recursive
		c96_2_bf.push(&cursor_stack);
		if(!int_value_stack.push(v_kaon2equal_2)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2029)) return -1;
		c96_2_bf.open(v_kaon2equal_2);
		while (c96_2_bf.fetch()) {
			if (!kaon2hu_set.contains(v_kaon2equal_1)) continue;
				if (!kaon2hu_set.contains(v_kaon2equal_2)) continue;
					if (!kaon2hu_set.contains(c96_2_bf.c2())) continue;
	case 1016:
	// Continuation for label 3
						v_kaon2equal_2 = c96_2_bf.c2();
		if(kaon2equal_set.insert(v_kaon2equal_1, v_kaon2equal_2)) {
						if(!kaon2equal_bf.insert(v_kaon2equal_1, v_kaon2equal_2)) return -1;
						if(!kaon2equal_fb.insert(v_kaon2equal_2, v_kaon2equal_1)) return -1;
						if(!kaon2equal_ff.insert(v_kaon2equal_1, v_kaon2equal_2)) return -1;
						// Push backtrack label to continuation
						if(!backtrack_stack.push(1016)) return -1;
						// Labels for next rules
						if(!backtrack_stack.push(283)) return -1;
						if(!backtrack_stack.push(282)) return -1;
						if(!backtrack_stack.push(281)) return -1;
						if(!backtrack_stack.push(280)) return -1;
						if(!backtrack_stack.push(279)) return -1;
						if(!backtrack_stack.push(278)) return -1;
						if(!backtrack_stack.push(277)) return -1;
						if(!backtrack_stack.push(276)) return -1;
						if(!backtrack_stack.push(275)) return -1;
						if(!backtrack_stack.push(274)) return -1;
						if(!backtrack_stack.push(273)) return -1;
						if(!backtrack_stack.push(271)) return -1;
						if(!backtrack_stack.push(270)) return -1;
						if(!backtrack_stack.push(269)) return -1;
						if(!backtrack_stack.push(268)) return -1;
						if(!backtrack_stack.push(267)) return -1;
						if(!backtrack_stack.push(266)) return -1;
						if(!backtrack_stack.push(265)) return -1;
						if(!backtrack_stack.push(264)) return -1;
						if(!backtrack_stack.push(263)) return -1;
						if(!backtrack_stack.push(262)) return -1;
						if(!backtrack_stack.push(261)) return -1;
						if(!backtrack_stack.push(260)) return -1;
						if(!backtrack_stack.push(254)) return -1;
						if(!backtrack_stack.push(253)) return -1;
						if(!backtrack_stack.push(152)) return -1;
						if(!backtrack_stack.push(142)) return -1;
						if(!backtrack_stack.push(141)) return -1;
						if(!backtrack_stack.push(140)) return -1;
						if(!backtrack_stack.push(139)) return -1;
						if(!backtrack_stack.push(138)) return -1;
						if(!backtrack_stack.push(137)) return -1;
						if(!backtrack_stack.push(134)) return -1;
						if(!backtrack_stack.push(133)) return -1;
						if(!backtrack_stack.push(127)) return -1;
						if(!backtrack_stack.push(126)) return -1;
						if(!backtrack_stack.push(125)) return -1;
						if(!backtrack_stack.push(124)) return -1;
						if(!backtrack_stack.push(85)) return -1;
						if(!backtrack_stack.push(82)) return -1;
						if(!backtrack_stack.push(81)) return -1;
						if(!backtrack_stack.push(73)) return -1;
						if(!backtrack_stack.push(72)) return -1;
						if(!backtrack_stack.push(69)) return -1;
						if(!backtrack_stack.push(62)) return -1;
						if(!backtrack_stack.push(57)) return -1;
						if(!backtrack_stack.push(56)) return -1;
						if(!backtrack_stack.push(55)) return -1;
						if(!backtrack_stack.push(45)) return -1;
						if(!backtrack_stack.push(42)) return -1;
						if(!backtrack_stack.push(41)) return -1;
						if(!backtrack_stack.push(35)) return -1;
						if(!backtrack_stack.push(34)) return -1;
						if(!backtrack_stack.push(33)) return -1;
						if(!backtrack_stack.push(29)) return -1;
						if(!backtrack_stack.push(28)) return -1;
						if(!backtrack_stack.push(21)) return -1;
						if(!backtrack_stack.push(18)) return -1;
						if(!backtrack_stack.push(13)) return -1;
						if(!backtrack_stack.push(5)) return -1;
						if(!backtrack_stack.push(4)) return -1;
						goto label_3;
		}
		}
		break;
	case 2029:
	// Label to restore values of label 3
		v_kaon2equal_2 = int_value_stack.pop();
		c96_2_bf.pop(&cursor_stack);
		break;
	case 4: label_4: 
	// using  symbolic fact kaon2equal(v_kaon2equal_1,v_kaon2equal_2) with literal 3 in rule 
	// 95: kaon2equal(A,B) :- kaon2hu(A), kaon2hu(B), kaon2equal(B,A).
	// resulting in symbolic fact kaon2equal(v_kaon2equal_1,v_kaon2equal_2)

		//recursive
		if(!int_value_stack.push(v_kaon2equal_2)) return -1;
		if(!int_value_stack.push(v_kaon2equal_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2030)) return -1;
		if (!kaon2hu_set.contains(v_kaon2equal_2)) continue;
			if (!kaon2hu_set.contains(v_kaon2equal_1)) continue;
	case 1017:
	// Continuation for label 4
				v_kaon2equal_1 = v_kaon2equal_2;
				v_kaon2equal_2 = v_kaon2equal_1;
		if(kaon2equal_set.insert(v_kaon2equal_1, v_kaon2equal_2)) {
				if(!kaon2equal_bf.insert(v_kaon2equal_1, v_kaon2equal_2)) return -1;
				if(!kaon2equal_fb.insert(v_kaon2equal_2, v_kaon2equal_1)) return -1;
				if(!kaon2equal_ff.insert(v_kaon2equal_1, v_kaon2equal_2)) return -1;
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1017)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(283)) return -1;
				if(!backtrack_stack.push(282)) return -1;
				if(!backtrack_stack.push(281)) return -1;
				if(!backtrack_stack.push(280)) return -1;
				if(!backtrack_stack.push(279)) return -1;
				if(!backtrack_stack.push(278)) return -1;
				if(!backtrack_stack.push(277)) return -1;
				if(!backtrack_stack.push(276)) return -1;
				if(!backtrack_stack.push(275)) return -1;
				if(!backtrack_stack.push(274)) return -1;
				if(!backtrack_stack.push(273)) return -1;
				if(!backtrack_stack.push(271)) return -1;
				if(!backtrack_stack.push(270)) return -1;
				if(!backtrack_stack.push(269)) return -1;
				if(!backtrack_stack.push(268)) return -1;
				if(!backtrack_stack.push(267)) return -1;
				if(!backtrack_stack.push(266)) return -1;
				if(!backtrack_stack.push(265)) return -1;
				if(!backtrack_stack.push(264)) return -1;
				if(!backtrack_stack.push(263)) return -1;
				if(!backtrack_stack.push(262)) return -1;
				if(!backtrack_stack.push(261)) return -1;
				if(!backtrack_stack.push(260)) return -1;
				if(!backtrack_stack.push(254)) return -1;
				if(!backtrack_stack.push(253)) return -1;
				if(!backtrack_stack.push(152)) return -1;
				if(!backtrack_stack.push(142)) return -1;
				if(!backtrack_stack.push(141)) return -1;
				if(!backtrack_stack.push(140)) return -1;
				if(!backtrack_stack.push(139)) return -1;
				if(!backtrack_stack.push(138)) return -1;
				if(!backtrack_stack.push(137)) return -1;
				if(!backtrack_stack.push(134)) return -1;
				if(!backtrack_stack.push(133)) return -1;
				if(!backtrack_stack.push(127)) return -1;
				if(!backtrack_stack.push(126)) return -1;
				if(!backtrack_stack.push(125)) return -1;
				if(!backtrack_stack.push(124)) return -1;
				if(!backtrack_stack.push(85)) return -1;
				if(!backtrack_stack.push(82)) return -1;
				if(!backtrack_stack.push(81)) return -1;
				if(!backtrack_stack.push(73)) return -1;
				if(!backtrack_stack.push(72)) return -1;
				if(!backtrack_stack.push(69)) return -1;
				if(!backtrack_stack.push(62)) return -1;
				if(!backtrack_stack.push(57)) return -1;
				if(!backtrack_stack.push(56)) return -1;
				if(!backtrack_stack.push(55)) return -1;
				if(!backtrack_stack.push(45)) return -1;
				if(!backtrack_stack.push(42)) return -1;
				if(!backtrack_stack.push(41)) return -1;
				if(!backtrack_stack.push(35)) return -1;
				if(!backtrack_stack.push(34)) return -1;
				if(!backtrack_stack.push(33)) return -1;
				if(!backtrack_stack.push(29)) return -1;
				if(!backtrack_stack.push(28)) return -1;
				if(!backtrack_stack.push(21)) return -1;
				if(!backtrack_stack.push(18)) return -1;
				if(!backtrack_stack.push(13)) return -1;
				if(!backtrack_stack.push(5)) return -1;
				if(!backtrack_stack.push(4)) return -1;
				goto label_3;
		}
		break;
	case 2030:
	// Label to restore values of label 4
		v_kaon2equal_1 = int_value_stack.pop();
		v_kaon2equal_2 = int_value_stack.pop();
		break;
	case 5: label_5: 
	// using  symbolic fact kaon2equal(v_kaon2equal_1,v_kaon2equal_2) with literal 2 in rule 
	// 96: kaon2equal(A,B) :- kaon2equal(A,C), kaon2equal(C,B), kaon2hu(A), kaon2hu(C), kaon2hu(B).
	// resulting in symbolic fact kaon2equal(v_kaon2equal_1,v_kaon2equal_2)

		//recursive
		c96_1_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_kaon2equal_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2031)) return -1;
		c96_1_fb.open(v_kaon2equal_1);
		while (c96_1_fb.fetch()) {
			if (!kaon2hu_set.contains(c96_1_fb.c2())) continue;
				if (!kaon2hu_set.contains(v_kaon2equal_1)) continue;
					if (!kaon2hu_set.contains(v_kaon2equal_2)) continue;
	case 1018:
	// Continuation for label 5
						v_kaon2equal_1 = c96_1_fb.c2();
		if(kaon2equal_set.insert(v_kaon2equal_1, v_kaon2equal_2)) {
						if(!kaon2equal_bf.insert(v_kaon2equal_1, v_kaon2equal_2)) return -1;
						if(!kaon2equal_fb.insert(v_kaon2equal_2, v_kaon2equal_1)) return -1;
						if(!kaon2equal_ff.insert(v_kaon2equal_1, v_kaon2equal_2)) return -1;
						// Push backtrack label to continuation
						if(!backtrack_stack.push(1018)) return -1;
						// Labels for next rules
						if(!backtrack_stack.push(283)) return -1;
						if(!backtrack_stack.push(282)) return -1;
						if(!backtrack_stack.push(281)) return -1;
						if(!backtrack_stack.push(280)) return -1;
						if(!backtrack_stack.push(279)) return -1;
						if(!backtrack_stack.push(278)) return -1;
						if(!backtrack_stack.push(277)) return -1;
						if(!backtrack_stack.push(276)) return -1;
						if(!backtrack_stack.push(275)) return -1;
						if(!backtrack_stack.push(274)) return -1;
						if(!backtrack_stack.push(273)) return -1;
						if(!backtrack_stack.push(271)) return -1;
						if(!backtrack_stack.push(270)) return -1;
						if(!backtrack_stack.push(269)) return -1;
						if(!backtrack_stack.push(268)) return -1;
						if(!backtrack_stack.push(267)) return -1;
						if(!backtrack_stack.push(266)) return -1;
						if(!backtrack_stack.push(265)) return -1;
						if(!backtrack_stack.push(264)) return -1;
						if(!backtrack_stack.push(263)) return -1;
						if(!backtrack_stack.push(262)) return -1;
						if(!backtrack_stack.push(261)) return -1;
						if(!backtrack_stack.push(260)) return -1;
						if(!backtrack_stack.push(254)) return -1;
						if(!backtrack_stack.push(253)) return -1;
						if(!backtrack_stack.push(152)) return -1;
						if(!backtrack_stack.push(142)) return -1;
						if(!backtrack_stack.push(141)) return -1;
						if(!backtrack_stack.push(140)) return -1;
						if(!backtrack_stack.push(139)) return -1;
						if(!backtrack_stack.push(138)) return -1;
						if(!backtrack_stack.push(137)) return -1;
						if(!backtrack_stack.push(134)) return -1;
						if(!backtrack_stack.push(133)) return -1;
						if(!backtrack_stack.push(127)) return -1;
						if(!backtrack_stack.push(126)) return -1;
						if(!backtrack_stack.push(125)) return -1;
						if(!backtrack_stack.push(124)) return -1;
						if(!backtrack_stack.push(85)) return -1;
						if(!backtrack_stack.push(82)) return -1;
						if(!backtrack_stack.push(81)) return -1;
						if(!backtrack_stack.push(73)) return -1;
						if(!backtrack_stack.push(72)) return -1;
						if(!backtrack_stack.push(69)) return -1;
						if(!backtrack_stack.push(62)) return -1;
						if(!backtrack_stack.push(57)) return -1;
						if(!backtrack_stack.push(56)) return -1;
						if(!backtrack_stack.push(55)) return -1;
						if(!backtrack_stack.push(45)) return -1;
						if(!backtrack_stack.push(42)) return -1;
						if(!backtrack_stack.push(41)) return -1;
						if(!backtrack_stack.push(35)) return -1;
						if(!backtrack_stack.push(34)) return -1;
						if(!backtrack_stack.push(33)) return -1;
						if(!backtrack_stack.push(29)) return -1;
						if(!backtrack_stack.push(28)) return -1;
						if(!backtrack_stack.push(21)) return -1;
						if(!backtrack_stack.push(18)) return -1;
						if(!backtrack_stack.push(13)) return -1;
						if(!backtrack_stack.push(5)) return -1;
						if(!backtrack_stack.push(4)) return -1;
						goto label_3;
		}
		}
		break;
	case 2031:
	// Label to restore values of label 5
		v_kaon2equal_1 = int_value_stack.pop();
		c96_1_fb.pop(&cursor_stack);
		break;
	case 6: label_6: 
	// using  symbolic fact kaon2hu(v_kaon2hu_1) with literal 1 in rule 
	// 95: kaon2equal(A,B) :- kaon2hu(A), kaon2hu(B), kaon2equal(B,A).
	// resulting in symbolic fact kaon2equal(v_kaon2equal_1,v_kaon2equal_2)

		c95_2_f.open();
		while (c95_2_f.fetch()) {
			if (!kaon2equal_set.contains(c95_2_f.c1(), v_kaon2hu_1)) continue;
	case 1019:
	// Continuation for label 6
				v_kaon2equal_1 = v_kaon2hu_1;
				v_kaon2equal_2 = c95_2_f.c1();
		if(kaon2equal_set.insert(v_kaon2equal_1, v_kaon2equal_2)) {
				if(!kaon2equal_bf.insert(v_kaon2equal_1, v_kaon2equal_2)) return -1;
				if(!kaon2equal_fb.insert(v_kaon2equal_2, v_kaon2equal_1)) return -1;
				if(!kaon2equal_ff.insert(v_kaon2equal_1, v_kaon2equal_2)) return -1;
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1019)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(283)) return -1;
				if(!backtrack_stack.push(282)) return -1;
				if(!backtrack_stack.push(281)) return -1;
				if(!backtrack_stack.push(280)) return -1;
				if(!backtrack_stack.push(279)) return -1;
				if(!backtrack_stack.push(278)) return -1;
				if(!backtrack_stack.push(277)) return -1;
				if(!backtrack_stack.push(276)) return -1;
				if(!backtrack_stack.push(275)) return -1;
				if(!backtrack_stack.push(274)) return -1;
				if(!backtrack_stack.push(273)) return -1;
				if(!backtrack_stack.push(271)) return -1;
				if(!backtrack_stack.push(270)) return -1;
				if(!backtrack_stack.push(269)) return -1;
				if(!backtrack_stack.push(268)) return -1;
				if(!backtrack_stack.push(267)) return -1;
				if(!backtrack_stack.push(266)) return -1;
				if(!backtrack_stack.push(265)) return -1;
				if(!backtrack_stack.push(264)) return -1;
				if(!backtrack_stack.push(263)) return -1;
				if(!backtrack_stack.push(262)) return -1;
				if(!backtrack_stack.push(261)) return -1;
				if(!backtrack_stack.push(260)) return -1;
				if(!backtrack_stack.push(254)) return -1;
				if(!backtrack_stack.push(253)) return -1;
				if(!backtrack_stack.push(152)) return -1;
				if(!backtrack_stack.push(142)) return -1;
				if(!backtrack_stack.push(141)) return -1;
				if(!backtrack_stack.push(140)) return -1;
				if(!backtrack_stack.push(139)) return -1;
				if(!backtrack_stack.push(138)) return -1;
				if(!backtrack_stack.push(137)) return -1;
				if(!backtrack_stack.push(134)) return -1;
				if(!backtrack_stack.push(133)) return -1;
				if(!backtrack_stack.push(127)) return -1;
				if(!backtrack_stack.push(126)) return -1;
				if(!backtrack_stack.push(125)) return -1;
				if(!backtrack_stack.push(124)) return -1;
				if(!backtrack_stack.push(85)) return -1;
				if(!backtrack_stack.push(82)) return -1;
				if(!backtrack_stack.push(81)) return -1;
				if(!backtrack_stack.push(73)) return -1;
				if(!backtrack_stack.push(72)) return -1;
				if(!backtrack_stack.push(69)) return -1;
				if(!backtrack_stack.push(62)) return -1;
				if(!backtrack_stack.push(57)) return -1;
				if(!backtrack_stack.push(56)) return -1;
				if(!backtrack_stack.push(55)) return -1;
				if(!backtrack_stack.push(45)) return -1;
				if(!backtrack_stack.push(42)) return -1;
				if(!backtrack_stack.push(41)) return -1;
				if(!backtrack_stack.push(35)) return -1;
				if(!backtrack_stack.push(34)) return -1;
				if(!backtrack_stack.push(33)) return -1;
				if(!backtrack_stack.push(29)) return -1;
				if(!backtrack_stack.push(28)) return -1;
				if(!backtrack_stack.push(21)) return -1;
				if(!backtrack_stack.push(18)) return -1;
				if(!backtrack_stack.push(13)) return -1;
				if(!backtrack_stack.push(5)) return -1;
				if(!backtrack_stack.push(4)) return -1;
				goto label_3;
		}
		}
		break;
	case 7: label_7: 
	// using  symbolic fact kaon2hu(v_kaon2hu_1) with literal 2 in rule 
	// 95: kaon2equal(A,B) :- kaon2hu(A), kaon2hu(B), kaon2equal(B,A).
	// resulting in symbolic fact kaon2equal(v_kaon2equal_1,v_kaon2equal_2)

		c95_1_f.open();
		while (c95_1_f.fetch()) {
			if (!kaon2equal_set.contains(v_kaon2hu_1, c95_1_f.c1())) continue;
	case 1020:
	// Continuation for label 7
				v_kaon2equal_2 = v_kaon2hu_1;
				v_kaon2equal_1 = c95_1_f.c1();
		if(kaon2equal_set.insert(v_kaon2equal_1, v_kaon2equal_2)) {
				if(!kaon2equal_bf.insert(v_kaon2equal_1, v_kaon2equal_2)) return -1;
				if(!kaon2equal_fb.insert(v_kaon2equal_2, v_kaon2equal_1)) return -1;
				if(!kaon2equal_ff.insert(v_kaon2equal_1, v_kaon2equal_2)) return -1;
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1020)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(283)) return -1;
				if(!backtrack_stack.push(282)) return -1;
				if(!backtrack_stack.push(281)) return -1;
				if(!backtrack_stack.push(280)) return -1;
				if(!backtrack_stack.push(279)) return -1;
				if(!backtrack_stack.push(278)) return -1;
				if(!backtrack_stack.push(277)) return -1;
				if(!backtrack_stack.push(276)) return -1;
				if(!backtrack_stack.push(275)) return -1;
				if(!backtrack_stack.push(274)) return -1;
				if(!backtrack_stack.push(273)) return -1;
				if(!backtrack_stack.push(271)) return -1;
				if(!backtrack_stack.push(270)) return -1;
				if(!backtrack_stack.push(269)) return -1;
				if(!backtrack_stack.push(268)) return -1;
				if(!backtrack_stack.push(267)) return -1;
				if(!backtrack_stack.push(266)) return -1;
				if(!backtrack_stack.push(265)) return -1;
				if(!backtrack_stack.push(264)) return -1;
				if(!backtrack_stack.push(263)) return -1;
				if(!backtrack_stack.push(262)) return -1;
				if(!backtrack_stack.push(261)) return -1;
				if(!backtrack_stack.push(260)) return -1;
				if(!backtrack_stack.push(254)) return -1;
				if(!backtrack_stack.push(253)) return -1;
				if(!backtrack_stack.push(152)) return -1;
				if(!backtrack_stack.push(142)) return -1;
				if(!backtrack_stack.push(141)) return -1;
				if(!backtrack_stack.push(140)) return -1;
				if(!backtrack_stack.push(139)) return -1;
				if(!backtrack_stack.push(138)) return -1;
				if(!backtrack_stack.push(137)) return -1;
				if(!backtrack_stack.push(134)) return -1;
				if(!backtrack_stack.push(133)) return -1;
				if(!backtrack_stack.push(127)) return -1;
				if(!backtrack_stack.push(126)) return -1;
				if(!backtrack_stack.push(125)) return -1;
				if(!backtrack_stack.push(124)) return -1;
				if(!backtrack_stack.push(85)) return -1;
				if(!backtrack_stack.push(82)) return -1;
				if(!backtrack_stack.push(81)) return -1;
				if(!backtrack_stack.push(73)) return -1;
				if(!backtrack_stack.push(72)) return -1;
				if(!backtrack_stack.push(69)) return -1;
				if(!backtrack_stack.push(62)) return -1;
				if(!backtrack_stack.push(57)) return -1;
				if(!backtrack_stack.push(56)) return -1;
				if(!backtrack_stack.push(55)) return -1;
				if(!backtrack_stack.push(45)) return -1;
				if(!backtrack_stack.push(42)) return -1;
				if(!backtrack_stack.push(41)) return -1;
				if(!backtrack_stack.push(35)) return -1;
				if(!backtrack_stack.push(34)) return -1;
				if(!backtrack_stack.push(33)) return -1;
				if(!backtrack_stack.push(29)) return -1;
				if(!backtrack_stack.push(28)) return -1;
				if(!backtrack_stack.push(21)) return -1;
				if(!backtrack_stack.push(18)) return -1;
				if(!backtrack_stack.push(13)) return -1;
				if(!backtrack_stack.push(5)) return -1;
				if(!backtrack_stack.push(4)) return -1;
				goto label_3;
		}
		}
		break;
	case 8: label_8: 
	// using  symbolic fact kaon2hu(v_kaon2hu_1) with literal 3 in rule 
	// 96: kaon2equal(A,B) :- kaon2equal(A,C), kaon2equal(C,B), kaon2hu(A), kaon2hu(C), kaon2hu(B).
	// resulting in symbolic fact kaon2equal(v_kaon2equal_1,v_kaon2equal_2)

		c96_1_bf.open(v_kaon2hu_1);
		while (c96_1_bf.fetch()) {
			c96_2_bf.open(c96_1_bf.c2());
			while (c96_2_bf.fetch()) {
				if (!kaon2hu_set.contains(c96_1_bf.c2())) continue;
					if (!kaon2hu_set.contains(c96_2_bf.c2())) continue;
	case 1021:
	// Continuation for label 8
						v_kaon2equal_1 = v_kaon2hu_1;
						v_kaon2equal_2 = c96_2_bf.c2();
		if(kaon2equal_set.insert(v_kaon2equal_1, v_kaon2equal_2)) {
						if(!kaon2equal_bf.insert(v_kaon2equal_1, v_kaon2equal_2)) return -1;
						if(!kaon2equal_fb.insert(v_kaon2equal_2, v_kaon2equal_1)) return -1;
						if(!kaon2equal_ff.insert(v_kaon2equal_1, v_kaon2equal_2)) return -1;
						// Push backtrack label to continuation
						if(!backtrack_stack.push(1021)) return -1;
						// Labels for next rules
						if(!backtrack_stack.push(283)) return -1;
						if(!backtrack_stack.push(282)) return -1;
						if(!backtrack_stack.push(281)) return -1;
						if(!backtrack_stack.push(280)) return -1;
						if(!backtrack_stack.push(279)) return -1;
						if(!backtrack_stack.push(278)) return -1;
						if(!backtrack_stack.push(277)) return -1;
						if(!backtrack_stack.push(276)) return -1;
						if(!backtrack_stack.push(275)) return -1;
						if(!backtrack_stack.push(274)) return -1;
						if(!backtrack_stack.push(273)) return -1;
						if(!backtrack_stack.push(271)) return -1;
						if(!backtrack_stack.push(270)) return -1;
						if(!backtrack_stack.push(269)) return -1;
						if(!backtrack_stack.push(268)) return -1;
						if(!backtrack_stack.push(267)) return -1;
						if(!backtrack_stack.push(266)) return -1;
						if(!backtrack_stack.push(265)) return -1;
						if(!backtrack_stack.push(264)) return -1;
						if(!backtrack_stack.push(263)) return -1;
						if(!backtrack_stack.push(262)) return -1;
						if(!backtrack_stack.push(261)) return -1;
						if(!backtrack_stack.push(260)) return -1;
						if(!backtrack_stack.push(254)) return -1;
						if(!backtrack_stack.push(253)) return -1;
						if(!backtrack_stack.push(152)) return -1;
						if(!backtrack_stack.push(142)) return -1;
						if(!backtrack_stack.push(141)) return -1;
						if(!backtrack_stack.push(140)) return -1;
						if(!backtrack_stack.push(139)) return -1;
						if(!backtrack_stack.push(138)) return -1;
						if(!backtrack_stack.push(137)) return -1;
						if(!backtrack_stack.push(134)) return -1;
						if(!backtrack_stack.push(133)) return -1;
						if(!backtrack_stack.push(127)) return -1;
						if(!backtrack_stack.push(126)) return -1;
						if(!backtrack_stack.push(125)) return -1;
						if(!backtrack_stack.push(124)) return -1;
						if(!backtrack_stack.push(85)) return -1;
						if(!backtrack_stack.push(82)) return -1;
						if(!backtrack_stack.push(81)) return -1;
						if(!backtrack_stack.push(73)) return -1;
						if(!backtrack_stack.push(72)) return -1;
						if(!backtrack_stack.push(69)) return -1;
						if(!backtrack_stack.push(62)) return -1;
						if(!backtrack_stack.push(57)) return -1;
						if(!backtrack_stack.push(56)) return -1;
						if(!backtrack_stack.push(55)) return -1;
						if(!backtrack_stack.push(45)) return -1;
						if(!backtrack_stack.push(42)) return -1;
						if(!backtrack_stack.push(41)) return -1;
						if(!backtrack_stack.push(35)) return -1;
						if(!backtrack_stack.push(34)) return -1;
						if(!backtrack_stack.push(33)) return -1;
						if(!backtrack_stack.push(29)) return -1;
						if(!backtrack_stack.push(28)) return -1;
						if(!backtrack_stack.push(21)) return -1;
						if(!backtrack_stack.push(18)) return -1;
						if(!backtrack_stack.push(13)) return -1;
						if(!backtrack_stack.push(5)) return -1;
						if(!backtrack_stack.push(4)) return -1;
						goto label_3;
		}
			}
		}
		break;
	case 9: label_9: 
	// using  symbolic fact kaon2hu(v_kaon2hu_1) with literal 4 in rule 
	// 96: kaon2equal(A,B) :- kaon2equal(A,C), kaon2equal(C,B), kaon2hu(A), kaon2hu(C), kaon2hu(B).
	// resulting in symbolic fact kaon2equal(v_kaon2equal_1,v_kaon2equal_2)

		c96_1_fb.open(v_kaon2hu_1);
		while (c96_1_fb.fetch()) {
			c96_2_bf.open(v_kaon2hu_1);
			while (c96_2_bf.fetch()) {
				if (!kaon2hu_set.contains(c96_1_fb.c2())) continue;
					if (!kaon2hu_set.contains(c96_2_bf.c2())) continue;
	case 1022:
	// Continuation for label 9
						v_kaon2equal_2 = c96_2_bf.c2();
						v_kaon2equal_1 = c96_1_fb.c2();
		if(kaon2equal_set.insert(v_kaon2equal_1, v_kaon2equal_2)) {
						if(!kaon2equal_bf.insert(v_kaon2equal_1, v_kaon2equal_2)) return -1;
						if(!kaon2equal_fb.insert(v_kaon2equal_2, v_kaon2equal_1)) return -1;
						if(!kaon2equal_ff.insert(v_kaon2equal_1, v_kaon2equal_2)) return -1;
						// Push backtrack label to continuation
						if(!backtrack_stack.push(1022)) return -1;
						// Labels for next rules
						if(!backtrack_stack.push(283)) return -1;
						if(!backtrack_stack.push(282)) return -1;
						if(!backtrack_stack.push(281)) return -1;
						if(!backtrack_stack.push(280)) return -1;
						if(!backtrack_stack.push(279)) return -1;
						if(!backtrack_stack.push(278)) return -1;
						if(!backtrack_stack.push(277)) return -1;
						if(!backtrack_stack.push(276)) return -1;
						if(!backtrack_stack.push(275)) return -1;
						if(!backtrack_stack.push(274)) return -1;
						if(!backtrack_stack.push(273)) return -1;
						if(!backtrack_stack.push(271)) return -1;
						if(!backtrack_stack.push(270)) return -1;
						if(!backtrack_stack.push(269)) return -1;
						if(!backtrack_stack.push(268)) return -1;
						if(!backtrack_stack.push(267)) return -1;
						if(!backtrack_stack.push(266)) return -1;
						if(!backtrack_stack.push(265)) return -1;
						if(!backtrack_stack.push(264)) return -1;
						if(!backtrack_stack.push(263)) return -1;
						if(!backtrack_stack.push(262)) return -1;
						if(!backtrack_stack.push(261)) return -1;
						if(!backtrack_stack.push(260)) return -1;
						if(!backtrack_stack.push(254)) return -1;
						if(!backtrack_stack.push(253)) return -1;
						if(!backtrack_stack.push(152)) return -1;
						if(!backtrack_stack.push(142)) return -1;
						if(!backtrack_stack.push(141)) return -1;
						if(!backtrack_stack.push(140)) return -1;
						if(!backtrack_stack.push(139)) return -1;
						if(!backtrack_stack.push(138)) return -1;
						if(!backtrack_stack.push(137)) return -1;
						if(!backtrack_stack.push(134)) return -1;
						if(!backtrack_stack.push(133)) return -1;
						if(!backtrack_stack.push(127)) return -1;
						if(!backtrack_stack.push(126)) return -1;
						if(!backtrack_stack.push(125)) return -1;
						if(!backtrack_stack.push(124)) return -1;
						if(!backtrack_stack.push(85)) return -1;
						if(!backtrack_stack.push(82)) return -1;
						if(!backtrack_stack.push(81)) return -1;
						if(!backtrack_stack.push(73)) return -1;
						if(!backtrack_stack.push(72)) return -1;
						if(!backtrack_stack.push(69)) return -1;
						if(!backtrack_stack.push(62)) return -1;
						if(!backtrack_stack.push(57)) return -1;
						if(!backtrack_stack.push(56)) return -1;
						if(!backtrack_stack.push(55)) return -1;
						if(!backtrack_stack.push(45)) return -1;
						if(!backtrack_stack.push(42)) return -1;
						if(!backtrack_stack.push(41)) return -1;
						if(!backtrack_stack.push(35)) return -1;
						if(!backtrack_stack.push(34)) return -1;
						if(!backtrack_stack.push(33)) return -1;
						if(!backtrack_stack.push(29)) return -1;
						if(!backtrack_stack.push(28)) return -1;
						if(!backtrack_stack.push(21)) return -1;
						if(!backtrack_stack.push(18)) return -1;
						if(!backtrack_stack.push(13)) return -1;
						if(!backtrack_stack.push(5)) return -1;
						if(!backtrack_stack.push(4)) return -1;
						goto label_3;
		}
			}
		}
		break;
	case 10: label_10: 
	// using  symbolic fact kaon2hu(v_kaon2hu_1) with literal 5 in rule 
	// 96: kaon2equal(A,B) :- kaon2equal(A,C), kaon2equal(C,B), kaon2hu(A), kaon2hu(C), kaon2hu(B).
	// resulting in symbolic fact kaon2equal(v_kaon2equal_1,v_kaon2equal_2)

		c96_1_ff.open();
		while (c96_1_ff.fetch()) {
			if (!kaon2equal_set.contains(c96_1_ff.c2(), v_kaon2hu_1)) continue;
				if (!kaon2hu_set.contains(c96_1_ff.c1())) continue;
					if (!kaon2hu_set.contains(c96_1_ff.c2())) continue;
	case 1023:
	// Continuation for label 10
						v_kaon2equal_2 = v_kaon2hu_1;
						v_kaon2equal_1 = c96_1_ff.c1();
		if(kaon2equal_set.insert(v_kaon2equal_1, v_kaon2equal_2)) {
						if(!kaon2equal_bf.insert(v_kaon2equal_1, v_kaon2equal_2)) return -1;
						if(!kaon2equal_fb.insert(v_kaon2equal_2, v_kaon2equal_1)) return -1;
						if(!kaon2equal_ff.insert(v_kaon2equal_1, v_kaon2equal_2)) return -1;
						// Push backtrack label to continuation
						if(!backtrack_stack.push(1023)) return -1;
						// Labels for next rules
						if(!backtrack_stack.push(283)) return -1;
						if(!backtrack_stack.push(282)) return -1;
						if(!backtrack_stack.push(281)) return -1;
						if(!backtrack_stack.push(280)) return -1;
						if(!backtrack_stack.push(279)) return -1;
						if(!backtrack_stack.push(278)) return -1;
						if(!backtrack_stack.push(277)) return -1;
						if(!backtrack_stack.push(276)) return -1;
						if(!backtrack_stack.push(275)) return -1;
						if(!backtrack_stack.push(274)) return -1;
						if(!backtrack_stack.push(273)) return -1;
						if(!backtrack_stack.push(271)) return -1;
						if(!backtrack_stack.push(270)) return -1;
						if(!backtrack_stack.push(269)) return -1;
						if(!backtrack_stack.push(268)) return -1;
						if(!backtrack_stack.push(267)) return -1;
						if(!backtrack_stack.push(266)) return -1;
						if(!backtrack_stack.push(265)) return -1;
						if(!backtrack_stack.push(264)) return -1;
						if(!backtrack_stack.push(263)) return -1;
						if(!backtrack_stack.push(262)) return -1;
						if(!backtrack_stack.push(261)) return -1;
						if(!backtrack_stack.push(260)) return -1;
						if(!backtrack_stack.push(254)) return -1;
						if(!backtrack_stack.push(253)) return -1;
						if(!backtrack_stack.push(152)) return -1;
						if(!backtrack_stack.push(142)) return -1;
						if(!backtrack_stack.push(141)) return -1;
						if(!backtrack_stack.push(140)) return -1;
						if(!backtrack_stack.push(139)) return -1;
						if(!backtrack_stack.push(138)) return -1;
						if(!backtrack_stack.push(137)) return -1;
						if(!backtrack_stack.push(134)) return -1;
						if(!backtrack_stack.push(133)) return -1;
						if(!backtrack_stack.push(127)) return -1;
						if(!backtrack_stack.push(126)) return -1;
						if(!backtrack_stack.push(125)) return -1;
						if(!backtrack_stack.push(124)) return -1;
						if(!backtrack_stack.push(85)) return -1;
						if(!backtrack_stack.push(82)) return -1;
						if(!backtrack_stack.push(81)) return -1;
						if(!backtrack_stack.push(73)) return -1;
						if(!backtrack_stack.push(72)) return -1;
						if(!backtrack_stack.push(69)) return -1;
						if(!backtrack_stack.push(62)) return -1;
						if(!backtrack_stack.push(57)) return -1;
						if(!backtrack_stack.push(56)) return -1;
						if(!backtrack_stack.push(55)) return -1;
						if(!backtrack_stack.push(45)) return -1;
						if(!backtrack_stack.push(42)) return -1;
						if(!backtrack_stack.push(41)) return -1;
						if(!backtrack_stack.push(35)) return -1;
						if(!backtrack_stack.push(34)) return -1;
						if(!backtrack_stack.push(33)) return -1;
						if(!backtrack_stack.push(29)) return -1;
						if(!backtrack_stack.push(28)) return -1;
						if(!backtrack_stack.push(21)) return -1;
						if(!backtrack_stack.push(18)) return -1;
						if(!backtrack_stack.push(13)) return -1;
						if(!backtrack_stack.push(5)) return -1;
						if(!backtrack_stack.push(4)) return -1;
						goto label_3;
		}
		}
		break;
	case 11:
	// applying rule 
	// 9: hasflavor(A,B) :- hasflavor_aux(A,B).
	// resulting in symbolic fact hasflavor(v_hasflavor_1,v_hasflavor_2)
		c9_1_ff.open();
	case 1024:
	// Continuation for label 11
		while (c9_1_ff.fetch()) {
			v_hasflavor_2 = c9_1_ff.c2();
			v_hasflavor_1 = c9_1_ff.c1();
		if(hasflavor_set.insert(v_hasflavor_1, v_hasflavor_2)) {
			if(!hasflavor_bf.insert(v_hasflavor_1, v_hasflavor_2)) return -1;
			if(!hasflavor_fb.insert(v_hasflavor_2, v_hasflavor_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1024)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(32)) return -1;
			if(!backtrack_stack.push(31)) return -1;
			if(!backtrack_stack.push(30)) return -1;
			if(!backtrack_stack.push(14)) return -1;
			goto label_12;
		}
		}
		break;
	case 12: label_12: 
	// using  symbolic fact hasflavor(v_hasflavor_1,v_hasflavor_2) with literal 1 in rule 
	// 8: kaon2equal(A,B) :- hasflavor(C,A), hasflavor(C,B).
	// resulting in symbolic fact kaon2equal(v_kaon2equal_1,v_kaon2equal_2)

		//recursive
		c8_2_bf.push(&cursor_stack);
		if(!int_value_stack.push(v_kaon2equal_2)) return -1;
		if(!int_value_stack.push(v_kaon2equal_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2038)) return -1;
		c8_2_bf.open(v_hasflavor_1);
	case 1025:
	// Continuation for label 12
		while (c8_2_bf.fetch()) {
			v_kaon2equal_1 = v_hasflavor_2;
			v_kaon2equal_2 = c8_2_bf.c2();
		if(kaon2equal_set.insert(v_kaon2equal_1, v_kaon2equal_2)) {
			if(!kaon2equal_bf.insert(v_kaon2equal_1, v_kaon2equal_2)) return -1;
			if(!kaon2equal_fb.insert(v_kaon2equal_2, v_kaon2equal_1)) return -1;
			if(!kaon2equal_ff.insert(v_kaon2equal_1, v_kaon2equal_2)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1025)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(283)) return -1;
			if(!backtrack_stack.push(282)) return -1;
			if(!backtrack_stack.push(281)) return -1;
			if(!backtrack_stack.push(280)) return -1;
			if(!backtrack_stack.push(279)) return -1;
			if(!backtrack_stack.push(278)) return -1;
			if(!backtrack_stack.push(277)) return -1;
			if(!backtrack_stack.push(276)) return -1;
			if(!backtrack_stack.push(275)) return -1;
			if(!backtrack_stack.push(274)) return -1;
			if(!backtrack_stack.push(273)) return -1;
			if(!backtrack_stack.push(271)) return -1;
			if(!backtrack_stack.push(270)) return -1;
			if(!backtrack_stack.push(269)) return -1;
			if(!backtrack_stack.push(268)) return -1;
			if(!backtrack_stack.push(267)) return -1;
			if(!backtrack_stack.push(266)) return -1;
			if(!backtrack_stack.push(265)) return -1;
			if(!backtrack_stack.push(264)) return -1;
			if(!backtrack_stack.push(263)) return -1;
			if(!backtrack_stack.push(262)) return -1;
			if(!backtrack_stack.push(261)) return -1;
			if(!backtrack_stack.push(260)) return -1;
			if(!backtrack_stack.push(254)) return -1;
			if(!backtrack_stack.push(253)) return -1;
			if(!backtrack_stack.push(152)) return -1;
			if(!backtrack_stack.push(142)) return -1;
			if(!backtrack_stack.push(141)) return -1;
			if(!backtrack_stack.push(140)) return -1;
			if(!backtrack_stack.push(139)) return -1;
			if(!backtrack_stack.push(138)) return -1;
			if(!backtrack_stack.push(137)) return -1;
			if(!backtrack_stack.push(134)) return -1;
			if(!backtrack_stack.push(133)) return -1;
			if(!backtrack_stack.push(127)) return -1;
			if(!backtrack_stack.push(126)) return -1;
			if(!backtrack_stack.push(125)) return -1;
			if(!backtrack_stack.push(124)) return -1;
			if(!backtrack_stack.push(85)) return -1;
			if(!backtrack_stack.push(82)) return -1;
			if(!backtrack_stack.push(81)) return -1;
			if(!backtrack_stack.push(73)) return -1;
			if(!backtrack_stack.push(72)) return -1;
			if(!backtrack_stack.push(69)) return -1;
			if(!backtrack_stack.push(62)) return -1;
			if(!backtrack_stack.push(57)) return -1;
			if(!backtrack_stack.push(56)) return -1;
			if(!backtrack_stack.push(55)) return -1;
			if(!backtrack_stack.push(45)) return -1;
			if(!backtrack_stack.push(42)) return -1;
			if(!backtrack_stack.push(41)) return -1;
			if(!backtrack_stack.push(35)) return -1;
			if(!backtrack_stack.push(34)) return -1;
			if(!backtrack_stack.push(33)) return -1;
			if(!backtrack_stack.push(29)) return -1;
			if(!backtrack_stack.push(28)) return -1;
			if(!backtrack_stack.push(21)) return -1;
			if(!backtrack_stack.push(18)) return -1;
			if(!backtrack_stack.push(13)) return -1;
			if(!backtrack_stack.push(5)) return -1;
			if(!backtrack_stack.push(4)) return -1;
			goto label_3;
		}
		}
		break;
	case 2038:
	// Label to restore values of label 12
		v_kaon2equal_1 = int_value_stack.pop();
		v_kaon2equal_2 = int_value_stack.pop();
		c8_2_bf.pop(&cursor_stack);
		break;
	case 13: label_13: 
	// using  symbolic fact kaon2equal(v_kaon2equal_1,v_kaon2equal_2) with literal 2 in rule 
	// 648: hasflavor(A,B) :- hasflavor(C,B), kaon2equal(A,C).
	// resulting in symbolic fact hasflavor(v_hasflavor_1,v_hasflavor_2)

		//recursive
		c648_1_bf.push(&cursor_stack);
		if(!int_value_stack.push(v_hasflavor_2)) return -1;
		if(!int_value_stack.push(v_hasflavor_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2039)) return -1;
		c648_1_bf.open(v_kaon2equal_2);
	case 1026:
	// Continuation for label 13
		while (c648_1_bf.fetch()) {
			v_hasflavor_1 = v_kaon2equal_1;
			v_hasflavor_2 = c648_1_bf.c2();
		if(hasflavor_set.insert(v_hasflavor_1, v_hasflavor_2)) {
			if(!hasflavor_bf.insert(v_hasflavor_1, v_hasflavor_2)) return -1;
			if(!hasflavor_fb.insert(v_hasflavor_2, v_hasflavor_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1026)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(32)) return -1;
			if(!backtrack_stack.push(31)) return -1;
			if(!backtrack_stack.push(30)) return -1;
			if(!backtrack_stack.push(14)) return -1;
			goto label_12;
		}
		}
		break;
	case 2039:
	// Label to restore values of label 13
		v_hasflavor_1 = int_value_stack.pop();
		v_hasflavor_2 = int_value_stack.pop();
		c648_1_bf.pop(&cursor_stack);
		break;
	case 14: label_14: 
	// using  symbolic fact hasflavor(v_hasflavor_1,v_hasflavor_2) with literal 1 in rule 
	// 642: haswinedescriptor(A,B) :- hasflavor(A,B).
	// resulting in symbolic fact haswinedescriptor(v_haswinedescriptor_1,v_haswinedescriptor_2)

		//recursive
		if(!int_value_stack.push(v_haswinedescriptor_2)) return -1;
		if(!int_value_stack.push(v_haswinedescriptor_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2040)) return -1;
		v_haswinedescriptor_1 = v_hasflavor_1;
		v_haswinedescriptor_2 = v_hasflavor_2;
		if(haswinedescriptor_set.insert(v_haswinedescriptor_1, v_haswinedescriptor_2)) {
		if(!haswinedescriptor_bf.insert(v_haswinedescriptor_1, v_haswinedescriptor_2)) return -1;
		if(!haswinedescriptor_fb.insert(v_haswinedescriptor_2, v_haswinedescriptor_1)) return -1;
		// Labels for next rules
		if(!backtrack_stack.push(26)) return -1;
		if(!backtrack_stack.push(25)) return -1;
		goto label_15;
		}
		break;
	case 2040:
	// Label to restore values of label 14
		v_haswinedescriptor_1 = int_value_stack.pop();
		v_haswinedescriptor_2 = int_value_stack.pop();
		break;
	case 15: label_15: 
	// using  symbolic fact haswinedescriptor(v_haswinedescriptor_1,v_haswinedescriptor_2) with literal 1 in rule 
	// 266: wine(A) :- haswinedescriptor(A,B).
	// resulting in symbolic fact wine(v_wine_1)

		//recursive
		if(!int_value_stack.push(v_wine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2041)) return -1;
		v_wine_1 = v_haswinedescriptor_1;
		if(wine_set.insert(v_wine_1)) {
		if(!wine_f.insert(v_wine_1)) return -1;
		// Labels for next rules
		if(!backtrack_stack.push(1007)) return -1;
		if(!backtrack_stack.push(989)) return -1;
		if(!backtrack_stack.push(954)) return -1;
		if(!backtrack_stack.push(938)) return -1;
		if(!backtrack_stack.push(911)) return -1;
		if(!backtrack_stack.push(903)) return -1;
		if(!backtrack_stack.push(902)) return -1;
		if(!backtrack_stack.push(901)) return -1;
		if(!backtrack_stack.push(849)) return -1;
		if(!backtrack_stack.push(848)) return -1;
		if(!backtrack_stack.push(821)) return -1;
		if(!backtrack_stack.push(820)) return -1;
		if(!backtrack_stack.push(737)) return -1;
		if(!backtrack_stack.push(712)) return -1;
		if(!backtrack_stack.push(674)) return -1;
		if(!backtrack_stack.push(672)) return -1;
		if(!backtrack_stack.push(583)) return -1;
		if(!backtrack_stack.push(575)) return -1;
		if(!backtrack_stack.push(563)) return -1;
		if(!backtrack_stack.push(551)) return -1;
		if(!backtrack_stack.push(549)) return -1;
		if(!backtrack_stack.push(483)) return -1;
		if(!backtrack_stack.push(473)) return -1;
		if(!backtrack_stack.push(470)) return -1;
		if(!backtrack_stack.push(430)) return -1;
		if(!backtrack_stack.push(401)) return -1;
		if(!backtrack_stack.push(399)) return -1;
		if(!backtrack_stack.push(380)) return -1;
		if(!backtrack_stack.push(364)) return -1;
		if(!backtrack_stack.push(363)) return -1;
		if(!backtrack_stack.push(329)) return -1;
		if(!backtrack_stack.push(328)) return -1;
		if(!backtrack_stack.push(327)) return -1;
		if(!backtrack_stack.push(295)) return -1;
		if(!backtrack_stack.push(294)) return -1;
		if(!backtrack_stack.push(216)) return -1;
		if(!backtrack_stack.push(210)) return -1;
		if(!backtrack_stack.push(192)) return -1;
		if(!backtrack_stack.push(180)) return -1;
		if(!backtrack_stack.push(179)) return -1;
		if(!backtrack_stack.push(178)) return -1;
		if(!backtrack_stack.push(154)) return -1;
		if(!backtrack_stack.push(153)) return -1;
		if(!backtrack_stack.push(100)) return -1;
		if(!backtrack_stack.push(76)) return -1;
		if(!backtrack_stack.push(75)) return -1;
		if(!backtrack_stack.push(67)) return -1;
		if(!backtrack_stack.push(63)) return -1;
		if(!backtrack_stack.push(53)) return -1;
		if(!backtrack_stack.push(46)) return -1;
		if(!backtrack_stack.push(19)) return -1;
		goto label_16;
		}
		break;
	case 2041:
	// Label to restore values of label 15
		v_wine_1 = int_value_stack.pop();
		break;
	case 16: label_16: 
	// using  symbolic fact wine(v_wine_1) with literal 1 in rule 
	// 28: hassugar(A,A) :- wine(A), kaon2namedobjects(A).
	// resulting in symbolic fact hassugar(v_hassugar_1,v_hassugar_2)

		//recursive
		if(!int_value_stack.push(v_hassugar_2)) return -1;
		if(!int_value_stack.push(v_hassugar_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2042)) return -1;
		if (!kaon2namedobjects_b.contains(v_wine_1)) continue;
	case 1029:
	// Continuation for label 16
			v_hassugar_1 = v_wine_1;
			v_hassugar_2 = v_wine_1;
		if(hassugar_set.insert(v_hassugar_1, v_hassugar_2)) {
			if(!hassugar_bf.insert(v_hassugar_1, v_hassugar_2)) return -1;
			if(!hassugar_fb.insert(v_hassugar_2, v_hassugar_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1029)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(755)) return -1;
			if(!backtrack_stack.push(238)) return -1;
			if(!backtrack_stack.push(214)) return -1;
			if(!backtrack_stack.push(27)) return -1;
			if(!backtrack_stack.push(24)) return -1;
			if(!backtrack_stack.push(23)) return -1;
			if(!backtrack_stack.push(22)) return -1;
			goto label_17;
		}
		break;
	case 2042:
	// Label to restore values of label 16
		v_hassugar_1 = int_value_stack.pop();
		v_hassugar_2 = int_value_stack.pop();
		break;
	case 17: label_17: 
	// using  symbolic fact hassugar(v_hassugar_1,v_hassugar_2) with literal 1 in rule 
	// 26: kaon2equal(A,B) :- hassugar(C,A), hassugar(C,B).
	// resulting in symbolic fact kaon2equal(v_kaon2equal_1,v_kaon2equal_2)

		//recursive
		c26_2_bf.push(&cursor_stack);
		if(!int_value_stack.push(v_kaon2equal_2)) return -1;
		if(!int_value_stack.push(v_kaon2equal_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2043)) return -1;
		c26_2_bf.open(v_hassugar_1);
	case 1030:
	// Continuation for label 17
		while (c26_2_bf.fetch()) {
			v_kaon2equal_1 = v_hassugar_2;
			v_kaon2equal_2 = c26_2_bf.c2();
		if(kaon2equal_set.insert(v_kaon2equal_1, v_kaon2equal_2)) {
			if(!kaon2equal_bf.insert(v_kaon2equal_1, v_kaon2equal_2)) return -1;
			if(!kaon2equal_fb.insert(v_kaon2equal_2, v_kaon2equal_1)) return -1;
			if(!kaon2equal_ff.insert(v_kaon2equal_1, v_kaon2equal_2)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1030)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(283)) return -1;
			if(!backtrack_stack.push(282)) return -1;
			if(!backtrack_stack.push(281)) return -1;
			if(!backtrack_stack.push(280)) return -1;
			if(!backtrack_stack.push(279)) return -1;
			if(!backtrack_stack.push(278)) return -1;
			if(!backtrack_stack.push(277)) return -1;
			if(!backtrack_stack.push(276)) return -1;
			if(!backtrack_stack.push(275)) return -1;
			if(!backtrack_stack.push(274)) return -1;
			if(!backtrack_stack.push(273)) return -1;
			if(!backtrack_stack.push(271)) return -1;
			if(!backtrack_stack.push(270)) return -1;
			if(!backtrack_stack.push(269)) return -1;
			if(!backtrack_stack.push(268)) return -1;
			if(!backtrack_stack.push(267)) return -1;
			if(!backtrack_stack.push(266)) return -1;
			if(!backtrack_stack.push(265)) return -1;
			if(!backtrack_stack.push(264)) return -1;
			if(!backtrack_stack.push(263)) return -1;
			if(!backtrack_stack.push(262)) return -1;
			if(!backtrack_stack.push(261)) return -1;
			if(!backtrack_stack.push(260)) return -1;
			if(!backtrack_stack.push(254)) return -1;
			if(!backtrack_stack.push(253)) return -1;
			if(!backtrack_stack.push(152)) return -1;
			if(!backtrack_stack.push(142)) return -1;
			if(!backtrack_stack.push(141)) return -1;
			if(!backtrack_stack.push(140)) return -1;
			if(!backtrack_stack.push(139)) return -1;
			if(!backtrack_stack.push(138)) return -1;
			if(!backtrack_stack.push(137)) return -1;
			if(!backtrack_stack.push(134)) return -1;
			if(!backtrack_stack.push(133)) return -1;
			if(!backtrack_stack.push(127)) return -1;
			if(!backtrack_stack.push(126)) return -1;
			if(!backtrack_stack.push(125)) return -1;
			if(!backtrack_stack.push(124)) return -1;
			if(!backtrack_stack.push(85)) return -1;
			if(!backtrack_stack.push(82)) return -1;
			if(!backtrack_stack.push(81)) return -1;
			if(!backtrack_stack.push(73)) return -1;
			if(!backtrack_stack.push(72)) return -1;
			if(!backtrack_stack.push(69)) return -1;
			if(!backtrack_stack.push(62)) return -1;
			if(!backtrack_stack.push(57)) return -1;
			if(!backtrack_stack.push(56)) return -1;
			if(!backtrack_stack.push(55)) return -1;
			if(!backtrack_stack.push(45)) return -1;
			if(!backtrack_stack.push(42)) return -1;
			if(!backtrack_stack.push(41)) return -1;
			if(!backtrack_stack.push(35)) return -1;
			if(!backtrack_stack.push(34)) return -1;
			if(!backtrack_stack.push(33)) return -1;
			if(!backtrack_stack.push(29)) return -1;
			if(!backtrack_stack.push(28)) return -1;
			if(!backtrack_stack.push(21)) return -1;
			if(!backtrack_stack.push(18)) return -1;
			if(!backtrack_stack.push(13)) return -1;
			if(!backtrack_stack.push(5)) return -1;
			if(!backtrack_stack.push(4)) return -1;
			goto label_3;
		}
		}
		break;
	case 2043:
	// Label to restore values of label 17
		v_kaon2equal_1 = int_value_stack.pop();
		v_kaon2equal_2 = int_value_stack.pop();
		c26_2_bf.pop(&cursor_stack);
		break;
	case 18: label_18: 
	// using  symbolic fact kaon2equal(v_kaon2equal_1,v_kaon2equal_2) with literal 2 in rule 
	// 516: wine(A) :- wine(B), kaon2equal(A,B).
	// resulting in symbolic fact wine(v_wine_1)

		//recursive
		if(!int_value_stack.push(v_wine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2044)) return -1;
		if (!wine_set.contains(v_kaon2equal_2)) continue;
	case 1031:
	// Continuation for label 18
			v_wine_1 = v_kaon2equal_1;
		if(wine_set.insert(v_wine_1)) {
			if(!wine_f.insert(v_wine_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1031)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(1007)) return -1;
			if(!backtrack_stack.push(989)) return -1;
			if(!backtrack_stack.push(954)) return -1;
			if(!backtrack_stack.push(938)) return -1;
			if(!backtrack_stack.push(911)) return -1;
			if(!backtrack_stack.push(903)) return -1;
			if(!backtrack_stack.push(902)) return -1;
			if(!backtrack_stack.push(901)) return -1;
			if(!backtrack_stack.push(849)) return -1;
			if(!backtrack_stack.push(848)) return -1;
			if(!backtrack_stack.push(821)) return -1;
			if(!backtrack_stack.push(820)) return -1;
			if(!backtrack_stack.push(737)) return -1;
			if(!backtrack_stack.push(712)) return -1;
			if(!backtrack_stack.push(674)) return -1;
			if(!backtrack_stack.push(672)) return -1;
			if(!backtrack_stack.push(583)) return -1;
			if(!backtrack_stack.push(575)) return -1;
			if(!backtrack_stack.push(563)) return -1;
			if(!backtrack_stack.push(551)) return -1;
			if(!backtrack_stack.push(549)) return -1;
			if(!backtrack_stack.push(483)) return -1;
			if(!backtrack_stack.push(473)) return -1;
			if(!backtrack_stack.push(470)) return -1;
			if(!backtrack_stack.push(430)) return -1;
			if(!backtrack_stack.push(401)) return -1;
			if(!backtrack_stack.push(399)) return -1;
			if(!backtrack_stack.push(380)) return -1;
			if(!backtrack_stack.push(364)) return -1;
			if(!backtrack_stack.push(363)) return -1;
			if(!backtrack_stack.push(329)) return -1;
			if(!backtrack_stack.push(328)) return -1;
			if(!backtrack_stack.push(327)) return -1;
			if(!backtrack_stack.push(295)) return -1;
			if(!backtrack_stack.push(294)) return -1;
			if(!backtrack_stack.push(216)) return -1;
			if(!backtrack_stack.push(210)) return -1;
			if(!backtrack_stack.push(192)) return -1;
			if(!backtrack_stack.push(180)) return -1;
			if(!backtrack_stack.push(179)) return -1;
			if(!backtrack_stack.push(178)) return -1;
			if(!backtrack_stack.push(154)) return -1;
			if(!backtrack_stack.push(153)) return -1;
			if(!backtrack_stack.push(100)) return -1;
			if(!backtrack_stack.push(76)) return -1;
			if(!backtrack_stack.push(75)) return -1;
			if(!backtrack_stack.push(67)) return -1;
			if(!backtrack_stack.push(63)) return -1;
			if(!backtrack_stack.push(53)) return -1;
			if(!backtrack_stack.push(46)) return -1;
			if(!backtrack_stack.push(19)) return -1;
			goto label_16;
		}
		break;
	case 2044:
	// Label to restore values of label 18
		v_wine_1 = int_value_stack.pop();
		break;
	case 19: label_19: 
	// using  symbolic fact wine(v_wine_1) with literal 1 in rule 
	// 65: hasmaker(A,A) :- wine(A), kaon2namedobjects(A).
	// resulting in symbolic fact hasmaker(v_hasmaker_1,v_hasmaker_2)

		//recursive
		if(!int_value_stack.push(v_hasmaker_2)) return -1;
		if(!int_value_stack.push(v_hasmaker_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2045)) return -1;
		if (!kaon2namedobjects_b.contains(v_wine_1)) continue;
	case 1032:
	// Continuation for label 19
			v_hasmaker_1 = v_wine_1;
			v_hasmaker_2 = v_wine_1;
		if(hasmaker_set.insert(v_hasmaker_1, v_hasmaker_2)) {
			if(!hasmaker_bf.insert(v_hasmaker_1, v_hasmaker_2)) return -1;
			if(!hasmaker_fb.insert(v_hasmaker_2, v_hasmaker_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1032)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(40)) return -1;
			if(!backtrack_stack.push(39)) return -1;
			if(!backtrack_stack.push(38)) return -1;
			if(!backtrack_stack.push(36)) return -1;
			goto label_20;
		}
		break;
	case 2045:
	// Label to restore values of label 19
		v_hasmaker_1 = int_value_stack.pop();
		v_hasmaker_2 = int_value_stack.pop();
		break;
	case 20: label_20: 
	// using  symbolic fact hasmaker(v_hasmaker_1,v_hasmaker_2) with literal 1 in rule 
	// 61: kaon2equal(A,B) :- hasmaker(C,A), hasmaker(C,B).
	// resulting in symbolic fact kaon2equal(v_kaon2equal_1,v_kaon2equal_2)

		//recursive
		c61_2_bf.push(&cursor_stack);
		if(!int_value_stack.push(v_kaon2equal_2)) return -1;
		if(!int_value_stack.push(v_kaon2equal_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2046)) return -1;
		c61_2_bf.open(v_hasmaker_1);
	case 1033:
	// Continuation for label 20
		while (c61_2_bf.fetch()) {
			v_kaon2equal_1 = v_hasmaker_2;
			v_kaon2equal_2 = c61_2_bf.c2();
		if(kaon2equal_set.insert(v_kaon2equal_1, v_kaon2equal_2)) {
			if(!kaon2equal_bf.insert(v_kaon2equal_1, v_kaon2equal_2)) return -1;
			if(!kaon2equal_fb.insert(v_kaon2equal_2, v_kaon2equal_1)) return -1;
			if(!kaon2equal_ff.insert(v_kaon2equal_1, v_kaon2equal_2)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1033)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(283)) return -1;
			if(!backtrack_stack.push(282)) return -1;
			if(!backtrack_stack.push(281)) return -1;
			if(!backtrack_stack.push(280)) return -1;
			if(!backtrack_stack.push(279)) return -1;
			if(!backtrack_stack.push(278)) return -1;
			if(!backtrack_stack.push(277)) return -1;
			if(!backtrack_stack.push(276)) return -1;
			if(!backtrack_stack.push(275)) return -1;
			if(!backtrack_stack.push(274)) return -1;
			if(!backtrack_stack.push(273)) return -1;
			if(!backtrack_stack.push(271)) return -1;
			if(!backtrack_stack.push(270)) return -1;
			if(!backtrack_stack.push(269)) return -1;
			if(!backtrack_stack.push(268)) return -1;
			if(!backtrack_stack.push(267)) return -1;
			if(!backtrack_stack.push(266)) return -1;
			if(!backtrack_stack.push(265)) return -1;
			if(!backtrack_stack.push(264)) return -1;
			if(!backtrack_stack.push(263)) return -1;
			if(!backtrack_stack.push(262)) return -1;
			if(!backtrack_stack.push(261)) return -1;
			if(!backtrack_stack.push(260)) return -1;
			if(!backtrack_stack.push(254)) return -1;
			if(!backtrack_stack.push(253)) return -1;
			if(!backtrack_stack.push(152)) return -1;
			if(!backtrack_stack.push(142)) return -1;
			if(!backtrack_stack.push(141)) return -1;
			if(!backtrack_stack.push(140)) return -1;
			if(!backtrack_stack.push(139)) return -1;
			if(!backtrack_stack.push(138)) return -1;
			if(!backtrack_stack.push(137)) return -1;
			if(!backtrack_stack.push(134)) return -1;
			if(!backtrack_stack.push(133)) return -1;
			if(!backtrack_stack.push(127)) return -1;
			if(!backtrack_stack.push(126)) return -1;
			if(!backtrack_stack.push(125)) return -1;
			if(!backtrack_stack.push(124)) return -1;
			if(!backtrack_stack.push(85)) return -1;
			if(!backtrack_stack.push(82)) return -1;
			if(!backtrack_stack.push(81)) return -1;
			if(!backtrack_stack.push(73)) return -1;
			if(!backtrack_stack.push(72)) return -1;
			if(!backtrack_stack.push(69)) return -1;
			if(!backtrack_stack.push(62)) return -1;
			if(!backtrack_stack.push(57)) return -1;
			if(!backtrack_stack.push(56)) return -1;
			if(!backtrack_stack.push(55)) return -1;
			if(!backtrack_stack.push(45)) return -1;
			if(!backtrack_stack.push(42)) return -1;
			if(!backtrack_stack.push(41)) return -1;
			if(!backtrack_stack.push(35)) return -1;
			if(!backtrack_stack.push(34)) return -1;
			if(!backtrack_stack.push(33)) return -1;
			if(!backtrack_stack.push(29)) return -1;
			if(!backtrack_stack.push(28)) return -1;
			if(!backtrack_stack.push(21)) return -1;
			if(!backtrack_stack.push(18)) return -1;
			if(!backtrack_stack.push(13)) return -1;
			if(!backtrack_stack.push(5)) return -1;
			if(!backtrack_stack.push(4)) return -1;
			goto label_3;
		}
		}
		break;
	case 2046:
	// Label to restore values of label 20
		v_kaon2equal_1 = int_value_stack.pop();
		v_kaon2equal_2 = int_value_stack.pop();
		c61_2_bf.pop(&cursor_stack);
		break;
	case 21: label_21: 
	// using  symbolic fact kaon2equal(v_kaon2equal_1,v_kaon2equal_2) with literal 2 in rule 
	// 593: hassugar(A,B) :- hassugar(C,B), kaon2equal(A,C).
	// resulting in symbolic fact hassugar(v_hassugar_1,v_hassugar_2)

		//recursive
		c593_1_bf.push(&cursor_stack);
		if(!int_value_stack.push(v_hassugar_2)) return -1;
		if(!int_value_stack.push(v_hassugar_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2047)) return -1;
		c593_1_bf.open(v_kaon2equal_2);
	case 1034:
	// Continuation for label 21
		while (c593_1_bf.fetch()) {
			v_hassugar_1 = v_kaon2equal_1;
			v_hassugar_2 = c593_1_bf.c2();
		if(hassugar_set.insert(v_hassugar_1, v_hassugar_2)) {
			if(!hassugar_bf.insert(v_hassugar_1, v_hassugar_2)) return -1;
			if(!hassugar_fb.insert(v_hassugar_2, v_hassugar_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1034)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(755)) return -1;
			if(!backtrack_stack.push(238)) return -1;
			if(!backtrack_stack.push(214)) return -1;
			if(!backtrack_stack.push(27)) return -1;
			if(!backtrack_stack.push(24)) return -1;
			if(!backtrack_stack.push(23)) return -1;
			if(!backtrack_stack.push(22)) return -1;
			goto label_17;
		}
		}
		break;
	case 2047:
	// Label to restore values of label 21
		v_hassugar_1 = int_value_stack.pop();
		v_hassugar_2 = int_value_stack.pop();
		c593_1_bf.pop(&cursor_stack);
		break;
	case 22: label_22: 
	// using  symbolic fact hassugar(v_hassugar_1,v_hassugar_2) with literal 1 in rule 
	// 593: hassugar(A,B) :- hassugar(C,B), kaon2equal(A,C).
	// resulting in symbolic fact hassugar(v_hassugar_1,v_hassugar_2)

		//recursive
		c593_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_hassugar_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2048)) return -1;
		c593_2_fb.open(v_hassugar_1);
	case 1035:
	// Continuation for label 22
		while (c593_2_fb.fetch()) {
			v_hassugar_1 = c593_2_fb.c2();
		if(hassugar_set.insert(v_hassugar_1, v_hassugar_2)) {
			if(!hassugar_bf.insert(v_hassugar_1, v_hassugar_2)) return -1;
			if(!hassugar_fb.insert(v_hassugar_2, v_hassugar_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1035)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(755)) return -1;
			if(!backtrack_stack.push(238)) return -1;
			if(!backtrack_stack.push(214)) return -1;
			if(!backtrack_stack.push(27)) return -1;
			if(!backtrack_stack.push(24)) return -1;
			if(!backtrack_stack.push(23)) return -1;
			if(!backtrack_stack.push(22)) return -1;
			goto label_17;
		}
		}
		break;
	case 2048:
	// Label to restore values of label 22
		v_hassugar_1 = int_value_stack.pop();
		c593_2_fb.pop(&cursor_stack);
		break;
	case 23: label_23: 
	// using  symbolic fact hassugar(v_hassugar_1,v_hassugar_2) with literal 1 in rule 
	// 594: hassugar(A,B) :- hassugar(A,C), kaon2equal(B,C).
	// resulting in symbolic fact hassugar(v_hassugar_1,v_hassugar_2)

		//recursive
		c594_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_hassugar_2)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2049)) return -1;
		c594_2_fb.open(v_hassugar_2);
	case 1036:
	// Continuation for label 23
		while (c594_2_fb.fetch()) {
			v_hassugar_2 = c594_2_fb.c2();
		if(hassugar_set.insert(v_hassugar_1, v_hassugar_2)) {
			if(!hassugar_bf.insert(v_hassugar_1, v_hassugar_2)) return -1;
			if(!hassugar_fb.insert(v_hassugar_2, v_hassugar_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1036)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(755)) return -1;
			if(!backtrack_stack.push(238)) return -1;
			if(!backtrack_stack.push(214)) return -1;
			if(!backtrack_stack.push(27)) return -1;
			if(!backtrack_stack.push(24)) return -1;
			if(!backtrack_stack.push(23)) return -1;
			if(!backtrack_stack.push(22)) return -1;
			goto label_17;
		}
		}
		break;
	case 2049:
	// Label to restore values of label 23
		v_hassugar_2 = int_value_stack.pop();
		c594_2_fb.pop(&cursor_stack);
		break;
	case 24: label_24: 
	// using  symbolic fact hassugar(v_hassugar_1,v_hassugar_2) with literal 1 in rule 
	// 650: haswinedescriptor(A,B) :- hassugar(A,B).
	// resulting in symbolic fact haswinedescriptor(v_haswinedescriptor_1,v_haswinedescriptor_2)

		//recursive
		if(!int_value_stack.push(v_haswinedescriptor_2)) return -1;
		if(!int_value_stack.push(v_haswinedescriptor_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2050)) return -1;
		v_haswinedescriptor_1 = v_hassugar_1;
		v_haswinedescriptor_2 = v_hassugar_2;
		if(haswinedescriptor_set.insert(v_haswinedescriptor_1, v_haswinedescriptor_2)) {
		if(!haswinedescriptor_bf.insert(v_haswinedescriptor_1, v_haswinedescriptor_2)) return -1;
		if(!haswinedescriptor_fb.insert(v_haswinedescriptor_2, v_haswinedescriptor_1)) return -1;
		// Labels for next rules
		if(!backtrack_stack.push(26)) return -1;
		if(!backtrack_stack.push(25)) return -1;
		goto label_15;
		}
		break;
	case 2050:
	// Label to restore values of label 24
		v_haswinedescriptor_1 = int_value_stack.pop();
		v_haswinedescriptor_2 = int_value_stack.pop();
		break;
	case 25: label_25: 
	// using  symbolic fact haswinedescriptor(v_haswinedescriptor_1,v_haswinedescriptor_2) with literal 1 in rule 
	// 651: haswinedescriptor(A,B) :- haswinedescriptor(C,B), kaon2equal(A,C).
	// resulting in symbolic fact haswinedescriptor(v_haswinedescriptor_1,v_haswinedescriptor_2)

		//recursive
		c651_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_haswinedescriptor_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2051)) return -1;
		c651_2_fb.open(v_haswinedescriptor_1);
	case 1038:
	// Continuation for label 25
		while (c651_2_fb.fetch()) {
			v_haswinedescriptor_1 = c651_2_fb.c2();
		if(haswinedescriptor_set.insert(v_haswinedescriptor_1, v_haswinedescriptor_2)) {
			if(!haswinedescriptor_bf.insert(v_haswinedescriptor_1, v_haswinedescriptor_2)) return -1;
			if(!haswinedescriptor_fb.insert(v_haswinedescriptor_2, v_haswinedescriptor_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1038)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(26)) return -1;
			if(!backtrack_stack.push(25)) return -1;
			goto label_15;
		}
		}
		break;
	case 2051:
	// Label to restore values of label 25
		v_haswinedescriptor_1 = int_value_stack.pop();
		c651_2_fb.pop(&cursor_stack);
		break;
	case 26: label_26: 
	// using  symbolic fact haswinedescriptor(v_haswinedescriptor_1,v_haswinedescriptor_2) with literal 1 in rule 
	// 652: haswinedescriptor(A,B) :- haswinedescriptor(A,C), kaon2equal(B,C).
	// resulting in symbolic fact haswinedescriptor(v_haswinedescriptor_1,v_haswinedescriptor_2)

		//recursive
		c652_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_haswinedescriptor_2)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2052)) return -1;
		c652_2_fb.open(v_haswinedescriptor_2);
	case 1039:
	// Continuation for label 26
		while (c652_2_fb.fetch()) {
			v_haswinedescriptor_2 = c652_2_fb.c2();
		if(haswinedescriptor_set.insert(v_haswinedescriptor_1, v_haswinedescriptor_2)) {
			if(!haswinedescriptor_bf.insert(v_haswinedescriptor_1, v_haswinedescriptor_2)) return -1;
			if(!haswinedescriptor_fb.insert(v_haswinedescriptor_2, v_haswinedescriptor_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1039)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(26)) return -1;
			if(!backtrack_stack.push(25)) return -1;
			goto label_15;
		}
		}
		break;
	case 2052:
	// Label to restore values of label 26
		v_haswinedescriptor_2 = int_value_stack.pop();
		c652_2_fb.pop(&cursor_stack);
		break;
	case 27: label_27: 
	// using  symbolic fact hassugar(v_hassugar_1,v_hassugar_2) with literal 2 in rule 
	// 26: kaon2equal(A,B) :- hassugar(C,A), hassugar(C,B).
	// resulting in symbolic fact kaon2equal(v_kaon2equal_1,v_kaon2equal_2)

		//recursive
		c26_1_bf.push(&cursor_stack);
		if(!int_value_stack.push(v_kaon2equal_1)) return -1;
		if(!int_value_stack.push(v_kaon2equal_2)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2053)) return -1;
		c26_1_bf.open(v_hassugar_1);
	case 1040:
	// Continuation for label 27
		while (c26_1_bf.fetch()) {
			v_kaon2equal_2 = v_hassugar_2;
			v_kaon2equal_1 = c26_1_bf.c2();
		if(kaon2equal_set.insert(v_kaon2equal_1, v_kaon2equal_2)) {
			if(!kaon2equal_bf.insert(v_kaon2equal_1, v_kaon2equal_2)) return -1;
			if(!kaon2equal_fb.insert(v_kaon2equal_2, v_kaon2equal_1)) return -1;
			if(!kaon2equal_ff.insert(v_kaon2equal_1, v_kaon2equal_2)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1040)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(283)) return -1;
			if(!backtrack_stack.push(282)) return -1;
			if(!backtrack_stack.push(281)) return -1;
			if(!backtrack_stack.push(280)) return -1;
			if(!backtrack_stack.push(279)) return -1;
			if(!backtrack_stack.push(278)) return -1;
			if(!backtrack_stack.push(277)) return -1;
			if(!backtrack_stack.push(276)) return -1;
			if(!backtrack_stack.push(275)) return -1;
			if(!backtrack_stack.push(274)) return -1;
			if(!backtrack_stack.push(273)) return -1;
			if(!backtrack_stack.push(271)) return -1;
			if(!backtrack_stack.push(270)) return -1;
			if(!backtrack_stack.push(269)) return -1;
			if(!backtrack_stack.push(268)) return -1;
			if(!backtrack_stack.push(267)) return -1;
			if(!backtrack_stack.push(266)) return -1;
			if(!backtrack_stack.push(265)) return -1;
			if(!backtrack_stack.push(264)) return -1;
			if(!backtrack_stack.push(263)) return -1;
			if(!backtrack_stack.push(262)) return -1;
			if(!backtrack_stack.push(261)) return -1;
			if(!backtrack_stack.push(260)) return -1;
			if(!backtrack_stack.push(254)) return -1;
			if(!backtrack_stack.push(253)) return -1;
			if(!backtrack_stack.push(152)) return -1;
			if(!backtrack_stack.push(142)) return -1;
			if(!backtrack_stack.push(141)) return -1;
			if(!backtrack_stack.push(140)) return -1;
			if(!backtrack_stack.push(139)) return -1;
			if(!backtrack_stack.push(138)) return -1;
			if(!backtrack_stack.push(137)) return -1;
			if(!backtrack_stack.push(134)) return -1;
			if(!backtrack_stack.push(133)) return -1;
			if(!backtrack_stack.push(127)) return -1;
			if(!backtrack_stack.push(126)) return -1;
			if(!backtrack_stack.push(125)) return -1;
			if(!backtrack_stack.push(124)) return -1;
			if(!backtrack_stack.push(85)) return -1;
			if(!backtrack_stack.push(82)) return -1;
			if(!backtrack_stack.push(81)) return -1;
			if(!backtrack_stack.push(73)) return -1;
			if(!backtrack_stack.push(72)) return -1;
			if(!backtrack_stack.push(69)) return -1;
			if(!backtrack_stack.push(62)) return -1;
			if(!backtrack_stack.push(57)) return -1;
			if(!backtrack_stack.push(56)) return -1;
			if(!backtrack_stack.push(55)) return -1;
			if(!backtrack_stack.push(45)) return -1;
			if(!backtrack_stack.push(42)) return -1;
			if(!backtrack_stack.push(41)) return -1;
			if(!backtrack_stack.push(35)) return -1;
			if(!backtrack_stack.push(34)) return -1;
			if(!backtrack_stack.push(33)) return -1;
			if(!backtrack_stack.push(29)) return -1;
			if(!backtrack_stack.push(28)) return -1;
			if(!backtrack_stack.push(21)) return -1;
			if(!backtrack_stack.push(18)) return -1;
			if(!backtrack_stack.push(13)) return -1;
			if(!backtrack_stack.push(5)) return -1;
			if(!backtrack_stack.push(4)) return -1;
			goto label_3;
		}
		}
		break;
	case 2053:
	// Label to restore values of label 27
		v_kaon2equal_2 = int_value_stack.pop();
		v_kaon2equal_1 = int_value_stack.pop();
		c26_1_bf.pop(&cursor_stack);
		break;
	case 28: label_28: 
	// using  symbolic fact kaon2equal(v_kaon2equal_1,v_kaon2equal_2) with literal 2 in rule 
	// 594: hassugar(A,B) :- hassugar(A,C), kaon2equal(B,C).
	// resulting in symbolic fact hassugar(v_hassugar_1,v_hassugar_2)

		//recursive
		c594_1_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_hassugar_1)) return -1;
		if(!int_value_stack.push(v_hassugar_2)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2054)) return -1;
		c594_1_fb.open(v_kaon2equal_2);
	case 1041:
	// Continuation for label 28
		while (c594_1_fb.fetch()) {
			v_hassugar_2 = v_kaon2equal_1;
			v_hassugar_1 = c594_1_fb.c2();
		if(hassugar_set.insert(v_hassugar_1, v_hassugar_2)) {
			if(!hassugar_bf.insert(v_hassugar_1, v_hassugar_2)) return -1;
			if(!hassugar_fb.insert(v_hassugar_2, v_hassugar_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1041)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(755)) return -1;
			if(!backtrack_stack.push(238)) return -1;
			if(!backtrack_stack.push(214)) return -1;
			if(!backtrack_stack.push(27)) return -1;
			if(!backtrack_stack.push(24)) return -1;
			if(!backtrack_stack.push(23)) return -1;
			if(!backtrack_stack.push(22)) return -1;
			goto label_17;
		}
		}
		break;
	case 2054:
	// Label to restore values of label 28
		v_hassugar_2 = int_value_stack.pop();
		v_hassugar_1 = int_value_stack.pop();
		c594_1_fb.pop(&cursor_stack);
		break;
	case 29: label_29: 
	// using  symbolic fact kaon2equal(v_kaon2equal_1,v_kaon2equal_2) with literal 2 in rule 
	// 649: hasflavor(A,B) :- hasflavor(A,C), kaon2equal(B,C).
	// resulting in symbolic fact hasflavor(v_hasflavor_1,v_hasflavor_2)

		//recursive
		c649_1_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_hasflavor_1)) return -1;
		if(!int_value_stack.push(v_hasflavor_2)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2055)) return -1;
		c649_1_fb.open(v_kaon2equal_2);
	case 1042:
	// Continuation for label 29
		while (c649_1_fb.fetch()) {
			v_hasflavor_2 = v_kaon2equal_1;
			v_hasflavor_1 = c649_1_fb.c2();
		if(hasflavor_set.insert(v_hasflavor_1, v_hasflavor_2)) {
			if(!hasflavor_bf.insert(v_hasflavor_1, v_hasflavor_2)) return -1;
			if(!hasflavor_fb.insert(v_hasflavor_2, v_hasflavor_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1042)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(32)) return -1;
			if(!backtrack_stack.push(31)) return -1;
			if(!backtrack_stack.push(30)) return -1;
			if(!backtrack_stack.push(14)) return -1;
			goto label_12;
		}
		}
		break;
	case 2055:
	// Label to restore values of label 29
		v_hasflavor_2 = int_value_stack.pop();
		v_hasflavor_1 = int_value_stack.pop();
		c649_1_fb.pop(&cursor_stack);
		break;
	case 30: label_30: 
	// using  symbolic fact hasflavor(v_hasflavor_1,v_hasflavor_2) with literal 1 in rule 
	// 648: hasflavor(A,B) :- hasflavor(C,B), kaon2equal(A,C).
	// resulting in symbolic fact hasflavor(v_hasflavor_1,v_hasflavor_2)

		//recursive
		c648_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_hasflavor_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2056)) return -1;
		c648_2_fb.open(v_hasflavor_1);
	case 1043:
	// Continuation for label 30
		while (c648_2_fb.fetch()) {
			v_hasflavor_1 = c648_2_fb.c2();
		if(hasflavor_set.insert(v_hasflavor_1, v_hasflavor_2)) {
			if(!hasflavor_bf.insert(v_hasflavor_1, v_hasflavor_2)) return -1;
			if(!hasflavor_fb.insert(v_hasflavor_2, v_hasflavor_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1043)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(32)) return -1;
			if(!backtrack_stack.push(31)) return -1;
			if(!backtrack_stack.push(30)) return -1;
			if(!backtrack_stack.push(14)) return -1;
			goto label_12;
		}
		}
		break;
	case 2056:
	// Label to restore values of label 30
		v_hasflavor_1 = int_value_stack.pop();
		c648_2_fb.pop(&cursor_stack);
		break;
	case 31: label_31: 
	// using  symbolic fact hasflavor(v_hasflavor_1,v_hasflavor_2) with literal 1 in rule 
	// 649: hasflavor(A,B) :- hasflavor(A,C), kaon2equal(B,C).
	// resulting in symbolic fact hasflavor(v_hasflavor_1,v_hasflavor_2)

		//recursive
		c649_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_hasflavor_2)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2057)) return -1;
		c649_2_fb.open(v_hasflavor_2);
	case 1044:
	// Continuation for label 31
		while (c649_2_fb.fetch()) {
			v_hasflavor_2 = c649_2_fb.c2();
		if(hasflavor_set.insert(v_hasflavor_1, v_hasflavor_2)) {
			if(!hasflavor_bf.insert(v_hasflavor_1, v_hasflavor_2)) return -1;
			if(!hasflavor_fb.insert(v_hasflavor_2, v_hasflavor_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1044)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(32)) return -1;
			if(!backtrack_stack.push(31)) return -1;
			if(!backtrack_stack.push(30)) return -1;
			if(!backtrack_stack.push(14)) return -1;
			goto label_12;
		}
		}
		break;
	case 2057:
	// Label to restore values of label 31
		v_hasflavor_2 = int_value_stack.pop();
		c649_2_fb.pop(&cursor_stack);
		break;
	case 32: label_32: 
	// using  symbolic fact hasflavor(v_hasflavor_1,v_hasflavor_2) with literal 2 in rule 
	// 8: kaon2equal(A,B) :- hasflavor(C,A), hasflavor(C,B).
	// resulting in symbolic fact kaon2equal(v_kaon2equal_1,v_kaon2equal_2)

		//recursive
		c8_1_bf.push(&cursor_stack);
		if(!int_value_stack.push(v_kaon2equal_1)) return -1;
		if(!int_value_stack.push(v_kaon2equal_2)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2058)) return -1;
		c8_1_bf.open(v_hasflavor_1);
	case 1045:
	// Continuation for label 32
		while (c8_1_bf.fetch()) {
			v_kaon2equal_2 = v_hasflavor_2;
			v_kaon2equal_1 = c8_1_bf.c2();
		if(kaon2equal_set.insert(v_kaon2equal_1, v_kaon2equal_2)) {
			if(!kaon2equal_bf.insert(v_kaon2equal_1, v_kaon2equal_2)) return -1;
			if(!kaon2equal_fb.insert(v_kaon2equal_2, v_kaon2equal_1)) return -1;
			if(!kaon2equal_ff.insert(v_kaon2equal_1, v_kaon2equal_2)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1045)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(283)) return -1;
			if(!backtrack_stack.push(282)) return -1;
			if(!backtrack_stack.push(281)) return -1;
			if(!backtrack_stack.push(280)) return -1;
			if(!backtrack_stack.push(279)) return -1;
			if(!backtrack_stack.push(278)) return -1;
			if(!backtrack_stack.push(277)) return -1;
			if(!backtrack_stack.push(276)) return -1;
			if(!backtrack_stack.push(275)) return -1;
			if(!backtrack_stack.push(274)) return -1;
			if(!backtrack_stack.push(273)) return -1;
			if(!backtrack_stack.push(271)) return -1;
			if(!backtrack_stack.push(270)) return -1;
			if(!backtrack_stack.push(269)) return -1;
			if(!backtrack_stack.push(268)) return -1;
			if(!backtrack_stack.push(267)) return -1;
			if(!backtrack_stack.push(266)) return -1;
			if(!backtrack_stack.push(265)) return -1;
			if(!backtrack_stack.push(264)) return -1;
			if(!backtrack_stack.push(263)) return -1;
			if(!backtrack_stack.push(262)) return -1;
			if(!backtrack_stack.push(261)) return -1;
			if(!backtrack_stack.push(260)) return -1;
			if(!backtrack_stack.push(254)) return -1;
			if(!backtrack_stack.push(253)) return -1;
			if(!backtrack_stack.push(152)) return -1;
			if(!backtrack_stack.push(142)) return -1;
			if(!backtrack_stack.push(141)) return -1;
			if(!backtrack_stack.push(140)) return -1;
			if(!backtrack_stack.push(139)) return -1;
			if(!backtrack_stack.push(138)) return -1;
			if(!backtrack_stack.push(137)) return -1;
			if(!backtrack_stack.push(134)) return -1;
			if(!backtrack_stack.push(133)) return -1;
			if(!backtrack_stack.push(127)) return -1;
			if(!backtrack_stack.push(126)) return -1;
			if(!backtrack_stack.push(125)) return -1;
			if(!backtrack_stack.push(124)) return -1;
			if(!backtrack_stack.push(85)) return -1;
			if(!backtrack_stack.push(82)) return -1;
			if(!backtrack_stack.push(81)) return -1;
			if(!backtrack_stack.push(73)) return -1;
			if(!backtrack_stack.push(72)) return -1;
			if(!backtrack_stack.push(69)) return -1;
			if(!backtrack_stack.push(62)) return -1;
			if(!backtrack_stack.push(57)) return -1;
			if(!backtrack_stack.push(56)) return -1;
			if(!backtrack_stack.push(55)) return -1;
			if(!backtrack_stack.push(45)) return -1;
			if(!backtrack_stack.push(42)) return -1;
			if(!backtrack_stack.push(41)) return -1;
			if(!backtrack_stack.push(35)) return -1;
			if(!backtrack_stack.push(34)) return -1;
			if(!backtrack_stack.push(33)) return -1;
			if(!backtrack_stack.push(29)) return -1;
			if(!backtrack_stack.push(28)) return -1;
			if(!backtrack_stack.push(21)) return -1;
			if(!backtrack_stack.push(18)) return -1;
			if(!backtrack_stack.push(13)) return -1;
			if(!backtrack_stack.push(5)) return -1;
			if(!backtrack_stack.push(4)) return -1;
			goto label_3;
		}
		}
		break;
	case 2058:
	// Label to restore values of label 32
		v_kaon2equal_2 = int_value_stack.pop();
		v_kaon2equal_1 = int_value_stack.pop();
		c8_1_bf.pop(&cursor_stack);
		break;
	case 33: label_33: 
	// using  symbolic fact kaon2equal(v_kaon2equal_1,v_kaon2equal_2) with literal 2 in rule 
	// 651: haswinedescriptor(A,B) :- haswinedescriptor(C,B), kaon2equal(A,C).
	// resulting in symbolic fact haswinedescriptor(v_haswinedescriptor_1,v_haswinedescriptor_2)

		//recursive
		c651_1_bf.push(&cursor_stack);
		if(!int_value_stack.push(v_haswinedescriptor_2)) return -1;
		if(!int_value_stack.push(v_haswinedescriptor_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2059)) return -1;
		c651_1_bf.open(v_kaon2equal_2);
	case 1046:
	// Continuation for label 33
		while (c651_1_bf.fetch()) {
			v_haswinedescriptor_1 = v_kaon2equal_1;
			v_haswinedescriptor_2 = c651_1_bf.c2();
		if(haswinedescriptor_set.insert(v_haswinedescriptor_1, v_haswinedescriptor_2)) {
			if(!haswinedescriptor_bf.insert(v_haswinedescriptor_1, v_haswinedescriptor_2)) return -1;
			if(!haswinedescriptor_fb.insert(v_haswinedescriptor_2, v_haswinedescriptor_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1046)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(26)) return -1;
			if(!backtrack_stack.push(25)) return -1;
			goto label_15;
		}
		}
		break;
	case 2059:
	// Label to restore values of label 33
		v_haswinedescriptor_1 = int_value_stack.pop();
		v_haswinedescriptor_2 = int_value_stack.pop();
		c651_1_bf.pop(&cursor_stack);
		break;
	case 34: label_34: 
	// using  symbolic fact kaon2equal(v_kaon2equal_1,v_kaon2equal_2) with literal 2 in rule 
	// 652: haswinedescriptor(A,B) :- haswinedescriptor(A,C), kaon2equal(B,C).
	// resulting in symbolic fact haswinedescriptor(v_haswinedescriptor_1,v_haswinedescriptor_2)

		//recursive
		c652_1_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_haswinedescriptor_1)) return -1;
		if(!int_value_stack.push(v_haswinedescriptor_2)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2060)) return -1;
		c652_1_fb.open(v_kaon2equal_2);
	case 1047:
	// Continuation for label 34
		while (c652_1_fb.fetch()) {
			v_haswinedescriptor_2 = v_kaon2equal_1;
			v_haswinedescriptor_1 = c652_1_fb.c2();
		if(haswinedescriptor_set.insert(v_haswinedescriptor_1, v_haswinedescriptor_2)) {
			if(!haswinedescriptor_bf.insert(v_haswinedescriptor_1, v_haswinedescriptor_2)) return -1;
			if(!haswinedescriptor_fb.insert(v_haswinedescriptor_2, v_haswinedescriptor_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1047)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(26)) return -1;
			if(!backtrack_stack.push(25)) return -1;
			goto label_15;
		}
		}
		break;
	case 2060:
	// Label to restore values of label 34
		v_haswinedescriptor_2 = int_value_stack.pop();
		v_haswinedescriptor_1 = int_value_stack.pop();
		c652_1_fb.pop(&cursor_stack);
		break;
	case 35: label_35: 
	// using  symbolic fact kaon2equal(v_kaon2equal_1,v_kaon2equal_2) with literal 2 in rule 
	// 792: hasmaker(A,B) :- hasmaker(C,B), kaon2equal(A,C).
	// resulting in symbolic fact hasmaker(v_hasmaker_1,v_hasmaker_2)

		//recursive
		c792_1_bf.push(&cursor_stack);
		if(!int_value_stack.push(v_hasmaker_2)) return -1;
		if(!int_value_stack.push(v_hasmaker_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2061)) return -1;
		c792_1_bf.open(v_kaon2equal_2);
	case 1048:
	// Continuation for label 35
		while (c792_1_bf.fetch()) {
			v_hasmaker_1 = v_kaon2equal_1;
			v_hasmaker_2 = c792_1_bf.c2();
		if(hasmaker_set.insert(v_hasmaker_1, v_hasmaker_2)) {
			if(!hasmaker_bf.insert(v_hasmaker_1, v_hasmaker_2)) return -1;
			if(!hasmaker_fb.insert(v_hasmaker_2, v_hasmaker_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1048)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(40)) return -1;
			if(!backtrack_stack.push(39)) return -1;
			if(!backtrack_stack.push(38)) return -1;
			if(!backtrack_stack.push(36)) return -1;
			goto label_20;
		}
		}
		break;
	case 2061:
	// Label to restore values of label 35
		v_hasmaker_1 = int_value_stack.pop();
		v_hasmaker_2 = int_value_stack.pop();
		c792_1_bf.pop(&cursor_stack);
		break;
	case 36: label_36: 
	// using  symbolic fact hasmaker(v_hasmaker_1,v_hasmaker_2) with literal 1 in rule 
	// 64: produceswine(A,B) :- hasmaker(B,A).
	// resulting in symbolic fact produceswine(v_produceswine_1,v_produceswine_2)

		//recursive
		if(!int_value_stack.push(v_produceswine_2)) return -1;
		if(!int_value_stack.push(v_produceswine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2062)) return -1;
		v_produceswine_1 = v_hasmaker_2;
		v_produceswine_2 = v_hasmaker_1;
		if(produceswine_set.insert(v_produceswine_1, v_produceswine_2)) {
		if(!produceswine_bf.insert(v_produceswine_1, v_produceswine_2)) return -1;
		if(!produceswine_fb.insert(v_produceswine_2, v_produceswine_1)) return -1;
		// Labels for next rules
		if(!backtrack_stack.push(44)) return -1;
		if(!backtrack_stack.push(43)) return -1;
		goto label_37;
		}
		break;
	case 2062:
	// Label to restore values of label 36
		v_produceswine_1 = int_value_stack.pop();
		v_produceswine_2 = int_value_stack.pop();
		break;
	case 37: label_37: 
	// using  symbolic fact produceswine(v_produceswine_1,v_produceswine_2) with literal 1 in rule 
	// 63: hasmaker(A,B) :- produceswine(B,A).
	// resulting in symbolic fact hasmaker(v_hasmaker_1,v_hasmaker_2)

		//recursive
		if(!int_value_stack.push(v_hasmaker_2)) return -1;
		if(!int_value_stack.push(v_hasmaker_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2063)) return -1;
		v_hasmaker_1 = v_produceswine_2;
		v_hasmaker_2 = v_produceswine_1;
		if(hasmaker_set.insert(v_hasmaker_1, v_hasmaker_2)) {
		if(!hasmaker_bf.insert(v_hasmaker_1, v_hasmaker_2)) return -1;
		if(!hasmaker_fb.insert(v_hasmaker_2, v_hasmaker_1)) return -1;
		// Labels for next rules
		if(!backtrack_stack.push(40)) return -1;
		if(!backtrack_stack.push(39)) return -1;
		if(!backtrack_stack.push(38)) return -1;
		if(!backtrack_stack.push(36)) return -1;
		goto label_20;
		}
		break;
	case 2063:
	// Label to restore values of label 37
		v_hasmaker_1 = int_value_stack.pop();
		v_hasmaker_2 = int_value_stack.pop();
		break;
	case 38: label_38: 
	// using  symbolic fact hasmaker(v_hasmaker_1,v_hasmaker_2) with literal 1 in rule 
	// 792: hasmaker(A,B) :- hasmaker(C,B), kaon2equal(A,C).
	// resulting in symbolic fact hasmaker(v_hasmaker_1,v_hasmaker_2)

		//recursive
		c792_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_hasmaker_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2064)) return -1;
		c792_2_fb.open(v_hasmaker_1);
	case 1051:
	// Continuation for label 38
		while (c792_2_fb.fetch()) {
			v_hasmaker_1 = c792_2_fb.c2();
		if(hasmaker_set.insert(v_hasmaker_1, v_hasmaker_2)) {
			if(!hasmaker_bf.insert(v_hasmaker_1, v_hasmaker_2)) return -1;
			if(!hasmaker_fb.insert(v_hasmaker_2, v_hasmaker_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1051)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(40)) return -1;
			if(!backtrack_stack.push(39)) return -1;
			if(!backtrack_stack.push(38)) return -1;
			if(!backtrack_stack.push(36)) return -1;
			goto label_20;
		}
		}
		break;
	case 2064:
	// Label to restore values of label 38
		v_hasmaker_1 = int_value_stack.pop();
		c792_2_fb.pop(&cursor_stack);
		break;
	case 39: label_39: 
	// using  symbolic fact hasmaker(v_hasmaker_1,v_hasmaker_2) with literal 1 in rule 
	// 793: hasmaker(A,B) :- hasmaker(A,C), kaon2equal(B,C).
	// resulting in symbolic fact hasmaker(v_hasmaker_1,v_hasmaker_2)

		//recursive
		c793_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_hasmaker_2)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2065)) return -1;
		c793_2_fb.open(v_hasmaker_2);
	case 1052:
	// Continuation for label 39
		while (c793_2_fb.fetch()) {
			v_hasmaker_2 = c793_2_fb.c2();
		if(hasmaker_set.insert(v_hasmaker_1, v_hasmaker_2)) {
			if(!hasmaker_bf.insert(v_hasmaker_1, v_hasmaker_2)) return -1;
			if(!hasmaker_fb.insert(v_hasmaker_2, v_hasmaker_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1052)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(40)) return -1;
			if(!backtrack_stack.push(39)) return -1;
			if(!backtrack_stack.push(38)) return -1;
			if(!backtrack_stack.push(36)) return -1;
			goto label_20;
		}
		}
		break;
	case 2065:
	// Label to restore values of label 39
		v_hasmaker_2 = int_value_stack.pop();
		c793_2_fb.pop(&cursor_stack);
		break;
	case 40: label_40: 
	// using  symbolic fact hasmaker(v_hasmaker_1,v_hasmaker_2) with literal 2 in rule 
	// 61: kaon2equal(A,B) :- hasmaker(C,A), hasmaker(C,B).
	// resulting in symbolic fact kaon2equal(v_kaon2equal_1,v_kaon2equal_2)

		//recursive
		c61_1_bf.push(&cursor_stack);
		if(!int_value_stack.push(v_kaon2equal_1)) return -1;
		if(!int_value_stack.push(v_kaon2equal_2)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2066)) return -1;
		c61_1_bf.open(v_hasmaker_1);
	case 1053:
	// Continuation for label 40
		while (c61_1_bf.fetch()) {
			v_kaon2equal_2 = v_hasmaker_2;
			v_kaon2equal_1 = c61_1_bf.c2();
		if(kaon2equal_set.insert(v_kaon2equal_1, v_kaon2equal_2)) {
			if(!kaon2equal_bf.insert(v_kaon2equal_1, v_kaon2equal_2)) return -1;
			if(!kaon2equal_fb.insert(v_kaon2equal_2, v_kaon2equal_1)) return -1;
			if(!kaon2equal_ff.insert(v_kaon2equal_1, v_kaon2equal_2)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1053)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(283)) return -1;
			if(!backtrack_stack.push(282)) return -1;
			if(!backtrack_stack.push(281)) return -1;
			if(!backtrack_stack.push(280)) return -1;
			if(!backtrack_stack.push(279)) return -1;
			if(!backtrack_stack.push(278)) return -1;
			if(!backtrack_stack.push(277)) return -1;
			if(!backtrack_stack.push(276)) return -1;
			if(!backtrack_stack.push(275)) return -1;
			if(!backtrack_stack.push(274)) return -1;
			if(!backtrack_stack.push(273)) return -1;
			if(!backtrack_stack.push(271)) return -1;
			if(!backtrack_stack.push(270)) return -1;
			if(!backtrack_stack.push(269)) return -1;
			if(!backtrack_stack.push(268)) return -1;
			if(!backtrack_stack.push(267)) return -1;
			if(!backtrack_stack.push(266)) return -1;
			if(!backtrack_stack.push(265)) return -1;
			if(!backtrack_stack.push(264)) return -1;
			if(!backtrack_stack.push(263)) return -1;
			if(!backtrack_stack.push(262)) return -1;
			if(!backtrack_stack.push(261)) return -1;
			if(!backtrack_stack.push(260)) return -1;
			if(!backtrack_stack.push(254)) return -1;
			if(!backtrack_stack.push(253)) return -1;
			if(!backtrack_stack.push(152)) return -1;
			if(!backtrack_stack.push(142)) return -1;
			if(!backtrack_stack.push(141)) return -1;
			if(!backtrack_stack.push(140)) return -1;
			if(!backtrack_stack.push(139)) return -1;
			if(!backtrack_stack.push(138)) return -1;
			if(!backtrack_stack.push(137)) return -1;
			if(!backtrack_stack.push(134)) return -1;
			if(!backtrack_stack.push(133)) return -1;
			if(!backtrack_stack.push(127)) return -1;
			if(!backtrack_stack.push(126)) return -1;
			if(!backtrack_stack.push(125)) return -1;
			if(!backtrack_stack.push(124)) return -1;
			if(!backtrack_stack.push(85)) return -1;
			if(!backtrack_stack.push(82)) return -1;
			if(!backtrack_stack.push(81)) return -1;
			if(!backtrack_stack.push(73)) return -1;
			if(!backtrack_stack.push(72)) return -1;
			if(!backtrack_stack.push(69)) return -1;
			if(!backtrack_stack.push(62)) return -1;
			if(!backtrack_stack.push(57)) return -1;
			if(!backtrack_stack.push(56)) return -1;
			if(!backtrack_stack.push(55)) return -1;
			if(!backtrack_stack.push(45)) return -1;
			if(!backtrack_stack.push(42)) return -1;
			if(!backtrack_stack.push(41)) return -1;
			if(!backtrack_stack.push(35)) return -1;
			if(!backtrack_stack.push(34)) return -1;
			if(!backtrack_stack.push(33)) return -1;
			if(!backtrack_stack.push(29)) return -1;
			if(!backtrack_stack.push(28)) return -1;
			if(!backtrack_stack.push(21)) return -1;
			if(!backtrack_stack.push(18)) return -1;
			if(!backtrack_stack.push(13)) return -1;
			if(!backtrack_stack.push(5)) return -1;
			if(!backtrack_stack.push(4)) return -1;
			goto label_3;
		}
		}
		break;
	case 2066:
	// Label to restore values of label 40
		v_kaon2equal_2 = int_value_stack.pop();
		v_kaon2equal_1 = int_value_stack.pop();
		c61_1_bf.pop(&cursor_stack);
		break;
	case 41: label_41: 
	// using  symbolic fact kaon2equal(v_kaon2equal_1,v_kaon2equal_2) with literal 2 in rule 
	// 793: hasmaker(A,B) :- hasmaker(A,C), kaon2equal(B,C).
	// resulting in symbolic fact hasmaker(v_hasmaker_1,v_hasmaker_2)

		//recursive
		c793_1_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_hasmaker_1)) return -1;
		if(!int_value_stack.push(v_hasmaker_2)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2067)) return -1;
		c793_1_fb.open(v_kaon2equal_2);
	case 1054:
	// Continuation for label 41
		while (c793_1_fb.fetch()) {
			v_hasmaker_2 = v_kaon2equal_1;
			v_hasmaker_1 = c793_1_fb.c2();
		if(hasmaker_set.insert(v_hasmaker_1, v_hasmaker_2)) {
			if(!hasmaker_bf.insert(v_hasmaker_1, v_hasmaker_2)) return -1;
			if(!hasmaker_fb.insert(v_hasmaker_2, v_hasmaker_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1054)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(40)) return -1;
			if(!backtrack_stack.push(39)) return -1;
			if(!backtrack_stack.push(38)) return -1;
			if(!backtrack_stack.push(36)) return -1;
			goto label_20;
		}
		}
		break;
	case 2067:
	// Label to restore values of label 41
		v_hasmaker_2 = int_value_stack.pop();
		v_hasmaker_1 = int_value_stack.pop();
		c793_1_fb.pop(&cursor_stack);
		break;
	case 42: label_42: 
	// using  symbolic fact kaon2equal(v_kaon2equal_1,v_kaon2equal_2) with literal 2 in rule 
	// 794: produceswine(A,B) :- produceswine(C,B), kaon2equal(A,C).
	// resulting in symbolic fact produceswine(v_produceswine_1,v_produceswine_2)

		//recursive
		c794_1_bf.push(&cursor_stack);
		if(!int_value_stack.push(v_produceswine_2)) return -1;
		if(!int_value_stack.push(v_produceswine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2068)) return -1;
		c794_1_bf.open(v_kaon2equal_2);
	case 1055:
	// Continuation for label 42
		while (c794_1_bf.fetch()) {
			v_produceswine_1 = v_kaon2equal_1;
			v_produceswine_2 = c794_1_bf.c2();
		if(produceswine_set.insert(v_produceswine_1, v_produceswine_2)) {
			if(!produceswine_bf.insert(v_produceswine_1, v_produceswine_2)) return -1;
			if(!produceswine_fb.insert(v_produceswine_2, v_produceswine_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1055)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(44)) return -1;
			if(!backtrack_stack.push(43)) return -1;
			goto label_37;
		}
		}
		break;
	case 2068:
	// Label to restore values of label 42
		v_produceswine_1 = int_value_stack.pop();
		v_produceswine_2 = int_value_stack.pop();
		c794_1_bf.pop(&cursor_stack);
		break;
	case 43: label_43: 
	// using  symbolic fact produceswine(v_produceswine_1,v_produceswine_2) with literal 1 in rule 
	// 794: produceswine(A,B) :- produceswine(C,B), kaon2equal(A,C).
	// resulting in symbolic fact produceswine(v_produceswine_1,v_produceswine_2)

		//recursive
		c794_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_produceswine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2069)) return -1;
		c794_2_fb.open(v_produceswine_1);
	case 1056:
	// Continuation for label 43
		while (c794_2_fb.fetch()) {
			v_produceswine_1 = c794_2_fb.c2();
		if(produceswine_set.insert(v_produceswine_1, v_produceswine_2)) {
			if(!produceswine_bf.insert(v_produceswine_1, v_produceswine_2)) return -1;
			if(!produceswine_fb.insert(v_produceswine_2, v_produceswine_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1056)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(44)) return -1;
			if(!backtrack_stack.push(43)) return -1;
			goto label_37;
		}
		}
		break;
	case 2069:
	// Label to restore values of label 43
		v_produceswine_1 = int_value_stack.pop();
		c794_2_fb.pop(&cursor_stack);
		break;
	case 44: label_44: 
	// using  symbolic fact produceswine(v_produceswine_1,v_produceswine_2) with literal 1 in rule 
	// 795: produceswine(A,B) :- produceswine(A,C), kaon2equal(B,C).
	// resulting in symbolic fact produceswine(v_produceswine_1,v_produceswine_2)

		//recursive
		c795_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_produceswine_2)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2070)) return -1;
		c795_2_fb.open(v_produceswine_2);
	case 1057:
	// Continuation for label 44
		while (c795_2_fb.fetch()) {
			v_produceswine_2 = c795_2_fb.c2();
		if(produceswine_set.insert(v_produceswine_1, v_produceswine_2)) {
			if(!produceswine_bf.insert(v_produceswine_1, v_produceswine_2)) return -1;
			if(!produceswine_fb.insert(v_produceswine_2, v_produceswine_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1057)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(44)) return -1;
			if(!backtrack_stack.push(43)) return -1;
			goto label_37;
		}
		}
		break;
	case 2070:
	// Label to restore values of label 44
		v_produceswine_2 = int_value_stack.pop();
		c795_2_fb.pop(&cursor_stack);
		break;
	case 45: label_45: 
	// using  symbolic fact kaon2equal(v_kaon2equal_1,v_kaon2equal_2) with literal 2 in rule 
	// 795: produceswine(A,B) :- produceswine(A,C), kaon2equal(B,C).
	// resulting in symbolic fact produceswine(v_produceswine_1,v_produceswine_2)

		//recursive
		c795_1_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_produceswine_1)) return -1;
		if(!int_value_stack.push(v_produceswine_2)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2071)) return -1;
		c795_1_fb.open(v_kaon2equal_2);
	case 1058:
	// Continuation for label 45
		while (c795_1_fb.fetch()) {
			v_produceswine_2 = v_kaon2equal_1;
			v_produceswine_1 = c795_1_fb.c2();
		if(produceswine_set.insert(v_produceswine_1, v_produceswine_2)) {
			if(!produceswine_bf.insert(v_produceswine_1, v_produceswine_2)) return -1;
			if(!produceswine_fb.insert(v_produceswine_2, v_produceswine_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1058)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(44)) return -1;
			if(!backtrack_stack.push(43)) return -1;
			goto label_37;
		}
		}
		break;
	case 2071:
	// Label to restore values of label 45
		v_produceswine_2 = int_value_stack.pop();
		v_produceswine_1 = int_value_stack.pop();
		c795_1_fb.pop(&cursor_stack);
		break;
	case 46: label_46: 
	// using  symbolic fact wine(v_wine_1) with literal 1 in rule 
	// 134: locatedin(A,A) :- wine(A), kaon2namedobjects(A).
	// resulting in symbolic fact locatedin(v_locatedin_1,v_locatedin_2)

		if (!kaon2namedobjects_b.contains(v_wine_1)) continue;
	case 1059:
	// Continuation for label 46
			v_locatedin_1 = v_wine_1;
			v_locatedin_2 = v_wine_1;
		if(locatedin_set.insert(v_locatedin_1, v_locatedin_2)) {
			if(!locatedin_bf.insert(v_locatedin_1, v_locatedin_2)) return -1;
			if(!locatedin_fb.insert(v_locatedin_2, v_locatedin_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1059)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(1005)) return -1;
			if(!backtrack_stack.push(999)) return -1;
			if(!backtrack_stack.push(987)) return -1;
			if(!backtrack_stack.push(981)) return -1;
			if(!backtrack_stack.push(969)) return -1;
			if(!backtrack_stack.push(963)) return -1;
			if(!backtrack_stack.push(952)) return -1;
			if(!backtrack_stack.push(946)) return -1;
			if(!backtrack_stack.push(936)) return -1;
			if(!backtrack_stack.push(930)) return -1;
			if(!backtrack_stack.push(920)) return -1;
			if(!backtrack_stack.push(917)) return -1;
			if(!backtrack_stack.push(898)) return -1;
			if(!backtrack_stack.push(897)) return -1;
			if(!backtrack_stack.push(896)) return -1;
			if(!backtrack_stack.push(893)) return -1;
			if(!backtrack_stack.push(726)) return -1;
			if(!backtrack_stack.push(721)) return -1;
			if(!backtrack_stack.push(694)) return -1;
			if(!backtrack_stack.push(693)) return -1;
			if(!backtrack_stack.push(688)) return -1;
			if(!backtrack_stack.push(687)) return -1;
			if(!backtrack_stack.push(636)) return -1;
			if(!backtrack_stack.push(631)) return -1;
			if(!backtrack_stack.push(592)) return -1;
			if(!backtrack_stack.push(591)) return -1;
			if(!backtrack_stack.push(529)) return -1;
			if(!backtrack_stack.push(528)) return -1;
			if(!backtrack_stack.push(513)) return -1;
			if(!backtrack_stack.push(512)) return -1;
			if(!backtrack_stack.push(511)) return -1;
			if(!backtrack_stack.push(506)) return -1;
			if(!backtrack_stack.push(455)) return -1;
			if(!backtrack_stack.push(450)) return -1;
			if(!backtrack_stack.push(441)) return -1;
			if(!backtrack_stack.push(440)) return -1;
			if(!backtrack_stack.push(439)) return -1;
			if(!backtrack_stack.push(434)) return -1;
			if(!backtrack_stack.push(339)) return -1;
			if(!backtrack_stack.push(338)) return -1;
			if(!backtrack_stack.push(249)) return -1;
			if(!backtrack_stack.push(244)) return -1;
			if(!backtrack_stack.push(52)) return -1;
			if(!backtrack_stack.push(51)) return -1;
			if(!backtrack_stack.push(50)) return -1;
			if(!backtrack_stack.push(49)) return -1;
			if(!backtrack_stack.push(48)) return -1;
			goto label_47;
		}
		break;
	case 47: label_47: 
	// using  symbolic fact locatedin(v_locatedin_1,v_locatedin_2) with literal 1 in rule 
	// 218: locatedin(A,B) :- locatedin(C,B), kaon2equal(A,C).
	// resulting in symbolic fact locatedin(v_locatedin_1,v_locatedin_2)

		//recursive
		c218_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_locatedin_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2073)) return -1;
		c218_2_fb.open(v_locatedin_1);
	case 1060:
	// Continuation for label 47
		while (c218_2_fb.fetch()) {
			v_locatedin_1 = c218_2_fb.c2();
		if(locatedin_set.insert(v_locatedin_1, v_locatedin_2)) {
			if(!locatedin_bf.insert(v_locatedin_1, v_locatedin_2)) return -1;
			if(!locatedin_fb.insert(v_locatedin_2, v_locatedin_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1060)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(1005)) return -1;
			if(!backtrack_stack.push(999)) return -1;
			if(!backtrack_stack.push(987)) return -1;
			if(!backtrack_stack.push(981)) return -1;
			if(!backtrack_stack.push(969)) return -1;
			if(!backtrack_stack.push(963)) return -1;
			if(!backtrack_stack.push(952)) return -1;
			if(!backtrack_stack.push(946)) return -1;
			if(!backtrack_stack.push(936)) return -1;
			if(!backtrack_stack.push(930)) return -1;
			if(!backtrack_stack.push(920)) return -1;
			if(!backtrack_stack.push(917)) return -1;
			if(!backtrack_stack.push(898)) return -1;
			if(!backtrack_stack.push(897)) return -1;
			if(!backtrack_stack.push(896)) return -1;
			if(!backtrack_stack.push(893)) return -1;
			if(!backtrack_stack.push(726)) return -1;
			if(!backtrack_stack.push(721)) return -1;
			if(!backtrack_stack.push(694)) return -1;
			if(!backtrack_stack.push(693)) return -1;
			if(!backtrack_stack.push(688)) return -1;
			if(!backtrack_stack.push(687)) return -1;
			if(!backtrack_stack.push(636)) return -1;
			if(!backtrack_stack.push(631)) return -1;
			if(!backtrack_stack.push(592)) return -1;
			if(!backtrack_stack.push(591)) return -1;
			if(!backtrack_stack.push(529)) return -1;
			if(!backtrack_stack.push(528)) return -1;
			if(!backtrack_stack.push(513)) return -1;
			if(!backtrack_stack.push(512)) return -1;
			if(!backtrack_stack.push(511)) return -1;
			if(!backtrack_stack.push(506)) return -1;
			if(!backtrack_stack.push(455)) return -1;
			if(!backtrack_stack.push(450)) return -1;
			if(!backtrack_stack.push(441)) return -1;
			if(!backtrack_stack.push(440)) return -1;
			if(!backtrack_stack.push(439)) return -1;
			if(!backtrack_stack.push(434)) return -1;
			if(!backtrack_stack.push(339)) return -1;
			if(!backtrack_stack.push(338)) return -1;
			if(!backtrack_stack.push(249)) return -1;
			if(!backtrack_stack.push(244)) return -1;
			if(!backtrack_stack.push(52)) return -1;
			if(!backtrack_stack.push(51)) return -1;
			if(!backtrack_stack.push(50)) return -1;
			if(!backtrack_stack.push(49)) return -1;
			if(!backtrack_stack.push(48)) return -1;
			goto label_47;
		}
		}
		break;
	case 2073:
	// Label to restore values of label 47
		v_locatedin_1 = int_value_stack.pop();
		c218_2_fb.pop(&cursor_stack);
		break;
	case 48: label_48: 
	// using  symbolic fact locatedin(v_locatedin_1,v_locatedin_2) with literal 1 in rule 
	// 219: locatedin(A,B) :- locatedin(A,C), kaon2equal(B,C).
	// resulting in symbolic fact locatedin(v_locatedin_1,v_locatedin_2)

		//recursive
		c219_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_locatedin_2)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2074)) return -1;
		c219_2_fb.open(v_locatedin_2);
	case 1061:
	// Continuation for label 48
		while (c219_2_fb.fetch()) {
			v_locatedin_2 = c219_2_fb.c2();
		if(locatedin_set.insert(v_locatedin_1, v_locatedin_2)) {
			if(!locatedin_bf.insert(v_locatedin_1, v_locatedin_2)) return -1;
			if(!locatedin_fb.insert(v_locatedin_2, v_locatedin_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1061)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(1005)) return -1;
			if(!backtrack_stack.push(999)) return -1;
			if(!backtrack_stack.push(987)) return -1;
			if(!backtrack_stack.push(981)) return -1;
			if(!backtrack_stack.push(969)) return -1;
			if(!backtrack_stack.push(963)) return -1;
			if(!backtrack_stack.push(952)) return -1;
			if(!backtrack_stack.push(946)) return -1;
			if(!backtrack_stack.push(936)) return -1;
			if(!backtrack_stack.push(930)) return -1;
			if(!backtrack_stack.push(920)) return -1;
			if(!backtrack_stack.push(917)) return -1;
			if(!backtrack_stack.push(898)) return -1;
			if(!backtrack_stack.push(897)) return -1;
			if(!backtrack_stack.push(896)) return -1;
			if(!backtrack_stack.push(893)) return -1;
			if(!backtrack_stack.push(726)) return -1;
			if(!backtrack_stack.push(721)) return -1;
			if(!backtrack_stack.push(694)) return -1;
			if(!backtrack_stack.push(693)) return -1;
			if(!backtrack_stack.push(688)) return -1;
			if(!backtrack_stack.push(687)) return -1;
			if(!backtrack_stack.push(636)) return -1;
			if(!backtrack_stack.push(631)) return -1;
			if(!backtrack_stack.push(592)) return -1;
			if(!backtrack_stack.push(591)) return -1;
			if(!backtrack_stack.push(529)) return -1;
			if(!backtrack_stack.push(528)) return -1;
			if(!backtrack_stack.push(513)) return -1;
			if(!backtrack_stack.push(512)) return -1;
			if(!backtrack_stack.push(511)) return -1;
			if(!backtrack_stack.push(506)) return -1;
			if(!backtrack_stack.push(455)) return -1;
			if(!backtrack_stack.push(450)) return -1;
			if(!backtrack_stack.push(441)) return -1;
			if(!backtrack_stack.push(440)) return -1;
			if(!backtrack_stack.push(439)) return -1;
			if(!backtrack_stack.push(434)) return -1;
			if(!backtrack_stack.push(339)) return -1;
			if(!backtrack_stack.push(338)) return -1;
			if(!backtrack_stack.push(249)) return -1;
			if(!backtrack_stack.push(244)) return -1;
			if(!backtrack_stack.push(52)) return -1;
			if(!backtrack_stack.push(51)) return -1;
			if(!backtrack_stack.push(50)) return -1;
			if(!backtrack_stack.push(49)) return -1;
			if(!backtrack_stack.push(48)) return -1;
			goto label_47;
		}
		}
		break;
	case 2074:
	// Label to restore values of label 48
		v_locatedin_2 = int_value_stack.pop();
		c219_2_fb.pop(&cursor_stack);
		break;
	case 49: label_49: 
	// using  symbolic fact locatedin(v_locatedin_1,v_locatedin_2) with literal 1 in rule 
	// 220: locatedin(A,B) :- locatedin(A,C), locatedin(C,B).
	// resulting in symbolic fact locatedin(v_locatedin_1,v_locatedin_2)

		//recursive
		c220_2_bf.push(&cursor_stack);
		if(!int_value_stack.push(v_locatedin_2)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2075)) return -1;
		c220_2_bf.open(v_locatedin_2);
	case 1062:
	// Continuation for label 49
		while (c220_2_bf.fetch()) {
			v_locatedin_2 = c220_2_bf.c2();
		if(locatedin_set.insert(v_locatedin_1, v_locatedin_2)) {
			if(!locatedin_bf.insert(v_locatedin_1, v_locatedin_2)) return -1;
			if(!locatedin_fb.insert(v_locatedin_2, v_locatedin_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1062)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(1005)) return -1;
			if(!backtrack_stack.push(999)) return -1;
			if(!backtrack_stack.push(987)) return -1;
			if(!backtrack_stack.push(981)) return -1;
			if(!backtrack_stack.push(969)) return -1;
			if(!backtrack_stack.push(963)) return -1;
			if(!backtrack_stack.push(952)) return -1;
			if(!backtrack_stack.push(946)) return -1;
			if(!backtrack_stack.push(936)) return -1;
			if(!backtrack_stack.push(930)) return -1;
			if(!backtrack_stack.push(920)) return -1;
			if(!backtrack_stack.push(917)) return -1;
			if(!backtrack_stack.push(898)) return -1;
			if(!backtrack_stack.push(897)) return -1;
			if(!backtrack_stack.push(896)) return -1;
			if(!backtrack_stack.push(893)) return -1;
			if(!backtrack_stack.push(726)) return -1;
			if(!backtrack_stack.push(721)) return -1;
			if(!backtrack_stack.push(694)) return -1;
			if(!backtrack_stack.push(693)) return -1;
			if(!backtrack_stack.push(688)) return -1;
			if(!backtrack_stack.push(687)) return -1;
			if(!backtrack_stack.push(636)) return -1;
			if(!backtrack_stack.push(631)) return -1;
			if(!backtrack_stack.push(592)) return -1;
			if(!backtrack_stack.push(591)) return -1;
			if(!backtrack_stack.push(529)) return -1;
			if(!backtrack_stack.push(528)) return -1;
			if(!backtrack_stack.push(513)) return -1;
			if(!backtrack_stack.push(512)) return -1;
			if(!backtrack_stack.push(511)) return -1;
			if(!backtrack_stack.push(506)) return -1;
			if(!backtrack_stack.push(455)) return -1;
			if(!backtrack_stack.push(450)) return -1;
			if(!backtrack_stack.push(441)) return -1;
			if(!backtrack_stack.push(440)) return -1;
			if(!backtrack_stack.push(439)) return -1;
			if(!backtrack_stack.push(434)) return -1;
			if(!backtrack_stack.push(339)) return -1;
			if(!backtrack_stack.push(338)) return -1;
			if(!backtrack_stack.push(249)) return -1;
			if(!backtrack_stack.push(244)) return -1;
			if(!backtrack_stack.push(52)) return -1;
			if(!backtrack_stack.push(51)) return -1;
			if(!backtrack_stack.push(50)) return -1;
			if(!backtrack_stack.push(49)) return -1;
			if(!backtrack_stack.push(48)) return -1;
			goto label_47;
		}
		}
		break;
	case 2075:
	// Label to restore values of label 49
		v_locatedin_2 = int_value_stack.pop();
		c220_2_bf.pop(&cursor_stack);
		break;
	case 50: label_50: 
	// using  symbolic fact locatedin(v_locatedin_1,v_locatedin_2) with literal 1 in rule 
	// 221: locatedin(A,B) :- locatedin(A,C), locatedin(C,B), kaon2namedobjects(B), kaon2namedobjects(C), kaon2namedobjects(A).
	// resulting in symbolic fact locatedin(v_locatedin_1,v_locatedin_2)

		//recursive
		c221_2_bf.push(&cursor_stack);
		if(!int_value_stack.push(v_locatedin_2)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2076)) return -1;
		c221_2_bf.open(v_locatedin_2);
		while (c221_2_bf.fetch()) {
			if (!kaon2namedobjects_b.contains(c221_2_bf.c2())) continue;
				if (!kaon2namedobjects_b.contains(v_locatedin_2)) continue;
					if (!kaon2namedobjects_b.contains(v_locatedin_1)) continue;
	case 1063:
	// Continuation for label 50
						v_locatedin_2 = c221_2_bf.c2();
		if(locatedin_set.insert(v_locatedin_1, v_locatedin_2)) {
						if(!locatedin_bf.insert(v_locatedin_1, v_locatedin_2)) return -1;
						if(!locatedin_fb.insert(v_locatedin_2, v_locatedin_1)) return -1;
						// Push backtrack label to continuation
						if(!backtrack_stack.push(1063)) return -1;
						// Labels for next rules
						if(!backtrack_stack.push(1005)) return -1;
						if(!backtrack_stack.push(999)) return -1;
						if(!backtrack_stack.push(987)) return -1;
						if(!backtrack_stack.push(981)) return -1;
						if(!backtrack_stack.push(969)) return -1;
						if(!backtrack_stack.push(963)) return -1;
						if(!backtrack_stack.push(952)) return -1;
						if(!backtrack_stack.push(946)) return -1;
						if(!backtrack_stack.push(936)) return -1;
						if(!backtrack_stack.push(930)) return -1;
						if(!backtrack_stack.push(920)) return -1;
						if(!backtrack_stack.push(917)) return -1;
						if(!backtrack_stack.push(898)) return -1;
						if(!backtrack_stack.push(897)) return -1;
						if(!backtrack_stack.push(896)) return -1;
						if(!backtrack_stack.push(893)) return -1;
						if(!backtrack_stack.push(726)) return -1;
						if(!backtrack_stack.push(721)) return -1;
						if(!backtrack_stack.push(694)) return -1;
						if(!backtrack_stack.push(693)) return -1;
						if(!backtrack_stack.push(688)) return -1;
						if(!backtrack_stack.push(687)) return -1;
						if(!backtrack_stack.push(636)) return -1;
						if(!backtrack_stack.push(631)) return -1;
						if(!backtrack_stack.push(592)) return -1;
						if(!backtrack_stack.push(591)) return -1;
						if(!backtrack_stack.push(529)) return -1;
						if(!backtrack_stack.push(528)) return -1;
						if(!backtrack_stack.push(513)) return -1;
						if(!backtrack_stack.push(512)) return -1;
						if(!backtrack_stack.push(511)) return -1;
						if(!backtrack_stack.push(506)) return -1;
						if(!backtrack_stack.push(455)) return -1;
						if(!backtrack_stack.push(450)) return -1;
						if(!backtrack_stack.push(441)) return -1;
						if(!backtrack_stack.push(440)) return -1;
						if(!backtrack_stack.push(439)) return -1;
						if(!backtrack_stack.push(434)) return -1;
						if(!backtrack_stack.push(339)) return -1;
						if(!backtrack_stack.push(338)) return -1;
						if(!backtrack_stack.push(249)) return -1;
						if(!backtrack_stack.push(244)) return -1;
						if(!backtrack_stack.push(52)) return -1;
						if(!backtrack_stack.push(51)) return -1;
						if(!backtrack_stack.push(50)) return -1;
						if(!backtrack_stack.push(49)) return -1;
						if(!backtrack_stack.push(48)) return -1;
						goto label_47;
		}
		}
		break;
	case 2076:
	// Label to restore values of label 50
		v_locatedin_2 = int_value_stack.pop();
		c221_2_bf.pop(&cursor_stack);
		break;
	case 51: label_51: 
	// using  symbolic fact locatedin(v_locatedin_1,v_locatedin_2) with literal 2 in rule 
	// 220: locatedin(A,B) :- locatedin(A,C), locatedin(C,B).
	// resulting in symbolic fact locatedin(v_locatedin_1,v_locatedin_2)

		//recursive
		c220_1_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_locatedin_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2077)) return -1;
		c220_1_fb.open(v_locatedin_1);
	case 1064:
	// Continuation for label 51
		while (c220_1_fb.fetch()) {
			v_locatedin_1 = c220_1_fb.c2();
		if(locatedin_set.insert(v_locatedin_1, v_locatedin_2)) {
			if(!locatedin_bf.insert(v_locatedin_1, v_locatedin_2)) return -1;
			if(!locatedin_fb.insert(v_locatedin_2, v_locatedin_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1064)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(1005)) return -1;
			if(!backtrack_stack.push(999)) return -1;
			if(!backtrack_stack.push(987)) return -1;
			if(!backtrack_stack.push(981)) return -1;
			if(!backtrack_stack.push(969)) return -1;
			if(!backtrack_stack.push(963)) return -1;
			if(!backtrack_stack.push(952)) return -1;
			if(!backtrack_stack.push(946)) return -1;
			if(!backtrack_stack.push(936)) return -1;
			if(!backtrack_stack.push(930)) return -1;
			if(!backtrack_stack.push(920)) return -1;
			if(!backtrack_stack.push(917)) return -1;
			if(!backtrack_stack.push(898)) return -1;
			if(!backtrack_stack.push(897)) return -1;
			if(!backtrack_stack.push(896)) return -1;
			if(!backtrack_stack.push(893)) return -1;
			if(!backtrack_stack.push(726)) return -1;
			if(!backtrack_stack.push(721)) return -1;
			if(!backtrack_stack.push(694)) return -1;
			if(!backtrack_stack.push(693)) return -1;
			if(!backtrack_stack.push(688)) return -1;
			if(!backtrack_stack.push(687)) return -1;
			if(!backtrack_stack.push(636)) return -1;
			if(!backtrack_stack.push(631)) return -1;
			if(!backtrack_stack.push(592)) return -1;
			if(!backtrack_stack.push(591)) return -1;
			if(!backtrack_stack.push(529)) return -1;
			if(!backtrack_stack.push(528)) return -1;
			if(!backtrack_stack.push(513)) return -1;
			if(!backtrack_stack.push(512)) return -1;
			if(!backtrack_stack.push(511)) return -1;
			if(!backtrack_stack.push(506)) return -1;
			if(!backtrack_stack.push(455)) return -1;
			if(!backtrack_stack.push(450)) return -1;
			if(!backtrack_stack.push(441)) return -1;
			if(!backtrack_stack.push(440)) return -1;
			if(!backtrack_stack.push(439)) return -1;
			if(!backtrack_stack.push(434)) return -1;
			if(!backtrack_stack.push(339)) return -1;
			if(!backtrack_stack.push(338)) return -1;
			if(!backtrack_stack.push(249)) return -1;
			if(!backtrack_stack.push(244)) return -1;
			if(!backtrack_stack.push(52)) return -1;
			if(!backtrack_stack.push(51)) return -1;
			if(!backtrack_stack.push(50)) return -1;
			if(!backtrack_stack.push(49)) return -1;
			if(!backtrack_stack.push(48)) return -1;
			goto label_47;
		}
		}
		break;
	case 2077:
	// Label to restore values of label 51
		v_locatedin_1 = int_value_stack.pop();
		c220_1_fb.pop(&cursor_stack);
		break;
	case 52: label_52: 
	// using  symbolic fact locatedin(v_locatedin_1,v_locatedin_2) with literal 2 in rule 
	// 221: locatedin(A,B) :- locatedin(A,C), locatedin(C,B), kaon2namedobjects(B), kaon2namedobjects(C), kaon2namedobjects(A).
	// resulting in symbolic fact locatedin(v_locatedin_1,v_locatedin_2)

		//recursive
		c221_1_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_locatedin_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2078)) return -1;
		c221_1_fb.open(v_locatedin_1);
		while (c221_1_fb.fetch()) {
			if (!kaon2namedobjects_b.contains(v_locatedin_2)) continue;
				if (!kaon2namedobjects_b.contains(v_locatedin_1)) continue;
					if (!kaon2namedobjects_b.contains(c221_1_fb.c2())) continue;
	case 1065:
	// Continuation for label 52
						v_locatedin_1 = c221_1_fb.c2();
		if(locatedin_set.insert(v_locatedin_1, v_locatedin_2)) {
						if(!locatedin_bf.insert(v_locatedin_1, v_locatedin_2)) return -1;
						if(!locatedin_fb.insert(v_locatedin_2, v_locatedin_1)) return -1;
						// Push backtrack label to continuation
						if(!backtrack_stack.push(1065)) return -1;
						// Labels for next rules
						if(!backtrack_stack.push(1005)) return -1;
						if(!backtrack_stack.push(999)) return -1;
						if(!backtrack_stack.push(987)) return -1;
						if(!backtrack_stack.push(981)) return -1;
						if(!backtrack_stack.push(969)) return -1;
						if(!backtrack_stack.push(963)) return -1;
						if(!backtrack_stack.push(952)) return -1;
						if(!backtrack_stack.push(946)) return -1;
						if(!backtrack_stack.push(936)) return -1;
						if(!backtrack_stack.push(930)) return -1;
						if(!backtrack_stack.push(920)) return -1;
						if(!backtrack_stack.push(917)) return -1;
						if(!backtrack_stack.push(898)) return -1;
						if(!backtrack_stack.push(897)) return -1;
						if(!backtrack_stack.push(896)) return -1;
						if(!backtrack_stack.push(893)) return -1;
						if(!backtrack_stack.push(726)) return -1;
						if(!backtrack_stack.push(721)) return -1;
						if(!backtrack_stack.push(694)) return -1;
						if(!backtrack_stack.push(693)) return -1;
						if(!backtrack_stack.push(688)) return -1;
						if(!backtrack_stack.push(687)) return -1;
						if(!backtrack_stack.push(636)) return -1;
						if(!backtrack_stack.push(631)) return -1;
						if(!backtrack_stack.push(592)) return -1;
						if(!backtrack_stack.push(591)) return -1;
						if(!backtrack_stack.push(529)) return -1;
						if(!backtrack_stack.push(528)) return -1;
						if(!backtrack_stack.push(513)) return -1;
						if(!backtrack_stack.push(512)) return -1;
						if(!backtrack_stack.push(511)) return -1;
						if(!backtrack_stack.push(506)) return -1;
						if(!backtrack_stack.push(455)) return -1;
						if(!backtrack_stack.push(450)) return -1;
						if(!backtrack_stack.push(441)) return -1;
						if(!backtrack_stack.push(440)) return -1;
						if(!backtrack_stack.push(439)) return -1;
						if(!backtrack_stack.push(434)) return -1;
						if(!backtrack_stack.push(339)) return -1;
						if(!backtrack_stack.push(338)) return -1;
						if(!backtrack_stack.push(249)) return -1;
						if(!backtrack_stack.push(244)) return -1;
						if(!backtrack_stack.push(52)) return -1;
						if(!backtrack_stack.push(51)) return -1;
						if(!backtrack_stack.push(50)) return -1;
						if(!backtrack_stack.push(49)) return -1;
						if(!backtrack_stack.push(48)) return -1;
						goto label_47;
		}
		}
		break;
	case 2078:
	// Label to restore values of label 52
		v_locatedin_1 = int_value_stack.pop();
		c221_1_fb.pop(&cursor_stack);
		break;
	case 53: label_53: 
	// using  symbolic fact wine(v_wine_1) with literal 1 in rule 
	// 268: hasbody(A,A) :- wine(A), kaon2namedobjects(A).
	// resulting in symbolic fact hasbody(v_hasbody_1,v_hasbody_2)

		//recursive
		if(!int_value_stack.push(v_hasbody_2)) return -1;
		if(!int_value_stack.push(v_hasbody_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2079)) return -1;
		if (!kaon2namedobjects_b.contains(v_wine_1)) continue;
	case 1066:
	// Continuation for label 53
			v_hasbody_1 = v_wine_1;
			v_hasbody_2 = v_wine_1;
		if(hasbody_set.insert(v_hasbody_1, v_hasbody_2)) {
			if(!hasbody_bf.insert(v_hasbody_1, v_hasbody_2)) return -1;
			if(!hasbody_fb.insert(v_hasbody_2, v_hasbody_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1066)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(429)) return -1;
			if(!backtrack_stack.push(61)) return -1;
			if(!backtrack_stack.push(60)) return -1;
			if(!backtrack_stack.push(59)) return -1;
			if(!backtrack_stack.push(58)) return -1;
			goto label_54;
		}
		break;
	case 2079:
	// Label to restore values of label 53
		v_hasbody_1 = int_value_stack.pop();
		v_hasbody_2 = int_value_stack.pop();
		break;
	case 54: label_54: 
	// using  symbolic fact hasbody(v_hasbody_1,v_hasbody_2) with literal 1 in rule 
	// 78: kaon2equal(A,B) :- hasbody(C,A), hasbody(C,B).
	// resulting in symbolic fact kaon2equal(v_kaon2equal_1,v_kaon2equal_2)

		//recursive
		c78_2_bf.push(&cursor_stack);
		if(!int_value_stack.push(v_kaon2equal_2)) return -1;
		if(!int_value_stack.push(v_kaon2equal_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2080)) return -1;
		c78_2_bf.open(v_hasbody_1);
	case 1067:
	// Continuation for label 54
		while (c78_2_bf.fetch()) {
			v_kaon2equal_1 = v_hasbody_2;
			v_kaon2equal_2 = c78_2_bf.c2();
		if(kaon2equal_set.insert(v_kaon2equal_1, v_kaon2equal_2)) {
			if(!kaon2equal_bf.insert(v_kaon2equal_1, v_kaon2equal_2)) return -1;
			if(!kaon2equal_fb.insert(v_kaon2equal_2, v_kaon2equal_1)) return -1;
			if(!kaon2equal_ff.insert(v_kaon2equal_1, v_kaon2equal_2)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1067)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(283)) return -1;
			if(!backtrack_stack.push(282)) return -1;
			if(!backtrack_stack.push(281)) return -1;
			if(!backtrack_stack.push(280)) return -1;
			if(!backtrack_stack.push(279)) return -1;
			if(!backtrack_stack.push(278)) return -1;
			if(!backtrack_stack.push(277)) return -1;
			if(!backtrack_stack.push(276)) return -1;
			if(!backtrack_stack.push(275)) return -1;
			if(!backtrack_stack.push(274)) return -1;
			if(!backtrack_stack.push(273)) return -1;
			if(!backtrack_stack.push(271)) return -1;
			if(!backtrack_stack.push(270)) return -1;
			if(!backtrack_stack.push(269)) return -1;
			if(!backtrack_stack.push(268)) return -1;
			if(!backtrack_stack.push(267)) return -1;
			if(!backtrack_stack.push(266)) return -1;
			if(!backtrack_stack.push(265)) return -1;
			if(!backtrack_stack.push(264)) return -1;
			if(!backtrack_stack.push(263)) return -1;
			if(!backtrack_stack.push(262)) return -1;
			if(!backtrack_stack.push(261)) return -1;
			if(!backtrack_stack.push(260)) return -1;
			if(!backtrack_stack.push(254)) return -1;
			if(!backtrack_stack.push(253)) return -1;
			if(!backtrack_stack.push(152)) return -1;
			if(!backtrack_stack.push(142)) return -1;
			if(!backtrack_stack.push(141)) return -1;
			if(!backtrack_stack.push(140)) return -1;
			if(!backtrack_stack.push(139)) return -1;
			if(!backtrack_stack.push(138)) return -1;
			if(!backtrack_stack.push(137)) return -1;
			if(!backtrack_stack.push(134)) return -1;
			if(!backtrack_stack.push(133)) return -1;
			if(!backtrack_stack.push(127)) return -1;
			if(!backtrack_stack.push(126)) return -1;
			if(!backtrack_stack.push(125)) return -1;
			if(!backtrack_stack.push(124)) return -1;
			if(!backtrack_stack.push(85)) return -1;
			if(!backtrack_stack.push(82)) return -1;
			if(!backtrack_stack.push(81)) return -1;
			if(!backtrack_stack.push(73)) return -1;
			if(!backtrack_stack.push(72)) return -1;
			if(!backtrack_stack.push(69)) return -1;
			if(!backtrack_stack.push(62)) return -1;
			if(!backtrack_stack.push(57)) return -1;
			if(!backtrack_stack.push(56)) return -1;
			if(!backtrack_stack.push(55)) return -1;
			if(!backtrack_stack.push(45)) return -1;
			if(!backtrack_stack.push(42)) return -1;
			if(!backtrack_stack.push(41)) return -1;
			if(!backtrack_stack.push(35)) return -1;
			if(!backtrack_stack.push(34)) return -1;
			if(!backtrack_stack.push(33)) return -1;
			if(!backtrack_stack.push(29)) return -1;
			if(!backtrack_stack.push(28)) return -1;
			if(!backtrack_stack.push(21)) return -1;
			if(!backtrack_stack.push(18)) return -1;
			if(!backtrack_stack.push(13)) return -1;
			if(!backtrack_stack.push(5)) return -1;
			if(!backtrack_stack.push(4)) return -1;
			goto label_3;
		}
		}
		break;
	case 2080:
	// Label to restore values of label 54
		v_kaon2equal_1 = int_value_stack.pop();
		v_kaon2equal_2 = int_value_stack.pop();
		c78_2_bf.pop(&cursor_stack);
		break;
	case 55: label_55: 
	// using  symbolic fact kaon2equal(v_kaon2equal_1,v_kaon2equal_2) with literal 2 in rule 
	// 218: locatedin(A,B) :- locatedin(C,B), kaon2equal(A,C).
	// resulting in symbolic fact locatedin(v_locatedin_1,v_locatedin_2)

		c218_1_bf.open(v_kaon2equal_2);
	case 1068:
	// Continuation for label 55
		while (c218_1_bf.fetch()) {
			v_locatedin_1 = v_kaon2equal_1;
			v_locatedin_2 = c218_1_bf.c2();
		if(locatedin_set.insert(v_locatedin_1, v_locatedin_2)) {
			if(!locatedin_bf.insert(v_locatedin_1, v_locatedin_2)) return -1;
			if(!locatedin_fb.insert(v_locatedin_2, v_locatedin_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1068)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(1005)) return -1;
			if(!backtrack_stack.push(999)) return -1;
			if(!backtrack_stack.push(987)) return -1;
			if(!backtrack_stack.push(981)) return -1;
			if(!backtrack_stack.push(969)) return -1;
			if(!backtrack_stack.push(963)) return -1;
			if(!backtrack_stack.push(952)) return -1;
			if(!backtrack_stack.push(946)) return -1;
			if(!backtrack_stack.push(936)) return -1;
			if(!backtrack_stack.push(930)) return -1;
			if(!backtrack_stack.push(920)) return -1;
			if(!backtrack_stack.push(917)) return -1;
			if(!backtrack_stack.push(898)) return -1;
			if(!backtrack_stack.push(897)) return -1;
			if(!backtrack_stack.push(896)) return -1;
			if(!backtrack_stack.push(893)) return -1;
			if(!backtrack_stack.push(726)) return -1;
			if(!backtrack_stack.push(721)) return -1;
			if(!backtrack_stack.push(694)) return -1;
			if(!backtrack_stack.push(693)) return -1;
			if(!backtrack_stack.push(688)) return -1;
			if(!backtrack_stack.push(687)) return -1;
			if(!backtrack_stack.push(636)) return -1;
			if(!backtrack_stack.push(631)) return -1;
			if(!backtrack_stack.push(592)) return -1;
			if(!backtrack_stack.push(591)) return -1;
			if(!backtrack_stack.push(529)) return -1;
			if(!backtrack_stack.push(528)) return -1;
			if(!backtrack_stack.push(513)) return -1;
			if(!backtrack_stack.push(512)) return -1;
			if(!backtrack_stack.push(511)) return -1;
			if(!backtrack_stack.push(506)) return -1;
			if(!backtrack_stack.push(455)) return -1;
			if(!backtrack_stack.push(450)) return -1;
			if(!backtrack_stack.push(441)) return -1;
			if(!backtrack_stack.push(440)) return -1;
			if(!backtrack_stack.push(439)) return -1;
			if(!backtrack_stack.push(434)) return -1;
			if(!backtrack_stack.push(339)) return -1;
			if(!backtrack_stack.push(338)) return -1;
			if(!backtrack_stack.push(249)) return -1;
			if(!backtrack_stack.push(244)) return -1;
			if(!backtrack_stack.push(52)) return -1;
			if(!backtrack_stack.push(51)) return -1;
			if(!backtrack_stack.push(50)) return -1;
			if(!backtrack_stack.push(49)) return -1;
			if(!backtrack_stack.push(48)) return -1;
			goto label_47;
		}
		}
		break;
	case 56: label_56: 
	// using  symbolic fact kaon2equal(v_kaon2equal_1,v_kaon2equal_2) with literal 2 in rule 
	// 219: locatedin(A,B) :- locatedin(A,C), kaon2equal(B,C).
	// resulting in symbolic fact locatedin(v_locatedin_1,v_locatedin_2)

		c219_1_fb.open(v_kaon2equal_2);
	case 1069:
	// Continuation for label 56
		while (c219_1_fb.fetch()) {
			v_locatedin_2 = v_kaon2equal_1;
			v_locatedin_1 = c219_1_fb.c2();
		if(locatedin_set.insert(v_locatedin_1, v_locatedin_2)) {
			if(!locatedin_bf.insert(v_locatedin_1, v_locatedin_2)) return -1;
			if(!locatedin_fb.insert(v_locatedin_2, v_locatedin_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1069)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(1005)) return -1;
			if(!backtrack_stack.push(999)) return -1;
			if(!backtrack_stack.push(987)) return -1;
			if(!backtrack_stack.push(981)) return -1;
			if(!backtrack_stack.push(969)) return -1;
			if(!backtrack_stack.push(963)) return -1;
			if(!backtrack_stack.push(952)) return -1;
			if(!backtrack_stack.push(946)) return -1;
			if(!backtrack_stack.push(936)) return -1;
			if(!backtrack_stack.push(930)) return -1;
			if(!backtrack_stack.push(920)) return -1;
			if(!backtrack_stack.push(917)) return -1;
			if(!backtrack_stack.push(898)) return -1;
			if(!backtrack_stack.push(897)) return -1;
			if(!backtrack_stack.push(896)) return -1;
			if(!backtrack_stack.push(893)) return -1;
			if(!backtrack_stack.push(726)) return -1;
			if(!backtrack_stack.push(721)) return -1;
			if(!backtrack_stack.push(694)) return -1;
			if(!backtrack_stack.push(693)) return -1;
			if(!backtrack_stack.push(688)) return -1;
			if(!backtrack_stack.push(687)) return -1;
			if(!backtrack_stack.push(636)) return -1;
			if(!backtrack_stack.push(631)) return -1;
			if(!backtrack_stack.push(592)) return -1;
			if(!backtrack_stack.push(591)) return -1;
			if(!backtrack_stack.push(529)) return -1;
			if(!backtrack_stack.push(528)) return -1;
			if(!backtrack_stack.push(513)) return -1;
			if(!backtrack_stack.push(512)) return -1;
			if(!backtrack_stack.push(511)) return -1;
			if(!backtrack_stack.push(506)) return -1;
			if(!backtrack_stack.push(455)) return -1;
			if(!backtrack_stack.push(450)) return -1;
			if(!backtrack_stack.push(441)) return -1;
			if(!backtrack_stack.push(440)) return -1;
			if(!backtrack_stack.push(439)) return -1;
			if(!backtrack_stack.push(434)) return -1;
			if(!backtrack_stack.push(339)) return -1;
			if(!backtrack_stack.push(338)) return -1;
			if(!backtrack_stack.push(249)) return -1;
			if(!backtrack_stack.push(244)) return -1;
			if(!backtrack_stack.push(52)) return -1;
			if(!backtrack_stack.push(51)) return -1;
			if(!backtrack_stack.push(50)) return -1;
			if(!backtrack_stack.push(49)) return -1;
			if(!backtrack_stack.push(48)) return -1;
			goto label_47;
		}
		}
		break;
	case 57: label_57: 
	// using  symbolic fact kaon2equal(v_kaon2equal_1,v_kaon2equal_2) with literal 2 in rule 
	// 621: hasbody(A,B) :- hasbody(C,B), kaon2equal(A,C).
	// resulting in symbolic fact hasbody(v_hasbody_1,v_hasbody_2)

		//recursive
		c621_1_bf.push(&cursor_stack);
		if(!int_value_stack.push(v_hasbody_2)) return -1;
		if(!int_value_stack.push(v_hasbody_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2083)) return -1;
		c621_1_bf.open(v_kaon2equal_2);
	case 1070:
	// Continuation for label 57
		while (c621_1_bf.fetch()) {
			v_hasbody_1 = v_kaon2equal_1;
			v_hasbody_2 = c621_1_bf.c2();
		if(hasbody_set.insert(v_hasbody_1, v_hasbody_2)) {
			if(!hasbody_bf.insert(v_hasbody_1, v_hasbody_2)) return -1;
			if(!hasbody_fb.insert(v_hasbody_2, v_hasbody_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1070)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(429)) return -1;
			if(!backtrack_stack.push(61)) return -1;
			if(!backtrack_stack.push(60)) return -1;
			if(!backtrack_stack.push(59)) return -1;
			if(!backtrack_stack.push(58)) return -1;
			goto label_54;
		}
		}
		break;
	case 2083:
	// Label to restore values of label 57
		v_hasbody_1 = int_value_stack.pop();
		v_hasbody_2 = int_value_stack.pop();
		c621_1_bf.pop(&cursor_stack);
		break;
	case 58: label_58: 
	// using  symbolic fact hasbody(v_hasbody_1,v_hasbody_2) with literal 1 in rule 
	// 267: haswinedescriptor(A,B) :- hasbody(A,B).
	// resulting in symbolic fact haswinedescriptor(v_haswinedescriptor_1,v_haswinedescriptor_2)

		//recursive
		if(!int_value_stack.push(v_haswinedescriptor_2)) return -1;
		if(!int_value_stack.push(v_haswinedescriptor_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2084)) return -1;
		v_haswinedescriptor_1 = v_hasbody_1;
		v_haswinedescriptor_2 = v_hasbody_2;
		if(haswinedescriptor_set.insert(v_haswinedescriptor_1, v_haswinedescriptor_2)) {
		if(!haswinedescriptor_bf.insert(v_haswinedescriptor_1, v_haswinedescriptor_2)) return -1;
		if(!haswinedescriptor_fb.insert(v_haswinedescriptor_2, v_haswinedescriptor_1)) return -1;
		// Labels for next rules
		if(!backtrack_stack.push(26)) return -1;
		if(!backtrack_stack.push(25)) return -1;
		goto label_15;
		}
		break;
	case 2084:
	// Label to restore values of label 58
		v_haswinedescriptor_1 = int_value_stack.pop();
		v_haswinedescriptor_2 = int_value_stack.pop();
		break;
	case 59: label_59: 
	// using  symbolic fact hasbody(v_hasbody_1,v_hasbody_2) with literal 1 in rule 
	// 621: hasbody(A,B) :- hasbody(C,B), kaon2equal(A,C).
	// resulting in symbolic fact hasbody(v_hasbody_1,v_hasbody_2)

		//recursive
		c621_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_hasbody_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2085)) return -1;
		c621_2_fb.open(v_hasbody_1);
	case 1072:
	// Continuation for label 59
		while (c621_2_fb.fetch()) {
			v_hasbody_1 = c621_2_fb.c2();
		if(hasbody_set.insert(v_hasbody_1, v_hasbody_2)) {
			if(!hasbody_bf.insert(v_hasbody_1, v_hasbody_2)) return -1;
			if(!hasbody_fb.insert(v_hasbody_2, v_hasbody_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1072)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(429)) return -1;
			if(!backtrack_stack.push(61)) return -1;
			if(!backtrack_stack.push(60)) return -1;
			if(!backtrack_stack.push(59)) return -1;
			if(!backtrack_stack.push(58)) return -1;
			goto label_54;
		}
		}
		break;
	case 2085:
	// Label to restore values of label 59
		v_hasbody_1 = int_value_stack.pop();
		c621_2_fb.pop(&cursor_stack);
		break;
	case 60: label_60: 
	// using  symbolic fact hasbody(v_hasbody_1,v_hasbody_2) with literal 1 in rule 
	// 622: hasbody(A,B) :- hasbody(A,C), kaon2equal(B,C).
	// resulting in symbolic fact hasbody(v_hasbody_1,v_hasbody_2)

		//recursive
		c622_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_hasbody_2)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2086)) return -1;
		c622_2_fb.open(v_hasbody_2);
	case 1073:
	// Continuation for label 60
		while (c622_2_fb.fetch()) {
			v_hasbody_2 = c622_2_fb.c2();
		if(hasbody_set.insert(v_hasbody_1, v_hasbody_2)) {
			if(!hasbody_bf.insert(v_hasbody_1, v_hasbody_2)) return -1;
			if(!hasbody_fb.insert(v_hasbody_2, v_hasbody_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1073)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(429)) return -1;
			if(!backtrack_stack.push(61)) return -1;
			if(!backtrack_stack.push(60)) return -1;
			if(!backtrack_stack.push(59)) return -1;
			if(!backtrack_stack.push(58)) return -1;
			goto label_54;
		}
		}
		break;
	case 2086:
	// Label to restore values of label 60
		v_hasbody_2 = int_value_stack.pop();
		c622_2_fb.pop(&cursor_stack);
		break;
	case 61: label_61: 
	// using  symbolic fact hasbody(v_hasbody_1,v_hasbody_2) with literal 2 in rule 
	// 78: kaon2equal(A,B) :- hasbody(C,A), hasbody(C,B).
	// resulting in symbolic fact kaon2equal(v_kaon2equal_1,v_kaon2equal_2)

		//recursive
		c78_1_bf.push(&cursor_stack);
		if(!int_value_stack.push(v_kaon2equal_1)) return -1;
		if(!int_value_stack.push(v_kaon2equal_2)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2087)) return -1;
		c78_1_bf.open(v_hasbody_1);
	case 1074:
	// Continuation for label 61
		while (c78_1_bf.fetch()) {
			v_kaon2equal_2 = v_hasbody_2;
			v_kaon2equal_1 = c78_1_bf.c2();
		if(kaon2equal_set.insert(v_kaon2equal_1, v_kaon2equal_2)) {
			if(!kaon2equal_bf.insert(v_kaon2equal_1, v_kaon2equal_2)) return -1;
			if(!kaon2equal_fb.insert(v_kaon2equal_2, v_kaon2equal_1)) return -1;
			if(!kaon2equal_ff.insert(v_kaon2equal_1, v_kaon2equal_2)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1074)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(283)) return -1;
			if(!backtrack_stack.push(282)) return -1;
			if(!backtrack_stack.push(281)) return -1;
			if(!backtrack_stack.push(280)) return -1;
			if(!backtrack_stack.push(279)) return -1;
			if(!backtrack_stack.push(278)) return -1;
			if(!backtrack_stack.push(277)) return -1;
			if(!backtrack_stack.push(276)) return -1;
			if(!backtrack_stack.push(275)) return -1;
			if(!backtrack_stack.push(274)) return -1;
			if(!backtrack_stack.push(273)) return -1;
			if(!backtrack_stack.push(271)) return -1;
			if(!backtrack_stack.push(270)) return -1;
			if(!backtrack_stack.push(269)) return -1;
			if(!backtrack_stack.push(268)) return -1;
			if(!backtrack_stack.push(267)) return -1;
			if(!backtrack_stack.push(266)) return -1;
			if(!backtrack_stack.push(265)) return -1;
			if(!backtrack_stack.push(264)) return -1;
			if(!backtrack_stack.push(263)) return -1;
			if(!backtrack_stack.push(262)) return -1;
			if(!backtrack_stack.push(261)) return -1;
			if(!backtrack_stack.push(260)) return -1;
			if(!backtrack_stack.push(254)) return -1;
			if(!backtrack_stack.push(253)) return -1;
			if(!backtrack_stack.push(152)) return -1;
			if(!backtrack_stack.push(142)) return -1;
			if(!backtrack_stack.push(141)) return -1;
			if(!backtrack_stack.push(140)) return -1;
			if(!backtrack_stack.push(139)) return -1;
			if(!backtrack_stack.push(138)) return -1;
			if(!backtrack_stack.push(137)) return -1;
			if(!backtrack_stack.push(134)) return -1;
			if(!backtrack_stack.push(133)) return -1;
			if(!backtrack_stack.push(127)) return -1;
			if(!backtrack_stack.push(126)) return -1;
			if(!backtrack_stack.push(125)) return -1;
			if(!backtrack_stack.push(124)) return -1;
			if(!backtrack_stack.push(85)) return -1;
			if(!backtrack_stack.push(82)) return -1;
			if(!backtrack_stack.push(81)) return -1;
			if(!backtrack_stack.push(73)) return -1;
			if(!backtrack_stack.push(72)) return -1;
			if(!backtrack_stack.push(69)) return -1;
			if(!backtrack_stack.push(62)) return -1;
			if(!backtrack_stack.push(57)) return -1;
			if(!backtrack_stack.push(56)) return -1;
			if(!backtrack_stack.push(55)) return -1;
			if(!backtrack_stack.push(45)) return -1;
			if(!backtrack_stack.push(42)) return -1;
			if(!backtrack_stack.push(41)) return -1;
			if(!backtrack_stack.push(35)) return -1;
			if(!backtrack_stack.push(34)) return -1;
			if(!backtrack_stack.push(33)) return -1;
			if(!backtrack_stack.push(29)) return -1;
			if(!backtrack_stack.push(28)) return -1;
			if(!backtrack_stack.push(21)) return -1;
			if(!backtrack_stack.push(18)) return -1;
			if(!backtrack_stack.push(13)) return -1;
			if(!backtrack_stack.push(5)) return -1;
			if(!backtrack_stack.push(4)) return -1;
			goto label_3;
		}
		}
		break;
	case 2087:
	// Label to restore values of label 61
		v_kaon2equal_2 = int_value_stack.pop();
		v_kaon2equal_1 = int_value_stack.pop();
		c78_1_bf.pop(&cursor_stack);
		break;
	case 62: label_62: 
	// using  symbolic fact kaon2equal(v_kaon2equal_1,v_kaon2equal_2) with literal 2 in rule 
	// 622: hasbody(A,B) :- hasbody(A,C), kaon2equal(B,C).
	// resulting in symbolic fact hasbody(v_hasbody_1,v_hasbody_2)

		//recursive
		c622_1_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_hasbody_1)) return -1;
		if(!int_value_stack.push(v_hasbody_2)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2088)) return -1;
		c622_1_fb.open(v_kaon2equal_2);
	case 1075:
	// Continuation for label 62
		while (c622_1_fb.fetch()) {
			v_hasbody_2 = v_kaon2equal_1;
			v_hasbody_1 = c622_1_fb.c2();
		if(hasbody_set.insert(v_hasbody_1, v_hasbody_2)) {
			if(!hasbody_bf.insert(v_hasbody_1, v_hasbody_2)) return -1;
			if(!hasbody_fb.insert(v_hasbody_2, v_hasbody_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1075)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(429)) return -1;
			if(!backtrack_stack.push(61)) return -1;
			if(!backtrack_stack.push(60)) return -1;
			if(!backtrack_stack.push(59)) return -1;
			if(!backtrack_stack.push(58)) return -1;
			goto label_54;
		}
		}
		break;
	case 2088:
	// Label to restore values of label 62
		v_hasbody_2 = int_value_stack.pop();
		v_hasbody_1 = int_value_stack.pop();
		c622_1_fb.pop(&cursor_stack);
		break;
	case 63: label_63: 
	// using  symbolic fact wine(v_wine_1) with literal 1 in rule 
	// 328: madefromgrape(A,A) :- wine(A), kaon2namedobjects(A).
	// resulting in symbolic fact madefromgrape(v_madefromgrape_1,v_madefromgrape_2)

		//recursive
		if(!int_value_stack.push(v_madefromgrape_2)) return -1;
		if(!int_value_stack.push(v_madefromgrape_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2089)) return -1;
		if (!kaon2namedobjects_b.contains(v_wine_1)) continue;
	case 1076:
	// Continuation for label 63
			v_madefromgrape_1 = v_wine_1;
			v_madefromgrape_2 = v_wine_1;
		if(madefromgrape_set.insert(v_madefromgrape_1, v_madefromgrape_2)) {
			if(!madefromgrape_bf.insert(v_madefromgrape_1, v_madefromgrape_2)) return -1;
			if(!madefromgrape_fb.insert(v_madefromgrape_2, v_madefromgrape_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1076)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(860)) return -1;
			if(!backtrack_stack.push(859)) return -1;
			if(!backtrack_stack.push(845)) return -1;
			if(!backtrack_stack.push(844)) return -1;
			if(!backtrack_stack.push(829)) return -1;
			if(!backtrack_stack.push(828)) return -1;
			if(!backtrack_stack.push(813)) return -1;
			if(!backtrack_stack.push(812)) return -1;
			if(!backtrack_stack.push(677)) return -1;
			if(!backtrack_stack.push(676)) return -1;
			if(!backtrack_stack.push(617)) return -1;
			if(!backtrack_stack.push(616)) return -1;
			if(!backtrack_stack.push(609)) return -1;
			if(!backtrack_stack.push(554)) return -1;
			if(!backtrack_stack.push(553)) return -1;
			if(!backtrack_stack.push(476)) return -1;
			if(!backtrack_stack.push(475)) return -1;
			if(!backtrack_stack.push(404)) return -1;
			if(!backtrack_stack.push(403)) return -1;
			if(!backtrack_stack.push(362)) return -1;
			if(!backtrack_stack.push(361)) return -1;
			if(!backtrack_stack.push(343)) return -1;
			if(!backtrack_stack.push(342)) return -1;
			if(!backtrack_stack.push(300)) return -1;
			if(!backtrack_stack.push(170)) return -1;
			if(!backtrack_stack.push(169)) return -1;
			if(!backtrack_stack.push(151)) return -1;
			if(!backtrack_stack.push(150)) return -1;
			if(!backtrack_stack.push(144)) return -1;
			if(!backtrack_stack.push(132)) return -1;
			if(!backtrack_stack.push(71)) return -1;
			if(!backtrack_stack.push(70)) return -1;
			if(!backtrack_stack.push(66)) return -1;
			goto label_64;
		}
		break;
	case 2089:
	// Label to restore values of label 63
		v_madefromgrape_1 = int_value_stack.pop();
		v_madefromgrape_2 = int_value_stack.pop();
		break;
	case 64: label_64: 
	// using  symbolic fact madefromgrape(v_madefromgrape_1,v_madefromgrape_2) with literal 1 in rule 
	// 51: madeintowine(A,B) :- madefromgrape(B,A).
	// resulting in symbolic fact madeintowine(v_madeintowine_1,v_madeintowine_2)

		//recursive
		if(!int_value_stack.push(v_madeintowine_2)) return -1;
		if(!int_value_stack.push(v_madeintowine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2090)) return -1;
		v_madeintowine_1 = v_madefromgrape_2;
		v_madeintowine_2 = v_madefromgrape_1;
		if(madeintowine_set.insert(v_madeintowine_1, v_madeintowine_2)) {
		if(!madeintowine_bf.insert(v_madeintowine_1, v_madeintowine_2)) return -1;
		if(!madeintowine_fb.insert(v_madeintowine_2, v_madeintowine_1)) return -1;
		// Labels for next rules
		if(!backtrack_stack.push(84)) return -1;
		if(!backtrack_stack.push(83)) return -1;
		goto label_65;
		}
		break;
	case 2090:
	// Label to restore values of label 64
		v_madeintowine_1 = int_value_stack.pop();
		v_madeintowine_2 = int_value_stack.pop();
		break;
	case 65: label_65: 
	// using  symbolic fact madeintowine(v_madeintowine_1,v_madeintowine_2) with literal 1 in rule 
	// 50: madefromgrape(A,B) :- madeintowine(B,A).
	// resulting in symbolic fact madefromgrape(v_madefromgrape_1,v_madefromgrape_2)

		//recursive
		if(!int_value_stack.push(v_madefromgrape_2)) return -1;
		if(!int_value_stack.push(v_madefromgrape_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2091)) return -1;
		v_madefromgrape_1 = v_madeintowine_2;
		v_madefromgrape_2 = v_madeintowine_1;
		if(madefromgrape_set.insert(v_madefromgrape_1, v_madefromgrape_2)) {
		if(!madefromgrape_bf.insert(v_madefromgrape_1, v_madefromgrape_2)) return -1;
		if(!madefromgrape_fb.insert(v_madefromgrape_2, v_madefromgrape_1)) return -1;
		// Labels for next rules
		if(!backtrack_stack.push(860)) return -1;
		if(!backtrack_stack.push(859)) return -1;
		if(!backtrack_stack.push(845)) return -1;
		if(!backtrack_stack.push(844)) return -1;
		if(!backtrack_stack.push(829)) return -1;
		if(!backtrack_stack.push(828)) return -1;
		if(!backtrack_stack.push(813)) return -1;
		if(!backtrack_stack.push(812)) return -1;
		if(!backtrack_stack.push(677)) return -1;
		if(!backtrack_stack.push(676)) return -1;
		if(!backtrack_stack.push(617)) return -1;
		if(!backtrack_stack.push(616)) return -1;
		if(!backtrack_stack.push(609)) return -1;
		if(!backtrack_stack.push(554)) return -1;
		if(!backtrack_stack.push(553)) return -1;
		if(!backtrack_stack.push(476)) return -1;
		if(!backtrack_stack.push(475)) return -1;
		if(!backtrack_stack.push(404)) return -1;
		if(!backtrack_stack.push(403)) return -1;
		if(!backtrack_stack.push(362)) return -1;
		if(!backtrack_stack.push(361)) return -1;
		if(!backtrack_stack.push(343)) return -1;
		if(!backtrack_stack.push(342)) return -1;
		if(!backtrack_stack.push(300)) return -1;
		if(!backtrack_stack.push(170)) return -1;
		if(!backtrack_stack.push(169)) return -1;
		if(!backtrack_stack.push(151)) return -1;
		if(!backtrack_stack.push(150)) return -1;
		if(!backtrack_stack.push(144)) return -1;
		if(!backtrack_stack.push(132)) return -1;
		if(!backtrack_stack.push(71)) return -1;
		if(!backtrack_stack.push(70)) return -1;
		if(!backtrack_stack.push(66)) return -1;
		goto label_64;
		}
		break;
	case 2091:
	// Label to restore values of label 65
		v_madefromgrape_1 = int_value_stack.pop();
		v_madefromgrape_2 = int_value_stack.pop();
		break;
	case 66: label_66: 
	// using  symbolic fact madefromgrape(v_madefromgrape_1,v_madefromgrape_2) with literal 1 in rule 
	// 327: wine(A) :- madefromgrape(A,B).
	// resulting in symbolic fact wine(v_wine_1)

		//recursive
		if(!int_value_stack.push(v_wine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2092)) return -1;
		v_wine_1 = v_madefromgrape_1;
		if(wine_set.insert(v_wine_1)) {
		if(!wine_f.insert(v_wine_1)) return -1;
		// Labels for next rules
		if(!backtrack_stack.push(1007)) return -1;
		if(!backtrack_stack.push(989)) return -1;
		if(!backtrack_stack.push(954)) return -1;
		if(!backtrack_stack.push(938)) return -1;
		if(!backtrack_stack.push(911)) return -1;
		if(!backtrack_stack.push(903)) return -1;
		if(!backtrack_stack.push(902)) return -1;
		if(!backtrack_stack.push(901)) return -1;
		if(!backtrack_stack.push(849)) return -1;
		if(!backtrack_stack.push(848)) return -1;
		if(!backtrack_stack.push(821)) return -1;
		if(!backtrack_stack.push(820)) return -1;
		if(!backtrack_stack.push(737)) return -1;
		if(!backtrack_stack.push(712)) return -1;
		if(!backtrack_stack.push(674)) return -1;
		if(!backtrack_stack.push(672)) return -1;
		if(!backtrack_stack.push(583)) return -1;
		if(!backtrack_stack.push(575)) return -1;
		if(!backtrack_stack.push(563)) return -1;
		if(!backtrack_stack.push(551)) return -1;
		if(!backtrack_stack.push(549)) return -1;
		if(!backtrack_stack.push(483)) return -1;
		if(!backtrack_stack.push(473)) return -1;
		if(!backtrack_stack.push(470)) return -1;
		if(!backtrack_stack.push(430)) return -1;
		if(!backtrack_stack.push(401)) return -1;
		if(!backtrack_stack.push(399)) return -1;
		if(!backtrack_stack.push(380)) return -1;
		if(!backtrack_stack.push(364)) return -1;
		if(!backtrack_stack.push(363)) return -1;
		if(!backtrack_stack.push(329)) return -1;
		if(!backtrack_stack.push(328)) return -1;
		if(!backtrack_stack.push(327)) return -1;
		if(!backtrack_stack.push(295)) return -1;
		if(!backtrack_stack.push(294)) return -1;
		if(!backtrack_stack.push(216)) return -1;
		if(!backtrack_stack.push(210)) return -1;
		if(!backtrack_stack.push(192)) return -1;
		if(!backtrack_stack.push(180)) return -1;
		if(!backtrack_stack.push(179)) return -1;
		if(!backtrack_stack.push(178)) return -1;
		if(!backtrack_stack.push(154)) return -1;
		if(!backtrack_stack.push(153)) return -1;
		if(!backtrack_stack.push(100)) return -1;
		if(!backtrack_stack.push(76)) return -1;
		if(!backtrack_stack.push(75)) return -1;
		if(!backtrack_stack.push(67)) return -1;
		if(!backtrack_stack.push(63)) return -1;
		if(!backtrack_stack.push(53)) return -1;
		if(!backtrack_stack.push(46)) return -1;
		if(!backtrack_stack.push(19)) return -1;
		goto label_16;
		}
		break;
	case 2092:
	// Label to restore values of label 66
		v_wine_1 = int_value_stack.pop();
		break;
	case 67: label_67: 
	// using  symbolic fact wine(v_wine_1) with literal 1 in rule 
	// 433: hascolor(A,A) :- wine(A), kaon2namedobjects(A).
	// resulting in symbolic fact hascolor(v_hascolor_1,v_hascolor_2)

		//recursive
		if(!int_value_stack.push(v_hascolor_2)) return -1;
		if(!int_value_stack.push(v_hascolor_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2093)) return -1;
		if (!kaon2namedobjects_b.contains(v_wine_1)) continue;
	case 1080:
	// Continuation for label 67
			v_hascolor_1 = v_wine_1;
			v_hascolor_2 = v_wine_1;
		if(hascolor_set.insert(v_hascolor_1, v_hascolor_2)) {
			if(!hascolor_bf.insert(v_hascolor_1, v_hascolor_2)) return -1;
			if(!hascolor_fb.insert(v_hascolor_2, v_hascolor_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1080)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(286)) return -1;
			if(!backtrack_stack.push(120)) return -1;
			if(!backtrack_stack.push(80)) return -1;
			if(!backtrack_stack.push(79)) return -1;
			if(!backtrack_stack.push(78)) return -1;
			if(!backtrack_stack.push(77)) return -1;
			if(!backtrack_stack.push(74)) return -1;
			goto label_68;
		}
		break;
	case 2093:
	// Label to restore values of label 67
		v_hascolor_1 = int_value_stack.pop();
		v_hascolor_2 = int_value_stack.pop();
		break;
	case 68: label_68: 
	// using  symbolic fact hascolor(v_hascolor_1,v_hascolor_2) with literal 1 in rule 
	// 40: kaon2equal(A,B) :- hascolor(C,A), hascolor(C,B).
	// resulting in symbolic fact kaon2equal(v_kaon2equal_1,v_kaon2equal_2)

		//recursive
		c40_2_bf.push(&cursor_stack);
		if(!int_value_stack.push(v_kaon2equal_2)) return -1;
		if(!int_value_stack.push(v_kaon2equal_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2094)) return -1;
		c40_2_bf.open(v_hascolor_1);
	case 1081:
	// Continuation for label 68
		while (c40_2_bf.fetch()) {
			v_kaon2equal_1 = v_hascolor_2;
			v_kaon2equal_2 = c40_2_bf.c2();
		if(kaon2equal_set.insert(v_kaon2equal_1, v_kaon2equal_2)) {
			if(!kaon2equal_bf.insert(v_kaon2equal_1, v_kaon2equal_2)) return -1;
			if(!kaon2equal_fb.insert(v_kaon2equal_2, v_kaon2equal_1)) return -1;
			if(!kaon2equal_ff.insert(v_kaon2equal_1, v_kaon2equal_2)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1081)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(283)) return -1;
			if(!backtrack_stack.push(282)) return -1;
			if(!backtrack_stack.push(281)) return -1;
			if(!backtrack_stack.push(280)) return -1;
			if(!backtrack_stack.push(279)) return -1;
			if(!backtrack_stack.push(278)) return -1;
			if(!backtrack_stack.push(277)) return -1;
			if(!backtrack_stack.push(276)) return -1;
			if(!backtrack_stack.push(275)) return -1;
			if(!backtrack_stack.push(274)) return -1;
			if(!backtrack_stack.push(273)) return -1;
			if(!backtrack_stack.push(271)) return -1;
			if(!backtrack_stack.push(270)) return -1;
			if(!backtrack_stack.push(269)) return -1;
			if(!backtrack_stack.push(268)) return -1;
			if(!backtrack_stack.push(267)) return -1;
			if(!backtrack_stack.push(266)) return -1;
			if(!backtrack_stack.push(265)) return -1;
			if(!backtrack_stack.push(264)) return -1;
			if(!backtrack_stack.push(263)) return -1;
			if(!backtrack_stack.push(262)) return -1;
			if(!backtrack_stack.push(261)) return -1;
			if(!backtrack_stack.push(260)) return -1;
			if(!backtrack_stack.push(254)) return -1;
			if(!backtrack_stack.push(253)) return -1;
			if(!backtrack_stack.push(152)) return -1;
			if(!backtrack_stack.push(142)) return -1;
			if(!backtrack_stack.push(141)) return -1;
			if(!backtrack_stack.push(140)) return -1;
			if(!backtrack_stack.push(139)) return -1;
			if(!backtrack_stack.push(138)) return -1;
			if(!backtrack_stack.push(137)) return -1;
			if(!backtrack_stack.push(134)) return -1;
			if(!backtrack_stack.push(133)) return -1;
			if(!backtrack_stack.push(127)) return -1;
			if(!backtrack_stack.push(126)) return -1;
			if(!backtrack_stack.push(125)) return -1;
			if(!backtrack_stack.push(124)) return -1;
			if(!backtrack_stack.push(85)) return -1;
			if(!backtrack_stack.push(82)) return -1;
			if(!backtrack_stack.push(81)) return -1;
			if(!backtrack_stack.push(73)) return -1;
			if(!backtrack_stack.push(72)) return -1;
			if(!backtrack_stack.push(69)) return -1;
			if(!backtrack_stack.push(62)) return -1;
			if(!backtrack_stack.push(57)) return -1;
			if(!backtrack_stack.push(56)) return -1;
			if(!backtrack_stack.push(55)) return -1;
			if(!backtrack_stack.push(45)) return -1;
			if(!backtrack_stack.push(42)) return -1;
			if(!backtrack_stack.push(41)) return -1;
			if(!backtrack_stack.push(35)) return -1;
			if(!backtrack_stack.push(34)) return -1;
			if(!backtrack_stack.push(33)) return -1;
			if(!backtrack_stack.push(29)) return -1;
			if(!backtrack_stack.push(28)) return -1;
			if(!backtrack_stack.push(21)) return -1;
			if(!backtrack_stack.push(18)) return -1;
			if(!backtrack_stack.push(13)) return -1;
			if(!backtrack_stack.push(5)) return -1;
			if(!backtrack_stack.push(4)) return -1;
			goto label_3;
		}
		}
		break;
	case 2094:
	// Label to restore values of label 68
		v_kaon2equal_1 = int_value_stack.pop();
		v_kaon2equal_2 = int_value_stack.pop();
		c40_2_bf.pop(&cursor_stack);
		break;
	case 69: label_69: 
	// using  symbolic fact kaon2equal(v_kaon2equal_1,v_kaon2equal_2) with literal 2 in rule 
	// 529: madefromgrape(A,B) :- madefromgrape(C,B), kaon2equal(A,C).
	// resulting in symbolic fact madefromgrape(v_madefromgrape_1,v_madefromgrape_2)

		//recursive
		c529_1_bf.push(&cursor_stack);
		if(!int_value_stack.push(v_madefromgrape_2)) return -1;
		if(!int_value_stack.push(v_madefromgrape_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2095)) return -1;
		c529_1_bf.open(v_kaon2equal_2);
	case 1082:
	// Continuation for label 69
		while (c529_1_bf.fetch()) {
			v_madefromgrape_1 = v_kaon2equal_1;
			v_madefromgrape_2 = c529_1_bf.c2();
		if(madefromgrape_set.insert(v_madefromgrape_1, v_madefromgrape_2)) {
			if(!madefromgrape_bf.insert(v_madefromgrape_1, v_madefromgrape_2)) return -1;
			if(!madefromgrape_fb.insert(v_madefromgrape_2, v_madefromgrape_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1082)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(860)) return -1;
			if(!backtrack_stack.push(859)) return -1;
			if(!backtrack_stack.push(845)) return -1;
			if(!backtrack_stack.push(844)) return -1;
			if(!backtrack_stack.push(829)) return -1;
			if(!backtrack_stack.push(828)) return -1;
			if(!backtrack_stack.push(813)) return -1;
			if(!backtrack_stack.push(812)) return -1;
			if(!backtrack_stack.push(677)) return -1;
			if(!backtrack_stack.push(676)) return -1;
			if(!backtrack_stack.push(617)) return -1;
			if(!backtrack_stack.push(616)) return -1;
			if(!backtrack_stack.push(609)) return -1;
			if(!backtrack_stack.push(554)) return -1;
			if(!backtrack_stack.push(553)) return -1;
			if(!backtrack_stack.push(476)) return -1;
			if(!backtrack_stack.push(475)) return -1;
			if(!backtrack_stack.push(404)) return -1;
			if(!backtrack_stack.push(403)) return -1;
			if(!backtrack_stack.push(362)) return -1;
			if(!backtrack_stack.push(361)) return -1;
			if(!backtrack_stack.push(343)) return -1;
			if(!backtrack_stack.push(342)) return -1;
			if(!backtrack_stack.push(300)) return -1;
			if(!backtrack_stack.push(170)) return -1;
			if(!backtrack_stack.push(169)) return -1;
			if(!backtrack_stack.push(151)) return -1;
			if(!backtrack_stack.push(150)) return -1;
			if(!backtrack_stack.push(144)) return -1;
			if(!backtrack_stack.push(132)) return -1;
			if(!backtrack_stack.push(71)) return -1;
			if(!backtrack_stack.push(70)) return -1;
			if(!backtrack_stack.push(66)) return -1;
			goto label_64;
		}
		}
		break;
	case 2095:
	// Label to restore values of label 69
		v_madefromgrape_1 = int_value_stack.pop();
		v_madefromgrape_2 = int_value_stack.pop();
		c529_1_bf.pop(&cursor_stack);
		break;
	case 70: label_70: 
	// using  symbolic fact madefromgrape(v_madefromgrape_1,v_madefromgrape_2) with literal 1 in rule 
	// 529: madefromgrape(A,B) :- madefromgrape(C,B), kaon2equal(A,C).
	// resulting in symbolic fact madefromgrape(v_madefromgrape_1,v_madefromgrape_2)

		//recursive
		c529_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_madefromgrape_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2096)) return -1;
		c529_2_fb.open(v_madefromgrape_1);
	case 1083:
	// Continuation for label 70
		while (c529_2_fb.fetch()) {
			v_madefromgrape_1 = c529_2_fb.c2();
		if(madefromgrape_set.insert(v_madefromgrape_1, v_madefromgrape_2)) {
			if(!madefromgrape_bf.insert(v_madefromgrape_1, v_madefromgrape_2)) return -1;
			if(!madefromgrape_fb.insert(v_madefromgrape_2, v_madefromgrape_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1083)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(860)) return -1;
			if(!backtrack_stack.push(859)) return -1;
			if(!backtrack_stack.push(845)) return -1;
			if(!backtrack_stack.push(844)) return -1;
			if(!backtrack_stack.push(829)) return -1;
			if(!backtrack_stack.push(828)) return -1;
			if(!backtrack_stack.push(813)) return -1;
			if(!backtrack_stack.push(812)) return -1;
			if(!backtrack_stack.push(677)) return -1;
			if(!backtrack_stack.push(676)) return -1;
			if(!backtrack_stack.push(617)) return -1;
			if(!backtrack_stack.push(616)) return -1;
			if(!backtrack_stack.push(609)) return -1;
			if(!backtrack_stack.push(554)) return -1;
			if(!backtrack_stack.push(553)) return -1;
			if(!backtrack_stack.push(476)) return -1;
			if(!backtrack_stack.push(475)) return -1;
			if(!backtrack_stack.push(404)) return -1;
			if(!backtrack_stack.push(403)) return -1;
			if(!backtrack_stack.push(362)) return -1;
			if(!backtrack_stack.push(361)) return -1;
			if(!backtrack_stack.push(343)) return -1;
			if(!backtrack_stack.push(342)) return -1;
			if(!backtrack_stack.push(300)) return -1;
			if(!backtrack_stack.push(170)) return -1;
			if(!backtrack_stack.push(169)) return -1;
			if(!backtrack_stack.push(151)) return -1;
			if(!backtrack_stack.push(150)) return -1;
			if(!backtrack_stack.push(144)) return -1;
			if(!backtrack_stack.push(132)) return -1;
			if(!backtrack_stack.push(71)) return -1;
			if(!backtrack_stack.push(70)) return -1;
			if(!backtrack_stack.push(66)) return -1;
			goto label_64;
		}
		}
		break;
	case 2096:
	// Label to restore values of label 70
		v_madefromgrape_1 = int_value_stack.pop();
		c529_2_fb.pop(&cursor_stack);
		break;
	case 71: label_71: 
	// using  symbolic fact madefromgrape(v_madefromgrape_1,v_madefromgrape_2) with literal 1 in rule 
	// 530: madefromgrape(A,B) :- madefromgrape(A,C), kaon2equal(B,C).
	// resulting in symbolic fact madefromgrape(v_madefromgrape_1,v_madefromgrape_2)

		//recursive
		c530_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_madefromgrape_2)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2097)) return -1;
		c530_2_fb.open(v_madefromgrape_2);
	case 1084:
	// Continuation for label 71
		while (c530_2_fb.fetch()) {
			v_madefromgrape_2 = c530_2_fb.c2();
		if(madefromgrape_set.insert(v_madefromgrape_1, v_madefromgrape_2)) {
			if(!madefromgrape_bf.insert(v_madefromgrape_1, v_madefromgrape_2)) return -1;
			if(!madefromgrape_fb.insert(v_madefromgrape_2, v_madefromgrape_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1084)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(860)) return -1;
			if(!backtrack_stack.push(859)) return -1;
			if(!backtrack_stack.push(845)) return -1;
			if(!backtrack_stack.push(844)) return -1;
			if(!backtrack_stack.push(829)) return -1;
			if(!backtrack_stack.push(828)) return -1;
			if(!backtrack_stack.push(813)) return -1;
			if(!backtrack_stack.push(812)) return -1;
			if(!backtrack_stack.push(677)) return -1;
			if(!backtrack_stack.push(676)) return -1;
			if(!backtrack_stack.push(617)) return -1;
			if(!backtrack_stack.push(616)) return -1;
			if(!backtrack_stack.push(609)) return -1;
			if(!backtrack_stack.push(554)) return -1;
			if(!backtrack_stack.push(553)) return -1;
			if(!backtrack_stack.push(476)) return -1;
			if(!backtrack_stack.push(475)) return -1;
			if(!backtrack_stack.push(404)) return -1;
			if(!backtrack_stack.push(403)) return -1;
			if(!backtrack_stack.push(362)) return -1;
			if(!backtrack_stack.push(361)) return -1;
			if(!backtrack_stack.push(343)) return -1;
			if(!backtrack_stack.push(342)) return -1;
			if(!backtrack_stack.push(300)) return -1;
			if(!backtrack_stack.push(170)) return -1;
			if(!backtrack_stack.push(169)) return -1;
			if(!backtrack_stack.push(151)) return -1;
			if(!backtrack_stack.push(150)) return -1;
			if(!backtrack_stack.push(144)) return -1;
			if(!backtrack_stack.push(132)) return -1;
			if(!backtrack_stack.push(71)) return -1;
			if(!backtrack_stack.push(70)) return -1;
			if(!backtrack_stack.push(66)) return -1;
			goto label_64;
		}
		}
		break;
	case 2097:
	// Label to restore values of label 71
		v_madefromgrape_2 = int_value_stack.pop();
		c530_2_fb.pop(&cursor_stack);
		break;
	case 72: label_72: 
	// using  symbolic fact kaon2equal(v_kaon2equal_1,v_kaon2equal_2) with literal 2 in rule 
	// 530: madefromgrape(A,B) :- madefromgrape(A,C), kaon2equal(B,C).
	// resulting in symbolic fact madefromgrape(v_madefromgrape_1,v_madefromgrape_2)

		//recursive
		c530_1_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_madefromgrape_1)) return -1;
		if(!int_value_stack.push(v_madefromgrape_2)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2098)) return -1;
		c530_1_fb.open(v_kaon2equal_2);
	case 1085:
	// Continuation for label 72
		while (c530_1_fb.fetch()) {
			v_madefromgrape_2 = v_kaon2equal_1;
			v_madefromgrape_1 = c530_1_fb.c2();
		if(madefromgrape_set.insert(v_madefromgrape_1, v_madefromgrape_2)) {
			if(!madefromgrape_bf.insert(v_madefromgrape_1, v_madefromgrape_2)) return -1;
			if(!madefromgrape_fb.insert(v_madefromgrape_2, v_madefromgrape_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1085)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(860)) return -1;
			if(!backtrack_stack.push(859)) return -1;
			if(!backtrack_stack.push(845)) return -1;
			if(!backtrack_stack.push(844)) return -1;
			if(!backtrack_stack.push(829)) return -1;
			if(!backtrack_stack.push(828)) return -1;
			if(!backtrack_stack.push(813)) return -1;
			if(!backtrack_stack.push(812)) return -1;
			if(!backtrack_stack.push(677)) return -1;
			if(!backtrack_stack.push(676)) return -1;
			if(!backtrack_stack.push(617)) return -1;
			if(!backtrack_stack.push(616)) return -1;
			if(!backtrack_stack.push(609)) return -1;
			if(!backtrack_stack.push(554)) return -1;
			if(!backtrack_stack.push(553)) return -1;
			if(!backtrack_stack.push(476)) return -1;
			if(!backtrack_stack.push(475)) return -1;
			if(!backtrack_stack.push(404)) return -1;
			if(!backtrack_stack.push(403)) return -1;
			if(!backtrack_stack.push(362)) return -1;
			if(!backtrack_stack.push(361)) return -1;
			if(!backtrack_stack.push(343)) return -1;
			if(!backtrack_stack.push(342)) return -1;
			if(!backtrack_stack.push(300)) return -1;
			if(!backtrack_stack.push(170)) return -1;
			if(!backtrack_stack.push(169)) return -1;
			if(!backtrack_stack.push(151)) return -1;
			if(!backtrack_stack.push(150)) return -1;
			if(!backtrack_stack.push(144)) return -1;
			if(!backtrack_stack.push(132)) return -1;
			if(!backtrack_stack.push(71)) return -1;
			if(!backtrack_stack.push(70)) return -1;
			if(!backtrack_stack.push(66)) return -1;
			goto label_64;
		}
		}
		break;
	case 2098:
	// Label to restore values of label 72
		v_madefromgrape_2 = int_value_stack.pop();
		v_madefromgrape_1 = int_value_stack.pop();
		c530_1_fb.pop(&cursor_stack);
		break;
	case 73: label_73: 
	// using  symbolic fact kaon2equal(v_kaon2equal_1,v_kaon2equal_2) with literal 2 in rule 
	// 601: hascolor(A,B) :- hascolor(C,B), kaon2equal(A,C).
	// resulting in symbolic fact hascolor(v_hascolor_1,v_hascolor_2)

		//recursive
		c601_1_bf.push(&cursor_stack);
		if(!int_value_stack.push(v_hascolor_2)) return -1;
		if(!int_value_stack.push(v_hascolor_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2099)) return -1;
		c601_1_bf.open(v_kaon2equal_2);
	case 1086:
	// Continuation for label 73
		while (c601_1_bf.fetch()) {
			v_hascolor_1 = v_kaon2equal_1;
			v_hascolor_2 = c601_1_bf.c2();
		if(hascolor_set.insert(v_hascolor_1, v_hascolor_2)) {
			if(!hascolor_bf.insert(v_hascolor_1, v_hascolor_2)) return -1;
			if(!hascolor_fb.insert(v_hascolor_2, v_hascolor_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1086)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(286)) return -1;
			if(!backtrack_stack.push(120)) return -1;
			if(!backtrack_stack.push(80)) return -1;
			if(!backtrack_stack.push(79)) return -1;
			if(!backtrack_stack.push(78)) return -1;
			if(!backtrack_stack.push(77)) return -1;
			if(!backtrack_stack.push(74)) return -1;
			goto label_68;
		}
		}
		break;
	case 2099:
	// Label to restore values of label 73
		v_hascolor_1 = int_value_stack.pop();
		v_hascolor_2 = int_value_stack.pop();
		c601_1_bf.pop(&cursor_stack);
		break;
	case 74: label_74: 
	// using  symbolic fact hascolor(v_hascolor_1,v_hascolor_2) with literal 1 in rule 
	// 432: wine(A) :- hascolor(A,B).
	// resulting in symbolic fact wine(v_wine_1)

		//recursive
		if(!int_value_stack.push(v_wine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2100)) return -1;
		v_wine_1 = v_hascolor_1;
		if(wine_set.insert(v_wine_1)) {
		if(!wine_f.insert(v_wine_1)) return -1;
		// Labels for next rules
		if(!backtrack_stack.push(1007)) return -1;
		if(!backtrack_stack.push(989)) return -1;
		if(!backtrack_stack.push(954)) return -1;
		if(!backtrack_stack.push(938)) return -1;
		if(!backtrack_stack.push(911)) return -1;
		if(!backtrack_stack.push(903)) return -1;
		if(!backtrack_stack.push(902)) return -1;
		if(!backtrack_stack.push(901)) return -1;
		if(!backtrack_stack.push(849)) return -1;
		if(!backtrack_stack.push(848)) return -1;
		if(!backtrack_stack.push(821)) return -1;
		if(!backtrack_stack.push(820)) return -1;
		if(!backtrack_stack.push(737)) return -1;
		if(!backtrack_stack.push(712)) return -1;
		if(!backtrack_stack.push(674)) return -1;
		if(!backtrack_stack.push(672)) return -1;
		if(!backtrack_stack.push(583)) return -1;
		if(!backtrack_stack.push(575)) return -1;
		if(!backtrack_stack.push(563)) return -1;
		if(!backtrack_stack.push(551)) return -1;
		if(!backtrack_stack.push(549)) return -1;
		if(!backtrack_stack.push(483)) return -1;
		if(!backtrack_stack.push(473)) return -1;
		if(!backtrack_stack.push(470)) return -1;
		if(!backtrack_stack.push(430)) return -1;
		if(!backtrack_stack.push(401)) return -1;
		if(!backtrack_stack.push(399)) return -1;
		if(!backtrack_stack.push(380)) return -1;
		if(!backtrack_stack.push(364)) return -1;
		if(!backtrack_stack.push(363)) return -1;
		if(!backtrack_stack.push(329)) return -1;
		if(!backtrack_stack.push(328)) return -1;
		if(!backtrack_stack.push(327)) return -1;
		if(!backtrack_stack.push(295)) return -1;
		if(!backtrack_stack.push(294)) return -1;
		if(!backtrack_stack.push(216)) return -1;
		if(!backtrack_stack.push(210)) return -1;
		if(!backtrack_stack.push(192)) return -1;
		if(!backtrack_stack.push(180)) return -1;
		if(!backtrack_stack.push(179)) return -1;
		if(!backtrack_stack.push(178)) return -1;
		if(!backtrack_stack.push(154)) return -1;
		if(!backtrack_stack.push(153)) return -1;
		if(!backtrack_stack.push(100)) return -1;
		if(!backtrack_stack.push(76)) return -1;
		if(!backtrack_stack.push(75)) return -1;
		if(!backtrack_stack.push(67)) return -1;
		if(!backtrack_stack.push(63)) return -1;
		if(!backtrack_stack.push(53)) return -1;
		if(!backtrack_stack.push(46)) return -1;
		if(!backtrack_stack.push(19)) return -1;
		goto label_16;
		}
		break;
	case 2100:
	// Label to restore values of label 74
		v_wine_1 = int_value_stack.pop();
		break;
	case 75: label_75: 
	// using  symbolic fact wine(v_wine_1) with literal 1 in rule 
	// 516: wine(A) :- wine(B), kaon2equal(A,B).
	// resulting in symbolic fact wine(v_wine_1)

		//recursive
		c516_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_wine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2101)) return -1;
		c516_2_fb.open(v_wine_1);
	case 1088:
	// Continuation for label 75
		while (c516_2_fb.fetch()) {
			v_wine_1 = c516_2_fb.c2();
		if(wine_set.insert(v_wine_1)) {
			if(!wine_f.insert(v_wine_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1088)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(1007)) return -1;
			if(!backtrack_stack.push(989)) return -1;
			if(!backtrack_stack.push(954)) return -1;
			if(!backtrack_stack.push(938)) return -1;
			if(!backtrack_stack.push(911)) return -1;
			if(!backtrack_stack.push(903)) return -1;
			if(!backtrack_stack.push(902)) return -1;
			if(!backtrack_stack.push(901)) return -1;
			if(!backtrack_stack.push(849)) return -1;
			if(!backtrack_stack.push(848)) return -1;
			if(!backtrack_stack.push(821)) return -1;
			if(!backtrack_stack.push(820)) return -1;
			if(!backtrack_stack.push(737)) return -1;
			if(!backtrack_stack.push(712)) return -1;
			if(!backtrack_stack.push(674)) return -1;
			if(!backtrack_stack.push(672)) return -1;
			if(!backtrack_stack.push(583)) return -1;
			if(!backtrack_stack.push(575)) return -1;
			if(!backtrack_stack.push(563)) return -1;
			if(!backtrack_stack.push(551)) return -1;
			if(!backtrack_stack.push(549)) return -1;
			if(!backtrack_stack.push(483)) return -1;
			if(!backtrack_stack.push(473)) return -1;
			if(!backtrack_stack.push(470)) return -1;
			if(!backtrack_stack.push(430)) return -1;
			if(!backtrack_stack.push(401)) return -1;
			if(!backtrack_stack.push(399)) return -1;
			if(!backtrack_stack.push(380)) return -1;
			if(!backtrack_stack.push(364)) return -1;
			if(!backtrack_stack.push(363)) return -1;
			if(!backtrack_stack.push(329)) return -1;
			if(!backtrack_stack.push(328)) return -1;
			if(!backtrack_stack.push(327)) return -1;
			if(!backtrack_stack.push(295)) return -1;
			if(!backtrack_stack.push(294)) return -1;
			if(!backtrack_stack.push(216)) return -1;
			if(!backtrack_stack.push(210)) return -1;
			if(!backtrack_stack.push(192)) return -1;
			if(!backtrack_stack.push(180)) return -1;
			if(!backtrack_stack.push(179)) return -1;
			if(!backtrack_stack.push(178)) return -1;
			if(!backtrack_stack.push(154)) return -1;
			if(!backtrack_stack.push(153)) return -1;
			if(!backtrack_stack.push(100)) return -1;
			if(!backtrack_stack.push(76)) return -1;
			if(!backtrack_stack.push(75)) return -1;
			if(!backtrack_stack.push(67)) return -1;
			if(!backtrack_stack.push(63)) return -1;
			if(!backtrack_stack.push(53)) return -1;
			if(!backtrack_stack.push(46)) return -1;
			if(!backtrack_stack.push(19)) return -1;
			goto label_16;
		}
		}
		break;
	case 2101:
	// Label to restore values of label 75
		v_wine_1 = int_value_stack.pop();
		c516_2_fb.pop(&cursor_stack);
		break;
	case 76: label_76: 
	// using  symbolic fact wine(v_wine_1) with literal 1 in rule 
	// 647: hasflavor(A,A) :- wine(A), kaon2namedobjects(A).
	// resulting in symbolic fact hasflavor(v_hasflavor_1,v_hasflavor_2)

		//recursive
		if(!int_value_stack.push(v_hasflavor_2)) return -1;
		if(!int_value_stack.push(v_hasflavor_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2102)) return -1;
		if (!kaon2namedobjects_b.contains(v_wine_1)) continue;
	case 1089:
	// Continuation for label 76
			v_hasflavor_1 = v_wine_1;
			v_hasflavor_2 = v_wine_1;
		if(hasflavor_set.insert(v_hasflavor_1, v_hasflavor_2)) {
			if(!hasflavor_bf.insert(v_hasflavor_1, v_hasflavor_2)) return -1;
			if(!hasflavor_fb.insert(v_hasflavor_2, v_hasflavor_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1089)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(32)) return -1;
			if(!backtrack_stack.push(31)) return -1;
			if(!backtrack_stack.push(30)) return -1;
			if(!backtrack_stack.push(14)) return -1;
			goto label_12;
		}
		break;
	case 2102:
	// Label to restore values of label 76
		v_hasflavor_1 = int_value_stack.pop();
		v_hasflavor_2 = int_value_stack.pop();
		break;
	case 77: label_77: 
	// using  symbolic fact hascolor(v_hascolor_1,v_hascolor_2) with literal 1 in rule 
	// 601: hascolor(A,B) :- hascolor(C,B), kaon2equal(A,C).
	// resulting in symbolic fact hascolor(v_hascolor_1,v_hascolor_2)

		//recursive
		c601_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_hascolor_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2103)) return -1;
		c601_2_fb.open(v_hascolor_1);
	case 1090:
	// Continuation for label 77
		while (c601_2_fb.fetch()) {
			v_hascolor_1 = c601_2_fb.c2();
		if(hascolor_set.insert(v_hascolor_1, v_hascolor_2)) {
			if(!hascolor_bf.insert(v_hascolor_1, v_hascolor_2)) return -1;
			if(!hascolor_fb.insert(v_hascolor_2, v_hascolor_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1090)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(286)) return -1;
			if(!backtrack_stack.push(120)) return -1;
			if(!backtrack_stack.push(80)) return -1;
			if(!backtrack_stack.push(79)) return -1;
			if(!backtrack_stack.push(78)) return -1;
			if(!backtrack_stack.push(77)) return -1;
			if(!backtrack_stack.push(74)) return -1;
			goto label_68;
		}
		}
		break;
	case 2103:
	// Label to restore values of label 77
		v_hascolor_1 = int_value_stack.pop();
		c601_2_fb.pop(&cursor_stack);
		break;
	case 78: label_78: 
	// using  symbolic fact hascolor(v_hascolor_1,v_hascolor_2) with literal 1 in rule 
	// 602: hascolor(A,B) :- hascolor(A,C), kaon2equal(B,C).
	// resulting in symbolic fact hascolor(v_hascolor_1,v_hascolor_2)

		//recursive
		c602_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_hascolor_2)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2104)) return -1;
		c602_2_fb.open(v_hascolor_2);
	case 1091:
	// Continuation for label 78
		while (c602_2_fb.fetch()) {
			v_hascolor_2 = c602_2_fb.c2();
		if(hascolor_set.insert(v_hascolor_1, v_hascolor_2)) {
			if(!hascolor_bf.insert(v_hascolor_1, v_hascolor_2)) return -1;
			if(!hascolor_fb.insert(v_hascolor_2, v_hascolor_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1091)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(286)) return -1;
			if(!backtrack_stack.push(120)) return -1;
			if(!backtrack_stack.push(80)) return -1;
			if(!backtrack_stack.push(79)) return -1;
			if(!backtrack_stack.push(78)) return -1;
			if(!backtrack_stack.push(77)) return -1;
			if(!backtrack_stack.push(74)) return -1;
			goto label_68;
		}
		}
		break;
	case 2104:
	// Label to restore values of label 78
		v_hascolor_2 = int_value_stack.pop();
		c602_2_fb.pop(&cursor_stack);
		break;
	case 79: label_79: 
	// using  symbolic fact hascolor(v_hascolor_1,v_hascolor_2) with literal 1 in rule 
	// 641: haswinedescriptor(A,B) :- hascolor(A,B).
	// resulting in symbolic fact haswinedescriptor(v_haswinedescriptor_1,v_haswinedescriptor_2)

		//recursive
		if(!int_value_stack.push(v_haswinedescriptor_2)) return -1;
		if(!int_value_stack.push(v_haswinedescriptor_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2105)) return -1;
		v_haswinedescriptor_1 = v_hascolor_1;
		v_haswinedescriptor_2 = v_hascolor_2;
		if(haswinedescriptor_set.insert(v_haswinedescriptor_1, v_haswinedescriptor_2)) {
		if(!haswinedescriptor_bf.insert(v_haswinedescriptor_1, v_haswinedescriptor_2)) return -1;
		if(!haswinedescriptor_fb.insert(v_haswinedescriptor_2, v_haswinedescriptor_1)) return -1;
		// Labels for next rules
		if(!backtrack_stack.push(26)) return -1;
		if(!backtrack_stack.push(25)) return -1;
		goto label_15;
		}
		break;
	case 2105:
	// Label to restore values of label 79
		v_haswinedescriptor_1 = int_value_stack.pop();
		v_haswinedescriptor_2 = int_value_stack.pop();
		break;
	case 80: label_80: 
	// using  symbolic fact hascolor(v_hascolor_1,v_hascolor_2) with literal 2 in rule 
	// 40: kaon2equal(A,B) :- hascolor(C,A), hascolor(C,B).
	// resulting in symbolic fact kaon2equal(v_kaon2equal_1,v_kaon2equal_2)

		//recursive
		c40_1_bf.push(&cursor_stack);
		if(!int_value_stack.push(v_kaon2equal_1)) return -1;
		if(!int_value_stack.push(v_kaon2equal_2)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2106)) return -1;
		c40_1_bf.open(v_hascolor_1);
	case 1093:
	// Continuation for label 80
		while (c40_1_bf.fetch()) {
			v_kaon2equal_2 = v_hascolor_2;
			v_kaon2equal_1 = c40_1_bf.c2();
		if(kaon2equal_set.insert(v_kaon2equal_1, v_kaon2equal_2)) {
			if(!kaon2equal_bf.insert(v_kaon2equal_1, v_kaon2equal_2)) return -1;
			if(!kaon2equal_fb.insert(v_kaon2equal_2, v_kaon2equal_1)) return -1;
			if(!kaon2equal_ff.insert(v_kaon2equal_1, v_kaon2equal_2)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1093)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(283)) return -1;
			if(!backtrack_stack.push(282)) return -1;
			if(!backtrack_stack.push(281)) return -1;
			if(!backtrack_stack.push(280)) return -1;
			if(!backtrack_stack.push(279)) return -1;
			if(!backtrack_stack.push(278)) return -1;
			if(!backtrack_stack.push(277)) return -1;
			if(!backtrack_stack.push(276)) return -1;
			if(!backtrack_stack.push(275)) return -1;
			if(!backtrack_stack.push(274)) return -1;
			if(!backtrack_stack.push(273)) return -1;
			if(!backtrack_stack.push(271)) return -1;
			if(!backtrack_stack.push(270)) return -1;
			if(!backtrack_stack.push(269)) return -1;
			if(!backtrack_stack.push(268)) return -1;
			if(!backtrack_stack.push(267)) return -1;
			if(!backtrack_stack.push(266)) return -1;
			if(!backtrack_stack.push(265)) return -1;
			if(!backtrack_stack.push(264)) return -1;
			if(!backtrack_stack.push(263)) return -1;
			if(!backtrack_stack.push(262)) return -1;
			if(!backtrack_stack.push(261)) return -1;
			if(!backtrack_stack.push(260)) return -1;
			if(!backtrack_stack.push(254)) return -1;
			if(!backtrack_stack.push(253)) return -1;
			if(!backtrack_stack.push(152)) return -1;
			if(!backtrack_stack.push(142)) return -1;
			if(!backtrack_stack.push(141)) return -1;
			if(!backtrack_stack.push(140)) return -1;
			if(!backtrack_stack.push(139)) return -1;
			if(!backtrack_stack.push(138)) return -1;
			if(!backtrack_stack.push(137)) return -1;
			if(!backtrack_stack.push(134)) return -1;
			if(!backtrack_stack.push(133)) return -1;
			if(!backtrack_stack.push(127)) return -1;
			if(!backtrack_stack.push(126)) return -1;
			if(!backtrack_stack.push(125)) return -1;
			if(!backtrack_stack.push(124)) return -1;
			if(!backtrack_stack.push(85)) return -1;
			if(!backtrack_stack.push(82)) return -1;
			if(!backtrack_stack.push(81)) return -1;
			if(!backtrack_stack.push(73)) return -1;
			if(!backtrack_stack.push(72)) return -1;
			if(!backtrack_stack.push(69)) return -1;
			if(!backtrack_stack.push(62)) return -1;
			if(!backtrack_stack.push(57)) return -1;
			if(!backtrack_stack.push(56)) return -1;
			if(!backtrack_stack.push(55)) return -1;
			if(!backtrack_stack.push(45)) return -1;
			if(!backtrack_stack.push(42)) return -1;
			if(!backtrack_stack.push(41)) return -1;
			if(!backtrack_stack.push(35)) return -1;
			if(!backtrack_stack.push(34)) return -1;
			if(!backtrack_stack.push(33)) return -1;
			if(!backtrack_stack.push(29)) return -1;
			if(!backtrack_stack.push(28)) return -1;
			if(!backtrack_stack.push(21)) return -1;
			if(!backtrack_stack.push(18)) return -1;
			if(!backtrack_stack.push(13)) return -1;
			if(!backtrack_stack.push(5)) return -1;
			if(!backtrack_stack.push(4)) return -1;
			goto label_3;
		}
		}
		break;
	case 2106:
	// Label to restore values of label 80
		v_kaon2equal_2 = int_value_stack.pop();
		v_kaon2equal_1 = int_value_stack.pop();
		c40_1_bf.pop(&cursor_stack);
		break;
	case 81: label_81: 
	// using  symbolic fact kaon2equal(v_kaon2equal_1,v_kaon2equal_2) with literal 2 in rule 
	// 602: hascolor(A,B) :- hascolor(A,C), kaon2equal(B,C).
	// resulting in symbolic fact hascolor(v_hascolor_1,v_hascolor_2)

		//recursive
		c602_1_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_hascolor_1)) return -1;
		if(!int_value_stack.push(v_hascolor_2)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2107)) return -1;
		c602_1_fb.open(v_kaon2equal_2);
	case 1094:
	// Continuation for label 81
		while (c602_1_fb.fetch()) {
			v_hascolor_2 = v_kaon2equal_1;
			v_hascolor_1 = c602_1_fb.c2();
		if(hascolor_set.insert(v_hascolor_1, v_hascolor_2)) {
			if(!hascolor_bf.insert(v_hascolor_1, v_hascolor_2)) return -1;
			if(!hascolor_fb.insert(v_hascolor_2, v_hascolor_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1094)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(286)) return -1;
			if(!backtrack_stack.push(120)) return -1;
			if(!backtrack_stack.push(80)) return -1;
			if(!backtrack_stack.push(79)) return -1;
			if(!backtrack_stack.push(78)) return -1;
			if(!backtrack_stack.push(77)) return -1;
			if(!backtrack_stack.push(74)) return -1;
			goto label_68;
		}
		}
		break;
	case 2107:
	// Label to restore values of label 81
		v_hascolor_2 = int_value_stack.pop();
		v_hascolor_1 = int_value_stack.pop();
		c602_1_fb.pop(&cursor_stack);
		break;
	case 82: label_82: 
	// using  symbolic fact kaon2equal(v_kaon2equal_1,v_kaon2equal_2) with literal 2 in rule 
	// 796: madeintowine(A,B) :- madeintowine(C,B), kaon2equal(A,C).
	// resulting in symbolic fact madeintowine(v_madeintowine_1,v_madeintowine_2)

		//recursive
		c796_1_bf.push(&cursor_stack);
		if(!int_value_stack.push(v_madeintowine_2)) return -1;
		if(!int_value_stack.push(v_madeintowine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2108)) return -1;
		c796_1_bf.open(v_kaon2equal_2);
	case 1095:
	// Continuation for label 82
		while (c796_1_bf.fetch()) {
			v_madeintowine_1 = v_kaon2equal_1;
			v_madeintowine_2 = c796_1_bf.c2();
		if(madeintowine_set.insert(v_madeintowine_1, v_madeintowine_2)) {
			if(!madeintowine_bf.insert(v_madeintowine_1, v_madeintowine_2)) return -1;
			if(!madeintowine_fb.insert(v_madeintowine_2, v_madeintowine_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1095)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(84)) return -1;
			if(!backtrack_stack.push(83)) return -1;
			goto label_65;
		}
		}
		break;
	case 2108:
	// Label to restore values of label 82
		v_madeintowine_1 = int_value_stack.pop();
		v_madeintowine_2 = int_value_stack.pop();
		c796_1_bf.pop(&cursor_stack);
		break;
	case 83: label_83: 
	// using  symbolic fact madeintowine(v_madeintowine_1,v_madeintowine_2) with literal 1 in rule 
	// 796: madeintowine(A,B) :- madeintowine(C,B), kaon2equal(A,C).
	// resulting in symbolic fact madeintowine(v_madeintowine_1,v_madeintowine_2)

		//recursive
		c796_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_madeintowine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2109)) return -1;
		c796_2_fb.open(v_madeintowine_1);
	case 1096:
	// Continuation for label 83
		while (c796_2_fb.fetch()) {
			v_madeintowine_1 = c796_2_fb.c2();
		if(madeintowine_set.insert(v_madeintowine_1, v_madeintowine_2)) {
			if(!madeintowine_bf.insert(v_madeintowine_1, v_madeintowine_2)) return -1;
			if(!madeintowine_fb.insert(v_madeintowine_2, v_madeintowine_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1096)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(84)) return -1;
			if(!backtrack_stack.push(83)) return -1;
			goto label_65;
		}
		}
		break;
	case 2109:
	// Label to restore values of label 83
		v_madeintowine_1 = int_value_stack.pop();
		c796_2_fb.pop(&cursor_stack);
		break;
	case 84: label_84: 
	// using  symbolic fact madeintowine(v_madeintowine_1,v_madeintowine_2) with literal 1 in rule 
	// 797: madeintowine(A,B) :- madeintowine(A,C), kaon2equal(B,C).
	// resulting in symbolic fact madeintowine(v_madeintowine_1,v_madeintowine_2)

		//recursive
		c797_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_madeintowine_2)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2110)) return -1;
		c797_2_fb.open(v_madeintowine_2);
	case 1097:
	// Continuation for label 84
		while (c797_2_fb.fetch()) {
			v_madeintowine_2 = c797_2_fb.c2();
		if(madeintowine_set.insert(v_madeintowine_1, v_madeintowine_2)) {
			if(!madeintowine_bf.insert(v_madeintowine_1, v_madeintowine_2)) return -1;
			if(!madeintowine_fb.insert(v_madeintowine_2, v_madeintowine_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1097)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(84)) return -1;
			if(!backtrack_stack.push(83)) return -1;
			goto label_65;
		}
		}
		break;
	case 2110:
	// Label to restore values of label 84
		v_madeintowine_2 = int_value_stack.pop();
		c797_2_fb.pop(&cursor_stack);
		break;
	case 85: label_85: 
	// using  symbolic fact kaon2equal(v_kaon2equal_1,v_kaon2equal_2) with literal 2 in rule 
	// 797: madeintowine(A,B) :- madeintowine(A,C), kaon2equal(B,C).
	// resulting in symbolic fact madeintowine(v_madeintowine_1,v_madeintowine_2)

		//recursive
		c797_1_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_madeintowine_1)) return -1;
		if(!int_value_stack.push(v_madeintowine_2)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2111)) return -1;
		c797_1_fb.open(v_kaon2equal_2);
	case 1098:
	// Continuation for label 85
		while (c797_1_fb.fetch()) {
			v_madeintowine_2 = v_kaon2equal_1;
			v_madeintowine_1 = c797_1_fb.c2();
		if(madeintowine_set.insert(v_madeintowine_1, v_madeintowine_2)) {
			if(!madeintowine_bf.insert(v_madeintowine_1, v_madeintowine_2)) return -1;
			if(!madeintowine_fb.insert(v_madeintowine_2, v_madeintowine_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1098)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(84)) return -1;
			if(!backtrack_stack.push(83)) return -1;
			goto label_65;
		}
		}
		break;
	case 2111:
	// Label to restore values of label 85
		v_madeintowine_2 = int_value_stack.pop();
		v_madeintowine_1 = int_value_stack.pop();
		c797_1_fb.pop(&cursor_stack);
		break;
	case 86:
	// applying rule 
	// 12: cheninblanc(A) :- cheninblanc_aux(A).
	// resulting in symbolic fact cheninblanc(v_cheninblanc_1)
		c12_1_f.open();
	case 1099:
	// Continuation for label 86
		while (c12_1_f.fetch()) {
			v_cheninblanc_1 = c12_1_f.c1();
		if(cheninblanc_set.insert(v_cheninblanc_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1099)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(122)) return -1;
			if(!backtrack_stack.push(121)) return -1;
			if(!backtrack_stack.push(94)) return -1;
			if(!backtrack_stack.push(93)) return -1;
			if(!backtrack_stack.push(91)) return -1;
			goto label_87;
		}
		}
		break;
	case 87: label_87: 
	// using  symbolic fact cheninblanc(v_cheninblanc_1) with literal 1 in rule 
	// 11: q4(A) :- cheninblanc(A).
	// resulting in symbolic fact q4(v_q4_1)

		v_q4_1 = v_cheninblanc_1;
		if(q4_set.insert(v_q4_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(90)) return -1;
		if(!backtrack_stack.push(89)) return -1;
		goto label_88;
		}
		break;
	case 88: label_88: 
	// using  symbolic fact q4(v_q4_1) with literal 1 in rule 
	// 10: hasflavor(A,A) :- q4(A), kaon2namedobjects(A).
	// resulting in symbolic fact hasflavor(v_hasflavor_1,v_hasflavor_2)

		if (!kaon2namedobjects_b.contains(v_q4_1)) continue;
	case 1101:
	// Continuation for label 88
			v_hasflavor_1 = v_q4_1;
			v_hasflavor_2 = v_q4_1;
		if(hasflavor_set.insert(v_hasflavor_1, v_hasflavor_2)) {
			if(!hasflavor_bf.insert(v_hasflavor_1, v_hasflavor_2)) return -1;
			if(!hasflavor_fb.insert(v_hasflavor_2, v_hasflavor_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1101)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(32)) return -1;
			if(!backtrack_stack.push(31)) return -1;
			if(!backtrack_stack.push(30)) return -1;
			if(!backtrack_stack.push(14)) return -1;
			goto label_12;
		}
		break;
	case 89: label_89: 
	// using  symbolic fact q4(v_q4_1) with literal 1 in rule 
	// 488: wine(A) :- q4(A).
	// resulting in symbolic fact wine(v_wine_1)

		v_wine_1 = v_q4_1;
		if(wine_set.insert(v_wine_1)) {
		if(!wine_f.insert(v_wine_1)) return -1;
		// Labels for next rules
		if(!backtrack_stack.push(1007)) return -1;
		if(!backtrack_stack.push(989)) return -1;
		if(!backtrack_stack.push(954)) return -1;
		if(!backtrack_stack.push(938)) return -1;
		if(!backtrack_stack.push(911)) return -1;
		if(!backtrack_stack.push(903)) return -1;
		if(!backtrack_stack.push(902)) return -1;
		if(!backtrack_stack.push(901)) return -1;
		if(!backtrack_stack.push(849)) return -1;
		if(!backtrack_stack.push(848)) return -1;
		if(!backtrack_stack.push(821)) return -1;
		if(!backtrack_stack.push(820)) return -1;
		if(!backtrack_stack.push(737)) return -1;
		if(!backtrack_stack.push(712)) return -1;
		if(!backtrack_stack.push(674)) return -1;
		if(!backtrack_stack.push(672)) return -1;
		if(!backtrack_stack.push(583)) return -1;
		if(!backtrack_stack.push(575)) return -1;
		if(!backtrack_stack.push(563)) return -1;
		if(!backtrack_stack.push(551)) return -1;
		if(!backtrack_stack.push(549)) return -1;
		if(!backtrack_stack.push(483)) return -1;
		if(!backtrack_stack.push(473)) return -1;
		if(!backtrack_stack.push(470)) return -1;
		if(!backtrack_stack.push(430)) return -1;
		if(!backtrack_stack.push(401)) return -1;
		if(!backtrack_stack.push(399)) return -1;
		if(!backtrack_stack.push(380)) return -1;
		if(!backtrack_stack.push(364)) return -1;
		if(!backtrack_stack.push(363)) return -1;
		if(!backtrack_stack.push(329)) return -1;
		if(!backtrack_stack.push(328)) return -1;
		if(!backtrack_stack.push(327)) return -1;
		if(!backtrack_stack.push(295)) return -1;
		if(!backtrack_stack.push(294)) return -1;
		if(!backtrack_stack.push(216)) return -1;
		if(!backtrack_stack.push(210)) return -1;
		if(!backtrack_stack.push(192)) return -1;
		if(!backtrack_stack.push(180)) return -1;
		if(!backtrack_stack.push(179)) return -1;
		if(!backtrack_stack.push(178)) return -1;
		if(!backtrack_stack.push(154)) return -1;
		if(!backtrack_stack.push(153)) return -1;
		if(!backtrack_stack.push(100)) return -1;
		if(!backtrack_stack.push(76)) return -1;
		if(!backtrack_stack.push(75)) return -1;
		if(!backtrack_stack.push(67)) return -1;
		if(!backtrack_stack.push(63)) return -1;
		if(!backtrack_stack.push(53)) return -1;
		if(!backtrack_stack.push(46)) return -1;
		if(!backtrack_stack.push(19)) return -1;
		goto label_16;
		}
		break;
	case 90: label_90: 
	// using  symbolic fact q4(v_q4_1) with literal 1 in rule 
	// 492: q4(A) :- q4(B), kaon2equal(A,B).
	// resulting in symbolic fact q4(v_q4_1)

		//recursive
		c492_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q4_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2116)) return -1;
		c492_2_fb.open(v_q4_1);
	case 1103:
	// Continuation for label 90
		while (c492_2_fb.fetch()) {
			v_q4_1 = c492_2_fb.c2();
		if(q4_set.insert(v_q4_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1103)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(90)) return -1;
			if(!backtrack_stack.push(89)) return -1;
			goto label_88;
		}
		}
		break;
	case 2116:
	// Label to restore values of label 90
		v_q4_1 = int_value_stack.pop();
		c492_2_fb.pop(&cursor_stack);
		break;
	case 91: label_91: 
	// using  symbolic fact cheninblanc(v_cheninblanc_1) with literal 1 in rule 
	// 14: q6(A) :- cheninblanc(A).
	// resulting in symbolic fact q6(v_q6_1)

		//recursive
		if(!int_value_stack.push(v_q6_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2117)) return -1;
		v_q6_1 = v_cheninblanc_1;
		if(q6_set.insert(v_q6_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(146)) return -1;
		if(!backtrack_stack.push(145)) return -1;
		if(!backtrack_stack.push(130)) return -1;
		if(!backtrack_stack.push(129)) return -1;
		if(!backtrack_stack.push(128)) return -1;
		goto label_92;
		}
		break;
	case 2117:
	// Label to restore values of label 91
		v_q6_1 = int_value_stack.pop();
		break;
	case 92: label_92: 
	// using  symbolic fact q6(v_q6_1) with literal 1 in rule 
	// 13: cheninblanc(A) :- q6(A).
	// resulting in symbolic fact cheninblanc(v_cheninblanc_1)

		//recursive
		if(!int_value_stack.push(v_cheninblanc_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2118)) return -1;
		v_cheninblanc_1 = v_q6_1;
		if(cheninblanc_set.insert(v_cheninblanc_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(122)) return -1;
		if(!backtrack_stack.push(121)) return -1;
		if(!backtrack_stack.push(94)) return -1;
		if(!backtrack_stack.push(93)) return -1;
		if(!backtrack_stack.push(91)) return -1;
		goto label_87;
		}
		break;
	case 2118:
	// Label to restore values of label 92
		v_cheninblanc_1 = int_value_stack.pop();
		break;
	case 93: label_93: 
	// using  symbolic fact cheninblanc(v_cheninblanc_1) with literal 1 in rule 
	// 15: cheninblanc(A) :- cheninblanc(B), kaon2equal(A,B).
	// resulting in symbolic fact cheninblanc(v_cheninblanc_1)

		//recursive
		c15_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_cheninblanc_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2119)) return -1;
		c15_2_fb.open(v_cheninblanc_1);
	case 1106:
	// Continuation for label 93
		while (c15_2_fb.fetch()) {
			v_cheninblanc_1 = c15_2_fb.c2();
		if(cheninblanc_set.insert(v_cheninblanc_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1106)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(122)) return -1;
			if(!backtrack_stack.push(121)) return -1;
			if(!backtrack_stack.push(94)) return -1;
			if(!backtrack_stack.push(93)) return -1;
			if(!backtrack_stack.push(91)) return -1;
			goto label_87;
		}
		}
		break;
	case 2119:
	// Label to restore values of label 93
		v_cheninblanc_1 = int_value_stack.pop();
		c15_2_fb.pop(&cursor_stack);
		break;
	case 94: label_94: 
	// using  symbolic fact cheninblanc(v_cheninblanc_1) with literal 1 in rule 
	// 399: q12(A) :- cheninblanc(A).
	// resulting in symbolic fact q12(v_q12_1)

		v_q12_1 = v_cheninblanc_1;
		if(q12_set.insert(v_q12_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(116)) return -1;
		if(!backtrack_stack.push(115)) return -1;
		if(!backtrack_stack.push(114)) return -1;
		if(!backtrack_stack.push(99)) return -1;
		goto label_95;
		}
		break;
	case 95: label_95: 
	// using  symbolic fact q12(v_q12_1) with literal 1 in rule 
	// 253: q21(A) :- q12(A).
	// resulting in symbolic fact q21(v_q21_1)

		//recursive
		if(!int_value_stack.push(v_q21_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2121)) return -1;
		v_q21_1 = v_q12_1;
		if(q21_set.insert(v_q21_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(119)) return -1;
		goto label_96;
		}
		break;
	case 2121:
	// Label to restore values of label 95
		v_q21_1 = int_value_stack.pop();
		break;
	case 96: label_96: 
	// using  symbolic fact q21(v_q21_1) with literal 1 in rule 
	// 454: whitewine(A) :- q21(A), wine(A).
	// resulting in symbolic fact whitewine(v_whitewine_1)

		//recursive
		if(!int_value_stack.push(v_whitewine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2122)) return -1;
		if (!wine_set.contains(v_q21_1)) continue;
	case 1109:
	// Continuation for label 96
			v_whitewine_1 = v_q21_1;
		if(whitewine_set.insert(v_whitewine_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1109)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(560)) return -1;
			if(!backtrack_stack.push(491)) return -1;
			if(!backtrack_stack.push(374)) return -1;
			if(!backtrack_stack.push(235)) return -1;
			if(!backtrack_stack.push(113)) return -1;
			if(!backtrack_stack.push(112)) return -1;
			if(!backtrack_stack.push(104)) return -1;
			if(!backtrack_stack.push(103)) return -1;
			if(!backtrack_stack.push(101)) return -1;
			if(!backtrack_stack.push(98)) return -1;
			goto label_97;
		}
		break;
	case 2122:
	// Label to restore values of label 96
		v_whitewine_1 = int_value_stack.pop();
		break;
	case 97: label_97: 
	// using  symbolic fact whitewine(v_whitewine_1) with literal 1 in rule 
	// 295: hassugar(A,A) :- whitewine(A), kaon2namedobjects(A).
	// resulting in symbolic fact hassugar(v_hassugar_1,v_hassugar_2)

		if (!kaon2namedobjects_b.contains(v_whitewine_1)) continue;
	case 1110:
	// Continuation for label 97
			v_hassugar_1 = v_whitewine_1;
			v_hassugar_2 = v_whitewine_1;
		if(hassugar_set.insert(v_hassugar_1, v_hassugar_2)) {
			if(!hassugar_bf.insert(v_hassugar_1, v_hassugar_2)) return -1;
			if(!hassugar_fb.insert(v_hassugar_2, v_hassugar_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1110)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(755)) return -1;
			if(!backtrack_stack.push(238)) return -1;
			if(!backtrack_stack.push(214)) return -1;
			if(!backtrack_stack.push(27)) return -1;
			if(!backtrack_stack.push(24)) return -1;
			if(!backtrack_stack.push(23)) return -1;
			if(!backtrack_stack.push(22)) return -1;
			goto label_17;
		}
		break;
	case 98: label_98: 
	// using  symbolic fact whitewine(v_whitewine_1) with literal 1 in rule 
	// 449: q12(A) :- whitewine(A).
	// resulting in symbolic fact q12(v_q12_1)

		//recursive
		if(!int_value_stack.push(v_q12_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2124)) return -1;
		v_q12_1 = v_whitewine_1;
		if(q12_set.insert(v_q12_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(116)) return -1;
		if(!backtrack_stack.push(115)) return -1;
		if(!backtrack_stack.push(114)) return -1;
		if(!backtrack_stack.push(99)) return -1;
		goto label_95;
		}
		break;
	case 2124:
	// Label to restore values of label 98
		v_q12_1 = int_value_stack.pop();
		break;
	case 99: label_99: 
	// using  symbolic fact q12(v_q12_1) with literal 1 in rule 
	// 396: wine(A) :- q12(A).
	// resulting in symbolic fact wine(v_wine_1)

		//recursive
		if(!int_value_stack.push(v_wine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2125)) return -1;
		v_wine_1 = v_q12_1;
		if(wine_set.insert(v_wine_1)) {
		if(!wine_f.insert(v_wine_1)) return -1;
		// Labels for next rules
		if(!backtrack_stack.push(1007)) return -1;
		if(!backtrack_stack.push(989)) return -1;
		if(!backtrack_stack.push(954)) return -1;
		if(!backtrack_stack.push(938)) return -1;
		if(!backtrack_stack.push(911)) return -1;
		if(!backtrack_stack.push(903)) return -1;
		if(!backtrack_stack.push(902)) return -1;
		if(!backtrack_stack.push(901)) return -1;
		if(!backtrack_stack.push(849)) return -1;
		if(!backtrack_stack.push(848)) return -1;
		if(!backtrack_stack.push(821)) return -1;
		if(!backtrack_stack.push(820)) return -1;
		if(!backtrack_stack.push(737)) return -1;
		if(!backtrack_stack.push(712)) return -1;
		if(!backtrack_stack.push(674)) return -1;
		if(!backtrack_stack.push(672)) return -1;
		if(!backtrack_stack.push(583)) return -1;
		if(!backtrack_stack.push(575)) return -1;
		if(!backtrack_stack.push(563)) return -1;
		if(!backtrack_stack.push(551)) return -1;
		if(!backtrack_stack.push(549)) return -1;
		if(!backtrack_stack.push(483)) return -1;
		if(!backtrack_stack.push(473)) return -1;
		if(!backtrack_stack.push(470)) return -1;
		if(!backtrack_stack.push(430)) return -1;
		if(!backtrack_stack.push(401)) return -1;
		if(!backtrack_stack.push(399)) return -1;
		if(!backtrack_stack.push(380)) return -1;
		if(!backtrack_stack.push(364)) return -1;
		if(!backtrack_stack.push(363)) return -1;
		if(!backtrack_stack.push(329)) return -1;
		if(!backtrack_stack.push(328)) return -1;
		if(!backtrack_stack.push(327)) return -1;
		if(!backtrack_stack.push(295)) return -1;
		if(!backtrack_stack.push(294)) return -1;
		if(!backtrack_stack.push(216)) return -1;
		if(!backtrack_stack.push(210)) return -1;
		if(!backtrack_stack.push(192)) return -1;
		if(!backtrack_stack.push(180)) return -1;
		if(!backtrack_stack.push(179)) return -1;
		if(!backtrack_stack.push(178)) return -1;
		if(!backtrack_stack.push(154)) return -1;
		if(!backtrack_stack.push(153)) return -1;
		if(!backtrack_stack.push(100)) return -1;
		if(!backtrack_stack.push(76)) return -1;
		if(!backtrack_stack.push(75)) return -1;
		if(!backtrack_stack.push(67)) return -1;
		if(!backtrack_stack.push(63)) return -1;
		if(!backtrack_stack.push(53)) return -1;
		if(!backtrack_stack.push(46)) return -1;
		if(!backtrack_stack.push(19)) return -1;
		goto label_16;
		}
		break;
	case 2125:
	// Label to restore values of label 99
		v_wine_1 = int_value_stack.pop();
		break;
	case 100: label_100: 
	// using  symbolic fact wine(v_wine_1) with literal 2 in rule 
	// 454: whitewine(A) :- q21(A), wine(A).
	// resulting in symbolic fact whitewine(v_whitewine_1)

		//recursive
		if(!int_value_stack.push(v_whitewine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2126)) return -1;
		if (!q21_set.contains(v_wine_1)) continue;
	case 1113:
	// Continuation for label 100
			v_whitewine_1 = v_wine_1;
		if(whitewine_set.insert(v_whitewine_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1113)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(560)) return -1;
			if(!backtrack_stack.push(491)) return -1;
			if(!backtrack_stack.push(374)) return -1;
			if(!backtrack_stack.push(235)) return -1;
			if(!backtrack_stack.push(113)) return -1;
			if(!backtrack_stack.push(112)) return -1;
			if(!backtrack_stack.push(104)) return -1;
			if(!backtrack_stack.push(103)) return -1;
			if(!backtrack_stack.push(101)) return -1;
			if(!backtrack_stack.push(98)) return -1;
			goto label_97;
		}
		break;
	case 2126:
	// Label to restore values of label 100
		v_whitewine_1 = int_value_stack.pop();
		break;
	case 101: label_101: 
	// using  symbolic fact whitewine(v_whitewine_1) with literal 1 in rule 
	// 453: whitenonsweetwine(A) :- whitewine(A), kaon2namedobjects(A).
	// resulting in symbolic fact whitenonsweetwine(v_whitenonsweetwine_1)

		//recursive
		if(!int_value_stack.push(v_whitenonsweetwine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2127)) return -1;
		if (!kaon2namedobjects_b.contains(v_whitewine_1)) continue;
	case 1114:
	// Continuation for label 101
			v_whitenonsweetwine_1 = v_whitewine_1;
		if(whitenonsweetwine_set.insert(v_whitenonsweetwine_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1114)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(107)) return -1;
			if(!backtrack_stack.push(105)) return -1;
			goto label_102;
		}
		break;
	case 2127:
	// Label to restore values of label 101
		v_whitenonsweetwine_1 = int_value_stack.pop();
		break;
	case 102: label_102: 
	// using  symbolic fact whitenonsweetwine(v_whitenonsweetwine_1) with literal 1 in rule 
	// 452: whitewine(A) :- whitenonsweetwine(A).
	// resulting in symbolic fact whitewine(v_whitewine_1)

		//recursive
		if(!int_value_stack.push(v_whitewine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2128)) return -1;
		v_whitewine_1 = v_whitenonsweetwine_1;
		if(whitewine_set.insert(v_whitewine_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(560)) return -1;
		if(!backtrack_stack.push(491)) return -1;
		if(!backtrack_stack.push(374)) return -1;
		if(!backtrack_stack.push(235)) return -1;
		if(!backtrack_stack.push(113)) return -1;
		if(!backtrack_stack.push(112)) return -1;
		if(!backtrack_stack.push(104)) return -1;
		if(!backtrack_stack.push(103)) return -1;
		if(!backtrack_stack.push(101)) return -1;
		if(!backtrack_stack.push(98)) return -1;
		goto label_97;
		}
		break;
	case 2128:
	// Label to restore values of label 102
		v_whitewine_1 = int_value_stack.pop();
		break;
	case 103: label_103: 
	// using  symbolic fact whitewine(v_whitewine_1) with literal 1 in rule 
	// 456: wine(A) :- whitewine(A).
	// resulting in symbolic fact wine(v_wine_1)

		//recursive
		if(!int_value_stack.push(v_wine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2129)) return -1;
		v_wine_1 = v_whitewine_1;
		if(wine_set.insert(v_wine_1)) {
		if(!wine_f.insert(v_wine_1)) return -1;
		// Labels for next rules
		if(!backtrack_stack.push(1007)) return -1;
		if(!backtrack_stack.push(989)) return -1;
		if(!backtrack_stack.push(954)) return -1;
		if(!backtrack_stack.push(938)) return -1;
		if(!backtrack_stack.push(911)) return -1;
		if(!backtrack_stack.push(903)) return -1;
		if(!backtrack_stack.push(902)) return -1;
		if(!backtrack_stack.push(901)) return -1;
		if(!backtrack_stack.push(849)) return -1;
		if(!backtrack_stack.push(848)) return -1;
		if(!backtrack_stack.push(821)) return -1;
		if(!backtrack_stack.push(820)) return -1;
		if(!backtrack_stack.push(737)) return -1;
		if(!backtrack_stack.push(712)) return -1;
		if(!backtrack_stack.push(674)) return -1;
		if(!backtrack_stack.push(672)) return -1;
		if(!backtrack_stack.push(583)) return -1;
		if(!backtrack_stack.push(575)) return -1;
		if(!backtrack_stack.push(563)) return -1;
		if(!backtrack_stack.push(551)) return -1;
		if(!backtrack_stack.push(549)) return -1;
		if(!backtrack_stack.push(483)) return -1;
		if(!backtrack_stack.push(473)) return -1;
		if(!backtrack_stack.push(470)) return -1;
		if(!backtrack_stack.push(430)) return -1;
		if(!backtrack_stack.push(401)) return -1;
		if(!backtrack_stack.push(399)) return -1;
		if(!backtrack_stack.push(380)) return -1;
		if(!backtrack_stack.push(364)) return -1;
		if(!backtrack_stack.push(363)) return -1;
		if(!backtrack_stack.push(329)) return -1;
		if(!backtrack_stack.push(328)) return -1;
		if(!backtrack_stack.push(327)) return -1;
		if(!backtrack_stack.push(295)) return -1;
		if(!backtrack_stack.push(294)) return -1;
		if(!backtrack_stack.push(216)) return -1;
		if(!backtrack_stack.push(210)) return -1;
		if(!backtrack_stack.push(192)) return -1;
		if(!backtrack_stack.push(180)) return -1;
		if(!backtrack_stack.push(179)) return -1;
		if(!backtrack_stack.push(178)) return -1;
		if(!backtrack_stack.push(154)) return -1;
		if(!backtrack_stack.push(153)) return -1;
		if(!backtrack_stack.push(100)) return -1;
		if(!backtrack_stack.push(76)) return -1;
		if(!backtrack_stack.push(75)) return -1;
		if(!backtrack_stack.push(67)) return -1;
		if(!backtrack_stack.push(63)) return -1;
		if(!backtrack_stack.push(53)) return -1;
		if(!backtrack_stack.push(46)) return -1;
		if(!backtrack_stack.push(19)) return -1;
		goto label_16;
		}
		break;
	case 2129:
	// Label to restore values of label 103
		v_wine_1 = int_value_stack.pop();
		break;
	case 104: label_104: 
	// using  symbolic fact whitewine(v_whitewine_1) with literal 1 in rule 
	// 457: whitewine(A) :- whitewine(B), kaon2equal(A,B).
	// resulting in symbolic fact whitewine(v_whitewine_1)

		//recursive
		c457_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_whitewine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2130)) return -1;
		c457_2_fb.open(v_whitewine_1);
	case 1117:
	// Continuation for label 104
		while (c457_2_fb.fetch()) {
			v_whitewine_1 = c457_2_fb.c2();
		if(whitewine_set.insert(v_whitewine_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1117)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(560)) return -1;
			if(!backtrack_stack.push(491)) return -1;
			if(!backtrack_stack.push(374)) return -1;
			if(!backtrack_stack.push(235)) return -1;
			if(!backtrack_stack.push(113)) return -1;
			if(!backtrack_stack.push(112)) return -1;
			if(!backtrack_stack.push(104)) return -1;
			if(!backtrack_stack.push(103)) return -1;
			if(!backtrack_stack.push(101)) return -1;
			if(!backtrack_stack.push(98)) return -1;
			goto label_97;
		}
		}
		break;
	case 2130:
	// Label to restore values of label 104
		v_whitewine_1 = int_value_stack.pop();
		c457_2_fb.pop(&cursor_stack);
		break;
	case 105: label_105: 
	// using  symbolic fact whitenonsweetwine(v_whitenonsweetwine_1) with literal 1 in rule 
	// 583: q36(A) :- whitenonsweetwine(A).
	// resulting in symbolic fact q36(v_q36_1)

		//recursive
		if(!int_value_stack.push(v_q36_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2131)) return -1;
		v_q36_1 = v_whitenonsweetwine_1;
		if(q36_set.insert(v_q36_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(111)) return -1;
		if(!backtrack_stack.push(108)) return -1;
		goto label_106;
		}
		break;
	case 2131:
	// Label to restore values of label 105
		v_q36_1 = int_value_stack.pop();
		break;
	case 106: label_106: 
	// using  symbolic fact q36(v_q36_1) with literal 1 in rule 
	// 582: whitenonsweetwine(A) :- q36(A), whitewine(A).
	// resulting in symbolic fact whitenonsweetwine(v_whitenonsweetwine_1)

		//recursive
		if(!int_value_stack.push(v_whitenonsweetwine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2132)) return -1;
		if (!whitewine_set.contains(v_q36_1)) continue;
	case 1119:
	// Continuation for label 106
			v_whitenonsweetwine_1 = v_q36_1;
		if(whitenonsweetwine_set.insert(v_whitenonsweetwine_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1119)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(107)) return -1;
			if(!backtrack_stack.push(105)) return -1;
			goto label_102;
		}
		break;
	case 2132:
	// Label to restore values of label 106
		v_whitenonsweetwine_1 = int_value_stack.pop();
		break;
	case 107: label_107: 
	// using  symbolic fact whitenonsweetwine(v_whitenonsweetwine_1) with literal 1 in rule 
	// 584: whitenonsweetwine(A) :- whitenonsweetwine(B), kaon2equal(A,B).
	// resulting in symbolic fact whitenonsweetwine(v_whitenonsweetwine_1)

		//recursive
		c584_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_whitenonsweetwine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2133)) return -1;
		c584_2_fb.open(v_whitenonsweetwine_1);
	case 1120:
	// Continuation for label 107
		while (c584_2_fb.fetch()) {
			v_whitenonsweetwine_1 = c584_2_fb.c2();
		if(whitenonsweetwine_set.insert(v_whitenonsweetwine_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1120)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(107)) return -1;
			if(!backtrack_stack.push(105)) return -1;
			goto label_102;
		}
		}
		break;
	case 2133:
	// Label to restore values of label 107
		v_whitenonsweetwine_1 = int_value_stack.pop();
		c584_2_fb.pop(&cursor_stack);
		break;
	case 108: label_108: 
	// using  symbolic fact q36(v_q36_1) with literal 1 in rule 
	// 587: ot____nom7(A) :- q36(B), hassugar(B,A).
	// resulting in symbolic fact ot____nom7(v_ot____nom7_1)

		//recursive
		c587_2_bf.push(&cursor_stack);
		if(!int_value_stack.push(v_ot____nom7_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2134)) return -1;
		c587_2_bf.open(v_q36_1);
	case 1121:
	// Continuation for label 108
		while (c587_2_bf.fetch()) {
			v_ot____nom7_1 = c587_2_bf.c2();
		if(ot____nom7_set.insert(v_ot____nom7_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1121)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(110)) return -1;
			goto label_109;
		}
		}
		break;
	case 2134:
	// Label to restore values of label 108
		v_ot____nom7_1 = int_value_stack.pop();
		c587_2_bf.pop(&cursor_stack);
		break;
	case 109: label_109: 
	// using  symbolic fact ot____nom7(v_ot____nom7_1) with literal 1 in rule 
	// 585: whitenonsweetwine(A) :- ot____nom7(A), whitewine(A), kaon2namedobjects(A).
	// resulting in symbolic fact whitenonsweetwine(v_whitenonsweetwine_1)

		//recursive
		if(!int_value_stack.push(v_whitenonsweetwine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2135)) return -1;
		if (!whitewine_set.contains(v_ot____nom7_1)) continue;
			if (!kaon2namedobjects_b.contains(v_ot____nom7_1)) continue;
	case 1122:
	// Continuation for label 109
				v_whitenonsweetwine_1 = v_ot____nom7_1;
		if(whitenonsweetwine_set.insert(v_whitenonsweetwine_1)) {
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1122)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(107)) return -1;
				if(!backtrack_stack.push(105)) return -1;
				goto label_102;
		}
		break;
	case 2135:
	// Label to restore values of label 109
		v_whitenonsweetwine_1 = int_value_stack.pop();
		break;
	case 110: label_110: 
	// using  symbolic fact ot____nom7(v_ot____nom7_1) with literal 1 in rule 
	// 595: ot____nom7(A) :- ot____nom7(B), kaon2equal(A,B).
	// resulting in symbolic fact ot____nom7(v_ot____nom7_1)

		//recursive
		c595_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_ot____nom7_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2136)) return -1;
		c595_2_fb.open(v_ot____nom7_1);
	case 1123:
	// Continuation for label 110
		while (c595_2_fb.fetch()) {
			v_ot____nom7_1 = c595_2_fb.c2();
		if(ot____nom7_set.insert(v_ot____nom7_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1123)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(110)) return -1;
			goto label_109;
		}
		}
		break;
	case 2136:
	// Label to restore values of label 110
		v_ot____nom7_1 = int_value_stack.pop();
		c595_2_fb.pop(&cursor_stack);
		break;
	case 111: label_111: 
	// using  symbolic fact q36(v_q36_1) with literal 1 in rule 
	// 590: q36(A) :- q36(B), kaon2equal(A,B).
	// resulting in symbolic fact q36(v_q36_1)

		//recursive
		c590_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q36_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2137)) return -1;
		c590_2_fb.open(v_q36_1);
	case 1124:
	// Continuation for label 111
		while (c590_2_fb.fetch()) {
			v_q36_1 = c590_2_fb.c2();
		if(q36_set.insert(v_q36_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1124)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(111)) return -1;
			if(!backtrack_stack.push(108)) return -1;
			goto label_106;
		}
		}
		break;
	case 2137:
	// Label to restore values of label 111
		v_q36_1 = int_value_stack.pop();
		c590_2_fb.pop(&cursor_stack);
		break;
	case 112: label_112: 
	// using  symbolic fact whitewine(v_whitewine_1) with literal 2 in rule 
	// 582: whitenonsweetwine(A) :- q36(A), whitewine(A).
	// resulting in symbolic fact whitenonsweetwine(v_whitenonsweetwine_1)

		if (!q36_set.contains(v_whitewine_1)) continue;
	case 1125:
	// Continuation for label 112
			v_whitenonsweetwine_1 = v_whitewine_1;
		if(whitenonsweetwine_set.insert(v_whitenonsweetwine_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1125)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(107)) return -1;
			if(!backtrack_stack.push(105)) return -1;
			goto label_102;
		}
		break;
	case 113: label_113: 
	// using  symbolic fact whitewine(v_whitewine_1) with literal 2 in rule 
	// 585: whitenonsweetwine(A) :- ot____nom7(A), whitewine(A), kaon2namedobjects(A).
	// resulting in symbolic fact whitenonsweetwine(v_whitenonsweetwine_1)

		if (!ot____nom7_set.contains(v_whitewine_1)) continue;
			if (!kaon2namedobjects_b.contains(v_whitewine_1)) continue;
	case 1126:
	// Continuation for label 113
				v_whitenonsweetwine_1 = v_whitewine_1;
		if(whitenonsweetwine_set.insert(v_whitenonsweetwine_1)) {
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1126)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(107)) return -1;
				if(!backtrack_stack.push(105)) return -1;
				goto label_102;
		}
		break;
	case 114: label_114: 
	// using  symbolic fact q12(v_q12_1) with literal 1 in rule 
	// 447: hascolor(A,A) :- q12(A), kaon2namedobjects(A).
	// resulting in symbolic fact hascolor(v_hascolor_1,v_hascolor_2)

		if (!kaon2namedobjects_b.contains(v_q12_1)) continue;
	case 1127:
	// Continuation for label 114
			v_hascolor_1 = v_q12_1;
			v_hascolor_2 = v_q12_1;
		if(hascolor_set.insert(v_hascolor_1, v_hascolor_2)) {
			if(!hascolor_bf.insert(v_hascolor_1, v_hascolor_2)) return -1;
			if(!hascolor_fb.insert(v_hascolor_2, v_hascolor_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1127)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(286)) return -1;
			if(!backtrack_stack.push(120)) return -1;
			if(!backtrack_stack.push(80)) return -1;
			if(!backtrack_stack.push(79)) return -1;
			if(!backtrack_stack.push(78)) return -1;
			if(!backtrack_stack.push(77)) return -1;
			if(!backtrack_stack.push(74)) return -1;
			goto label_68;
		}
		break;
	case 115: label_115: 
	// using  symbolic fact q12(v_q12_1) with literal 1 in rule 
	// 600: q12(A) :- q12(B), kaon2equal(A,B).
	// resulting in symbolic fact q12(v_q12_1)

		//recursive
		c600_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q12_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2141)) return -1;
		c600_2_fb.open(v_q12_1);
	case 1128:
	// Continuation for label 115
		while (c600_2_fb.fetch()) {
			v_q12_1 = c600_2_fb.c2();
		if(q12_set.insert(v_q12_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1128)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(116)) return -1;
			if(!backtrack_stack.push(115)) return -1;
			if(!backtrack_stack.push(114)) return -1;
			if(!backtrack_stack.push(99)) return -1;
			goto label_95;
		}
		}
		break;
	case 2141:
	// Label to restore values of label 115
		v_q12_1 = int_value_stack.pop();
		c600_2_fb.pop(&cursor_stack);
		break;
	case 116: label_116: 
	// using  symbolic fact q12(v_q12_1) with literal 1 in rule 
	// 657: ot____nom5(A) :- q12(A), kaon2namedobjects(A).
	// resulting in symbolic fact ot____nom5(v_ot____nom5_1)

		//recursive
		if(!int_value_stack.push(v_ot____nom5_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2142)) return -1;
		if (!kaon2namedobjects_b.contains(v_q12_1)) continue;
	case 1129:
	// Continuation for label 116
			v_ot____nom5_1 = v_q12_1;
		if(ot____nom5_set.insert(v_ot____nom5_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1129)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(118)) return -1;
			goto label_117;
		}
		break;
	case 2142:
	// Label to restore values of label 116
		v_ot____nom5_1 = int_value_stack.pop();
		break;
	case 117: label_117: 
	// using  symbolic fact ot____nom5(v_ot____nom5_1) with literal 1 in rule 
	// 658: ot____nom5(A) :- ot____nom5(B), kaon2equal(A,B).
	// resulting in symbolic fact ot____nom5(v_ot____nom5_1)

		//recursive
		c658_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_ot____nom5_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2143)) return -1;
		c658_2_fb.open(v_ot____nom5_1);
	case 1130:
	// Continuation for label 117
		while (c658_2_fb.fetch()) {
			v_ot____nom5_1 = c658_2_fb.c2();
		if(ot____nom5_set.insert(v_ot____nom5_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1130)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(118)) return -1;
			goto label_117;
		}
		}
		break;
	case 2143:
	// Label to restore values of label 117
		v_ot____nom5_1 = int_value_stack.pop();
		c658_2_fb.pop(&cursor_stack);
		break;
	case 118: label_118: 
	// using  symbolic fact ot____nom5(v_ot____nom5_1) with literal 2 in rule 
	// 258: q21(A) :- hascolor(A,B), ot____nom5(B).
	// resulting in symbolic fact q21(v_q21_1)

		//recursive
		c258_1_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q21_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2144)) return -1;
		c258_1_fb.open(v_ot____nom5_1);
	case 1131:
	// Continuation for label 118
		while (c258_1_fb.fetch()) {
			v_q21_1 = c258_1_fb.c2();
		if(q21_set.insert(v_q21_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1131)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(119)) return -1;
			goto label_96;
		}
		}
		break;
	case 2144:
	// Label to restore values of label 118
		v_q21_1 = int_value_stack.pop();
		c258_1_fb.pop(&cursor_stack);
		break;
	case 119: label_119: 
	// using  symbolic fact q21(v_q21_1) with literal 1 in rule 
	// 455: q21(A) :- q21(B), kaon2equal(A,B).
	// resulting in symbolic fact q21(v_q21_1)

		//recursive
		c455_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q21_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2145)) return -1;
		c455_2_fb.open(v_q21_1);
	case 1132:
	// Continuation for label 119
		while (c455_2_fb.fetch()) {
			v_q21_1 = c455_2_fb.c2();
		if(q21_set.insert(v_q21_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1132)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(119)) return -1;
			goto label_96;
		}
		}
		break;
	case 2145:
	// Label to restore values of label 119
		v_q21_1 = int_value_stack.pop();
		c455_2_fb.pop(&cursor_stack);
		break;
	case 120: label_120: 
	// using  symbolic fact hascolor(v_hascolor_1,v_hascolor_2) with literal 1 in rule 
	// 258: q21(A) :- hascolor(A,B), ot____nom5(B).
	// resulting in symbolic fact q21(v_q21_1)

		if (!ot____nom5_set.contains(v_hascolor_2)) continue;
	case 1133:
	// Continuation for label 120
			v_q21_1 = v_hascolor_1;
		if(q21_set.insert(v_q21_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1133)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(119)) return -1;
			goto label_96;
		}
		break;
	case 121: label_121: 
	// using  symbolic fact cheninblanc(v_cheninblanc_1) with literal 1 in rule 
	// 589: q36(A) :- cheninblanc(A).
	// resulting in symbolic fact q36(v_q36_1)

		v_q36_1 = v_cheninblanc_1;
		if(q36_set.insert(v_q36_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(111)) return -1;
		if(!backtrack_stack.push(108)) return -1;
		goto label_106;
		}
		break;
	case 122: label_122: 
	// using  symbolic fact cheninblanc(v_cheninblanc_1) with literal 1 in rule 
	// 775: q3(A) :- cheninblanc(A).
	// resulting in symbolic fact q3(v_q3_1)

		//recursive
		if(!int_value_stack.push(v_q3_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2148)) return -1;
		v_q3_1 = v_cheninblanc_1;
		if(q3_set.insert(v_q3_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(143)) return -1;
		goto label_123;
		}
		break;
	case 2148:
	// Label to restore values of label 122
		v_q3_1 = int_value_stack.pop();
		break;
	case 123: label_123: 
	// using  symbolic fact q3(v_q3_1) with literal 1 in rule 
	// 88: kaon2equal(A,B) :- q3(C), madefromgrape(C,A), madefromgrape(C,B).
	// resulting in symbolic fact kaon2equal(v_kaon2equal_1,v_kaon2equal_2)

		//recursive
		c88_3_bf.push(&cursor_stack);
		c88_2_bf.push(&cursor_stack);
		if(!int_value_stack.push(v_kaon2equal_1)) return -1;
		if(!int_value_stack.push(v_kaon2equal_2)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2149)) return -1;
		c88_2_bf.open(v_q3_1);
		while (c88_2_bf.fetch()) {
			c88_3_bf.open(v_q3_1);
	case 1136:
	// Continuation for label 123
			while (c88_3_bf.fetch()) {
				v_kaon2equal_2 = c88_3_bf.c2();
				v_kaon2equal_1 = c88_2_bf.c2();
		if(kaon2equal_set.insert(v_kaon2equal_1, v_kaon2equal_2)) {
				if(!kaon2equal_bf.insert(v_kaon2equal_1, v_kaon2equal_2)) return -1;
				if(!kaon2equal_fb.insert(v_kaon2equal_2, v_kaon2equal_1)) return -1;
				if(!kaon2equal_ff.insert(v_kaon2equal_1, v_kaon2equal_2)) return -1;
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1136)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(283)) return -1;
				if(!backtrack_stack.push(282)) return -1;
				if(!backtrack_stack.push(281)) return -1;
				if(!backtrack_stack.push(280)) return -1;
				if(!backtrack_stack.push(279)) return -1;
				if(!backtrack_stack.push(278)) return -1;
				if(!backtrack_stack.push(277)) return -1;
				if(!backtrack_stack.push(276)) return -1;
				if(!backtrack_stack.push(275)) return -1;
				if(!backtrack_stack.push(274)) return -1;
				if(!backtrack_stack.push(273)) return -1;
				if(!backtrack_stack.push(271)) return -1;
				if(!backtrack_stack.push(270)) return -1;
				if(!backtrack_stack.push(269)) return -1;
				if(!backtrack_stack.push(268)) return -1;
				if(!backtrack_stack.push(267)) return -1;
				if(!backtrack_stack.push(266)) return -1;
				if(!backtrack_stack.push(265)) return -1;
				if(!backtrack_stack.push(264)) return -1;
				if(!backtrack_stack.push(263)) return -1;
				if(!backtrack_stack.push(262)) return -1;
				if(!backtrack_stack.push(261)) return -1;
				if(!backtrack_stack.push(260)) return -1;
				if(!backtrack_stack.push(254)) return -1;
				if(!backtrack_stack.push(253)) return -1;
				if(!backtrack_stack.push(152)) return -1;
				if(!backtrack_stack.push(142)) return -1;
				if(!backtrack_stack.push(141)) return -1;
				if(!backtrack_stack.push(140)) return -1;
				if(!backtrack_stack.push(139)) return -1;
				if(!backtrack_stack.push(138)) return -1;
				if(!backtrack_stack.push(137)) return -1;
				if(!backtrack_stack.push(134)) return -1;
				if(!backtrack_stack.push(133)) return -1;
				if(!backtrack_stack.push(127)) return -1;
				if(!backtrack_stack.push(126)) return -1;
				if(!backtrack_stack.push(125)) return -1;
				if(!backtrack_stack.push(124)) return -1;
				if(!backtrack_stack.push(85)) return -1;
				if(!backtrack_stack.push(82)) return -1;
				if(!backtrack_stack.push(81)) return -1;
				if(!backtrack_stack.push(73)) return -1;
				if(!backtrack_stack.push(72)) return -1;
				if(!backtrack_stack.push(69)) return -1;
				if(!backtrack_stack.push(62)) return -1;
				if(!backtrack_stack.push(57)) return -1;
				if(!backtrack_stack.push(56)) return -1;
				if(!backtrack_stack.push(55)) return -1;
				if(!backtrack_stack.push(45)) return -1;
				if(!backtrack_stack.push(42)) return -1;
				if(!backtrack_stack.push(41)) return -1;
				if(!backtrack_stack.push(35)) return -1;
				if(!backtrack_stack.push(34)) return -1;
				if(!backtrack_stack.push(33)) return -1;
				if(!backtrack_stack.push(29)) return -1;
				if(!backtrack_stack.push(28)) return -1;
				if(!backtrack_stack.push(21)) return -1;
				if(!backtrack_stack.push(18)) return -1;
				if(!backtrack_stack.push(13)) return -1;
				if(!backtrack_stack.push(5)) return -1;
				if(!backtrack_stack.push(4)) return -1;
				goto label_3;
		}
			}
		}
		break;
	case 2149:
	// Label to restore values of label 123
		v_kaon2equal_2 = int_value_stack.pop();
		v_kaon2equal_1 = int_value_stack.pop();
		c88_2_bf.pop(&cursor_stack);
		c88_3_bf.pop(&cursor_stack);
		break;
	case 124: label_124: 
	// using  symbolic fact kaon2equal(v_kaon2equal_1,v_kaon2equal_2) with literal 2 in rule 
	// 15: cheninblanc(A) :- cheninblanc(B), kaon2equal(A,B).
	// resulting in symbolic fact cheninblanc(v_cheninblanc_1)

		//recursive
		if(!int_value_stack.push(v_cheninblanc_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2150)) return -1;
		if (!cheninblanc_set.contains(v_kaon2equal_2)) continue;
	case 1137:
	// Continuation for label 124
			v_cheninblanc_1 = v_kaon2equal_1;
		if(cheninblanc_set.insert(v_cheninblanc_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1137)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(122)) return -1;
			if(!backtrack_stack.push(121)) return -1;
			if(!backtrack_stack.push(94)) return -1;
			if(!backtrack_stack.push(93)) return -1;
			if(!backtrack_stack.push(91)) return -1;
			goto label_87;
		}
		break;
	case 2150:
	// Label to restore values of label 124
		v_cheninblanc_1 = int_value_stack.pop();
		break;
	case 125: label_125: 
	// using  symbolic fact kaon2equal(v_kaon2equal_1,v_kaon2equal_2) with literal 2 in rule 
	// 455: q21(A) :- q21(B), kaon2equal(A,B).
	// resulting in symbolic fact q21(v_q21_1)

		if (!q21_set.contains(v_kaon2equal_2)) continue;
	case 1138:
	// Continuation for label 125
			v_q21_1 = v_kaon2equal_1;
		if(q21_set.insert(v_q21_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1138)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(119)) return -1;
			goto label_96;
		}
		break;
	case 126: label_126: 
	// using  symbolic fact kaon2equal(v_kaon2equal_1,v_kaon2equal_2) with literal 2 in rule 
	// 457: whitewine(A) :- whitewine(B), kaon2equal(A,B).
	// resulting in symbolic fact whitewine(v_whitewine_1)

		if (!whitewine_set.contains(v_kaon2equal_2)) continue;
	case 1139:
	// Continuation for label 126
			v_whitewine_1 = v_kaon2equal_1;
		if(whitewine_set.insert(v_whitewine_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1139)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(560)) return -1;
			if(!backtrack_stack.push(491)) return -1;
			if(!backtrack_stack.push(374)) return -1;
			if(!backtrack_stack.push(235)) return -1;
			if(!backtrack_stack.push(113)) return -1;
			if(!backtrack_stack.push(112)) return -1;
			if(!backtrack_stack.push(104)) return -1;
			if(!backtrack_stack.push(103)) return -1;
			if(!backtrack_stack.push(101)) return -1;
			if(!backtrack_stack.push(98)) return -1;
			goto label_97;
		}
		break;
	case 127: label_127: 
	// using  symbolic fact kaon2equal(v_kaon2equal_1,v_kaon2equal_2) with literal 2 in rule 
	// 468: q6(A) :- q6(B), kaon2equal(A,B).
	// resulting in symbolic fact q6(v_q6_1)

		//recursive
		if(!int_value_stack.push(v_q6_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2153)) return -1;
		if (!q6_set.contains(v_kaon2equal_2)) continue;
	case 1140:
	// Continuation for label 127
			v_q6_1 = v_kaon2equal_1;
		if(q6_set.insert(v_q6_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1140)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(146)) return -1;
			if(!backtrack_stack.push(145)) return -1;
			if(!backtrack_stack.push(130)) return -1;
			if(!backtrack_stack.push(129)) return -1;
			if(!backtrack_stack.push(128)) return -1;
			goto label_92;
		}
		break;
	case 2153:
	// Label to restore values of label 127
		v_q6_1 = int_value_stack.pop();
		break;
	case 128: label_128: 
	// using  symbolic fact q6(v_q6_1) with literal 1 in rule 
	// 466: wine(A) :- q6(A).
	// resulting in symbolic fact wine(v_wine_1)

		//recursive
		if(!int_value_stack.push(v_wine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2154)) return -1;
		v_wine_1 = v_q6_1;
		if(wine_set.insert(v_wine_1)) {
		if(!wine_f.insert(v_wine_1)) return -1;
		// Labels for next rules
		if(!backtrack_stack.push(1007)) return -1;
		if(!backtrack_stack.push(989)) return -1;
		if(!backtrack_stack.push(954)) return -1;
		if(!backtrack_stack.push(938)) return -1;
		if(!backtrack_stack.push(911)) return -1;
		if(!backtrack_stack.push(903)) return -1;
		if(!backtrack_stack.push(902)) return -1;
		if(!backtrack_stack.push(901)) return -1;
		if(!backtrack_stack.push(849)) return -1;
		if(!backtrack_stack.push(848)) return -1;
		if(!backtrack_stack.push(821)) return -1;
		if(!backtrack_stack.push(820)) return -1;
		if(!backtrack_stack.push(737)) return -1;
		if(!backtrack_stack.push(712)) return -1;
		if(!backtrack_stack.push(674)) return -1;
		if(!backtrack_stack.push(672)) return -1;
		if(!backtrack_stack.push(583)) return -1;
		if(!backtrack_stack.push(575)) return -1;
		if(!backtrack_stack.push(563)) return -1;
		if(!backtrack_stack.push(551)) return -1;
		if(!backtrack_stack.push(549)) return -1;
		if(!backtrack_stack.push(483)) return -1;
		if(!backtrack_stack.push(473)) return -1;
		if(!backtrack_stack.push(470)) return -1;
		if(!backtrack_stack.push(430)) return -1;
		if(!backtrack_stack.push(401)) return -1;
		if(!backtrack_stack.push(399)) return -1;
		if(!backtrack_stack.push(380)) return -1;
		if(!backtrack_stack.push(364)) return -1;
		if(!backtrack_stack.push(363)) return -1;
		if(!backtrack_stack.push(329)) return -1;
		if(!backtrack_stack.push(328)) return -1;
		if(!backtrack_stack.push(327)) return -1;
		if(!backtrack_stack.push(295)) return -1;
		if(!backtrack_stack.push(294)) return -1;
		if(!backtrack_stack.push(216)) return -1;
		if(!backtrack_stack.push(210)) return -1;
		if(!backtrack_stack.push(192)) return -1;
		if(!backtrack_stack.push(180)) return -1;
		if(!backtrack_stack.push(179)) return -1;
		if(!backtrack_stack.push(178)) return -1;
		if(!backtrack_stack.push(154)) return -1;
		if(!backtrack_stack.push(153)) return -1;
		if(!backtrack_stack.push(100)) return -1;
		if(!backtrack_stack.push(76)) return -1;
		if(!backtrack_stack.push(75)) return -1;
		if(!backtrack_stack.push(67)) return -1;
		if(!backtrack_stack.push(63)) return -1;
		if(!backtrack_stack.push(53)) return -1;
		if(!backtrack_stack.push(46)) return -1;
		if(!backtrack_stack.push(19)) return -1;
		goto label_16;
		}
		break;
	case 2154:
	// Label to restore values of label 128
		v_wine_1 = int_value_stack.pop();
		break;
	case 129: label_129: 
	// using  symbolic fact q6(v_q6_1) with literal 1 in rule 
	// 468: q6(A) :- q6(B), kaon2equal(A,B).
	// resulting in symbolic fact q6(v_q6_1)

		//recursive
		c468_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q6_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2155)) return -1;
		c468_2_fb.open(v_q6_1);
	case 1142:
	// Continuation for label 129
		while (c468_2_fb.fetch()) {
			v_q6_1 = c468_2_fb.c2();
		if(q6_set.insert(v_q6_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1142)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(146)) return -1;
			if(!backtrack_stack.push(145)) return -1;
			if(!backtrack_stack.push(130)) return -1;
			if(!backtrack_stack.push(129)) return -1;
			if(!backtrack_stack.push(128)) return -1;
			goto label_92;
		}
		}
		break;
	case 2155:
	// Label to restore values of label 129
		v_q6_1 = int_value_stack.pop();
		c468_2_fb.pop(&cursor_stack);
		break;
	case 130: label_130: 
	// using  symbolic fact q6(v_q6_1) with literal 1 in rule 
	// 498: q5(A) :- q6(A).
	// resulting in symbolic fact q5(v_q5_1)

		//recursive
		if(!int_value_stack.push(v_q5_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2156)) return -1;
		v_q5_1 = v_q6_1;
		if(q5_set.insert(v_q5_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(136)) return -1;
		if(!backtrack_stack.push(135)) return -1;
		goto label_131;
		}
		break;
	case 2156:
	// Label to restore values of label 130
		v_q5_1 = int_value_stack.pop();
		break;
	case 131: label_131: 
	// using  symbolic fact q5(v_q5_1) with literal 1 in rule 
	// 356: madefromgrape(A,A) :- q5(A), kaon2namedobjects(A).
	// resulting in symbolic fact madefromgrape(v_madefromgrape_1,v_madefromgrape_2)

		//recursive
		if(!int_value_stack.push(v_madefromgrape_2)) return -1;
		if(!int_value_stack.push(v_madefromgrape_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2157)) return -1;
		if (!kaon2namedobjects_b.contains(v_q5_1)) continue;
	case 1144:
	// Continuation for label 131
			v_madefromgrape_1 = v_q5_1;
			v_madefromgrape_2 = v_q5_1;
		if(madefromgrape_set.insert(v_madefromgrape_1, v_madefromgrape_2)) {
			if(!madefromgrape_bf.insert(v_madefromgrape_1, v_madefromgrape_2)) return -1;
			if(!madefromgrape_fb.insert(v_madefromgrape_2, v_madefromgrape_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1144)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(860)) return -1;
			if(!backtrack_stack.push(859)) return -1;
			if(!backtrack_stack.push(845)) return -1;
			if(!backtrack_stack.push(844)) return -1;
			if(!backtrack_stack.push(829)) return -1;
			if(!backtrack_stack.push(828)) return -1;
			if(!backtrack_stack.push(813)) return -1;
			if(!backtrack_stack.push(812)) return -1;
			if(!backtrack_stack.push(677)) return -1;
			if(!backtrack_stack.push(676)) return -1;
			if(!backtrack_stack.push(617)) return -1;
			if(!backtrack_stack.push(616)) return -1;
			if(!backtrack_stack.push(609)) return -1;
			if(!backtrack_stack.push(554)) return -1;
			if(!backtrack_stack.push(553)) return -1;
			if(!backtrack_stack.push(476)) return -1;
			if(!backtrack_stack.push(475)) return -1;
			if(!backtrack_stack.push(404)) return -1;
			if(!backtrack_stack.push(403)) return -1;
			if(!backtrack_stack.push(362)) return -1;
			if(!backtrack_stack.push(361)) return -1;
			if(!backtrack_stack.push(343)) return -1;
			if(!backtrack_stack.push(342)) return -1;
			if(!backtrack_stack.push(300)) return -1;
			if(!backtrack_stack.push(170)) return -1;
			if(!backtrack_stack.push(169)) return -1;
			if(!backtrack_stack.push(151)) return -1;
			if(!backtrack_stack.push(150)) return -1;
			if(!backtrack_stack.push(144)) return -1;
			if(!backtrack_stack.push(132)) return -1;
			if(!backtrack_stack.push(71)) return -1;
			if(!backtrack_stack.push(70)) return -1;
			if(!backtrack_stack.push(66)) return -1;
			goto label_64;
		}
		break;
	case 2157:
	// Label to restore values of label 131
		v_madefromgrape_1 = int_value_stack.pop();
		v_madefromgrape_2 = int_value_stack.pop();
		break;
	case 132: label_132: 
	// using  symbolic fact madefromgrape(v_madefromgrape_1,v_madefromgrape_2) with literal 2 in rule 
	// 88: kaon2equal(A,B) :- q3(C), madefromgrape(C,A), madefromgrape(C,B).
	// resulting in symbolic fact kaon2equal(v_kaon2equal_1,v_kaon2equal_2)

		//recursive
		c88_3_bf.push(&cursor_stack);
		if(!int_value_stack.push(v_kaon2equal_2)) return -1;
		if(!int_value_stack.push(v_kaon2equal_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2158)) return -1;
		if (!q3_set.contains(v_madefromgrape_1)) continue;
			c88_3_bf.open(v_madefromgrape_1);
	case 1145:
	// Continuation for label 132
			while (c88_3_bf.fetch()) {
				v_kaon2equal_1 = v_madefromgrape_2;
				v_kaon2equal_2 = c88_3_bf.c2();
		if(kaon2equal_set.insert(v_kaon2equal_1, v_kaon2equal_2)) {
				if(!kaon2equal_bf.insert(v_kaon2equal_1, v_kaon2equal_2)) return -1;
				if(!kaon2equal_fb.insert(v_kaon2equal_2, v_kaon2equal_1)) return -1;
				if(!kaon2equal_ff.insert(v_kaon2equal_1, v_kaon2equal_2)) return -1;
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1145)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(283)) return -1;
				if(!backtrack_stack.push(282)) return -1;
				if(!backtrack_stack.push(281)) return -1;
				if(!backtrack_stack.push(280)) return -1;
				if(!backtrack_stack.push(279)) return -1;
				if(!backtrack_stack.push(278)) return -1;
				if(!backtrack_stack.push(277)) return -1;
				if(!backtrack_stack.push(276)) return -1;
				if(!backtrack_stack.push(275)) return -1;
				if(!backtrack_stack.push(274)) return -1;
				if(!backtrack_stack.push(273)) return -1;
				if(!backtrack_stack.push(271)) return -1;
				if(!backtrack_stack.push(270)) return -1;
				if(!backtrack_stack.push(269)) return -1;
				if(!backtrack_stack.push(268)) return -1;
				if(!backtrack_stack.push(267)) return -1;
				if(!backtrack_stack.push(266)) return -1;
				if(!backtrack_stack.push(265)) return -1;
				if(!backtrack_stack.push(264)) return -1;
				if(!backtrack_stack.push(263)) return -1;
				if(!backtrack_stack.push(262)) return -1;
				if(!backtrack_stack.push(261)) return -1;
				if(!backtrack_stack.push(260)) return -1;
				if(!backtrack_stack.push(254)) return -1;
				if(!backtrack_stack.push(253)) return -1;
				if(!backtrack_stack.push(152)) return -1;
				if(!backtrack_stack.push(142)) return -1;
				if(!backtrack_stack.push(141)) return -1;
				if(!backtrack_stack.push(140)) return -1;
				if(!backtrack_stack.push(139)) return -1;
				if(!backtrack_stack.push(138)) return -1;
				if(!backtrack_stack.push(137)) return -1;
				if(!backtrack_stack.push(134)) return -1;
				if(!backtrack_stack.push(133)) return -1;
				if(!backtrack_stack.push(127)) return -1;
				if(!backtrack_stack.push(126)) return -1;
				if(!backtrack_stack.push(125)) return -1;
				if(!backtrack_stack.push(124)) return -1;
				if(!backtrack_stack.push(85)) return -1;
				if(!backtrack_stack.push(82)) return -1;
				if(!backtrack_stack.push(81)) return -1;
				if(!backtrack_stack.push(73)) return -1;
				if(!backtrack_stack.push(72)) return -1;
				if(!backtrack_stack.push(69)) return -1;
				if(!backtrack_stack.push(62)) return -1;
				if(!backtrack_stack.push(57)) return -1;
				if(!backtrack_stack.push(56)) return -1;
				if(!backtrack_stack.push(55)) return -1;
				if(!backtrack_stack.push(45)) return -1;
				if(!backtrack_stack.push(42)) return -1;
				if(!backtrack_stack.push(41)) return -1;
				if(!backtrack_stack.push(35)) return -1;
				if(!backtrack_stack.push(34)) return -1;
				if(!backtrack_stack.push(33)) return -1;
				if(!backtrack_stack.push(29)) return -1;
				if(!backtrack_stack.push(28)) return -1;
				if(!backtrack_stack.push(21)) return -1;
				if(!backtrack_stack.push(18)) return -1;
				if(!backtrack_stack.push(13)) return -1;
				if(!backtrack_stack.push(5)) return -1;
				if(!backtrack_stack.push(4)) return -1;
				goto label_3;
		}
			}
		break;
	case 2158:
	// Label to restore values of label 132
		v_kaon2equal_1 = int_value_stack.pop();
		v_kaon2equal_2 = int_value_stack.pop();
		c88_3_bf.pop(&cursor_stack);
		break;
	case 133: label_133: 
	// using  symbolic fact kaon2equal(v_kaon2equal_1,v_kaon2equal_2) with literal 2 in rule 
	// 492: q4(A) :- q4(B), kaon2equal(A,B).
	// resulting in symbolic fact q4(v_q4_1)

		if (!q4_set.contains(v_kaon2equal_2)) continue;
	case 1146:
	// Continuation for label 133
			v_q4_1 = v_kaon2equal_1;
		if(q4_set.insert(v_q4_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1146)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(90)) return -1;
			if(!backtrack_stack.push(89)) return -1;
			goto label_88;
		}
		break;
	case 134: label_134: 
	// using  symbolic fact kaon2equal(v_kaon2equal_1,v_kaon2equal_2) with literal 2 in rule 
	// 507: q5(A) :- q5(B), kaon2equal(A,B).
	// resulting in symbolic fact q5(v_q5_1)

		//recursive
		if(!int_value_stack.push(v_q5_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2160)) return -1;
		if (!q5_set.contains(v_kaon2equal_2)) continue;
	case 1147:
	// Continuation for label 134
			v_q5_1 = v_kaon2equal_1;
		if(q5_set.insert(v_q5_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1147)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(136)) return -1;
			if(!backtrack_stack.push(135)) return -1;
			goto label_131;
		}
		break;
	case 2160:
	// Label to restore values of label 134
		v_q5_1 = int_value_stack.pop();
		break;
	case 135: label_135: 
	// using  symbolic fact q5(v_q5_1) with literal 1 in rule 
	// 497: wine(A) :- q5(A).
	// resulting in symbolic fact wine(v_wine_1)

		v_wine_1 = v_q5_1;
		if(wine_set.insert(v_wine_1)) {
		if(!wine_f.insert(v_wine_1)) return -1;
		// Labels for next rules
		if(!backtrack_stack.push(1007)) return -1;
		if(!backtrack_stack.push(989)) return -1;
		if(!backtrack_stack.push(954)) return -1;
		if(!backtrack_stack.push(938)) return -1;
		if(!backtrack_stack.push(911)) return -1;
		if(!backtrack_stack.push(903)) return -1;
		if(!backtrack_stack.push(902)) return -1;
		if(!backtrack_stack.push(901)) return -1;
		if(!backtrack_stack.push(849)) return -1;
		if(!backtrack_stack.push(848)) return -1;
		if(!backtrack_stack.push(821)) return -1;
		if(!backtrack_stack.push(820)) return -1;
		if(!backtrack_stack.push(737)) return -1;
		if(!backtrack_stack.push(712)) return -1;
		if(!backtrack_stack.push(674)) return -1;
		if(!backtrack_stack.push(672)) return -1;
		if(!backtrack_stack.push(583)) return -1;
		if(!backtrack_stack.push(575)) return -1;
		if(!backtrack_stack.push(563)) return -1;
		if(!backtrack_stack.push(551)) return -1;
		if(!backtrack_stack.push(549)) return -1;
		if(!backtrack_stack.push(483)) return -1;
		if(!backtrack_stack.push(473)) return -1;
		if(!backtrack_stack.push(470)) return -1;
		if(!backtrack_stack.push(430)) return -1;
		if(!backtrack_stack.push(401)) return -1;
		if(!backtrack_stack.push(399)) return -1;
		if(!backtrack_stack.push(380)) return -1;
		if(!backtrack_stack.push(364)) return -1;
		if(!backtrack_stack.push(363)) return -1;
		if(!backtrack_stack.push(329)) return -1;
		if(!backtrack_stack.push(328)) return -1;
		if(!backtrack_stack.push(327)) return -1;
		if(!backtrack_stack.push(295)) return -1;
		if(!backtrack_stack.push(294)) return -1;
		if(!backtrack_stack.push(216)) return -1;
		if(!backtrack_stack.push(210)) return -1;
		if(!backtrack_stack.push(192)) return -1;
		if(!backtrack_stack.push(180)) return -1;
		if(!backtrack_stack.push(179)) return -1;
		if(!backtrack_stack.push(178)) return -1;
		if(!backtrack_stack.push(154)) return -1;
		if(!backtrack_stack.push(153)) return -1;
		if(!backtrack_stack.push(100)) return -1;
		if(!backtrack_stack.push(76)) return -1;
		if(!backtrack_stack.push(75)) return -1;
		if(!backtrack_stack.push(67)) return -1;
		if(!backtrack_stack.push(63)) return -1;
		if(!backtrack_stack.push(53)) return -1;
		if(!backtrack_stack.push(46)) return -1;
		if(!backtrack_stack.push(19)) return -1;
		goto label_16;
		}
		break;
	case 136: label_136: 
	// using  symbolic fact q5(v_q5_1) with literal 1 in rule 
	// 507: q5(A) :- q5(B), kaon2equal(A,B).
	// resulting in symbolic fact q5(v_q5_1)

		//recursive
		c507_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q5_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2162)) return -1;
		c507_2_fb.open(v_q5_1);
	case 1149:
	// Continuation for label 136
		while (c507_2_fb.fetch()) {
			v_q5_1 = c507_2_fb.c2();
		if(q5_set.insert(v_q5_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1149)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(136)) return -1;
			if(!backtrack_stack.push(135)) return -1;
			goto label_131;
		}
		}
		break;
	case 2162:
	// Label to restore values of label 136
		v_q5_1 = int_value_stack.pop();
		c507_2_fb.pop(&cursor_stack);
		break;
	case 137: label_137: 
	// using  symbolic fact kaon2equal(v_kaon2equal_1,v_kaon2equal_2) with literal 2 in rule 
	// 584: whitenonsweetwine(A) :- whitenonsweetwine(B), kaon2equal(A,B).
	// resulting in symbolic fact whitenonsweetwine(v_whitenonsweetwine_1)

		if (!whitenonsweetwine_set.contains(v_kaon2equal_2)) continue;
	case 1150:
	// Continuation for label 137
			v_whitenonsweetwine_1 = v_kaon2equal_1;
		if(whitenonsweetwine_set.insert(v_whitenonsweetwine_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1150)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(107)) return -1;
			if(!backtrack_stack.push(105)) return -1;
			goto label_102;
		}
		break;
	case 138: label_138: 
	// using  symbolic fact kaon2equal(v_kaon2equal_1,v_kaon2equal_2) with literal 2 in rule 
	// 590: q36(A) :- q36(B), kaon2equal(A,B).
	// resulting in symbolic fact q36(v_q36_1)

		if (!q36_set.contains(v_kaon2equal_2)) continue;
	case 1151:
	// Continuation for label 138
			v_q36_1 = v_kaon2equal_1;
		if(q36_set.insert(v_q36_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1151)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(111)) return -1;
			if(!backtrack_stack.push(108)) return -1;
			goto label_106;
		}
		break;
	case 139: label_139: 
	// using  symbolic fact kaon2equal(v_kaon2equal_1,v_kaon2equal_2) with literal 2 in rule 
	// 595: ot____nom7(A) :- ot____nom7(B), kaon2equal(A,B).
	// resulting in symbolic fact ot____nom7(v_ot____nom7_1)

		if (!ot____nom7_set.contains(v_kaon2equal_2)) continue;
	case 1152:
	// Continuation for label 139
			v_ot____nom7_1 = v_kaon2equal_1;
		if(ot____nom7_set.insert(v_ot____nom7_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1152)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(110)) return -1;
			goto label_109;
		}
		break;
	case 140: label_140: 
	// using  symbolic fact kaon2equal(v_kaon2equal_1,v_kaon2equal_2) with literal 2 in rule 
	// 600: q12(A) :- q12(B), kaon2equal(A,B).
	// resulting in symbolic fact q12(v_q12_1)

		if (!q12_set.contains(v_kaon2equal_2)) continue;
	case 1153:
	// Continuation for label 140
			v_q12_1 = v_kaon2equal_1;
		if(q12_set.insert(v_q12_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1153)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(116)) return -1;
			if(!backtrack_stack.push(115)) return -1;
			if(!backtrack_stack.push(114)) return -1;
			if(!backtrack_stack.push(99)) return -1;
			goto label_95;
		}
		break;
	case 141: label_141: 
	// using  symbolic fact kaon2equal(v_kaon2equal_1,v_kaon2equal_2) with literal 2 in rule 
	// 658: ot____nom5(A) :- ot____nom5(B), kaon2equal(A,B).
	// resulting in symbolic fact ot____nom5(v_ot____nom5_1)

		if (!ot____nom5_set.contains(v_kaon2equal_2)) continue;
	case 1154:
	// Continuation for label 141
			v_ot____nom5_1 = v_kaon2equal_1;
		if(ot____nom5_set.insert(v_ot____nom5_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1154)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(118)) return -1;
			goto label_117;
		}
		break;
	case 142: label_142: 
	// using  symbolic fact kaon2equal(v_kaon2equal_1,v_kaon2equal_2) with literal 2 in rule 
	// 789: q3(A) :- q3(B), kaon2equal(A,B).
	// resulting in symbolic fact q3(v_q3_1)

		//recursive
		if(!int_value_stack.push(v_q3_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2168)) return -1;
		if (!q3_set.contains(v_kaon2equal_2)) continue;
	case 1155:
	// Continuation for label 142
			v_q3_1 = v_kaon2equal_1;
		if(q3_set.insert(v_q3_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1155)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(143)) return -1;
			goto label_123;
		}
		break;
	case 2168:
	// Label to restore values of label 142
		v_q3_1 = int_value_stack.pop();
		break;
	case 143: label_143: 
	// using  symbolic fact q3(v_q3_1) with literal 1 in rule 
	// 789: q3(A) :- q3(B), kaon2equal(A,B).
	// resulting in symbolic fact q3(v_q3_1)

		//recursive
		c789_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q3_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2169)) return -1;
		c789_2_fb.open(v_q3_1);
	case 1156:
	// Continuation for label 143
		while (c789_2_fb.fetch()) {
			v_q3_1 = c789_2_fb.c2();
		if(q3_set.insert(v_q3_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1156)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(143)) return -1;
			goto label_123;
		}
		}
		break;
	case 2169:
	// Label to restore values of label 143
		v_q3_1 = int_value_stack.pop();
		c789_2_fb.pop(&cursor_stack);
		break;
	case 144: label_144: 
	// using  symbolic fact madefromgrape(v_madefromgrape_1,v_madefromgrape_2) with literal 3 in rule 
	// 88: kaon2equal(A,B) :- q3(C), madefromgrape(C,A), madefromgrape(C,B).
	// resulting in symbolic fact kaon2equal(v_kaon2equal_1,v_kaon2equal_2)

		//recursive
		c88_2_bf.push(&cursor_stack);
		if(!int_value_stack.push(v_kaon2equal_1)) return -1;
		if(!int_value_stack.push(v_kaon2equal_2)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2170)) return -1;
		if (!q3_set.contains(v_madefromgrape_1)) continue;
			c88_2_bf.open(v_madefromgrape_1);
	case 1157:
	// Continuation for label 144
			while (c88_2_bf.fetch()) {
				v_kaon2equal_2 = v_madefromgrape_2;
				v_kaon2equal_1 = c88_2_bf.c2();
		if(kaon2equal_set.insert(v_kaon2equal_1, v_kaon2equal_2)) {
				if(!kaon2equal_bf.insert(v_kaon2equal_1, v_kaon2equal_2)) return -1;
				if(!kaon2equal_fb.insert(v_kaon2equal_2, v_kaon2equal_1)) return -1;
				if(!kaon2equal_ff.insert(v_kaon2equal_1, v_kaon2equal_2)) return -1;
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1157)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(283)) return -1;
				if(!backtrack_stack.push(282)) return -1;
				if(!backtrack_stack.push(281)) return -1;
				if(!backtrack_stack.push(280)) return -1;
				if(!backtrack_stack.push(279)) return -1;
				if(!backtrack_stack.push(278)) return -1;
				if(!backtrack_stack.push(277)) return -1;
				if(!backtrack_stack.push(276)) return -1;
				if(!backtrack_stack.push(275)) return -1;
				if(!backtrack_stack.push(274)) return -1;
				if(!backtrack_stack.push(273)) return -1;
				if(!backtrack_stack.push(271)) return -1;
				if(!backtrack_stack.push(270)) return -1;
				if(!backtrack_stack.push(269)) return -1;
				if(!backtrack_stack.push(268)) return -1;
				if(!backtrack_stack.push(267)) return -1;
				if(!backtrack_stack.push(266)) return -1;
				if(!backtrack_stack.push(265)) return -1;
				if(!backtrack_stack.push(264)) return -1;
				if(!backtrack_stack.push(263)) return -1;
				if(!backtrack_stack.push(262)) return -1;
				if(!backtrack_stack.push(261)) return -1;
				if(!backtrack_stack.push(260)) return -1;
				if(!backtrack_stack.push(254)) return -1;
				if(!backtrack_stack.push(253)) return -1;
				if(!backtrack_stack.push(152)) return -1;
				if(!backtrack_stack.push(142)) return -1;
				if(!backtrack_stack.push(141)) return -1;
				if(!backtrack_stack.push(140)) return -1;
				if(!backtrack_stack.push(139)) return -1;
				if(!backtrack_stack.push(138)) return -1;
				if(!backtrack_stack.push(137)) return -1;
				if(!backtrack_stack.push(134)) return -1;
				if(!backtrack_stack.push(133)) return -1;
				if(!backtrack_stack.push(127)) return -1;
				if(!backtrack_stack.push(126)) return -1;
				if(!backtrack_stack.push(125)) return -1;
				if(!backtrack_stack.push(124)) return -1;
				if(!backtrack_stack.push(85)) return -1;
				if(!backtrack_stack.push(82)) return -1;
				if(!backtrack_stack.push(81)) return -1;
				if(!backtrack_stack.push(73)) return -1;
				if(!backtrack_stack.push(72)) return -1;
				if(!backtrack_stack.push(69)) return -1;
				if(!backtrack_stack.push(62)) return -1;
				if(!backtrack_stack.push(57)) return -1;
				if(!backtrack_stack.push(56)) return -1;
				if(!backtrack_stack.push(55)) return -1;
				if(!backtrack_stack.push(45)) return -1;
				if(!backtrack_stack.push(42)) return -1;
				if(!backtrack_stack.push(41)) return -1;
				if(!backtrack_stack.push(35)) return -1;
				if(!backtrack_stack.push(34)) return -1;
				if(!backtrack_stack.push(33)) return -1;
				if(!backtrack_stack.push(29)) return -1;
				if(!backtrack_stack.push(28)) return -1;
				if(!backtrack_stack.push(21)) return -1;
				if(!backtrack_stack.push(18)) return -1;
				if(!backtrack_stack.push(13)) return -1;
				if(!backtrack_stack.push(5)) return -1;
				if(!backtrack_stack.push(4)) return -1;
				goto label_3;
		}
			}
		break;
	case 2170:
	// Label to restore values of label 144
		v_kaon2equal_2 = int_value_stack.pop();
		v_kaon2equal_1 = int_value_stack.pop();
		c88_2_bf.pop(&cursor_stack);
		break;
	case 145: label_145: 
	// using  symbolic fact q6(v_q6_1) with literal 1 in rule 
	// 521: madefromgrape(A,A) :- q6(A), kaon2namedobjects(A).
	// resulting in symbolic fact madefromgrape(v_madefromgrape_1,v_madefromgrape_2)

		//recursive
		if(!int_value_stack.push(v_madefromgrape_2)) return -1;
		if(!int_value_stack.push(v_madefromgrape_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2171)) return -1;
		if (!kaon2namedobjects_b.contains(v_q6_1)) continue;
	case 1158:
	// Continuation for label 145
			v_madefromgrape_1 = v_q6_1;
			v_madefromgrape_2 = v_q6_1;
		if(madefromgrape_set.insert(v_madefromgrape_1, v_madefromgrape_2)) {
			if(!madefromgrape_bf.insert(v_madefromgrape_1, v_madefromgrape_2)) return -1;
			if(!madefromgrape_fb.insert(v_madefromgrape_2, v_madefromgrape_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1158)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(860)) return -1;
			if(!backtrack_stack.push(859)) return -1;
			if(!backtrack_stack.push(845)) return -1;
			if(!backtrack_stack.push(844)) return -1;
			if(!backtrack_stack.push(829)) return -1;
			if(!backtrack_stack.push(828)) return -1;
			if(!backtrack_stack.push(813)) return -1;
			if(!backtrack_stack.push(812)) return -1;
			if(!backtrack_stack.push(677)) return -1;
			if(!backtrack_stack.push(676)) return -1;
			if(!backtrack_stack.push(617)) return -1;
			if(!backtrack_stack.push(616)) return -1;
			if(!backtrack_stack.push(609)) return -1;
			if(!backtrack_stack.push(554)) return -1;
			if(!backtrack_stack.push(553)) return -1;
			if(!backtrack_stack.push(476)) return -1;
			if(!backtrack_stack.push(475)) return -1;
			if(!backtrack_stack.push(404)) return -1;
			if(!backtrack_stack.push(403)) return -1;
			if(!backtrack_stack.push(362)) return -1;
			if(!backtrack_stack.push(361)) return -1;
			if(!backtrack_stack.push(343)) return -1;
			if(!backtrack_stack.push(342)) return -1;
			if(!backtrack_stack.push(300)) return -1;
			if(!backtrack_stack.push(170)) return -1;
			if(!backtrack_stack.push(169)) return -1;
			if(!backtrack_stack.push(151)) return -1;
			if(!backtrack_stack.push(150)) return -1;
			if(!backtrack_stack.push(144)) return -1;
			if(!backtrack_stack.push(132)) return -1;
			if(!backtrack_stack.push(71)) return -1;
			if(!backtrack_stack.push(70)) return -1;
			if(!backtrack_stack.push(66)) return -1;
			goto label_64;
		}
		break;
	case 2171:
	// Label to restore values of label 145
		v_madefromgrape_1 = int_value_stack.pop();
		v_madefromgrape_2 = int_value_stack.pop();
		break;
	case 146: label_146: 
	// using  symbolic fact q6(v_q6_1) with literal 1 in rule 
	// 548: ot____nom34(A) :- q6(A), kaon2namedobjects(A).
	// resulting in symbolic fact ot____nom34(v_ot____nom34_1)

		//recursive
		if(!int_value_stack.push(v_ot____nom34_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2172)) return -1;
		if (!kaon2namedobjects_b.contains(v_q6_1)) continue;
	case 1159:
	// Continuation for label 146
			v_ot____nom34_1 = v_q6_1;
		if(ot____nom34_set.insert(v_ot____nom34_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1159)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(149)) return -1;
			if(!backtrack_stack.push(148)) return -1;
			goto label_147;
		}
		break;
	case 2172:
	// Label to restore values of label 146
		v_ot____nom34_1 = int_value_stack.pop();
		break;
	case 147: label_147: 
	// using  symbolic fact ot____nom34(v_ot____nom34_1) with literal 1 in rule 
	// 549: ot____nom34(A) :- ot____nom34(B), kaon2equal(A,B).
	// resulting in symbolic fact ot____nom34(v_ot____nom34_1)

		//recursive
		c549_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_ot____nom34_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2173)) return -1;
		c549_2_fb.open(v_ot____nom34_1);
	case 1160:
	// Continuation for label 147
		while (c549_2_fb.fetch()) {
			v_ot____nom34_1 = c549_2_fb.c2();
		if(ot____nom34_set.insert(v_ot____nom34_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1160)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(149)) return -1;
			if(!backtrack_stack.push(148)) return -1;
			goto label_147;
		}
		}
		break;
	case 2173:
	// Label to restore values of label 147
		v_ot____nom34_1 = int_value_stack.pop();
		c549_2_fb.pop(&cursor_stack);
		break;
	case 148: label_148: 
	// using  symbolic fact ot____nom34(v_ot____nom34_1) with literal 3 in rule 
	// 16: cheninblanc(A) :- wine(A), madefromgrape(A,B), ot____nom34(B).
	// resulting in symbolic fact cheninblanc(v_cheninblanc_1)

		//recursive
		c16_1_f.push(&cursor_stack);
		if(!int_value_stack.push(v_cheninblanc_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2174)) return -1;
		c16_1_f.open();
		while (c16_1_f.fetch()) {
			if (!madefromgrape_set.contains(c16_1_f.c1(), v_ot____nom34_1)) continue;
	case 1161:
	// Continuation for label 148
				v_cheninblanc_1 = c16_1_f.c1();
		if(cheninblanc_set.insert(v_cheninblanc_1)) {
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1161)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(122)) return -1;
				if(!backtrack_stack.push(121)) return -1;
				if(!backtrack_stack.push(94)) return -1;
				if(!backtrack_stack.push(93)) return -1;
				if(!backtrack_stack.push(91)) return -1;
				goto label_87;
		}
		}
		break;
	case 2174:
	// Label to restore values of label 148
		v_cheninblanc_1 = int_value_stack.pop();
		c16_1_f.pop(&cursor_stack);
		break;
	case 149: label_149: 
	// using  symbolic fact ot____nom34(v_ot____nom34_1) with literal 3 in rule 
	// 546: q5(A) :- wine(A), madefromgrape(A,B), ot____nom34(B).
	// resulting in symbolic fact q5(v_q5_1)

		c546_1_f.open();
		while (c546_1_f.fetch()) {
			if (!madefromgrape_set.contains(c546_1_f.c1(), v_ot____nom34_1)) continue;
	case 1162:
	// Continuation for label 149
				v_q5_1 = c546_1_f.c1();
		if(q5_set.insert(v_q5_1)) {
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1162)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(136)) return -1;
				if(!backtrack_stack.push(135)) return -1;
				goto label_131;
		}
		}
		break;
	case 150: label_150: 
	// using  symbolic fact madefromgrape(v_madefromgrape_1,v_madefromgrape_2) with literal 2 in rule 
	// 16: cheninblanc(A) :- wine(A), madefromgrape(A,B), ot____nom34(B).
	// resulting in symbolic fact cheninblanc(v_cheninblanc_1)

		//recursive
		if(!int_value_stack.push(v_cheninblanc_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2176)) return -1;
		if (!wine_set.contains(v_madefromgrape_1)) continue;
			if (!ot____nom34_set.contains(v_madefromgrape_2)) continue;
	case 1163:
	// Continuation for label 150
				v_cheninblanc_1 = v_madefromgrape_1;
		if(cheninblanc_set.insert(v_cheninblanc_1)) {
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1163)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(122)) return -1;
				if(!backtrack_stack.push(121)) return -1;
				if(!backtrack_stack.push(94)) return -1;
				if(!backtrack_stack.push(93)) return -1;
				if(!backtrack_stack.push(91)) return -1;
				goto label_87;
		}
		break;
	case 2176:
	// Label to restore values of label 150
		v_cheninblanc_1 = int_value_stack.pop();
		break;
	case 151: label_151: 
	// using  symbolic fact madefromgrape(v_madefromgrape_1,v_madefromgrape_2) with literal 2 in rule 
	// 546: q5(A) :- wine(A), madefromgrape(A,B), ot____nom34(B).
	// resulting in symbolic fact q5(v_q5_1)

		if (!wine_set.contains(v_madefromgrape_1)) continue;
			if (!ot____nom34_set.contains(v_madefromgrape_2)) continue;
	case 1164:
	// Continuation for label 151
				v_q5_1 = v_madefromgrape_1;
		if(q5_set.insert(v_q5_1)) {
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1164)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(136)) return -1;
				if(!backtrack_stack.push(135)) return -1;
				goto label_131;
		}
		break;
	case 152: label_152: 
	// using  symbolic fact kaon2equal(v_kaon2equal_1,v_kaon2equal_2) with literal 2 in rule 
	// 549: ot____nom34(A) :- ot____nom34(B), kaon2equal(A,B).
	// resulting in symbolic fact ot____nom34(v_ot____nom34_1)

		if (!ot____nom34_set.contains(v_kaon2equal_2)) continue;
	case 1165:
	// Continuation for label 152
			v_ot____nom34_1 = v_kaon2equal_1;
		if(ot____nom34_set.insert(v_ot____nom34_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1165)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(149)) return -1;
			if(!backtrack_stack.push(148)) return -1;
			goto label_147;
		}
		break;
	case 153: label_153: 
	// using  symbolic fact wine(v_wine_1) with literal 1 in rule 
	// 16: cheninblanc(A) :- wine(A), madefromgrape(A,B), ot____nom34(B).
	// resulting in symbolic fact cheninblanc(v_cheninblanc_1)

		//recursive
		c16_2_bf.push(&cursor_stack);
		if(!int_value_stack.push(v_cheninblanc_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2179)) return -1;
		c16_2_bf.open(v_wine_1);
		while (c16_2_bf.fetch()) {
			if (!ot____nom34_set.contains(c16_2_bf.c2())) continue;
	case 1166:
	// Continuation for label 153
				v_cheninblanc_1 = v_wine_1;
		if(cheninblanc_set.insert(v_cheninblanc_1)) {
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1166)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(122)) return -1;
				if(!backtrack_stack.push(121)) return -1;
				if(!backtrack_stack.push(94)) return -1;
				if(!backtrack_stack.push(93)) return -1;
				if(!backtrack_stack.push(91)) return -1;
				goto label_87;
		}
		}
		break;
	case 2179:
	// Label to restore values of label 153
		v_cheninblanc_1 = int_value_stack.pop();
		c16_2_bf.pop(&cursor_stack);
		break;
	case 154: label_154: 
	// using  symbolic fact wine(v_wine_1) with literal 1 in rule 
	// 546: q5(A) :- wine(A), madefromgrape(A,B), ot____nom34(B).
	// resulting in symbolic fact q5(v_q5_1)

		c546_2_bf.open(v_wine_1);
		while (c546_2_bf.fetch()) {
			if (!ot____nom34_set.contains(c546_2_bf.c2())) continue;
	case 1167:
	// Continuation for label 154
				v_q5_1 = v_wine_1;
		if(q5_set.insert(v_q5_1)) {
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1167)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(136)) return -1;
				if(!backtrack_stack.push(135)) return -1;
				goto label_131;
		}
		}
		break;
	case 155:
	// applying rule 
	// 21: beaujolais(A) :- beaujolais_aux(A).
	// resulting in symbolic fact beaujolais(v_beaujolais_1)
		c21_1_f.open();
	case 1168:
	// Continuation for label 155
		while (c21_1_f.fetch()) {
			v_beaujolais_1 = c21_1_f.c1();
		if(beaujolais_set.insert(v_beaujolais_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1168)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(239)) return -1;
			if(!backtrack_stack.push(200)) return -1;
			if(!backtrack_stack.push(186)) return -1;
			if(!backtrack_stack.push(185)) return -1;
			if(!backtrack_stack.push(181)) return -1;
			if(!backtrack_stack.push(175)) return -1;
			if(!backtrack_stack.push(174)) return -1;
			if(!backtrack_stack.push(171)) return -1;
			goto label_156;
		}
		}
		break;
	case 156: label_156: 
	// using  symbolic fact beaujolais(v_beaujolais_1) with literal 1 in rule 
	// 20: q14(A) :- beaujolais(A).
	// resulting in symbolic fact q14(v_q14_1)

		v_q14_1 = v_beaujolais_1;
		if(q14_set.insert(v_q14_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(163)) return -1;
		if(!backtrack_stack.push(162)) return -1;
		if(!backtrack_stack.push(161)) return -1;
		if(!backtrack_stack.push(160)) return -1;
		if(!backtrack_stack.push(158)) return -1;
		goto label_157;
		}
		break;
	case 157: label_157: 
	// using  symbolic fact q14(v_q14_1) with literal 1 in rule 
	// 17: wine(A) :- q14(A).
	// resulting in symbolic fact wine(v_wine_1)

		v_wine_1 = v_q14_1;
		if(wine_set.insert(v_wine_1)) {
		if(!wine_f.insert(v_wine_1)) return -1;
		// Labels for next rules
		if(!backtrack_stack.push(1007)) return -1;
		if(!backtrack_stack.push(989)) return -1;
		if(!backtrack_stack.push(954)) return -1;
		if(!backtrack_stack.push(938)) return -1;
		if(!backtrack_stack.push(911)) return -1;
		if(!backtrack_stack.push(903)) return -1;
		if(!backtrack_stack.push(902)) return -1;
		if(!backtrack_stack.push(901)) return -1;
		if(!backtrack_stack.push(849)) return -1;
		if(!backtrack_stack.push(848)) return -1;
		if(!backtrack_stack.push(821)) return -1;
		if(!backtrack_stack.push(820)) return -1;
		if(!backtrack_stack.push(737)) return -1;
		if(!backtrack_stack.push(712)) return -1;
		if(!backtrack_stack.push(674)) return -1;
		if(!backtrack_stack.push(672)) return -1;
		if(!backtrack_stack.push(583)) return -1;
		if(!backtrack_stack.push(575)) return -1;
		if(!backtrack_stack.push(563)) return -1;
		if(!backtrack_stack.push(551)) return -1;
		if(!backtrack_stack.push(549)) return -1;
		if(!backtrack_stack.push(483)) return -1;
		if(!backtrack_stack.push(473)) return -1;
		if(!backtrack_stack.push(470)) return -1;
		if(!backtrack_stack.push(430)) return -1;
		if(!backtrack_stack.push(401)) return -1;
		if(!backtrack_stack.push(399)) return -1;
		if(!backtrack_stack.push(380)) return -1;
		if(!backtrack_stack.push(364)) return -1;
		if(!backtrack_stack.push(363)) return -1;
		if(!backtrack_stack.push(329)) return -1;
		if(!backtrack_stack.push(328)) return -1;
		if(!backtrack_stack.push(327)) return -1;
		if(!backtrack_stack.push(295)) return -1;
		if(!backtrack_stack.push(294)) return -1;
		if(!backtrack_stack.push(216)) return -1;
		if(!backtrack_stack.push(210)) return -1;
		if(!backtrack_stack.push(192)) return -1;
		if(!backtrack_stack.push(180)) return -1;
		if(!backtrack_stack.push(179)) return -1;
		if(!backtrack_stack.push(178)) return -1;
		if(!backtrack_stack.push(154)) return -1;
		if(!backtrack_stack.push(153)) return -1;
		if(!backtrack_stack.push(100)) return -1;
		if(!backtrack_stack.push(76)) return -1;
		if(!backtrack_stack.push(75)) return -1;
		if(!backtrack_stack.push(67)) return -1;
		if(!backtrack_stack.push(63)) return -1;
		if(!backtrack_stack.push(53)) return -1;
		if(!backtrack_stack.push(46)) return -1;
		if(!backtrack_stack.push(19)) return -1;
		goto label_16;
		}
		break;
	case 158: label_158: 
	// using  symbolic fact q14(v_q14_1) with literal 1 in rule 
	// 19: gamay(A) :- q14(A).
	// resulting in symbolic fact gamay(v_gamay_1)

		//recursive
		if(!int_value_stack.push(v_gamay_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2184)) return -1;
		v_gamay_1 = v_q14_1;
		if(gamay_set.insert(v_gamay_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(168)) return -1;
		if(!backtrack_stack.push(167)) return -1;
		goto label_159;
		}
		break;
	case 2184:
	// Label to restore values of label 158
		v_gamay_1 = int_value_stack.pop();
		break;
	case 159: label_159: 
	// using  symbolic fact gamay(v_gamay_1) with literal 1 in rule 
	// 18: q14(A) :- gamay(A).
	// resulting in symbolic fact q14(v_q14_1)

		//recursive
		if(!int_value_stack.push(v_q14_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2185)) return -1;
		v_q14_1 = v_gamay_1;
		if(q14_set.insert(v_q14_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(163)) return -1;
		if(!backtrack_stack.push(162)) return -1;
		if(!backtrack_stack.push(161)) return -1;
		if(!backtrack_stack.push(160)) return -1;
		if(!backtrack_stack.push(158)) return -1;
		goto label_157;
		}
		break;
	case 2185:
	// Label to restore values of label 159
		v_q14_1 = int_value_stack.pop();
		break;
	case 160: label_160: 
	// using  symbolic fact q14(v_q14_1) with literal 1 in rule 
	// 501: q5(A) :- q14(A).
	// resulting in symbolic fact q5(v_q5_1)

		v_q5_1 = v_q14_1;
		if(q5_set.insert(v_q5_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(136)) return -1;
		if(!backtrack_stack.push(135)) return -1;
		goto label_131;
		}
		break;
	case 161: label_161: 
	// using  symbolic fact q14(v_q14_1) with literal 1 in rule 
	// 502: q14(A) :- q14(B), kaon2equal(A,B).
	// resulting in symbolic fact q14(v_q14_1)

		//recursive
		c502_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q14_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2187)) return -1;
		c502_2_fb.open(v_q14_1);
	case 1174:
	// Continuation for label 161
		while (c502_2_fb.fetch()) {
			v_q14_1 = c502_2_fb.c2();
		if(q14_set.insert(v_q14_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1174)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(163)) return -1;
			if(!backtrack_stack.push(162)) return -1;
			if(!backtrack_stack.push(161)) return -1;
			if(!backtrack_stack.push(160)) return -1;
			if(!backtrack_stack.push(158)) return -1;
			goto label_157;
		}
		}
		break;
	case 2187:
	// Label to restore values of label 161
		v_q14_1 = int_value_stack.pop();
		c502_2_fb.pop(&cursor_stack);
		break;
	case 162: label_162: 
	// using  symbolic fact q14(v_q14_1) with literal 1 in rule 
	// 518: madefromgrape(A,A) :- q14(A), kaon2namedobjects(A).
	// resulting in symbolic fact madefromgrape(v_madefromgrape_1,v_madefromgrape_2)

		//recursive
		if(!int_value_stack.push(v_madefromgrape_2)) return -1;
		if(!int_value_stack.push(v_madefromgrape_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2188)) return -1;
		if (!kaon2namedobjects_b.contains(v_q14_1)) continue;
	case 1175:
	// Continuation for label 162
			v_madefromgrape_1 = v_q14_1;
			v_madefromgrape_2 = v_q14_1;
		if(madefromgrape_set.insert(v_madefromgrape_1, v_madefromgrape_2)) {
			if(!madefromgrape_bf.insert(v_madefromgrape_1, v_madefromgrape_2)) return -1;
			if(!madefromgrape_fb.insert(v_madefromgrape_2, v_madefromgrape_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1175)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(860)) return -1;
			if(!backtrack_stack.push(859)) return -1;
			if(!backtrack_stack.push(845)) return -1;
			if(!backtrack_stack.push(844)) return -1;
			if(!backtrack_stack.push(829)) return -1;
			if(!backtrack_stack.push(828)) return -1;
			if(!backtrack_stack.push(813)) return -1;
			if(!backtrack_stack.push(812)) return -1;
			if(!backtrack_stack.push(677)) return -1;
			if(!backtrack_stack.push(676)) return -1;
			if(!backtrack_stack.push(617)) return -1;
			if(!backtrack_stack.push(616)) return -1;
			if(!backtrack_stack.push(609)) return -1;
			if(!backtrack_stack.push(554)) return -1;
			if(!backtrack_stack.push(553)) return -1;
			if(!backtrack_stack.push(476)) return -1;
			if(!backtrack_stack.push(475)) return -1;
			if(!backtrack_stack.push(404)) return -1;
			if(!backtrack_stack.push(403)) return -1;
			if(!backtrack_stack.push(362)) return -1;
			if(!backtrack_stack.push(361)) return -1;
			if(!backtrack_stack.push(343)) return -1;
			if(!backtrack_stack.push(342)) return -1;
			if(!backtrack_stack.push(300)) return -1;
			if(!backtrack_stack.push(170)) return -1;
			if(!backtrack_stack.push(169)) return -1;
			if(!backtrack_stack.push(151)) return -1;
			if(!backtrack_stack.push(150)) return -1;
			if(!backtrack_stack.push(144)) return -1;
			if(!backtrack_stack.push(132)) return -1;
			if(!backtrack_stack.push(71)) return -1;
			if(!backtrack_stack.push(70)) return -1;
			if(!backtrack_stack.push(66)) return -1;
			goto label_64;
		}
		break;
	case 2188:
	// Label to restore values of label 162
		v_madefromgrape_1 = int_value_stack.pop();
		v_madefromgrape_2 = int_value_stack.pop();
		break;
	case 163: label_163: 
	// using  symbolic fact q14(v_q14_1) with literal 1 in rule 
	// 560: ot____nom33(A) :- q14(A), kaon2namedobjects(A).
	// resulting in symbolic fact ot____nom33(v_ot____nom33_1)

		//recursive
		if(!int_value_stack.push(v_ot____nom33_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2189)) return -1;
		if (!kaon2namedobjects_b.contains(v_q14_1)) continue;
	case 1176:
	// Continuation for label 163
			v_ot____nom33_1 = v_q14_1;
		if(ot____nom33_set.insert(v_ot____nom33_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1176)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(166)) return -1;
			if(!backtrack_stack.push(165)) return -1;
			goto label_164;
		}
		break;
	case 2189:
	// Label to restore values of label 163
		v_ot____nom33_1 = int_value_stack.pop();
		break;
	case 164: label_164: 
	// using  symbolic fact ot____nom33(v_ot____nom33_1) with literal 1 in rule 
	// 561: ot____nom33(A) :- ot____nom33(B), kaon2equal(A,B).
	// resulting in symbolic fact ot____nom33(v_ot____nom33_1)

		//recursive
		c561_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_ot____nom33_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2190)) return -1;
		c561_2_fb.open(v_ot____nom33_1);
	case 1177:
	// Continuation for label 164
		while (c561_2_fb.fetch()) {
			v_ot____nom33_1 = c561_2_fb.c2();
		if(ot____nom33_set.insert(v_ot____nom33_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1177)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(166)) return -1;
			if(!backtrack_stack.push(165)) return -1;
			goto label_164;
		}
		}
		break;
	case 2190:
	// Label to restore values of label 164
		v_ot____nom33_1 = int_value_stack.pop();
		c561_2_fb.pop(&cursor_stack);
		break;
	case 165: label_165: 
	// using  symbolic fact ot____nom33(v_ot____nom33_1) with literal 3 in rule 
	// 558: q5(A) :- wine(A), madefromgrape(A,B), ot____nom33(B).
	// resulting in symbolic fact q5(v_q5_1)

		c558_1_f.open();
		while (c558_1_f.fetch()) {
			if (!madefromgrape_set.contains(c558_1_f.c1(), v_ot____nom33_1)) continue;
	case 1178:
	// Continuation for label 165
				v_q5_1 = c558_1_f.c1();
		if(q5_set.insert(v_q5_1)) {
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1178)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(136)) return -1;
				if(!backtrack_stack.push(135)) return -1;
				goto label_131;
		}
		}
		break;
	case 166: label_166: 
	// using  symbolic fact ot____nom33(v_ot____nom33_1) with literal 3 in rule 
	// 788: gamay(A) :- wine(A), madefromgrape(A,B), ot____nom33(B).
	// resulting in symbolic fact gamay(v_gamay_1)

		//recursive
		c788_1_f.push(&cursor_stack);
		if(!int_value_stack.push(v_gamay_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2192)) return -1;
		c788_1_f.open();
		while (c788_1_f.fetch()) {
			if (!madefromgrape_set.contains(c788_1_f.c1(), v_ot____nom33_1)) continue;
	case 1179:
	// Continuation for label 166
				v_gamay_1 = c788_1_f.c1();
		if(gamay_set.insert(v_gamay_1)) {
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1179)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(168)) return -1;
				if(!backtrack_stack.push(167)) return -1;
				goto label_159;
		}
		}
		break;
	case 2192:
	// Label to restore values of label 166
		v_gamay_1 = int_value_stack.pop();
		c788_1_f.pop(&cursor_stack);
		break;
	case 167: label_167: 
	// using  symbolic fact gamay(v_gamay_1) with literal 1 in rule 
	// 786: q3(A) :- gamay(A).
	// resulting in symbolic fact q3(v_q3_1)

		v_q3_1 = v_gamay_1;
		if(q3_set.insert(v_q3_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(143)) return -1;
		goto label_123;
		}
		break;
	case 168: label_168: 
	// using  symbolic fact gamay(v_gamay_1) with literal 1 in rule 
	// 787: gamay(A) :- gamay(B), kaon2equal(A,B).
	// resulting in symbolic fact gamay(v_gamay_1)

		//recursive
		c787_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_gamay_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2194)) return -1;
		c787_2_fb.open(v_gamay_1);
	case 1181:
	// Continuation for label 168
		while (c787_2_fb.fetch()) {
			v_gamay_1 = c787_2_fb.c2();
		if(gamay_set.insert(v_gamay_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1181)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(168)) return -1;
			if(!backtrack_stack.push(167)) return -1;
			goto label_159;
		}
		}
		break;
	case 2194:
	// Label to restore values of label 168
		v_gamay_1 = int_value_stack.pop();
		c787_2_fb.pop(&cursor_stack);
		break;
	case 169: label_169: 
	// using  symbolic fact madefromgrape(v_madefromgrape_1,v_madefromgrape_2) with literal 2 in rule 
	// 558: q5(A) :- wine(A), madefromgrape(A,B), ot____nom33(B).
	// resulting in symbolic fact q5(v_q5_1)

		if (!wine_set.contains(v_madefromgrape_1)) continue;
			if (!ot____nom33_set.contains(v_madefromgrape_2)) continue;
	case 1182:
	// Continuation for label 169
				v_q5_1 = v_madefromgrape_1;
		if(q5_set.insert(v_q5_1)) {
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1182)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(136)) return -1;
				if(!backtrack_stack.push(135)) return -1;
				goto label_131;
		}
		break;
	case 170: label_170: 
	// using  symbolic fact madefromgrape(v_madefromgrape_1,v_madefromgrape_2) with literal 2 in rule 
	// 788: gamay(A) :- wine(A), madefromgrape(A,B), ot____nom33(B).
	// resulting in symbolic fact gamay(v_gamay_1)

		//recursive
		if(!int_value_stack.push(v_gamay_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2196)) return -1;
		if (!wine_set.contains(v_madefromgrape_1)) continue;
			if (!ot____nom33_set.contains(v_madefromgrape_2)) continue;
	case 1183:
	// Continuation for label 170
				v_gamay_1 = v_madefromgrape_1;
		if(gamay_set.insert(v_gamay_1)) {
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1183)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(168)) return -1;
				if(!backtrack_stack.push(167)) return -1;
				goto label_159;
		}
		break;
	case 2196:
	// Label to restore values of label 170
		v_gamay_1 = int_value_stack.pop();
		break;
	case 171: label_171: 
	// using  symbolic fact beaujolais(v_beaujolais_1) with literal 1 in rule 
	// 24: q22(A) :- beaujolais(A).
	// resulting in symbolic fact q22(v_q22_1)

		//recursive
		if(!int_value_stack.push(v_q22_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2197)) return -1;
		v_q22_1 = v_beaujolais_1;
		if(q22_set.insert(v_q22_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(248)) return -1;
		if(!backtrack_stack.push(245)) return -1;
		if(!backtrack_stack.push(243)) return -1;
		goto label_172;
		}
		break;
	case 2197:
	// Label to restore values of label 171
		v_q22_1 = int_value_stack.pop();
		break;
	case 172: label_172: 
	// using  symbolic fact q22(v_q22_1) with literal 1 in rule 
	// 23: q23(A) :- q22(A).
	// resulting in symbolic fact q23(v_q23_1)

		//recursive
		if(!int_value_stack.push(v_q23_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2198)) return -1;
		v_q23_1 = v_q22_1;
		if(q23_set.insert(v_q23_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(242)) return -1;
		if(!backtrack_stack.push(241)) return -1;
		goto label_173;
		}
		break;
	case 2198:
	// Label to restore values of label 172
		v_q23_1 = int_value_stack.pop();
		break;
	case 173: label_173: 
	// using  symbolic fact q23(v_q23_1) with literal 1 in rule 
	// 22: beaujolais(A) :- q23(A), wine(A).
	// resulting in symbolic fact beaujolais(v_beaujolais_1)

		//recursive
		if(!int_value_stack.push(v_beaujolais_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2199)) return -1;
		if (!wine_set.contains(v_q23_1)) continue;
	case 1186:
	// Continuation for label 173
			v_beaujolais_1 = v_q23_1;
		if(beaujolais_set.insert(v_beaujolais_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1186)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(239)) return -1;
			if(!backtrack_stack.push(200)) return -1;
			if(!backtrack_stack.push(186)) return -1;
			if(!backtrack_stack.push(185)) return -1;
			if(!backtrack_stack.push(181)) return -1;
			if(!backtrack_stack.push(175)) return -1;
			if(!backtrack_stack.push(174)) return -1;
			if(!backtrack_stack.push(171)) return -1;
			goto label_156;
		}
		break;
	case 2199:
	// Label to restore values of label 173
		v_beaujolais_1 = int_value_stack.pop();
		break;
	case 174: label_174: 
	// using  symbolic fact beaujolais(v_beaujolais_1) with literal 1 in rule 
	// 25: beaujolais(A) :- beaujolais(B), kaon2equal(A,B).
	// resulting in symbolic fact beaujolais(v_beaujolais_1)

		//recursive
		c25_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_beaujolais_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2200)) return -1;
		c25_2_fb.open(v_beaujolais_1);
	case 1187:
	// Continuation for label 174
		while (c25_2_fb.fetch()) {
			v_beaujolais_1 = c25_2_fb.c2();
		if(beaujolais_set.insert(v_beaujolais_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1187)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(239)) return -1;
			if(!backtrack_stack.push(200)) return -1;
			if(!backtrack_stack.push(186)) return -1;
			if(!backtrack_stack.push(185)) return -1;
			if(!backtrack_stack.push(181)) return -1;
			if(!backtrack_stack.push(175)) return -1;
			if(!backtrack_stack.push(174)) return -1;
			if(!backtrack_stack.push(171)) return -1;
			goto label_156;
		}
		}
		break;
	case 2200:
	// Label to restore values of label 174
		v_beaujolais_1 = int_value_stack.pop();
		c25_2_fb.pop(&cursor_stack);
		break;
	case 175: label_175: 
	// using  symbolic fact beaujolais(v_beaujolais_1) with literal 1 in rule 
	// 81: q45(A) :- beaujolais(A).
	// resulting in symbolic fact q45(v_q45_1)

		//recursive
		if(!int_value_stack.push(v_q45_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2201)) return -1;
		v_q45_1 = v_beaujolais_1;
		if(q45_set.insert(v_q45_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(240)) return -1;
		if(!backtrack_stack.push(177)) return -1;
		goto label_176;
		}
		break;
	case 2201:
	// Label to restore values of label 175
		v_q45_1 = int_value_stack.pop();
		break;
	case 176: label_176: 
	// using  symbolic fact q45(v_q45_1) with literal 1 in rule 
	// 80: hasbody(A,A) :- q45(A), kaon2namedobjects(A).
	// resulting in symbolic fact hasbody(v_hasbody_1,v_hasbody_2)

		if (!kaon2namedobjects_b.contains(v_q45_1)) continue;
	case 1189:
	// Continuation for label 176
			v_hasbody_1 = v_q45_1;
			v_hasbody_2 = v_q45_1;
		if(hasbody_set.insert(v_hasbody_1, v_hasbody_2)) {
			if(!hasbody_bf.insert(v_hasbody_1, v_hasbody_2)) return -1;
			if(!hasbody_fb.insert(v_hasbody_2, v_hasbody_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1189)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(429)) return -1;
			if(!backtrack_stack.push(61)) return -1;
			if(!backtrack_stack.push(60)) return -1;
			if(!backtrack_stack.push(59)) return -1;
			if(!backtrack_stack.push(58)) return -1;
			goto label_54;
		}
		break;
	case 177: label_177: 
	// using  symbolic fact q45(v_q45_1) with literal 1 in rule 
	// 403: wine(A) :- q45(A).
	// resulting in symbolic fact wine(v_wine_1)

		//recursive
		if(!int_value_stack.push(v_wine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2203)) return -1;
		v_wine_1 = v_q45_1;
		if(wine_set.insert(v_wine_1)) {
		if(!wine_f.insert(v_wine_1)) return -1;
		// Labels for next rules
		if(!backtrack_stack.push(1007)) return -1;
		if(!backtrack_stack.push(989)) return -1;
		if(!backtrack_stack.push(954)) return -1;
		if(!backtrack_stack.push(938)) return -1;
		if(!backtrack_stack.push(911)) return -1;
		if(!backtrack_stack.push(903)) return -1;
		if(!backtrack_stack.push(902)) return -1;
		if(!backtrack_stack.push(901)) return -1;
		if(!backtrack_stack.push(849)) return -1;
		if(!backtrack_stack.push(848)) return -1;
		if(!backtrack_stack.push(821)) return -1;
		if(!backtrack_stack.push(820)) return -1;
		if(!backtrack_stack.push(737)) return -1;
		if(!backtrack_stack.push(712)) return -1;
		if(!backtrack_stack.push(674)) return -1;
		if(!backtrack_stack.push(672)) return -1;
		if(!backtrack_stack.push(583)) return -1;
		if(!backtrack_stack.push(575)) return -1;
		if(!backtrack_stack.push(563)) return -1;
		if(!backtrack_stack.push(551)) return -1;
		if(!backtrack_stack.push(549)) return -1;
		if(!backtrack_stack.push(483)) return -1;
		if(!backtrack_stack.push(473)) return -1;
		if(!backtrack_stack.push(470)) return -1;
		if(!backtrack_stack.push(430)) return -1;
		if(!backtrack_stack.push(401)) return -1;
		if(!backtrack_stack.push(399)) return -1;
		if(!backtrack_stack.push(380)) return -1;
		if(!backtrack_stack.push(364)) return -1;
		if(!backtrack_stack.push(363)) return -1;
		if(!backtrack_stack.push(329)) return -1;
		if(!backtrack_stack.push(328)) return -1;
		if(!backtrack_stack.push(327)) return -1;
		if(!backtrack_stack.push(295)) return -1;
		if(!backtrack_stack.push(294)) return -1;
		if(!backtrack_stack.push(216)) return -1;
		if(!backtrack_stack.push(210)) return -1;
		if(!backtrack_stack.push(192)) return -1;
		if(!backtrack_stack.push(180)) return -1;
		if(!backtrack_stack.push(179)) return -1;
		if(!backtrack_stack.push(178)) return -1;
		if(!backtrack_stack.push(154)) return -1;
		if(!backtrack_stack.push(153)) return -1;
		if(!backtrack_stack.push(100)) return -1;
		if(!backtrack_stack.push(76)) return -1;
		if(!backtrack_stack.push(75)) return -1;
		if(!backtrack_stack.push(67)) return -1;
		if(!backtrack_stack.push(63)) return -1;
		if(!backtrack_stack.push(53)) return -1;
		if(!backtrack_stack.push(46)) return -1;
		if(!backtrack_stack.push(19)) return -1;
		goto label_16;
		}
		break;
	case 2203:
	// Label to restore values of label 177
		v_wine_1 = int_value_stack.pop();
		break;
	case 178: label_178: 
	// using  symbolic fact wine(v_wine_1) with literal 1 in rule 
	// 558: q5(A) :- wine(A), madefromgrape(A,B), ot____nom33(B).
	// resulting in symbolic fact q5(v_q5_1)

		c558_2_bf.open(v_wine_1);
		while (c558_2_bf.fetch()) {
			if (!ot____nom33_set.contains(c558_2_bf.c2())) continue;
	case 1191:
	// Continuation for label 178
				v_q5_1 = v_wine_1;
		if(q5_set.insert(v_q5_1)) {
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1191)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(136)) return -1;
				if(!backtrack_stack.push(135)) return -1;
				goto label_131;
		}
		}
		break;
	case 179: label_179: 
	// using  symbolic fact wine(v_wine_1) with literal 1 in rule 
	// 788: gamay(A) :- wine(A), madefromgrape(A,B), ot____nom33(B).
	// resulting in symbolic fact gamay(v_gamay_1)

		c788_2_bf.open(v_wine_1);
		while (c788_2_bf.fetch()) {
			if (!ot____nom33_set.contains(c788_2_bf.c2())) continue;
	case 1192:
	// Continuation for label 179
				v_gamay_1 = v_wine_1;
		if(gamay_set.insert(v_gamay_1)) {
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1192)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(168)) return -1;
				if(!backtrack_stack.push(167)) return -1;
				goto label_159;
		}
		}
		break;
	case 180: label_180: 
	// using  symbolic fact wine(v_wine_1) with literal 2 in rule 
	// 22: beaujolais(A) :- q23(A), wine(A).
	// resulting in symbolic fact beaujolais(v_beaujolais_1)

		//recursive
		if(!int_value_stack.push(v_beaujolais_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2206)) return -1;
		if (!q23_set.contains(v_wine_1)) continue;
	case 1193:
	// Continuation for label 180
			v_beaujolais_1 = v_wine_1;
		if(beaujolais_set.insert(v_beaujolais_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1193)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(239)) return -1;
			if(!backtrack_stack.push(200)) return -1;
			if(!backtrack_stack.push(186)) return -1;
			if(!backtrack_stack.push(185)) return -1;
			if(!backtrack_stack.push(181)) return -1;
			if(!backtrack_stack.push(175)) return -1;
			if(!backtrack_stack.push(174)) return -1;
			if(!backtrack_stack.push(171)) return -1;
			goto label_156;
		}
		break;
	case 2206:
	// Label to restore values of label 180
		v_beaujolais_1 = int_value_stack.pop();
		break;
	case 181: label_181: 
	// using  symbolic fact beaujolais(v_beaujolais_1) with literal 1 in rule 
	// 138: q20(A) :- beaujolais(A).
	// resulting in symbolic fact q20(v_q20_1)

		//recursive
		if(!int_value_stack.push(v_q20_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2207)) return -1;
		v_q20_1 = v_beaujolais_1;
		if(q20_set.insert(v_q20_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(184)) return -1;
		if(!backtrack_stack.push(183)) return -1;
		goto label_182;
		}
		break;
	case 2207:
	// Label to restore values of label 181
		v_q20_1 = int_value_stack.pop();
		break;
	case 182: label_182: 
	// using  symbolic fact q20(v_q20_1) with literal 1 in rule 
	// 135: wine(A) :- q20(A).
	// resulting in symbolic fact wine(v_wine_1)

		//recursive
		if(!int_value_stack.push(v_wine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2208)) return -1;
		v_wine_1 = v_q20_1;
		if(wine_set.insert(v_wine_1)) {
		if(!wine_f.insert(v_wine_1)) return -1;
		// Labels for next rules
		if(!backtrack_stack.push(1007)) return -1;
		if(!backtrack_stack.push(989)) return -1;
		if(!backtrack_stack.push(954)) return -1;
		if(!backtrack_stack.push(938)) return -1;
		if(!backtrack_stack.push(911)) return -1;
		if(!backtrack_stack.push(903)) return -1;
		if(!backtrack_stack.push(902)) return -1;
		if(!backtrack_stack.push(901)) return -1;
		if(!backtrack_stack.push(849)) return -1;
		if(!backtrack_stack.push(848)) return -1;
		if(!backtrack_stack.push(821)) return -1;
		if(!backtrack_stack.push(820)) return -1;
		if(!backtrack_stack.push(737)) return -1;
		if(!backtrack_stack.push(712)) return -1;
		if(!backtrack_stack.push(674)) return -1;
		if(!backtrack_stack.push(672)) return -1;
		if(!backtrack_stack.push(583)) return -1;
		if(!backtrack_stack.push(575)) return -1;
		if(!backtrack_stack.push(563)) return -1;
		if(!backtrack_stack.push(551)) return -1;
		if(!backtrack_stack.push(549)) return -1;
		if(!backtrack_stack.push(483)) return -1;
		if(!backtrack_stack.push(473)) return -1;
		if(!backtrack_stack.push(470)) return -1;
		if(!backtrack_stack.push(430)) return -1;
		if(!backtrack_stack.push(401)) return -1;
		if(!backtrack_stack.push(399)) return -1;
		if(!backtrack_stack.push(380)) return -1;
		if(!backtrack_stack.push(364)) return -1;
		if(!backtrack_stack.push(363)) return -1;
		if(!backtrack_stack.push(329)) return -1;
		if(!backtrack_stack.push(328)) return -1;
		if(!backtrack_stack.push(327)) return -1;
		if(!backtrack_stack.push(295)) return -1;
		if(!backtrack_stack.push(294)) return -1;
		if(!backtrack_stack.push(216)) return -1;
		if(!backtrack_stack.push(210)) return -1;
		if(!backtrack_stack.push(192)) return -1;
		if(!backtrack_stack.push(180)) return -1;
		if(!backtrack_stack.push(179)) return -1;
		if(!backtrack_stack.push(178)) return -1;
		if(!backtrack_stack.push(154)) return -1;
		if(!backtrack_stack.push(153)) return -1;
		if(!backtrack_stack.push(100)) return -1;
		if(!backtrack_stack.push(76)) return -1;
		if(!backtrack_stack.push(75)) return -1;
		if(!backtrack_stack.push(67)) return -1;
		if(!backtrack_stack.push(63)) return -1;
		if(!backtrack_stack.push(53)) return -1;
		if(!backtrack_stack.push(46)) return -1;
		if(!backtrack_stack.push(19)) return -1;
		goto label_16;
		}
		break;
	case 2208:
	// Label to restore values of label 182
		v_wine_1 = int_value_stack.pop();
		break;
	case 183: label_183: 
	// using  symbolic fact q20(v_q20_1) with literal 1 in rule 
	// 644: hasflavor(A,A) :- q20(A), kaon2namedobjects(A).
	// resulting in symbolic fact hasflavor(v_hasflavor_1,v_hasflavor_2)

		if (!kaon2namedobjects_b.contains(v_q20_1)) continue;
	case 1196:
	// Continuation for label 183
			v_hasflavor_1 = v_q20_1;
			v_hasflavor_2 = v_q20_1;
		if(hasflavor_set.insert(v_hasflavor_1, v_hasflavor_2)) {
			if(!hasflavor_bf.insert(v_hasflavor_1, v_hasflavor_2)) return -1;
			if(!hasflavor_fb.insert(v_hasflavor_2, v_hasflavor_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1196)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(32)) return -1;
			if(!backtrack_stack.push(31)) return -1;
			if(!backtrack_stack.push(30)) return -1;
			if(!backtrack_stack.push(14)) return -1;
			goto label_12;
		}
		break;
	case 184: label_184: 
	// using  symbolic fact q20(v_q20_1) with literal 1 in rule 
	// 646: q20(A) :- q20(B), kaon2equal(A,B).
	// resulting in symbolic fact q20(v_q20_1)

		//recursive
		c646_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q20_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2210)) return -1;
		c646_2_fb.open(v_q20_1);
	case 1197:
	// Continuation for label 184
		while (c646_2_fb.fetch()) {
			v_q20_1 = c646_2_fb.c2();
		if(q20_set.insert(v_q20_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1197)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(184)) return -1;
			if(!backtrack_stack.push(183)) return -1;
			goto label_182;
		}
		}
		break;
	case 2210:
	// Label to restore values of label 184
		v_q20_1 = int_value_stack.pop();
		c646_2_fb.pop(&cursor_stack);
		break;
	case 185: label_185: 
	// using  symbolic fact beaujolais(v_beaujolais_1) with literal 1 in rule 
	// 365: wine(A) :- beaujolais(A).
	// resulting in symbolic fact wine(v_wine_1)

		//recursive
		if(!int_value_stack.push(v_wine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2211)) return -1;
		v_wine_1 = v_beaujolais_1;
		if(wine_set.insert(v_wine_1)) {
		if(!wine_f.insert(v_wine_1)) return -1;
		// Labels for next rules
		if(!backtrack_stack.push(1007)) return -1;
		if(!backtrack_stack.push(989)) return -1;
		if(!backtrack_stack.push(954)) return -1;
		if(!backtrack_stack.push(938)) return -1;
		if(!backtrack_stack.push(911)) return -1;
		if(!backtrack_stack.push(903)) return -1;
		if(!backtrack_stack.push(902)) return -1;
		if(!backtrack_stack.push(901)) return -1;
		if(!backtrack_stack.push(849)) return -1;
		if(!backtrack_stack.push(848)) return -1;
		if(!backtrack_stack.push(821)) return -1;
		if(!backtrack_stack.push(820)) return -1;
		if(!backtrack_stack.push(737)) return -1;
		if(!backtrack_stack.push(712)) return -1;
		if(!backtrack_stack.push(674)) return -1;
		if(!backtrack_stack.push(672)) return -1;
		if(!backtrack_stack.push(583)) return -1;
		if(!backtrack_stack.push(575)) return -1;
		if(!backtrack_stack.push(563)) return -1;
		if(!backtrack_stack.push(551)) return -1;
		if(!backtrack_stack.push(549)) return -1;
		if(!backtrack_stack.push(483)) return -1;
		if(!backtrack_stack.push(473)) return -1;
		if(!backtrack_stack.push(470)) return -1;
		if(!backtrack_stack.push(430)) return -1;
		if(!backtrack_stack.push(401)) return -1;
		if(!backtrack_stack.push(399)) return -1;
		if(!backtrack_stack.push(380)) return -1;
		if(!backtrack_stack.push(364)) return -1;
		if(!backtrack_stack.push(363)) return -1;
		if(!backtrack_stack.push(329)) return -1;
		if(!backtrack_stack.push(328)) return -1;
		if(!backtrack_stack.push(327)) return -1;
		if(!backtrack_stack.push(295)) return -1;
		if(!backtrack_stack.push(294)) return -1;
		if(!backtrack_stack.push(216)) return -1;
		if(!backtrack_stack.push(210)) return -1;
		if(!backtrack_stack.push(192)) return -1;
		if(!backtrack_stack.push(180)) return -1;
		if(!backtrack_stack.push(179)) return -1;
		if(!backtrack_stack.push(178)) return -1;
		if(!backtrack_stack.push(154)) return -1;
		if(!backtrack_stack.push(153)) return -1;
		if(!backtrack_stack.push(100)) return -1;
		if(!backtrack_stack.push(76)) return -1;
		if(!backtrack_stack.push(75)) return -1;
		if(!backtrack_stack.push(67)) return -1;
		if(!backtrack_stack.push(63)) return -1;
		if(!backtrack_stack.push(53)) return -1;
		if(!backtrack_stack.push(46)) return -1;
		if(!backtrack_stack.push(19)) return -1;
		goto label_16;
		}
		break;
	case 2211:
	// Label to restore values of label 185
		v_wine_1 = int_value_stack.pop();
		break;
	case 186: label_186: 
	// using  symbolic fact beaujolais(v_beaujolais_1) with literal 1 in rule 
	// 438: q0(A) :- beaujolais(A).
	// resulting in symbolic fact q0(v_q0_1)

		//recursive
		if(!int_value_stack.push(v_q0_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2212)) return -1;
		v_q0_1 = v_beaujolais_1;
		if(q0_set.insert(v_q0_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(198)) return -1;
		if(!backtrack_stack.push(194)) return -1;
		if(!backtrack_stack.push(189)) return -1;
		if(!backtrack_stack.push(188)) return -1;
		goto label_187;
		}
		break;
	case 2212:
	// Label to restore values of label 186
		v_q0_1 = int_value_stack.pop();
		break;
	case 187: label_187: 
	// using  symbolic fact q0(v_q0_1) with literal 1 in rule 
	// 42: hascolor(A,A) :- q0(A), kaon2namedobjects(A).
	// resulting in symbolic fact hascolor(v_hascolor_1,v_hascolor_2)

		if (!kaon2namedobjects_b.contains(v_q0_1)) continue;
	case 1200:
	// Continuation for label 187
			v_hascolor_1 = v_q0_1;
			v_hascolor_2 = v_q0_1;
		if(hascolor_set.insert(v_hascolor_1, v_hascolor_2)) {
			if(!hascolor_bf.insert(v_hascolor_1, v_hascolor_2)) return -1;
			if(!hascolor_fb.insert(v_hascolor_2, v_hascolor_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1200)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(286)) return -1;
			if(!backtrack_stack.push(120)) return -1;
			if(!backtrack_stack.push(80)) return -1;
			if(!backtrack_stack.push(79)) return -1;
			if(!backtrack_stack.push(78)) return -1;
			if(!backtrack_stack.push(77)) return -1;
			if(!backtrack_stack.push(74)) return -1;
			goto label_68;
		}
		break;
	case 188: label_188: 
	// using  symbolic fact q0(v_q0_1) with literal 1 in rule 
	// 319: wine(A) :- q0(A).
	// resulting in symbolic fact wine(v_wine_1)

		//recursive
		if(!int_value_stack.push(v_wine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2214)) return -1;
		v_wine_1 = v_q0_1;
		if(wine_set.insert(v_wine_1)) {
		if(!wine_f.insert(v_wine_1)) return -1;
		// Labels for next rules
		if(!backtrack_stack.push(1007)) return -1;
		if(!backtrack_stack.push(989)) return -1;
		if(!backtrack_stack.push(954)) return -1;
		if(!backtrack_stack.push(938)) return -1;
		if(!backtrack_stack.push(911)) return -1;
		if(!backtrack_stack.push(903)) return -1;
		if(!backtrack_stack.push(902)) return -1;
		if(!backtrack_stack.push(901)) return -1;
		if(!backtrack_stack.push(849)) return -1;
		if(!backtrack_stack.push(848)) return -1;
		if(!backtrack_stack.push(821)) return -1;
		if(!backtrack_stack.push(820)) return -1;
		if(!backtrack_stack.push(737)) return -1;
		if(!backtrack_stack.push(712)) return -1;
		if(!backtrack_stack.push(674)) return -1;
		if(!backtrack_stack.push(672)) return -1;
		if(!backtrack_stack.push(583)) return -1;
		if(!backtrack_stack.push(575)) return -1;
		if(!backtrack_stack.push(563)) return -1;
		if(!backtrack_stack.push(551)) return -1;
		if(!backtrack_stack.push(549)) return -1;
		if(!backtrack_stack.push(483)) return -1;
		if(!backtrack_stack.push(473)) return -1;
		if(!backtrack_stack.push(470)) return -1;
		if(!backtrack_stack.push(430)) return -1;
		if(!backtrack_stack.push(401)) return -1;
		if(!backtrack_stack.push(399)) return -1;
		if(!backtrack_stack.push(380)) return -1;
		if(!backtrack_stack.push(364)) return -1;
		if(!backtrack_stack.push(363)) return -1;
		if(!backtrack_stack.push(329)) return -1;
		if(!backtrack_stack.push(328)) return -1;
		if(!backtrack_stack.push(327)) return -1;
		if(!backtrack_stack.push(295)) return -1;
		if(!backtrack_stack.push(294)) return -1;
		if(!backtrack_stack.push(216)) return -1;
		if(!backtrack_stack.push(210)) return -1;
		if(!backtrack_stack.push(192)) return -1;
		if(!backtrack_stack.push(180)) return -1;
		if(!backtrack_stack.push(179)) return -1;
		if(!backtrack_stack.push(178)) return -1;
		if(!backtrack_stack.push(154)) return -1;
		if(!backtrack_stack.push(153)) return -1;
		if(!backtrack_stack.push(100)) return -1;
		if(!backtrack_stack.push(76)) return -1;
		if(!backtrack_stack.push(75)) return -1;
		if(!backtrack_stack.push(67)) return -1;
		if(!backtrack_stack.push(63)) return -1;
		if(!backtrack_stack.push(53)) return -1;
		if(!backtrack_stack.push(46)) return -1;
		if(!backtrack_stack.push(19)) return -1;
		goto label_16;
		}
		break;
	case 2214:
	// Label to restore values of label 188
		v_wine_1 = int_value_stack.pop();
		break;
	case 189: label_189: 
	// using  symbolic fact q0(v_q0_1) with literal 1 in rule 
	// 437: q41(A) :- q0(A).
	// resulting in symbolic fact q41(v_q41_1)

		//recursive
		if(!int_value_stack.push(v_q41_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2215)) return -1;
		v_q41_1 = v_q0_1;
		if(q41_set.insert(v_q41_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(272)) return -1;
		if(!backtrack_stack.push(197)) return -1;
		goto label_190;
		}
		break;
	case 2215:
	// Label to restore values of label 189
		v_q41_1 = int_value_stack.pop();
		break;
	case 190: label_190: 
	// using  symbolic fact q41(v_q41_1) with literal 1 in rule 
	// 436: redwine(A) :- q41(A), wine(A).
	// resulting in symbolic fact redwine(v_redwine_1)

		//recursive
		if(!int_value_stack.push(v_redwine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2216)) return -1;
		if (!wine_set.contains(v_q41_1)) continue;
	case 1203:
	// Continuation for label 190
			v_redwine_1 = v_q41_1;
		if(redwine_set.insert(v_redwine_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1203)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(569)) return -1;
			if(!backtrack_stack.push(384)) return -1;
			if(!backtrack_stack.push(205)) return -1;
			if(!backtrack_stack.push(199)) return -1;
			if(!backtrack_stack.push(193)) return -1;
			goto label_191;
		}
		break;
	case 2216:
	// Label to restore values of label 190
		v_redwine_1 = int_value_stack.pop();
		break;
	case 191: label_191: 
	// using  symbolic fact redwine(v_redwine_1) with literal 1 in rule 
	// 434: wine(A) :- redwine(A).
	// resulting in symbolic fact wine(v_wine_1)

		//recursive
		if(!int_value_stack.push(v_wine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2217)) return -1;
		v_wine_1 = v_redwine_1;
		if(wine_set.insert(v_wine_1)) {
		if(!wine_f.insert(v_wine_1)) return -1;
		// Labels for next rules
		if(!backtrack_stack.push(1007)) return -1;
		if(!backtrack_stack.push(989)) return -1;
		if(!backtrack_stack.push(954)) return -1;
		if(!backtrack_stack.push(938)) return -1;
		if(!backtrack_stack.push(911)) return -1;
		if(!backtrack_stack.push(903)) return -1;
		if(!backtrack_stack.push(902)) return -1;
		if(!backtrack_stack.push(901)) return -1;
		if(!backtrack_stack.push(849)) return -1;
		if(!backtrack_stack.push(848)) return -1;
		if(!backtrack_stack.push(821)) return -1;
		if(!backtrack_stack.push(820)) return -1;
		if(!backtrack_stack.push(737)) return -1;
		if(!backtrack_stack.push(712)) return -1;
		if(!backtrack_stack.push(674)) return -1;
		if(!backtrack_stack.push(672)) return -1;
		if(!backtrack_stack.push(583)) return -1;
		if(!backtrack_stack.push(575)) return -1;
		if(!backtrack_stack.push(563)) return -1;
		if(!backtrack_stack.push(551)) return -1;
		if(!backtrack_stack.push(549)) return -1;
		if(!backtrack_stack.push(483)) return -1;
		if(!backtrack_stack.push(473)) return -1;
		if(!backtrack_stack.push(470)) return -1;
		if(!backtrack_stack.push(430)) return -1;
		if(!backtrack_stack.push(401)) return -1;
		if(!backtrack_stack.push(399)) return -1;
		if(!backtrack_stack.push(380)) return -1;
		if(!backtrack_stack.push(364)) return -1;
		if(!backtrack_stack.push(363)) return -1;
		if(!backtrack_stack.push(329)) return -1;
		if(!backtrack_stack.push(328)) return -1;
		if(!backtrack_stack.push(327)) return -1;
		if(!backtrack_stack.push(295)) return -1;
		if(!backtrack_stack.push(294)) return -1;
		if(!backtrack_stack.push(216)) return -1;
		if(!backtrack_stack.push(210)) return -1;
		if(!backtrack_stack.push(192)) return -1;
		if(!backtrack_stack.push(180)) return -1;
		if(!backtrack_stack.push(179)) return -1;
		if(!backtrack_stack.push(178)) return -1;
		if(!backtrack_stack.push(154)) return -1;
		if(!backtrack_stack.push(153)) return -1;
		if(!backtrack_stack.push(100)) return -1;
		if(!backtrack_stack.push(76)) return -1;
		if(!backtrack_stack.push(75)) return -1;
		if(!backtrack_stack.push(67)) return -1;
		if(!backtrack_stack.push(63)) return -1;
		if(!backtrack_stack.push(53)) return -1;
		if(!backtrack_stack.push(46)) return -1;
		if(!backtrack_stack.push(19)) return -1;
		goto label_16;
		}
		break;
	case 2217:
	// Label to restore values of label 191
		v_wine_1 = int_value_stack.pop();
		break;
	case 192: label_192: 
	// using  symbolic fact wine(v_wine_1) with literal 2 in rule 
	// 436: redwine(A) :- q41(A), wine(A).
	// resulting in symbolic fact redwine(v_redwine_1)

		//recursive
		if(!int_value_stack.push(v_redwine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2218)) return -1;
		if (!q41_set.contains(v_wine_1)) continue;
	case 1205:
	// Continuation for label 192
			v_redwine_1 = v_wine_1;
		if(redwine_set.insert(v_redwine_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1205)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(569)) return -1;
			if(!backtrack_stack.push(384)) return -1;
			if(!backtrack_stack.push(205)) return -1;
			if(!backtrack_stack.push(199)) return -1;
			if(!backtrack_stack.push(193)) return -1;
			goto label_191;
		}
		break;
	case 2218:
	// Label to restore values of label 192
		v_redwine_1 = int_value_stack.pop();
		break;
	case 193: label_193: 
	// using  symbolic fact redwine(v_redwine_1) with literal 1 in rule 
	// 606: q0(A) :- redwine(A).
	// resulting in symbolic fact q0(v_q0_1)

		//recursive
		if(!int_value_stack.push(v_q0_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2219)) return -1;
		v_q0_1 = v_redwine_1;
		if(q0_set.insert(v_q0_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(198)) return -1;
		if(!backtrack_stack.push(194)) return -1;
		if(!backtrack_stack.push(189)) return -1;
		if(!backtrack_stack.push(188)) return -1;
		goto label_187;
		}
		break;
	case 2219:
	// Label to restore values of label 193
		v_q0_1 = int_value_stack.pop();
		break;
	case 194: label_194: 
	// using  symbolic fact q0(v_q0_1) with literal 1 in rule 
	// 604: ot____nom28(A) :- q0(A), kaon2namedobjects(A).
	// resulting in symbolic fact ot____nom28(v_ot____nom28_1)

		//recursive
		if(!int_value_stack.push(v_ot____nom28_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2220)) return -1;
		if (!kaon2namedobjects_b.contains(v_q0_1)) continue;
	case 1207:
	// Continuation for label 194
			v_ot____nom28_1 = v_q0_1;
		if(ot____nom28_set.insert(v_ot____nom28_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1207)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(196)) return -1;
			goto label_195;
		}
		break;
	case 2220:
	// Label to restore values of label 194
		v_ot____nom28_1 = int_value_stack.pop();
		break;
	case 195: label_195: 
	// using  symbolic fact ot____nom28(v_ot____nom28_1) with literal 1 in rule 
	// 611: ot____nom28(A) :- ot____nom28(B), kaon2equal(A,B).
	// resulting in symbolic fact ot____nom28(v_ot____nom28_1)

		//recursive
		c611_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_ot____nom28_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2221)) return -1;
		c611_2_fb.open(v_ot____nom28_1);
	case 1208:
	// Continuation for label 195
		while (c611_2_fb.fetch()) {
			v_ot____nom28_1 = c611_2_fb.c2();
		if(ot____nom28_set.insert(v_ot____nom28_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1208)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(196)) return -1;
			goto label_195;
		}
		}
		break;
	case 2221:
	// Label to restore values of label 195
		v_ot____nom28_1 = int_value_stack.pop();
		c611_2_fb.pop(&cursor_stack);
		break;
	case 196: label_196: 
	// using  symbolic fact ot____nom28(v_ot____nom28_1) with literal 2 in rule 
	// 446: q41(A) :- hascolor(A,B), ot____nom28(B).
	// resulting in symbolic fact q41(v_q41_1)

		//recursive
		c446_1_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q41_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2222)) return -1;
		c446_1_fb.open(v_ot____nom28_1);
	case 1209:
	// Continuation for label 196
		while (c446_1_fb.fetch()) {
			v_q41_1 = c446_1_fb.c2();
		if(q41_set.insert(v_q41_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1209)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(272)) return -1;
			if(!backtrack_stack.push(197)) return -1;
			goto label_190;
		}
		}
		break;
	case 2222:
	// Label to restore values of label 196
		v_q41_1 = int_value_stack.pop();
		c446_1_fb.pop(&cursor_stack);
		break;
	case 197: label_197: 
	// using  symbolic fact q41(v_q41_1) with literal 1 in rule 
	// 612: q41(A) :- q41(B), kaon2equal(A,B).
	// resulting in symbolic fact q41(v_q41_1)

		//recursive
		c612_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q41_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2223)) return -1;
		c612_2_fb.open(v_q41_1);
	case 1210:
	// Continuation for label 197
		while (c612_2_fb.fetch()) {
			v_q41_1 = c612_2_fb.c2();
		if(q41_set.insert(v_q41_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1210)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(272)) return -1;
			if(!backtrack_stack.push(197)) return -1;
			goto label_190;
		}
		}
		break;
	case 2223:
	// Label to restore values of label 197
		v_q41_1 = int_value_stack.pop();
		c612_2_fb.pop(&cursor_stack);
		break;
	case 198: label_198: 
	// using  symbolic fact q0(v_q0_1) with literal 1 in rule 
	// 610: q0(A) :- q0(B), kaon2equal(A,B).
	// resulting in symbolic fact q0(v_q0_1)

		//recursive
		c610_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q0_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2224)) return -1;
		c610_2_fb.open(v_q0_1);
	case 1211:
	// Continuation for label 198
		while (c610_2_fb.fetch()) {
			v_q0_1 = c610_2_fb.c2();
		if(q0_set.insert(v_q0_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1211)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(198)) return -1;
			if(!backtrack_stack.push(194)) return -1;
			if(!backtrack_stack.push(189)) return -1;
			if(!backtrack_stack.push(188)) return -1;
			goto label_187;
		}
		}
		break;
	case 2224:
	// Label to restore values of label 198
		v_q0_1 = int_value_stack.pop();
		c610_2_fb.pop(&cursor_stack);
		break;
	case 199: label_199: 
	// using  symbolic fact redwine(v_redwine_1) with literal 1 in rule 
	// 607: redwine(A) :- redwine(B), kaon2equal(A,B).
	// resulting in symbolic fact redwine(v_redwine_1)

		//recursive
		c607_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_redwine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2225)) return -1;
		c607_2_fb.open(v_redwine_1);
	case 1212:
	// Continuation for label 199
		while (c607_2_fb.fetch()) {
			v_redwine_1 = c607_2_fb.c2();
		if(redwine_set.insert(v_redwine_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1212)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(569)) return -1;
			if(!backtrack_stack.push(384)) return -1;
			if(!backtrack_stack.push(205)) return -1;
			if(!backtrack_stack.push(199)) return -1;
			if(!backtrack_stack.push(193)) return -1;
			goto label_191;
		}
		}
		break;
	case 2225:
	// Label to restore values of label 199
		v_redwine_1 = int_value_stack.pop();
		c607_2_fb.pop(&cursor_stack);
		break;
	case 200: label_200: 
	// using  symbolic fact beaujolais(v_beaujolais_1) with literal 1 in rule 
	// 634: q31(A) :- beaujolais(A).
	// resulting in symbolic fact q31(v_q31_1)

		//recursive
		if(!int_value_stack.push(v_q31_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2226)) return -1;
		v_q31_1 = v_beaujolais_1;
		if(q31_set.insert(v_q31_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(221)) return -1;
		if(!backtrack_stack.push(220)) return -1;
		if(!backtrack_stack.push(215)) return -1;
		if(!backtrack_stack.push(213)) return -1;
		goto label_201;
		}
		break;
	case 2226:
	// Label to restore values of label 200
		v_q31_1 = int_value_stack.pop();
		break;
	case 201: label_201: 
	// using  symbolic fact q31(v_q31_1) with literal 1 in rule 
	// 57: q46(A) :- q31(A).
	// resulting in symbolic fact q46(v_q46_1)

		//recursive
		if(!int_value_stack.push(v_q46_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2227)) return -1;
		v_q46_1 = v_q31_1;
		if(q46_set.insert(v_q46_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(233)) return -1;
		if(!backtrack_stack.push(224)) return -1;
		goto label_202;
		}
		break;
	case 2227:
	// Label to restore values of label 201
		v_q46_1 = int_value_stack.pop();
		break;
	case 202: label_202: 
	// using  symbolic fact q46(v_q46_1) with literal 1 in rule 
	// 240: drywine(A) :- q46(A), wine(A).
	// resulting in symbolic fact drywine(v_drywine_1)

		//recursive
		if(!int_value_stack.push(v_drywine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2228)) return -1;
		if (!wine_set.contains(v_q46_1)) continue;
	case 1215:
	// Continuation for label 202
			v_drywine_1 = v_q46_1;
		if(drywine_set.insert(v_drywine_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1215)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(212)) return -1;
			if(!backtrack_stack.push(211)) return -1;
			if(!backtrack_stack.push(209)) return -1;
			if(!backtrack_stack.push(207)) return -1;
			goto label_203;
		}
		break;
	case 2228:
	// Label to restore values of label 202
		v_drywine_1 = int_value_stack.pop();
		break;
	case 203: label_203: 
	// using  symbolic fact drywine(v_drywine_1) with literal 1 in rule 
	// 235: dryredwine(A) :- drywine(A), redwine(A).
	// resulting in symbolic fact dryredwine(v_dryredwine_1)

		//recursive
		if(!int_value_stack.push(v_dryredwine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2229)) return -1;
		if (!redwine_set.contains(v_drywine_1)) continue;
	case 1216:
	// Continuation for label 203
			v_dryredwine_1 = v_drywine_1;
		if(dryredwine_set.insert(v_dryredwine_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1216)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(237)) return -1;
			if(!backtrack_stack.push(206)) return -1;
			goto label_204;
		}
		break;
	case 2229:
	// Label to restore values of label 203
		v_dryredwine_1 = int_value_stack.pop();
		break;
	case 204: label_204: 
	// using  symbolic fact dryredwine(v_dryredwine_1) with literal 1 in rule 
	// 234: redwine(A) :- dryredwine(A).
	// resulting in symbolic fact redwine(v_redwine_1)

		//recursive
		if(!int_value_stack.push(v_redwine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2230)) return -1;
		v_redwine_1 = v_dryredwine_1;
		if(redwine_set.insert(v_redwine_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(569)) return -1;
		if(!backtrack_stack.push(384)) return -1;
		if(!backtrack_stack.push(205)) return -1;
		if(!backtrack_stack.push(199)) return -1;
		if(!backtrack_stack.push(193)) return -1;
		goto label_191;
		}
		break;
	case 2230:
	// Label to restore values of label 204
		v_redwine_1 = int_value_stack.pop();
		break;
	case 205: label_205: 
	// using  symbolic fact redwine(v_redwine_1) with literal 2 in rule 
	// 235: dryredwine(A) :- drywine(A), redwine(A).
	// resulting in symbolic fact dryredwine(v_dryredwine_1)

		//recursive
		if(!int_value_stack.push(v_dryredwine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2231)) return -1;
		if (!drywine_set.contains(v_redwine_1)) continue;
	case 1218:
	// Continuation for label 205
			v_dryredwine_1 = v_redwine_1;
		if(dryredwine_set.insert(v_dryredwine_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1218)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(237)) return -1;
			if(!backtrack_stack.push(206)) return -1;
			goto label_204;
		}
		break;
	case 2231:
	// Label to restore values of label 205
		v_dryredwine_1 = int_value_stack.pop();
		break;
	case 206: label_206: 
	// using  symbolic fact dryredwine(v_dryredwine_1) with literal 1 in rule 
	// 238: drywine(A) :- dryredwine(A).
	// resulting in symbolic fact drywine(v_drywine_1)

		//recursive
		if(!int_value_stack.push(v_drywine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2232)) return -1;
		v_drywine_1 = v_dryredwine_1;
		if(drywine_set.insert(v_drywine_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(212)) return -1;
		if(!backtrack_stack.push(211)) return -1;
		if(!backtrack_stack.push(209)) return -1;
		if(!backtrack_stack.push(207)) return -1;
		goto label_203;
		}
		break;
	case 2232:
	// Label to restore values of label 206
		v_drywine_1 = int_value_stack.pop();
		break;
	case 207: label_207: 
	// using  symbolic fact drywine(v_drywine_1) with literal 1 in rule 
	// 237: drywhitewine(A) :- drywine(A), whitewine(A).
	// resulting in symbolic fact drywhitewine(v_drywhitewine_1)

		//recursive
		if(!int_value_stack.push(v_drywhitewine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2233)) return -1;
		if (!whitewine_set.contains(v_drywine_1)) continue;
	case 1220:
	// Continuation for label 207
			v_drywhitewine_1 = v_drywine_1;
		if(drywhitewine_set.insert(v_drywhitewine_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1220)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(236)) return -1;
			if(!backtrack_stack.push(234)) return -1;
			goto label_208;
		}
		break;
	case 2233:
	// Label to restore values of label 207
		v_drywhitewine_1 = int_value_stack.pop();
		break;
	case 208: label_208: 
	// using  symbolic fact drywhitewine(v_drywhitewine_1) with literal 1 in rule 
	// 236: drywine(A) :- drywhitewine(A).
	// resulting in symbolic fact drywine(v_drywine_1)

		//recursive
		if(!int_value_stack.push(v_drywine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2234)) return -1;
		v_drywine_1 = v_drywhitewine_1;
		if(drywine_set.insert(v_drywine_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(212)) return -1;
		if(!backtrack_stack.push(211)) return -1;
		if(!backtrack_stack.push(209)) return -1;
		if(!backtrack_stack.push(207)) return -1;
		goto label_203;
		}
		break;
	case 2234:
	// Label to restore values of label 208
		v_drywine_1 = int_value_stack.pop();
		break;
	case 209: label_209: 
	// using  symbolic fact drywine(v_drywine_1) with literal 1 in rule 
	// 469: wine(A) :- drywine(A).
	// resulting in symbolic fact wine(v_wine_1)

		//recursive
		if(!int_value_stack.push(v_wine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2235)) return -1;
		v_wine_1 = v_drywine_1;
		if(wine_set.insert(v_wine_1)) {
		if(!wine_f.insert(v_wine_1)) return -1;
		// Labels for next rules
		if(!backtrack_stack.push(1007)) return -1;
		if(!backtrack_stack.push(989)) return -1;
		if(!backtrack_stack.push(954)) return -1;
		if(!backtrack_stack.push(938)) return -1;
		if(!backtrack_stack.push(911)) return -1;
		if(!backtrack_stack.push(903)) return -1;
		if(!backtrack_stack.push(902)) return -1;
		if(!backtrack_stack.push(901)) return -1;
		if(!backtrack_stack.push(849)) return -1;
		if(!backtrack_stack.push(848)) return -1;
		if(!backtrack_stack.push(821)) return -1;
		if(!backtrack_stack.push(820)) return -1;
		if(!backtrack_stack.push(737)) return -1;
		if(!backtrack_stack.push(712)) return -1;
		if(!backtrack_stack.push(674)) return -1;
		if(!backtrack_stack.push(672)) return -1;
		if(!backtrack_stack.push(583)) return -1;
		if(!backtrack_stack.push(575)) return -1;
		if(!backtrack_stack.push(563)) return -1;
		if(!backtrack_stack.push(551)) return -1;
		if(!backtrack_stack.push(549)) return -1;
		if(!backtrack_stack.push(483)) return -1;
		if(!backtrack_stack.push(473)) return -1;
		if(!backtrack_stack.push(470)) return -1;
		if(!backtrack_stack.push(430)) return -1;
		if(!backtrack_stack.push(401)) return -1;
		if(!backtrack_stack.push(399)) return -1;
		if(!backtrack_stack.push(380)) return -1;
		if(!backtrack_stack.push(364)) return -1;
		if(!backtrack_stack.push(363)) return -1;
		if(!backtrack_stack.push(329)) return -1;
		if(!backtrack_stack.push(328)) return -1;
		if(!backtrack_stack.push(327)) return -1;
		if(!backtrack_stack.push(295)) return -1;
		if(!backtrack_stack.push(294)) return -1;
		if(!backtrack_stack.push(216)) return -1;
		if(!backtrack_stack.push(210)) return -1;
		if(!backtrack_stack.push(192)) return -1;
		if(!backtrack_stack.push(180)) return -1;
		if(!backtrack_stack.push(179)) return -1;
		if(!backtrack_stack.push(178)) return -1;
		if(!backtrack_stack.push(154)) return -1;
		if(!backtrack_stack.push(153)) return -1;
		if(!backtrack_stack.push(100)) return -1;
		if(!backtrack_stack.push(76)) return -1;
		if(!backtrack_stack.push(75)) return -1;
		if(!backtrack_stack.push(67)) return -1;
		if(!backtrack_stack.push(63)) return -1;
		if(!backtrack_stack.push(53)) return -1;
		if(!backtrack_stack.push(46)) return -1;
		if(!backtrack_stack.push(19)) return -1;
		goto label_16;
		}
		break;
	case 2235:
	// Label to restore values of label 209
		v_wine_1 = int_value_stack.pop();
		break;
	case 210: label_210: 
	// using  symbolic fact wine(v_wine_1) with literal 2 in rule 
	// 240: drywine(A) :- q46(A), wine(A).
	// resulting in symbolic fact drywine(v_drywine_1)

		//recursive
		if(!int_value_stack.push(v_drywine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2236)) return -1;
		if (!q46_set.contains(v_wine_1)) continue;
	case 1223:
	// Continuation for label 210
			v_drywine_1 = v_wine_1;
		if(drywine_set.insert(v_drywine_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1223)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(212)) return -1;
			if(!backtrack_stack.push(211)) return -1;
			if(!backtrack_stack.push(209)) return -1;
			if(!backtrack_stack.push(207)) return -1;
			goto label_203;
		}
		break;
	case 2236:
	// Label to restore values of label 210
		v_drywine_1 = int_value_stack.pop();
		break;
	case 211: label_211: 
	// using  symbolic fact drywine(v_drywine_1) with literal 1 in rule 
	// 470: drywine(A) :- drywine(B), kaon2equal(A,B).
	// resulting in symbolic fact drywine(v_drywine_1)

		//recursive
		c470_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_drywine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2237)) return -1;
		c470_2_fb.open(v_drywine_1);
	case 1224:
	// Continuation for label 211
		while (c470_2_fb.fetch()) {
			v_drywine_1 = c470_2_fb.c2();
		if(drywine_set.insert(v_drywine_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1224)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(212)) return -1;
			if(!backtrack_stack.push(211)) return -1;
			if(!backtrack_stack.push(209)) return -1;
			if(!backtrack_stack.push(207)) return -1;
			goto label_203;
		}
		}
		break;
	case 2237:
	// Label to restore values of label 211
		v_drywine_1 = int_value_stack.pop();
		c470_2_fb.pop(&cursor_stack);
		break;
	case 212: label_212: 
	// using  symbolic fact drywine(v_drywine_1) with literal 1 in rule 
	// 633: q31(A) :- drywine(A).
	// resulting in symbolic fact q31(v_q31_1)

		//recursive
		if(!int_value_stack.push(v_q31_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2238)) return -1;
		v_q31_1 = v_drywine_1;
		if(q31_set.insert(v_q31_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(221)) return -1;
		if(!backtrack_stack.push(220)) return -1;
		if(!backtrack_stack.push(215)) return -1;
		if(!backtrack_stack.push(213)) return -1;
		goto label_201;
		}
		break;
	case 2238:
	// Label to restore values of label 212
		v_q31_1 = int_value_stack.pop();
		break;
	case 213: label_213: 
	// using  symbolic fact q31(v_q31_1) with literal 1 in rule 
	// 242: hassugar(A,A) :- q31(A), kaon2namedobjects(A).
	// resulting in symbolic fact hassugar(v_hassugar_1,v_hassugar_2)

		if (!kaon2namedobjects_b.contains(v_q31_1)) continue;
	case 1226:
	// Continuation for label 213
			v_hassugar_1 = v_q31_1;
			v_hassugar_2 = v_q31_1;
		if(hassugar_set.insert(v_hassugar_1, v_hassugar_2)) {
			if(!hassugar_bf.insert(v_hassugar_1, v_hassugar_2)) return -1;
			if(!hassugar_fb.insert(v_hassugar_2, v_hassugar_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1226)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(755)) return -1;
			if(!backtrack_stack.push(238)) return -1;
			if(!backtrack_stack.push(214)) return -1;
			if(!backtrack_stack.push(27)) return -1;
			if(!backtrack_stack.push(24)) return -1;
			if(!backtrack_stack.push(23)) return -1;
			if(!backtrack_stack.push(22)) return -1;
			goto label_17;
		}
		break;
	case 214: label_214: 
	// using  symbolic fact hassugar(v_hassugar_1,v_hassugar_2) with literal 2 in rule 
	// 587: ot____nom7(A) :- q36(B), hassugar(B,A).
	// resulting in symbolic fact ot____nom7(v_ot____nom7_1)

		if (!q36_set.contains(v_hassugar_1)) continue;
	case 1227:
	// Continuation for label 214
			v_ot____nom7_1 = v_hassugar_2;
		if(ot____nom7_set.insert(v_ot____nom7_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1227)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(110)) return -1;
			goto label_109;
		}
		break;
	case 215: label_215: 
	// using  symbolic fact q31(v_q31_1) with literal 1 in rule 
	// 269: wine(A) :- q31(A).
	// resulting in symbolic fact wine(v_wine_1)

		//recursive
		if(!int_value_stack.push(v_wine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2241)) return -1;
		v_wine_1 = v_q31_1;
		if(wine_set.insert(v_wine_1)) {
		if(!wine_f.insert(v_wine_1)) return -1;
		// Labels for next rules
		if(!backtrack_stack.push(1007)) return -1;
		if(!backtrack_stack.push(989)) return -1;
		if(!backtrack_stack.push(954)) return -1;
		if(!backtrack_stack.push(938)) return -1;
		if(!backtrack_stack.push(911)) return -1;
		if(!backtrack_stack.push(903)) return -1;
		if(!backtrack_stack.push(902)) return -1;
		if(!backtrack_stack.push(901)) return -1;
		if(!backtrack_stack.push(849)) return -1;
		if(!backtrack_stack.push(848)) return -1;
		if(!backtrack_stack.push(821)) return -1;
		if(!backtrack_stack.push(820)) return -1;
		if(!backtrack_stack.push(737)) return -1;
		if(!backtrack_stack.push(712)) return -1;
		if(!backtrack_stack.push(674)) return -1;
		if(!backtrack_stack.push(672)) return -1;
		if(!backtrack_stack.push(583)) return -1;
		if(!backtrack_stack.push(575)) return -1;
		if(!backtrack_stack.push(563)) return -1;
		if(!backtrack_stack.push(551)) return -1;
		if(!backtrack_stack.push(549)) return -1;
		if(!backtrack_stack.push(483)) return -1;
		if(!backtrack_stack.push(473)) return -1;
		if(!backtrack_stack.push(470)) return -1;
		if(!backtrack_stack.push(430)) return -1;
		if(!backtrack_stack.push(401)) return -1;
		if(!backtrack_stack.push(399)) return -1;
		if(!backtrack_stack.push(380)) return -1;
		if(!backtrack_stack.push(364)) return -1;
		if(!backtrack_stack.push(363)) return -1;
		if(!backtrack_stack.push(329)) return -1;
		if(!backtrack_stack.push(328)) return -1;
		if(!backtrack_stack.push(327)) return -1;
		if(!backtrack_stack.push(295)) return -1;
		if(!backtrack_stack.push(294)) return -1;
		if(!backtrack_stack.push(216)) return -1;
		if(!backtrack_stack.push(210)) return -1;
		if(!backtrack_stack.push(192)) return -1;
		if(!backtrack_stack.push(180)) return -1;
		if(!backtrack_stack.push(179)) return -1;
		if(!backtrack_stack.push(178)) return -1;
		if(!backtrack_stack.push(154)) return -1;
		if(!backtrack_stack.push(153)) return -1;
		if(!backtrack_stack.push(100)) return -1;
		if(!backtrack_stack.push(76)) return -1;
		if(!backtrack_stack.push(75)) return -1;
		if(!backtrack_stack.push(67)) return -1;
		if(!backtrack_stack.push(63)) return -1;
		if(!backtrack_stack.push(53)) return -1;
		if(!backtrack_stack.push(46)) return -1;
		if(!backtrack_stack.push(19)) return -1;
		goto label_16;
		}
		break;
	case 2241:
	// Label to restore values of label 215
		v_wine_1 = int_value_stack.pop();
		break;
	case 216: label_216: 
	// using  symbolic fact wine(v_wine_1) with literal 2 in rule 
	// 615: tablewine(A) :- q46(A), wine(A).
	// resulting in symbolic fact tablewine(v_tablewine_1)

		//recursive
		if(!int_value_stack.push(v_tablewine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2242)) return -1;
		if (!q46_set.contains(v_wine_1)) continue;
	case 1229:
	// Continuation for label 216
			v_tablewine_1 = v_wine_1;
		if(tablewine_set.insert(v_tablewine_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1229)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(227)) return -1;
			if(!backtrack_stack.push(225)) return -1;
			if(!backtrack_stack.push(219)) return -1;
			if(!backtrack_stack.push(218)) return -1;
			goto label_217;
		}
		break;
	case 2242:
	// Label to restore values of label 216
		v_tablewine_1 = int_value_stack.pop();
		break;
	case 217: label_217: 
	// using  symbolic fact tablewine(v_tablewine_1) with literal 1 in rule 
	// 250: wine(A) :- tablewine(A).
	// resulting in symbolic fact wine(v_wine_1)

		//recursive
		if(!int_value_stack.push(v_wine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2243)) return -1;
		v_wine_1 = v_tablewine_1;
		if(wine_set.insert(v_wine_1)) {
		if(!wine_f.insert(v_wine_1)) return -1;
		// Labels for next rules
		if(!backtrack_stack.push(1007)) return -1;
		if(!backtrack_stack.push(989)) return -1;
		if(!backtrack_stack.push(954)) return -1;
		if(!backtrack_stack.push(938)) return -1;
		if(!backtrack_stack.push(911)) return -1;
		if(!backtrack_stack.push(903)) return -1;
		if(!backtrack_stack.push(902)) return -1;
		if(!backtrack_stack.push(901)) return -1;
		if(!backtrack_stack.push(849)) return -1;
		if(!backtrack_stack.push(848)) return -1;
		if(!backtrack_stack.push(821)) return -1;
		if(!backtrack_stack.push(820)) return -1;
		if(!backtrack_stack.push(737)) return -1;
		if(!backtrack_stack.push(712)) return -1;
		if(!backtrack_stack.push(674)) return -1;
		if(!backtrack_stack.push(672)) return -1;
		if(!backtrack_stack.push(583)) return -1;
		if(!backtrack_stack.push(575)) return -1;
		if(!backtrack_stack.push(563)) return -1;
		if(!backtrack_stack.push(551)) return -1;
		if(!backtrack_stack.push(549)) return -1;
		if(!backtrack_stack.push(483)) return -1;
		if(!backtrack_stack.push(473)) return -1;
		if(!backtrack_stack.push(470)) return -1;
		if(!backtrack_stack.push(430)) return -1;
		if(!backtrack_stack.push(401)) return -1;
		if(!backtrack_stack.push(399)) return -1;
		if(!backtrack_stack.push(380)) return -1;
		if(!backtrack_stack.push(364)) return -1;
		if(!backtrack_stack.push(363)) return -1;
		if(!backtrack_stack.push(329)) return -1;
		if(!backtrack_stack.push(328)) return -1;
		if(!backtrack_stack.push(327)) return -1;
		if(!backtrack_stack.push(295)) return -1;
		if(!backtrack_stack.push(294)) return -1;
		if(!backtrack_stack.push(216)) return -1;
		if(!backtrack_stack.push(210)) return -1;
		if(!backtrack_stack.push(192)) return -1;
		if(!backtrack_stack.push(180)) return -1;
		if(!backtrack_stack.push(179)) return -1;
		if(!backtrack_stack.push(178)) return -1;
		if(!backtrack_stack.push(154)) return -1;
		if(!backtrack_stack.push(153)) return -1;
		if(!backtrack_stack.push(100)) return -1;
		if(!backtrack_stack.push(76)) return -1;
		if(!backtrack_stack.push(75)) return -1;
		if(!backtrack_stack.push(67)) return -1;
		if(!backtrack_stack.push(63)) return -1;
		if(!backtrack_stack.push(53)) return -1;
		if(!backtrack_stack.push(46)) return -1;
		if(!backtrack_stack.push(19)) return -1;
		goto label_16;
		}
		break;
	case 2243:
	// Label to restore values of label 217
		v_wine_1 = int_value_stack.pop();
		break;
	case 218: label_218: 
	// using  symbolic fact tablewine(v_tablewine_1) with literal 1 in rule 
	// 617: tablewine(A) :- tablewine(B), kaon2equal(A,B).
	// resulting in symbolic fact tablewine(v_tablewine_1)

		//recursive
		c617_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_tablewine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2244)) return -1;
		c617_2_fb.open(v_tablewine_1);
	case 1231:
	// Continuation for label 218
		while (c617_2_fb.fetch()) {
			v_tablewine_1 = c617_2_fb.c2();
		if(tablewine_set.insert(v_tablewine_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1231)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(227)) return -1;
			if(!backtrack_stack.push(225)) return -1;
			if(!backtrack_stack.push(219)) return -1;
			if(!backtrack_stack.push(218)) return -1;
			goto label_217;
		}
		}
		break;
	case 2244:
	// Label to restore values of label 218
		v_tablewine_1 = int_value_stack.pop();
		c617_2_fb.pop(&cursor_stack);
		break;
	case 219: label_219: 
	// using  symbolic fact tablewine(v_tablewine_1) with literal 1 in rule 
	// 632: q31(A) :- tablewine(A).
	// resulting in symbolic fact q31(v_q31_1)

		//recursive
		if(!int_value_stack.push(v_q31_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2245)) return -1;
		v_q31_1 = v_tablewine_1;
		if(q31_set.insert(v_q31_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(221)) return -1;
		if(!backtrack_stack.push(220)) return -1;
		if(!backtrack_stack.push(215)) return -1;
		if(!backtrack_stack.push(213)) return -1;
		goto label_201;
		}
		break;
	case 2245:
	// Label to restore values of label 219
		v_q31_1 = int_value_stack.pop();
		break;
	case 220: label_220: 
	// using  symbolic fact q31(v_q31_1) with literal 1 in rule 
	// 640: q31(A) :- q31(B), kaon2equal(A,B).
	// resulting in symbolic fact q31(v_q31_1)

		//recursive
		c640_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q31_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2246)) return -1;
		c640_2_fb.open(v_q31_1);
	case 1233:
	// Continuation for label 220
		while (c640_2_fb.fetch()) {
			v_q31_1 = c640_2_fb.c2();
		if(q31_set.insert(v_q31_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1233)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(221)) return -1;
			if(!backtrack_stack.push(220)) return -1;
			if(!backtrack_stack.push(215)) return -1;
			if(!backtrack_stack.push(213)) return -1;
			goto label_201;
		}
		}
		break;
	case 2246:
	// Label to restore values of label 220
		v_q31_1 = int_value_stack.pop();
		c640_2_fb.pop(&cursor_stack);
		break;
	case 221: label_221: 
	// using  symbolic fact q31(v_q31_1) with literal 1 in rule 
	// 660: ot____nom12(A) :- q31(A), kaon2namedobjects(A).
	// resulting in symbolic fact ot____nom12(v_ot____nom12_1)

		//recursive
		if(!int_value_stack.push(v_ot____nom12_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2247)) return -1;
		if (!kaon2namedobjects_b.contains(v_q31_1)) continue;
	case 1234:
	// Continuation for label 221
			v_ot____nom12_1 = v_q31_1;
		if(ot____nom12_set.insert(v_ot____nom12_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1234)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(223)) return -1;
			goto label_222;
		}
		break;
	case 2247:
	// Label to restore values of label 221
		v_ot____nom12_1 = int_value_stack.pop();
		break;
	case 222: label_222: 
	// using  symbolic fact ot____nom12(v_ot____nom12_1) with literal 1 in rule 
	// 661: ot____nom12(A) :- ot____nom12(B), kaon2equal(A,B).
	// resulting in symbolic fact ot____nom12(v_ot____nom12_1)

		//recursive
		c661_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_ot____nom12_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2248)) return -1;
		c661_2_fb.open(v_ot____nom12_1);
	case 1235:
	// Continuation for label 222
		while (c661_2_fb.fetch()) {
			v_ot____nom12_1 = c661_2_fb.c2();
		if(ot____nom12_set.insert(v_ot____nom12_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1235)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(223)) return -1;
			goto label_222;
		}
		}
		break;
	case 2248:
	// Label to restore values of label 222
		v_ot____nom12_1 = int_value_stack.pop();
		c661_2_fb.pop(&cursor_stack);
		break;
	case 223: label_223: 
	// using  symbolic fact ot____nom12(v_ot____nom12_1) with literal 2 in rule 
	// 241: q46(A) :- hassugar(A,B), ot____nom12(B).
	// resulting in symbolic fact q46(v_q46_1)

		//recursive
		c241_1_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q46_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2249)) return -1;
		c241_1_fb.open(v_ot____nom12_1);
	case 1236:
	// Continuation for label 223
		while (c241_1_fb.fetch()) {
			v_q46_1 = c241_1_fb.c2();
		if(q46_set.insert(v_q46_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1236)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(233)) return -1;
			if(!backtrack_stack.push(224)) return -1;
			goto label_202;
		}
		}
		break;
	case 2249:
	// Label to restore values of label 223
		v_q46_1 = int_value_stack.pop();
		c241_1_fb.pop(&cursor_stack);
		break;
	case 224: label_224: 
	// using  symbolic fact q46(v_q46_1) with literal 1 in rule 
	// 615: tablewine(A) :- q46(A), wine(A).
	// resulting in symbolic fact tablewine(v_tablewine_1)

		//recursive
		if(!int_value_stack.push(v_tablewine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2250)) return -1;
		if (!wine_set.contains(v_q46_1)) continue;
	case 1237:
	// Continuation for label 224
			v_tablewine_1 = v_q46_1;
		if(tablewine_set.insert(v_tablewine_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1237)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(227)) return -1;
			if(!backtrack_stack.push(225)) return -1;
			if(!backtrack_stack.push(219)) return -1;
			if(!backtrack_stack.push(218)) return -1;
			goto label_217;
		}
		break;
	case 2250:
	// Label to restore values of label 224
		v_tablewine_1 = int_value_stack.pop();
		break;
	case 225: label_225: 
	// using  symbolic fact tablewine(v_tablewine_1) with literal 2 in rule 
	// 252: whitetablewine(A) :- q21(A), tablewine(A).
	// resulting in symbolic fact whitetablewine(v_whitetablewine_1)

		//recursive
		if(!int_value_stack.push(v_whitetablewine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2251)) return -1;
		if (!q21_set.contains(v_tablewine_1)) continue;
	case 1238:
	// Continuation for label 225
			v_whitetablewine_1 = v_tablewine_1;
		if(whitetablewine_set.insert(v_whitetablewine_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1238)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(232)) return -1;
			if(!backtrack_stack.push(231)) return -1;
			goto label_226;
		}
		break;
	case 2251:
	// Label to restore values of label 225
		v_whitetablewine_1 = int_value_stack.pop();
		break;
	case 226: label_226: 
	// using  symbolic fact whitetablewine(v_whitetablewine_1) with literal 1 in rule 
	// 251: tablewine(A) :- whitetablewine(A).
	// resulting in symbolic fact tablewine(v_tablewine_1)

		//recursive
		if(!int_value_stack.push(v_tablewine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2252)) return -1;
		v_tablewine_1 = v_whitetablewine_1;
		if(tablewine_set.insert(v_tablewine_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(227)) return -1;
		if(!backtrack_stack.push(225)) return -1;
		if(!backtrack_stack.push(219)) return -1;
		if(!backtrack_stack.push(218)) return -1;
		goto label_217;
		}
		break;
	case 2252:
	// Label to restore values of label 226
		v_tablewine_1 = int_value_stack.pop();
		break;
	case 227: label_227: 
	// using  symbolic fact tablewine(v_tablewine_1) with literal 2 in rule 
	// 445: redtablewine(A) :- q41(A), tablewine(A).
	// resulting in symbolic fact redtablewine(v_redtablewine_1)

		//recursive
		if(!int_value_stack.push(v_redtablewine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2253)) return -1;
		if (!q41_set.contains(v_tablewine_1)) continue;
	case 1240:
	// Continuation for label 227
			v_redtablewine_1 = v_tablewine_1;
		if(redtablewine_set.insert(v_redtablewine_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1240)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(230)) return -1;
			if(!backtrack_stack.push(229)) return -1;
			goto label_228;
		}
		break;
	case 2253:
	// Label to restore values of label 227
		v_redtablewine_1 = int_value_stack.pop();
		break;
	case 228: label_228: 
	// using  symbolic fact redtablewine(v_redtablewine_1) with literal 1 in rule 
	// 443: q0(A) :- redtablewine(A).
	// resulting in symbolic fact q0(v_q0_1)

		v_q0_1 = v_redtablewine_1;
		if(q0_set.insert(v_q0_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(198)) return -1;
		if(!backtrack_stack.push(194)) return -1;
		if(!backtrack_stack.push(189)) return -1;
		if(!backtrack_stack.push(188)) return -1;
		goto label_187;
		}
		break;
	case 229: label_229: 
	// using  symbolic fact redtablewine(v_redtablewine_1) with literal 1 in rule 
	// 613: tablewine(A) :- redtablewine(A).
	// resulting in symbolic fact tablewine(v_tablewine_1)

		//recursive
		if(!int_value_stack.push(v_tablewine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2255)) return -1;
		v_tablewine_1 = v_redtablewine_1;
		if(tablewine_set.insert(v_tablewine_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(227)) return -1;
		if(!backtrack_stack.push(225)) return -1;
		if(!backtrack_stack.push(219)) return -1;
		if(!backtrack_stack.push(218)) return -1;
		goto label_217;
		}
		break;
	case 2255:
	// Label to restore values of label 229
		v_tablewine_1 = int_value_stack.pop();
		break;
	case 230: label_230: 
	// using  symbolic fact redtablewine(v_redtablewine_1) with literal 1 in rule 
	// 614: redtablewine(A) :- redtablewine(B), kaon2equal(A,B).
	// resulting in symbolic fact redtablewine(v_redtablewine_1)

		//recursive
		c614_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_redtablewine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2256)) return -1;
		c614_2_fb.open(v_redtablewine_1);
	case 1243:
	// Continuation for label 230
		while (c614_2_fb.fetch()) {
			v_redtablewine_1 = c614_2_fb.c2();
		if(redtablewine_set.insert(v_redtablewine_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1243)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(230)) return -1;
			if(!backtrack_stack.push(229)) return -1;
			goto label_228;
		}
		}
		break;
	case 2256:
	// Label to restore values of label 230
		v_redtablewine_1 = int_value_stack.pop();
		c614_2_fb.pop(&cursor_stack);
		break;
	case 231: label_231: 
	// using  symbolic fact whitetablewine(v_whitetablewine_1) with literal 1 in rule 
	// 397: q12(A) :- whitetablewine(A).
	// resulting in symbolic fact q12(v_q12_1)

		v_q12_1 = v_whitetablewine_1;
		if(q12_set.insert(v_q12_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(116)) return -1;
		if(!backtrack_stack.push(115)) return -1;
		if(!backtrack_stack.push(114)) return -1;
		if(!backtrack_stack.push(99)) return -1;
		goto label_95;
		}
		break;
	case 232: label_232: 
	// using  symbolic fact whitetablewine(v_whitetablewine_1) with literal 1 in rule 
	// 398: whitetablewine(A) :- whitetablewine(B), kaon2equal(A,B).
	// resulting in symbolic fact whitetablewine(v_whitetablewine_1)

		//recursive
		c398_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_whitetablewine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2258)) return -1;
		c398_2_fb.open(v_whitetablewine_1);
	case 1245:
	// Continuation for label 232
		while (c398_2_fb.fetch()) {
			v_whitetablewine_1 = c398_2_fb.c2();
		if(whitetablewine_set.insert(v_whitetablewine_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1245)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(232)) return -1;
			if(!backtrack_stack.push(231)) return -1;
			goto label_226;
		}
		}
		break;
	case 2258:
	// Label to restore values of label 232
		v_whitetablewine_1 = int_value_stack.pop();
		c398_2_fb.pop(&cursor_stack);
		break;
	case 233: label_233: 
	// using  symbolic fact q46(v_q46_1) with literal 1 in rule 
	// 616: q46(A) :- q46(B), kaon2equal(A,B).
	// resulting in symbolic fact q46(v_q46_1)

		//recursive
		c616_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q46_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2259)) return -1;
		c616_2_fb.open(v_q46_1);
	case 1246:
	// Continuation for label 233
		while (c616_2_fb.fetch()) {
			v_q46_1 = c616_2_fb.c2();
		if(q46_set.insert(v_q46_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1246)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(233)) return -1;
			if(!backtrack_stack.push(224)) return -1;
			goto label_202;
		}
		}
		break;
	case 2259:
	// Label to restore values of label 233
		v_q46_1 = int_value_stack.pop();
		c616_2_fb.pop(&cursor_stack);
		break;
	case 234: label_234: 
	// using  symbolic fact drywhitewine(v_drywhitewine_1) with literal 1 in rule 
	// 297: whitewine(A) :- drywhitewine(A).
	// resulting in symbolic fact whitewine(v_whitewine_1)

		//recursive
		if(!int_value_stack.push(v_whitewine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2260)) return -1;
		v_whitewine_1 = v_drywhitewine_1;
		if(whitewine_set.insert(v_whitewine_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(560)) return -1;
		if(!backtrack_stack.push(491)) return -1;
		if(!backtrack_stack.push(374)) return -1;
		if(!backtrack_stack.push(235)) return -1;
		if(!backtrack_stack.push(113)) return -1;
		if(!backtrack_stack.push(112)) return -1;
		if(!backtrack_stack.push(104)) return -1;
		if(!backtrack_stack.push(103)) return -1;
		if(!backtrack_stack.push(101)) return -1;
		if(!backtrack_stack.push(98)) return -1;
		goto label_97;
		}
		break;
	case 2260:
	// Label to restore values of label 234
		v_whitewine_1 = int_value_stack.pop();
		break;
	case 235: label_235: 
	// using  symbolic fact whitewine(v_whitewine_1) with literal 2 in rule 
	// 237: drywhitewine(A) :- drywine(A), whitewine(A).
	// resulting in symbolic fact drywhitewine(v_drywhitewine_1)

		//recursive
		if(!int_value_stack.push(v_drywhitewine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2261)) return -1;
		if (!drywine_set.contains(v_whitewine_1)) continue;
	case 1248:
	// Continuation for label 235
			v_drywhitewine_1 = v_whitewine_1;
		if(drywhitewine_set.insert(v_drywhitewine_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1248)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(236)) return -1;
			if(!backtrack_stack.push(234)) return -1;
			goto label_208;
		}
		break;
	case 2261:
	// Label to restore values of label 235
		v_drywhitewine_1 = int_value_stack.pop();
		break;
	case 236: label_236: 
	// using  symbolic fact drywhitewine(v_drywhitewine_1) with literal 1 in rule 
	// 298: drywhitewine(A) :- drywhitewine(B), kaon2equal(A,B).
	// resulting in symbolic fact drywhitewine(v_drywhitewine_1)

		//recursive
		c298_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_drywhitewine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2262)) return -1;
		c298_2_fb.open(v_drywhitewine_1);
	case 1249:
	// Continuation for label 236
		while (c298_2_fb.fetch()) {
			v_drywhitewine_1 = c298_2_fb.c2();
		if(drywhitewine_set.insert(v_drywhitewine_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1249)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(236)) return -1;
			if(!backtrack_stack.push(234)) return -1;
			goto label_208;
		}
		}
		break;
	case 2262:
	// Label to restore values of label 236
		v_drywhitewine_1 = int_value_stack.pop();
		c298_2_fb.pop(&cursor_stack);
		break;
	case 237: label_237: 
	// using  symbolic fact dryredwine(v_dryredwine_1) with literal 1 in rule 
	// 239: dryredwine(A) :- dryredwine(B), kaon2equal(A,B).
	// resulting in symbolic fact dryredwine(v_dryredwine_1)

		//recursive
		c239_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_dryredwine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2263)) return -1;
		c239_2_fb.open(v_dryredwine_1);
	case 1250:
	// Continuation for label 237
		while (c239_2_fb.fetch()) {
			v_dryredwine_1 = c239_2_fb.c2();
		if(dryredwine_set.insert(v_dryredwine_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1250)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(237)) return -1;
			if(!backtrack_stack.push(206)) return -1;
			goto label_204;
		}
		}
		break;
	case 2263:
	// Label to restore values of label 237
		v_dryredwine_1 = int_value_stack.pop();
		c239_2_fb.pop(&cursor_stack);
		break;
	case 238: label_238: 
	// using  symbolic fact hassugar(v_hassugar_1,v_hassugar_2) with literal 1 in rule 
	// 241: q46(A) :- hassugar(A,B), ot____nom12(B).
	// resulting in symbolic fact q46(v_q46_1)

		if (!ot____nom12_set.contains(v_hassugar_2)) continue;
	case 1251:
	// Continuation for label 238
			v_q46_1 = v_hassugar_1;
		if(q46_set.insert(v_q46_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1251)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(233)) return -1;
			if(!backtrack_stack.push(224)) return -1;
			goto label_202;
		}
		break;
	case 239: label_239: 
	// using  symbolic fact beaujolais(v_beaujolais_1) with literal 1 in rule 
	// 778: q3(A) :- beaujolais(A).
	// resulting in symbolic fact q3(v_q3_1)

		v_q3_1 = v_beaujolais_1;
		if(q3_set.insert(v_q3_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(143)) return -1;
		goto label_123;
		}
		break;
	case 240: label_240: 
	// using  symbolic fact q45(v_q45_1) with literal 1 in rule 
	// 405: q45(A) :- q45(B), kaon2equal(A,B).
	// resulting in symbolic fact q45(v_q45_1)

		//recursive
		c405_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q45_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2266)) return -1;
		c405_2_fb.open(v_q45_1);
	case 1253:
	// Continuation for label 240
		while (c405_2_fb.fetch()) {
			v_q45_1 = c405_2_fb.c2();
		if(q45_set.insert(v_q45_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1253)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(240)) return -1;
			if(!backtrack_stack.push(177)) return -1;
			goto label_176;
		}
		}
		break;
	case 2266:
	// Label to restore values of label 240
		v_q45_1 = int_value_stack.pop();
		c405_2_fb.pop(&cursor_stack);
		break;
	case 241: label_241: 
	// using  symbolic fact q23(v_q23_1) with literal 1 in rule 
	// 715: q23(A) :- q23(B), kaon2equal(A,B).
	// resulting in symbolic fact q23(v_q23_1)

		//recursive
		c715_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q23_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2267)) return -1;
		c715_2_fb.open(v_q23_1);
	case 1254:
	// Continuation for label 241
		while (c715_2_fb.fetch()) {
			v_q23_1 = c715_2_fb.c2();
		if(q23_set.insert(v_q23_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1254)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(242)) return -1;
			if(!backtrack_stack.push(241)) return -1;
			goto label_173;
		}
		}
		break;
	case 2267:
	// Label to restore values of label 241
		v_q23_1 = int_value_stack.pop();
		c715_2_fb.pop(&cursor_stack);
		break;
	case 242: label_242: 
	// using  symbolic fact q23(v_q23_1) with literal 2 in rule 
	// 163: q22(A) :- locatedin(A,B), q23(B).
	// resulting in symbolic fact q22(v_q22_1)

		//recursive
		c163_1_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q22_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2268)) return -1;
		c163_1_fb.open(v_q23_1);
	case 1255:
	// Continuation for label 242
		while (c163_1_fb.fetch()) {
			v_q22_1 = c163_1_fb.c2();
		if(q22_set.insert(v_q22_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1255)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(248)) return -1;
			if(!backtrack_stack.push(245)) return -1;
			if(!backtrack_stack.push(243)) return -1;
			goto label_172;
		}
		}
		break;
	case 2268:
	// Label to restore values of label 242
		v_q22_1 = int_value_stack.pop();
		c163_1_fb.pop(&cursor_stack);
		break;
	case 243: label_243: 
	// using  symbolic fact q22(v_q22_1) with literal 1 in rule 
	// 162: locatedin(A,A) :- q22(A), kaon2namedobjects(A).
	// resulting in symbolic fact locatedin(v_locatedin_1,v_locatedin_2)

		//recursive
		if(!int_value_stack.push(v_locatedin_2)) return -1;
		if(!int_value_stack.push(v_locatedin_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2269)) return -1;
		if (!kaon2namedobjects_b.contains(v_q22_1)) continue;
	case 1256:
	// Continuation for label 243
			v_locatedin_1 = v_q22_1;
			v_locatedin_2 = v_q22_1;
		if(locatedin_set.insert(v_locatedin_1, v_locatedin_2)) {
			if(!locatedin_bf.insert(v_locatedin_1, v_locatedin_2)) return -1;
			if(!locatedin_fb.insert(v_locatedin_2, v_locatedin_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1256)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(1005)) return -1;
			if(!backtrack_stack.push(999)) return -1;
			if(!backtrack_stack.push(987)) return -1;
			if(!backtrack_stack.push(981)) return -1;
			if(!backtrack_stack.push(969)) return -1;
			if(!backtrack_stack.push(963)) return -1;
			if(!backtrack_stack.push(952)) return -1;
			if(!backtrack_stack.push(946)) return -1;
			if(!backtrack_stack.push(936)) return -1;
			if(!backtrack_stack.push(930)) return -1;
			if(!backtrack_stack.push(920)) return -1;
			if(!backtrack_stack.push(917)) return -1;
			if(!backtrack_stack.push(898)) return -1;
			if(!backtrack_stack.push(897)) return -1;
			if(!backtrack_stack.push(896)) return -1;
			if(!backtrack_stack.push(893)) return -1;
			if(!backtrack_stack.push(726)) return -1;
			if(!backtrack_stack.push(721)) return -1;
			if(!backtrack_stack.push(694)) return -1;
			if(!backtrack_stack.push(693)) return -1;
			if(!backtrack_stack.push(688)) return -1;
			if(!backtrack_stack.push(687)) return -1;
			if(!backtrack_stack.push(636)) return -1;
			if(!backtrack_stack.push(631)) return -1;
			if(!backtrack_stack.push(592)) return -1;
			if(!backtrack_stack.push(591)) return -1;
			if(!backtrack_stack.push(529)) return -1;
			if(!backtrack_stack.push(528)) return -1;
			if(!backtrack_stack.push(513)) return -1;
			if(!backtrack_stack.push(512)) return -1;
			if(!backtrack_stack.push(511)) return -1;
			if(!backtrack_stack.push(506)) return -1;
			if(!backtrack_stack.push(455)) return -1;
			if(!backtrack_stack.push(450)) return -1;
			if(!backtrack_stack.push(441)) return -1;
			if(!backtrack_stack.push(440)) return -1;
			if(!backtrack_stack.push(439)) return -1;
			if(!backtrack_stack.push(434)) return -1;
			if(!backtrack_stack.push(339)) return -1;
			if(!backtrack_stack.push(338)) return -1;
			if(!backtrack_stack.push(249)) return -1;
			if(!backtrack_stack.push(244)) return -1;
			if(!backtrack_stack.push(52)) return -1;
			if(!backtrack_stack.push(51)) return -1;
			if(!backtrack_stack.push(50)) return -1;
			if(!backtrack_stack.push(49)) return -1;
			if(!backtrack_stack.push(48)) return -1;
			goto label_47;
		}
		break;
	case 2269:
	// Label to restore values of label 243
		v_locatedin_1 = int_value_stack.pop();
		v_locatedin_2 = int_value_stack.pop();
		break;
	case 244: label_244: 
	// using  symbolic fact locatedin(v_locatedin_1,v_locatedin_2) with literal 1 in rule 
	// 163: q22(A) :- locatedin(A,B), q23(B).
	// resulting in symbolic fact q22(v_q22_1)

		//recursive
		if(!int_value_stack.push(v_q22_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2270)) return -1;
		if (!q23_set.contains(v_locatedin_2)) continue;
	case 1257:
	// Continuation for label 244
			v_q22_1 = v_locatedin_1;
		if(q22_set.insert(v_q22_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1257)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(248)) return -1;
			if(!backtrack_stack.push(245)) return -1;
			if(!backtrack_stack.push(243)) return -1;
			goto label_172;
		}
		break;
	case 2270:
	// Label to restore values of label 244
		v_q22_1 = int_value_stack.pop();
		break;
	case 245: label_245: 
	// using  symbolic fact q22(v_q22_1) with literal 1 in rule 
	// 712: ot____nom26(A) :- q22(A), kaon2namedobjects(A).
	// resulting in symbolic fact ot____nom26(v_ot____nom26_1)

		//recursive
		if(!int_value_stack.push(v_ot____nom26_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2271)) return -1;
		if (!kaon2namedobjects_b.contains(v_q22_1)) continue;
	case 1258:
	// Continuation for label 245
			v_ot____nom26_1 = v_q22_1;
		if(ot____nom26_set.insert(v_ot____nom26_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1258)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(247)) return -1;
			goto label_246;
		}
		break;
	case 2271:
	// Label to restore values of label 245
		v_ot____nom26_1 = int_value_stack.pop();
		break;
	case 246: label_246: 
	// using  symbolic fact ot____nom26(v_ot____nom26_1) with literal 1 in rule 
	// 714: ot____nom26(A) :- ot____nom26(B), kaon2equal(A,B).
	// resulting in symbolic fact ot____nom26(v_ot____nom26_1)

		//recursive
		c714_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_ot____nom26_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2272)) return -1;
		c714_2_fb.open(v_ot____nom26_1);
	case 1259:
	// Continuation for label 246
		while (c714_2_fb.fetch()) {
			v_ot____nom26_1 = c714_2_fb.c2();
		if(ot____nom26_set.insert(v_ot____nom26_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1259)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(247)) return -1;
			goto label_246;
		}
		}
		break;
	case 2272:
	// Label to restore values of label 246
		v_ot____nom26_1 = int_value_stack.pop();
		c714_2_fb.pop(&cursor_stack);
		break;
	case 247: label_247: 
	// using  symbolic fact ot____nom26(v_ot____nom26_1) with literal 2 in rule 
	// 710: q23(A) :- locatedin(A,B), ot____nom26(B).
	// resulting in symbolic fact q23(v_q23_1)

		//recursive
		c710_1_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q23_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2273)) return -1;
		c710_1_fb.open(v_ot____nom26_1);
	case 1260:
	// Continuation for label 247
		while (c710_1_fb.fetch()) {
			v_q23_1 = c710_1_fb.c2();
		if(q23_set.insert(v_q23_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1260)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(242)) return -1;
			if(!backtrack_stack.push(241)) return -1;
			goto label_173;
		}
		}
		break;
	case 2273:
	// Label to restore values of label 247
		v_q23_1 = int_value_stack.pop();
		c710_1_fb.pop(&cursor_stack);
		break;
	case 248: label_248: 
	// using  symbolic fact q22(v_q22_1) with literal 1 in rule 
	// 713: q22(A) :- q22(B), kaon2equal(A,B).
	// resulting in symbolic fact q22(v_q22_1)

		//recursive
		c713_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q22_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2274)) return -1;
		c713_2_fb.open(v_q22_1);
	case 1261:
	// Continuation for label 248
		while (c713_2_fb.fetch()) {
			v_q22_1 = c713_2_fb.c2();
		if(q22_set.insert(v_q22_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1261)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(248)) return -1;
			if(!backtrack_stack.push(245)) return -1;
			if(!backtrack_stack.push(243)) return -1;
			goto label_172;
		}
		}
		break;
	case 2274:
	// Label to restore values of label 248
		v_q22_1 = int_value_stack.pop();
		c713_2_fb.pop(&cursor_stack);
		break;
	case 249: label_249: 
	// using  symbolic fact locatedin(v_locatedin_1,v_locatedin_2) with literal 1 in rule 
	// 710: q23(A) :- locatedin(A,B), ot____nom26(B).
	// resulting in symbolic fact q23(v_q23_1)

		//recursive
		if(!int_value_stack.push(v_q23_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2275)) return -1;
		if (!ot____nom26_set.contains(v_locatedin_2)) continue;
	case 1262:
	// Continuation for label 249
			v_q23_1 = v_locatedin_1;
		if(q23_set.insert(v_q23_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1262)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(242)) return -1;
			if(!backtrack_stack.push(241)) return -1;
			goto label_173;
		}
		break;
	case 2275:
	// Label to restore values of label 249
		v_q23_1 = int_value_stack.pop();
		break;
	case 250:
	// applying rule 
	// 27: hassugar(A,B) :- hassugar_aux(A,B).
	// resulting in symbolic fact hassugar(v_hassugar_1,v_hassugar_2)
		c27_1_ff.open();
	case 1263:
	// Continuation for label 250
		while (c27_1_ff.fetch()) {
			v_hassugar_2 = c27_1_ff.c2();
			v_hassugar_1 = c27_1_ff.c1();
		if(hassugar_set.insert(v_hassugar_1, v_hassugar_2)) {
			if(!hassugar_bf.insert(v_hassugar_1, v_hassugar_2)) return -1;
			if(!hassugar_fb.insert(v_hassugar_2, v_hassugar_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1263)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(755)) return -1;
			if(!backtrack_stack.push(238)) return -1;
			if(!backtrack_stack.push(214)) return -1;
			if(!backtrack_stack.push(27)) return -1;
			if(!backtrack_stack.push(24)) return -1;
			if(!backtrack_stack.push(23)) return -1;
			if(!backtrack_stack.push(22)) return -1;
			goto label_17;
		}
		}
		break;
	case 251:
	// applying rule 
	// 35: hasvintageyear(A,B) :- hasvintageyear_aux(A,B).
	// resulting in symbolic fact hasvintageyear(v_hasvintageyear_1,v_hasvintageyear_2)
		c35_1_ff.open();
	case 1264:
	// Continuation for label 251
		while (c35_1_ff.fetch()) {
			v_hasvintageyear_2 = c35_1_ff.c2();
			v_hasvintageyear_1 = c35_1_ff.c1();
		if(hasvintageyear_set.insert(v_hasvintageyear_1, v_hasvintageyear_2)) {
			if(!hasvintageyear_bf.insert(v_hasvintageyear_1, v_hasvintageyear_2)) return -1;
			if(!hasvintageyear_fb.insert(v_hasvintageyear_2, v_hasvintageyear_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1264)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(259)) return -1;
			if(!backtrack_stack.push(258)) return -1;
			if(!backtrack_stack.push(257)) return -1;
			if(!backtrack_stack.push(255)) return -1;
			goto label_252;
		}
		}
		break;
	case 252: label_252: 
	// using  symbolic fact hasvintageyear(v_hasvintageyear_1,v_hasvintageyear_2) with literal 1 in rule 
	// 34: kaon2equal(A,B) :- hasvintageyear(C,A), hasvintageyear(C,B).
	// resulting in symbolic fact kaon2equal(v_kaon2equal_1,v_kaon2equal_2)

		//recursive
		c34_2_bf.push(&cursor_stack);
		if(!int_value_stack.push(v_kaon2equal_2)) return -1;
		if(!int_value_stack.push(v_kaon2equal_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2278)) return -1;
		c34_2_bf.open(v_hasvintageyear_1);
	case 1265:
	// Continuation for label 252
		while (c34_2_bf.fetch()) {
			v_kaon2equal_1 = v_hasvintageyear_2;
			v_kaon2equal_2 = c34_2_bf.c2();
		if(kaon2equal_set.insert(v_kaon2equal_1, v_kaon2equal_2)) {
			if(!kaon2equal_bf.insert(v_kaon2equal_1, v_kaon2equal_2)) return -1;
			if(!kaon2equal_fb.insert(v_kaon2equal_2, v_kaon2equal_1)) return -1;
			if(!kaon2equal_ff.insert(v_kaon2equal_1, v_kaon2equal_2)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1265)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(283)) return -1;
			if(!backtrack_stack.push(282)) return -1;
			if(!backtrack_stack.push(281)) return -1;
			if(!backtrack_stack.push(280)) return -1;
			if(!backtrack_stack.push(279)) return -1;
			if(!backtrack_stack.push(278)) return -1;
			if(!backtrack_stack.push(277)) return -1;
			if(!backtrack_stack.push(276)) return -1;
			if(!backtrack_stack.push(275)) return -1;
			if(!backtrack_stack.push(274)) return -1;
			if(!backtrack_stack.push(273)) return -1;
			if(!backtrack_stack.push(271)) return -1;
			if(!backtrack_stack.push(270)) return -1;
			if(!backtrack_stack.push(269)) return -1;
			if(!backtrack_stack.push(268)) return -1;
			if(!backtrack_stack.push(267)) return -1;
			if(!backtrack_stack.push(266)) return -1;
			if(!backtrack_stack.push(265)) return -1;
			if(!backtrack_stack.push(264)) return -1;
			if(!backtrack_stack.push(263)) return -1;
			if(!backtrack_stack.push(262)) return -1;
			if(!backtrack_stack.push(261)) return -1;
			if(!backtrack_stack.push(260)) return -1;
			if(!backtrack_stack.push(254)) return -1;
			if(!backtrack_stack.push(253)) return -1;
			if(!backtrack_stack.push(152)) return -1;
			if(!backtrack_stack.push(142)) return -1;
			if(!backtrack_stack.push(141)) return -1;
			if(!backtrack_stack.push(140)) return -1;
			if(!backtrack_stack.push(139)) return -1;
			if(!backtrack_stack.push(138)) return -1;
			if(!backtrack_stack.push(137)) return -1;
			if(!backtrack_stack.push(134)) return -1;
			if(!backtrack_stack.push(133)) return -1;
			if(!backtrack_stack.push(127)) return -1;
			if(!backtrack_stack.push(126)) return -1;
			if(!backtrack_stack.push(125)) return -1;
			if(!backtrack_stack.push(124)) return -1;
			if(!backtrack_stack.push(85)) return -1;
			if(!backtrack_stack.push(82)) return -1;
			if(!backtrack_stack.push(81)) return -1;
			if(!backtrack_stack.push(73)) return -1;
			if(!backtrack_stack.push(72)) return -1;
			if(!backtrack_stack.push(69)) return -1;
			if(!backtrack_stack.push(62)) return -1;
			if(!backtrack_stack.push(57)) return -1;
			if(!backtrack_stack.push(56)) return -1;
			if(!backtrack_stack.push(55)) return -1;
			if(!backtrack_stack.push(45)) return -1;
			if(!backtrack_stack.push(42)) return -1;
			if(!backtrack_stack.push(41)) return -1;
			if(!backtrack_stack.push(35)) return -1;
			if(!backtrack_stack.push(34)) return -1;
			if(!backtrack_stack.push(33)) return -1;
			if(!backtrack_stack.push(29)) return -1;
			if(!backtrack_stack.push(28)) return -1;
			if(!backtrack_stack.push(21)) return -1;
			if(!backtrack_stack.push(18)) return -1;
			if(!backtrack_stack.push(13)) return -1;
			if(!backtrack_stack.push(5)) return -1;
			if(!backtrack_stack.push(4)) return -1;
			goto label_3;
		}
		}
		break;
	case 2278:
	// Label to restore values of label 252
		v_kaon2equal_1 = int_value_stack.pop();
		v_kaon2equal_2 = int_value_stack.pop();
		c34_2_bf.pop(&cursor_stack);
		break;
	case 253: label_253: 
	// using  symbolic fact kaon2equal(v_kaon2equal_1,v_kaon2equal_2) with literal 2 in rule 
	// 25: beaujolais(A) :- beaujolais(B), kaon2equal(A,B).
	// resulting in symbolic fact beaujolais(v_beaujolais_1)

		if (!beaujolais_set.contains(v_kaon2equal_2)) continue;
	case 1266:
	// Continuation for label 253
			v_beaujolais_1 = v_kaon2equal_1;
		if(beaujolais_set.insert(v_beaujolais_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1266)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(239)) return -1;
			if(!backtrack_stack.push(200)) return -1;
			if(!backtrack_stack.push(186)) return -1;
			if(!backtrack_stack.push(185)) return -1;
			if(!backtrack_stack.push(181)) return -1;
			if(!backtrack_stack.push(175)) return -1;
			if(!backtrack_stack.push(174)) return -1;
			if(!backtrack_stack.push(171)) return -1;
			goto label_156;
		}
		break;
	case 254: label_254: 
	// using  symbolic fact kaon2equal(v_kaon2equal_1,v_kaon2equal_2) with literal 2 in rule 
	// 38: hasvintageyear(A,B) :- hasvintageyear(C,B), kaon2equal(A,C).
	// resulting in symbolic fact hasvintageyear(v_hasvintageyear_1,v_hasvintageyear_2)

		//recursive
		c38_1_bf.push(&cursor_stack);
		if(!int_value_stack.push(v_hasvintageyear_2)) return -1;
		if(!int_value_stack.push(v_hasvintageyear_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2280)) return -1;
		c38_1_bf.open(v_kaon2equal_2);
	case 1267:
	// Continuation for label 254
		while (c38_1_bf.fetch()) {
			v_hasvintageyear_1 = v_kaon2equal_1;
			v_hasvintageyear_2 = c38_1_bf.c2();
		if(hasvintageyear_set.insert(v_hasvintageyear_1, v_hasvintageyear_2)) {
			if(!hasvintageyear_bf.insert(v_hasvintageyear_1, v_hasvintageyear_2)) return -1;
			if(!hasvintageyear_fb.insert(v_hasvintageyear_2, v_hasvintageyear_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1267)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(259)) return -1;
			if(!backtrack_stack.push(258)) return -1;
			if(!backtrack_stack.push(257)) return -1;
			if(!backtrack_stack.push(255)) return -1;
			goto label_252;
		}
		}
		break;
	case 2280:
	// Label to restore values of label 254
		v_hasvintageyear_1 = int_value_stack.pop();
		v_hasvintageyear_2 = int_value_stack.pop();
		c38_1_bf.pop(&cursor_stack);
		break;
	case 255: label_255: 
	// using  symbolic fact hasvintageyear(v_hasvintageyear_1,v_hasvintageyear_2) with literal 1 in rule 
	// 37: vintage(A) :- hasvintageyear(A,B).
	// resulting in symbolic fact vintage(v_vintage_1)

		//recursive
		if(!int_value_stack.push(v_vintage_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2281)) return -1;
		v_vintage_1 = v_hasvintageyear_1;
		if(vintage_set.insert(v_vintage_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(284)) return -1;
		goto label_256;
		}
		break;
	case 2281:
	// Label to restore values of label 255
		v_vintage_1 = int_value_stack.pop();
		break;
	case 256: label_256: 
	// using  symbolic fact vintage(v_vintage_1) with literal 1 in rule 
	// 36: hasvintageyear(A,A) :- vintage(A), kaon2namedobjects(A).
	// resulting in symbolic fact hasvintageyear(v_hasvintageyear_1,v_hasvintageyear_2)

		//recursive
		if(!int_value_stack.push(v_hasvintageyear_2)) return -1;
		if(!int_value_stack.push(v_hasvintageyear_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2282)) return -1;
		if (!kaon2namedobjects_b.contains(v_vintage_1)) continue;
	case 1269:
	// Continuation for label 256
			v_hasvintageyear_1 = v_vintage_1;
			v_hasvintageyear_2 = v_vintage_1;
		if(hasvintageyear_set.insert(v_hasvintageyear_1, v_hasvintageyear_2)) {
			if(!hasvintageyear_bf.insert(v_hasvintageyear_1, v_hasvintageyear_2)) return -1;
			if(!hasvintageyear_fb.insert(v_hasvintageyear_2, v_hasvintageyear_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1269)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(259)) return -1;
			if(!backtrack_stack.push(258)) return -1;
			if(!backtrack_stack.push(257)) return -1;
			if(!backtrack_stack.push(255)) return -1;
			goto label_252;
		}
		break;
	case 2282:
	// Label to restore values of label 256
		v_hasvintageyear_1 = int_value_stack.pop();
		v_hasvintageyear_2 = int_value_stack.pop();
		break;
	case 257: label_257: 
	// using  symbolic fact hasvintageyear(v_hasvintageyear_1,v_hasvintageyear_2) with literal 1 in rule 
	// 38: hasvintageyear(A,B) :- hasvintageyear(C,B), kaon2equal(A,C).
	// resulting in symbolic fact hasvintageyear(v_hasvintageyear_1,v_hasvintageyear_2)

		//recursive
		c38_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_hasvintageyear_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2283)) return -1;
		c38_2_fb.open(v_hasvintageyear_1);
	case 1270:
	// Continuation for label 257
		while (c38_2_fb.fetch()) {
			v_hasvintageyear_1 = c38_2_fb.c2();
		if(hasvintageyear_set.insert(v_hasvintageyear_1, v_hasvintageyear_2)) {
			if(!hasvintageyear_bf.insert(v_hasvintageyear_1, v_hasvintageyear_2)) return -1;
			if(!hasvintageyear_fb.insert(v_hasvintageyear_2, v_hasvintageyear_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1270)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(259)) return -1;
			if(!backtrack_stack.push(258)) return -1;
			if(!backtrack_stack.push(257)) return -1;
			if(!backtrack_stack.push(255)) return -1;
			goto label_252;
		}
		}
		break;
	case 2283:
	// Label to restore values of label 257
		v_hasvintageyear_1 = int_value_stack.pop();
		c38_2_fb.pop(&cursor_stack);
		break;
	case 258: label_258: 
	// using  symbolic fact hasvintageyear(v_hasvintageyear_1,v_hasvintageyear_2) with literal 1 in rule 
	// 39: hasvintageyear(A,B) :- hasvintageyear(A,C), kaon2equal(B,C).
	// resulting in symbolic fact hasvintageyear(v_hasvintageyear_1,v_hasvintageyear_2)

		//recursive
		c39_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_hasvintageyear_2)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2284)) return -1;
		c39_2_fb.open(v_hasvintageyear_2);
	case 1271:
	// Continuation for label 258
		while (c39_2_fb.fetch()) {
			v_hasvintageyear_2 = c39_2_fb.c2();
		if(hasvintageyear_set.insert(v_hasvintageyear_1, v_hasvintageyear_2)) {
			if(!hasvintageyear_bf.insert(v_hasvintageyear_1, v_hasvintageyear_2)) return -1;
			if(!hasvintageyear_fb.insert(v_hasvintageyear_2, v_hasvintageyear_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1271)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(259)) return -1;
			if(!backtrack_stack.push(258)) return -1;
			if(!backtrack_stack.push(257)) return -1;
			if(!backtrack_stack.push(255)) return -1;
			goto label_252;
		}
		}
		break;
	case 2284:
	// Label to restore values of label 258
		v_hasvintageyear_2 = int_value_stack.pop();
		c39_2_fb.pop(&cursor_stack);
		break;
	case 259: label_259: 
	// using  symbolic fact hasvintageyear(v_hasvintageyear_1,v_hasvintageyear_2) with literal 2 in rule 
	// 34: kaon2equal(A,B) :- hasvintageyear(C,A), hasvintageyear(C,B).
	// resulting in symbolic fact kaon2equal(v_kaon2equal_1,v_kaon2equal_2)

		//recursive
		c34_1_bf.push(&cursor_stack);
		if(!int_value_stack.push(v_kaon2equal_1)) return -1;
		if(!int_value_stack.push(v_kaon2equal_2)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2285)) return -1;
		c34_1_bf.open(v_hasvintageyear_1);
	case 1272:
	// Continuation for label 259
		while (c34_1_bf.fetch()) {
			v_kaon2equal_2 = v_hasvintageyear_2;
			v_kaon2equal_1 = c34_1_bf.c2();
		if(kaon2equal_set.insert(v_kaon2equal_1, v_kaon2equal_2)) {
			if(!kaon2equal_bf.insert(v_kaon2equal_1, v_kaon2equal_2)) return -1;
			if(!kaon2equal_fb.insert(v_kaon2equal_2, v_kaon2equal_1)) return -1;
			if(!kaon2equal_ff.insert(v_kaon2equal_1, v_kaon2equal_2)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1272)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(283)) return -1;
			if(!backtrack_stack.push(282)) return -1;
			if(!backtrack_stack.push(281)) return -1;
			if(!backtrack_stack.push(280)) return -1;
			if(!backtrack_stack.push(279)) return -1;
			if(!backtrack_stack.push(278)) return -1;
			if(!backtrack_stack.push(277)) return -1;
			if(!backtrack_stack.push(276)) return -1;
			if(!backtrack_stack.push(275)) return -1;
			if(!backtrack_stack.push(274)) return -1;
			if(!backtrack_stack.push(273)) return -1;
			if(!backtrack_stack.push(271)) return -1;
			if(!backtrack_stack.push(270)) return -1;
			if(!backtrack_stack.push(269)) return -1;
			if(!backtrack_stack.push(268)) return -1;
			if(!backtrack_stack.push(267)) return -1;
			if(!backtrack_stack.push(266)) return -1;
			if(!backtrack_stack.push(265)) return -1;
			if(!backtrack_stack.push(264)) return -1;
			if(!backtrack_stack.push(263)) return -1;
			if(!backtrack_stack.push(262)) return -1;
			if(!backtrack_stack.push(261)) return -1;
			if(!backtrack_stack.push(260)) return -1;
			if(!backtrack_stack.push(254)) return -1;
			if(!backtrack_stack.push(253)) return -1;
			if(!backtrack_stack.push(152)) return -1;
			if(!backtrack_stack.push(142)) return -1;
			if(!backtrack_stack.push(141)) return -1;
			if(!backtrack_stack.push(140)) return -1;
			if(!backtrack_stack.push(139)) return -1;
			if(!backtrack_stack.push(138)) return -1;
			if(!backtrack_stack.push(137)) return -1;
			if(!backtrack_stack.push(134)) return -1;
			if(!backtrack_stack.push(133)) return -1;
			if(!backtrack_stack.push(127)) return -1;
			if(!backtrack_stack.push(126)) return -1;
			if(!backtrack_stack.push(125)) return -1;
			if(!backtrack_stack.push(124)) return -1;
			if(!backtrack_stack.push(85)) return -1;
			if(!backtrack_stack.push(82)) return -1;
			if(!backtrack_stack.push(81)) return -1;
			if(!backtrack_stack.push(73)) return -1;
			if(!backtrack_stack.push(72)) return -1;
			if(!backtrack_stack.push(69)) return -1;
			if(!backtrack_stack.push(62)) return -1;
			if(!backtrack_stack.push(57)) return -1;
			if(!backtrack_stack.push(56)) return -1;
			if(!backtrack_stack.push(55)) return -1;
			if(!backtrack_stack.push(45)) return -1;
			if(!backtrack_stack.push(42)) return -1;
			if(!backtrack_stack.push(41)) return -1;
			if(!backtrack_stack.push(35)) return -1;
			if(!backtrack_stack.push(34)) return -1;
			if(!backtrack_stack.push(33)) return -1;
			if(!backtrack_stack.push(29)) return -1;
			if(!backtrack_stack.push(28)) return -1;
			if(!backtrack_stack.push(21)) return -1;
			if(!backtrack_stack.push(18)) return -1;
			if(!backtrack_stack.push(13)) return -1;
			if(!backtrack_stack.push(5)) return -1;
			if(!backtrack_stack.push(4)) return -1;
			goto label_3;
		}
		}
		break;
	case 2285:
	// Label to restore values of label 259
		v_kaon2equal_2 = int_value_stack.pop();
		v_kaon2equal_1 = int_value_stack.pop();
		c34_1_bf.pop(&cursor_stack);
		break;
	case 260: label_260: 
	// using  symbolic fact kaon2equal(v_kaon2equal_1,v_kaon2equal_2) with literal 2 in rule 
	// 39: hasvintageyear(A,B) :- hasvintageyear(A,C), kaon2equal(B,C).
	// resulting in symbolic fact hasvintageyear(v_hasvintageyear_1,v_hasvintageyear_2)

		//recursive
		c39_1_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_hasvintageyear_1)) return -1;
		if(!int_value_stack.push(v_hasvintageyear_2)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2286)) return -1;
		c39_1_fb.open(v_kaon2equal_2);
	case 1273:
	// Continuation for label 260
		while (c39_1_fb.fetch()) {
			v_hasvintageyear_2 = v_kaon2equal_1;
			v_hasvintageyear_1 = c39_1_fb.c2();
		if(hasvintageyear_set.insert(v_hasvintageyear_1, v_hasvintageyear_2)) {
			if(!hasvintageyear_bf.insert(v_hasvintageyear_1, v_hasvintageyear_2)) return -1;
			if(!hasvintageyear_fb.insert(v_hasvintageyear_2, v_hasvintageyear_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1273)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(259)) return -1;
			if(!backtrack_stack.push(258)) return -1;
			if(!backtrack_stack.push(257)) return -1;
			if(!backtrack_stack.push(255)) return -1;
			goto label_252;
		}
		}
		break;
	case 2286:
	// Label to restore values of label 260
		v_hasvintageyear_2 = int_value_stack.pop();
		v_hasvintageyear_1 = int_value_stack.pop();
		c39_1_fb.pop(&cursor_stack);
		break;
	case 261: label_261: 
	// using  symbolic fact kaon2equal(v_kaon2equal_1,v_kaon2equal_2) with literal 2 in rule 
	// 239: dryredwine(A) :- dryredwine(B), kaon2equal(A,B).
	// resulting in symbolic fact dryredwine(v_dryredwine_1)

		if (!dryredwine_set.contains(v_kaon2equal_2)) continue;
	case 1274:
	// Continuation for label 261
			v_dryredwine_1 = v_kaon2equal_1;
		if(dryredwine_set.insert(v_dryredwine_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1274)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(237)) return -1;
			if(!backtrack_stack.push(206)) return -1;
			goto label_204;
		}
		break;
	case 262: label_262: 
	// using  symbolic fact kaon2equal(v_kaon2equal_1,v_kaon2equal_2) with literal 2 in rule 
	// 298: drywhitewine(A) :- drywhitewine(B), kaon2equal(A,B).
	// resulting in symbolic fact drywhitewine(v_drywhitewine_1)

		if (!drywhitewine_set.contains(v_kaon2equal_2)) continue;
	case 1275:
	// Continuation for label 262
			v_drywhitewine_1 = v_kaon2equal_1;
		if(drywhitewine_set.insert(v_drywhitewine_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1275)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(236)) return -1;
			if(!backtrack_stack.push(234)) return -1;
			goto label_208;
		}
		break;
	case 263: label_263: 
	// using  symbolic fact kaon2equal(v_kaon2equal_1,v_kaon2equal_2) with literal 2 in rule 
	// 398: whitetablewine(A) :- whitetablewine(B), kaon2equal(A,B).
	// resulting in symbolic fact whitetablewine(v_whitetablewine_1)

		if (!whitetablewine_set.contains(v_kaon2equal_2)) continue;
	case 1276:
	// Continuation for label 263
			v_whitetablewine_1 = v_kaon2equal_1;
		if(whitetablewine_set.insert(v_whitetablewine_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1276)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(232)) return -1;
			if(!backtrack_stack.push(231)) return -1;
			goto label_226;
		}
		break;
	case 264: label_264: 
	// using  symbolic fact kaon2equal(v_kaon2equal_1,v_kaon2equal_2) with literal 2 in rule 
	// 405: q45(A) :- q45(B), kaon2equal(A,B).
	// resulting in symbolic fact q45(v_q45_1)

		if (!q45_set.contains(v_kaon2equal_2)) continue;
	case 1277:
	// Continuation for label 264
			v_q45_1 = v_kaon2equal_1;
		if(q45_set.insert(v_q45_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1277)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(240)) return -1;
			if(!backtrack_stack.push(177)) return -1;
			goto label_176;
		}
		break;
	case 265: label_265: 
	// using  symbolic fact kaon2equal(v_kaon2equal_1,v_kaon2equal_2) with literal 2 in rule 
	// 470: drywine(A) :- drywine(B), kaon2equal(A,B).
	// resulting in symbolic fact drywine(v_drywine_1)

		if (!drywine_set.contains(v_kaon2equal_2)) continue;
	case 1278:
	// Continuation for label 265
			v_drywine_1 = v_kaon2equal_1;
		if(drywine_set.insert(v_drywine_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1278)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(212)) return -1;
			if(!backtrack_stack.push(211)) return -1;
			if(!backtrack_stack.push(209)) return -1;
			if(!backtrack_stack.push(207)) return -1;
			goto label_203;
		}
		break;
	case 266: label_266: 
	// using  symbolic fact kaon2equal(v_kaon2equal_1,v_kaon2equal_2) with literal 2 in rule 
	// 502: q14(A) :- q14(B), kaon2equal(A,B).
	// resulting in symbolic fact q14(v_q14_1)

		if (!q14_set.contains(v_kaon2equal_2)) continue;
	case 1279:
	// Continuation for label 266
			v_q14_1 = v_kaon2equal_1;
		if(q14_set.insert(v_q14_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1279)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(163)) return -1;
			if(!backtrack_stack.push(162)) return -1;
			if(!backtrack_stack.push(161)) return -1;
			if(!backtrack_stack.push(160)) return -1;
			if(!backtrack_stack.push(158)) return -1;
			goto label_157;
		}
		break;
	case 267: label_267: 
	// using  symbolic fact kaon2equal(v_kaon2equal_1,v_kaon2equal_2) with literal 2 in rule 
	// 561: ot____nom33(A) :- ot____nom33(B), kaon2equal(A,B).
	// resulting in symbolic fact ot____nom33(v_ot____nom33_1)

		if (!ot____nom33_set.contains(v_kaon2equal_2)) continue;
	case 1280:
	// Continuation for label 267
			v_ot____nom33_1 = v_kaon2equal_1;
		if(ot____nom33_set.insert(v_ot____nom33_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1280)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(166)) return -1;
			if(!backtrack_stack.push(165)) return -1;
			goto label_164;
		}
		break;
	case 268: label_268: 
	// using  symbolic fact kaon2equal(v_kaon2equal_1,v_kaon2equal_2) with literal 2 in rule 
	// 607: redwine(A) :- redwine(B), kaon2equal(A,B).
	// resulting in symbolic fact redwine(v_redwine_1)

		if (!redwine_set.contains(v_kaon2equal_2)) continue;
	case 1281:
	// Continuation for label 268
			v_redwine_1 = v_kaon2equal_1;
		if(redwine_set.insert(v_redwine_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1281)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(569)) return -1;
			if(!backtrack_stack.push(384)) return -1;
			if(!backtrack_stack.push(205)) return -1;
			if(!backtrack_stack.push(199)) return -1;
			if(!backtrack_stack.push(193)) return -1;
			goto label_191;
		}
		break;
	case 269: label_269: 
	// using  symbolic fact kaon2equal(v_kaon2equal_1,v_kaon2equal_2) with literal 2 in rule 
	// 610: q0(A) :- q0(B), kaon2equal(A,B).
	// resulting in symbolic fact q0(v_q0_1)

		if (!q0_set.contains(v_kaon2equal_2)) continue;
	case 1282:
	// Continuation for label 269
			v_q0_1 = v_kaon2equal_1;
		if(q0_set.insert(v_q0_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1282)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(198)) return -1;
			if(!backtrack_stack.push(194)) return -1;
			if(!backtrack_stack.push(189)) return -1;
			if(!backtrack_stack.push(188)) return -1;
			goto label_187;
		}
		break;
	case 270: label_270: 
	// using  symbolic fact kaon2equal(v_kaon2equal_1,v_kaon2equal_2) with literal 2 in rule 
	// 611: ot____nom28(A) :- ot____nom28(B), kaon2equal(A,B).
	// resulting in symbolic fact ot____nom28(v_ot____nom28_1)

		if (!ot____nom28_set.contains(v_kaon2equal_2)) continue;
	case 1283:
	// Continuation for label 270
			v_ot____nom28_1 = v_kaon2equal_1;
		if(ot____nom28_set.insert(v_ot____nom28_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1283)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(196)) return -1;
			goto label_195;
		}
		break;
	case 271: label_271: 
	// using  symbolic fact kaon2equal(v_kaon2equal_1,v_kaon2equal_2) with literal 2 in rule 
	// 612: q41(A) :- q41(B), kaon2equal(A,B).
	// resulting in symbolic fact q41(v_q41_1)

		if (!q41_set.contains(v_kaon2equal_2)) continue;
	case 1284:
	// Continuation for label 271
			v_q41_1 = v_kaon2equal_1;
		if(q41_set.insert(v_q41_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1284)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(272)) return -1;
			if(!backtrack_stack.push(197)) return -1;
			goto label_190;
		}
		break;
	case 272: label_272: 
	// using  symbolic fact q41(v_q41_1) with literal 1 in rule 
	// 445: redtablewine(A) :- q41(A), tablewine(A).
	// resulting in symbolic fact redtablewine(v_redtablewine_1)

		if (!tablewine_set.contains(v_q41_1)) continue;
	case 1285:
	// Continuation for label 272
			v_redtablewine_1 = v_q41_1;
		if(redtablewine_set.insert(v_redtablewine_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1285)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(230)) return -1;
			if(!backtrack_stack.push(229)) return -1;
			goto label_228;
		}
		break;
	case 273: label_273: 
	// using  symbolic fact kaon2equal(v_kaon2equal_1,v_kaon2equal_2) with literal 2 in rule 
	// 614: redtablewine(A) :- redtablewine(B), kaon2equal(A,B).
	// resulting in symbolic fact redtablewine(v_redtablewine_1)

		if (!redtablewine_set.contains(v_kaon2equal_2)) continue;
	case 1286:
	// Continuation for label 273
			v_redtablewine_1 = v_kaon2equal_1;
		if(redtablewine_set.insert(v_redtablewine_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1286)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(230)) return -1;
			if(!backtrack_stack.push(229)) return -1;
			goto label_228;
		}
		break;
	case 274: label_274: 
	// using  symbolic fact kaon2equal(v_kaon2equal_1,v_kaon2equal_2) with literal 2 in rule 
	// 616: q46(A) :- q46(B), kaon2equal(A,B).
	// resulting in symbolic fact q46(v_q46_1)

		if (!q46_set.contains(v_kaon2equal_2)) continue;
	case 1287:
	// Continuation for label 274
			v_q46_1 = v_kaon2equal_1;
		if(q46_set.insert(v_q46_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1287)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(233)) return -1;
			if(!backtrack_stack.push(224)) return -1;
			goto label_202;
		}
		break;
	case 275: label_275: 
	// using  symbolic fact kaon2equal(v_kaon2equal_1,v_kaon2equal_2) with literal 2 in rule 
	// 617: tablewine(A) :- tablewine(B), kaon2equal(A,B).
	// resulting in symbolic fact tablewine(v_tablewine_1)

		if (!tablewine_set.contains(v_kaon2equal_2)) continue;
	case 1288:
	// Continuation for label 275
			v_tablewine_1 = v_kaon2equal_1;
		if(tablewine_set.insert(v_tablewine_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1288)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(227)) return -1;
			if(!backtrack_stack.push(225)) return -1;
			if(!backtrack_stack.push(219)) return -1;
			if(!backtrack_stack.push(218)) return -1;
			goto label_217;
		}
		break;
	case 276: label_276: 
	// using  symbolic fact kaon2equal(v_kaon2equal_1,v_kaon2equal_2) with literal 2 in rule 
	// 640: q31(A) :- q31(B), kaon2equal(A,B).
	// resulting in symbolic fact q31(v_q31_1)

		if (!q31_set.contains(v_kaon2equal_2)) continue;
	case 1289:
	// Continuation for label 276
			v_q31_1 = v_kaon2equal_1;
		if(q31_set.insert(v_q31_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1289)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(221)) return -1;
			if(!backtrack_stack.push(220)) return -1;
			if(!backtrack_stack.push(215)) return -1;
			if(!backtrack_stack.push(213)) return -1;
			goto label_201;
		}
		break;
	case 277: label_277: 
	// using  symbolic fact kaon2equal(v_kaon2equal_1,v_kaon2equal_2) with literal 2 in rule 
	// 646: q20(A) :- q20(B), kaon2equal(A,B).
	// resulting in symbolic fact q20(v_q20_1)

		if (!q20_set.contains(v_kaon2equal_2)) continue;
	case 1290:
	// Continuation for label 277
			v_q20_1 = v_kaon2equal_1;
		if(q20_set.insert(v_q20_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1290)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(184)) return -1;
			if(!backtrack_stack.push(183)) return -1;
			goto label_182;
		}
		break;
	case 278: label_278: 
	// using  symbolic fact kaon2equal(v_kaon2equal_1,v_kaon2equal_2) with literal 2 in rule 
	// 661: ot____nom12(A) :- ot____nom12(B), kaon2equal(A,B).
	// resulting in symbolic fact ot____nom12(v_ot____nom12_1)

		if (!ot____nom12_set.contains(v_kaon2equal_2)) continue;
	case 1291:
	// Continuation for label 278
			v_ot____nom12_1 = v_kaon2equal_1;
		if(ot____nom12_set.insert(v_ot____nom12_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1291)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(223)) return -1;
			goto label_222;
		}
		break;
	case 279: label_279: 
	// using  symbolic fact kaon2equal(v_kaon2equal_1,v_kaon2equal_2) with literal 2 in rule 
	// 713: q22(A) :- q22(B), kaon2equal(A,B).
	// resulting in symbolic fact q22(v_q22_1)

		if (!q22_set.contains(v_kaon2equal_2)) continue;
	case 1292:
	// Continuation for label 279
			v_q22_1 = v_kaon2equal_1;
		if(q22_set.insert(v_q22_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1292)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(248)) return -1;
			if(!backtrack_stack.push(245)) return -1;
			if(!backtrack_stack.push(243)) return -1;
			goto label_172;
		}
		break;
	case 280: label_280: 
	// using  symbolic fact kaon2equal(v_kaon2equal_1,v_kaon2equal_2) with literal 2 in rule 
	// 714: ot____nom26(A) :- ot____nom26(B), kaon2equal(A,B).
	// resulting in symbolic fact ot____nom26(v_ot____nom26_1)

		if (!ot____nom26_set.contains(v_kaon2equal_2)) continue;
	case 1293:
	// Continuation for label 280
			v_ot____nom26_1 = v_kaon2equal_1;
		if(ot____nom26_set.insert(v_ot____nom26_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1293)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(247)) return -1;
			goto label_246;
		}
		break;
	case 281: label_281: 
	// using  symbolic fact kaon2equal(v_kaon2equal_1,v_kaon2equal_2) with literal 2 in rule 
	// 715: q23(A) :- q23(B), kaon2equal(A,B).
	// resulting in symbolic fact q23(v_q23_1)

		if (!q23_set.contains(v_kaon2equal_2)) continue;
	case 1294:
	// Continuation for label 281
			v_q23_1 = v_kaon2equal_1;
		if(q23_set.insert(v_q23_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1294)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(242)) return -1;
			if(!backtrack_stack.push(241)) return -1;
			goto label_173;
		}
		break;
	case 282: label_282: 
	// using  symbolic fact kaon2equal(v_kaon2equal_1,v_kaon2equal_2) with literal 2 in rule 
	// 787: gamay(A) :- gamay(B), kaon2equal(A,B).
	// resulting in symbolic fact gamay(v_gamay_1)

		if (!gamay_set.contains(v_kaon2equal_2)) continue;
	case 1295:
	// Continuation for label 282
			v_gamay_1 = v_kaon2equal_1;
		if(gamay_set.insert(v_gamay_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1295)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(168)) return -1;
			if(!backtrack_stack.push(167)) return -1;
			goto label_159;
		}
		break;
	case 283: label_283: 
	// using  symbolic fact kaon2equal(v_kaon2equal_1,v_kaon2equal_2) with literal 2 in rule 
	// 798: vintage(A) :- vintage(B), kaon2equal(A,B).
	// resulting in symbolic fact vintage(v_vintage_1)

		//recursive
		if(!int_value_stack.push(v_vintage_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2309)) return -1;
		if (!vintage_set.contains(v_kaon2equal_2)) continue;
	case 1296:
	// Continuation for label 283
			v_vintage_1 = v_kaon2equal_1;
		if(vintage_set.insert(v_vintage_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1296)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(284)) return -1;
			goto label_256;
		}
		break;
	case 2309:
	// Label to restore values of label 283
		v_vintage_1 = int_value_stack.pop();
		break;
	case 284: label_284: 
	// using  symbolic fact vintage(v_vintage_1) with literal 1 in rule 
	// 798: vintage(A) :- vintage(B), kaon2equal(A,B).
	// resulting in symbolic fact vintage(v_vintage_1)

		//recursive
		c798_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_vintage_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2310)) return -1;
		c798_2_fb.open(v_vintage_1);
	case 1297:
	// Continuation for label 284
		while (c798_2_fb.fetch()) {
			v_vintage_1 = c798_2_fb.c2();
		if(vintage_set.insert(v_vintage_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1297)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(284)) return -1;
			goto label_256;
		}
		}
		break;
	case 2310:
	// Label to restore values of label 284
		v_vintage_1 = int_value_stack.pop();
		c798_2_fb.pop(&cursor_stack);
		break;
	case 285:
	// applying rule 
	// 41: hascolor(A,B) :- hascolor_aux(A,B).
	// resulting in symbolic fact hascolor(v_hascolor_1,v_hascolor_2)
		c41_1_ff.open();
	case 1298:
	// Continuation for label 285
		while (c41_1_ff.fetch()) {
			v_hascolor_2 = c41_1_ff.c2();
			v_hascolor_1 = c41_1_ff.c1();
		if(hascolor_set.insert(v_hascolor_1, v_hascolor_2)) {
			if(!hascolor_bf.insert(v_hascolor_1, v_hascolor_2)) return -1;
			if(!hascolor_fb.insert(v_hascolor_2, v_hascolor_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1298)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(286)) return -1;
			if(!backtrack_stack.push(120)) return -1;
			if(!backtrack_stack.push(80)) return -1;
			if(!backtrack_stack.push(79)) return -1;
			if(!backtrack_stack.push(78)) return -1;
			if(!backtrack_stack.push(77)) return -1;
			if(!backtrack_stack.push(74)) return -1;
			goto label_68;
		}
		}
		break;
	case 286: label_286: 
	// using  symbolic fact hascolor(v_hascolor_1,v_hascolor_2) with literal 1 in rule 
	// 446: q41(A) :- hascolor(A,B), ot____nom28(B).
	// resulting in symbolic fact q41(v_q41_1)

		if (!ot____nom28_set.contains(v_hascolor_2)) continue;
	case 1299:
	// Continuation for label 286
			v_q41_1 = v_hascolor_1;
		if(q41_set.insert(v_q41_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1299)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(272)) return -1;
			if(!backtrack_stack.push(197)) return -1;
			goto label_190;
		}
		break;
	case 287:
	// applying rule 
	// 44: meritage(A) :- meritage_aux(A).
	// resulting in symbolic fact meritage(v_meritage_1)
		c44_1_f.open();
	case 1300:
	// Continuation for label 287
		while (c44_1_f.fetch()) {
			v_meritage_1 = c44_1_f.c1();
		if(meritage_set.insert(v_meritage_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1300)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(293)) return -1;
			if(!backtrack_stack.push(292)) return -1;
			if(!backtrack_stack.push(290)) return -1;
			if(!backtrack_stack.push(289)) return -1;
			goto label_288;
		}
		}
		break;
	case 288: label_288: 
	// using  symbolic fact meritage(v_meritage_1) with literal 1 in rule 
	// 43: q0(A) :- meritage(A).
	// resulting in symbolic fact q0(v_q0_1)

		v_q0_1 = v_meritage_1;
		if(q0_set.insert(v_q0_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(198)) return -1;
		if(!backtrack_stack.push(194)) return -1;
		if(!backtrack_stack.push(189)) return -1;
		if(!backtrack_stack.push(188)) return -1;
		goto label_187;
		}
		break;
	case 289: label_289: 
	// using  symbolic fact meritage(v_meritage_1) with literal 1 in rule 
	// 45: meritage(A) :- meritage(B), kaon2equal(A,B).
	// resulting in symbolic fact meritage(v_meritage_1)

		//recursive
		c45_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_meritage_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2315)) return -1;
		c45_2_fb.open(v_meritage_1);
	case 1302:
	// Continuation for label 289
		while (c45_2_fb.fetch()) {
			v_meritage_1 = c45_2_fb.c2();
		if(meritage_set.insert(v_meritage_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1302)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(293)) return -1;
			if(!backtrack_stack.push(292)) return -1;
			if(!backtrack_stack.push(290)) return -1;
			if(!backtrack_stack.push(289)) return -1;
			goto label_288;
		}
		}
		break;
	case 2315:
	// Label to restore values of label 289
		v_meritage_1 = int_value_stack.pop();
		c45_2_fb.pop(&cursor_stack);
		break;
	case 290: label_290: 
	// using  symbolic fact meritage(v_meritage_1) with literal 1 in rule 
	// 48: ot____nom8(A) :- meritage(B), madefromgrape(B,A).
	// resulting in symbolic fact ot____nom8(v_ot____nom8_1)

		//recursive
		c48_2_bf.push(&cursor_stack);
		if(!int_value_stack.push(v_ot____nom8_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2316)) return -1;
		c48_2_bf.open(v_meritage_1);
	case 1303:
	// Continuation for label 290
		while (c48_2_bf.fetch()) {
			v_ot____nom8_1 = c48_2_bf.c2();
		if(ot____nom8_set.insert(v_ot____nom8_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1303)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(297)) return -1;
			if(!backtrack_stack.push(296)) return -1;
			goto label_291;
		}
		}
		break;
	case 2316:
	// Label to restore values of label 290
		v_ot____nom8_1 = int_value_stack.pop();
		c48_2_bf.pop(&cursor_stack);
		break;
	case 291: label_291: 
	// using  symbolic fact ot____nom8(v_ot____nom8_1) with literal 1 in rule 
	// 46: meritage(A) :- ot____nom8(A), wine(A), kaon2namedobjects(A).
	// resulting in symbolic fact meritage(v_meritage_1)

		//recursive
		if(!int_value_stack.push(v_meritage_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2317)) return -1;
		if (!wine_set.contains(v_ot____nom8_1)) continue;
			if (!kaon2namedobjects_b.contains(v_ot____nom8_1)) continue;
	case 1304:
	// Continuation for label 291
				v_meritage_1 = v_ot____nom8_1;
		if(meritage_set.insert(v_meritage_1)) {
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1304)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(293)) return -1;
				if(!backtrack_stack.push(292)) return -1;
				if(!backtrack_stack.push(290)) return -1;
				if(!backtrack_stack.push(289)) return -1;
				goto label_288;
		}
		break;
	case 2317:
	// Label to restore values of label 291
		v_meritage_1 = int_value_stack.pop();
		break;
	case 292: label_292: 
	// using  symbolic fact meritage(v_meritage_1) with literal 1 in rule 
	// 357: q5(A) :- meritage(A).
	// resulting in symbolic fact q5(v_q5_1)

		v_q5_1 = v_meritage_1;
		if(q5_set.insert(v_q5_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(136)) return -1;
		if(!backtrack_stack.push(135)) return -1;
		goto label_131;
		}
		break;
	case 293: label_293: 
	// using  symbolic fact meritage(v_meritage_1) with literal 1 in rule 
	// 417: wine(A) :- meritage(A).
	// resulting in symbolic fact wine(v_wine_1)

		//recursive
		if(!int_value_stack.push(v_wine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2319)) return -1;
		v_wine_1 = v_meritage_1;
		if(wine_set.insert(v_wine_1)) {
		if(!wine_f.insert(v_wine_1)) return -1;
		// Labels for next rules
		if(!backtrack_stack.push(1007)) return -1;
		if(!backtrack_stack.push(989)) return -1;
		if(!backtrack_stack.push(954)) return -1;
		if(!backtrack_stack.push(938)) return -1;
		if(!backtrack_stack.push(911)) return -1;
		if(!backtrack_stack.push(903)) return -1;
		if(!backtrack_stack.push(902)) return -1;
		if(!backtrack_stack.push(901)) return -1;
		if(!backtrack_stack.push(849)) return -1;
		if(!backtrack_stack.push(848)) return -1;
		if(!backtrack_stack.push(821)) return -1;
		if(!backtrack_stack.push(820)) return -1;
		if(!backtrack_stack.push(737)) return -1;
		if(!backtrack_stack.push(712)) return -1;
		if(!backtrack_stack.push(674)) return -1;
		if(!backtrack_stack.push(672)) return -1;
		if(!backtrack_stack.push(583)) return -1;
		if(!backtrack_stack.push(575)) return -1;
		if(!backtrack_stack.push(563)) return -1;
		if(!backtrack_stack.push(551)) return -1;
		if(!backtrack_stack.push(549)) return -1;
		if(!backtrack_stack.push(483)) return -1;
		if(!backtrack_stack.push(473)) return -1;
		if(!backtrack_stack.push(470)) return -1;
		if(!backtrack_stack.push(430)) return -1;
		if(!backtrack_stack.push(401)) return -1;
		if(!backtrack_stack.push(399)) return -1;
		if(!backtrack_stack.push(380)) return -1;
		if(!backtrack_stack.push(364)) return -1;
		if(!backtrack_stack.push(363)) return -1;
		if(!backtrack_stack.push(329)) return -1;
		if(!backtrack_stack.push(328)) return -1;
		if(!backtrack_stack.push(327)) return -1;
		if(!backtrack_stack.push(295)) return -1;
		if(!backtrack_stack.push(294)) return -1;
		if(!backtrack_stack.push(216)) return -1;
		if(!backtrack_stack.push(210)) return -1;
		if(!backtrack_stack.push(192)) return -1;
		if(!backtrack_stack.push(180)) return -1;
		if(!backtrack_stack.push(179)) return -1;
		if(!backtrack_stack.push(178)) return -1;
		if(!backtrack_stack.push(154)) return -1;
		if(!backtrack_stack.push(153)) return -1;
		if(!backtrack_stack.push(100)) return -1;
		if(!backtrack_stack.push(76)) return -1;
		if(!backtrack_stack.push(75)) return -1;
		if(!backtrack_stack.push(67)) return -1;
		if(!backtrack_stack.push(63)) return -1;
		if(!backtrack_stack.push(53)) return -1;
		if(!backtrack_stack.push(46)) return -1;
		if(!backtrack_stack.push(19)) return -1;
		goto label_16;
		}
		break;
	case 2319:
	// Label to restore values of label 293
		v_wine_1 = int_value_stack.pop();
		break;
	case 294: label_294: 
	// using  symbolic fact wine(v_wine_1) with literal 2 in rule 
	// 46: meritage(A) :- ot____nom8(A), wine(A), kaon2namedobjects(A).
	// resulting in symbolic fact meritage(v_meritage_1)

		//recursive
		if(!int_value_stack.push(v_meritage_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2320)) return -1;
		if (!ot____nom8_set.contains(v_wine_1)) continue;
			if (!kaon2namedobjects_b.contains(v_wine_1)) continue;
	case 1307:
	// Continuation for label 294
				v_meritage_1 = v_wine_1;
		if(meritage_set.insert(v_meritage_1)) {
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1307)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(293)) return -1;
				if(!backtrack_stack.push(292)) return -1;
				if(!backtrack_stack.push(290)) return -1;
				if(!backtrack_stack.push(289)) return -1;
				goto label_288;
		}
		break;
	case 2320:
	// Label to restore values of label 294
		v_meritage_1 = int_value_stack.pop();
		break;
	case 295: label_295: 
	// using  symbolic fact wine(v_wine_1) with literal 2 in rule 
	// 790: q3(A) :- ot____nom8(A), wine(A), kaon2namedobjects(A).
	// resulting in symbolic fact q3(v_q3_1)

		if (!ot____nom8_set.contains(v_wine_1)) continue;
			if (!kaon2namedobjects_b.contains(v_wine_1)) continue;
	case 1308:
	// Continuation for label 295
				v_q3_1 = v_wine_1;
		if(q3_set.insert(v_q3_1)) {
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1308)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(143)) return -1;
				goto label_123;
		}
		break;
	case 296: label_296: 
	// using  symbolic fact ot____nom8(v_ot____nom8_1) with literal 1 in rule 
	// 790: q3(A) :- ot____nom8(A), wine(A), kaon2namedobjects(A).
	// resulting in symbolic fact q3(v_q3_1)

		if (!wine_set.contains(v_ot____nom8_1)) continue;
			if (!kaon2namedobjects_b.contains(v_ot____nom8_1)) continue;
	case 1309:
	// Continuation for label 296
				v_q3_1 = v_ot____nom8_1;
		if(q3_set.insert(v_q3_1)) {
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1309)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(143)) return -1;
				goto label_123;
		}
		break;
	case 297: label_297: 
	// using  symbolic fact ot____nom8(v_ot____nom8_1) with literal 1 in rule 
	// 791: ot____nom8(A) :- ot____nom8(B), kaon2equal(A,B).
	// resulting in symbolic fact ot____nom8(v_ot____nom8_1)

		//recursive
		c791_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_ot____nom8_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2323)) return -1;
		c791_2_fb.open(v_ot____nom8_1);
	case 1310:
	// Continuation for label 297
		while (c791_2_fb.fetch()) {
			v_ot____nom8_1 = c791_2_fb.c2();
		if(ot____nom8_set.insert(v_ot____nom8_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1310)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(297)) return -1;
			if(!backtrack_stack.push(296)) return -1;
			goto label_291;
		}
		}
		break;
	case 2323:
	// Label to restore values of label 297
		v_ot____nom8_1 = int_value_stack.pop();
		c791_2_fb.pop(&cursor_stack);
		break;
	case 298:
	// applying rule 
	// 47: ot____nom8(A) :- ot____nom8_aux(A).
	// resulting in symbolic fact ot____nom8(v_ot____nom8_1)
		c47_1_f.open();
	case 1311:
	// Continuation for label 298
		while (c47_1_f.fetch()) {
			v_ot____nom8_1 = c47_1_f.c1();
		if(ot____nom8_set.insert(v_ot____nom8_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1311)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(297)) return -1;
			if(!backtrack_stack.push(296)) return -1;
			goto label_291;
		}
		}
		break;
	case 299:
	// applying rule 
	// 49: madefromgrape(A,B) :- madefromgrape_aux(A,B).
	// resulting in symbolic fact madefromgrape(v_madefromgrape_1,v_madefromgrape_2)
		c49_1_ff.open();
	case 1312:
	// Continuation for label 299
		while (c49_1_ff.fetch()) {
			v_madefromgrape_2 = c49_1_ff.c2();
			v_madefromgrape_1 = c49_1_ff.c1();
		if(madefromgrape_set.insert(v_madefromgrape_1, v_madefromgrape_2)) {
			if(!madefromgrape_bf.insert(v_madefromgrape_1, v_madefromgrape_2)) return -1;
			if(!madefromgrape_fb.insert(v_madefromgrape_2, v_madefromgrape_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1312)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(860)) return -1;
			if(!backtrack_stack.push(859)) return -1;
			if(!backtrack_stack.push(845)) return -1;
			if(!backtrack_stack.push(844)) return -1;
			if(!backtrack_stack.push(829)) return -1;
			if(!backtrack_stack.push(828)) return -1;
			if(!backtrack_stack.push(813)) return -1;
			if(!backtrack_stack.push(812)) return -1;
			if(!backtrack_stack.push(677)) return -1;
			if(!backtrack_stack.push(676)) return -1;
			if(!backtrack_stack.push(617)) return -1;
			if(!backtrack_stack.push(616)) return -1;
			if(!backtrack_stack.push(609)) return -1;
			if(!backtrack_stack.push(554)) return -1;
			if(!backtrack_stack.push(553)) return -1;
			if(!backtrack_stack.push(476)) return -1;
			if(!backtrack_stack.push(475)) return -1;
			if(!backtrack_stack.push(404)) return -1;
			if(!backtrack_stack.push(403)) return -1;
			if(!backtrack_stack.push(362)) return -1;
			if(!backtrack_stack.push(361)) return -1;
			if(!backtrack_stack.push(343)) return -1;
			if(!backtrack_stack.push(342)) return -1;
			if(!backtrack_stack.push(300)) return -1;
			if(!backtrack_stack.push(170)) return -1;
			if(!backtrack_stack.push(169)) return -1;
			if(!backtrack_stack.push(151)) return -1;
			if(!backtrack_stack.push(150)) return -1;
			if(!backtrack_stack.push(144)) return -1;
			if(!backtrack_stack.push(132)) return -1;
			if(!backtrack_stack.push(71)) return -1;
			if(!backtrack_stack.push(70)) return -1;
			if(!backtrack_stack.push(66)) return -1;
			goto label_64;
		}
		}
		break;
	case 300: label_300: 
	// using  symbolic fact madefromgrape(v_madefromgrape_1,v_madefromgrape_2) with literal 2 in rule 
	// 48: ot____nom8(A) :- meritage(B), madefromgrape(B,A).
	// resulting in symbolic fact ot____nom8(v_ot____nom8_1)

		if (!meritage_set.contains(v_madefromgrape_1)) continue;
	case 1313:
	// Continuation for label 300
			v_ot____nom8_1 = v_madefromgrape_2;
		if(ot____nom8_set.insert(v_ot____nom8_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1313)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(297)) return -1;
			if(!backtrack_stack.push(296)) return -1;
			goto label_291;
		}
		break;
	case 301:
	// applying rule 
	// 53: riesling(A) :- riesling_aux(A).
	// resulting in symbolic fact riesling(v_riesling_1)
		c53_1_f.open();
	case 1314:
	// Continuation for label 301
		while (c53_1_f.fetch()) {
			v_riesling_1 = c53_1_f.c1();
		if(riesling_set.insert(v_riesling_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1314)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(750)) return -1;
			if(!backtrack_stack.push(310)) return -1;
			if(!backtrack_stack.push(309)) return -1;
			if(!backtrack_stack.push(308)) return -1;
			if(!backtrack_stack.push(305)) return -1;
			if(!backtrack_stack.push(304)) return -1;
			if(!backtrack_stack.push(303)) return -1;
			goto label_302;
		}
		}
		break;
	case 302: label_302: 
	// using  symbolic fact riesling(v_riesling_1) with literal 1 in rule 
	// 52: madefromgrape(A,A) :- riesling(A), kaon2namedobjects(A).
	// resulting in symbolic fact madefromgrape(v_madefromgrape_1,v_madefromgrape_2)

		if (!kaon2namedobjects_b.contains(v_riesling_1)) continue;
	case 1315:
	// Continuation for label 302
			v_madefromgrape_1 = v_riesling_1;
			v_madefromgrape_2 = v_riesling_1;
		if(madefromgrape_set.insert(v_madefromgrape_1, v_madefromgrape_2)) {
			if(!madefromgrape_bf.insert(v_madefromgrape_1, v_madefromgrape_2)) return -1;
			if(!madefromgrape_fb.insert(v_madefromgrape_2, v_madefromgrape_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1315)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(860)) return -1;
			if(!backtrack_stack.push(859)) return -1;
			if(!backtrack_stack.push(845)) return -1;
			if(!backtrack_stack.push(844)) return -1;
			if(!backtrack_stack.push(829)) return -1;
			if(!backtrack_stack.push(828)) return -1;
			if(!backtrack_stack.push(813)) return -1;
			if(!backtrack_stack.push(812)) return -1;
			if(!backtrack_stack.push(677)) return -1;
			if(!backtrack_stack.push(676)) return -1;
			if(!backtrack_stack.push(617)) return -1;
			if(!backtrack_stack.push(616)) return -1;
			if(!backtrack_stack.push(609)) return -1;
			if(!backtrack_stack.push(554)) return -1;
			if(!backtrack_stack.push(553)) return -1;
			if(!backtrack_stack.push(476)) return -1;
			if(!backtrack_stack.push(475)) return -1;
			if(!backtrack_stack.push(404)) return -1;
			if(!backtrack_stack.push(403)) return -1;
			if(!backtrack_stack.push(362)) return -1;
			if(!backtrack_stack.push(361)) return -1;
			if(!backtrack_stack.push(343)) return -1;
			if(!backtrack_stack.push(342)) return -1;
			if(!backtrack_stack.push(300)) return -1;
			if(!backtrack_stack.push(170)) return -1;
			if(!backtrack_stack.push(169)) return -1;
			if(!backtrack_stack.push(151)) return -1;
			if(!backtrack_stack.push(150)) return -1;
			if(!backtrack_stack.push(144)) return -1;
			if(!backtrack_stack.push(132)) return -1;
			if(!backtrack_stack.push(71)) return -1;
			if(!backtrack_stack.push(70)) return -1;
			if(!backtrack_stack.push(66)) return -1;
			goto label_64;
		}
		break;
	case 303: label_303: 
	// using  symbolic fact riesling(v_riesling_1) with literal 1 in rule 
	// 428: wine(A) :- riesling(A).
	// resulting in symbolic fact wine(v_wine_1)

		v_wine_1 = v_riesling_1;
		if(wine_set.insert(v_wine_1)) {
		if(!wine_f.insert(v_wine_1)) return -1;
		// Labels for next rules
		if(!backtrack_stack.push(1007)) return -1;
		if(!backtrack_stack.push(989)) return -1;
		if(!backtrack_stack.push(954)) return -1;
		if(!backtrack_stack.push(938)) return -1;
		if(!backtrack_stack.push(911)) return -1;
		if(!backtrack_stack.push(903)) return -1;
		if(!backtrack_stack.push(902)) return -1;
		if(!backtrack_stack.push(901)) return -1;
		if(!backtrack_stack.push(849)) return -1;
		if(!backtrack_stack.push(848)) return -1;
		if(!backtrack_stack.push(821)) return -1;
		if(!backtrack_stack.push(820)) return -1;
		if(!backtrack_stack.push(737)) return -1;
		if(!backtrack_stack.push(712)) return -1;
		if(!backtrack_stack.push(674)) return -1;
		if(!backtrack_stack.push(672)) return -1;
		if(!backtrack_stack.push(583)) return -1;
		if(!backtrack_stack.push(575)) return -1;
		if(!backtrack_stack.push(563)) return -1;
		if(!backtrack_stack.push(551)) return -1;
		if(!backtrack_stack.push(549)) return -1;
		if(!backtrack_stack.push(483)) return -1;
		if(!backtrack_stack.push(473)) return -1;
		if(!backtrack_stack.push(470)) return -1;
		if(!backtrack_stack.push(430)) return -1;
		if(!backtrack_stack.push(401)) return -1;
		if(!backtrack_stack.push(399)) return -1;
		if(!backtrack_stack.push(380)) return -1;
		if(!backtrack_stack.push(364)) return -1;
		if(!backtrack_stack.push(363)) return -1;
		if(!backtrack_stack.push(329)) return -1;
		if(!backtrack_stack.push(328)) return -1;
		if(!backtrack_stack.push(327)) return -1;
		if(!backtrack_stack.push(295)) return -1;
		if(!backtrack_stack.push(294)) return -1;
		if(!backtrack_stack.push(216)) return -1;
		if(!backtrack_stack.push(210)) return -1;
		if(!backtrack_stack.push(192)) return -1;
		if(!backtrack_stack.push(180)) return -1;
		if(!backtrack_stack.push(179)) return -1;
		if(!backtrack_stack.push(178)) return -1;
		if(!backtrack_stack.push(154)) return -1;
		if(!backtrack_stack.push(153)) return -1;
		if(!backtrack_stack.push(100)) return -1;
		if(!backtrack_stack.push(76)) return -1;
		if(!backtrack_stack.push(75)) return -1;
		if(!backtrack_stack.push(67)) return -1;
		if(!backtrack_stack.push(63)) return -1;
		if(!backtrack_stack.push(53)) return -1;
		if(!backtrack_stack.push(46)) return -1;
		if(!backtrack_stack.push(19)) return -1;
		goto label_16;
		}
		break;
	case 304: label_304: 
	// using  symbolic fact riesling(v_riesling_1) with literal 1 in rule 
	// 430: riesling(A) :- riesling(B), kaon2equal(A,B).
	// resulting in symbolic fact riesling(v_riesling_1)

		//recursive
		c430_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_riesling_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2330)) return -1;
		c430_2_fb.open(v_riesling_1);
	case 1317:
	// Continuation for label 304
		while (c430_2_fb.fetch()) {
			v_riesling_1 = c430_2_fb.c2();
		if(riesling_set.insert(v_riesling_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1317)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(750)) return -1;
			if(!backtrack_stack.push(310)) return -1;
			if(!backtrack_stack.push(309)) return -1;
			if(!backtrack_stack.push(308)) return -1;
			if(!backtrack_stack.push(305)) return -1;
			if(!backtrack_stack.push(304)) return -1;
			if(!backtrack_stack.push(303)) return -1;
			goto label_302;
		}
		}
		break;
	case 2330:
	// Label to restore values of label 304
		v_riesling_1 = int_value_stack.pop();
		c430_2_fb.pop(&cursor_stack);
		break;
	case 305: label_305: 
	// using  symbolic fact riesling(v_riesling_1) with literal 1 in rule 
	// 552: ot____nom37(A) :- riesling(A), kaon2namedobjects(A).
	// resulting in symbolic fact ot____nom37(v_ot____nom37_1)

		//recursive
		if(!int_value_stack.push(v_ot____nom37_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2331)) return -1;
		if (!kaon2namedobjects_b.contains(v_riesling_1)) continue;
	case 1318:
	// Continuation for label 305
			v_ot____nom37_1 = v_riesling_1;
		if(ot____nom37_set.insert(v_ot____nom37_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1318)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(316)) return -1;
			if(!backtrack_stack.push(307)) return -1;
			goto label_306;
		}
		break;
	case 2331:
	// Label to restore values of label 305
		v_ot____nom37_1 = int_value_stack.pop();
		break;
	case 306: label_306: 
	// using  symbolic fact ot____nom37(v_ot____nom37_1) with literal 1 in rule 
	// 553: ot____nom37(A) :- ot____nom37(B), kaon2equal(A,B).
	// resulting in symbolic fact ot____nom37(v_ot____nom37_1)

		//recursive
		c553_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_ot____nom37_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2332)) return -1;
		c553_2_fb.open(v_ot____nom37_1);
	case 1319:
	// Continuation for label 306
		while (c553_2_fb.fetch()) {
			v_ot____nom37_1 = c553_2_fb.c2();
		if(ot____nom37_set.insert(v_ot____nom37_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1319)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(316)) return -1;
			if(!backtrack_stack.push(307)) return -1;
			goto label_306;
		}
		}
		break;
	case 2332:
	// Label to restore values of label 306
		v_ot____nom37_1 = int_value_stack.pop();
		c553_2_fb.pop(&cursor_stack);
		break;
	case 307: label_307: 
	// using  symbolic fact ot____nom37(v_ot____nom37_1) with literal 3 in rule 
	// 431: riesling(A) :- wine(A), madefromgrape(A,B), ot____nom37(B).
	// resulting in symbolic fact riesling(v_riesling_1)

		//recursive
		c431_1_f.push(&cursor_stack);
		if(!int_value_stack.push(v_riesling_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2333)) return -1;
		c431_1_f.open();
		while (c431_1_f.fetch()) {
			if (!madefromgrape_set.contains(c431_1_f.c1(), v_ot____nom37_1)) continue;
	case 1320:
	// Continuation for label 307
				v_riesling_1 = c431_1_f.c1();
		if(riesling_set.insert(v_riesling_1)) {
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1320)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(750)) return -1;
				if(!backtrack_stack.push(310)) return -1;
				if(!backtrack_stack.push(309)) return -1;
				if(!backtrack_stack.push(308)) return -1;
				if(!backtrack_stack.push(305)) return -1;
				if(!backtrack_stack.push(304)) return -1;
				if(!backtrack_stack.push(303)) return -1;
				goto label_302;
		}
		}
		break;
	case 2333:
	// Label to restore values of label 307
		v_riesling_1 = int_value_stack.pop();
		c431_1_f.pop(&cursor_stack);
		break;
	case 308: label_308: 
	// using  symbolic fact riesling(v_riesling_1) with literal 1 in rule 
	// 599: q12(A) :- riesling(A).
	// resulting in symbolic fact q12(v_q12_1)

		v_q12_1 = v_riesling_1;
		if(q12_set.insert(v_q12_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(116)) return -1;
		if(!backtrack_stack.push(115)) return -1;
		if(!backtrack_stack.push(114)) return -1;
		if(!backtrack_stack.push(99)) return -1;
		goto label_95;
		}
		break;
	case 309: label_309: 
	// using  symbolic fact riesling(v_riesling_1) with literal 1 in rule 
	// 770: q3(A) :- riesling(A).
	// resulting in symbolic fact q3(v_q3_1)

		v_q3_1 = v_riesling_1;
		if(q3_set.insert(v_q3_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(143)) return -1;
		goto label_123;
		}
		break;
	case 310: label_310: 
	// using  symbolic fact riesling(v_riesling_1) with literal 2 in rule 
	// 56: dryriesling(A) :- q46(A), riesling(A).
	// resulting in symbolic fact dryriesling(v_dryriesling_1)

		//recursive
		if(!int_value_stack.push(v_dryriesling_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2336)) return -1;
		if (!q46_set.contains(v_riesling_1)) continue;
	case 1323:
	// Continuation for label 310
			v_dryriesling_1 = v_riesling_1;
		if(dryriesling_set.insert(v_dryriesling_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1323)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(315)) return -1;
			if(!backtrack_stack.push(314)) return -1;
			if(!backtrack_stack.push(313)) return -1;
			if(!backtrack_stack.push(312)) return -1;
			goto label_311;
		}
		break;
	case 2336:
	// Label to restore values of label 310
		v_dryriesling_1 = int_value_stack.pop();
		break;
	case 311: label_311: 
	// using  symbolic fact dryriesling(v_dryriesling_1) with literal 1 in rule 
	// 54: riesling(A) :- dryriesling(A).
	// resulting in symbolic fact riesling(v_riesling_1)

		//recursive
		if(!int_value_stack.push(v_riesling_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2337)) return -1;
		v_riesling_1 = v_dryriesling_1;
		if(riesling_set.insert(v_riesling_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(750)) return -1;
		if(!backtrack_stack.push(310)) return -1;
		if(!backtrack_stack.push(309)) return -1;
		if(!backtrack_stack.push(308)) return -1;
		if(!backtrack_stack.push(305)) return -1;
		if(!backtrack_stack.push(304)) return -1;
		if(!backtrack_stack.push(303)) return -1;
		goto label_302;
		}
		break;
	case 2337:
	// Label to restore values of label 311
		v_riesling_1 = int_value_stack.pop();
		break;
	case 312: label_312: 
	// using  symbolic fact dryriesling(v_dryriesling_1) with literal 1 in rule 
	// 139: q20(A) :- dryriesling(A).
	// resulting in symbolic fact q20(v_q20_1)

		v_q20_1 = v_dryriesling_1;
		if(q20_set.insert(v_q20_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(184)) return -1;
		if(!backtrack_stack.push(183)) return -1;
		goto label_182;
		}
		break;
	case 313: label_313: 
	// using  symbolic fact dryriesling(v_dryriesling_1) with literal 1 in rule 
	// 140: dryriesling(A) :- dryriesling(B), kaon2equal(A,B).
	// resulting in symbolic fact dryriesling(v_dryriesling_1)

		//recursive
		c140_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_dryriesling_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2339)) return -1;
		c140_2_fb.open(v_dryriesling_1);
	case 1326:
	// Continuation for label 313
		while (c140_2_fb.fetch()) {
			v_dryriesling_1 = c140_2_fb.c2();
		if(dryriesling_set.insert(v_dryriesling_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1326)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(315)) return -1;
			if(!backtrack_stack.push(314)) return -1;
			if(!backtrack_stack.push(313)) return -1;
			if(!backtrack_stack.push(312)) return -1;
			goto label_311;
		}
		}
		break;
	case 2339:
	// Label to restore values of label 313
		v_dryriesling_1 = int_value_stack.pop();
		c140_2_fb.pop(&cursor_stack);
		break;
	case 314: label_314: 
	// using  symbolic fact dryriesling(v_dryriesling_1) with literal 1 in rule 
	// 596: q12(A) :- dryriesling(A).
	// resulting in symbolic fact q12(v_q12_1)

		v_q12_1 = v_dryriesling_1;
		if(q12_set.insert(v_q12_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(116)) return -1;
		if(!backtrack_stack.push(115)) return -1;
		if(!backtrack_stack.push(114)) return -1;
		if(!backtrack_stack.push(99)) return -1;
		goto label_95;
		}
		break;
	case 315: label_315: 
	// using  symbolic fact dryriesling(v_dryriesling_1) with literal 1 in rule 
	// 638: q31(A) :- dryriesling(A).
	// resulting in symbolic fact q31(v_q31_1)

		v_q31_1 = v_dryriesling_1;
		if(q31_set.insert(v_q31_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(221)) return -1;
		if(!backtrack_stack.push(220)) return -1;
		if(!backtrack_stack.push(215)) return -1;
		if(!backtrack_stack.push(213)) return -1;
		goto label_201;
		}
		break;
	case 316: label_316: 
	// using  symbolic fact ot____nom37(v_ot____nom37_1) with literal 3 in rule 
	// 550: q5(A) :- wine(A), madefromgrape(A,B), ot____nom37(B).
	// resulting in symbolic fact q5(v_q5_1)

		c550_1_f.open();
		while (c550_1_f.fetch()) {
			if (!madefromgrape_set.contains(c550_1_f.c1(), v_ot____nom37_1)) continue;
	case 1329:
	// Continuation for label 316
				v_q5_1 = c550_1_f.c1();
		if(q5_set.insert(v_q5_1)) {
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1329)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(136)) return -1;
				if(!backtrack_stack.push(135)) return -1;
				goto label_131;
		}
		}
		break;
	case 317:
	// applying rule 
	// 55: dryriesling(A) :- dryriesling_aux(A).
	// resulting in symbolic fact dryriesling(v_dryriesling_1)
		c55_1_f.open();
	case 1330:
	// Continuation for label 317
		while (c55_1_f.fetch()) {
			v_dryriesling_1 = c55_1_f.c1();
		if(dryriesling_set.insert(v_dryriesling_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1330)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(315)) return -1;
			if(!backtrack_stack.push(314)) return -1;
			if(!backtrack_stack.push(313)) return -1;
			if(!backtrack_stack.push(312)) return -1;
			goto label_311;
		}
		}
		break;
	case 318:
	// applying rule 
	// 59: chianti(A) :- chianti_aux(A).
	// resulting in symbolic fact chianti(v_chianti_1)
		c59_1_f.open();
	case 1331:
	// Continuation for label 318
		while (c59_1_f.fetch()) {
			v_chianti_1 = c59_1_f.c1();
		if(chianti_set.insert(v_chianti_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1331)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(344)) return -1;
			if(!backtrack_stack.push(341)) return -1;
			if(!backtrack_stack.push(340)) return -1;
			if(!backtrack_stack.push(337)) return -1;
			if(!backtrack_stack.push(321)) return -1;
			if(!backtrack_stack.push(320)) return -1;
			goto label_319;
		}
		}
		break;
	case 319: label_319: 
	// using  symbolic fact chianti(v_chianti_1) with literal 1 in rule 
	// 58: q31(A) :- chianti(A).
	// resulting in symbolic fact q31(v_q31_1)

		v_q31_1 = v_chianti_1;
		if(q31_set.insert(v_q31_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(221)) return -1;
		if(!backtrack_stack.push(220)) return -1;
		if(!backtrack_stack.push(215)) return -1;
		if(!backtrack_stack.push(213)) return -1;
		goto label_201;
		}
		break;
	case 320: label_320: 
	// using  symbolic fact chianti(v_chianti_1) with literal 1 in rule 
	// 60: chianti(A) :- chianti(B), kaon2equal(A,B).
	// resulting in symbolic fact chianti(v_chianti_1)

		//recursive
		c60_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_chianti_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2346)) return -1;
		c60_2_fb.open(v_chianti_1);
	case 1333:
	// Continuation for label 320
		while (c60_2_fb.fetch()) {
			v_chianti_1 = c60_2_fb.c2();
		if(chianti_set.insert(v_chianti_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1333)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(344)) return -1;
			if(!backtrack_stack.push(341)) return -1;
			if(!backtrack_stack.push(340)) return -1;
			if(!backtrack_stack.push(337)) return -1;
			if(!backtrack_stack.push(321)) return -1;
			if(!backtrack_stack.push(320)) return -1;
			goto label_319;
		}
		}
		break;
	case 2346:
	// Label to restore values of label 320
		v_chianti_1 = int_value_stack.pop();
		c60_2_fb.pop(&cursor_stack);
		break;
	case 321: label_321: 
	// using  symbolic fact chianti(v_chianti_1) with literal 1 in rule 
	// 116: italianwine(A) :- chianti(A).
	// resulting in symbolic fact italianwine(v_italianwine_1)

		v_italianwine_1 = v_chianti_1;
		if(italianwine_set.insert(v_italianwine_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(330)) return -1;
		if(!backtrack_stack.push(326)) return -1;
		goto label_322;
		}
		break;
	case 322: label_322: 
	// using  symbolic fact italianwine(v_italianwine_1) with literal 1 in rule 
	// 115: q61(A) :- italianwine(A).
	// resulting in symbolic fact q61(v_q61_1)

		//recursive
		if(!int_value_stack.push(v_q61_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2348)) return -1;
		v_q61_1 = v_italianwine_1;
		if(q61_set.insert(v_q61_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(336)) return -1;
		if(!backtrack_stack.push(333)) return -1;
		if(!backtrack_stack.push(324)) return -1;
		goto label_323;
		}
		break;
	case 2348:
	// Label to restore values of label 322
		v_q61_1 = int_value_stack.pop();
		break;
	case 323: label_323: 
	// using  symbolic fact q61(v_q61_1) with literal 1 in rule 
	// 114: locatedin(A,A) :- q61(A), kaon2namedobjects(A).
	// resulting in symbolic fact locatedin(v_locatedin_1,v_locatedin_2)

		if (!kaon2namedobjects_b.contains(v_q61_1)) continue;
	case 1336:
	// Continuation for label 323
			v_locatedin_1 = v_q61_1;
			v_locatedin_2 = v_q61_1;
		if(locatedin_set.insert(v_locatedin_1, v_locatedin_2)) {
			if(!locatedin_bf.insert(v_locatedin_1, v_locatedin_2)) return -1;
			if(!locatedin_fb.insert(v_locatedin_2, v_locatedin_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1336)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(1005)) return -1;
			if(!backtrack_stack.push(999)) return -1;
			if(!backtrack_stack.push(987)) return -1;
			if(!backtrack_stack.push(981)) return -1;
			if(!backtrack_stack.push(969)) return -1;
			if(!backtrack_stack.push(963)) return -1;
			if(!backtrack_stack.push(952)) return -1;
			if(!backtrack_stack.push(946)) return -1;
			if(!backtrack_stack.push(936)) return -1;
			if(!backtrack_stack.push(930)) return -1;
			if(!backtrack_stack.push(920)) return -1;
			if(!backtrack_stack.push(917)) return -1;
			if(!backtrack_stack.push(898)) return -1;
			if(!backtrack_stack.push(897)) return -1;
			if(!backtrack_stack.push(896)) return -1;
			if(!backtrack_stack.push(893)) return -1;
			if(!backtrack_stack.push(726)) return -1;
			if(!backtrack_stack.push(721)) return -1;
			if(!backtrack_stack.push(694)) return -1;
			if(!backtrack_stack.push(693)) return -1;
			if(!backtrack_stack.push(688)) return -1;
			if(!backtrack_stack.push(687)) return -1;
			if(!backtrack_stack.push(636)) return -1;
			if(!backtrack_stack.push(631)) return -1;
			if(!backtrack_stack.push(592)) return -1;
			if(!backtrack_stack.push(591)) return -1;
			if(!backtrack_stack.push(529)) return -1;
			if(!backtrack_stack.push(528)) return -1;
			if(!backtrack_stack.push(513)) return -1;
			if(!backtrack_stack.push(512)) return -1;
			if(!backtrack_stack.push(511)) return -1;
			if(!backtrack_stack.push(506)) return -1;
			if(!backtrack_stack.push(455)) return -1;
			if(!backtrack_stack.push(450)) return -1;
			if(!backtrack_stack.push(441)) return -1;
			if(!backtrack_stack.push(440)) return -1;
			if(!backtrack_stack.push(439)) return -1;
			if(!backtrack_stack.push(434)) return -1;
			if(!backtrack_stack.push(339)) return -1;
			if(!backtrack_stack.push(338)) return -1;
			if(!backtrack_stack.push(249)) return -1;
			if(!backtrack_stack.push(244)) return -1;
			if(!backtrack_stack.push(52)) return -1;
			if(!backtrack_stack.push(51)) return -1;
			if(!backtrack_stack.push(50)) return -1;
			if(!backtrack_stack.push(49)) return -1;
			if(!backtrack_stack.push(48)) return -1;
			goto label_47;
		}
		break;
	case 324: label_324: 
	// using  symbolic fact q61(v_q61_1) with literal 1 in rule 
	// 118: q62(A) :- q61(A).
	// resulting in symbolic fact q62(v_q62_1)

		//recursive
		if(!int_value_stack.push(v_q62_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2350)) return -1;
		v_q62_1 = v_q61_1;
		if(q62_set.insert(v_q62_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(332)) return -1;
		if(!backtrack_stack.push(331)) return -1;
		goto label_325;
		}
		break;
	case 2350:
	// Label to restore values of label 324
		v_q62_1 = int_value_stack.pop();
		break;
	case 325: label_325: 
	// using  symbolic fact q62(v_q62_1) with literal 1 in rule 
	// 117: italianwine(A) :- q62(A), wine(A).
	// resulting in symbolic fact italianwine(v_italianwine_1)

		//recursive
		if(!int_value_stack.push(v_italianwine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2351)) return -1;
		if (!wine_set.contains(v_q62_1)) continue;
	case 1338:
	// Continuation for label 325
			v_italianwine_1 = v_q62_1;
		if(italianwine_set.insert(v_italianwine_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1338)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(330)) return -1;
			if(!backtrack_stack.push(326)) return -1;
			goto label_322;
		}
		break;
	case 2351:
	// Label to restore values of label 325
		v_italianwine_1 = int_value_stack.pop();
		break;
	case 326: label_326: 
	// using  symbolic fact italianwine(v_italianwine_1) with literal 1 in rule 
	// 493: wine(A) :- italianwine(A).
	// resulting in symbolic fact wine(v_wine_1)

		//recursive
		if(!int_value_stack.push(v_wine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2352)) return -1;
		v_wine_1 = v_italianwine_1;
		if(wine_set.insert(v_wine_1)) {
		if(!wine_f.insert(v_wine_1)) return -1;
		// Labels for next rules
		if(!backtrack_stack.push(1007)) return -1;
		if(!backtrack_stack.push(989)) return -1;
		if(!backtrack_stack.push(954)) return -1;
		if(!backtrack_stack.push(938)) return -1;
		if(!backtrack_stack.push(911)) return -1;
		if(!backtrack_stack.push(903)) return -1;
		if(!backtrack_stack.push(902)) return -1;
		if(!backtrack_stack.push(901)) return -1;
		if(!backtrack_stack.push(849)) return -1;
		if(!backtrack_stack.push(848)) return -1;
		if(!backtrack_stack.push(821)) return -1;
		if(!backtrack_stack.push(820)) return -1;
		if(!backtrack_stack.push(737)) return -1;
		if(!backtrack_stack.push(712)) return -1;
		if(!backtrack_stack.push(674)) return -1;
		if(!backtrack_stack.push(672)) return -1;
		if(!backtrack_stack.push(583)) return -1;
		if(!backtrack_stack.push(575)) return -1;
		if(!backtrack_stack.push(563)) return -1;
		if(!backtrack_stack.push(551)) return -1;
		if(!backtrack_stack.push(549)) return -1;
		if(!backtrack_stack.push(483)) return -1;
		if(!backtrack_stack.push(473)) return -1;
		if(!backtrack_stack.push(470)) return -1;
		if(!backtrack_stack.push(430)) return -1;
		if(!backtrack_stack.push(401)) return -1;
		if(!backtrack_stack.push(399)) return -1;
		if(!backtrack_stack.push(380)) return -1;
		if(!backtrack_stack.push(364)) return -1;
		if(!backtrack_stack.push(363)) return -1;
		if(!backtrack_stack.push(329)) return -1;
		if(!backtrack_stack.push(328)) return -1;
		if(!backtrack_stack.push(327)) return -1;
		if(!backtrack_stack.push(295)) return -1;
		if(!backtrack_stack.push(294)) return -1;
		if(!backtrack_stack.push(216)) return -1;
		if(!backtrack_stack.push(210)) return -1;
		if(!backtrack_stack.push(192)) return -1;
		if(!backtrack_stack.push(180)) return -1;
		if(!backtrack_stack.push(179)) return -1;
		if(!backtrack_stack.push(178)) return -1;
		if(!backtrack_stack.push(154)) return -1;
		if(!backtrack_stack.push(153)) return -1;
		if(!backtrack_stack.push(100)) return -1;
		if(!backtrack_stack.push(76)) return -1;
		if(!backtrack_stack.push(75)) return -1;
		if(!backtrack_stack.push(67)) return -1;
		if(!backtrack_stack.push(63)) return -1;
		if(!backtrack_stack.push(53)) return -1;
		if(!backtrack_stack.push(46)) return -1;
		if(!backtrack_stack.push(19)) return -1;
		goto label_16;
		}
		break;
	case 2352:
	// Label to restore values of label 326
		v_wine_1 = int_value_stack.pop();
		break;
	case 327: label_327: 
	// using  symbolic fact wine(v_wine_1) with literal 1 in rule 
	// 431: riesling(A) :- wine(A), madefromgrape(A,B), ot____nom37(B).
	// resulting in symbolic fact riesling(v_riesling_1)

		c431_2_bf.open(v_wine_1);
		while (c431_2_bf.fetch()) {
			if (!ot____nom37_set.contains(c431_2_bf.c2())) continue;
	case 1340:
	// Continuation for label 327
				v_riesling_1 = v_wine_1;
		if(riesling_set.insert(v_riesling_1)) {
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1340)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(750)) return -1;
				if(!backtrack_stack.push(310)) return -1;
				if(!backtrack_stack.push(309)) return -1;
				if(!backtrack_stack.push(308)) return -1;
				if(!backtrack_stack.push(305)) return -1;
				if(!backtrack_stack.push(304)) return -1;
				if(!backtrack_stack.push(303)) return -1;
				goto label_302;
		}
		}
		break;
	case 328: label_328: 
	// using  symbolic fact wine(v_wine_1) with literal 1 in rule 
	// 550: q5(A) :- wine(A), madefromgrape(A,B), ot____nom37(B).
	// resulting in symbolic fact q5(v_q5_1)

		c550_2_bf.open(v_wine_1);
		while (c550_2_bf.fetch()) {
			if (!ot____nom37_set.contains(c550_2_bf.c2())) continue;
	case 1341:
	// Continuation for label 328
				v_q5_1 = v_wine_1;
		if(q5_set.insert(v_q5_1)) {
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1341)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(136)) return -1;
				if(!backtrack_stack.push(135)) return -1;
				goto label_131;
		}
		}
		break;
	case 329: label_329: 
	// using  symbolic fact wine(v_wine_1) with literal 2 in rule 
	// 117: italianwine(A) :- q62(A), wine(A).
	// resulting in symbolic fact italianwine(v_italianwine_1)

		//recursive
		if(!int_value_stack.push(v_italianwine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2355)) return -1;
		if (!q62_set.contains(v_wine_1)) continue;
	case 1342:
	// Continuation for label 329
			v_italianwine_1 = v_wine_1;
		if(italianwine_set.insert(v_italianwine_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1342)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(330)) return -1;
			if(!backtrack_stack.push(326)) return -1;
			goto label_322;
		}
		break;
	case 2355:
	// Label to restore values of label 329
		v_italianwine_1 = int_value_stack.pop();
		break;
	case 330: label_330: 
	// using  symbolic fact italianwine(v_italianwine_1) with literal 1 in rule 
	// 494: italianwine(A) :- italianwine(B), kaon2equal(A,B).
	// resulting in symbolic fact italianwine(v_italianwine_1)

		//recursive
		c494_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_italianwine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2356)) return -1;
		c494_2_fb.open(v_italianwine_1);
	case 1343:
	// Continuation for label 330
		while (c494_2_fb.fetch()) {
			v_italianwine_1 = c494_2_fb.c2();
		if(italianwine_set.insert(v_italianwine_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1343)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(330)) return -1;
			if(!backtrack_stack.push(326)) return -1;
			goto label_322;
		}
		}
		break;
	case 2356:
	// Label to restore values of label 330
		v_italianwine_1 = int_value_stack.pop();
		c494_2_fb.pop(&cursor_stack);
		break;
	case 331: label_331: 
	// using  symbolic fact q62(v_q62_1) with literal 1 in rule 
	// 749: q62(A) :- q62(B), kaon2equal(A,B).
	// resulting in symbolic fact q62(v_q62_1)

		//recursive
		c749_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q62_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2357)) return -1;
		c749_2_fb.open(v_q62_1);
	case 1344:
	// Continuation for label 331
		while (c749_2_fb.fetch()) {
			v_q62_1 = c749_2_fb.c2();
		if(q62_set.insert(v_q62_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1344)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(332)) return -1;
			if(!backtrack_stack.push(331)) return -1;
			goto label_325;
		}
		}
		break;
	case 2357:
	// Label to restore values of label 331
		v_q62_1 = int_value_stack.pop();
		c749_2_fb.pop(&cursor_stack);
		break;
	case 332: label_332: 
	// using  symbolic fact q62(v_q62_1) with literal 2 in rule 
	// 119: q61(A) :- locatedin(A,B), q62(B).
	// resulting in symbolic fact q61(v_q61_1)

		//recursive
		c119_1_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q61_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2358)) return -1;
		c119_1_fb.open(v_q62_1);
	case 1345:
	// Continuation for label 332
		while (c119_1_fb.fetch()) {
			v_q61_1 = c119_1_fb.c2();
		if(q61_set.insert(v_q61_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1345)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(336)) return -1;
			if(!backtrack_stack.push(333)) return -1;
			if(!backtrack_stack.push(324)) return -1;
			goto label_323;
		}
		}
		break;
	case 2358:
	// Label to restore values of label 332
		v_q61_1 = int_value_stack.pop();
		c119_1_fb.pop(&cursor_stack);
		break;
	case 333: label_333: 
	// using  symbolic fact q61(v_q61_1) with literal 1 in rule 
	// 746: ot____nom47(A) :- q61(A), kaon2namedobjects(A).
	// resulting in symbolic fact ot____nom47(v_ot____nom47_1)

		//recursive
		if(!int_value_stack.push(v_ot____nom47_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2359)) return -1;
		if (!kaon2namedobjects_b.contains(v_q61_1)) continue;
	case 1346:
	// Continuation for label 333
			v_ot____nom47_1 = v_q61_1;
		if(ot____nom47_set.insert(v_ot____nom47_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1346)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(335)) return -1;
			goto label_334;
		}
		break;
	case 2359:
	// Label to restore values of label 333
		v_ot____nom47_1 = int_value_stack.pop();
		break;
	case 334: label_334: 
	// using  symbolic fact ot____nom47(v_ot____nom47_1) with literal 1 in rule 
	// 748: ot____nom47(A) :- ot____nom47(B), kaon2equal(A,B).
	// resulting in symbolic fact ot____nom47(v_ot____nom47_1)

		//recursive
		c748_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_ot____nom47_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2360)) return -1;
		c748_2_fb.open(v_ot____nom47_1);
	case 1347:
	// Continuation for label 334
		while (c748_2_fb.fetch()) {
			v_ot____nom47_1 = c748_2_fb.c2();
		if(ot____nom47_set.insert(v_ot____nom47_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1347)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(335)) return -1;
			goto label_334;
		}
		}
		break;
	case 2360:
	// Label to restore values of label 334
		v_ot____nom47_1 = int_value_stack.pop();
		c748_2_fb.pop(&cursor_stack);
		break;
	case 335: label_335: 
	// using  symbolic fact ot____nom47(v_ot____nom47_1) with literal 2 in rule 
	// 744: q62(A) :- locatedin(A,B), ot____nom47(B).
	// resulting in symbolic fact q62(v_q62_1)

		//recursive
		c744_1_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q62_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2361)) return -1;
		c744_1_fb.open(v_ot____nom47_1);
	case 1348:
	// Continuation for label 335
		while (c744_1_fb.fetch()) {
			v_q62_1 = c744_1_fb.c2();
		if(q62_set.insert(v_q62_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1348)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(332)) return -1;
			if(!backtrack_stack.push(331)) return -1;
			goto label_325;
		}
		}
		break;
	case 2361:
	// Label to restore values of label 335
		v_q62_1 = int_value_stack.pop();
		c744_1_fb.pop(&cursor_stack);
		break;
	case 336: label_336: 
	// using  symbolic fact q61(v_q61_1) with literal 1 in rule 
	// 747: q61(A) :- q61(B), kaon2equal(A,B).
	// resulting in symbolic fact q61(v_q61_1)

		//recursive
		c747_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q61_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2362)) return -1;
		c747_2_fb.open(v_q61_1);
	case 1349:
	// Continuation for label 336
		while (c747_2_fb.fetch()) {
			v_q61_1 = c747_2_fb.c2();
		if(q61_set.insert(v_q61_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1349)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(336)) return -1;
			if(!backtrack_stack.push(333)) return -1;
			if(!backtrack_stack.push(324)) return -1;
			goto label_323;
		}
		}
		break;
	case 2362:
	// Label to restore values of label 336
		v_q61_1 = int_value_stack.pop();
		c747_2_fb.pop(&cursor_stack);
		break;
	case 337: label_337: 
	// using  symbolic fact chianti(v_chianti_1) with literal 1 in rule 
	// 164: locatedin(A,A) :- chianti(A), kaon2namedobjects(A).
	// resulting in symbolic fact locatedin(v_locatedin_1,v_locatedin_2)

		if (!kaon2namedobjects_b.contains(v_chianti_1)) continue;
	case 1350:
	// Continuation for label 337
			v_locatedin_1 = v_chianti_1;
			v_locatedin_2 = v_chianti_1;
		if(locatedin_set.insert(v_locatedin_1, v_locatedin_2)) {
			if(!locatedin_bf.insert(v_locatedin_1, v_locatedin_2)) return -1;
			if(!locatedin_fb.insert(v_locatedin_2, v_locatedin_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1350)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(1005)) return -1;
			if(!backtrack_stack.push(999)) return -1;
			if(!backtrack_stack.push(987)) return -1;
			if(!backtrack_stack.push(981)) return -1;
			if(!backtrack_stack.push(969)) return -1;
			if(!backtrack_stack.push(963)) return -1;
			if(!backtrack_stack.push(952)) return -1;
			if(!backtrack_stack.push(946)) return -1;
			if(!backtrack_stack.push(936)) return -1;
			if(!backtrack_stack.push(930)) return -1;
			if(!backtrack_stack.push(920)) return -1;
			if(!backtrack_stack.push(917)) return -1;
			if(!backtrack_stack.push(898)) return -1;
			if(!backtrack_stack.push(897)) return -1;
			if(!backtrack_stack.push(896)) return -1;
			if(!backtrack_stack.push(893)) return -1;
			if(!backtrack_stack.push(726)) return -1;
			if(!backtrack_stack.push(721)) return -1;
			if(!backtrack_stack.push(694)) return -1;
			if(!backtrack_stack.push(693)) return -1;
			if(!backtrack_stack.push(688)) return -1;
			if(!backtrack_stack.push(687)) return -1;
			if(!backtrack_stack.push(636)) return -1;
			if(!backtrack_stack.push(631)) return -1;
			if(!backtrack_stack.push(592)) return -1;
			if(!backtrack_stack.push(591)) return -1;
			if(!backtrack_stack.push(529)) return -1;
			if(!backtrack_stack.push(528)) return -1;
			if(!backtrack_stack.push(513)) return -1;
			if(!backtrack_stack.push(512)) return -1;
			if(!backtrack_stack.push(511)) return -1;
			if(!backtrack_stack.push(506)) return -1;
			if(!backtrack_stack.push(455)) return -1;
			if(!backtrack_stack.push(450)) return -1;
			if(!backtrack_stack.push(441)) return -1;
			if(!backtrack_stack.push(440)) return -1;
			if(!backtrack_stack.push(439)) return -1;
			if(!backtrack_stack.push(434)) return -1;
			if(!backtrack_stack.push(339)) return -1;
			if(!backtrack_stack.push(338)) return -1;
			if(!backtrack_stack.push(249)) return -1;
			if(!backtrack_stack.push(244)) return -1;
			if(!backtrack_stack.push(52)) return -1;
			if(!backtrack_stack.push(51)) return -1;
			if(!backtrack_stack.push(50)) return -1;
			if(!backtrack_stack.push(49)) return -1;
			if(!backtrack_stack.push(48)) return -1;
			goto label_47;
		}
		break;
	case 338: label_338: 
	// using  symbolic fact locatedin(v_locatedin_1,v_locatedin_2) with literal 1 in rule 
	// 119: q61(A) :- locatedin(A,B), q62(B).
	// resulting in symbolic fact q61(v_q61_1)

		if (!q62_set.contains(v_locatedin_2)) continue;
	case 1351:
	// Continuation for label 338
			v_q61_1 = v_locatedin_1;
		if(q61_set.insert(v_q61_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1351)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(336)) return -1;
			if(!backtrack_stack.push(333)) return -1;
			if(!backtrack_stack.push(324)) return -1;
			goto label_323;
		}
		break;
	case 339: label_339: 
	// using  symbolic fact locatedin(v_locatedin_1,v_locatedin_2) with literal 1 in rule 
	// 744: q62(A) :- locatedin(A,B), ot____nom47(B).
	// resulting in symbolic fact q62(v_q62_1)

		if (!ot____nom47_set.contains(v_locatedin_2)) continue;
	case 1352:
	// Continuation for label 339
			v_q62_1 = v_locatedin_1;
		if(q62_set.insert(v_q62_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1352)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(332)) return -1;
			if(!backtrack_stack.push(331)) return -1;
			goto label_325;
		}
		break;
	case 340: label_340: 
	// using  symbolic fact chianti(v_chianti_1) with literal 1 in rule 
	// 489: q4(A) :- chianti(A).
	// resulting in symbolic fact q4(v_q4_1)

		v_q4_1 = v_chianti_1;
		if(q4_set.insert(v_q4_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(90)) return -1;
		if(!backtrack_stack.push(89)) return -1;
		goto label_88;
		}
		break;
	case 341: label_341: 
	// using  symbolic fact chianti(v_chianti_1) with literal 1 in rule 
	// 524: madefromgrape(A,A) :- chianti(A), kaon2namedobjects(A).
	// resulting in symbolic fact madefromgrape(v_madefromgrape_1,v_madefromgrape_2)

		if (!kaon2namedobjects_b.contains(v_chianti_1)) continue;
	case 1354:
	// Continuation for label 341
			v_madefromgrape_1 = v_chianti_1;
			v_madefromgrape_2 = v_chianti_1;
		if(madefromgrape_set.insert(v_madefromgrape_1, v_madefromgrape_2)) {
			if(!madefromgrape_bf.insert(v_madefromgrape_1, v_madefromgrape_2)) return -1;
			if(!madefromgrape_fb.insert(v_madefromgrape_2, v_madefromgrape_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1354)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(860)) return -1;
			if(!backtrack_stack.push(859)) return -1;
			if(!backtrack_stack.push(845)) return -1;
			if(!backtrack_stack.push(844)) return -1;
			if(!backtrack_stack.push(829)) return -1;
			if(!backtrack_stack.push(828)) return -1;
			if(!backtrack_stack.push(813)) return -1;
			if(!backtrack_stack.push(812)) return -1;
			if(!backtrack_stack.push(677)) return -1;
			if(!backtrack_stack.push(676)) return -1;
			if(!backtrack_stack.push(617)) return -1;
			if(!backtrack_stack.push(616)) return -1;
			if(!backtrack_stack.push(609)) return -1;
			if(!backtrack_stack.push(554)) return -1;
			if(!backtrack_stack.push(553)) return -1;
			if(!backtrack_stack.push(476)) return -1;
			if(!backtrack_stack.push(475)) return -1;
			if(!backtrack_stack.push(404)) return -1;
			if(!backtrack_stack.push(403)) return -1;
			if(!backtrack_stack.push(362)) return -1;
			if(!backtrack_stack.push(361)) return -1;
			if(!backtrack_stack.push(343)) return -1;
			if(!backtrack_stack.push(342)) return -1;
			if(!backtrack_stack.push(300)) return -1;
			if(!backtrack_stack.push(170)) return -1;
			if(!backtrack_stack.push(169)) return -1;
			if(!backtrack_stack.push(151)) return -1;
			if(!backtrack_stack.push(150)) return -1;
			if(!backtrack_stack.push(144)) return -1;
			if(!backtrack_stack.push(132)) return -1;
			if(!backtrack_stack.push(71)) return -1;
			if(!backtrack_stack.push(70)) return -1;
			if(!backtrack_stack.push(66)) return -1;
			goto label_64;
		}
		break;
	case 342: label_342: 
	// using  symbolic fact madefromgrape(v_madefromgrape_1,v_madefromgrape_2) with literal 2 in rule 
	// 431: riesling(A) :- wine(A), madefromgrape(A,B), ot____nom37(B).
	// resulting in symbolic fact riesling(v_riesling_1)

		if (!wine_set.contains(v_madefromgrape_1)) continue;
			if (!ot____nom37_set.contains(v_madefromgrape_2)) continue;
	case 1355:
	// Continuation for label 342
				v_riesling_1 = v_madefromgrape_1;
		if(riesling_set.insert(v_riesling_1)) {
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1355)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(750)) return -1;
				if(!backtrack_stack.push(310)) return -1;
				if(!backtrack_stack.push(309)) return -1;
				if(!backtrack_stack.push(308)) return -1;
				if(!backtrack_stack.push(305)) return -1;
				if(!backtrack_stack.push(304)) return -1;
				if(!backtrack_stack.push(303)) return -1;
				goto label_302;
		}
		break;
	case 343: label_343: 
	// using  symbolic fact madefromgrape(v_madefromgrape_1,v_madefromgrape_2) with literal 2 in rule 
	// 550: q5(A) :- wine(A), madefromgrape(A,B), ot____nom37(B).
	// resulting in symbolic fact q5(v_q5_1)

		if (!wine_set.contains(v_madefromgrape_1)) continue;
			if (!ot____nom37_set.contains(v_madefromgrape_2)) continue;
	case 1356:
	// Continuation for label 343
				v_q5_1 = v_madefromgrape_1;
		if(q5_set.insert(v_q5_1)) {
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1356)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(136)) return -1;
				if(!backtrack_stack.push(135)) return -1;
				goto label_131;
		}
		break;
	case 344: label_344: 
	// using  symbolic fact chianti(v_chianti_1) with literal 1 in rule 
	// 608: q0(A) :- chianti(A).
	// resulting in symbolic fact q0(v_q0_1)

		v_q0_1 = v_chianti_1;
		if(q0_set.insert(v_q0_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(198)) return -1;
		if(!backtrack_stack.push(194)) return -1;
		if(!backtrack_stack.push(189)) return -1;
		if(!backtrack_stack.push(188)) return -1;
		goto label_187;
		}
		break;
	case 345:
	// applying rule 
	// 62: hasmaker(A,B) :- hasmaker_aux(A,B).
	// resulting in symbolic fact hasmaker(v_hasmaker_1,v_hasmaker_2)
		c62_1_ff.open();
	case 1358:
	// Continuation for label 345
		while (c62_1_ff.fetch()) {
			v_hasmaker_2 = c62_1_ff.c2();
			v_hasmaker_1 = c62_1_ff.c1();
		if(hasmaker_set.insert(v_hasmaker_1, v_hasmaker_2)) {
			if(!hasmaker_bf.insert(v_hasmaker_1, v_hasmaker_2)) return -1;
			if(!hasmaker_fb.insert(v_hasmaker_2, v_hasmaker_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1358)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(40)) return -1;
			if(!backtrack_stack.push(39)) return -1;
			if(!backtrack_stack.push(38)) return -1;
			if(!backtrack_stack.push(36)) return -1;
			goto label_20;
		}
		}
		break;
	case 346:
	// applying rule 
	// 68: chardonnay(A) :- chardonnay_aux(A).
	// resulting in symbolic fact chardonnay(v_chardonnay_1)
		c68_1_f.open();
	case 1359:
	// Continuation for label 346
		while (c68_1_f.fetch()) {
			v_chardonnay_1 = c68_1_f.c1();
		if(chardonnay_set.insert(v_chardonnay_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1359)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(353)) return -1;
			if(!backtrack_stack.push(352)) return -1;
			if(!backtrack_stack.push(351)) return -1;
			if(!backtrack_stack.push(350)) return -1;
			goto label_347;
		}
		}
		break;
	case 347: label_347: 
	// using  symbolic fact chardonnay(v_chardonnay_1) with literal 1 in rule 
	// 67: q63(A) :- chardonnay(A).
	// resulting in symbolic fact q63(v_q63_1)

		//recursive
		if(!int_value_stack.push(v_q63_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2373)) return -1;
		v_q63_1 = v_chardonnay_1;
		if(q63_set.insert(v_q63_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(357)) return -1;
		if(!backtrack_stack.push(356)) return -1;
		if(!backtrack_stack.push(355)) return -1;
		if(!backtrack_stack.push(354)) return -1;
		if(!backtrack_stack.push(349)) return -1;
		goto label_348;
		}
		break;
	case 2373:
	// Label to restore values of label 347
		v_q63_1 = int_value_stack.pop();
		break;
	case 348: label_348: 
	// using  symbolic fact q63(v_q63_1) with literal 1 in rule 
	// 66: wine(A) :- q63(A).
	// resulting in symbolic fact wine(v_wine_1)

		v_wine_1 = v_q63_1;
		if(wine_set.insert(v_wine_1)) {
		if(!wine_f.insert(v_wine_1)) return -1;
		// Labels for next rules
		if(!backtrack_stack.push(1007)) return -1;
		if(!backtrack_stack.push(989)) return -1;
		if(!backtrack_stack.push(954)) return -1;
		if(!backtrack_stack.push(938)) return -1;
		if(!backtrack_stack.push(911)) return -1;
		if(!backtrack_stack.push(903)) return -1;
		if(!backtrack_stack.push(902)) return -1;
		if(!backtrack_stack.push(901)) return -1;
		if(!backtrack_stack.push(849)) return -1;
		if(!backtrack_stack.push(848)) return -1;
		if(!backtrack_stack.push(821)) return -1;
		if(!backtrack_stack.push(820)) return -1;
		if(!backtrack_stack.push(737)) return -1;
		if(!backtrack_stack.push(712)) return -1;
		if(!backtrack_stack.push(674)) return -1;
		if(!backtrack_stack.push(672)) return -1;
		if(!backtrack_stack.push(583)) return -1;
		if(!backtrack_stack.push(575)) return -1;
		if(!backtrack_stack.push(563)) return -1;
		if(!backtrack_stack.push(551)) return -1;
		if(!backtrack_stack.push(549)) return -1;
		if(!backtrack_stack.push(483)) return -1;
		if(!backtrack_stack.push(473)) return -1;
		if(!backtrack_stack.push(470)) return -1;
		if(!backtrack_stack.push(430)) return -1;
		if(!backtrack_stack.push(401)) return -1;
		if(!backtrack_stack.push(399)) return -1;
		if(!backtrack_stack.push(380)) return -1;
		if(!backtrack_stack.push(364)) return -1;
		if(!backtrack_stack.push(363)) return -1;
		if(!backtrack_stack.push(329)) return -1;
		if(!backtrack_stack.push(328)) return -1;
		if(!backtrack_stack.push(327)) return -1;
		if(!backtrack_stack.push(295)) return -1;
		if(!backtrack_stack.push(294)) return -1;
		if(!backtrack_stack.push(216)) return -1;
		if(!backtrack_stack.push(210)) return -1;
		if(!backtrack_stack.push(192)) return -1;
		if(!backtrack_stack.push(180)) return -1;
		if(!backtrack_stack.push(179)) return -1;
		if(!backtrack_stack.push(178)) return -1;
		if(!backtrack_stack.push(154)) return -1;
		if(!backtrack_stack.push(153)) return -1;
		if(!backtrack_stack.push(100)) return -1;
		if(!backtrack_stack.push(76)) return -1;
		if(!backtrack_stack.push(75)) return -1;
		if(!backtrack_stack.push(67)) return -1;
		if(!backtrack_stack.push(63)) return -1;
		if(!backtrack_stack.push(53)) return -1;
		if(!backtrack_stack.push(46)) return -1;
		if(!backtrack_stack.push(19)) return -1;
		goto label_16;
		}
		break;
	case 349: label_349: 
	// using  symbolic fact q63(v_q63_1) with literal 1 in rule 
	// 69: chardonnay(A) :- q63(A).
	// resulting in symbolic fact chardonnay(v_chardonnay_1)

		//recursive
		if(!int_value_stack.push(v_chardonnay_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2375)) return -1;
		v_chardonnay_1 = v_q63_1;
		if(chardonnay_set.insert(v_chardonnay_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(353)) return -1;
		if(!backtrack_stack.push(352)) return -1;
		if(!backtrack_stack.push(351)) return -1;
		if(!backtrack_stack.push(350)) return -1;
		goto label_347;
		}
		break;
	case 2375:
	// Label to restore values of label 349
		v_chardonnay_1 = int_value_stack.pop();
		break;
	case 350: label_350: 
	// using  symbolic fact chardonnay(v_chardonnay_1) with literal 1 in rule 
	// 400: q12(A) :- chardonnay(A).
	// resulting in symbolic fact q12(v_q12_1)

		v_q12_1 = v_chardonnay_1;
		if(q12_set.insert(v_q12_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(116)) return -1;
		if(!backtrack_stack.push(115)) return -1;
		if(!backtrack_stack.push(114)) return -1;
		if(!backtrack_stack.push(99)) return -1;
		goto label_95;
		}
		break;
	case 351: label_351: 
	// using  symbolic fact chardonnay(v_chardonnay_1) with literal 1 in rule 
	// 401: chardonnay(A) :- chardonnay(B), kaon2equal(A,B).
	// resulting in symbolic fact chardonnay(v_chardonnay_1)

		//recursive
		c401_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_chardonnay_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2377)) return -1;
		c401_2_fb.open(v_chardonnay_1);
	case 1364:
	// Continuation for label 351
		while (c401_2_fb.fetch()) {
			v_chardonnay_1 = c401_2_fb.c2();
		if(chardonnay_set.insert(v_chardonnay_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1364)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(353)) return -1;
			if(!backtrack_stack.push(352)) return -1;
			if(!backtrack_stack.push(351)) return -1;
			if(!backtrack_stack.push(350)) return -1;
			goto label_347;
		}
		}
		break;
	case 2377:
	// Label to restore values of label 351
		v_chardonnay_1 = int_value_stack.pop();
		c401_2_fb.pop(&cursor_stack);
		break;
	case 352: label_352: 
	// using  symbolic fact chardonnay(v_chardonnay_1) with literal 1 in rule 
	// 495: wine(A) :- chardonnay(A).
	// resulting in symbolic fact wine(v_wine_1)

		//recursive
		if(!int_value_stack.push(v_wine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2378)) return -1;
		v_wine_1 = v_chardonnay_1;
		if(wine_set.insert(v_wine_1)) {
		if(!wine_f.insert(v_wine_1)) return -1;
		// Labels for next rules
		if(!backtrack_stack.push(1007)) return -1;
		if(!backtrack_stack.push(989)) return -1;
		if(!backtrack_stack.push(954)) return -1;
		if(!backtrack_stack.push(938)) return -1;
		if(!backtrack_stack.push(911)) return -1;
		if(!backtrack_stack.push(903)) return -1;
		if(!backtrack_stack.push(902)) return -1;
		if(!backtrack_stack.push(901)) return -1;
		if(!backtrack_stack.push(849)) return -1;
		if(!backtrack_stack.push(848)) return -1;
		if(!backtrack_stack.push(821)) return -1;
		if(!backtrack_stack.push(820)) return -1;
		if(!backtrack_stack.push(737)) return -1;
		if(!backtrack_stack.push(712)) return -1;
		if(!backtrack_stack.push(674)) return -1;
		if(!backtrack_stack.push(672)) return -1;
		if(!backtrack_stack.push(583)) return -1;
		if(!backtrack_stack.push(575)) return -1;
		if(!backtrack_stack.push(563)) return -1;
		if(!backtrack_stack.push(551)) return -1;
		if(!backtrack_stack.push(549)) return -1;
		if(!backtrack_stack.push(483)) return -1;
		if(!backtrack_stack.push(473)) return -1;
		if(!backtrack_stack.push(470)) return -1;
		if(!backtrack_stack.push(430)) return -1;
		if(!backtrack_stack.push(401)) return -1;
		if(!backtrack_stack.push(399)) return -1;
		if(!backtrack_stack.push(380)) return -1;
		if(!backtrack_stack.push(364)) return -1;
		if(!backtrack_stack.push(363)) return -1;
		if(!backtrack_stack.push(329)) return -1;
		if(!backtrack_stack.push(328)) return -1;
		if(!backtrack_stack.push(327)) return -1;
		if(!backtrack_stack.push(295)) return -1;
		if(!backtrack_stack.push(294)) return -1;
		if(!backtrack_stack.push(216)) return -1;
		if(!backtrack_stack.push(210)) return -1;
		if(!backtrack_stack.push(192)) return -1;
		if(!backtrack_stack.push(180)) return -1;
		if(!backtrack_stack.push(179)) return -1;
		if(!backtrack_stack.push(178)) return -1;
		if(!backtrack_stack.push(154)) return -1;
		if(!backtrack_stack.push(153)) return -1;
		if(!backtrack_stack.push(100)) return -1;
		if(!backtrack_stack.push(76)) return -1;
		if(!backtrack_stack.push(75)) return -1;
		if(!backtrack_stack.push(67)) return -1;
		if(!backtrack_stack.push(63)) return -1;
		if(!backtrack_stack.push(53)) return -1;
		if(!backtrack_stack.push(46)) return -1;
		if(!backtrack_stack.push(19)) return -1;
		goto label_16;
		}
		break;
	case 2378:
	// Label to restore values of label 352
		v_wine_1 = int_value_stack.pop();
		break;
	case 353: label_353: 
	// using  symbolic fact chardonnay(v_chardonnay_1) with literal 1 in rule 
	// 768: q3(A) :- chardonnay(A).
	// resulting in symbolic fact q3(v_q3_1)

		v_q3_1 = v_chardonnay_1;
		if(q3_set.insert(v_q3_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(143)) return -1;
		goto label_123;
		}
		break;
	case 354: label_354: 
	// using  symbolic fact q63(v_q63_1) with literal 1 in rule 
	// 504: q5(A) :- q63(A).
	// resulting in symbolic fact q5(v_q5_1)

		v_q5_1 = v_q63_1;
		if(q5_set.insert(v_q5_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(136)) return -1;
		if(!backtrack_stack.push(135)) return -1;
		goto label_131;
		}
		break;
	case 355: label_355: 
	// using  symbolic fact q63(v_q63_1) with literal 1 in rule 
	// 505: q63(A) :- q63(B), kaon2equal(A,B).
	// resulting in symbolic fact q63(v_q63_1)

		//recursive
		c505_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q63_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2381)) return -1;
		c505_2_fb.open(v_q63_1);
	case 1368:
	// Continuation for label 355
		while (c505_2_fb.fetch()) {
			v_q63_1 = c505_2_fb.c2();
		if(q63_set.insert(v_q63_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1368)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(357)) return -1;
			if(!backtrack_stack.push(356)) return -1;
			if(!backtrack_stack.push(355)) return -1;
			if(!backtrack_stack.push(354)) return -1;
			if(!backtrack_stack.push(349)) return -1;
			goto label_348;
		}
		}
		break;
	case 2381:
	// Label to restore values of label 355
		v_q63_1 = int_value_stack.pop();
		c505_2_fb.pop(&cursor_stack);
		break;
	case 356: label_356: 
	// using  symbolic fact q63(v_q63_1) with literal 1 in rule 
	// 528: madefromgrape(A,A) :- q63(A), kaon2namedobjects(A).
	// resulting in symbolic fact madefromgrape(v_madefromgrape_1,v_madefromgrape_2)

		//recursive
		if(!int_value_stack.push(v_madefromgrape_2)) return -1;
		if(!int_value_stack.push(v_madefromgrape_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2382)) return -1;
		if (!kaon2namedobjects_b.contains(v_q63_1)) continue;
	case 1369:
	// Continuation for label 356
			v_madefromgrape_1 = v_q63_1;
			v_madefromgrape_2 = v_q63_1;
		if(madefromgrape_set.insert(v_madefromgrape_1, v_madefromgrape_2)) {
			if(!madefromgrape_bf.insert(v_madefromgrape_1, v_madefromgrape_2)) return -1;
			if(!madefromgrape_fb.insert(v_madefromgrape_2, v_madefromgrape_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1369)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(860)) return -1;
			if(!backtrack_stack.push(859)) return -1;
			if(!backtrack_stack.push(845)) return -1;
			if(!backtrack_stack.push(844)) return -1;
			if(!backtrack_stack.push(829)) return -1;
			if(!backtrack_stack.push(828)) return -1;
			if(!backtrack_stack.push(813)) return -1;
			if(!backtrack_stack.push(812)) return -1;
			if(!backtrack_stack.push(677)) return -1;
			if(!backtrack_stack.push(676)) return -1;
			if(!backtrack_stack.push(617)) return -1;
			if(!backtrack_stack.push(616)) return -1;
			if(!backtrack_stack.push(609)) return -1;
			if(!backtrack_stack.push(554)) return -1;
			if(!backtrack_stack.push(553)) return -1;
			if(!backtrack_stack.push(476)) return -1;
			if(!backtrack_stack.push(475)) return -1;
			if(!backtrack_stack.push(404)) return -1;
			if(!backtrack_stack.push(403)) return -1;
			if(!backtrack_stack.push(362)) return -1;
			if(!backtrack_stack.push(361)) return -1;
			if(!backtrack_stack.push(343)) return -1;
			if(!backtrack_stack.push(342)) return -1;
			if(!backtrack_stack.push(300)) return -1;
			if(!backtrack_stack.push(170)) return -1;
			if(!backtrack_stack.push(169)) return -1;
			if(!backtrack_stack.push(151)) return -1;
			if(!backtrack_stack.push(150)) return -1;
			if(!backtrack_stack.push(144)) return -1;
			if(!backtrack_stack.push(132)) return -1;
			if(!backtrack_stack.push(71)) return -1;
			if(!backtrack_stack.push(70)) return -1;
			if(!backtrack_stack.push(66)) return -1;
			goto label_64;
		}
		break;
	case 2382:
	// Label to restore values of label 356
		v_madefromgrape_1 = int_value_stack.pop();
		v_madefromgrape_2 = int_value_stack.pop();
		break;
	case 357: label_357: 
	// using  symbolic fact q63(v_q63_1) with literal 1 in rule 
	// 540: ot____nom24(A) :- q63(A), kaon2namedobjects(A).
	// resulting in symbolic fact ot____nom24(v_ot____nom24_1)

		//recursive
		if(!int_value_stack.push(v_ot____nom24_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2383)) return -1;
		if (!kaon2namedobjects_b.contains(v_q63_1)) continue;
	case 1370:
	// Continuation for label 357
			v_ot____nom24_1 = v_q63_1;
		if(ot____nom24_set.insert(v_ot____nom24_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1370)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(360)) return -1;
			if(!backtrack_stack.push(359)) return -1;
			goto label_358;
		}
		break;
	case 2383:
	// Label to restore values of label 357
		v_ot____nom24_1 = int_value_stack.pop();
		break;
	case 358: label_358: 
	// using  symbolic fact ot____nom24(v_ot____nom24_1) with literal 1 in rule 
	// 541: ot____nom24(A) :- ot____nom24(B), kaon2equal(A,B).
	// resulting in symbolic fact ot____nom24(v_ot____nom24_1)

		//recursive
		c541_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_ot____nom24_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2384)) return -1;
		c541_2_fb.open(v_ot____nom24_1);
	case 1371:
	// Continuation for label 358
		while (c541_2_fb.fetch()) {
			v_ot____nom24_1 = c541_2_fb.c2();
		if(ot____nom24_set.insert(v_ot____nom24_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1371)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(360)) return -1;
			if(!backtrack_stack.push(359)) return -1;
			goto label_358;
		}
		}
		break;
	case 2384:
	// Label to restore values of label 358
		v_ot____nom24_1 = int_value_stack.pop();
		c541_2_fb.pop(&cursor_stack);
		break;
	case 359: label_359: 
	// using  symbolic fact ot____nom24(v_ot____nom24_1) with literal 3 in rule 
	// 402: chardonnay(A) :- wine(A), madefromgrape(A,B), ot____nom24(B).
	// resulting in symbolic fact chardonnay(v_chardonnay_1)

		//recursive
		c402_1_f.push(&cursor_stack);
		if(!int_value_stack.push(v_chardonnay_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2385)) return -1;
		c402_1_f.open();
		while (c402_1_f.fetch()) {
			if (!madefromgrape_set.contains(c402_1_f.c1(), v_ot____nom24_1)) continue;
	case 1372:
	// Continuation for label 359
				v_chardonnay_1 = c402_1_f.c1();
		if(chardonnay_set.insert(v_chardonnay_1)) {
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1372)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(353)) return -1;
				if(!backtrack_stack.push(352)) return -1;
				if(!backtrack_stack.push(351)) return -1;
				if(!backtrack_stack.push(350)) return -1;
				goto label_347;
		}
		}
		break;
	case 2385:
	// Label to restore values of label 359
		v_chardonnay_1 = int_value_stack.pop();
		c402_1_f.pop(&cursor_stack);
		break;
	case 360: label_360: 
	// using  symbolic fact ot____nom24(v_ot____nom24_1) with literal 3 in rule 
	// 538: q5(A) :- wine(A), madefromgrape(A,B), ot____nom24(B).
	// resulting in symbolic fact q5(v_q5_1)

		c538_1_f.open();
		while (c538_1_f.fetch()) {
			if (!madefromgrape_set.contains(c538_1_f.c1(), v_ot____nom24_1)) continue;
	case 1373:
	// Continuation for label 360
				v_q5_1 = c538_1_f.c1();
		if(q5_set.insert(v_q5_1)) {
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1373)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(136)) return -1;
				if(!backtrack_stack.push(135)) return -1;
				goto label_131;
		}
		}
		break;
	case 361: label_361: 
	// using  symbolic fact madefromgrape(v_madefromgrape_1,v_madefromgrape_2) with literal 2 in rule 
	// 402: chardonnay(A) :- wine(A), madefromgrape(A,B), ot____nom24(B).
	// resulting in symbolic fact chardonnay(v_chardonnay_1)

		//recursive
		if(!int_value_stack.push(v_chardonnay_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2387)) return -1;
		if (!wine_set.contains(v_madefromgrape_1)) continue;
			if (!ot____nom24_set.contains(v_madefromgrape_2)) continue;
	case 1374:
	// Continuation for label 361
				v_chardonnay_1 = v_madefromgrape_1;
		if(chardonnay_set.insert(v_chardonnay_1)) {
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1374)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(353)) return -1;
				if(!backtrack_stack.push(352)) return -1;
				if(!backtrack_stack.push(351)) return -1;
				if(!backtrack_stack.push(350)) return -1;
				goto label_347;
		}
		break;
	case 2387:
	// Label to restore values of label 361
		v_chardonnay_1 = int_value_stack.pop();
		break;
	case 362: label_362: 
	// using  symbolic fact madefromgrape(v_madefromgrape_1,v_madefromgrape_2) with literal 2 in rule 
	// 538: q5(A) :- wine(A), madefromgrape(A,B), ot____nom24(B).
	// resulting in symbolic fact q5(v_q5_1)

		if (!wine_set.contains(v_madefromgrape_1)) continue;
			if (!ot____nom24_set.contains(v_madefromgrape_2)) continue;
	case 1375:
	// Continuation for label 362
				v_q5_1 = v_madefromgrape_1;
		if(q5_set.insert(v_q5_1)) {
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1375)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(136)) return -1;
				if(!backtrack_stack.push(135)) return -1;
				goto label_131;
		}
		break;
	case 363: label_363: 
	// using  symbolic fact wine(v_wine_1) with literal 1 in rule 
	// 402: chardonnay(A) :- wine(A), madefromgrape(A,B), ot____nom24(B).
	// resulting in symbolic fact chardonnay(v_chardonnay_1)

		//recursive
		c402_2_bf.push(&cursor_stack);
		if(!int_value_stack.push(v_chardonnay_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2389)) return -1;
		c402_2_bf.open(v_wine_1);
		while (c402_2_bf.fetch()) {
			if (!ot____nom24_set.contains(c402_2_bf.c2())) continue;
	case 1376:
	// Continuation for label 363
				v_chardonnay_1 = v_wine_1;
		if(chardonnay_set.insert(v_chardonnay_1)) {
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1376)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(353)) return -1;
				if(!backtrack_stack.push(352)) return -1;
				if(!backtrack_stack.push(351)) return -1;
				if(!backtrack_stack.push(350)) return -1;
				goto label_347;
		}
		}
		break;
	case 2389:
	// Label to restore values of label 363
		v_chardonnay_1 = int_value_stack.pop();
		c402_2_bf.pop(&cursor_stack);
		break;
	case 364: label_364: 
	// using  symbolic fact wine(v_wine_1) with literal 1 in rule 
	// 538: q5(A) :- wine(A), madefromgrape(A,B), ot____nom24(B).
	// resulting in symbolic fact q5(v_q5_1)

		c538_2_bf.open(v_wine_1);
		while (c538_2_bf.fetch()) {
			if (!ot____nom24_set.contains(c538_2_bf.c2())) continue;
	case 1377:
	// Continuation for label 364
				v_q5_1 = v_wine_1;
		if(q5_set.insert(v_q5_1)) {
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1377)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(136)) return -1;
				if(!backtrack_stack.push(135)) return -1;
				goto label_131;
		}
		}
		break;
	case 365:
	// applying rule 
	// 71: whiteburgundy(A) :- whiteburgundy_aux(A).
	// resulting in symbolic fact whiteburgundy(v_whiteburgundy_1)
		c71_1_f.open();
	case 1378:
	// Continuation for label 365
		while (c71_1_f.fetch()) {
			v_whiteburgundy_1 = c71_1_f.c1();
		if(whiteburgundy_set.insert(v_whiteburgundy_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1378)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(376)) return -1;
			if(!backtrack_stack.push(375)) return -1;
			if(!backtrack_stack.push(373)) return -1;
			if(!backtrack_stack.push(367)) return -1;
			goto label_366;
		}
		}
		break;
	case 366: label_366: 
	// using  symbolic fact whiteburgundy(v_whiteburgundy_1) with literal 1 in rule 
	// 70: q63(A) :- whiteburgundy(A).
	// resulting in symbolic fact q63(v_q63_1)

		v_q63_1 = v_whiteburgundy_1;
		if(q63_set.insert(v_q63_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(357)) return -1;
		if(!backtrack_stack.push(356)) return -1;
		if(!backtrack_stack.push(355)) return -1;
		if(!backtrack_stack.push(354)) return -1;
		if(!backtrack_stack.push(349)) return -1;
		goto label_348;
		}
		break;
	case 367: label_367: 
	// using  symbolic fact whiteburgundy(v_whiteburgundy_1) with literal 1 in rule 
	// 186: burgundy(A) :- whiteburgundy(A).
	// resulting in symbolic fact burgundy(v_burgundy_1)

		//recursive
		if(!int_value_stack.push(v_burgundy_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2393)) return -1;
		v_burgundy_1 = v_whiteburgundy_1;
		if(burgundy_set.insert(v_burgundy_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(382)) return -1;
		if(!backtrack_stack.push(381)) return -1;
		if(!backtrack_stack.push(379)) return -1;
		if(!backtrack_stack.push(377)) return -1;
		if(!backtrack_stack.push(372)) return -1;
		goto label_368;
		}
		break;
	case 2393:
	// Label to restore values of label 367
		v_burgundy_1 = int_value_stack.pop();
		break;
	case 368: label_368: 
	// using  symbolic fact burgundy(v_burgundy_1) with literal 1 in rule 
	// 185: q29(A) :- burgundy(A).
	// resulting in symbolic fact q29(v_q29_1)

		//recursive
		if(!int_value_stack.push(v_q29_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2394)) return -1;
		v_q29_1 = v_burgundy_1;
		if(q29_set.insert(v_q29_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(411)) return -1;
		if(!backtrack_stack.push(408)) return -1;
		if(!backtrack_stack.push(370)) return -1;
		goto label_369;
		}
		break;
	case 2394:
	// Label to restore values of label 368
		v_q29_1 = int_value_stack.pop();
		break;
	case 369: label_369: 
	// using  symbolic fact q29(v_q29_1) with literal 1 in rule 
	// 184: locatedin(A,A) :- q29(A), kaon2namedobjects(A).
	// resulting in symbolic fact locatedin(v_locatedin_1,v_locatedin_2)

		if (!kaon2namedobjects_b.contains(v_q29_1)) continue;
	case 1382:
	// Continuation for label 369
			v_locatedin_1 = v_q29_1;
			v_locatedin_2 = v_q29_1;
		if(locatedin_set.insert(v_locatedin_1, v_locatedin_2)) {
			if(!locatedin_bf.insert(v_locatedin_1, v_locatedin_2)) return -1;
			if(!locatedin_fb.insert(v_locatedin_2, v_locatedin_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1382)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(1005)) return -1;
			if(!backtrack_stack.push(999)) return -1;
			if(!backtrack_stack.push(987)) return -1;
			if(!backtrack_stack.push(981)) return -1;
			if(!backtrack_stack.push(969)) return -1;
			if(!backtrack_stack.push(963)) return -1;
			if(!backtrack_stack.push(952)) return -1;
			if(!backtrack_stack.push(946)) return -1;
			if(!backtrack_stack.push(936)) return -1;
			if(!backtrack_stack.push(930)) return -1;
			if(!backtrack_stack.push(920)) return -1;
			if(!backtrack_stack.push(917)) return -1;
			if(!backtrack_stack.push(898)) return -1;
			if(!backtrack_stack.push(897)) return -1;
			if(!backtrack_stack.push(896)) return -1;
			if(!backtrack_stack.push(893)) return -1;
			if(!backtrack_stack.push(726)) return -1;
			if(!backtrack_stack.push(721)) return -1;
			if(!backtrack_stack.push(694)) return -1;
			if(!backtrack_stack.push(693)) return -1;
			if(!backtrack_stack.push(688)) return -1;
			if(!backtrack_stack.push(687)) return -1;
			if(!backtrack_stack.push(636)) return -1;
			if(!backtrack_stack.push(631)) return -1;
			if(!backtrack_stack.push(592)) return -1;
			if(!backtrack_stack.push(591)) return -1;
			if(!backtrack_stack.push(529)) return -1;
			if(!backtrack_stack.push(528)) return -1;
			if(!backtrack_stack.push(513)) return -1;
			if(!backtrack_stack.push(512)) return -1;
			if(!backtrack_stack.push(511)) return -1;
			if(!backtrack_stack.push(506)) return -1;
			if(!backtrack_stack.push(455)) return -1;
			if(!backtrack_stack.push(450)) return -1;
			if(!backtrack_stack.push(441)) return -1;
			if(!backtrack_stack.push(440)) return -1;
			if(!backtrack_stack.push(439)) return -1;
			if(!backtrack_stack.push(434)) return -1;
			if(!backtrack_stack.push(339)) return -1;
			if(!backtrack_stack.push(338)) return -1;
			if(!backtrack_stack.push(249)) return -1;
			if(!backtrack_stack.push(244)) return -1;
			if(!backtrack_stack.push(52)) return -1;
			if(!backtrack_stack.push(51)) return -1;
			if(!backtrack_stack.push(50)) return -1;
			if(!backtrack_stack.push(49)) return -1;
			if(!backtrack_stack.push(48)) return -1;
			goto label_47;
		}
		break;
	case 370: label_370: 
	// using  symbolic fact q29(v_q29_1) with literal 1 in rule 
	// 193: q30(A) :- q29(A).
	// resulting in symbolic fact q30(v_q30_1)

		//recursive
		if(!int_value_stack.push(v_q30_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2396)) return -1;
		v_q30_1 = v_q29_1;
		if(q30_set.insert(v_q30_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(407)) return -1;
		if(!backtrack_stack.push(406)) return -1;
		goto label_371;
		}
		break;
	case 2396:
	// Label to restore values of label 370
		v_q30_1 = int_value_stack.pop();
		break;
	case 371: label_371: 
	// using  symbolic fact q30(v_q30_1) with literal 1 in rule 
	// 192: burgundy(A) :- q30(A), wine(A).
	// resulting in symbolic fact burgundy(v_burgundy_1)

		//recursive
		if(!int_value_stack.push(v_burgundy_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2397)) return -1;
		if (!wine_set.contains(v_q30_1)) continue;
	case 1384:
	// Continuation for label 371
			v_burgundy_1 = v_q30_1;
		if(burgundy_set.insert(v_burgundy_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1384)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(382)) return -1;
			if(!backtrack_stack.push(381)) return -1;
			if(!backtrack_stack.push(379)) return -1;
			if(!backtrack_stack.push(377)) return -1;
			if(!backtrack_stack.push(372)) return -1;
			goto label_368;
		}
		break;
	case 2397:
	// Label to restore values of label 371
		v_burgundy_1 = int_value_stack.pop();
		break;
	case 372: label_372: 
	// using  symbolic fact burgundy(v_burgundy_1) with literal 1 in rule 
	// 187: whiteburgundy(A) :- burgundy(A), whitewine(A).
	// resulting in symbolic fact whiteburgundy(v_whiteburgundy_1)

		//recursive
		if(!int_value_stack.push(v_whiteburgundy_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2398)) return -1;
		if (!whitewine_set.contains(v_burgundy_1)) continue;
	case 1385:
	// Continuation for label 372
			v_whiteburgundy_1 = v_burgundy_1;
		if(whiteburgundy_set.insert(v_whiteburgundy_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1385)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(376)) return -1;
			if(!backtrack_stack.push(375)) return -1;
			if(!backtrack_stack.push(373)) return -1;
			if(!backtrack_stack.push(367)) return -1;
			goto label_366;
		}
		break;
	case 2398:
	// Label to restore values of label 372
		v_whiteburgundy_1 = int_value_stack.pop();
		break;
	case 373: label_373: 
	// using  symbolic fact whiteburgundy(v_whiteburgundy_1) with literal 1 in rule 
	// 299: whitewine(A) :- whiteburgundy(A).
	// resulting in symbolic fact whitewine(v_whitewine_1)

		//recursive
		if(!int_value_stack.push(v_whitewine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2399)) return -1;
		v_whitewine_1 = v_whiteburgundy_1;
		if(whitewine_set.insert(v_whitewine_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(560)) return -1;
		if(!backtrack_stack.push(491)) return -1;
		if(!backtrack_stack.push(374)) return -1;
		if(!backtrack_stack.push(235)) return -1;
		if(!backtrack_stack.push(113)) return -1;
		if(!backtrack_stack.push(112)) return -1;
		if(!backtrack_stack.push(104)) return -1;
		if(!backtrack_stack.push(103)) return -1;
		if(!backtrack_stack.push(101)) return -1;
		if(!backtrack_stack.push(98)) return -1;
		goto label_97;
		}
		break;
	case 2399:
	// Label to restore values of label 373
		v_whitewine_1 = int_value_stack.pop();
		break;
	case 374: label_374: 
	// using  symbolic fact whitewine(v_whitewine_1) with literal 2 in rule 
	// 187: whiteburgundy(A) :- burgundy(A), whitewine(A).
	// resulting in symbolic fact whiteburgundy(v_whiteburgundy_1)

		//recursive
		if(!int_value_stack.push(v_whiteburgundy_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2400)) return -1;
		if (!burgundy_set.contains(v_whitewine_1)) continue;
	case 1387:
	// Continuation for label 374
			v_whiteburgundy_1 = v_whitewine_1;
		if(whiteburgundy_set.insert(v_whiteburgundy_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1387)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(376)) return -1;
			if(!backtrack_stack.push(375)) return -1;
			if(!backtrack_stack.push(373)) return -1;
			if(!backtrack_stack.push(367)) return -1;
			goto label_366;
		}
		break;
	case 2400:
	// Label to restore values of label 374
		v_whiteburgundy_1 = int_value_stack.pop();
		break;
	case 375: label_375: 
	// using  symbolic fact whiteburgundy(v_whiteburgundy_1) with literal 1 in rule 
	// 300: whiteburgundy(A) :- whiteburgundy(B), kaon2equal(A,B).
	// resulting in symbolic fact whiteburgundy(v_whiteburgundy_1)

		//recursive
		c300_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_whiteburgundy_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2401)) return -1;
		c300_2_fb.open(v_whiteburgundy_1);
	case 1388:
	// Continuation for label 375
		while (c300_2_fb.fetch()) {
			v_whiteburgundy_1 = c300_2_fb.c2();
		if(whiteburgundy_set.insert(v_whiteburgundy_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1388)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(376)) return -1;
			if(!backtrack_stack.push(375)) return -1;
			if(!backtrack_stack.push(373)) return -1;
			if(!backtrack_stack.push(367)) return -1;
			goto label_366;
		}
		}
		break;
	case 2401:
	// Label to restore values of label 375
		v_whiteburgundy_1 = int_value_stack.pop();
		c300_2_fb.pop(&cursor_stack);
		break;
	case 376: label_376: 
	// using  symbolic fact whiteburgundy(v_whiteburgundy_1) with literal 1 in rule 
	// 773: q3(A) :- whiteburgundy(A).
	// resulting in symbolic fact q3(v_q3_1)

		v_q3_1 = v_whiteburgundy_1;
		if(q3_set.insert(v_q3_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(143)) return -1;
		goto label_123;
		}
		break;
	case 377: label_377: 
	// using  symbolic fact burgundy(v_burgundy_1) with literal 1 in rule 
	// 191: redburgundy(A) :- burgundy(A), redwine(A).
	// resulting in symbolic fact redburgundy(v_redburgundy_1)

		//recursive
		if(!int_value_stack.push(v_redburgundy_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2403)) return -1;
		if (!redwine_set.contains(v_burgundy_1)) continue;
	case 1390:
	// Continuation for label 377
			v_redburgundy_1 = v_burgundy_1;
		if(redburgundy_set.insert(v_redburgundy_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1390)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(643)) return -1;
			if(!backtrack_stack.push(405)) return -1;
			if(!backtrack_stack.push(386)) return -1;
			if(!backtrack_stack.push(385)) return -1;
			if(!backtrack_stack.push(383)) return -1;
			goto label_378;
		}
		break;
	case 2403:
	// Label to restore values of label 377
		v_redburgundy_1 = int_value_stack.pop();
		break;
	case 378: label_378: 
	// using  symbolic fact redburgundy(v_redburgundy_1) with literal 1 in rule 
	// 188: burgundy(A) :- redburgundy(A).
	// resulting in symbolic fact burgundy(v_burgundy_1)

		//recursive
		if(!int_value_stack.push(v_burgundy_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2404)) return -1;
		v_burgundy_1 = v_redburgundy_1;
		if(burgundy_set.insert(v_burgundy_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(382)) return -1;
		if(!backtrack_stack.push(381)) return -1;
		if(!backtrack_stack.push(379)) return -1;
		if(!backtrack_stack.push(377)) return -1;
		if(!backtrack_stack.push(372)) return -1;
		goto label_368;
		}
		break;
	case 2404:
	// Label to restore values of label 378
		v_burgundy_1 = int_value_stack.pop();
		break;
	case 379: label_379: 
	// using  symbolic fact burgundy(v_burgundy_1) with literal 1 in rule 
	// 475: wine(A) :- burgundy(A).
	// resulting in symbolic fact wine(v_wine_1)

		//recursive
		if(!int_value_stack.push(v_wine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2405)) return -1;
		v_wine_1 = v_burgundy_1;
		if(wine_set.insert(v_wine_1)) {
		if(!wine_f.insert(v_wine_1)) return -1;
		// Labels for next rules
		if(!backtrack_stack.push(1007)) return -1;
		if(!backtrack_stack.push(989)) return -1;
		if(!backtrack_stack.push(954)) return -1;
		if(!backtrack_stack.push(938)) return -1;
		if(!backtrack_stack.push(911)) return -1;
		if(!backtrack_stack.push(903)) return -1;
		if(!backtrack_stack.push(902)) return -1;
		if(!backtrack_stack.push(901)) return -1;
		if(!backtrack_stack.push(849)) return -1;
		if(!backtrack_stack.push(848)) return -1;
		if(!backtrack_stack.push(821)) return -1;
		if(!backtrack_stack.push(820)) return -1;
		if(!backtrack_stack.push(737)) return -1;
		if(!backtrack_stack.push(712)) return -1;
		if(!backtrack_stack.push(674)) return -1;
		if(!backtrack_stack.push(672)) return -1;
		if(!backtrack_stack.push(583)) return -1;
		if(!backtrack_stack.push(575)) return -1;
		if(!backtrack_stack.push(563)) return -1;
		if(!backtrack_stack.push(551)) return -1;
		if(!backtrack_stack.push(549)) return -1;
		if(!backtrack_stack.push(483)) return -1;
		if(!backtrack_stack.push(473)) return -1;
		if(!backtrack_stack.push(470)) return -1;
		if(!backtrack_stack.push(430)) return -1;
		if(!backtrack_stack.push(401)) return -1;
		if(!backtrack_stack.push(399)) return -1;
		if(!backtrack_stack.push(380)) return -1;
		if(!backtrack_stack.push(364)) return -1;
		if(!backtrack_stack.push(363)) return -1;
		if(!backtrack_stack.push(329)) return -1;
		if(!backtrack_stack.push(328)) return -1;
		if(!backtrack_stack.push(327)) return -1;
		if(!backtrack_stack.push(295)) return -1;
		if(!backtrack_stack.push(294)) return -1;
		if(!backtrack_stack.push(216)) return -1;
		if(!backtrack_stack.push(210)) return -1;
		if(!backtrack_stack.push(192)) return -1;
		if(!backtrack_stack.push(180)) return -1;
		if(!backtrack_stack.push(179)) return -1;
		if(!backtrack_stack.push(178)) return -1;
		if(!backtrack_stack.push(154)) return -1;
		if(!backtrack_stack.push(153)) return -1;
		if(!backtrack_stack.push(100)) return -1;
		if(!backtrack_stack.push(76)) return -1;
		if(!backtrack_stack.push(75)) return -1;
		if(!backtrack_stack.push(67)) return -1;
		if(!backtrack_stack.push(63)) return -1;
		if(!backtrack_stack.push(53)) return -1;
		if(!backtrack_stack.push(46)) return -1;
		if(!backtrack_stack.push(19)) return -1;
		goto label_16;
		}
		break;
	case 2405:
	// Label to restore values of label 379
		v_wine_1 = int_value_stack.pop();
		break;
	case 380: label_380: 
	// using  symbolic fact wine(v_wine_1) with literal 2 in rule 
	// 192: burgundy(A) :- q30(A), wine(A).
	// resulting in symbolic fact burgundy(v_burgundy_1)

		//recursive
		if(!int_value_stack.push(v_burgundy_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2406)) return -1;
		if (!q30_set.contains(v_wine_1)) continue;
	case 1393:
	// Continuation for label 380
			v_burgundy_1 = v_wine_1;
		if(burgundy_set.insert(v_burgundy_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1393)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(382)) return -1;
			if(!backtrack_stack.push(381)) return -1;
			if(!backtrack_stack.push(379)) return -1;
			if(!backtrack_stack.push(377)) return -1;
			if(!backtrack_stack.push(372)) return -1;
			goto label_368;
		}
		break;
	case 2406:
	// Label to restore values of label 380
		v_burgundy_1 = int_value_stack.pop();
		break;
	case 381: label_381: 
	// using  symbolic fact burgundy(v_burgundy_1) with literal 1 in rule 
	// 476: burgundy(A) :- burgundy(B), kaon2equal(A,B).
	// resulting in symbolic fact burgundy(v_burgundy_1)

		//recursive
		c476_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_burgundy_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2407)) return -1;
		c476_2_fb.open(v_burgundy_1);
	case 1394:
	// Continuation for label 381
		while (c476_2_fb.fetch()) {
			v_burgundy_1 = c476_2_fb.c2();
		if(burgundy_set.insert(v_burgundy_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1394)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(382)) return -1;
			if(!backtrack_stack.push(381)) return -1;
			if(!backtrack_stack.push(379)) return -1;
			if(!backtrack_stack.push(377)) return -1;
			if(!backtrack_stack.push(372)) return -1;
			goto label_368;
		}
		}
		break;
	case 2407:
	// Label to restore values of label 381
		v_burgundy_1 = int_value_stack.pop();
		c476_2_fb.pop(&cursor_stack);
		break;
	case 382: label_382: 
	// using  symbolic fact burgundy(v_burgundy_1) with literal 1 in rule 
	// 637: q31(A) :- burgundy(A).
	// resulting in symbolic fact q31(v_q31_1)

		v_q31_1 = v_burgundy_1;
		if(q31_set.insert(v_q31_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(221)) return -1;
		if(!backtrack_stack.push(220)) return -1;
		if(!backtrack_stack.push(215)) return -1;
		if(!backtrack_stack.push(213)) return -1;
		goto label_201;
		}
		break;
	case 383: label_383: 
	// using  symbolic fact redburgundy(v_redburgundy_1) with literal 1 in rule 
	// 230: redwine(A) :- redburgundy(A).
	// resulting in symbolic fact redwine(v_redwine_1)

		//recursive
		if(!int_value_stack.push(v_redwine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2409)) return -1;
		v_redwine_1 = v_redburgundy_1;
		if(redwine_set.insert(v_redwine_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(569)) return -1;
		if(!backtrack_stack.push(384)) return -1;
		if(!backtrack_stack.push(205)) return -1;
		if(!backtrack_stack.push(199)) return -1;
		if(!backtrack_stack.push(193)) return -1;
		goto label_191;
		}
		break;
	case 2409:
	// Label to restore values of label 383
		v_redwine_1 = int_value_stack.pop();
		break;
	case 384: label_384: 
	// using  symbolic fact redwine(v_redwine_1) with literal 2 in rule 
	// 191: redburgundy(A) :- burgundy(A), redwine(A).
	// resulting in symbolic fact redburgundy(v_redburgundy_1)

		//recursive
		if(!int_value_stack.push(v_redburgundy_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2410)) return -1;
		if (!burgundy_set.contains(v_redwine_1)) continue;
	case 1397:
	// Continuation for label 384
			v_redburgundy_1 = v_redwine_1;
		if(redburgundy_set.insert(v_redburgundy_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1397)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(643)) return -1;
			if(!backtrack_stack.push(405)) return -1;
			if(!backtrack_stack.push(386)) return -1;
			if(!backtrack_stack.push(385)) return -1;
			if(!backtrack_stack.push(383)) return -1;
			goto label_378;
		}
		break;
	case 2410:
	// Label to restore values of label 384
		v_redburgundy_1 = int_value_stack.pop();
		break;
	case 385: label_385: 
	// using  symbolic fact redburgundy(v_redburgundy_1) with literal 1 in rule 
	// 231: redburgundy(A) :- redburgundy(B), kaon2equal(A,B).
	// resulting in symbolic fact redburgundy(v_redburgundy_1)

		//recursive
		c231_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_redburgundy_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2411)) return -1;
		c231_2_fb.open(v_redburgundy_1);
	case 1398:
	// Continuation for label 385
		while (c231_2_fb.fetch()) {
			v_redburgundy_1 = c231_2_fb.c2();
		if(redburgundy_set.insert(v_redburgundy_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1398)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(643)) return -1;
			if(!backtrack_stack.push(405)) return -1;
			if(!backtrack_stack.push(386)) return -1;
			if(!backtrack_stack.push(385)) return -1;
			if(!backtrack_stack.push(383)) return -1;
			goto label_378;
		}
		}
		break;
	case 2411:
	// Label to restore values of label 385
		v_redburgundy_1 = int_value_stack.pop();
		c231_2_fb.pop(&cursor_stack);
		break;
	case 386: label_386: 
	// using  symbolic fact redburgundy(v_redburgundy_1) with literal 1 in rule 
	// 354: q72(A) :- redburgundy(A).
	// resulting in symbolic fact q72(v_q72_1)

		v_q72_1 = v_redburgundy_1;
		if(q72_set.insert(v_q72_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(394)) return -1;
		if(!backtrack_stack.push(393)) return -1;
		if(!backtrack_stack.push(392)) return -1;
		if(!backtrack_stack.push(391)) return -1;
		if(!backtrack_stack.push(390)) return -1;
		goto label_387;
		}
		break;
	case 387: label_387: 
	// using  symbolic fact q72(v_q72_1) with literal 1 in rule 
	// 323: pinotnoir(A) :- q72(A).
	// resulting in symbolic fact pinotnoir(v_pinotnoir_1)

		//recursive
		if(!int_value_stack.push(v_pinotnoir_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2413)) return -1;
		v_pinotnoir_1 = v_q72_1;
		if(pinotnoir_set.insert(v_pinotnoir_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(400)) return -1;
		if(!backtrack_stack.push(398)) return -1;
		if(!backtrack_stack.push(397)) return -1;
		if(!backtrack_stack.push(389)) return -1;
		goto label_388;
		}
		break;
	case 2413:
	// Label to restore values of label 387
		v_pinotnoir_1 = int_value_stack.pop();
		break;
	case 388: label_388: 
	// using  symbolic fact pinotnoir(v_pinotnoir_1) with literal 1 in rule 
	// 321: q0(A) :- pinotnoir(A).
	// resulting in symbolic fact q0(v_q0_1)

		v_q0_1 = v_pinotnoir_1;
		if(q0_set.insert(v_q0_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(198)) return -1;
		if(!backtrack_stack.push(194)) return -1;
		if(!backtrack_stack.push(189)) return -1;
		if(!backtrack_stack.push(188)) return -1;
		goto label_187;
		}
		break;
	case 389: label_389: 
	// using  symbolic fact pinotnoir(v_pinotnoir_1) with literal 1 in rule 
	// 324: q72(A) :- pinotnoir(A).
	// resulting in symbolic fact q72(v_q72_1)

		//recursive
		if(!int_value_stack.push(v_q72_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2415)) return -1;
		v_q72_1 = v_pinotnoir_1;
		if(q72_set.insert(v_q72_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(394)) return -1;
		if(!backtrack_stack.push(393)) return -1;
		if(!backtrack_stack.push(392)) return -1;
		if(!backtrack_stack.push(391)) return -1;
		if(!backtrack_stack.push(390)) return -1;
		goto label_387;
		}
		break;
	case 2415:
	// Label to restore values of label 389
		v_q72_1 = int_value_stack.pop();
		break;
	case 390: label_390: 
	// using  symbolic fact q72(v_q72_1) with literal 1 in rule 
	// 353: madefromgrape(A,A) :- q72(A), kaon2namedobjects(A).
	// resulting in symbolic fact madefromgrape(v_madefromgrape_1,v_madefromgrape_2)

		if (!kaon2namedobjects_b.contains(v_q72_1)) continue;
	case 1403:
	// Continuation for label 390
			v_madefromgrape_1 = v_q72_1;
			v_madefromgrape_2 = v_q72_1;
		if(madefromgrape_set.insert(v_madefromgrape_1, v_madefromgrape_2)) {
			if(!madefromgrape_bf.insert(v_madefromgrape_1, v_madefromgrape_2)) return -1;
			if(!madefromgrape_fb.insert(v_madefromgrape_2, v_madefromgrape_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1403)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(860)) return -1;
			if(!backtrack_stack.push(859)) return -1;
			if(!backtrack_stack.push(845)) return -1;
			if(!backtrack_stack.push(844)) return -1;
			if(!backtrack_stack.push(829)) return -1;
			if(!backtrack_stack.push(828)) return -1;
			if(!backtrack_stack.push(813)) return -1;
			if(!backtrack_stack.push(812)) return -1;
			if(!backtrack_stack.push(677)) return -1;
			if(!backtrack_stack.push(676)) return -1;
			if(!backtrack_stack.push(617)) return -1;
			if(!backtrack_stack.push(616)) return -1;
			if(!backtrack_stack.push(609)) return -1;
			if(!backtrack_stack.push(554)) return -1;
			if(!backtrack_stack.push(553)) return -1;
			if(!backtrack_stack.push(476)) return -1;
			if(!backtrack_stack.push(475)) return -1;
			if(!backtrack_stack.push(404)) return -1;
			if(!backtrack_stack.push(403)) return -1;
			if(!backtrack_stack.push(362)) return -1;
			if(!backtrack_stack.push(361)) return -1;
			if(!backtrack_stack.push(343)) return -1;
			if(!backtrack_stack.push(342)) return -1;
			if(!backtrack_stack.push(300)) return -1;
			if(!backtrack_stack.push(170)) return -1;
			if(!backtrack_stack.push(169)) return -1;
			if(!backtrack_stack.push(151)) return -1;
			if(!backtrack_stack.push(150)) return -1;
			if(!backtrack_stack.push(144)) return -1;
			if(!backtrack_stack.push(132)) return -1;
			if(!backtrack_stack.push(71)) return -1;
			if(!backtrack_stack.push(70)) return -1;
			if(!backtrack_stack.push(66)) return -1;
			goto label_64;
		}
		break;
	case 391: label_391: 
	// using  symbolic fact q72(v_q72_1) with literal 1 in rule 
	// 355: q72(A) :- q72(B), kaon2equal(A,B).
	// resulting in symbolic fact q72(v_q72_1)

		//recursive
		c355_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q72_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2417)) return -1;
		c355_2_fb.open(v_q72_1);
	case 1404:
	// Continuation for label 391
		while (c355_2_fb.fetch()) {
			v_q72_1 = c355_2_fb.c2();
		if(q72_set.insert(v_q72_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1404)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(394)) return -1;
			if(!backtrack_stack.push(393)) return -1;
			if(!backtrack_stack.push(392)) return -1;
			if(!backtrack_stack.push(391)) return -1;
			if(!backtrack_stack.push(390)) return -1;
			goto label_387;
		}
		}
		break;
	case 2417:
	// Label to restore values of label 391
		v_q72_1 = int_value_stack.pop();
		c355_2_fb.pop(&cursor_stack);
		break;
	case 392: label_392: 
	// using  symbolic fact q72(v_q72_1) with literal 1 in rule 
	// 478: wine(A) :- q72(A).
	// resulting in symbolic fact wine(v_wine_1)

		v_wine_1 = v_q72_1;
		if(wine_set.insert(v_wine_1)) {
		if(!wine_f.insert(v_wine_1)) return -1;
		// Labels for next rules
		if(!backtrack_stack.push(1007)) return -1;
		if(!backtrack_stack.push(989)) return -1;
		if(!backtrack_stack.push(954)) return -1;
		if(!backtrack_stack.push(938)) return -1;
		if(!backtrack_stack.push(911)) return -1;
		if(!backtrack_stack.push(903)) return -1;
		if(!backtrack_stack.push(902)) return -1;
		if(!backtrack_stack.push(901)) return -1;
		if(!backtrack_stack.push(849)) return -1;
		if(!backtrack_stack.push(848)) return -1;
		if(!backtrack_stack.push(821)) return -1;
		if(!backtrack_stack.push(820)) return -1;
		if(!backtrack_stack.push(737)) return -1;
		if(!backtrack_stack.push(712)) return -1;
		if(!backtrack_stack.push(674)) return -1;
		if(!backtrack_stack.push(672)) return -1;
		if(!backtrack_stack.push(583)) return -1;
		if(!backtrack_stack.push(575)) return -1;
		if(!backtrack_stack.push(563)) return -1;
		if(!backtrack_stack.push(551)) return -1;
		if(!backtrack_stack.push(549)) return -1;
		if(!backtrack_stack.push(483)) return -1;
		if(!backtrack_stack.push(473)) return -1;
		if(!backtrack_stack.push(470)) return -1;
		if(!backtrack_stack.push(430)) return -1;
		if(!backtrack_stack.push(401)) return -1;
		if(!backtrack_stack.push(399)) return -1;
		if(!backtrack_stack.push(380)) return -1;
		if(!backtrack_stack.push(364)) return -1;
		if(!backtrack_stack.push(363)) return -1;
		if(!backtrack_stack.push(329)) return -1;
		if(!backtrack_stack.push(328)) return -1;
		if(!backtrack_stack.push(327)) return -1;
		if(!backtrack_stack.push(295)) return -1;
		if(!backtrack_stack.push(294)) return -1;
		if(!backtrack_stack.push(216)) return -1;
		if(!backtrack_stack.push(210)) return -1;
		if(!backtrack_stack.push(192)) return -1;
		if(!backtrack_stack.push(180)) return -1;
		if(!backtrack_stack.push(179)) return -1;
		if(!backtrack_stack.push(178)) return -1;
		if(!backtrack_stack.push(154)) return -1;
		if(!backtrack_stack.push(153)) return -1;
		if(!backtrack_stack.push(100)) return -1;
		if(!backtrack_stack.push(76)) return -1;
		if(!backtrack_stack.push(75)) return -1;
		if(!backtrack_stack.push(67)) return -1;
		if(!backtrack_stack.push(63)) return -1;
		if(!backtrack_stack.push(53)) return -1;
		if(!backtrack_stack.push(46)) return -1;
		if(!backtrack_stack.push(19)) return -1;
		goto label_16;
		}
		break;
	case 393: label_393: 
	// using  symbolic fact q72(v_q72_1) with literal 1 in rule 
	// 500: q5(A) :- q72(A).
	// resulting in symbolic fact q5(v_q5_1)

		v_q5_1 = v_q72_1;
		if(q5_set.insert(v_q5_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(136)) return -1;
		if(!backtrack_stack.push(135)) return -1;
		goto label_131;
		}
		break;
	case 394: label_394: 
	// using  symbolic fact q72(v_q72_1) with literal 1 in rule 
	// 564: ot____nom39(A) :- q72(A), kaon2namedobjects(A).
	// resulting in symbolic fact ot____nom39(v_ot____nom39_1)

		//recursive
		if(!int_value_stack.push(v_ot____nom39_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2420)) return -1;
		if (!kaon2namedobjects_b.contains(v_q72_1)) continue;
	case 1407:
	// Continuation for label 394
			v_ot____nom39_1 = v_q72_1;
		if(ot____nom39_set.insert(v_ot____nom39_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1407)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(402)) return -1;
			if(!backtrack_stack.push(396)) return -1;
			goto label_395;
		}
		break;
	case 2420:
	// Label to restore values of label 394
		v_ot____nom39_1 = int_value_stack.pop();
		break;
	case 395: label_395: 
	// using  symbolic fact ot____nom39(v_ot____nom39_1) with literal 1 in rule 
	// 565: ot____nom39(A) :- ot____nom39(B), kaon2equal(A,B).
	// resulting in symbolic fact ot____nom39(v_ot____nom39_1)

		//recursive
		c565_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_ot____nom39_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2421)) return -1;
		c565_2_fb.open(v_ot____nom39_1);
	case 1408:
	// Continuation for label 395
		while (c565_2_fb.fetch()) {
			v_ot____nom39_1 = c565_2_fb.c2();
		if(ot____nom39_set.insert(v_ot____nom39_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1408)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(402)) return -1;
			if(!backtrack_stack.push(396)) return -1;
			goto label_395;
		}
		}
		break;
	case 2421:
	// Label to restore values of label 395
		v_ot____nom39_1 = int_value_stack.pop();
		c565_2_fb.pop(&cursor_stack);
		break;
	case 396: label_396: 
	// using  symbolic fact ot____nom39(v_ot____nom39_1) with literal 3 in rule 
	// 326: pinotnoir(A) :- wine(A), madefromgrape(A,B), ot____nom39(B).
	// resulting in symbolic fact pinotnoir(v_pinotnoir_1)

		//recursive
		c326_1_f.push(&cursor_stack);
		if(!int_value_stack.push(v_pinotnoir_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2422)) return -1;
		c326_1_f.open();
		while (c326_1_f.fetch()) {
			if (!madefromgrape_set.contains(c326_1_f.c1(), v_ot____nom39_1)) continue;
	case 1409:
	// Continuation for label 396
				v_pinotnoir_1 = c326_1_f.c1();
		if(pinotnoir_set.insert(v_pinotnoir_1)) {
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1409)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(400)) return -1;
				if(!backtrack_stack.push(398)) return -1;
				if(!backtrack_stack.push(397)) return -1;
				if(!backtrack_stack.push(389)) return -1;
				goto label_388;
		}
		}
		break;
	case 2422:
	// Label to restore values of label 396
		v_pinotnoir_1 = int_value_stack.pop();
		c326_1_f.pop(&cursor_stack);
		break;
	case 397: label_397: 
	// using  symbolic fact pinotnoir(v_pinotnoir_1) with literal 1 in rule 
	// 325: pinotnoir(A) :- pinotnoir(B), kaon2equal(A,B).
	// resulting in symbolic fact pinotnoir(v_pinotnoir_1)

		//recursive
		c325_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_pinotnoir_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2423)) return -1;
		c325_2_fb.open(v_pinotnoir_1);
	case 1410:
	// Continuation for label 397
		while (c325_2_fb.fetch()) {
			v_pinotnoir_1 = c325_2_fb.c2();
		if(pinotnoir_set.insert(v_pinotnoir_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1410)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(400)) return -1;
			if(!backtrack_stack.push(398)) return -1;
			if(!backtrack_stack.push(397)) return -1;
			if(!backtrack_stack.push(389)) return -1;
			goto label_388;
		}
		}
		break;
	case 2423:
	// Label to restore values of label 397
		v_pinotnoir_1 = int_value_stack.pop();
		c325_2_fb.pop(&cursor_stack);
		break;
	case 398: label_398: 
	// using  symbolic fact pinotnoir(v_pinotnoir_1) with literal 1 in rule 
	// 411: wine(A) :- pinotnoir(A).
	// resulting in symbolic fact wine(v_wine_1)

		//recursive
		if(!int_value_stack.push(v_wine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2424)) return -1;
		v_wine_1 = v_pinotnoir_1;
		if(wine_set.insert(v_wine_1)) {
		if(!wine_f.insert(v_wine_1)) return -1;
		// Labels for next rules
		if(!backtrack_stack.push(1007)) return -1;
		if(!backtrack_stack.push(989)) return -1;
		if(!backtrack_stack.push(954)) return -1;
		if(!backtrack_stack.push(938)) return -1;
		if(!backtrack_stack.push(911)) return -1;
		if(!backtrack_stack.push(903)) return -1;
		if(!backtrack_stack.push(902)) return -1;
		if(!backtrack_stack.push(901)) return -1;
		if(!backtrack_stack.push(849)) return -1;
		if(!backtrack_stack.push(848)) return -1;
		if(!backtrack_stack.push(821)) return -1;
		if(!backtrack_stack.push(820)) return -1;
		if(!backtrack_stack.push(737)) return -1;
		if(!backtrack_stack.push(712)) return -1;
		if(!backtrack_stack.push(674)) return -1;
		if(!backtrack_stack.push(672)) return -1;
		if(!backtrack_stack.push(583)) return -1;
		if(!backtrack_stack.push(575)) return -1;
		if(!backtrack_stack.push(563)) return -1;
		if(!backtrack_stack.push(551)) return -1;
		if(!backtrack_stack.push(549)) return -1;
		if(!backtrack_stack.push(483)) return -1;
		if(!backtrack_stack.push(473)) return -1;
		if(!backtrack_stack.push(470)) return -1;
		if(!backtrack_stack.push(430)) return -1;
		if(!backtrack_stack.push(401)) return -1;
		if(!backtrack_stack.push(399)) return -1;
		if(!backtrack_stack.push(380)) return -1;
		if(!backtrack_stack.push(364)) return -1;
		if(!backtrack_stack.push(363)) return -1;
		if(!backtrack_stack.push(329)) return -1;
		if(!backtrack_stack.push(328)) return -1;
		if(!backtrack_stack.push(327)) return -1;
		if(!backtrack_stack.push(295)) return -1;
		if(!backtrack_stack.push(294)) return -1;
		if(!backtrack_stack.push(216)) return -1;
		if(!backtrack_stack.push(210)) return -1;
		if(!backtrack_stack.push(192)) return -1;
		if(!backtrack_stack.push(180)) return -1;
		if(!backtrack_stack.push(179)) return -1;
		if(!backtrack_stack.push(178)) return -1;
		if(!backtrack_stack.push(154)) return -1;
		if(!backtrack_stack.push(153)) return -1;
		if(!backtrack_stack.push(100)) return -1;
		if(!backtrack_stack.push(76)) return -1;
		if(!backtrack_stack.push(75)) return -1;
		if(!backtrack_stack.push(67)) return -1;
		if(!backtrack_stack.push(63)) return -1;
		if(!backtrack_stack.push(53)) return -1;
		if(!backtrack_stack.push(46)) return -1;
		if(!backtrack_stack.push(19)) return -1;
		goto label_16;
		}
		break;
	case 2424:
	// Label to restore values of label 398
		v_wine_1 = int_value_stack.pop();
		break;
	case 399: label_399: 
	// using  symbolic fact wine(v_wine_1) with literal 1 in rule 
	// 326: pinotnoir(A) :- wine(A), madefromgrape(A,B), ot____nom39(B).
	// resulting in symbolic fact pinotnoir(v_pinotnoir_1)

		//recursive
		c326_2_bf.push(&cursor_stack);
		if(!int_value_stack.push(v_pinotnoir_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2425)) return -1;
		c326_2_bf.open(v_wine_1);
		while (c326_2_bf.fetch()) {
			if (!ot____nom39_set.contains(c326_2_bf.c2())) continue;
	case 1412:
	// Continuation for label 399
				v_pinotnoir_1 = v_wine_1;
		if(pinotnoir_set.insert(v_pinotnoir_1)) {
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1412)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(400)) return -1;
				if(!backtrack_stack.push(398)) return -1;
				if(!backtrack_stack.push(397)) return -1;
				if(!backtrack_stack.push(389)) return -1;
				goto label_388;
		}
		}
		break;
	case 2425:
	// Label to restore values of label 399
		v_pinotnoir_1 = int_value_stack.pop();
		c326_2_bf.pop(&cursor_stack);
		break;
	case 400: label_400: 
	// using  symbolic fact pinotnoir(v_pinotnoir_1) with literal 1 in rule 
	// 780: q3(A) :- pinotnoir(A).
	// resulting in symbolic fact q3(v_q3_1)

		v_q3_1 = v_pinotnoir_1;
		if(q3_set.insert(v_q3_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(143)) return -1;
		goto label_123;
		}
		break;
	case 401: label_401: 
	// using  symbolic fact wine(v_wine_1) with literal 1 in rule 
	// 562: q5(A) :- wine(A), madefromgrape(A,B), ot____nom39(B).
	// resulting in symbolic fact q5(v_q5_1)

		c562_2_bf.open(v_wine_1);
		while (c562_2_bf.fetch()) {
			if (!ot____nom39_set.contains(c562_2_bf.c2())) continue;
	case 1414:
	// Continuation for label 401
				v_q5_1 = v_wine_1;
		if(q5_set.insert(v_q5_1)) {
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1414)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(136)) return -1;
				if(!backtrack_stack.push(135)) return -1;
				goto label_131;
		}
		}
		break;
	case 402: label_402: 
	// using  symbolic fact ot____nom39(v_ot____nom39_1) with literal 3 in rule 
	// 562: q5(A) :- wine(A), madefromgrape(A,B), ot____nom39(B).
	// resulting in symbolic fact q5(v_q5_1)

		c562_1_f.open();
		while (c562_1_f.fetch()) {
			if (!madefromgrape_set.contains(c562_1_f.c1(), v_ot____nom39_1)) continue;
	case 1415:
	// Continuation for label 402
				v_q5_1 = c562_1_f.c1();
		if(q5_set.insert(v_q5_1)) {
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1415)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(136)) return -1;
				if(!backtrack_stack.push(135)) return -1;
				goto label_131;
		}
		}
		break;
	case 403: label_403: 
	// using  symbolic fact madefromgrape(v_madefromgrape_1,v_madefromgrape_2) with literal 2 in rule 
	// 326: pinotnoir(A) :- wine(A), madefromgrape(A,B), ot____nom39(B).
	// resulting in symbolic fact pinotnoir(v_pinotnoir_1)

		if (!wine_set.contains(v_madefromgrape_1)) continue;
			if (!ot____nom39_set.contains(v_madefromgrape_2)) continue;
	case 1416:
	// Continuation for label 403
				v_pinotnoir_1 = v_madefromgrape_1;
		if(pinotnoir_set.insert(v_pinotnoir_1)) {
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1416)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(400)) return -1;
				if(!backtrack_stack.push(398)) return -1;
				if(!backtrack_stack.push(397)) return -1;
				if(!backtrack_stack.push(389)) return -1;
				goto label_388;
		}
		break;
	case 404: label_404: 
	// using  symbolic fact madefromgrape(v_madefromgrape_1,v_madefromgrape_2) with literal 2 in rule 
	// 562: q5(A) :- wine(A), madefromgrape(A,B), ot____nom39(B).
	// resulting in symbolic fact q5(v_q5_1)

		if (!wine_set.contains(v_madefromgrape_1)) continue;
			if (!ot____nom39_set.contains(v_madefromgrape_2)) continue;
	case 1417:
	// Continuation for label 404
				v_q5_1 = v_madefromgrape_1;
		if(q5_set.insert(v_q5_1)) {
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1417)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(136)) return -1;
				if(!backtrack_stack.push(135)) return -1;
				goto label_131;
		}
		break;
	case 405: label_405: 
	// using  symbolic fact redburgundy(v_redburgundy_1) with literal 1 in rule 
	// 782: q3(A) :- redburgundy(A).
	// resulting in symbolic fact q3(v_q3_1)

		v_q3_1 = v_redburgundy_1;
		if(q3_set.insert(v_q3_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(143)) return -1;
		goto label_123;
		}
		break;
	case 406: label_406: 
	// using  symbolic fact q30(v_q30_1) with literal 1 in rule 
	// 679: q30(A) :- q30(B), kaon2equal(A,B).
	// resulting in symbolic fact q30(v_q30_1)

		//recursive
		c679_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q30_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2432)) return -1;
		c679_2_fb.open(v_q30_1);
	case 1419:
	// Continuation for label 406
		while (c679_2_fb.fetch()) {
			v_q30_1 = c679_2_fb.c2();
		if(q30_set.insert(v_q30_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1419)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(407)) return -1;
			if(!backtrack_stack.push(406)) return -1;
			goto label_371;
		}
		}
		break;
	case 2432:
	// Label to restore values of label 406
		v_q30_1 = int_value_stack.pop();
		c679_2_fb.pop(&cursor_stack);
		break;
	case 407: label_407: 
	// using  symbolic fact q30(v_q30_1) with literal 2 in rule 
	// 194: q29(A) :- locatedin(A,B), q30(B).
	// resulting in symbolic fact q29(v_q29_1)

		//recursive
		c194_1_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q29_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2433)) return -1;
		c194_1_fb.open(v_q30_1);
	case 1420:
	// Continuation for label 407
		while (c194_1_fb.fetch()) {
			v_q29_1 = c194_1_fb.c2();
		if(q29_set.insert(v_q29_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1420)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(411)) return -1;
			if(!backtrack_stack.push(408)) return -1;
			if(!backtrack_stack.push(370)) return -1;
			goto label_369;
		}
		}
		break;
	case 2433:
	// Label to restore values of label 407
		v_q29_1 = int_value_stack.pop();
		c194_1_fb.pop(&cursor_stack);
		break;
	case 408: label_408: 
	// using  symbolic fact q29(v_q29_1) with literal 1 in rule 
	// 676: ot____nom44(A) :- q29(A), kaon2namedobjects(A).
	// resulting in symbolic fact ot____nom44(v_ot____nom44_1)

		//recursive
		if(!int_value_stack.push(v_ot____nom44_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2434)) return -1;
		if (!kaon2namedobjects_b.contains(v_q29_1)) continue;
	case 1421:
	// Continuation for label 408
			v_ot____nom44_1 = v_q29_1;
		if(ot____nom44_set.insert(v_ot____nom44_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1421)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(410)) return -1;
			goto label_409;
		}
		break;
	case 2434:
	// Label to restore values of label 408
		v_ot____nom44_1 = int_value_stack.pop();
		break;
	case 409: label_409: 
	// using  symbolic fact ot____nom44(v_ot____nom44_1) with literal 1 in rule 
	// 678: ot____nom44(A) :- ot____nom44(B), kaon2equal(A,B).
	// resulting in symbolic fact ot____nom44(v_ot____nom44_1)

		//recursive
		c678_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_ot____nom44_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2435)) return -1;
		c678_2_fb.open(v_ot____nom44_1);
	case 1422:
	// Continuation for label 409
		while (c678_2_fb.fetch()) {
			v_ot____nom44_1 = c678_2_fb.c2();
		if(ot____nom44_set.insert(v_ot____nom44_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1422)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(410)) return -1;
			goto label_409;
		}
		}
		break;
	case 2435:
	// Label to restore values of label 409
		v_ot____nom44_1 = int_value_stack.pop();
		c678_2_fb.pop(&cursor_stack);
		break;
	case 410: label_410: 
	// using  symbolic fact ot____nom44(v_ot____nom44_1) with literal 2 in rule 
	// 674: q30(A) :- locatedin(A,B), ot____nom44(B).
	// resulting in symbolic fact q30(v_q30_1)

		//recursive
		c674_1_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q30_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2436)) return -1;
		c674_1_fb.open(v_ot____nom44_1);
	case 1423:
	// Continuation for label 410
		while (c674_1_fb.fetch()) {
			v_q30_1 = c674_1_fb.c2();
		if(q30_set.insert(v_q30_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1423)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(407)) return -1;
			if(!backtrack_stack.push(406)) return -1;
			goto label_371;
		}
		}
		break;
	case 2436:
	// Label to restore values of label 410
		v_q30_1 = int_value_stack.pop();
		c674_1_fb.pop(&cursor_stack);
		break;
	case 411: label_411: 
	// using  symbolic fact q29(v_q29_1) with literal 1 in rule 
	// 677: q29(A) :- q29(B), kaon2equal(A,B).
	// resulting in symbolic fact q29(v_q29_1)

		//recursive
		c677_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q29_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2437)) return -1;
		c677_2_fb.open(v_q29_1);
	case 1424:
	// Continuation for label 411
		while (c677_2_fb.fetch()) {
			v_q29_1 = c677_2_fb.c2();
		if(q29_set.insert(v_q29_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1424)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(411)) return -1;
			if(!backtrack_stack.push(408)) return -1;
			if(!backtrack_stack.push(370)) return -1;
			goto label_369;
		}
		}
		break;
	case 2437:
	// Label to restore values of label 411
		v_q29_1 = int_value_stack.pop();
		c677_2_fb.pop(&cursor_stack);
		break;
	case 412:
	// applying rule 
	// 73: meursault(A) :- meursault_aux(A).
	// resulting in symbolic fact meursault(v_meursault_1)
		c73_1_f.open();
	case 1425:
	// Continuation for label 412
		while (c73_1_f.fetch()) {
			v_meursault_1 = c73_1_f.c1();
		if(meursault_set.insert(v_meursault_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1425)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(418)) return -1;
			if(!backtrack_stack.push(417)) return -1;
			if(!backtrack_stack.push(414)) return -1;
			goto label_413;
		}
		}
		break;
	case 413: label_413: 
	// using  symbolic fact meursault(v_meursault_1) with literal 1 in rule 
	// 72: whiteburgundy(A) :- meursault(A).
	// resulting in symbolic fact whiteburgundy(v_whiteburgundy_1)

		v_whiteburgundy_1 = v_meursault_1;
		if(whiteburgundy_set.insert(v_whiteburgundy_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(376)) return -1;
		if(!backtrack_stack.push(375)) return -1;
		if(!backtrack_stack.push(373)) return -1;
		if(!backtrack_stack.push(367)) return -1;
		goto label_366;
		}
		break;
	case 414: label_414: 
	// using  symbolic fact meursault(v_meursault_1) with literal 1 in rule 
	// 76: q16(A) :- meursault(A).
	// resulting in symbolic fact q16(v_q16_1)

		//recursive
		if(!int_value_stack.push(v_q16_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2440)) return -1;
		v_q16_1 = v_meursault_1;
		if(q16_set.insert(v_q16_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(438)) return -1;
		if(!backtrack_stack.push(435)) return -1;
		if(!backtrack_stack.push(433)) return -1;
		goto label_415;
		}
		break;
	case 2440:
	// Label to restore values of label 414
		v_q16_1 = int_value_stack.pop();
		break;
	case 415: label_415: 
	// using  symbolic fact q16(v_q16_1) with literal 1 in rule 
	// 75: q17(A) :- q16(A).
	// resulting in symbolic fact q17(v_q17_1)

		//recursive
		if(!int_value_stack.push(v_q17_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2441)) return -1;
		v_q17_1 = v_q16_1;
		if(q17_set.insert(v_q17_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(432)) return -1;
		if(!backtrack_stack.push(431)) return -1;
		goto label_416;
		}
		break;
	case 2441:
	// Label to restore values of label 415
		v_q17_1 = int_value_stack.pop();
		break;
	case 416: label_416: 
	// using  symbolic fact q17(v_q17_1) with literal 1 in rule 
	// 74: meursault(A) :- q17(A), whiteburgundy(A).
	// resulting in symbolic fact meursault(v_meursault_1)

		//recursive
		if(!int_value_stack.push(v_meursault_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2442)) return -1;
		if (!whiteburgundy_set.contains(v_q17_1)) continue;
	case 1429:
	// Continuation for label 416
			v_meursault_1 = v_q17_1;
		if(meursault_set.insert(v_meursault_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1429)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(418)) return -1;
			if(!backtrack_stack.push(417)) return -1;
			if(!backtrack_stack.push(414)) return -1;
			goto label_413;
		}
		break;
	case 2442:
	// Label to restore values of label 416
		v_meursault_1 = int_value_stack.pop();
		break;
	case 417: label_417: 
	// using  symbolic fact meursault(v_meursault_1) with literal 1 in rule 
	// 77: meursault(A) :- meursault(B), kaon2equal(A,B).
	// resulting in symbolic fact meursault(v_meursault_1)

		//recursive
		c77_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_meursault_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2443)) return -1;
		c77_2_fb.open(v_meursault_1);
	case 1430:
	// Continuation for label 417
		while (c77_2_fb.fetch()) {
			v_meursault_1 = c77_2_fb.c2();
		if(meursault_set.insert(v_meursault_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1430)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(418)) return -1;
			if(!backtrack_stack.push(417)) return -1;
			if(!backtrack_stack.push(414)) return -1;
			goto label_413;
		}
		}
		break;
	case 2443:
	// Label to restore values of label 417
		v_meursault_1 = int_value_stack.pop();
		c77_2_fb.pop(&cursor_stack);
		break;
	case 418: label_418: 
	// using  symbolic fact meursault(v_meursault_1) with literal 1 in rule 
	// 372: q35(A) :- meursault(A).
	// resulting in symbolic fact q35(v_q35_1)

		v_q35_1 = v_meursault_1;
		if(q35_set.insert(v_q35_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(425)) return -1;
		if(!backtrack_stack.push(424)) return -1;
		if(!backtrack_stack.push(423)) return -1;
		if(!backtrack_stack.push(422)) return -1;
		goto label_419;
		}
		break;
	case 419: label_419: 
	// using  symbolic fact q35(v_q35_1) with literal 1 in rule 
	// 367: fullbodiedwine(A) :- q35(A).
	// resulting in symbolic fact fullbodiedwine(v_fullbodiedwine_1)

		//recursive
		if(!int_value_stack.push(v_fullbodiedwine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2445)) return -1;
		v_fullbodiedwine_1 = v_q35_1;
		if(fullbodiedwine_set.insert(v_fullbodiedwine_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(428)) return -1;
		if(!backtrack_stack.push(421)) return -1;
		goto label_420;
		}
		break;
	case 2445:
	// Label to restore values of label 419
		v_fullbodiedwine_1 = int_value_stack.pop();
		break;
	case 420: label_420: 
	// using  symbolic fact fullbodiedwine(v_fullbodiedwine_1) with literal 1 in rule 
	// 366: wine(A) :- fullbodiedwine(A).
	// resulting in symbolic fact wine(v_wine_1)

		v_wine_1 = v_fullbodiedwine_1;
		if(wine_set.insert(v_wine_1)) {
		if(!wine_f.insert(v_wine_1)) return -1;
		// Labels for next rules
		if(!backtrack_stack.push(1007)) return -1;
		if(!backtrack_stack.push(989)) return -1;
		if(!backtrack_stack.push(954)) return -1;
		if(!backtrack_stack.push(938)) return -1;
		if(!backtrack_stack.push(911)) return -1;
		if(!backtrack_stack.push(903)) return -1;
		if(!backtrack_stack.push(902)) return -1;
		if(!backtrack_stack.push(901)) return -1;
		if(!backtrack_stack.push(849)) return -1;
		if(!backtrack_stack.push(848)) return -1;
		if(!backtrack_stack.push(821)) return -1;
		if(!backtrack_stack.push(820)) return -1;
		if(!backtrack_stack.push(737)) return -1;
		if(!backtrack_stack.push(712)) return -1;
		if(!backtrack_stack.push(674)) return -1;
		if(!backtrack_stack.push(672)) return -1;
		if(!backtrack_stack.push(583)) return -1;
		if(!backtrack_stack.push(575)) return -1;
		if(!backtrack_stack.push(563)) return -1;
		if(!backtrack_stack.push(551)) return -1;
		if(!backtrack_stack.push(549)) return -1;
		if(!backtrack_stack.push(483)) return -1;
		if(!backtrack_stack.push(473)) return -1;
		if(!backtrack_stack.push(470)) return -1;
		if(!backtrack_stack.push(430)) return -1;
		if(!backtrack_stack.push(401)) return -1;
		if(!backtrack_stack.push(399)) return -1;
		if(!backtrack_stack.push(380)) return -1;
		if(!backtrack_stack.push(364)) return -1;
		if(!backtrack_stack.push(363)) return -1;
		if(!backtrack_stack.push(329)) return -1;
		if(!backtrack_stack.push(328)) return -1;
		if(!backtrack_stack.push(327)) return -1;
		if(!backtrack_stack.push(295)) return -1;
		if(!backtrack_stack.push(294)) return -1;
		if(!backtrack_stack.push(216)) return -1;
		if(!backtrack_stack.push(210)) return -1;
		if(!backtrack_stack.push(192)) return -1;
		if(!backtrack_stack.push(180)) return -1;
		if(!backtrack_stack.push(179)) return -1;
		if(!backtrack_stack.push(178)) return -1;
		if(!backtrack_stack.push(154)) return -1;
		if(!backtrack_stack.push(153)) return -1;
		if(!backtrack_stack.push(100)) return -1;
		if(!backtrack_stack.push(76)) return -1;
		if(!backtrack_stack.push(75)) return -1;
		if(!backtrack_stack.push(67)) return -1;
		if(!backtrack_stack.push(63)) return -1;
		if(!backtrack_stack.push(53)) return -1;
		if(!backtrack_stack.push(46)) return -1;
		if(!backtrack_stack.push(19)) return -1;
		goto label_16;
		}
		break;
	case 421: label_421: 
	// using  symbolic fact fullbodiedwine(v_fullbodiedwine_1) with literal 1 in rule 
	// 373: q35(A) :- fullbodiedwine(A).
	// resulting in symbolic fact q35(v_q35_1)

		//recursive
		if(!int_value_stack.push(v_q35_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2447)) return -1;
		v_q35_1 = v_fullbodiedwine_1;
		if(q35_set.insert(v_q35_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(425)) return -1;
		if(!backtrack_stack.push(424)) return -1;
		if(!backtrack_stack.push(423)) return -1;
		if(!backtrack_stack.push(422)) return -1;
		goto label_419;
		}
		break;
	case 2447:
	// Label to restore values of label 421
		v_q35_1 = int_value_stack.pop();
		break;
	case 422: label_422: 
	// using  symbolic fact q35(v_q35_1) with literal 1 in rule 
	// 412: wine(A) :- q35(A).
	// resulting in symbolic fact wine(v_wine_1)

		v_wine_1 = v_q35_1;
		if(wine_set.insert(v_wine_1)) {
		if(!wine_f.insert(v_wine_1)) return -1;
		// Labels for next rules
		if(!backtrack_stack.push(1007)) return -1;
		if(!backtrack_stack.push(989)) return -1;
		if(!backtrack_stack.push(954)) return -1;
		if(!backtrack_stack.push(938)) return -1;
		if(!backtrack_stack.push(911)) return -1;
		if(!backtrack_stack.push(903)) return -1;
		if(!backtrack_stack.push(902)) return -1;
		if(!backtrack_stack.push(901)) return -1;
		if(!backtrack_stack.push(849)) return -1;
		if(!backtrack_stack.push(848)) return -1;
		if(!backtrack_stack.push(821)) return -1;
		if(!backtrack_stack.push(820)) return -1;
		if(!backtrack_stack.push(737)) return -1;
		if(!backtrack_stack.push(712)) return -1;
		if(!backtrack_stack.push(674)) return -1;
		if(!backtrack_stack.push(672)) return -1;
		if(!backtrack_stack.push(583)) return -1;
		if(!backtrack_stack.push(575)) return -1;
		if(!backtrack_stack.push(563)) return -1;
		if(!backtrack_stack.push(551)) return -1;
		if(!backtrack_stack.push(549)) return -1;
		if(!backtrack_stack.push(483)) return -1;
		if(!backtrack_stack.push(473)) return -1;
		if(!backtrack_stack.push(470)) return -1;
		if(!backtrack_stack.push(430)) return -1;
		if(!backtrack_stack.push(401)) return -1;
		if(!backtrack_stack.push(399)) return -1;
		if(!backtrack_stack.push(380)) return -1;
		if(!backtrack_stack.push(364)) return -1;
		if(!backtrack_stack.push(363)) return -1;
		if(!backtrack_stack.push(329)) return -1;
		if(!backtrack_stack.push(328)) return -1;
		if(!backtrack_stack.push(327)) return -1;
		if(!backtrack_stack.push(295)) return -1;
		if(!backtrack_stack.push(294)) return -1;
		if(!backtrack_stack.push(216)) return -1;
		if(!backtrack_stack.push(210)) return -1;
		if(!backtrack_stack.push(192)) return -1;
		if(!backtrack_stack.push(180)) return -1;
		if(!backtrack_stack.push(179)) return -1;
		if(!backtrack_stack.push(178)) return -1;
		if(!backtrack_stack.push(154)) return -1;
		if(!backtrack_stack.push(153)) return -1;
		if(!backtrack_stack.push(100)) return -1;
		if(!backtrack_stack.push(76)) return -1;
		if(!backtrack_stack.push(75)) return -1;
		if(!backtrack_stack.push(67)) return -1;
		if(!backtrack_stack.push(63)) return -1;
		if(!backtrack_stack.push(53)) return -1;
		if(!backtrack_stack.push(46)) return -1;
		if(!backtrack_stack.push(19)) return -1;
		goto label_16;
		}
		break;
	case 423: label_423: 
	// using  symbolic fact q35(v_q35_1) with literal 1 in rule 
	// 413: q35(A) :- q35(B), kaon2equal(A,B).
	// resulting in symbolic fact q35(v_q35_1)

		//recursive
		c413_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q35_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2449)) return -1;
		c413_2_fb.open(v_q35_1);
	case 1436:
	// Continuation for label 423
		while (c413_2_fb.fetch()) {
			v_q35_1 = c413_2_fb.c2();
		if(q35_set.insert(v_q35_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1436)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(425)) return -1;
			if(!backtrack_stack.push(424)) return -1;
			if(!backtrack_stack.push(423)) return -1;
			if(!backtrack_stack.push(422)) return -1;
			goto label_419;
		}
		}
		break;
	case 2449:
	// Label to restore values of label 423
		v_q35_1 = int_value_stack.pop();
		c413_2_fb.pop(&cursor_stack);
		break;
	case 424: label_424: 
	// using  symbolic fact q35(v_q35_1) with literal 1 in rule 
	// 619: hasbody(A,A) :- q35(A), kaon2namedobjects(A).
	// resulting in symbolic fact hasbody(v_hasbody_1,v_hasbody_2)

		//recursive
		if(!int_value_stack.push(v_hasbody_2)) return -1;
		if(!int_value_stack.push(v_hasbody_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2450)) return -1;
		if (!kaon2namedobjects_b.contains(v_q35_1)) continue;
	case 1437:
	// Continuation for label 424
			v_hasbody_1 = v_q35_1;
			v_hasbody_2 = v_q35_1;
		if(hasbody_set.insert(v_hasbody_1, v_hasbody_2)) {
			if(!hasbody_bf.insert(v_hasbody_1, v_hasbody_2)) return -1;
			if(!hasbody_fb.insert(v_hasbody_2, v_hasbody_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1437)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(429)) return -1;
			if(!backtrack_stack.push(61)) return -1;
			if(!backtrack_stack.push(60)) return -1;
			if(!backtrack_stack.push(59)) return -1;
			if(!backtrack_stack.push(58)) return -1;
			goto label_54;
		}
		break;
	case 2450:
	// Label to restore values of label 424
		v_hasbody_1 = int_value_stack.pop();
		v_hasbody_2 = int_value_stack.pop();
		break;
	case 425: label_425: 
	// using  symbolic fact q35(v_q35_1) with literal 1 in rule 
	// 624: ot____nom45(A) :- q35(A), kaon2namedobjects(A).
	// resulting in symbolic fact ot____nom45(v_ot____nom45_1)

		//recursive
		if(!int_value_stack.push(v_ot____nom45_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2451)) return -1;
		if (!kaon2namedobjects_b.contains(v_q35_1)) continue;
	case 1438:
	// Continuation for label 425
			v_ot____nom45_1 = v_q35_1;
		if(ot____nom45_set.insert(v_ot____nom45_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1438)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(427)) return -1;
			goto label_426;
		}
		break;
	case 2451:
	// Label to restore values of label 425
		v_ot____nom45_1 = int_value_stack.pop();
		break;
	case 426: label_426: 
	// using  symbolic fact ot____nom45(v_ot____nom45_1) with literal 1 in rule 
	// 625: ot____nom45(A) :- ot____nom45(B), kaon2equal(A,B).
	// resulting in symbolic fact ot____nom45(v_ot____nom45_1)

		//recursive
		c625_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_ot____nom45_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2452)) return -1;
		c625_2_fb.open(v_ot____nom45_1);
	case 1439:
	// Continuation for label 426
		while (c625_2_fb.fetch()) {
			v_ot____nom45_1 = c625_2_fb.c2();
		if(ot____nom45_set.insert(v_ot____nom45_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1439)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(427)) return -1;
			goto label_426;
		}
		}
		break;
	case 2452:
	// Label to restore values of label 426
		v_ot____nom45_1 = int_value_stack.pop();
		c625_2_fb.pop(&cursor_stack);
		break;
	case 427: label_427: 
	// using  symbolic fact ot____nom45(v_ot____nom45_1) with literal 3 in rule 
	// 375: fullbodiedwine(A) :- wine(A), hasbody(A,B), ot____nom45(B).
	// resulting in symbolic fact fullbodiedwine(v_fullbodiedwine_1)

		//recursive
		c375_1_f.push(&cursor_stack);
		if(!int_value_stack.push(v_fullbodiedwine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2453)) return -1;
		c375_1_f.open();
		while (c375_1_f.fetch()) {
			if (!hasbody_set.contains(c375_1_f.c1(), v_ot____nom45_1)) continue;
	case 1440:
	// Continuation for label 427
				v_fullbodiedwine_1 = c375_1_f.c1();
		if(fullbodiedwine_set.insert(v_fullbodiedwine_1)) {
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1440)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(428)) return -1;
				if(!backtrack_stack.push(421)) return -1;
				goto label_420;
		}
		}
		break;
	case 2453:
	// Label to restore values of label 427
		v_fullbodiedwine_1 = int_value_stack.pop();
		c375_1_f.pop(&cursor_stack);
		break;
	case 428: label_428: 
	// using  symbolic fact fullbodiedwine(v_fullbodiedwine_1) with literal 1 in rule 
	// 374: fullbodiedwine(A) :- fullbodiedwine(B), kaon2equal(A,B).
	// resulting in symbolic fact fullbodiedwine(v_fullbodiedwine_1)

		//recursive
		c374_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_fullbodiedwine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2454)) return -1;
		c374_2_fb.open(v_fullbodiedwine_1);
	case 1441:
	// Continuation for label 428
		while (c374_2_fb.fetch()) {
			v_fullbodiedwine_1 = c374_2_fb.c2();
		if(fullbodiedwine_set.insert(v_fullbodiedwine_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1441)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(428)) return -1;
			if(!backtrack_stack.push(421)) return -1;
			goto label_420;
		}
		}
		break;
	case 2454:
	// Label to restore values of label 428
		v_fullbodiedwine_1 = int_value_stack.pop();
		c374_2_fb.pop(&cursor_stack);
		break;
	case 429: label_429: 
	// using  symbolic fact hasbody(v_hasbody_1,v_hasbody_2) with literal 2 in rule 
	// 375: fullbodiedwine(A) :- wine(A), hasbody(A,B), ot____nom45(B).
	// resulting in symbolic fact fullbodiedwine(v_fullbodiedwine_1)

		//recursive
		if(!int_value_stack.push(v_fullbodiedwine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2455)) return -1;
		if (!wine_set.contains(v_hasbody_1)) continue;
			if (!ot____nom45_set.contains(v_hasbody_2)) continue;
	case 1442:
	// Continuation for label 429
				v_fullbodiedwine_1 = v_hasbody_1;
		if(fullbodiedwine_set.insert(v_fullbodiedwine_1)) {
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1442)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(428)) return -1;
				if(!backtrack_stack.push(421)) return -1;
				goto label_420;
		}
		break;
	case 2455:
	// Label to restore values of label 429
		v_fullbodiedwine_1 = int_value_stack.pop();
		break;
	case 430: label_430: 
	// using  symbolic fact wine(v_wine_1) with literal 1 in rule 
	// 375: fullbodiedwine(A) :- wine(A), hasbody(A,B), ot____nom45(B).
	// resulting in symbolic fact fullbodiedwine(v_fullbodiedwine_1)

		c375_2_bf.open(v_wine_1);
		while (c375_2_bf.fetch()) {
			if (!ot____nom45_set.contains(c375_2_bf.c2())) continue;
	case 1443:
	// Continuation for label 430
				v_fullbodiedwine_1 = v_wine_1;
		if(fullbodiedwine_set.insert(v_fullbodiedwine_1)) {
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1443)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(428)) return -1;
				if(!backtrack_stack.push(421)) return -1;
				goto label_420;
		}
		}
		break;
	case 431: label_431: 
	// using  symbolic fact q17(v_q17_1) with literal 1 in rule 
	// 691: q17(A) :- q17(B), kaon2equal(A,B).
	// resulting in symbolic fact q17(v_q17_1)

		//recursive
		c691_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q17_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2457)) return -1;
		c691_2_fb.open(v_q17_1);
	case 1444:
	// Continuation for label 431
		while (c691_2_fb.fetch()) {
			v_q17_1 = c691_2_fb.c2();
		if(q17_set.insert(v_q17_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1444)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(432)) return -1;
			if(!backtrack_stack.push(431)) return -1;
			goto label_416;
		}
		}
		break;
	case 2457:
	// Label to restore values of label 431
		v_q17_1 = int_value_stack.pop();
		c691_2_fb.pop(&cursor_stack);
		break;
	case 432: label_432: 
	// using  symbolic fact q17(v_q17_1) with literal 2 in rule 
	// 181: q16(A) :- locatedin(A,B), q17(B).
	// resulting in symbolic fact q16(v_q16_1)

		//recursive
		c181_1_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q16_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2458)) return -1;
		c181_1_fb.open(v_q17_1);
	case 1445:
	// Continuation for label 432
		while (c181_1_fb.fetch()) {
			v_q16_1 = c181_1_fb.c2();
		if(q16_set.insert(v_q16_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1445)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(438)) return -1;
			if(!backtrack_stack.push(435)) return -1;
			if(!backtrack_stack.push(433)) return -1;
			goto label_415;
		}
		}
		break;
	case 2458:
	// Label to restore values of label 432
		v_q16_1 = int_value_stack.pop();
		c181_1_fb.pop(&cursor_stack);
		break;
	case 433: label_433: 
	// using  symbolic fact q16(v_q16_1) with literal 1 in rule 
	// 180: locatedin(A,A) :- q16(A), kaon2namedobjects(A).
	// resulting in symbolic fact locatedin(v_locatedin_1,v_locatedin_2)

		//recursive
		if(!int_value_stack.push(v_locatedin_2)) return -1;
		if(!int_value_stack.push(v_locatedin_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2459)) return -1;
		if (!kaon2namedobjects_b.contains(v_q16_1)) continue;
	case 1446:
	// Continuation for label 433
			v_locatedin_1 = v_q16_1;
			v_locatedin_2 = v_q16_1;
		if(locatedin_set.insert(v_locatedin_1, v_locatedin_2)) {
			if(!locatedin_bf.insert(v_locatedin_1, v_locatedin_2)) return -1;
			if(!locatedin_fb.insert(v_locatedin_2, v_locatedin_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1446)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(1005)) return -1;
			if(!backtrack_stack.push(999)) return -1;
			if(!backtrack_stack.push(987)) return -1;
			if(!backtrack_stack.push(981)) return -1;
			if(!backtrack_stack.push(969)) return -1;
			if(!backtrack_stack.push(963)) return -1;
			if(!backtrack_stack.push(952)) return -1;
			if(!backtrack_stack.push(946)) return -1;
			if(!backtrack_stack.push(936)) return -1;
			if(!backtrack_stack.push(930)) return -1;
			if(!backtrack_stack.push(920)) return -1;
			if(!backtrack_stack.push(917)) return -1;
			if(!backtrack_stack.push(898)) return -1;
			if(!backtrack_stack.push(897)) return -1;
			if(!backtrack_stack.push(896)) return -1;
			if(!backtrack_stack.push(893)) return -1;
			if(!backtrack_stack.push(726)) return -1;
			if(!backtrack_stack.push(721)) return -1;
			if(!backtrack_stack.push(694)) return -1;
			if(!backtrack_stack.push(693)) return -1;
			if(!backtrack_stack.push(688)) return -1;
			if(!backtrack_stack.push(687)) return -1;
			if(!backtrack_stack.push(636)) return -1;
			if(!backtrack_stack.push(631)) return -1;
			if(!backtrack_stack.push(592)) return -1;
			if(!backtrack_stack.push(591)) return -1;
			if(!backtrack_stack.push(529)) return -1;
			if(!backtrack_stack.push(528)) return -1;
			if(!backtrack_stack.push(513)) return -1;
			if(!backtrack_stack.push(512)) return -1;
			if(!backtrack_stack.push(511)) return -1;
			if(!backtrack_stack.push(506)) return -1;
			if(!backtrack_stack.push(455)) return -1;
			if(!backtrack_stack.push(450)) return -1;
			if(!backtrack_stack.push(441)) return -1;
			if(!backtrack_stack.push(440)) return -1;
			if(!backtrack_stack.push(439)) return -1;
			if(!backtrack_stack.push(434)) return -1;
			if(!backtrack_stack.push(339)) return -1;
			if(!backtrack_stack.push(338)) return -1;
			if(!backtrack_stack.push(249)) return -1;
			if(!backtrack_stack.push(244)) return -1;
			if(!backtrack_stack.push(52)) return -1;
			if(!backtrack_stack.push(51)) return -1;
			if(!backtrack_stack.push(50)) return -1;
			if(!backtrack_stack.push(49)) return -1;
			if(!backtrack_stack.push(48)) return -1;
			goto label_47;
		}
		break;
	case 2459:
	// Label to restore values of label 433
		v_locatedin_1 = int_value_stack.pop();
		v_locatedin_2 = int_value_stack.pop();
		break;
	case 434: label_434: 
	// using  symbolic fact locatedin(v_locatedin_1,v_locatedin_2) with literal 1 in rule 
	// 181: q16(A) :- locatedin(A,B), q17(B).
	// resulting in symbolic fact q16(v_q16_1)

		//recursive
		if(!int_value_stack.push(v_q16_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2460)) return -1;
		if (!q17_set.contains(v_locatedin_2)) continue;
	case 1447:
	// Continuation for label 434
			v_q16_1 = v_locatedin_1;
		if(q16_set.insert(v_q16_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1447)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(438)) return -1;
			if(!backtrack_stack.push(435)) return -1;
			if(!backtrack_stack.push(433)) return -1;
			goto label_415;
		}
		break;
	case 2460:
	// Label to restore values of label 434
		v_q16_1 = int_value_stack.pop();
		break;
	case 435: label_435: 
	// using  symbolic fact q16(v_q16_1) with literal 1 in rule 
	// 688: ot____nom31(A) :- q16(A), kaon2namedobjects(A).
	// resulting in symbolic fact ot____nom31(v_ot____nom31_1)

		//recursive
		if(!int_value_stack.push(v_ot____nom31_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2461)) return -1;
		if (!kaon2namedobjects_b.contains(v_q16_1)) continue;
	case 1448:
	// Continuation for label 435
			v_ot____nom31_1 = v_q16_1;
		if(ot____nom31_set.insert(v_ot____nom31_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1448)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(437)) return -1;
			goto label_436;
		}
		break;
	case 2461:
	// Label to restore values of label 435
		v_ot____nom31_1 = int_value_stack.pop();
		break;
	case 436: label_436: 
	// using  symbolic fact ot____nom31(v_ot____nom31_1) with literal 1 in rule 
	// 690: ot____nom31(A) :- ot____nom31(B), kaon2equal(A,B).
	// resulting in symbolic fact ot____nom31(v_ot____nom31_1)

		//recursive
		c690_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_ot____nom31_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2462)) return -1;
		c690_2_fb.open(v_ot____nom31_1);
	case 1449:
	// Continuation for label 436
		while (c690_2_fb.fetch()) {
			v_ot____nom31_1 = c690_2_fb.c2();
		if(ot____nom31_set.insert(v_ot____nom31_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1449)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(437)) return -1;
			goto label_436;
		}
		}
		break;
	case 2462:
	// Label to restore values of label 436
		v_ot____nom31_1 = int_value_stack.pop();
		c690_2_fb.pop(&cursor_stack);
		break;
	case 437: label_437: 
	// using  symbolic fact ot____nom31(v_ot____nom31_1) with literal 2 in rule 
	// 686: q17(A) :- locatedin(A,B), ot____nom31(B).
	// resulting in symbolic fact q17(v_q17_1)

		//recursive
		c686_1_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q17_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2463)) return -1;
		c686_1_fb.open(v_ot____nom31_1);
	case 1450:
	// Continuation for label 437
		while (c686_1_fb.fetch()) {
			v_q17_1 = c686_1_fb.c2();
		if(q17_set.insert(v_q17_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1450)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(432)) return -1;
			if(!backtrack_stack.push(431)) return -1;
			goto label_416;
		}
		}
		break;
	case 2463:
	// Label to restore values of label 437
		v_q17_1 = int_value_stack.pop();
		c686_1_fb.pop(&cursor_stack);
		break;
	case 438: label_438: 
	// using  symbolic fact q16(v_q16_1) with literal 1 in rule 
	// 689: q16(A) :- q16(B), kaon2equal(A,B).
	// resulting in symbolic fact q16(v_q16_1)

		//recursive
		c689_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q16_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2464)) return -1;
		c689_2_fb.open(v_q16_1);
	case 1451:
	// Continuation for label 438
		while (c689_2_fb.fetch()) {
			v_q16_1 = c689_2_fb.c2();
		if(q16_set.insert(v_q16_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1451)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(438)) return -1;
			if(!backtrack_stack.push(435)) return -1;
			if(!backtrack_stack.push(433)) return -1;
			goto label_415;
		}
		}
		break;
	case 2464:
	// Label to restore values of label 438
		v_q16_1 = int_value_stack.pop();
		c689_2_fb.pop(&cursor_stack);
		break;
	case 439: label_439: 
	// using  symbolic fact locatedin(v_locatedin_1,v_locatedin_2) with literal 1 in rule 
	// 194: q29(A) :- locatedin(A,B), q30(B).
	// resulting in symbolic fact q29(v_q29_1)

		if (!q30_set.contains(v_locatedin_2)) continue;
	case 1452:
	// Continuation for label 439
			v_q29_1 = v_locatedin_1;
		if(q29_set.insert(v_q29_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1452)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(411)) return -1;
			if(!backtrack_stack.push(408)) return -1;
			if(!backtrack_stack.push(370)) return -1;
			goto label_369;
		}
		break;
	case 440: label_440: 
	// using  symbolic fact locatedin(v_locatedin_1,v_locatedin_2) with literal 1 in rule 
	// 674: q30(A) :- locatedin(A,B), ot____nom44(B).
	// resulting in symbolic fact q30(v_q30_1)

		if (!ot____nom44_set.contains(v_locatedin_2)) continue;
	case 1453:
	// Continuation for label 440
			v_q30_1 = v_locatedin_1;
		if(q30_set.insert(v_q30_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1453)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(407)) return -1;
			if(!backtrack_stack.push(406)) return -1;
			goto label_371;
		}
		break;
	case 441: label_441: 
	// using  symbolic fact locatedin(v_locatedin_1,v_locatedin_2) with literal 1 in rule 
	// 686: q17(A) :- locatedin(A,B), ot____nom31(B).
	// resulting in symbolic fact q17(v_q17_1)

		//recursive
		if(!int_value_stack.push(v_q17_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2467)) return -1;
		if (!ot____nom31_set.contains(v_locatedin_2)) continue;
	case 1454:
	// Continuation for label 441
			v_q17_1 = v_locatedin_1;
		if(q17_set.insert(v_q17_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1454)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(432)) return -1;
			if(!backtrack_stack.push(431)) return -1;
			goto label_416;
		}
		break;
	case 2467:
	// Label to restore values of label 441
		v_q17_1 = int_value_stack.pop();
		break;
	case 442:
	// applying rule 
	// 79: hasbody(A,B) :- hasbody_aux(A,B).
	// resulting in symbolic fact hasbody(v_hasbody_1,v_hasbody_2)
		c79_1_ff.open();
	case 1455:
	// Continuation for label 442
		while (c79_1_ff.fetch()) {
			v_hasbody_2 = c79_1_ff.c2();
			v_hasbody_1 = c79_1_ff.c1();
		if(hasbody_set.insert(v_hasbody_1, v_hasbody_2)) {
			if(!hasbody_bf.insert(v_hasbody_1, v_hasbody_2)) return -1;
			if(!hasbody_fb.insert(v_hasbody_2, v_hasbody_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1455)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(429)) return -1;
			if(!backtrack_stack.push(61)) return -1;
			if(!backtrack_stack.push(60)) return -1;
			if(!backtrack_stack.push(59)) return -1;
			if(!backtrack_stack.push(58)) return -1;
			goto label_54;
		}
		}
		break;
	case 443:
	// applying rule 
	// 83: muscadet(A) :- muscadet_aux(A).
	// resulting in symbolic fact muscadet(v_muscadet_1)
		c83_1_f.open();
	case 1456:
	// Continuation for label 443
		while (c83_1_f.fetch()) {
			v_muscadet_1 = c83_1_f.c1();
		if(muscadet_set.insert(v_muscadet_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1456)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(489)) return -1;
			if(!backtrack_stack.push(488)) return -1;
			if(!backtrack_stack.push(477)) return -1;
			if(!backtrack_stack.push(458)) return -1;
			if(!backtrack_stack.push(457)) return -1;
			if(!backtrack_stack.push(456)) return -1;
			if(!backtrack_stack.push(445)) return -1;
			goto label_444;
		}
		}
		break;
	case 444: label_444: 
	// using  symbolic fact muscadet(v_muscadet_1) with literal 1 in rule 
	// 82: q45(A) :- muscadet(A).
	// resulting in symbolic fact q45(v_q45_1)

		v_q45_1 = v_muscadet_1;
		if(q45_set.insert(v_q45_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(240)) return -1;
		if(!backtrack_stack.push(177)) return -1;
		goto label_176;
		}
		break;
	case 445: label_445: 
	// using  symbolic fact muscadet(v_muscadet_1) with literal 1 in rule 
	// 86: q55(A) :- muscadet(A).
	// resulting in symbolic fact q55(v_q55_1)

		v_q55_1 = v_muscadet_1;
		if(q55_set.insert(v_q55_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(454)) return -1;
		if(!backtrack_stack.push(451)) return -1;
		if(!backtrack_stack.push(449)) return -1;
		goto label_446;
		}
		break;
	case 446: label_446: 
	// using  symbolic fact q55(v_q55_1) with literal 1 in rule 
	// 85: q56(A) :- q55(A).
	// resulting in symbolic fact q56(v_q56_1)

		//recursive
		if(!int_value_stack.push(v_q56_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2472)) return -1;
		v_q56_1 = v_q55_1;
		if(q56_set.insert(v_q56_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(448)) return -1;
		goto label_447;
		}
		break;
	case 2472:
	// Label to restore values of label 446
		v_q56_1 = int_value_stack.pop();
		break;
	case 447: label_447: 
	// using  symbolic fact q56(v_q56_1) with literal 1 in rule 
	// 697: q56(A) :- q56(B), kaon2equal(A,B).
	// resulting in symbolic fact q56(v_q56_1)

		//recursive
		c697_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q56_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2473)) return -1;
		c697_2_fb.open(v_q56_1);
	case 1460:
	// Continuation for label 447
		while (c697_2_fb.fetch()) {
			v_q56_1 = c697_2_fb.c2();
		if(q56_set.insert(v_q56_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1460)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(448)) return -1;
			goto label_447;
		}
		}
		break;
	case 2473:
	// Label to restore values of label 447
		v_q56_1 = int_value_stack.pop();
		c697_2_fb.pop(&cursor_stack);
		break;
	case 448: label_448: 
	// using  symbolic fact q56(v_q56_1) with literal 2 in rule 
	// 179: q55(A) :- locatedin(A,B), q56(B).
	// resulting in symbolic fact q55(v_q55_1)

		//recursive
		c179_1_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q55_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2474)) return -1;
		c179_1_fb.open(v_q56_1);
	case 1461:
	// Continuation for label 448
		while (c179_1_fb.fetch()) {
			v_q55_1 = c179_1_fb.c2();
		if(q55_set.insert(v_q55_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1461)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(454)) return -1;
			if(!backtrack_stack.push(451)) return -1;
			if(!backtrack_stack.push(449)) return -1;
			goto label_446;
		}
		}
		break;
	case 2474:
	// Label to restore values of label 448
		v_q55_1 = int_value_stack.pop();
		c179_1_fb.pop(&cursor_stack);
		break;
	case 449: label_449: 
	// using  symbolic fact q55(v_q55_1) with literal 1 in rule 
	// 178: locatedin(A,A) :- q55(A), kaon2namedobjects(A).
	// resulting in symbolic fact locatedin(v_locatedin_1,v_locatedin_2)

		//recursive
		if(!int_value_stack.push(v_locatedin_2)) return -1;
		if(!int_value_stack.push(v_locatedin_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2475)) return -1;
		if (!kaon2namedobjects_b.contains(v_q55_1)) continue;
	case 1462:
	// Continuation for label 449
			v_locatedin_1 = v_q55_1;
			v_locatedin_2 = v_q55_1;
		if(locatedin_set.insert(v_locatedin_1, v_locatedin_2)) {
			if(!locatedin_bf.insert(v_locatedin_1, v_locatedin_2)) return -1;
			if(!locatedin_fb.insert(v_locatedin_2, v_locatedin_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1462)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(1005)) return -1;
			if(!backtrack_stack.push(999)) return -1;
			if(!backtrack_stack.push(987)) return -1;
			if(!backtrack_stack.push(981)) return -1;
			if(!backtrack_stack.push(969)) return -1;
			if(!backtrack_stack.push(963)) return -1;
			if(!backtrack_stack.push(952)) return -1;
			if(!backtrack_stack.push(946)) return -1;
			if(!backtrack_stack.push(936)) return -1;
			if(!backtrack_stack.push(930)) return -1;
			if(!backtrack_stack.push(920)) return -1;
			if(!backtrack_stack.push(917)) return -1;
			if(!backtrack_stack.push(898)) return -1;
			if(!backtrack_stack.push(897)) return -1;
			if(!backtrack_stack.push(896)) return -1;
			if(!backtrack_stack.push(893)) return -1;
			if(!backtrack_stack.push(726)) return -1;
			if(!backtrack_stack.push(721)) return -1;
			if(!backtrack_stack.push(694)) return -1;
			if(!backtrack_stack.push(693)) return -1;
			if(!backtrack_stack.push(688)) return -1;
			if(!backtrack_stack.push(687)) return -1;
			if(!backtrack_stack.push(636)) return -1;
			if(!backtrack_stack.push(631)) return -1;
			if(!backtrack_stack.push(592)) return -1;
			if(!backtrack_stack.push(591)) return -1;
			if(!backtrack_stack.push(529)) return -1;
			if(!backtrack_stack.push(528)) return -1;
			if(!backtrack_stack.push(513)) return -1;
			if(!backtrack_stack.push(512)) return -1;
			if(!backtrack_stack.push(511)) return -1;
			if(!backtrack_stack.push(506)) return -1;
			if(!backtrack_stack.push(455)) return -1;
			if(!backtrack_stack.push(450)) return -1;
			if(!backtrack_stack.push(441)) return -1;
			if(!backtrack_stack.push(440)) return -1;
			if(!backtrack_stack.push(439)) return -1;
			if(!backtrack_stack.push(434)) return -1;
			if(!backtrack_stack.push(339)) return -1;
			if(!backtrack_stack.push(338)) return -1;
			if(!backtrack_stack.push(249)) return -1;
			if(!backtrack_stack.push(244)) return -1;
			if(!backtrack_stack.push(52)) return -1;
			if(!backtrack_stack.push(51)) return -1;
			if(!backtrack_stack.push(50)) return -1;
			if(!backtrack_stack.push(49)) return -1;
			if(!backtrack_stack.push(48)) return -1;
			goto label_47;
		}
		break;
	case 2475:
	// Label to restore values of label 449
		v_locatedin_1 = int_value_stack.pop();
		v_locatedin_2 = int_value_stack.pop();
		break;
	case 450: label_450: 
	// using  symbolic fact locatedin(v_locatedin_1,v_locatedin_2) with literal 1 in rule 
	// 179: q55(A) :- locatedin(A,B), q56(B).
	// resulting in symbolic fact q55(v_q55_1)

		//recursive
		if(!int_value_stack.push(v_q55_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2476)) return -1;
		if (!q56_set.contains(v_locatedin_2)) continue;
	case 1463:
	// Continuation for label 450
			v_q55_1 = v_locatedin_1;
		if(q55_set.insert(v_q55_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1463)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(454)) return -1;
			if(!backtrack_stack.push(451)) return -1;
			if(!backtrack_stack.push(449)) return -1;
			goto label_446;
		}
		break;
	case 2476:
	// Label to restore values of label 450
		v_q55_1 = int_value_stack.pop();
		break;
	case 451: label_451: 
	// using  symbolic fact q55(v_q55_1) with literal 1 in rule 
	// 694: ot____nom16(A) :- q55(A), kaon2namedobjects(A).
	// resulting in symbolic fact ot____nom16(v_ot____nom16_1)

		//recursive
		if(!int_value_stack.push(v_ot____nom16_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2477)) return -1;
		if (!kaon2namedobjects_b.contains(v_q55_1)) continue;
	case 1464:
	// Continuation for label 451
			v_ot____nom16_1 = v_q55_1;
		if(ot____nom16_set.insert(v_ot____nom16_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1464)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(453)) return -1;
			goto label_452;
		}
		break;
	case 2477:
	// Label to restore values of label 451
		v_ot____nom16_1 = int_value_stack.pop();
		break;
	case 452: label_452: 
	// using  symbolic fact ot____nom16(v_ot____nom16_1) with literal 1 in rule 
	// 696: ot____nom16(A) :- ot____nom16(B), kaon2equal(A,B).
	// resulting in symbolic fact ot____nom16(v_ot____nom16_1)

		//recursive
		c696_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_ot____nom16_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2478)) return -1;
		c696_2_fb.open(v_ot____nom16_1);
	case 1465:
	// Continuation for label 452
		while (c696_2_fb.fetch()) {
			v_ot____nom16_1 = c696_2_fb.c2();
		if(ot____nom16_set.insert(v_ot____nom16_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1465)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(453)) return -1;
			goto label_452;
		}
		}
		break;
	case 2478:
	// Label to restore values of label 452
		v_ot____nom16_1 = int_value_stack.pop();
		c696_2_fb.pop(&cursor_stack);
		break;
	case 453: label_453: 
	// using  symbolic fact ot____nom16(v_ot____nom16_1) with literal 2 in rule 
	// 692: q56(A) :- locatedin(A,B), ot____nom16(B).
	// resulting in symbolic fact q56(v_q56_1)

		//recursive
		c692_1_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q56_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2479)) return -1;
		c692_1_fb.open(v_ot____nom16_1);
	case 1466:
	// Continuation for label 453
		while (c692_1_fb.fetch()) {
			v_q56_1 = c692_1_fb.c2();
		if(q56_set.insert(v_q56_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1466)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(448)) return -1;
			goto label_447;
		}
		}
		break;
	case 2479:
	// Label to restore values of label 453
		v_q56_1 = int_value_stack.pop();
		c692_1_fb.pop(&cursor_stack);
		break;
	case 454: label_454: 
	// using  symbolic fact q55(v_q55_1) with literal 1 in rule 
	// 695: q55(A) :- q55(B), kaon2equal(A,B).
	// resulting in symbolic fact q55(v_q55_1)

		//recursive
		c695_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q55_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2480)) return -1;
		c695_2_fb.open(v_q55_1);
	case 1467:
	// Continuation for label 454
		while (c695_2_fb.fetch()) {
			v_q55_1 = c695_2_fb.c2();
		if(q55_set.insert(v_q55_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1467)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(454)) return -1;
			if(!backtrack_stack.push(451)) return -1;
			if(!backtrack_stack.push(449)) return -1;
			goto label_446;
		}
		}
		break;
	case 2480:
	// Label to restore values of label 454
		v_q55_1 = int_value_stack.pop();
		c695_2_fb.pop(&cursor_stack);
		break;
	case 455: label_455: 
	// using  symbolic fact locatedin(v_locatedin_1,v_locatedin_2) with literal 1 in rule 
	// 692: q56(A) :- locatedin(A,B), ot____nom16(B).
	// resulting in symbolic fact q56(v_q56_1)

		//recursive
		if(!int_value_stack.push(v_q56_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2481)) return -1;
		if (!ot____nom16_set.contains(v_locatedin_2)) continue;
	case 1468:
	// Continuation for label 455
			v_q56_1 = v_locatedin_1;
		if(q56_set.insert(v_q56_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1468)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(448)) return -1;
			goto label_447;
		}
		break;
	case 2481:
	// Label to restore values of label 455
		v_q56_1 = int_value_stack.pop();
		break;
	case 456: label_456: 
	// using  symbolic fact muscadet(v_muscadet_1) with literal 1 in rule 
	// 87: muscadet(A) :- muscadet(B), kaon2equal(A,B).
	// resulting in symbolic fact muscadet(v_muscadet_1)

		//recursive
		c87_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_muscadet_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2482)) return -1;
		c87_2_fb.open(v_muscadet_1);
	case 1469:
	// Continuation for label 456
		while (c87_2_fb.fetch()) {
			v_muscadet_1 = c87_2_fb.c2();
		if(muscadet_set.insert(v_muscadet_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1469)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(489)) return -1;
			if(!backtrack_stack.push(488)) return -1;
			if(!backtrack_stack.push(477)) return -1;
			if(!backtrack_stack.push(458)) return -1;
			if(!backtrack_stack.push(457)) return -1;
			if(!backtrack_stack.push(456)) return -1;
			if(!backtrack_stack.push(445)) return -1;
			goto label_444;
		}
		}
		break;
	case 2482:
	// Label to restore values of label 456
		v_muscadet_1 = int_value_stack.pop();
		c87_2_fb.pop(&cursor_stack);
		break;
	case 457: label_457: 
	// using  symbolic fact muscadet(v_muscadet_1) with literal 1 in rule 
	// 137: q20(A) :- muscadet(A).
	// resulting in symbolic fact q20(v_q20_1)

		v_q20_1 = v_muscadet_1;
		if(q20_set.insert(v_q20_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(184)) return -1;
		if(!backtrack_stack.push(183)) return -1;
		goto label_182;
		}
		break;
	case 458: label_458: 
	// using  symbolic fact muscadet(v_muscadet_1) with literal 1 in rule 
	// 361: q24(A) :- muscadet(A).
	// resulting in symbolic fact q24(v_q24_1)

		v_q24_1 = v_muscadet_1;
		if(q24_set.insert(v_q24_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(465)) return -1;
		if(!backtrack_stack.push(464)) return -1;
		if(!backtrack_stack.push(463)) return -1;
		if(!backtrack_stack.push(462)) return -1;
		if(!backtrack_stack.push(460)) return -1;
		goto label_459;
		}
		break;
	case 459: label_459: 
	// using  symbolic fact q24(v_q24_1) with literal 1 in rule 
	// 358: q5(A) :- q24(A).
	// resulting in symbolic fact q5(v_q5_1)

		v_q5_1 = v_q24_1;
		if(q5_set.insert(v_q5_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(136)) return -1;
		if(!backtrack_stack.push(135)) return -1;
		goto label_131;
		}
		break;
	case 460: label_460: 
	// using  symbolic fact q24(v_q24_1) with literal 1 in rule 
	// 360: pinotblanc(A) :- q24(A).
	// resulting in symbolic fact pinotblanc(v_pinotblanc_1)

		//recursive
		if(!int_value_stack.push(v_pinotblanc_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2486)) return -1;
		v_pinotblanc_1 = v_q24_1;
		if(pinotblanc_set.insert(v_pinotblanc_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(472)) return -1;
		if(!backtrack_stack.push(471)) return -1;
		if(!backtrack_stack.push(469)) return -1;
		if(!backtrack_stack.push(468)) return -1;
		goto label_461;
		}
		break;
	case 2486:
	// Label to restore values of label 460
		v_pinotblanc_1 = int_value_stack.pop();
		break;
	case 461: label_461: 
	// using  symbolic fact pinotblanc(v_pinotblanc_1) with literal 1 in rule 
	// 359: q24(A) :- pinotblanc(A).
	// resulting in symbolic fact q24(v_q24_1)

		//recursive
		if(!int_value_stack.push(v_q24_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2487)) return -1;
		v_q24_1 = v_pinotblanc_1;
		if(q24_set.insert(v_q24_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(465)) return -1;
		if(!backtrack_stack.push(464)) return -1;
		if(!backtrack_stack.push(463)) return -1;
		if(!backtrack_stack.push(462)) return -1;
		if(!backtrack_stack.push(460)) return -1;
		goto label_459;
		}
		break;
	case 2487:
	// Label to restore values of label 461
		v_q24_1 = int_value_stack.pop();
		break;
	case 462: label_462: 
	// using  symbolic fact q24(v_q24_1) with literal 1 in rule 
	// 362: q24(A) :- q24(B), kaon2equal(A,B).
	// resulting in symbolic fact q24(v_q24_1)

		//recursive
		c362_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q24_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2488)) return -1;
		c362_2_fb.open(v_q24_1);
	case 1475:
	// Continuation for label 462
		while (c362_2_fb.fetch()) {
			v_q24_1 = c362_2_fb.c2();
		if(q24_set.insert(v_q24_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1475)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(465)) return -1;
			if(!backtrack_stack.push(464)) return -1;
			if(!backtrack_stack.push(463)) return -1;
			if(!backtrack_stack.push(462)) return -1;
			if(!backtrack_stack.push(460)) return -1;
			goto label_459;
		}
		}
		break;
	case 2488:
	// Label to restore values of label 462
		v_q24_1 = int_value_stack.pop();
		c362_2_fb.pop(&cursor_stack);
		break;
	case 463: label_463: 
	// using  symbolic fact q24(v_q24_1) with literal 1 in rule 
	// 414: wine(A) :- q24(A).
	// resulting in symbolic fact wine(v_wine_1)

		v_wine_1 = v_q24_1;
		if(wine_set.insert(v_wine_1)) {
		if(!wine_f.insert(v_wine_1)) return -1;
		// Labels for next rules
		if(!backtrack_stack.push(1007)) return -1;
		if(!backtrack_stack.push(989)) return -1;
		if(!backtrack_stack.push(954)) return -1;
		if(!backtrack_stack.push(938)) return -1;
		if(!backtrack_stack.push(911)) return -1;
		if(!backtrack_stack.push(903)) return -1;
		if(!backtrack_stack.push(902)) return -1;
		if(!backtrack_stack.push(901)) return -1;
		if(!backtrack_stack.push(849)) return -1;
		if(!backtrack_stack.push(848)) return -1;
		if(!backtrack_stack.push(821)) return -1;
		if(!backtrack_stack.push(820)) return -1;
		if(!backtrack_stack.push(737)) return -1;
		if(!backtrack_stack.push(712)) return -1;
		if(!backtrack_stack.push(674)) return -1;
		if(!backtrack_stack.push(672)) return -1;
		if(!backtrack_stack.push(583)) return -1;
		if(!backtrack_stack.push(575)) return -1;
		if(!backtrack_stack.push(563)) return -1;
		if(!backtrack_stack.push(551)) return -1;
		if(!backtrack_stack.push(549)) return -1;
		if(!backtrack_stack.push(483)) return -1;
		if(!backtrack_stack.push(473)) return -1;
		if(!backtrack_stack.push(470)) return -1;
		if(!backtrack_stack.push(430)) return -1;
		if(!backtrack_stack.push(401)) return -1;
		if(!backtrack_stack.push(399)) return -1;
		if(!backtrack_stack.push(380)) return -1;
		if(!backtrack_stack.push(364)) return -1;
		if(!backtrack_stack.push(363)) return -1;
		if(!backtrack_stack.push(329)) return -1;
		if(!backtrack_stack.push(328)) return -1;
		if(!backtrack_stack.push(327)) return -1;
		if(!backtrack_stack.push(295)) return -1;
		if(!backtrack_stack.push(294)) return -1;
		if(!backtrack_stack.push(216)) return -1;
		if(!backtrack_stack.push(210)) return -1;
		if(!backtrack_stack.push(192)) return -1;
		if(!backtrack_stack.push(180)) return -1;
		if(!backtrack_stack.push(179)) return -1;
		if(!backtrack_stack.push(178)) return -1;
		if(!backtrack_stack.push(154)) return -1;
		if(!backtrack_stack.push(153)) return -1;
		if(!backtrack_stack.push(100)) return -1;
		if(!backtrack_stack.push(76)) return -1;
		if(!backtrack_stack.push(75)) return -1;
		if(!backtrack_stack.push(67)) return -1;
		if(!backtrack_stack.push(63)) return -1;
		if(!backtrack_stack.push(53)) return -1;
		if(!backtrack_stack.push(46)) return -1;
		if(!backtrack_stack.push(19)) return -1;
		goto label_16;
		}
		break;
	case 464: label_464: 
	// using  symbolic fact q24(v_q24_1) with literal 1 in rule 
	// 519: madefromgrape(A,A) :- q24(A), kaon2namedobjects(A).
	// resulting in symbolic fact madefromgrape(v_madefromgrape_1,v_madefromgrape_2)

		//recursive
		if(!int_value_stack.push(v_madefromgrape_2)) return -1;
		if(!int_value_stack.push(v_madefromgrape_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2490)) return -1;
		if (!kaon2namedobjects_b.contains(v_q24_1)) continue;
	case 1477:
	// Continuation for label 464
			v_madefromgrape_1 = v_q24_1;
			v_madefromgrape_2 = v_q24_1;
		if(madefromgrape_set.insert(v_madefromgrape_1, v_madefromgrape_2)) {
			if(!madefromgrape_bf.insert(v_madefromgrape_1, v_madefromgrape_2)) return -1;
			if(!madefromgrape_fb.insert(v_madefromgrape_2, v_madefromgrape_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1477)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(860)) return -1;
			if(!backtrack_stack.push(859)) return -1;
			if(!backtrack_stack.push(845)) return -1;
			if(!backtrack_stack.push(844)) return -1;
			if(!backtrack_stack.push(829)) return -1;
			if(!backtrack_stack.push(828)) return -1;
			if(!backtrack_stack.push(813)) return -1;
			if(!backtrack_stack.push(812)) return -1;
			if(!backtrack_stack.push(677)) return -1;
			if(!backtrack_stack.push(676)) return -1;
			if(!backtrack_stack.push(617)) return -1;
			if(!backtrack_stack.push(616)) return -1;
			if(!backtrack_stack.push(609)) return -1;
			if(!backtrack_stack.push(554)) return -1;
			if(!backtrack_stack.push(553)) return -1;
			if(!backtrack_stack.push(476)) return -1;
			if(!backtrack_stack.push(475)) return -1;
			if(!backtrack_stack.push(404)) return -1;
			if(!backtrack_stack.push(403)) return -1;
			if(!backtrack_stack.push(362)) return -1;
			if(!backtrack_stack.push(361)) return -1;
			if(!backtrack_stack.push(343)) return -1;
			if(!backtrack_stack.push(342)) return -1;
			if(!backtrack_stack.push(300)) return -1;
			if(!backtrack_stack.push(170)) return -1;
			if(!backtrack_stack.push(169)) return -1;
			if(!backtrack_stack.push(151)) return -1;
			if(!backtrack_stack.push(150)) return -1;
			if(!backtrack_stack.push(144)) return -1;
			if(!backtrack_stack.push(132)) return -1;
			if(!backtrack_stack.push(71)) return -1;
			if(!backtrack_stack.push(70)) return -1;
			if(!backtrack_stack.push(66)) return -1;
			goto label_64;
		}
		break;
	case 2490:
	// Label to restore values of label 464
		v_madefromgrape_1 = int_value_stack.pop();
		v_madefromgrape_2 = int_value_stack.pop();
		break;
	case 465: label_465: 
	// using  symbolic fact q24(v_q24_1) with literal 1 in rule 
	// 576: ot____nom21(A) :- q24(A), kaon2namedobjects(A).
	// resulting in symbolic fact ot____nom21(v_ot____nom21_1)

		//recursive
		if(!int_value_stack.push(v_ot____nom21_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2491)) return -1;
		if (!kaon2namedobjects_b.contains(v_q24_1)) continue;
	case 1478:
	// Continuation for label 465
			v_ot____nom21_1 = v_q24_1;
		if(ot____nom21_set.insert(v_ot____nom21_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1478)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(474)) return -1;
			if(!backtrack_stack.push(467)) return -1;
			goto label_466;
		}
		break;
	case 2491:
	// Label to restore values of label 465
		v_ot____nom21_1 = int_value_stack.pop();
		break;
	case 466: label_466: 
	// using  symbolic fact ot____nom21(v_ot____nom21_1) with literal 1 in rule 
	// 577: ot____nom21(A) :- ot____nom21(B), kaon2equal(A,B).
	// resulting in symbolic fact ot____nom21(v_ot____nom21_1)

		//recursive
		c577_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_ot____nom21_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2492)) return -1;
		c577_2_fb.open(v_ot____nom21_1);
	case 1479:
	// Continuation for label 466
		while (c577_2_fb.fetch()) {
			v_ot____nom21_1 = c577_2_fb.c2();
		if(ot____nom21_set.insert(v_ot____nom21_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1479)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(474)) return -1;
			if(!backtrack_stack.push(467)) return -1;
			goto label_466;
		}
		}
		break;
	case 2492:
	// Label to restore values of label 466
		v_ot____nom21_1 = int_value_stack.pop();
		c577_2_fb.pop(&cursor_stack);
		break;
	case 467: label_467: 
	// using  symbolic fact ot____nom21(v_ot____nom21_1) with literal 3 in rule 
	// 364: pinotblanc(A) :- wine(A), madefromgrape(A,B), ot____nom21(B).
	// resulting in symbolic fact pinotblanc(v_pinotblanc_1)

		//recursive
		c364_1_f.push(&cursor_stack);
		if(!int_value_stack.push(v_pinotblanc_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2493)) return -1;
		c364_1_f.open();
		while (c364_1_f.fetch()) {
			if (!madefromgrape_set.contains(c364_1_f.c1(), v_ot____nom21_1)) continue;
	case 1480:
	// Continuation for label 467
				v_pinotblanc_1 = c364_1_f.c1();
		if(pinotblanc_set.insert(v_pinotblanc_1)) {
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1480)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(472)) return -1;
				if(!backtrack_stack.push(471)) return -1;
				if(!backtrack_stack.push(469)) return -1;
				if(!backtrack_stack.push(468)) return -1;
				goto label_461;
		}
		}
		break;
	case 2493:
	// Label to restore values of label 467
		v_pinotblanc_1 = int_value_stack.pop();
		c364_1_f.pop(&cursor_stack);
		break;
	case 468: label_468: 
	// using  symbolic fact pinotblanc(v_pinotblanc_1) with literal 1 in rule 
	// 363: pinotblanc(A) :- pinotblanc(B), kaon2equal(A,B).
	// resulting in symbolic fact pinotblanc(v_pinotblanc_1)

		//recursive
		c363_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_pinotblanc_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2494)) return -1;
		c363_2_fb.open(v_pinotblanc_1);
	case 1481:
	// Continuation for label 468
		while (c363_2_fb.fetch()) {
			v_pinotblanc_1 = c363_2_fb.c2();
		if(pinotblanc_set.insert(v_pinotblanc_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1481)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(472)) return -1;
			if(!backtrack_stack.push(471)) return -1;
			if(!backtrack_stack.push(469)) return -1;
			if(!backtrack_stack.push(468)) return -1;
			goto label_461;
		}
		}
		break;
	case 2494:
	// Label to restore values of label 468
		v_pinotblanc_1 = int_value_stack.pop();
		c363_2_fb.pop(&cursor_stack);
		break;
	case 469: label_469: 
	// using  symbolic fact pinotblanc(v_pinotblanc_1) with literal 1 in rule 
	// 423: wine(A) :- pinotblanc(A).
	// resulting in symbolic fact wine(v_wine_1)

		//recursive
		if(!int_value_stack.push(v_wine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2495)) return -1;
		v_wine_1 = v_pinotblanc_1;
		if(wine_set.insert(v_wine_1)) {
		if(!wine_f.insert(v_wine_1)) return -1;
		// Labels for next rules
		if(!backtrack_stack.push(1007)) return -1;
		if(!backtrack_stack.push(989)) return -1;
		if(!backtrack_stack.push(954)) return -1;
		if(!backtrack_stack.push(938)) return -1;
		if(!backtrack_stack.push(911)) return -1;
		if(!backtrack_stack.push(903)) return -1;
		if(!backtrack_stack.push(902)) return -1;
		if(!backtrack_stack.push(901)) return -1;
		if(!backtrack_stack.push(849)) return -1;
		if(!backtrack_stack.push(848)) return -1;
		if(!backtrack_stack.push(821)) return -1;
		if(!backtrack_stack.push(820)) return -1;
		if(!backtrack_stack.push(737)) return -1;
		if(!backtrack_stack.push(712)) return -1;
		if(!backtrack_stack.push(674)) return -1;
		if(!backtrack_stack.push(672)) return -1;
		if(!backtrack_stack.push(583)) return -1;
		if(!backtrack_stack.push(575)) return -1;
		if(!backtrack_stack.push(563)) return -1;
		if(!backtrack_stack.push(551)) return -1;
		if(!backtrack_stack.push(549)) return -1;
		if(!backtrack_stack.push(483)) return -1;
		if(!backtrack_stack.push(473)) return -1;
		if(!backtrack_stack.push(470)) return -1;
		if(!backtrack_stack.push(430)) return -1;
		if(!backtrack_stack.push(401)) return -1;
		if(!backtrack_stack.push(399)) return -1;
		if(!backtrack_stack.push(380)) return -1;
		if(!backtrack_stack.push(364)) return -1;
		if(!backtrack_stack.push(363)) return -1;
		if(!backtrack_stack.push(329)) return -1;
		if(!backtrack_stack.push(328)) return -1;
		if(!backtrack_stack.push(327)) return -1;
		if(!backtrack_stack.push(295)) return -1;
		if(!backtrack_stack.push(294)) return -1;
		if(!backtrack_stack.push(216)) return -1;
		if(!backtrack_stack.push(210)) return -1;
		if(!backtrack_stack.push(192)) return -1;
		if(!backtrack_stack.push(180)) return -1;
		if(!backtrack_stack.push(179)) return -1;
		if(!backtrack_stack.push(178)) return -1;
		if(!backtrack_stack.push(154)) return -1;
		if(!backtrack_stack.push(153)) return -1;
		if(!backtrack_stack.push(100)) return -1;
		if(!backtrack_stack.push(76)) return -1;
		if(!backtrack_stack.push(75)) return -1;
		if(!backtrack_stack.push(67)) return -1;
		if(!backtrack_stack.push(63)) return -1;
		if(!backtrack_stack.push(53)) return -1;
		if(!backtrack_stack.push(46)) return -1;
		if(!backtrack_stack.push(19)) return -1;
		goto label_16;
		}
		break;
	case 2495:
	// Label to restore values of label 469
		v_wine_1 = int_value_stack.pop();
		break;
	case 470: label_470: 
	// using  symbolic fact wine(v_wine_1) with literal 1 in rule 
	// 364: pinotblanc(A) :- wine(A), madefromgrape(A,B), ot____nom21(B).
	// resulting in symbolic fact pinotblanc(v_pinotblanc_1)

		//recursive
		c364_2_bf.push(&cursor_stack);
		if(!int_value_stack.push(v_pinotblanc_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2496)) return -1;
		c364_2_bf.open(v_wine_1);
		while (c364_2_bf.fetch()) {
			if (!ot____nom21_set.contains(c364_2_bf.c2())) continue;
	case 1483:
	// Continuation for label 470
				v_pinotblanc_1 = v_wine_1;
		if(pinotblanc_set.insert(v_pinotblanc_1)) {
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1483)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(472)) return -1;
				if(!backtrack_stack.push(471)) return -1;
				if(!backtrack_stack.push(469)) return -1;
				if(!backtrack_stack.push(468)) return -1;
				goto label_461;
		}
		}
		break;
	case 2496:
	// Label to restore values of label 470
		v_pinotblanc_1 = int_value_stack.pop();
		c364_2_bf.pop(&cursor_stack);
		break;
	case 471: label_471: 
	// using  symbolic fact pinotblanc(v_pinotblanc_1) with literal 1 in rule 
	// 448: q12(A) :- pinotblanc(A).
	// resulting in symbolic fact q12(v_q12_1)

		v_q12_1 = v_pinotblanc_1;
		if(q12_set.insert(v_q12_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(116)) return -1;
		if(!backtrack_stack.push(115)) return -1;
		if(!backtrack_stack.push(114)) return -1;
		if(!backtrack_stack.push(99)) return -1;
		goto label_95;
		}
		break;
	case 472: label_472: 
	// using  symbolic fact pinotblanc(v_pinotblanc_1) with literal 1 in rule 
	// 785: q3(A) :- pinotblanc(A).
	// resulting in symbolic fact q3(v_q3_1)

		v_q3_1 = v_pinotblanc_1;
		if(q3_set.insert(v_q3_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(143)) return -1;
		goto label_123;
		}
		break;
	case 473: label_473: 
	// using  symbolic fact wine(v_wine_1) with literal 1 in rule 
	// 574: q5(A) :- wine(A), madefromgrape(A,B), ot____nom21(B).
	// resulting in symbolic fact q5(v_q5_1)

		c574_2_bf.open(v_wine_1);
		while (c574_2_bf.fetch()) {
			if (!ot____nom21_set.contains(c574_2_bf.c2())) continue;
	case 1486:
	// Continuation for label 473
				v_q5_1 = v_wine_1;
		if(q5_set.insert(v_q5_1)) {
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1486)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(136)) return -1;
				if(!backtrack_stack.push(135)) return -1;
				goto label_131;
		}
		}
		break;
	case 474: label_474: 
	// using  symbolic fact ot____nom21(v_ot____nom21_1) with literal 3 in rule 
	// 574: q5(A) :- wine(A), madefromgrape(A,B), ot____nom21(B).
	// resulting in symbolic fact q5(v_q5_1)

		c574_1_f.open();
		while (c574_1_f.fetch()) {
			if (!madefromgrape_set.contains(c574_1_f.c1(), v_ot____nom21_1)) continue;
	case 1487:
	// Continuation for label 474
				v_q5_1 = c574_1_f.c1();
		if(q5_set.insert(v_q5_1)) {
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1487)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(136)) return -1;
				if(!backtrack_stack.push(135)) return -1;
				goto label_131;
		}
		}
		break;
	case 475: label_475: 
	// using  symbolic fact madefromgrape(v_madefromgrape_1,v_madefromgrape_2) with literal 2 in rule 
	// 364: pinotblanc(A) :- wine(A), madefromgrape(A,B), ot____nom21(B).
	// resulting in symbolic fact pinotblanc(v_pinotblanc_1)

		//recursive
		if(!int_value_stack.push(v_pinotblanc_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2501)) return -1;
		if (!wine_set.contains(v_madefromgrape_1)) continue;
			if (!ot____nom21_set.contains(v_madefromgrape_2)) continue;
	case 1488:
	// Continuation for label 475
				v_pinotblanc_1 = v_madefromgrape_1;
		if(pinotblanc_set.insert(v_pinotblanc_1)) {
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1488)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(472)) return -1;
				if(!backtrack_stack.push(471)) return -1;
				if(!backtrack_stack.push(469)) return -1;
				if(!backtrack_stack.push(468)) return -1;
				goto label_461;
		}
		break;
	case 2501:
	// Label to restore values of label 475
		v_pinotblanc_1 = int_value_stack.pop();
		break;
	case 476: label_476: 
	// using  symbolic fact madefromgrape(v_madefromgrape_1,v_madefromgrape_2) with literal 2 in rule 
	// 574: q5(A) :- wine(A), madefromgrape(A,B), ot____nom21(B).
	// resulting in symbolic fact q5(v_q5_1)

		if (!wine_set.contains(v_madefromgrape_1)) continue;
			if (!ot____nom21_set.contains(v_madefromgrape_2)) continue;
	case 1489:
	// Continuation for label 476
				v_q5_1 = v_madefromgrape_1;
		if(q5_set.insert(v_q5_1)) {
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1489)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(136)) return -1;
				if(!backtrack_stack.push(135)) return -1;
				goto label_131;
		}
		break;
	case 477: label_477: 
	// using  symbolic fact muscadet(v_muscadet_1) with literal 1 in rule 
	// 382: loire(A) :- muscadet(A).
	// resulting in symbolic fact loire(v_loire_1)

		//recursive
		if(!int_value_stack.push(v_loire_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2503)) return -1;
		v_loire_1 = v_muscadet_1;
		if(loire_set.insert(v_loire_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(971)) return -1;
		if(!backtrack_stack.push(715)) return -1;
		if(!backtrack_stack.push(714)) return -1;
		if(!backtrack_stack.push(487)) return -1;
		if(!backtrack_stack.push(486)) return -1;
		if(!backtrack_stack.push(484)) return -1;
		if(!backtrack_stack.push(482)) return -1;
		goto label_478;
		}
		break;
	case 2503:
	// Label to restore values of label 477
		v_loire_1 = int_value_stack.pop();
		break;
	case 478: label_478: 
	// using  symbolic fact loire(v_loire_1) with literal 1 in rule 
	// 100: q57(A) :- loire(A).
	// resulting in symbolic fact q57(v_q57_1)

		//recursive
		if(!int_value_stack.push(v_q57_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2504)) return -1;
		v_q57_1 = v_loire_1;
		if(q57_set.insert(v_q57_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(498)) return -1;
		if(!backtrack_stack.push(495)) return -1;
		if(!backtrack_stack.push(480)) return -1;
		goto label_479;
		}
		break;
	case 2504:
	// Label to restore values of label 478
		v_q57_1 = int_value_stack.pop();
		break;
	case 479: label_479: 
	// using  symbolic fact q57(v_q57_1) with literal 1 in rule 
	// 99: locatedin(A,A) :- q57(A), kaon2namedobjects(A).
	// resulting in symbolic fact locatedin(v_locatedin_1,v_locatedin_2)

		if (!kaon2namedobjects_b.contains(v_q57_1)) continue;
	case 1492:
	// Continuation for label 479
			v_locatedin_1 = v_q57_1;
			v_locatedin_2 = v_q57_1;
		if(locatedin_set.insert(v_locatedin_1, v_locatedin_2)) {
			if(!locatedin_bf.insert(v_locatedin_1, v_locatedin_2)) return -1;
			if(!locatedin_fb.insert(v_locatedin_2, v_locatedin_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1492)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(1005)) return -1;
			if(!backtrack_stack.push(999)) return -1;
			if(!backtrack_stack.push(987)) return -1;
			if(!backtrack_stack.push(981)) return -1;
			if(!backtrack_stack.push(969)) return -1;
			if(!backtrack_stack.push(963)) return -1;
			if(!backtrack_stack.push(952)) return -1;
			if(!backtrack_stack.push(946)) return -1;
			if(!backtrack_stack.push(936)) return -1;
			if(!backtrack_stack.push(930)) return -1;
			if(!backtrack_stack.push(920)) return -1;
			if(!backtrack_stack.push(917)) return -1;
			if(!backtrack_stack.push(898)) return -1;
			if(!backtrack_stack.push(897)) return -1;
			if(!backtrack_stack.push(896)) return -1;
			if(!backtrack_stack.push(893)) return -1;
			if(!backtrack_stack.push(726)) return -1;
			if(!backtrack_stack.push(721)) return -1;
			if(!backtrack_stack.push(694)) return -1;
			if(!backtrack_stack.push(693)) return -1;
			if(!backtrack_stack.push(688)) return -1;
			if(!backtrack_stack.push(687)) return -1;
			if(!backtrack_stack.push(636)) return -1;
			if(!backtrack_stack.push(631)) return -1;
			if(!backtrack_stack.push(592)) return -1;
			if(!backtrack_stack.push(591)) return -1;
			if(!backtrack_stack.push(529)) return -1;
			if(!backtrack_stack.push(528)) return -1;
			if(!backtrack_stack.push(513)) return -1;
			if(!backtrack_stack.push(512)) return -1;
			if(!backtrack_stack.push(511)) return -1;
			if(!backtrack_stack.push(506)) return -1;
			if(!backtrack_stack.push(455)) return -1;
			if(!backtrack_stack.push(450)) return -1;
			if(!backtrack_stack.push(441)) return -1;
			if(!backtrack_stack.push(440)) return -1;
			if(!backtrack_stack.push(439)) return -1;
			if(!backtrack_stack.push(434)) return -1;
			if(!backtrack_stack.push(339)) return -1;
			if(!backtrack_stack.push(338)) return -1;
			if(!backtrack_stack.push(249)) return -1;
			if(!backtrack_stack.push(244)) return -1;
			if(!backtrack_stack.push(52)) return -1;
			if(!backtrack_stack.push(51)) return -1;
			if(!backtrack_stack.push(50)) return -1;
			if(!backtrack_stack.push(49)) return -1;
			if(!backtrack_stack.push(48)) return -1;
			goto label_47;
		}
		break;
	case 480: label_480: 
	// using  symbolic fact q57(v_q57_1) with literal 1 in rule 
	// 384: q58(A) :- q57(A).
	// resulting in symbolic fact q58(v_q58_1)

		//recursive
		if(!int_value_stack.push(v_q58_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2506)) return -1;
		v_q58_1 = v_q57_1;
		if(q58_set.insert(v_q58_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(494)) return -1;
		if(!backtrack_stack.push(493)) return -1;
		goto label_481;
		}
		break;
	case 2506:
	// Label to restore values of label 480
		v_q58_1 = int_value_stack.pop();
		break;
	case 481: label_481: 
	// using  symbolic fact q58(v_q58_1) with literal 1 in rule 
	// 383: loire(A) :- q58(A), wine(A).
	// resulting in symbolic fact loire(v_loire_1)

		//recursive
		if(!int_value_stack.push(v_loire_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2507)) return -1;
		if (!wine_set.contains(v_q58_1)) continue;
	case 1494:
	// Continuation for label 481
			v_loire_1 = v_q58_1;
		if(loire_set.insert(v_loire_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1494)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(971)) return -1;
			if(!backtrack_stack.push(715)) return -1;
			if(!backtrack_stack.push(714)) return -1;
			if(!backtrack_stack.push(487)) return -1;
			if(!backtrack_stack.push(486)) return -1;
			if(!backtrack_stack.push(484)) return -1;
			if(!backtrack_stack.push(482)) return -1;
			goto label_478;
		}
		break;
	case 2507:
	// Label to restore values of label 481
		v_loire_1 = int_value_stack.pop();
		break;
	case 482: label_482: 
	// using  symbolic fact loire(v_loire_1) with literal 1 in rule 
	// 376: wine(A) :- loire(A).
	// resulting in symbolic fact wine(v_wine_1)

		//recursive
		if(!int_value_stack.push(v_wine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2508)) return -1;
		v_wine_1 = v_loire_1;
		if(wine_set.insert(v_wine_1)) {
		if(!wine_f.insert(v_wine_1)) return -1;
		// Labels for next rules
		if(!backtrack_stack.push(1007)) return -1;
		if(!backtrack_stack.push(989)) return -1;
		if(!backtrack_stack.push(954)) return -1;
		if(!backtrack_stack.push(938)) return -1;
		if(!backtrack_stack.push(911)) return -1;
		if(!backtrack_stack.push(903)) return -1;
		if(!backtrack_stack.push(902)) return -1;
		if(!backtrack_stack.push(901)) return -1;
		if(!backtrack_stack.push(849)) return -1;
		if(!backtrack_stack.push(848)) return -1;
		if(!backtrack_stack.push(821)) return -1;
		if(!backtrack_stack.push(820)) return -1;
		if(!backtrack_stack.push(737)) return -1;
		if(!backtrack_stack.push(712)) return -1;
		if(!backtrack_stack.push(674)) return -1;
		if(!backtrack_stack.push(672)) return -1;
		if(!backtrack_stack.push(583)) return -1;
		if(!backtrack_stack.push(575)) return -1;
		if(!backtrack_stack.push(563)) return -1;
		if(!backtrack_stack.push(551)) return -1;
		if(!backtrack_stack.push(549)) return -1;
		if(!backtrack_stack.push(483)) return -1;
		if(!backtrack_stack.push(473)) return -1;
		if(!backtrack_stack.push(470)) return -1;
		if(!backtrack_stack.push(430)) return -1;
		if(!backtrack_stack.push(401)) return -1;
		if(!backtrack_stack.push(399)) return -1;
		if(!backtrack_stack.push(380)) return -1;
		if(!backtrack_stack.push(364)) return -1;
		if(!backtrack_stack.push(363)) return -1;
		if(!backtrack_stack.push(329)) return -1;
		if(!backtrack_stack.push(328)) return -1;
		if(!backtrack_stack.push(327)) return -1;
		if(!backtrack_stack.push(295)) return -1;
		if(!backtrack_stack.push(294)) return -1;
		if(!backtrack_stack.push(216)) return -1;
		if(!backtrack_stack.push(210)) return -1;
		if(!backtrack_stack.push(192)) return -1;
		if(!backtrack_stack.push(180)) return -1;
		if(!backtrack_stack.push(179)) return -1;
		if(!backtrack_stack.push(178)) return -1;
		if(!backtrack_stack.push(154)) return -1;
		if(!backtrack_stack.push(153)) return -1;
		if(!backtrack_stack.push(100)) return -1;
		if(!backtrack_stack.push(76)) return -1;
		if(!backtrack_stack.push(75)) return -1;
		if(!backtrack_stack.push(67)) return -1;
		if(!backtrack_stack.push(63)) return -1;
		if(!backtrack_stack.push(53)) return -1;
		if(!backtrack_stack.push(46)) return -1;
		if(!backtrack_stack.push(19)) return -1;
		goto label_16;
		}
		break;
	case 2508:
	// Label to restore values of label 482
		v_wine_1 = int_value_stack.pop();
		break;
	case 483: label_483: 
	// using  symbolic fact wine(v_wine_1) with literal 2 in rule 
	// 383: loire(A) :- q58(A), wine(A).
	// resulting in symbolic fact loire(v_loire_1)

		//recursive
		if(!int_value_stack.push(v_loire_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2509)) return -1;
		if (!q58_set.contains(v_wine_1)) continue;
	case 1496:
	// Continuation for label 483
			v_loire_1 = v_wine_1;
		if(loire_set.insert(v_loire_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1496)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(971)) return -1;
			if(!backtrack_stack.push(715)) return -1;
			if(!backtrack_stack.push(714)) return -1;
			if(!backtrack_stack.push(487)) return -1;
			if(!backtrack_stack.push(486)) return -1;
			if(!backtrack_stack.push(484)) return -1;
			if(!backtrack_stack.push(482)) return -1;
			goto label_478;
		}
		break;
	case 2509:
	// Label to restore values of label 483
		v_loire_1 = int_value_stack.pop();
		break;
	case 484: label_484: 
	// using  symbolic fact loire(v_loire_1) with literal 1 in rule 
	// 380: whiteloire(A) :- loire(A), whitewine(A).
	// resulting in symbolic fact whiteloire(v_whiteloire_1)

		//recursive
		if(!int_value_stack.push(v_whiteloire_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2510)) return -1;
		if (!whitewine_set.contains(v_loire_1)) continue;
	case 1497:
	// Continuation for label 484
			v_whiteloire_1 = v_loire_1;
		if(whiteloire_set.insert(v_whiteloire_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1497)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(492)) return -1;
			if(!backtrack_stack.push(490)) return -1;
			goto label_485;
		}
		break;
	case 2510:
	// Label to restore values of label 484
		v_whiteloire_1 = int_value_stack.pop();
		break;
	case 485: label_485: 
	// using  symbolic fact whiteloire(v_whiteloire_1) with literal 1 in rule 
	// 379: loire(A) :- whiteloire(A).
	// resulting in symbolic fact loire(v_loire_1)

		//recursive
		if(!int_value_stack.push(v_loire_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2511)) return -1;
		v_loire_1 = v_whiteloire_1;
		if(loire_set.insert(v_loire_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(971)) return -1;
		if(!backtrack_stack.push(715)) return -1;
		if(!backtrack_stack.push(714)) return -1;
		if(!backtrack_stack.push(487)) return -1;
		if(!backtrack_stack.push(486)) return -1;
		if(!backtrack_stack.push(484)) return -1;
		if(!backtrack_stack.push(482)) return -1;
		goto label_478;
		}
		break;
	case 2511:
	// Label to restore values of label 485
		v_loire_1 = int_value_stack.pop();
		break;
	case 486: label_486: 
	// using  symbolic fact loire(v_loire_1) with literal 1 in rule 
	// 618: loire(A) :- loire(B), kaon2equal(A,B).
	// resulting in symbolic fact loire(v_loire_1)

		//recursive
		c618_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_loire_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2512)) return -1;
		c618_2_fb.open(v_loire_1);
	case 1499:
	// Continuation for label 486
		while (c618_2_fb.fetch()) {
			v_loire_1 = c618_2_fb.c2();
		if(loire_set.insert(v_loire_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1499)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(971)) return -1;
			if(!backtrack_stack.push(715)) return -1;
			if(!backtrack_stack.push(714)) return -1;
			if(!backtrack_stack.push(487)) return -1;
			if(!backtrack_stack.push(486)) return -1;
			if(!backtrack_stack.push(484)) return -1;
			if(!backtrack_stack.push(482)) return -1;
			goto label_478;
		}
		}
		break;
	case 2512:
	// Label to restore values of label 486
		v_loire_1 = int_value_stack.pop();
		c618_2_fb.pop(&cursor_stack);
		break;
	case 487: label_487: 
	// using  symbolic fact loire(v_loire_1) with literal 2 in rule 
	// 84: muscadet(A) :- q56(A), loire(A).
	// resulting in symbolic fact muscadet(v_muscadet_1)

		//recursive
		if(!int_value_stack.push(v_muscadet_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2513)) return -1;
		if (!q56_set.contains(v_loire_1)) continue;
	case 1500:
	// Continuation for label 487
			v_muscadet_1 = v_loire_1;
		if(muscadet_set.insert(v_muscadet_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1500)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(489)) return -1;
			if(!backtrack_stack.push(488)) return -1;
			if(!backtrack_stack.push(477)) return -1;
			if(!backtrack_stack.push(458)) return -1;
			if(!backtrack_stack.push(457)) return -1;
			if(!backtrack_stack.push(456)) return -1;
			if(!backtrack_stack.push(445)) return -1;
			goto label_444;
		}
		break;
	case 2513:
	// Label to restore values of label 487
		v_muscadet_1 = int_value_stack.pop();
		break;
	case 488: label_488: 
	// using  symbolic fact muscadet(v_muscadet_1) with literal 1 in rule 
	// 631: q31(A) :- muscadet(A).
	// resulting in symbolic fact q31(v_q31_1)

		v_q31_1 = v_muscadet_1;
		if(q31_set.insert(v_q31_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(221)) return -1;
		if(!backtrack_stack.push(220)) return -1;
		if(!backtrack_stack.push(215)) return -1;
		if(!backtrack_stack.push(213)) return -1;
		goto label_201;
		}
		break;
	case 489: label_489: 
	// using  symbolic fact muscadet(v_muscadet_1) with literal 1 in rule 
	// 783: q3(A) :- muscadet(A).
	// resulting in symbolic fact q3(v_q3_1)

		v_q3_1 = v_muscadet_1;
		if(q3_set.insert(v_q3_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(143)) return -1;
		goto label_123;
		}
		break;
	case 490: label_490: 
	// using  symbolic fact whiteloire(v_whiteloire_1) with literal 1 in rule 
	// 450: whitewine(A) :- whiteloire(A).
	// resulting in symbolic fact whitewine(v_whitewine_1)

		//recursive
		if(!int_value_stack.push(v_whitewine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2516)) return -1;
		v_whitewine_1 = v_whiteloire_1;
		if(whitewine_set.insert(v_whitewine_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(560)) return -1;
		if(!backtrack_stack.push(491)) return -1;
		if(!backtrack_stack.push(374)) return -1;
		if(!backtrack_stack.push(235)) return -1;
		if(!backtrack_stack.push(113)) return -1;
		if(!backtrack_stack.push(112)) return -1;
		if(!backtrack_stack.push(104)) return -1;
		if(!backtrack_stack.push(103)) return -1;
		if(!backtrack_stack.push(101)) return -1;
		if(!backtrack_stack.push(98)) return -1;
		goto label_97;
		}
		break;
	case 2516:
	// Label to restore values of label 490
		v_whitewine_1 = int_value_stack.pop();
		break;
	case 491: label_491: 
	// using  symbolic fact whitewine(v_whitewine_1) with literal 2 in rule 
	// 380: whiteloire(A) :- loire(A), whitewine(A).
	// resulting in symbolic fact whiteloire(v_whiteloire_1)

		//recursive
		if(!int_value_stack.push(v_whiteloire_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2517)) return -1;
		if (!loire_set.contains(v_whitewine_1)) continue;
	case 1504:
	// Continuation for label 491
			v_whiteloire_1 = v_whitewine_1;
		if(whiteloire_set.insert(v_whiteloire_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1504)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(492)) return -1;
			if(!backtrack_stack.push(490)) return -1;
			goto label_485;
		}
		break;
	case 2517:
	// Label to restore values of label 491
		v_whiteloire_1 = int_value_stack.pop();
		break;
	case 492: label_492: 
	// using  symbolic fact whiteloire(v_whiteloire_1) with literal 1 in rule 
	// 451: whiteloire(A) :- whiteloire(B), kaon2equal(A,B).
	// resulting in symbolic fact whiteloire(v_whiteloire_1)

		//recursive
		c451_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_whiteloire_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2518)) return -1;
		c451_2_fb.open(v_whiteloire_1);
	case 1505:
	// Continuation for label 492
		while (c451_2_fb.fetch()) {
			v_whiteloire_1 = c451_2_fb.c2();
		if(whiteloire_set.insert(v_whiteloire_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1505)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(492)) return -1;
			if(!backtrack_stack.push(490)) return -1;
			goto label_485;
		}
		}
		break;
	case 2518:
	// Label to restore values of label 492
		v_whiteloire_1 = int_value_stack.pop();
		c451_2_fb.pop(&cursor_stack);
		break;
	case 493: label_493: 
	// using  symbolic fact q58(v_q58_1) with literal 1 in rule 
	// 391: q58(A) :- q58(B), kaon2equal(A,B).
	// resulting in symbolic fact q58(v_q58_1)

		//recursive
		c391_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q58_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2519)) return -1;
		c391_2_fb.open(v_q58_1);
	case 1506:
	// Continuation for label 493
		while (c391_2_fb.fetch()) {
			v_q58_1 = c391_2_fb.c2();
		if(q58_set.insert(v_q58_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1506)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(494)) return -1;
			if(!backtrack_stack.push(493)) return -1;
			goto label_481;
		}
		}
		break;
	case 2519:
	// Label to restore values of label 493
		v_q58_1 = int_value_stack.pop();
		c391_2_fb.pop(&cursor_stack);
		break;
	case 494: label_494: 
	// using  symbolic fact q58(v_q58_1) with literal 2 in rule 
	// 385: q57(A) :- locatedin(A,B), q58(B).
	// resulting in symbolic fact q57(v_q57_1)

		//recursive
		c385_1_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q57_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2520)) return -1;
		c385_1_fb.open(v_q58_1);
	case 1507:
	// Continuation for label 494
		while (c385_1_fb.fetch()) {
			v_q57_1 = c385_1_fb.c2();
		if(q57_set.insert(v_q57_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1507)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(498)) return -1;
			if(!backtrack_stack.push(495)) return -1;
			if(!backtrack_stack.push(480)) return -1;
			goto label_479;
		}
		}
		break;
	case 2520:
	// Label to restore values of label 494
		v_q57_1 = int_value_stack.pop();
		c385_1_fb.pop(&cursor_stack);
		break;
	case 495: label_495: 
	// using  symbolic fact q57(v_q57_1) with literal 1 in rule 
	// 388: ot____nom19(A) :- q57(A), kaon2namedobjects(A).
	// resulting in symbolic fact ot____nom19(v_ot____nom19_1)

		//recursive
		if(!int_value_stack.push(v_ot____nom19_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2521)) return -1;
		if (!kaon2namedobjects_b.contains(v_q57_1)) continue;
	case 1508:
	// Continuation for label 495
			v_ot____nom19_1 = v_q57_1;
		if(ot____nom19_set.insert(v_ot____nom19_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1508)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(497)) return -1;
			goto label_496;
		}
		break;
	case 2521:
	// Label to restore values of label 495
		v_ot____nom19_1 = int_value_stack.pop();
		break;
	case 496: label_496: 
	// using  symbolic fact ot____nom19(v_ot____nom19_1) with literal 1 in rule 
	// 390: ot____nom19(A) :- ot____nom19(B), kaon2equal(A,B).
	// resulting in symbolic fact ot____nom19(v_ot____nom19_1)

		//recursive
		c390_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_ot____nom19_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2522)) return -1;
		c390_2_fb.open(v_ot____nom19_1);
	case 1509:
	// Continuation for label 496
		while (c390_2_fb.fetch()) {
			v_ot____nom19_1 = c390_2_fb.c2();
		if(ot____nom19_set.insert(v_ot____nom19_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1509)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(497)) return -1;
			goto label_496;
		}
		}
		break;
	case 2522:
	// Label to restore values of label 496
		v_ot____nom19_1 = int_value_stack.pop();
		c390_2_fb.pop(&cursor_stack);
		break;
	case 497: label_497: 
	// using  symbolic fact ot____nom19(v_ot____nom19_1) with literal 2 in rule 
	// 386: q58(A) :- locatedin(A,B), ot____nom19(B).
	// resulting in symbolic fact q58(v_q58_1)

		//recursive
		c386_1_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q58_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2523)) return -1;
		c386_1_fb.open(v_ot____nom19_1);
	case 1510:
	// Continuation for label 497
		while (c386_1_fb.fetch()) {
			v_q58_1 = c386_1_fb.c2();
		if(q58_set.insert(v_q58_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1510)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(494)) return -1;
			if(!backtrack_stack.push(493)) return -1;
			goto label_481;
		}
		}
		break;
	case 2523:
	// Label to restore values of label 497
		v_q58_1 = int_value_stack.pop();
		c386_1_fb.pop(&cursor_stack);
		break;
	case 498: label_498: 
	// using  symbolic fact q57(v_q57_1) with literal 1 in rule 
	// 389: q57(A) :- q57(B), kaon2equal(A,B).
	// resulting in symbolic fact q57(v_q57_1)

		//recursive
		c389_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q57_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2524)) return -1;
		c389_2_fb.open(v_q57_1);
	case 1511:
	// Continuation for label 498
		while (c389_2_fb.fetch()) {
			v_q57_1 = c389_2_fb.c2();
		if(q57_set.insert(v_q57_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1511)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(498)) return -1;
			if(!backtrack_stack.push(495)) return -1;
			if(!backtrack_stack.push(480)) return -1;
			goto label_479;
		}
		}
		break;
	case 2524:
	// Label to restore values of label 498
		v_q57_1 = int_value_stack.pop();
		c389_2_fb.pop(&cursor_stack);
		break;
	case 499:
	// applying rule 
	// 90: margaux(A) :- margaux_aux(A).
	// resulting in symbolic fact margaux(v_margaux_1)
		c90_1_f.open();
	case 1512:
	// Continuation for label 499
		while (c90_1_f.fetch()) {
			v_margaux_1 = c90_1_f.c1();
		if(margaux_set.insert(v_margaux_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1512)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(555)) return -1;
			if(!backtrack_stack.push(535)) return -1;
			if(!backtrack_stack.push(515)) return -1;
			if(!backtrack_stack.push(514)) return -1;
			if(!backtrack_stack.push(501)) return -1;
			goto label_500;
		}
		}
		break;
	case 500: label_500: 
	// using  symbolic fact margaux(v_margaux_1) with literal 1 in rule 
	// 89: q3(A) :- margaux(A).
	// resulting in symbolic fact q3(v_q3_1)

		v_q3_1 = v_margaux_1;
		if(q3_set.insert(v_q3_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(143)) return -1;
		goto label_123;
		}
		break;
	case 501: label_501: 
	// using  symbolic fact margaux(v_margaux_1) with literal 1 in rule 
	// 93: q18(A) :- margaux(A).
	// resulting in symbolic fact q18(v_q18_1)

		v_q18_1 = v_margaux_1;
		if(q18_set.insert(v_q18_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(508)) return -1;
		if(!backtrack_stack.push(507)) return -1;
		if(!backtrack_stack.push(505)) return -1;
		goto label_502;
		}
		break;
	case 502: label_502: 
	// using  symbolic fact q18(v_q18_1) with literal 1 in rule 
	// 92: q19(A) :- q18(A).
	// resulting in symbolic fact q19(v_q19_1)

		//recursive
		if(!int_value_stack.push(v_q19_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2528)) return -1;
		v_q19_1 = v_q18_1;
		if(q19_set.insert(v_q19_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(504)) return -1;
		goto label_503;
		}
		break;
	case 2528:
	// Label to restore values of label 502
		v_q19_1 = int_value_stack.pop();
		break;
	case 503: label_503: 
	// using  symbolic fact q19(v_q19_1) with literal 1 in rule 
	// 765: q19(A) :- q19(B), kaon2equal(A,B).
	// resulting in symbolic fact q19(v_q19_1)

		//recursive
		c765_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q19_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2529)) return -1;
		c765_2_fb.open(v_q19_1);
	case 1516:
	// Continuation for label 503
		while (c765_2_fb.fetch()) {
			v_q19_1 = c765_2_fb.c2();
		if(q19_set.insert(v_q19_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1516)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(504)) return -1;
			goto label_503;
		}
		}
		break;
	case 2529:
	// Label to restore values of label 503
		v_q19_1 = int_value_stack.pop();
		c765_2_fb.pop(&cursor_stack);
		break;
	case 504: label_504: 
	// using  symbolic fact q19(v_q19_1) with literal 2 in rule 
	// 97: q18(A) :- locatedin(A,B), q19(B).
	// resulting in symbolic fact q18(v_q18_1)

		//recursive
		c97_1_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q18_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2530)) return -1;
		c97_1_fb.open(v_q19_1);
	case 1517:
	// Continuation for label 504
		while (c97_1_fb.fetch()) {
			v_q18_1 = c97_1_fb.c2();
		if(q18_set.insert(v_q18_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1517)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(508)) return -1;
			if(!backtrack_stack.push(507)) return -1;
			if(!backtrack_stack.push(505)) return -1;
			goto label_502;
		}
		}
		break;
	case 2530:
	// Label to restore values of label 504
		v_q18_1 = int_value_stack.pop();
		c97_1_fb.pop(&cursor_stack);
		break;
	case 505: label_505: 
	// using  symbolic fact q18(v_q18_1) with literal 1 in rule 
	// 148: locatedin(A,A) :- q18(A), kaon2namedobjects(A).
	// resulting in symbolic fact locatedin(v_locatedin_1,v_locatedin_2)

		//recursive
		if(!int_value_stack.push(v_locatedin_2)) return -1;
		if(!int_value_stack.push(v_locatedin_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2531)) return -1;
		if (!kaon2namedobjects_b.contains(v_q18_1)) continue;
	case 1518:
	// Continuation for label 505
			v_locatedin_1 = v_q18_1;
			v_locatedin_2 = v_q18_1;
		if(locatedin_set.insert(v_locatedin_1, v_locatedin_2)) {
			if(!locatedin_bf.insert(v_locatedin_1, v_locatedin_2)) return -1;
			if(!locatedin_fb.insert(v_locatedin_2, v_locatedin_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1518)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(1005)) return -1;
			if(!backtrack_stack.push(999)) return -1;
			if(!backtrack_stack.push(987)) return -1;
			if(!backtrack_stack.push(981)) return -1;
			if(!backtrack_stack.push(969)) return -1;
			if(!backtrack_stack.push(963)) return -1;
			if(!backtrack_stack.push(952)) return -1;
			if(!backtrack_stack.push(946)) return -1;
			if(!backtrack_stack.push(936)) return -1;
			if(!backtrack_stack.push(930)) return -1;
			if(!backtrack_stack.push(920)) return -1;
			if(!backtrack_stack.push(917)) return -1;
			if(!backtrack_stack.push(898)) return -1;
			if(!backtrack_stack.push(897)) return -1;
			if(!backtrack_stack.push(896)) return -1;
			if(!backtrack_stack.push(893)) return -1;
			if(!backtrack_stack.push(726)) return -1;
			if(!backtrack_stack.push(721)) return -1;
			if(!backtrack_stack.push(694)) return -1;
			if(!backtrack_stack.push(693)) return -1;
			if(!backtrack_stack.push(688)) return -1;
			if(!backtrack_stack.push(687)) return -1;
			if(!backtrack_stack.push(636)) return -1;
			if(!backtrack_stack.push(631)) return -1;
			if(!backtrack_stack.push(592)) return -1;
			if(!backtrack_stack.push(591)) return -1;
			if(!backtrack_stack.push(529)) return -1;
			if(!backtrack_stack.push(528)) return -1;
			if(!backtrack_stack.push(513)) return -1;
			if(!backtrack_stack.push(512)) return -1;
			if(!backtrack_stack.push(511)) return -1;
			if(!backtrack_stack.push(506)) return -1;
			if(!backtrack_stack.push(455)) return -1;
			if(!backtrack_stack.push(450)) return -1;
			if(!backtrack_stack.push(441)) return -1;
			if(!backtrack_stack.push(440)) return -1;
			if(!backtrack_stack.push(439)) return -1;
			if(!backtrack_stack.push(434)) return -1;
			if(!backtrack_stack.push(339)) return -1;
			if(!backtrack_stack.push(338)) return -1;
			if(!backtrack_stack.push(249)) return -1;
			if(!backtrack_stack.push(244)) return -1;
			if(!backtrack_stack.push(52)) return -1;
			if(!backtrack_stack.push(51)) return -1;
			if(!backtrack_stack.push(50)) return -1;
			if(!backtrack_stack.push(49)) return -1;
			if(!backtrack_stack.push(48)) return -1;
			goto label_47;
		}
		break;
	case 2531:
	// Label to restore values of label 505
		v_locatedin_1 = int_value_stack.pop();
		v_locatedin_2 = int_value_stack.pop();
		break;
	case 506: label_506: 
	// using  symbolic fact locatedin(v_locatedin_1,v_locatedin_2) with literal 1 in rule 
	// 97: q18(A) :- locatedin(A,B), q19(B).
	// resulting in symbolic fact q18(v_q18_1)

		//recursive
		if(!int_value_stack.push(v_q18_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2532)) return -1;
		if (!q19_set.contains(v_locatedin_2)) continue;
	case 1519:
	// Continuation for label 506
			v_q18_1 = v_locatedin_1;
		if(q18_set.insert(v_q18_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1519)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(508)) return -1;
			if(!backtrack_stack.push(507)) return -1;
			if(!backtrack_stack.push(505)) return -1;
			goto label_502;
		}
		break;
	case 2532:
	// Label to restore values of label 506
		v_q18_1 = int_value_stack.pop();
		break;
	case 507: label_507: 
	// using  symbolic fact q18(v_q18_1) with literal 1 in rule 
	// 149: q18(A) :- q18(B), kaon2equal(A,B).
	// resulting in symbolic fact q18(v_q18_1)

		//recursive
		c149_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q18_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2533)) return -1;
		c149_2_fb.open(v_q18_1);
	case 1520:
	// Continuation for label 507
		while (c149_2_fb.fetch()) {
			v_q18_1 = c149_2_fb.c2();
		if(q18_set.insert(v_q18_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1520)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(508)) return -1;
			if(!backtrack_stack.push(507)) return -1;
			if(!backtrack_stack.push(505)) return -1;
			goto label_502;
		}
		}
		break;
	case 2533:
	// Label to restore values of label 507
		v_q18_1 = int_value_stack.pop();
		c149_2_fb.pop(&cursor_stack);
		break;
	case 508: label_508: 
	// using  symbolic fact q18(v_q18_1) with literal 1 in rule 
	// 763: ot____nom22(A) :- q18(A), kaon2namedobjects(A).
	// resulting in symbolic fact ot____nom22(v_ot____nom22_1)

		//recursive
		if(!int_value_stack.push(v_ot____nom22_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2534)) return -1;
		if (!kaon2namedobjects_b.contains(v_q18_1)) continue;
	case 1521:
	// Continuation for label 508
			v_ot____nom22_1 = v_q18_1;
		if(ot____nom22_set.insert(v_ot____nom22_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1521)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(510)) return -1;
			goto label_509;
		}
		break;
	case 2534:
	// Label to restore values of label 508
		v_ot____nom22_1 = int_value_stack.pop();
		break;
	case 509: label_509: 
	// using  symbolic fact ot____nom22(v_ot____nom22_1) with literal 1 in rule 
	// 764: ot____nom22(A) :- ot____nom22(B), kaon2equal(A,B).
	// resulting in symbolic fact ot____nom22(v_ot____nom22_1)

		//recursive
		c764_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_ot____nom22_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2535)) return -1;
		c764_2_fb.open(v_ot____nom22_1);
	case 1522:
	// Continuation for label 509
		while (c764_2_fb.fetch()) {
			v_ot____nom22_1 = c764_2_fb.c2();
		if(ot____nom22_set.insert(v_ot____nom22_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1522)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(510)) return -1;
			goto label_509;
		}
		}
		break;
	case 2535:
	// Label to restore values of label 509
		v_ot____nom22_1 = int_value_stack.pop();
		c764_2_fb.pop(&cursor_stack);
		break;
	case 510: label_510: 
	// using  symbolic fact ot____nom22(v_ot____nom22_1) with literal 2 in rule 
	// 761: q19(A) :- locatedin(A,B), ot____nom22(B).
	// resulting in symbolic fact q19(v_q19_1)

		//recursive
		c761_1_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q19_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2536)) return -1;
		c761_1_fb.open(v_ot____nom22_1);
	case 1523:
	// Continuation for label 510
		while (c761_1_fb.fetch()) {
			v_q19_1 = c761_1_fb.c2();
		if(q19_set.insert(v_q19_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1523)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(504)) return -1;
			goto label_503;
		}
		}
		break;
	case 2536:
	// Label to restore values of label 510
		v_q19_1 = int_value_stack.pop();
		c761_1_fb.pop(&cursor_stack);
		break;
	case 511: label_511: 
	// using  symbolic fact locatedin(v_locatedin_1,v_locatedin_2) with literal 1 in rule 
	// 385: q57(A) :- locatedin(A,B), q58(B).
	// resulting in symbolic fact q57(v_q57_1)

		if (!q58_set.contains(v_locatedin_2)) continue;
	case 1524:
	// Continuation for label 511
			v_q57_1 = v_locatedin_1;
		if(q57_set.insert(v_q57_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1524)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(498)) return -1;
			if(!backtrack_stack.push(495)) return -1;
			if(!backtrack_stack.push(480)) return -1;
			goto label_479;
		}
		break;
	case 512: label_512: 
	// using  symbolic fact locatedin(v_locatedin_1,v_locatedin_2) with literal 1 in rule 
	// 386: q58(A) :- locatedin(A,B), ot____nom19(B).
	// resulting in symbolic fact q58(v_q58_1)

		if (!ot____nom19_set.contains(v_locatedin_2)) continue;
	case 1525:
	// Continuation for label 512
			v_q58_1 = v_locatedin_1;
		if(q58_set.insert(v_q58_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1525)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(494)) return -1;
			if(!backtrack_stack.push(493)) return -1;
			goto label_481;
		}
		break;
	case 513: label_513: 
	// using  symbolic fact locatedin(v_locatedin_1,v_locatedin_2) with literal 1 in rule 
	// 761: q19(A) :- locatedin(A,B), ot____nom22(B).
	// resulting in symbolic fact q19(v_q19_1)

		//recursive
		if(!int_value_stack.push(v_q19_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2539)) return -1;
		if (!ot____nom22_set.contains(v_locatedin_2)) continue;
	case 1526:
	// Continuation for label 513
			v_q19_1 = v_locatedin_1;
		if(q19_set.insert(v_q19_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1526)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(504)) return -1;
			goto label_503;
		}
		break;
	case 2539:
	// Label to restore values of label 513
		v_q19_1 = int_value_stack.pop();
		break;
	case 514: label_514: 
	// using  symbolic fact margaux(v_margaux_1) with literal 1 in rule 
	// 94: margaux(A) :- margaux(B), kaon2equal(A,B).
	// resulting in symbolic fact margaux(v_margaux_1)

		//recursive
		c94_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_margaux_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2540)) return -1;
		c94_2_fb.open(v_margaux_1);
	case 1527:
	// Continuation for label 514
		while (c94_2_fb.fetch()) {
			v_margaux_1 = c94_2_fb.c2();
		if(margaux_set.insert(v_margaux_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1527)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(555)) return -1;
			if(!backtrack_stack.push(535)) return -1;
			if(!backtrack_stack.push(515)) return -1;
			if(!backtrack_stack.push(514)) return -1;
			if(!backtrack_stack.push(501)) return -1;
			goto label_500;
		}
		}
		break;
	case 2540:
	// Label to restore values of label 514
		v_margaux_1 = int_value_stack.pop();
		c94_2_fb.pop(&cursor_stack);
		break;
	case 515: label_515: 
	// using  symbolic fact margaux(v_margaux_1) with literal 1 in rule 
	// 201: medoc(A) :- margaux(A).
	// resulting in symbolic fact medoc(v_medoc_1)

		//recursive
		if(!int_value_stack.push(v_medoc_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2541)) return -1;
		v_medoc_1 = v_margaux_1;
		if(medoc_set.insert(v_medoc_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(534)) return -1;
		if(!backtrack_stack.push(533)) return -1;
		if(!backtrack_stack.push(532)) return -1;
		if(!backtrack_stack.push(531)) return -1;
		if(!backtrack_stack.push(525)) return -1;
		goto label_516;
		}
		break;
	case 2541:
	// Label to restore values of label 515
		v_medoc_1 = int_value_stack.pop();
		break;
	case 516: label_516: 
	// using  symbolic fact medoc(v_medoc_1) with literal 1 in rule 
	// 126: q33(A) :- medoc(A).
	// resulting in symbolic fact q33(v_q33_1)

		v_q33_1 = v_medoc_1;
		if(q33_set.insert(v_q33_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(524)) return -1;
		if(!backtrack_stack.push(521)) return -1;
		if(!backtrack_stack.push(518)) return -1;
		goto label_517;
		}
		break;
	case 517: label_517: 
	// using  symbolic fact q33(v_q33_1) with literal 1 in rule 
	// 125: locatedin(A,A) :- q33(A), kaon2namedobjects(A).
	// resulting in symbolic fact locatedin(v_locatedin_1,v_locatedin_2)

		if (!kaon2namedobjects_b.contains(v_q33_1)) continue;
	case 1530:
	// Continuation for label 517
			v_locatedin_1 = v_q33_1;
			v_locatedin_2 = v_q33_1;
		if(locatedin_set.insert(v_locatedin_1, v_locatedin_2)) {
			if(!locatedin_bf.insert(v_locatedin_1, v_locatedin_2)) return -1;
			if(!locatedin_fb.insert(v_locatedin_2, v_locatedin_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1530)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(1005)) return -1;
			if(!backtrack_stack.push(999)) return -1;
			if(!backtrack_stack.push(987)) return -1;
			if(!backtrack_stack.push(981)) return -1;
			if(!backtrack_stack.push(969)) return -1;
			if(!backtrack_stack.push(963)) return -1;
			if(!backtrack_stack.push(952)) return -1;
			if(!backtrack_stack.push(946)) return -1;
			if(!backtrack_stack.push(936)) return -1;
			if(!backtrack_stack.push(930)) return -1;
			if(!backtrack_stack.push(920)) return -1;
			if(!backtrack_stack.push(917)) return -1;
			if(!backtrack_stack.push(898)) return -1;
			if(!backtrack_stack.push(897)) return -1;
			if(!backtrack_stack.push(896)) return -1;
			if(!backtrack_stack.push(893)) return -1;
			if(!backtrack_stack.push(726)) return -1;
			if(!backtrack_stack.push(721)) return -1;
			if(!backtrack_stack.push(694)) return -1;
			if(!backtrack_stack.push(693)) return -1;
			if(!backtrack_stack.push(688)) return -1;
			if(!backtrack_stack.push(687)) return -1;
			if(!backtrack_stack.push(636)) return -1;
			if(!backtrack_stack.push(631)) return -1;
			if(!backtrack_stack.push(592)) return -1;
			if(!backtrack_stack.push(591)) return -1;
			if(!backtrack_stack.push(529)) return -1;
			if(!backtrack_stack.push(528)) return -1;
			if(!backtrack_stack.push(513)) return -1;
			if(!backtrack_stack.push(512)) return -1;
			if(!backtrack_stack.push(511)) return -1;
			if(!backtrack_stack.push(506)) return -1;
			if(!backtrack_stack.push(455)) return -1;
			if(!backtrack_stack.push(450)) return -1;
			if(!backtrack_stack.push(441)) return -1;
			if(!backtrack_stack.push(440)) return -1;
			if(!backtrack_stack.push(439)) return -1;
			if(!backtrack_stack.push(434)) return -1;
			if(!backtrack_stack.push(339)) return -1;
			if(!backtrack_stack.push(338)) return -1;
			if(!backtrack_stack.push(249)) return -1;
			if(!backtrack_stack.push(244)) return -1;
			if(!backtrack_stack.push(52)) return -1;
			if(!backtrack_stack.push(51)) return -1;
			if(!backtrack_stack.push(50)) return -1;
			if(!backtrack_stack.push(49)) return -1;
			if(!backtrack_stack.push(48)) return -1;
			goto label_47;
		}
		break;
	case 518: label_518: 
	// using  symbolic fact q33(v_q33_1) with literal 1 in rule 
	// 203: q34(A) :- q33(A).
	// resulting in symbolic fact q34(v_q34_1)

		//recursive
		if(!int_value_stack.push(v_q34_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2544)) return -1;
		v_q34_1 = v_q33_1;
		if(q34_set.insert(v_q34_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(530)) return -1;
		if(!backtrack_stack.push(520)) return -1;
		goto label_519;
		}
		break;
	case 2544:
	// Label to restore values of label 518
		v_q34_1 = int_value_stack.pop();
		break;
	case 519: label_519: 
	// using  symbolic fact q34(v_q34_1) with literal 1 in rule 
	// 673: q34(A) :- q34(B), kaon2equal(A,B).
	// resulting in symbolic fact q34(v_q34_1)

		//recursive
		c673_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q34_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2545)) return -1;
		c673_2_fb.open(v_q34_1);
	case 1532:
	// Continuation for label 519
		while (c673_2_fb.fetch()) {
			v_q34_1 = c673_2_fb.c2();
		if(q34_set.insert(v_q34_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1532)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(530)) return -1;
			if(!backtrack_stack.push(520)) return -1;
			goto label_519;
		}
		}
		break;
	case 2545:
	// Label to restore values of label 519
		v_q34_1 = int_value_stack.pop();
		c673_2_fb.pop(&cursor_stack);
		break;
	case 520: label_520: 
	// using  symbolic fact q34(v_q34_1) with literal 2 in rule 
	// 204: q33(A) :- locatedin(A,B), q34(B).
	// resulting in symbolic fact q33(v_q33_1)

		//recursive
		c204_1_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q33_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2546)) return -1;
		c204_1_fb.open(v_q34_1);
	case 1533:
	// Continuation for label 520
		while (c204_1_fb.fetch()) {
			v_q33_1 = c204_1_fb.c2();
		if(q33_set.insert(v_q33_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1533)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(524)) return -1;
			if(!backtrack_stack.push(521)) return -1;
			if(!backtrack_stack.push(518)) return -1;
			goto label_517;
		}
		}
		break;
	case 2546:
	// Label to restore values of label 520
		v_q33_1 = int_value_stack.pop();
		c204_1_fb.pop(&cursor_stack);
		break;
	case 521: label_521: 
	// using  symbolic fact q33(v_q33_1) with literal 1 in rule 
	// 670: ot____nom2(A) :- q33(A), kaon2namedobjects(A).
	// resulting in symbolic fact ot____nom2(v_ot____nom2_1)

		//recursive
		if(!int_value_stack.push(v_ot____nom2_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2547)) return -1;
		if (!kaon2namedobjects_b.contains(v_q33_1)) continue;
	case 1534:
	// Continuation for label 521
			v_ot____nom2_1 = v_q33_1;
		if(ot____nom2_set.insert(v_ot____nom2_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1534)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(523)) return -1;
			goto label_522;
		}
		break;
	case 2547:
	// Label to restore values of label 521
		v_ot____nom2_1 = int_value_stack.pop();
		break;
	case 522: label_522: 
	// using  symbolic fact ot____nom2(v_ot____nom2_1) with literal 1 in rule 
	// 672: ot____nom2(A) :- ot____nom2(B), kaon2equal(A,B).
	// resulting in symbolic fact ot____nom2(v_ot____nom2_1)

		//recursive
		c672_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_ot____nom2_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2548)) return -1;
		c672_2_fb.open(v_ot____nom2_1);
	case 1535:
	// Continuation for label 522
		while (c672_2_fb.fetch()) {
			v_ot____nom2_1 = c672_2_fb.c2();
		if(ot____nom2_set.insert(v_ot____nom2_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1535)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(523)) return -1;
			goto label_522;
		}
		}
		break;
	case 2548:
	// Label to restore values of label 522
		v_ot____nom2_1 = int_value_stack.pop();
		c672_2_fb.pop(&cursor_stack);
		break;
	case 523: label_523: 
	// using  symbolic fact ot____nom2(v_ot____nom2_1) with literal 2 in rule 
	// 668: q34(A) :- locatedin(A,B), ot____nom2(B).
	// resulting in symbolic fact q34(v_q34_1)

		//recursive
		c668_1_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q34_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2549)) return -1;
		c668_1_fb.open(v_ot____nom2_1);
	case 1536:
	// Continuation for label 523
		while (c668_1_fb.fetch()) {
			v_q34_1 = c668_1_fb.c2();
		if(q34_set.insert(v_q34_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1536)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(530)) return -1;
			if(!backtrack_stack.push(520)) return -1;
			goto label_519;
		}
		}
		break;
	case 2549:
	// Label to restore values of label 523
		v_q34_1 = int_value_stack.pop();
		c668_1_fb.pop(&cursor_stack);
		break;
	case 524: label_524: 
	// using  symbolic fact q33(v_q33_1) with literal 1 in rule 
	// 671: q33(A) :- q33(B), kaon2equal(A,B).
	// resulting in symbolic fact q33(v_q33_1)

		//recursive
		c671_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q33_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2550)) return -1;
		c671_2_fb.open(v_q33_1);
	case 1537:
	// Continuation for label 524
		while (c671_2_fb.fetch()) {
			v_q33_1 = c671_2_fb.c2();
		if(q33_set.insert(v_q33_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1537)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(524)) return -1;
			if(!backtrack_stack.push(521)) return -1;
			if(!backtrack_stack.push(518)) return -1;
			goto label_517;
		}
		}
		break;
	case 2550:
	// Label to restore values of label 524
		v_q33_1 = int_value_stack.pop();
		c671_2_fb.pop(&cursor_stack);
		break;
	case 525: label_525: 
	// using  symbolic fact medoc(v_medoc_1) with literal 1 in rule 
	// 200: bordeaux(A) :- medoc(A).
	// resulting in symbolic fact bordeaux(v_bordeaux_1)

		//recursive
		if(!int_value_stack.push(v_bordeaux_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2551)) return -1;
		v_bordeaux_1 = v_medoc_1;
		if(bordeaux_set.insert(v_bordeaux_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(781)) return -1;
		if(!backtrack_stack.push(566)) return -1;
		if(!backtrack_stack.push(565)) return -1;
		if(!backtrack_stack.push(564)) return -1;
		if(!backtrack_stack.push(562)) return -1;
		if(!backtrack_stack.push(558)) return -1;
		goto label_526;
		}
		break;
	case 2551:
	// Label to restore values of label 525
		v_bordeaux_1 = int_value_stack.pop();
		break;
	case 526: label_526: 
	// using  symbolic fact bordeaux(v_bordeaux_1) with literal 1 in rule 
	// 196: q1(A) :- bordeaux(A).
	// resulting in symbolic fact q1(v_q1_1)

		//recursive
		if(!int_value_stack.push(v_q1_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2552)) return -1;
		v_q1_1 = v_bordeaux_1;
		if(q1_set.insert(v_q1_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(589)) return -1;
		if(!backtrack_stack.push(586)) return -1;
		if(!backtrack_stack.push(556)) return -1;
		goto label_527;
		}
		break;
	case 2552:
	// Label to restore values of label 526
		v_q1_1 = int_value_stack.pop();
		break;
	case 527: label_527: 
	// using  symbolic fact q1(v_q1_1) with literal 1 in rule 
	// 195: locatedin(A,A) :- q1(A), kaon2namedobjects(A).
	// resulting in symbolic fact locatedin(v_locatedin_1,v_locatedin_2)

		//recursive
		if(!int_value_stack.push(v_locatedin_2)) return -1;
		if(!int_value_stack.push(v_locatedin_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2553)) return -1;
		if (!kaon2namedobjects_b.contains(v_q1_1)) continue;
	case 1540:
	// Continuation for label 527
			v_locatedin_1 = v_q1_1;
			v_locatedin_2 = v_q1_1;
		if(locatedin_set.insert(v_locatedin_1, v_locatedin_2)) {
			if(!locatedin_bf.insert(v_locatedin_1, v_locatedin_2)) return -1;
			if(!locatedin_fb.insert(v_locatedin_2, v_locatedin_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1540)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(1005)) return -1;
			if(!backtrack_stack.push(999)) return -1;
			if(!backtrack_stack.push(987)) return -1;
			if(!backtrack_stack.push(981)) return -1;
			if(!backtrack_stack.push(969)) return -1;
			if(!backtrack_stack.push(963)) return -1;
			if(!backtrack_stack.push(952)) return -1;
			if(!backtrack_stack.push(946)) return -1;
			if(!backtrack_stack.push(936)) return -1;
			if(!backtrack_stack.push(930)) return -1;
			if(!backtrack_stack.push(920)) return -1;
			if(!backtrack_stack.push(917)) return -1;
			if(!backtrack_stack.push(898)) return -1;
			if(!backtrack_stack.push(897)) return -1;
			if(!backtrack_stack.push(896)) return -1;
			if(!backtrack_stack.push(893)) return -1;
			if(!backtrack_stack.push(726)) return -1;
			if(!backtrack_stack.push(721)) return -1;
			if(!backtrack_stack.push(694)) return -1;
			if(!backtrack_stack.push(693)) return -1;
			if(!backtrack_stack.push(688)) return -1;
			if(!backtrack_stack.push(687)) return -1;
			if(!backtrack_stack.push(636)) return -1;
			if(!backtrack_stack.push(631)) return -1;
			if(!backtrack_stack.push(592)) return -1;
			if(!backtrack_stack.push(591)) return -1;
			if(!backtrack_stack.push(529)) return -1;
			if(!backtrack_stack.push(528)) return -1;
			if(!backtrack_stack.push(513)) return -1;
			if(!backtrack_stack.push(512)) return -1;
			if(!backtrack_stack.push(511)) return -1;
			if(!backtrack_stack.push(506)) return -1;
			if(!backtrack_stack.push(455)) return -1;
			if(!backtrack_stack.push(450)) return -1;
			if(!backtrack_stack.push(441)) return -1;
			if(!backtrack_stack.push(440)) return -1;
			if(!backtrack_stack.push(439)) return -1;
			if(!backtrack_stack.push(434)) return -1;
			if(!backtrack_stack.push(339)) return -1;
			if(!backtrack_stack.push(338)) return -1;
			if(!backtrack_stack.push(249)) return -1;
			if(!backtrack_stack.push(244)) return -1;
			if(!backtrack_stack.push(52)) return -1;
			if(!backtrack_stack.push(51)) return -1;
			if(!backtrack_stack.push(50)) return -1;
			if(!backtrack_stack.push(49)) return -1;
			if(!backtrack_stack.push(48)) return -1;
			goto label_47;
		}
		break;
	case 2553:
	// Label to restore values of label 527
		v_locatedin_1 = int_value_stack.pop();
		v_locatedin_2 = int_value_stack.pop();
		break;
	case 528: label_528: 
	// using  symbolic fact locatedin(v_locatedin_1,v_locatedin_2) with literal 1 in rule 
	// 204: q33(A) :- locatedin(A,B), q34(B).
	// resulting in symbolic fact q33(v_q33_1)

		if (!q34_set.contains(v_locatedin_2)) continue;
	case 1541:
	// Continuation for label 528
			v_q33_1 = v_locatedin_1;
		if(q33_set.insert(v_q33_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1541)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(524)) return -1;
			if(!backtrack_stack.push(521)) return -1;
			if(!backtrack_stack.push(518)) return -1;
			goto label_517;
		}
		break;
	case 529: label_529: 
	// using  symbolic fact locatedin(v_locatedin_1,v_locatedin_2) with literal 1 in rule 
	// 668: q34(A) :- locatedin(A,B), ot____nom2(B).
	// resulting in symbolic fact q34(v_q34_1)

		//recursive
		if(!int_value_stack.push(v_q34_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2555)) return -1;
		if (!ot____nom2_set.contains(v_locatedin_2)) continue;
	case 1542:
	// Continuation for label 529
			v_q34_1 = v_locatedin_1;
		if(q34_set.insert(v_q34_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1542)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(530)) return -1;
			if(!backtrack_stack.push(520)) return -1;
			goto label_519;
		}
		break;
	case 2555:
	// Label to restore values of label 529
		v_q34_1 = int_value_stack.pop();
		break;
	case 530: label_530: 
	// using  symbolic fact q34(v_q34_1) with literal 1 in rule 
	// 202: medoc(A) :- q34(A), bordeaux(A).
	// resulting in symbolic fact medoc(v_medoc_1)

		//recursive
		if(!int_value_stack.push(v_medoc_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2556)) return -1;
		if (!bordeaux_set.contains(v_q34_1)) continue;
	case 1543:
	// Continuation for label 530
			v_medoc_1 = v_q34_1;
		if(medoc_set.insert(v_medoc_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1543)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(534)) return -1;
			if(!backtrack_stack.push(533)) return -1;
			if(!backtrack_stack.push(532)) return -1;
			if(!backtrack_stack.push(531)) return -1;
			if(!backtrack_stack.push(525)) return -1;
			goto label_516;
		}
		break;
	case 2556:
	// Label to restore values of label 530
		v_medoc_1 = int_value_stack.pop();
		break;
	case 531: label_531: 
	// using  symbolic fact medoc(v_medoc_1) with literal 1 in rule 
	// 270: q31(A) :- medoc(A).
	// resulting in symbolic fact q31(v_q31_1)

		v_q31_1 = v_medoc_1;
		if(q31_set.insert(v_q31_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(221)) return -1;
		if(!backtrack_stack.push(220)) return -1;
		if(!backtrack_stack.push(215)) return -1;
		if(!backtrack_stack.push(213)) return -1;
		goto label_201;
		}
		break;
	case 532: label_532: 
	// using  symbolic fact medoc(v_medoc_1) with literal 1 in rule 
	// 271: medoc(A) :- medoc(B), kaon2equal(A,B).
	// resulting in symbolic fact medoc(v_medoc_1)

		//recursive
		c271_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_medoc_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2558)) return -1;
		c271_2_fb.open(v_medoc_1);
	case 1545:
	// Continuation for label 532
		while (c271_2_fb.fetch()) {
			v_medoc_1 = c271_2_fb.c2();
		if(medoc_set.insert(v_medoc_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1545)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(534)) return -1;
			if(!backtrack_stack.push(533)) return -1;
			if(!backtrack_stack.push(532)) return -1;
			if(!backtrack_stack.push(531)) return -1;
			if(!backtrack_stack.push(525)) return -1;
			goto label_516;
		}
		}
		break;
	case 2558:
	// Label to restore values of label 532
		v_medoc_1 = int_value_stack.pop();
		c271_2_fb.pop(&cursor_stack);
		break;
	case 533: label_533: 
	// using  symbolic fact medoc(v_medoc_1) with literal 1 in rule 
	// 441: q0(A) :- medoc(A).
	// resulting in symbolic fact q0(v_q0_1)

		v_q0_1 = v_medoc_1;
		if(q0_set.insert(v_q0_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(198)) return -1;
		if(!backtrack_stack.push(194)) return -1;
		if(!backtrack_stack.push(189)) return -1;
		if(!backtrack_stack.push(188)) return -1;
		goto label_187;
		}
		break;
	case 534: label_534: 
	// using  symbolic fact medoc(v_medoc_1) with literal 2 in rule 
	// 91: margaux(A) :- q19(A), medoc(A).
	// resulting in symbolic fact margaux(v_margaux_1)

		//recursive
		if(!int_value_stack.push(v_margaux_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2560)) return -1;
		if (!q19_set.contains(v_medoc_1)) continue;
	case 1547:
	// Continuation for label 534
			v_margaux_1 = v_medoc_1;
		if(margaux_set.insert(v_margaux_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1547)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(555)) return -1;
			if(!backtrack_stack.push(535)) return -1;
			if(!backtrack_stack.push(515)) return -1;
			if(!backtrack_stack.push(514)) return -1;
			if(!backtrack_stack.push(501)) return -1;
			goto label_500;
		}
		break;
	case 2560:
	// Label to restore values of label 534
		v_margaux_1 = int_value_stack.pop();
		break;
	case 535: label_535: 
	// using  symbolic fact margaux(v_margaux_1) with literal 1 in rule 
	// 275: q13(A) :- margaux(A).
	// resulting in symbolic fact q13(v_q13_1)

		v_q13_1 = v_margaux_1;
		if(q13_set.insert(v_q13_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(543)) return -1;
		if(!backtrack_stack.push(542)) return -1;
		if(!backtrack_stack.push(541)) return -1;
		if(!backtrack_stack.push(540)) return -1;
		if(!backtrack_stack.push(539)) return -1;
		goto label_536;
		}
		break;
	case 536: label_536: 
	// using  symbolic fact q13(v_q13_1) with literal 1 in rule 
	// 274: merlot(A) :- q13(A).
	// resulting in symbolic fact merlot(v_merlot_1)

		//recursive
		if(!int_value_stack.push(v_merlot_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2562)) return -1;
		v_merlot_1 = v_q13_1;
		if(merlot_set.insert(v_merlot_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(550)) return -1;
		if(!backtrack_stack.push(548)) return -1;
		if(!backtrack_stack.push(547)) return -1;
		if(!backtrack_stack.push(546)) return -1;
		if(!backtrack_stack.push(538)) return -1;
		goto label_537;
		}
		break;
	case 2562:
	// Label to restore values of label 536
		v_merlot_1 = int_value_stack.pop();
		break;
	case 537: label_537: 
	// using  symbolic fact merlot(v_merlot_1) with literal 1 in rule 
	// 272: q31(A) :- merlot(A).
	// resulting in symbolic fact q31(v_q31_1)

		v_q31_1 = v_merlot_1;
		if(q31_set.insert(v_q31_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(221)) return -1;
		if(!backtrack_stack.push(220)) return -1;
		if(!backtrack_stack.push(215)) return -1;
		if(!backtrack_stack.push(213)) return -1;
		goto label_201;
		}
		break;
	case 538: label_538: 
	// using  symbolic fact merlot(v_merlot_1) with literal 1 in rule 
	// 276: q13(A) :- merlot(A).
	// resulting in symbolic fact q13(v_q13_1)

		//recursive
		if(!int_value_stack.push(v_q13_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2564)) return -1;
		v_q13_1 = v_merlot_1;
		if(q13_set.insert(v_q13_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(543)) return -1;
		if(!backtrack_stack.push(542)) return -1;
		if(!backtrack_stack.push(541)) return -1;
		if(!backtrack_stack.push(540)) return -1;
		if(!backtrack_stack.push(539)) return -1;
		goto label_536;
		}
		break;
	case 2564:
	// Label to restore values of label 538
		v_q13_1 = int_value_stack.pop();
		break;
	case 539: label_539: 
	// using  symbolic fact q13(v_q13_1) with literal 1 in rule 
	// 471: wine(A) :- q13(A).
	// resulting in symbolic fact wine(v_wine_1)

		v_wine_1 = v_q13_1;
		if(wine_set.insert(v_wine_1)) {
		if(!wine_f.insert(v_wine_1)) return -1;
		// Labels for next rules
		if(!backtrack_stack.push(1007)) return -1;
		if(!backtrack_stack.push(989)) return -1;
		if(!backtrack_stack.push(954)) return -1;
		if(!backtrack_stack.push(938)) return -1;
		if(!backtrack_stack.push(911)) return -1;
		if(!backtrack_stack.push(903)) return -1;
		if(!backtrack_stack.push(902)) return -1;
		if(!backtrack_stack.push(901)) return -1;
		if(!backtrack_stack.push(849)) return -1;
		if(!backtrack_stack.push(848)) return -1;
		if(!backtrack_stack.push(821)) return -1;
		if(!backtrack_stack.push(820)) return -1;
		if(!backtrack_stack.push(737)) return -1;
		if(!backtrack_stack.push(712)) return -1;
		if(!backtrack_stack.push(674)) return -1;
		if(!backtrack_stack.push(672)) return -1;
		if(!backtrack_stack.push(583)) return -1;
		if(!backtrack_stack.push(575)) return -1;
		if(!backtrack_stack.push(563)) return -1;
		if(!backtrack_stack.push(551)) return -1;
		if(!backtrack_stack.push(549)) return -1;
		if(!backtrack_stack.push(483)) return -1;
		if(!backtrack_stack.push(473)) return -1;
		if(!backtrack_stack.push(470)) return -1;
		if(!backtrack_stack.push(430)) return -1;
		if(!backtrack_stack.push(401)) return -1;
		if(!backtrack_stack.push(399)) return -1;
		if(!backtrack_stack.push(380)) return -1;
		if(!backtrack_stack.push(364)) return -1;
		if(!backtrack_stack.push(363)) return -1;
		if(!backtrack_stack.push(329)) return -1;
		if(!backtrack_stack.push(328)) return -1;
		if(!backtrack_stack.push(327)) return -1;
		if(!backtrack_stack.push(295)) return -1;
		if(!backtrack_stack.push(294)) return -1;
		if(!backtrack_stack.push(216)) return -1;
		if(!backtrack_stack.push(210)) return -1;
		if(!backtrack_stack.push(192)) return -1;
		if(!backtrack_stack.push(180)) return -1;
		if(!backtrack_stack.push(179)) return -1;
		if(!backtrack_stack.push(178)) return -1;
		if(!backtrack_stack.push(154)) return -1;
		if(!backtrack_stack.push(153)) return -1;
		if(!backtrack_stack.push(100)) return -1;
		if(!backtrack_stack.push(76)) return -1;
		if(!backtrack_stack.push(75)) return -1;
		if(!backtrack_stack.push(67)) return -1;
		if(!backtrack_stack.push(63)) return -1;
		if(!backtrack_stack.push(53)) return -1;
		if(!backtrack_stack.push(46)) return -1;
		if(!backtrack_stack.push(19)) return -1;
		goto label_16;
		}
		break;
	case 540: label_540: 
	// using  symbolic fact q13(v_q13_1) with literal 1 in rule 
	// 472: q13(A) :- q13(B), kaon2equal(A,B).
	// resulting in symbolic fact q13(v_q13_1)

		//recursive
		c472_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q13_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2566)) return -1;
		c472_2_fb.open(v_q13_1);
	case 1553:
	// Continuation for label 540
		while (c472_2_fb.fetch()) {
			v_q13_1 = c472_2_fb.c2();
		if(q13_set.insert(v_q13_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1553)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(543)) return -1;
			if(!backtrack_stack.push(542)) return -1;
			if(!backtrack_stack.push(541)) return -1;
			if(!backtrack_stack.push(540)) return -1;
			if(!backtrack_stack.push(539)) return -1;
			goto label_536;
		}
		}
		break;
	case 2566:
	// Label to restore values of label 540
		v_q13_1 = int_value_stack.pop();
		c472_2_fb.pop(&cursor_stack);
		break;
	case 541: label_541: 
	// using  symbolic fact q13(v_q13_1) with literal 1 in rule 
	// 499: q5(A) :- q13(A).
	// resulting in symbolic fact q5(v_q5_1)

		v_q5_1 = v_q13_1;
		if(q5_set.insert(v_q5_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(136)) return -1;
		if(!backtrack_stack.push(135)) return -1;
		goto label_131;
		}
		break;
	case 542: label_542: 
	// using  symbolic fact q13(v_q13_1) with literal 1 in rule 
	// 523: madefromgrape(A,A) :- q13(A), kaon2namedobjects(A).
	// resulting in symbolic fact madefromgrape(v_madefromgrape_1,v_madefromgrape_2)

		//recursive
		if(!int_value_stack.push(v_madefromgrape_2)) return -1;
		if(!int_value_stack.push(v_madefromgrape_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2568)) return -1;
		if (!kaon2namedobjects_b.contains(v_q13_1)) continue;
	case 1555:
	// Continuation for label 542
			v_madefromgrape_1 = v_q13_1;
			v_madefromgrape_2 = v_q13_1;
		if(madefromgrape_set.insert(v_madefromgrape_1, v_madefromgrape_2)) {
			if(!madefromgrape_bf.insert(v_madefromgrape_1, v_madefromgrape_2)) return -1;
			if(!madefromgrape_fb.insert(v_madefromgrape_2, v_madefromgrape_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1555)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(860)) return -1;
			if(!backtrack_stack.push(859)) return -1;
			if(!backtrack_stack.push(845)) return -1;
			if(!backtrack_stack.push(844)) return -1;
			if(!backtrack_stack.push(829)) return -1;
			if(!backtrack_stack.push(828)) return -1;
			if(!backtrack_stack.push(813)) return -1;
			if(!backtrack_stack.push(812)) return -1;
			if(!backtrack_stack.push(677)) return -1;
			if(!backtrack_stack.push(676)) return -1;
			if(!backtrack_stack.push(617)) return -1;
			if(!backtrack_stack.push(616)) return -1;
			if(!backtrack_stack.push(609)) return -1;
			if(!backtrack_stack.push(554)) return -1;
			if(!backtrack_stack.push(553)) return -1;
			if(!backtrack_stack.push(476)) return -1;
			if(!backtrack_stack.push(475)) return -1;
			if(!backtrack_stack.push(404)) return -1;
			if(!backtrack_stack.push(403)) return -1;
			if(!backtrack_stack.push(362)) return -1;
			if(!backtrack_stack.push(361)) return -1;
			if(!backtrack_stack.push(343)) return -1;
			if(!backtrack_stack.push(342)) return -1;
			if(!backtrack_stack.push(300)) return -1;
			if(!backtrack_stack.push(170)) return -1;
			if(!backtrack_stack.push(169)) return -1;
			if(!backtrack_stack.push(151)) return -1;
			if(!backtrack_stack.push(150)) return -1;
			if(!backtrack_stack.push(144)) return -1;
			if(!backtrack_stack.push(132)) return -1;
			if(!backtrack_stack.push(71)) return -1;
			if(!backtrack_stack.push(70)) return -1;
			if(!backtrack_stack.push(66)) return -1;
			goto label_64;
		}
		break;
	case 2568:
	// Label to restore values of label 542
		v_madefromgrape_1 = int_value_stack.pop();
		v_madefromgrape_2 = int_value_stack.pop();
		break;
	case 543: label_543: 
	// using  symbolic fact q13(v_q13_1) with literal 1 in rule 
	// 536: ot____nom43(A) :- q13(A), kaon2namedobjects(A).
	// resulting in symbolic fact ot____nom43(v_ot____nom43_1)

		//recursive
		if(!int_value_stack.push(v_ot____nom43_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2569)) return -1;
		if (!kaon2namedobjects_b.contains(v_q13_1)) continue;
	case 1556:
	// Continuation for label 543
			v_ot____nom43_1 = v_q13_1;
		if(ot____nom43_set.insert(v_ot____nom43_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1556)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(552)) return -1;
			if(!backtrack_stack.push(545)) return -1;
			goto label_544;
		}
		break;
	case 2569:
	// Label to restore values of label 543
		v_ot____nom43_1 = int_value_stack.pop();
		break;
	case 544: label_544: 
	// using  symbolic fact ot____nom43(v_ot____nom43_1) with literal 1 in rule 
	// 537: ot____nom43(A) :- ot____nom43(B), kaon2equal(A,B).
	// resulting in symbolic fact ot____nom43(v_ot____nom43_1)

		//recursive
		c537_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_ot____nom43_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2570)) return -1;
		c537_2_fb.open(v_ot____nom43_1);
	case 1557:
	// Continuation for label 544
		while (c537_2_fb.fetch()) {
			v_ot____nom43_1 = c537_2_fb.c2();
		if(ot____nom43_set.insert(v_ot____nom43_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1557)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(552)) return -1;
			if(!backtrack_stack.push(545)) return -1;
			goto label_544;
		}
		}
		break;
	case 2570:
	// Label to restore values of label 544
		v_ot____nom43_1 = int_value_stack.pop();
		c537_2_fb.pop(&cursor_stack);
		break;
	case 545: label_545: 
	// using  symbolic fact ot____nom43(v_ot____nom43_1) with literal 3 in rule 
	// 278: merlot(A) :- wine(A), madefromgrape(A,B), ot____nom43(B).
	// resulting in symbolic fact merlot(v_merlot_1)

		//recursive
		c278_1_f.push(&cursor_stack);
		if(!int_value_stack.push(v_merlot_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2571)) return -1;
		c278_1_f.open();
		while (c278_1_f.fetch()) {
			if (!madefromgrape_set.contains(c278_1_f.c1(), v_ot____nom43_1)) continue;
	case 1558:
	// Continuation for label 545
				v_merlot_1 = c278_1_f.c1();
		if(merlot_set.insert(v_merlot_1)) {
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1558)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(550)) return -1;
				if(!backtrack_stack.push(548)) return -1;
				if(!backtrack_stack.push(547)) return -1;
				if(!backtrack_stack.push(546)) return -1;
				if(!backtrack_stack.push(538)) return -1;
				goto label_537;
		}
		}
		break;
	case 2571:
	// Label to restore values of label 545
		v_merlot_1 = int_value_stack.pop();
		c278_1_f.pop(&cursor_stack);
		break;
	case 546: label_546: 
	// using  symbolic fact merlot(v_merlot_1) with literal 1 in rule 
	// 277: merlot(A) :- merlot(B), kaon2equal(A,B).
	// resulting in symbolic fact merlot(v_merlot_1)

		//recursive
		c277_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_merlot_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2572)) return -1;
		c277_2_fb.open(v_merlot_1);
	case 1559:
	// Continuation for label 546
		while (c277_2_fb.fetch()) {
			v_merlot_1 = c277_2_fb.c2();
		if(merlot_set.insert(v_merlot_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1559)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(550)) return -1;
			if(!backtrack_stack.push(548)) return -1;
			if(!backtrack_stack.push(547)) return -1;
			if(!backtrack_stack.push(546)) return -1;
			if(!backtrack_stack.push(538)) return -1;
			goto label_537;
		}
		}
		break;
	case 2572:
	// Label to restore values of label 546
		v_merlot_1 = int_value_stack.pop();
		c277_2_fb.pop(&cursor_stack);
		break;
	case 547: label_547: 
	// using  symbolic fact merlot(v_merlot_1) with literal 1 in rule 
	// 320: q0(A) :- merlot(A).
	// resulting in symbolic fact q0(v_q0_1)

		v_q0_1 = v_merlot_1;
		if(q0_set.insert(v_q0_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(198)) return -1;
		if(!backtrack_stack.push(194)) return -1;
		if(!backtrack_stack.push(189)) return -1;
		if(!backtrack_stack.push(188)) return -1;
		goto label_187;
		}
		break;
	case 548: label_548: 
	// using  symbolic fact merlot(v_merlot_1) with literal 1 in rule 
	// 515: wine(A) :- merlot(A).
	// resulting in symbolic fact wine(v_wine_1)

		//recursive
		if(!int_value_stack.push(v_wine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2574)) return -1;
		v_wine_1 = v_merlot_1;
		if(wine_set.insert(v_wine_1)) {
		if(!wine_f.insert(v_wine_1)) return -1;
		// Labels for next rules
		if(!backtrack_stack.push(1007)) return -1;
		if(!backtrack_stack.push(989)) return -1;
		if(!backtrack_stack.push(954)) return -1;
		if(!backtrack_stack.push(938)) return -1;
		if(!backtrack_stack.push(911)) return -1;
		if(!backtrack_stack.push(903)) return -1;
		if(!backtrack_stack.push(902)) return -1;
		if(!backtrack_stack.push(901)) return -1;
		if(!backtrack_stack.push(849)) return -1;
		if(!backtrack_stack.push(848)) return -1;
		if(!backtrack_stack.push(821)) return -1;
		if(!backtrack_stack.push(820)) return -1;
		if(!backtrack_stack.push(737)) return -1;
		if(!backtrack_stack.push(712)) return -1;
		if(!backtrack_stack.push(674)) return -1;
		if(!backtrack_stack.push(672)) return -1;
		if(!backtrack_stack.push(583)) return -1;
		if(!backtrack_stack.push(575)) return -1;
		if(!backtrack_stack.push(563)) return -1;
		if(!backtrack_stack.push(551)) return -1;
		if(!backtrack_stack.push(549)) return -1;
		if(!backtrack_stack.push(483)) return -1;
		if(!backtrack_stack.push(473)) return -1;
		if(!backtrack_stack.push(470)) return -1;
		if(!backtrack_stack.push(430)) return -1;
		if(!backtrack_stack.push(401)) return -1;
		if(!backtrack_stack.push(399)) return -1;
		if(!backtrack_stack.push(380)) return -1;
		if(!backtrack_stack.push(364)) return -1;
		if(!backtrack_stack.push(363)) return -1;
		if(!backtrack_stack.push(329)) return -1;
		if(!backtrack_stack.push(328)) return -1;
		if(!backtrack_stack.push(327)) return -1;
		if(!backtrack_stack.push(295)) return -1;
		if(!backtrack_stack.push(294)) return -1;
		if(!backtrack_stack.push(216)) return -1;
		if(!backtrack_stack.push(210)) return -1;
		if(!backtrack_stack.push(192)) return -1;
		if(!backtrack_stack.push(180)) return -1;
		if(!backtrack_stack.push(179)) return -1;
		if(!backtrack_stack.push(178)) return -1;
		if(!backtrack_stack.push(154)) return -1;
		if(!backtrack_stack.push(153)) return -1;
		if(!backtrack_stack.push(100)) return -1;
		if(!backtrack_stack.push(76)) return -1;
		if(!backtrack_stack.push(75)) return -1;
		if(!backtrack_stack.push(67)) return -1;
		if(!backtrack_stack.push(63)) return -1;
		if(!backtrack_stack.push(53)) return -1;
		if(!backtrack_stack.push(46)) return -1;
		if(!backtrack_stack.push(19)) return -1;
		goto label_16;
		}
		break;
	case 2574:
	// Label to restore values of label 548
		v_wine_1 = int_value_stack.pop();
		break;
	case 549: label_549: 
	// using  symbolic fact wine(v_wine_1) with literal 1 in rule 
	// 278: merlot(A) :- wine(A), madefromgrape(A,B), ot____nom43(B).
	// resulting in symbolic fact merlot(v_merlot_1)

		//recursive
		c278_2_bf.push(&cursor_stack);
		if(!int_value_stack.push(v_merlot_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2575)) return -1;
		c278_2_bf.open(v_wine_1);
		while (c278_2_bf.fetch()) {
			if (!ot____nom43_set.contains(c278_2_bf.c2())) continue;
	case 1562:
	// Continuation for label 549
				v_merlot_1 = v_wine_1;
		if(merlot_set.insert(v_merlot_1)) {
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1562)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(550)) return -1;
				if(!backtrack_stack.push(548)) return -1;
				if(!backtrack_stack.push(547)) return -1;
				if(!backtrack_stack.push(546)) return -1;
				if(!backtrack_stack.push(538)) return -1;
				goto label_537;
		}
		}
		break;
	case 2575:
	// Label to restore values of label 549
		v_merlot_1 = int_value_stack.pop();
		c278_2_bf.pop(&cursor_stack);
		break;
	case 550: label_550: 
	// using  symbolic fact merlot(v_merlot_1) with literal 1 in rule 
	// 784: q3(A) :- merlot(A).
	// resulting in symbolic fact q3(v_q3_1)

		v_q3_1 = v_merlot_1;
		if(q3_set.insert(v_q3_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(143)) return -1;
		goto label_123;
		}
		break;
	case 551: label_551: 
	// using  symbolic fact wine(v_wine_1) with literal 1 in rule 
	// 534: q5(A) :- wine(A), madefromgrape(A,B), ot____nom43(B).
	// resulting in symbolic fact q5(v_q5_1)

		c534_2_bf.open(v_wine_1);
		while (c534_2_bf.fetch()) {
			if (!ot____nom43_set.contains(c534_2_bf.c2())) continue;
	case 1564:
	// Continuation for label 551
				v_q5_1 = v_wine_1;
		if(q5_set.insert(v_q5_1)) {
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1564)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(136)) return -1;
				if(!backtrack_stack.push(135)) return -1;
				goto label_131;
		}
		}
		break;
	case 552: label_552: 
	// using  symbolic fact ot____nom43(v_ot____nom43_1) with literal 3 in rule 
	// 534: q5(A) :- wine(A), madefromgrape(A,B), ot____nom43(B).
	// resulting in symbolic fact q5(v_q5_1)

		c534_1_f.open();
		while (c534_1_f.fetch()) {
			if (!madefromgrape_set.contains(c534_1_f.c1(), v_ot____nom43_1)) continue;
	case 1565:
	// Continuation for label 552
				v_q5_1 = c534_1_f.c1();
		if(q5_set.insert(v_q5_1)) {
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1565)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(136)) return -1;
				if(!backtrack_stack.push(135)) return -1;
				goto label_131;
		}
		}
		break;
	case 553: label_553: 
	// using  symbolic fact madefromgrape(v_madefromgrape_1,v_madefromgrape_2) with literal 2 in rule 
	// 278: merlot(A) :- wine(A), madefromgrape(A,B), ot____nom43(B).
	// resulting in symbolic fact merlot(v_merlot_1)

		//recursive
		if(!int_value_stack.push(v_merlot_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2579)) return -1;
		if (!wine_set.contains(v_madefromgrape_1)) continue;
			if (!ot____nom43_set.contains(v_madefromgrape_2)) continue;
	case 1566:
	// Continuation for label 553
				v_merlot_1 = v_madefromgrape_1;
		if(merlot_set.insert(v_merlot_1)) {
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1566)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(550)) return -1;
				if(!backtrack_stack.push(548)) return -1;
				if(!backtrack_stack.push(547)) return -1;
				if(!backtrack_stack.push(546)) return -1;
				if(!backtrack_stack.push(538)) return -1;
				goto label_537;
		}
		break;
	case 2579:
	// Label to restore values of label 553
		v_merlot_1 = int_value_stack.pop();
		break;
	case 554: label_554: 
	// using  symbolic fact madefromgrape(v_madefromgrape_1,v_madefromgrape_2) with literal 2 in rule 
	// 534: q5(A) :- wine(A), madefromgrape(A,B), ot____nom43(B).
	// resulting in symbolic fact q5(v_q5_1)

		if (!wine_set.contains(v_madefromgrape_1)) continue;
			if (!ot____nom43_set.contains(v_madefromgrape_2)) continue;
	case 1567:
	// Continuation for label 554
				v_q5_1 = v_madefromgrape_1;
		if(q5_set.insert(v_q5_1)) {
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1567)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(136)) return -1;
				if(!backtrack_stack.push(135)) return -1;
				goto label_131;
		}
		break;
	case 555: label_555: 
	// using  symbolic fact margaux(v_margaux_1) with literal 1 in rule 
	// 645: q20(A) :- margaux(A).
	// resulting in symbolic fact q20(v_q20_1)

		v_q20_1 = v_margaux_1;
		if(q20_set.insert(v_q20_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(184)) return -1;
		if(!backtrack_stack.push(183)) return -1;
		goto label_182;
		}
		break;
	case 556: label_556: 
	// using  symbolic fact q1(v_q1_1) with literal 1 in rule 
	// 307: q2(A) :- q1(A).
	// resulting in symbolic fact q2(v_q2_1)

		//recursive
		if(!int_value_stack.push(v_q2_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2582)) return -1;
		v_q2_1 = v_q1_1;
		if(q2_set.insert(v_q2_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(585)) return -1;
		if(!backtrack_stack.push(584)) return -1;
		goto label_557;
		}
		break;
	case 2582:
	// Label to restore values of label 556
		v_q2_1 = int_value_stack.pop();
		break;
	case 557: label_557: 
	// using  symbolic fact q2(v_q2_1) with literal 1 in rule 
	// 306: bordeaux(A) :- q2(A), wine(A).
	// resulting in symbolic fact bordeaux(v_bordeaux_1)

		//recursive
		if(!int_value_stack.push(v_bordeaux_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2583)) return -1;
		if (!wine_set.contains(v_q2_1)) continue;
	case 1570:
	// Continuation for label 557
			v_bordeaux_1 = v_q2_1;
		if(bordeaux_set.insert(v_bordeaux_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1570)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(781)) return -1;
			if(!backtrack_stack.push(566)) return -1;
			if(!backtrack_stack.push(565)) return -1;
			if(!backtrack_stack.push(564)) return -1;
			if(!backtrack_stack.push(562)) return -1;
			if(!backtrack_stack.push(558)) return -1;
			goto label_526;
		}
		break;
	case 2583:
	// Label to restore values of label 557
		v_bordeaux_1 = int_value_stack.pop();
		break;
	case 558: label_558: 
	// using  symbolic fact bordeaux(v_bordeaux_1) with literal 1 in rule 
	// 302: whitebordeaux(A) :- bordeaux(A), whitewine(A).
	// resulting in symbolic fact whitebordeaux(v_whitebordeaux_1)

		//recursive
		if(!int_value_stack.push(v_whitebordeaux_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2584)) return -1;
		if (!whitewine_set.contains(v_bordeaux_1)) continue;
	case 1571:
	// Continuation for label 558
			v_whitebordeaux_1 = v_bordeaux_1;
		if(whitebordeaux_set.insert(v_whitebordeaux_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1571)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(572)) return -1;
			if(!backtrack_stack.push(571)) return -1;
			if(!backtrack_stack.push(561)) return -1;
			goto label_559;
		}
		break;
	case 2584:
	// Label to restore values of label 558
		v_whitebordeaux_1 = int_value_stack.pop();
		break;
	case 559: label_559: 
	// using  symbolic fact whitebordeaux(v_whitebordeaux_1) with literal 1 in rule 
	// 301: whitewine(A) :- whitebordeaux(A).
	// resulting in symbolic fact whitewine(v_whitewine_1)

		//recursive
		if(!int_value_stack.push(v_whitewine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2585)) return -1;
		v_whitewine_1 = v_whitebordeaux_1;
		if(whitewine_set.insert(v_whitewine_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(560)) return -1;
		if(!backtrack_stack.push(491)) return -1;
		if(!backtrack_stack.push(374)) return -1;
		if(!backtrack_stack.push(235)) return -1;
		if(!backtrack_stack.push(113)) return -1;
		if(!backtrack_stack.push(112)) return -1;
		if(!backtrack_stack.push(104)) return -1;
		if(!backtrack_stack.push(103)) return -1;
		if(!backtrack_stack.push(101)) return -1;
		if(!backtrack_stack.push(98)) return -1;
		goto label_97;
		}
		break;
	case 2585:
	// Label to restore values of label 559
		v_whitewine_1 = int_value_stack.pop();
		break;
	case 560: label_560: 
	// using  symbolic fact whitewine(v_whitewine_1) with literal 2 in rule 
	// 302: whitebordeaux(A) :- bordeaux(A), whitewine(A).
	// resulting in symbolic fact whitebordeaux(v_whitebordeaux_1)

		//recursive
		if(!int_value_stack.push(v_whitebordeaux_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2586)) return -1;
		if (!bordeaux_set.contains(v_whitewine_1)) continue;
	case 1573:
	// Continuation for label 560
			v_whitebordeaux_1 = v_whitewine_1;
		if(whitebordeaux_set.insert(v_whitebordeaux_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1573)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(572)) return -1;
			if(!backtrack_stack.push(571)) return -1;
			if(!backtrack_stack.push(561)) return -1;
			goto label_559;
		}
		break;
	case 2586:
	// Label to restore values of label 560
		v_whitebordeaux_1 = int_value_stack.pop();
		break;
	case 561: label_561: 
	// using  symbolic fact whitebordeaux(v_whitebordeaux_1) with literal 1 in rule 
	// 304: bordeaux(A) :- whitebordeaux(A).
	// resulting in symbolic fact bordeaux(v_bordeaux_1)

		//recursive
		if(!int_value_stack.push(v_bordeaux_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2587)) return -1;
		v_bordeaux_1 = v_whitebordeaux_1;
		if(bordeaux_set.insert(v_bordeaux_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(781)) return -1;
		if(!backtrack_stack.push(566)) return -1;
		if(!backtrack_stack.push(565)) return -1;
		if(!backtrack_stack.push(564)) return -1;
		if(!backtrack_stack.push(562)) return -1;
		if(!backtrack_stack.push(558)) return -1;
		goto label_526;
		}
		break;
	case 2587:
	// Label to restore values of label 561
		v_bordeaux_1 = int_value_stack.pop();
		break;
	case 562: label_562: 
	// using  symbolic fact bordeaux(v_bordeaux_1) with literal 1 in rule 
	// 426: wine(A) :- bordeaux(A).
	// resulting in symbolic fact wine(v_wine_1)

		//recursive
		if(!int_value_stack.push(v_wine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2588)) return -1;
		v_wine_1 = v_bordeaux_1;
		if(wine_set.insert(v_wine_1)) {
		if(!wine_f.insert(v_wine_1)) return -1;
		// Labels for next rules
		if(!backtrack_stack.push(1007)) return -1;
		if(!backtrack_stack.push(989)) return -1;
		if(!backtrack_stack.push(954)) return -1;
		if(!backtrack_stack.push(938)) return -1;
		if(!backtrack_stack.push(911)) return -1;
		if(!backtrack_stack.push(903)) return -1;
		if(!backtrack_stack.push(902)) return -1;
		if(!backtrack_stack.push(901)) return -1;
		if(!backtrack_stack.push(849)) return -1;
		if(!backtrack_stack.push(848)) return -1;
		if(!backtrack_stack.push(821)) return -1;
		if(!backtrack_stack.push(820)) return -1;
		if(!backtrack_stack.push(737)) return -1;
		if(!backtrack_stack.push(712)) return -1;
		if(!backtrack_stack.push(674)) return -1;
		if(!backtrack_stack.push(672)) return -1;
		if(!backtrack_stack.push(583)) return -1;
		if(!backtrack_stack.push(575)) return -1;
		if(!backtrack_stack.push(563)) return -1;
		if(!backtrack_stack.push(551)) return -1;
		if(!backtrack_stack.push(549)) return -1;
		if(!backtrack_stack.push(483)) return -1;
		if(!backtrack_stack.push(473)) return -1;
		if(!backtrack_stack.push(470)) return -1;
		if(!backtrack_stack.push(430)) return -1;
		if(!backtrack_stack.push(401)) return -1;
		if(!backtrack_stack.push(399)) return -1;
		if(!backtrack_stack.push(380)) return -1;
		if(!backtrack_stack.push(364)) return -1;
		if(!backtrack_stack.push(363)) return -1;
		if(!backtrack_stack.push(329)) return -1;
		if(!backtrack_stack.push(328)) return -1;
		if(!backtrack_stack.push(327)) return -1;
		if(!backtrack_stack.push(295)) return -1;
		if(!backtrack_stack.push(294)) return -1;
		if(!backtrack_stack.push(216)) return -1;
		if(!backtrack_stack.push(210)) return -1;
		if(!backtrack_stack.push(192)) return -1;
		if(!backtrack_stack.push(180)) return -1;
		if(!backtrack_stack.push(179)) return -1;
		if(!backtrack_stack.push(178)) return -1;
		if(!backtrack_stack.push(154)) return -1;
		if(!backtrack_stack.push(153)) return -1;
		if(!backtrack_stack.push(100)) return -1;
		if(!backtrack_stack.push(76)) return -1;
		if(!backtrack_stack.push(75)) return -1;
		if(!backtrack_stack.push(67)) return -1;
		if(!backtrack_stack.push(63)) return -1;
		if(!backtrack_stack.push(53)) return -1;
		if(!backtrack_stack.push(46)) return -1;
		if(!backtrack_stack.push(19)) return -1;
		goto label_16;
		}
		break;
	case 2588:
	// Label to restore values of label 562
		v_wine_1 = int_value_stack.pop();
		break;
	case 563: label_563: 
	// using  symbolic fact wine(v_wine_1) with literal 2 in rule 
	// 306: bordeaux(A) :- q2(A), wine(A).
	// resulting in symbolic fact bordeaux(v_bordeaux_1)

		//recursive
		if(!int_value_stack.push(v_bordeaux_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2589)) return -1;
		if (!q2_set.contains(v_wine_1)) continue;
	case 1576:
	// Continuation for label 563
			v_bordeaux_1 = v_wine_1;
		if(bordeaux_set.insert(v_bordeaux_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1576)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(781)) return -1;
			if(!backtrack_stack.push(566)) return -1;
			if(!backtrack_stack.push(565)) return -1;
			if(!backtrack_stack.push(564)) return -1;
			if(!backtrack_stack.push(562)) return -1;
			if(!backtrack_stack.push(558)) return -1;
			goto label_526;
		}
		break;
	case 2589:
	// Label to restore values of label 563
		v_bordeaux_1 = int_value_stack.pop();
		break;
	case 564: label_564: 
	// using  symbolic fact bordeaux(v_bordeaux_1) with literal 1 in rule 
	// 427: bordeaux(A) :- bordeaux(B), kaon2equal(A,B).
	// resulting in symbolic fact bordeaux(v_bordeaux_1)

		//recursive
		c427_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_bordeaux_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2590)) return -1;
		c427_2_fb.open(v_bordeaux_1);
	case 1577:
	// Continuation for label 564
		while (c427_2_fb.fetch()) {
			v_bordeaux_1 = c427_2_fb.c2();
		if(bordeaux_set.insert(v_bordeaux_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1577)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(781)) return -1;
			if(!backtrack_stack.push(566)) return -1;
			if(!backtrack_stack.push(565)) return -1;
			if(!backtrack_stack.push(564)) return -1;
			if(!backtrack_stack.push(562)) return -1;
			if(!backtrack_stack.push(558)) return -1;
			goto label_526;
		}
		}
		break;
	case 2590:
	// Label to restore values of label 564
		v_bordeaux_1 = int_value_stack.pop();
		c427_2_fb.pop(&cursor_stack);
		break;
	case 565: label_565: 
	// using  symbolic fact bordeaux(v_bordeaux_1) with literal 2 in rule 
	// 202: medoc(A) :- q34(A), bordeaux(A).
	// resulting in symbolic fact medoc(v_medoc_1)

		//recursive
		if(!int_value_stack.push(v_medoc_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2591)) return -1;
		if (!q34_set.contains(v_bordeaux_1)) continue;
	case 1578:
	// Continuation for label 565
			v_medoc_1 = v_bordeaux_1;
		if(medoc_set.insert(v_medoc_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1578)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(534)) return -1;
			if(!backtrack_stack.push(533)) return -1;
			if(!backtrack_stack.push(532)) return -1;
			if(!backtrack_stack.push(531)) return -1;
			if(!backtrack_stack.push(525)) return -1;
			goto label_516;
		}
		break;
	case 2591:
	// Label to restore values of label 565
		v_medoc_1 = int_value_stack.pop();
		break;
	case 566: label_566: 
	// using  symbolic fact bordeaux(v_bordeaux_1) with literal 2 in rule 
	// 229: redbordeaux(A) :- redwine(A), bordeaux(A).
	// resulting in symbolic fact redbordeaux(v_redbordeaux_1)

		//recursive
		if(!int_value_stack.push(v_redbordeaux_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2592)) return -1;
		if (!redwine_set.contains(v_bordeaux_1)) continue;
	case 1579:
	// Continuation for label 566
			v_redbordeaux_1 = v_bordeaux_1;
		if(redbordeaux_set.insert(v_redbordeaux_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1579)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(570)) return -1;
			if(!backtrack_stack.push(568)) return -1;
			goto label_567;
		}
		break;
	case 2592:
	// Label to restore values of label 566
		v_redbordeaux_1 = int_value_stack.pop();
		break;
	case 567: label_567: 
	// using  symbolic fact redbordeaux(v_redbordeaux_1) with literal 1 in rule 
	// 228: bordeaux(A) :- redbordeaux(A).
	// resulting in symbolic fact bordeaux(v_bordeaux_1)

		//recursive
		if(!int_value_stack.push(v_bordeaux_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2593)) return -1;
		v_bordeaux_1 = v_redbordeaux_1;
		if(bordeaux_set.insert(v_bordeaux_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(781)) return -1;
		if(!backtrack_stack.push(566)) return -1;
		if(!backtrack_stack.push(565)) return -1;
		if(!backtrack_stack.push(564)) return -1;
		if(!backtrack_stack.push(562)) return -1;
		if(!backtrack_stack.push(558)) return -1;
		goto label_526;
		}
		break;
	case 2593:
	// Label to restore values of label 567
		v_bordeaux_1 = int_value_stack.pop();
		break;
	case 568: label_568: 
	// using  symbolic fact redbordeaux(v_redbordeaux_1) with literal 1 in rule 
	// 232: redwine(A) :- redbordeaux(A).
	// resulting in symbolic fact redwine(v_redwine_1)

		//recursive
		if(!int_value_stack.push(v_redwine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2594)) return -1;
		v_redwine_1 = v_redbordeaux_1;
		if(redwine_set.insert(v_redwine_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(569)) return -1;
		if(!backtrack_stack.push(384)) return -1;
		if(!backtrack_stack.push(205)) return -1;
		if(!backtrack_stack.push(199)) return -1;
		if(!backtrack_stack.push(193)) return -1;
		goto label_191;
		}
		break;
	case 2594:
	// Label to restore values of label 568
		v_redwine_1 = int_value_stack.pop();
		break;
	case 569: label_569: 
	// using  symbolic fact redwine(v_redwine_1) with literal 1 in rule 
	// 229: redbordeaux(A) :- redwine(A), bordeaux(A).
	// resulting in symbolic fact redbordeaux(v_redbordeaux_1)

		//recursive
		if(!int_value_stack.push(v_redbordeaux_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2595)) return -1;
		if (!bordeaux_set.contains(v_redwine_1)) continue;
	case 1582:
	// Continuation for label 569
			v_redbordeaux_1 = v_redwine_1;
		if(redbordeaux_set.insert(v_redbordeaux_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1582)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(570)) return -1;
			if(!backtrack_stack.push(568)) return -1;
			goto label_567;
		}
		break;
	case 2595:
	// Label to restore values of label 569
		v_redbordeaux_1 = int_value_stack.pop();
		break;
	case 570: label_570: 
	// using  symbolic fact redbordeaux(v_redbordeaux_1) with literal 1 in rule 
	// 233: redbordeaux(A) :- redbordeaux(B), kaon2equal(A,B).
	// resulting in symbolic fact redbordeaux(v_redbordeaux_1)

		//recursive
		c233_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_redbordeaux_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2596)) return -1;
		c233_2_fb.open(v_redbordeaux_1);
	case 1583:
	// Continuation for label 570
		while (c233_2_fb.fetch()) {
			v_redbordeaux_1 = c233_2_fb.c2();
		if(redbordeaux_set.insert(v_redbordeaux_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1583)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(570)) return -1;
			if(!backtrack_stack.push(568)) return -1;
			goto label_567;
		}
		}
		break;
	case 2596:
	// Label to restore values of label 570
		v_redbordeaux_1 = int_value_stack.pop();
		c233_2_fb.pop(&cursor_stack);
		break;
	case 571: label_571: 
	// using  symbolic fact whitebordeaux(v_whitebordeaux_1) with literal 1 in rule 
	// 305: whitebordeaux(A) :- whitebordeaux(B), kaon2equal(A,B).
	// resulting in symbolic fact whitebordeaux(v_whitebordeaux_1)

		//recursive
		c305_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_whitebordeaux_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2597)) return -1;
		c305_2_fb.open(v_whitebordeaux_1);
	case 1584:
	// Continuation for label 571
		while (c305_2_fb.fetch()) {
			v_whitebordeaux_1 = c305_2_fb.c2();
		if(whitebordeaux_set.insert(v_whitebordeaux_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1584)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(572)) return -1;
			if(!backtrack_stack.push(571)) return -1;
			if(!backtrack_stack.push(561)) return -1;
			goto label_559;
		}
		}
		break;
	case 2597:
	// Label to restore values of label 571
		v_whitebordeaux_1 = int_value_stack.pop();
		c305_2_fb.pop(&cursor_stack);
		break;
	case 572: label_572: 
	// using  symbolic fact whitebordeaux(v_whitebordeaux_1) with literal 1 in rule 
	// 346: q68(A) :- whitebordeaux(A).
	// resulting in symbolic fact q68(v_q68_1)

		v_q68_1 = v_whitebordeaux_1;
		if(q68_set.insert(v_q68_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(582)) return -1;
		if(!backtrack_stack.push(577)) return -1;
		goto label_573;
		}
		break;
	case 573: label_573: 
	// using  symbolic fact q68(v_q68_1) with literal 1 in rule 
	// 345: semillonorsauvignonblanc(A) :- q68(A), wine(A).
	// resulting in symbolic fact semillonorsauvignonblanc(v_semillonorsauvignonblanc_1)

		//recursive
		if(!int_value_stack.push(v_semillonorsauvignonblanc_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2599)) return -1;
		if (!wine_set.contains(v_q68_1)) continue;
	case 1586:
	// Continuation for label 573
			v_semillonorsauvignonblanc_1 = v_q68_1;
		if(semillonorsauvignonblanc_set.insert(v_semillonorsauvignonblanc_1)) {
			if(!semillonorsauvignonblanc_f.insert(v_semillonorsauvignonblanc_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1586)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(817)) return -1;
			if(!backtrack_stack.push(816)) return -1;
			if(!backtrack_stack.push(618)) return -1;
			if(!backtrack_stack.push(603)) return -1;
			if(!backtrack_stack.push(580)) return -1;
			if(!backtrack_stack.push(579)) return -1;
			if(!backtrack_stack.push(576)) return -1;
			goto label_574;
		}
		break;
	case 2599:
	// Label to restore values of label 573
		v_semillonorsauvignonblanc_1 = int_value_stack.pop();
		break;
	case 574: label_574: 
	// using  symbolic fact semillonorsauvignonblanc(v_semillonorsauvignonblanc_1) with literal 1 in rule 
	// 333: wine(A) :- semillonorsauvignonblanc(A).
	// resulting in symbolic fact wine(v_wine_1)

		//recursive
		if(!int_value_stack.push(v_wine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2600)) return -1;
		v_wine_1 = v_semillonorsauvignonblanc_1;
		if(wine_set.insert(v_wine_1)) {
		if(!wine_f.insert(v_wine_1)) return -1;
		// Labels for next rules
		if(!backtrack_stack.push(1007)) return -1;
		if(!backtrack_stack.push(989)) return -1;
		if(!backtrack_stack.push(954)) return -1;
		if(!backtrack_stack.push(938)) return -1;
		if(!backtrack_stack.push(911)) return -1;
		if(!backtrack_stack.push(903)) return -1;
		if(!backtrack_stack.push(902)) return -1;
		if(!backtrack_stack.push(901)) return -1;
		if(!backtrack_stack.push(849)) return -1;
		if(!backtrack_stack.push(848)) return -1;
		if(!backtrack_stack.push(821)) return -1;
		if(!backtrack_stack.push(820)) return -1;
		if(!backtrack_stack.push(737)) return -1;
		if(!backtrack_stack.push(712)) return -1;
		if(!backtrack_stack.push(674)) return -1;
		if(!backtrack_stack.push(672)) return -1;
		if(!backtrack_stack.push(583)) return -1;
		if(!backtrack_stack.push(575)) return -1;
		if(!backtrack_stack.push(563)) return -1;
		if(!backtrack_stack.push(551)) return -1;
		if(!backtrack_stack.push(549)) return -1;
		if(!backtrack_stack.push(483)) return -1;
		if(!backtrack_stack.push(473)) return -1;
		if(!backtrack_stack.push(470)) return -1;
		if(!backtrack_stack.push(430)) return -1;
		if(!backtrack_stack.push(401)) return -1;
		if(!backtrack_stack.push(399)) return -1;
		if(!backtrack_stack.push(380)) return -1;
		if(!backtrack_stack.push(364)) return -1;
		if(!backtrack_stack.push(363)) return -1;
		if(!backtrack_stack.push(329)) return -1;
		if(!backtrack_stack.push(328)) return -1;
		if(!backtrack_stack.push(327)) return -1;
		if(!backtrack_stack.push(295)) return -1;
		if(!backtrack_stack.push(294)) return -1;
		if(!backtrack_stack.push(216)) return -1;
		if(!backtrack_stack.push(210)) return -1;
		if(!backtrack_stack.push(192)) return -1;
		if(!backtrack_stack.push(180)) return -1;
		if(!backtrack_stack.push(179)) return -1;
		if(!backtrack_stack.push(178)) return -1;
		if(!backtrack_stack.push(154)) return -1;
		if(!backtrack_stack.push(153)) return -1;
		if(!backtrack_stack.push(100)) return -1;
		if(!backtrack_stack.push(76)) return -1;
		if(!backtrack_stack.push(75)) return -1;
		if(!backtrack_stack.push(67)) return -1;
		if(!backtrack_stack.push(63)) return -1;
		if(!backtrack_stack.push(53)) return -1;
		if(!backtrack_stack.push(46)) return -1;
		if(!backtrack_stack.push(19)) return -1;
		goto label_16;
		}
		break;
	case 2600:
	// Label to restore values of label 574
		v_wine_1 = int_value_stack.pop();
		break;
	case 575: label_575: 
	// using  symbolic fact wine(v_wine_1) with literal 2 in rule 
	// 345: semillonorsauvignonblanc(A) :- q68(A), wine(A).
	// resulting in symbolic fact semillonorsauvignonblanc(v_semillonorsauvignonblanc_1)

		//recursive
		if(!int_value_stack.push(v_semillonorsauvignonblanc_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2601)) return -1;
		if (!q68_set.contains(v_wine_1)) continue;
	case 1588:
	// Continuation for label 575
			v_semillonorsauvignonblanc_1 = v_wine_1;
		if(semillonorsauvignonblanc_set.insert(v_semillonorsauvignonblanc_1)) {
			if(!semillonorsauvignonblanc_f.insert(v_semillonorsauvignonblanc_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1588)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(817)) return -1;
			if(!backtrack_stack.push(816)) return -1;
			if(!backtrack_stack.push(618)) return -1;
			if(!backtrack_stack.push(603)) return -1;
			if(!backtrack_stack.push(580)) return -1;
			if(!backtrack_stack.push(579)) return -1;
			if(!backtrack_stack.push(576)) return -1;
			goto label_574;
		}
		break;
	case 2601:
	// Label to restore values of label 575
		v_semillonorsauvignonblanc_1 = int_value_stack.pop();
		break;
	case 576: label_576: 
	// using  symbolic fact semillonorsauvignonblanc(v_semillonorsauvignonblanc_1) with literal 1 in rule 
	// 347: q68(A) :- semillonorsauvignonblanc(A).
	// resulting in symbolic fact q68(v_q68_1)

		//recursive
		if(!int_value_stack.push(v_q68_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2602)) return -1;
		v_q68_1 = v_semillonorsauvignonblanc_1;
		if(q68_set.insert(v_q68_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(582)) return -1;
		if(!backtrack_stack.push(577)) return -1;
		goto label_573;
		}
		break;
	case 2602:
	// Label to restore values of label 576
		v_q68_1 = int_value_stack.pop();
		break;
	case 577: label_577: 
	// using  symbolic fact q68(v_q68_1) with literal 1 in rule 
	// 351: ot____nom6(A) :- q68(B), madefromgrape(B,A).
	// resulting in symbolic fact ot____nom6(v_ot____nom6_1)

		//recursive
		c351_2_bf.push(&cursor_stack);
		if(!int_value_stack.push(v_ot____nom6_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2603)) return -1;
		c351_2_bf.open(v_q68_1);
	case 1590:
	// Continuation for label 577
		while (c351_2_bf.fetch()) {
			v_ot____nom6_1 = c351_2_bf.c2();
		if(ot____nom6_set.insert(v_ot____nom6_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1590)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(581)) return -1;
			goto label_578;
		}
		}
		break;
	case 2603:
	// Label to restore values of label 577
		v_ot____nom6_1 = int_value_stack.pop();
		c351_2_bf.pop(&cursor_stack);
		break;
	case 578: label_578: 
	// using  symbolic fact ot____nom6(v_ot____nom6_1) with literal 1 in rule 
	// 349: semillonorsauvignonblanc(A) :- ot____nom6(A), wine(A), kaon2namedobjects(A).
	// resulting in symbolic fact semillonorsauvignonblanc(v_semillonorsauvignonblanc_1)

		//recursive
		if(!int_value_stack.push(v_semillonorsauvignonblanc_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2604)) return -1;
		if (!wine_set.contains(v_ot____nom6_1)) continue;
			if (!kaon2namedobjects_b.contains(v_ot____nom6_1)) continue;
	case 1591:
	// Continuation for label 578
				v_semillonorsauvignonblanc_1 = v_ot____nom6_1;
		if(semillonorsauvignonblanc_set.insert(v_semillonorsauvignonblanc_1)) {
				if(!semillonorsauvignonblanc_f.insert(v_semillonorsauvignonblanc_1)) return -1;
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1591)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(817)) return -1;
				if(!backtrack_stack.push(816)) return -1;
				if(!backtrack_stack.push(618)) return -1;
				if(!backtrack_stack.push(603)) return -1;
				if(!backtrack_stack.push(580)) return -1;
				if(!backtrack_stack.push(579)) return -1;
				if(!backtrack_stack.push(576)) return -1;
				goto label_574;
		}
		break;
	case 2604:
	// Label to restore values of label 578
		v_semillonorsauvignonblanc_1 = int_value_stack.pop();
		break;
	case 579: label_579: 
	// using  symbolic fact semillonorsauvignonblanc(v_semillonorsauvignonblanc_1) with literal 1 in rule 
	// 348: semillonorsauvignonblanc(A) :- semillonorsauvignonblanc(B), kaon2equal(A,B).
	// resulting in symbolic fact semillonorsauvignonblanc(v_semillonorsauvignonblanc_1)

		//recursive
		c348_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_semillonorsauvignonblanc_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2605)) return -1;
		c348_2_fb.open(v_semillonorsauvignonblanc_1);
	case 1592:
	// Continuation for label 579
		while (c348_2_fb.fetch()) {
			v_semillonorsauvignonblanc_1 = c348_2_fb.c2();
		if(semillonorsauvignonblanc_set.insert(v_semillonorsauvignonblanc_1)) {
			if(!semillonorsauvignonblanc_f.insert(v_semillonorsauvignonblanc_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1592)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(817)) return -1;
			if(!backtrack_stack.push(816)) return -1;
			if(!backtrack_stack.push(618)) return -1;
			if(!backtrack_stack.push(603)) return -1;
			if(!backtrack_stack.push(580)) return -1;
			if(!backtrack_stack.push(579)) return -1;
			if(!backtrack_stack.push(576)) return -1;
			goto label_574;
		}
		}
		break;
	case 2605:
	// Label to restore values of label 579
		v_semillonorsauvignonblanc_1 = int_value_stack.pop();
		c348_2_fb.pop(&cursor_stack);
		break;
	case 580: label_580: 
	// using  symbolic fact semillonorsauvignonblanc(v_semillonorsauvignonblanc_1) with literal 1 in rule 
	// 598: q12(A) :- semillonorsauvignonblanc(A).
	// resulting in symbolic fact q12(v_q12_1)

		v_q12_1 = v_semillonorsauvignonblanc_1;
		if(q12_set.insert(v_q12_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(116)) return -1;
		if(!backtrack_stack.push(115)) return -1;
		if(!backtrack_stack.push(114)) return -1;
		if(!backtrack_stack.push(99)) return -1;
		goto label_95;
		}
		break;
	case 581: label_581: 
	// using  symbolic fact ot____nom6(v_ot____nom6_1) with literal 1 in rule 
	// 626: ot____nom6(A) :- ot____nom6(B), kaon2equal(A,B).
	// resulting in symbolic fact ot____nom6(v_ot____nom6_1)

		//recursive
		c626_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_ot____nom6_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2607)) return -1;
		c626_2_fb.open(v_ot____nom6_1);
	case 1594:
	// Continuation for label 581
		while (c626_2_fb.fetch()) {
			v_ot____nom6_1 = c626_2_fb.c2();
		if(ot____nom6_set.insert(v_ot____nom6_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1594)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(581)) return -1;
			goto label_578;
		}
		}
		break;
	case 2607:
	// Label to restore values of label 581
		v_ot____nom6_1 = int_value_stack.pop();
		c626_2_fb.pop(&cursor_stack);
		break;
	case 582: label_582: 
	// using  symbolic fact q68(v_q68_1) with literal 1 in rule 
	// 352: q68(A) :- q68(B), kaon2equal(A,B).
	// resulting in symbolic fact q68(v_q68_1)

		//recursive
		c352_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q68_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2608)) return -1;
		c352_2_fb.open(v_q68_1);
	case 1595:
	// Continuation for label 582
		while (c352_2_fb.fetch()) {
			v_q68_1 = c352_2_fb.c2();
		if(q68_set.insert(v_q68_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1595)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(582)) return -1;
			if(!backtrack_stack.push(577)) return -1;
			goto label_573;
		}
		}
		break;
	case 2608:
	// Label to restore values of label 582
		v_q68_1 = int_value_stack.pop();
		c352_2_fb.pop(&cursor_stack);
		break;
	case 583: label_583: 
	// using  symbolic fact wine(v_wine_1) with literal 2 in rule 
	// 349: semillonorsauvignonblanc(A) :- ot____nom6(A), wine(A), kaon2namedobjects(A).
	// resulting in symbolic fact semillonorsauvignonblanc(v_semillonorsauvignonblanc_1)

		//recursive
		if(!int_value_stack.push(v_semillonorsauvignonblanc_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2609)) return -1;
		if (!ot____nom6_set.contains(v_wine_1)) continue;
			if (!kaon2namedobjects_b.contains(v_wine_1)) continue;
	case 1596:
	// Continuation for label 583
				v_semillonorsauvignonblanc_1 = v_wine_1;
		if(semillonorsauvignonblanc_set.insert(v_semillonorsauvignonblanc_1)) {
				if(!semillonorsauvignonblanc_f.insert(v_semillonorsauvignonblanc_1)) return -1;
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1596)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(817)) return -1;
				if(!backtrack_stack.push(816)) return -1;
				if(!backtrack_stack.push(618)) return -1;
				if(!backtrack_stack.push(603)) return -1;
				if(!backtrack_stack.push(580)) return -1;
				if(!backtrack_stack.push(579)) return -1;
				if(!backtrack_stack.push(576)) return -1;
				goto label_574;
		}
		break;
	case 2609:
	// Label to restore values of label 583
		v_semillonorsauvignonblanc_1 = int_value_stack.pop();
		break;
	case 584: label_584: 
	// using  symbolic fact q2(v_q2_1) with literal 1 in rule 
	// 314: q2(A) :- q2(B), kaon2equal(A,B).
	// resulting in symbolic fact q2(v_q2_1)

		//recursive
		c314_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q2_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2610)) return -1;
		c314_2_fb.open(v_q2_1);
	case 1597:
	// Continuation for label 584
		while (c314_2_fb.fetch()) {
			v_q2_1 = c314_2_fb.c2();
		if(q2_set.insert(v_q2_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1597)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(585)) return -1;
			if(!backtrack_stack.push(584)) return -1;
			goto label_557;
		}
		}
		break;
	case 2610:
	// Label to restore values of label 584
		v_q2_1 = int_value_stack.pop();
		c314_2_fb.pop(&cursor_stack);
		break;
	case 585: label_585: 
	// using  symbolic fact q2(v_q2_1) with literal 2 in rule 
	// 308: q1(A) :- locatedin(A,B), q2(B).
	// resulting in symbolic fact q1(v_q1_1)

		//recursive
		c308_1_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q1_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2611)) return -1;
		c308_1_fb.open(v_q2_1);
	case 1598:
	// Continuation for label 585
		while (c308_1_fb.fetch()) {
			v_q1_1 = c308_1_fb.c2();
		if(q1_set.insert(v_q1_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1598)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(589)) return -1;
			if(!backtrack_stack.push(586)) return -1;
			if(!backtrack_stack.push(556)) return -1;
			goto label_527;
		}
		}
		break;
	case 2611:
	// Label to restore values of label 585
		v_q1_1 = int_value_stack.pop();
		c308_1_fb.pop(&cursor_stack);
		break;
	case 586: label_586: 
	// using  symbolic fact q1(v_q1_1) with literal 1 in rule 
	// 311: ot____nom23(A) :- q1(A), kaon2namedobjects(A).
	// resulting in symbolic fact ot____nom23(v_ot____nom23_1)

		//recursive
		if(!int_value_stack.push(v_ot____nom23_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2612)) return -1;
		if (!kaon2namedobjects_b.contains(v_q1_1)) continue;
	case 1599:
	// Continuation for label 586
			v_ot____nom23_1 = v_q1_1;
		if(ot____nom23_set.insert(v_ot____nom23_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1599)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(588)) return -1;
			goto label_587;
		}
		break;
	case 2612:
	// Label to restore values of label 586
		v_ot____nom23_1 = int_value_stack.pop();
		break;
	case 587: label_587: 
	// using  symbolic fact ot____nom23(v_ot____nom23_1) with literal 1 in rule 
	// 313: ot____nom23(A) :- ot____nom23(B), kaon2equal(A,B).
	// resulting in symbolic fact ot____nom23(v_ot____nom23_1)

		//recursive
		c313_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_ot____nom23_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2613)) return -1;
		c313_2_fb.open(v_ot____nom23_1);
	case 1600:
	// Continuation for label 587
		while (c313_2_fb.fetch()) {
			v_ot____nom23_1 = c313_2_fb.c2();
		if(ot____nom23_set.insert(v_ot____nom23_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1600)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(588)) return -1;
			goto label_587;
		}
		}
		break;
	case 2613:
	// Label to restore values of label 587
		v_ot____nom23_1 = int_value_stack.pop();
		c313_2_fb.pop(&cursor_stack);
		break;
	case 588: label_588: 
	// using  symbolic fact ot____nom23(v_ot____nom23_1) with literal 2 in rule 
	// 309: q2(A) :- locatedin(A,B), ot____nom23(B).
	// resulting in symbolic fact q2(v_q2_1)

		//recursive
		c309_1_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q2_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2614)) return -1;
		c309_1_fb.open(v_ot____nom23_1);
	case 1601:
	// Continuation for label 588
		while (c309_1_fb.fetch()) {
			v_q2_1 = c309_1_fb.c2();
		if(q2_set.insert(v_q2_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1601)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(585)) return -1;
			if(!backtrack_stack.push(584)) return -1;
			goto label_557;
		}
		}
		break;
	case 2614:
	// Label to restore values of label 588
		v_q2_1 = int_value_stack.pop();
		c309_1_fb.pop(&cursor_stack);
		break;
	case 589: label_589: 
	// using  symbolic fact q1(v_q1_1) with literal 1 in rule 
	// 312: q1(A) :- q1(B), kaon2equal(A,B).
	// resulting in symbolic fact q1(v_q1_1)

		//recursive
		c312_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q1_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2615)) return -1;
		c312_2_fb.open(v_q1_1);
	case 1602:
	// Continuation for label 589
		while (c312_2_fb.fetch()) {
			v_q1_1 = c312_2_fb.c2();
		if(q1_set.insert(v_q1_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1602)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(589)) return -1;
			if(!backtrack_stack.push(586)) return -1;
			if(!backtrack_stack.push(556)) return -1;
			goto label_527;
		}
		}
		break;
	case 2615:
	// Label to restore values of label 589
		v_q1_1 = int_value_stack.pop();
		c312_2_fb.pop(&cursor_stack);
		break;
	case 590:
	// applying rule 
	// 98: locatedin(A,B) :- locatedin_aux(A,B).
	// resulting in symbolic fact locatedin(v_locatedin_1,v_locatedin_2)
		c98_1_ff.open();
	case 1603:
	// Continuation for label 590
		while (c98_1_ff.fetch()) {
			v_locatedin_2 = c98_1_ff.c2();
			v_locatedin_1 = c98_1_ff.c1();
		if(locatedin_set.insert(v_locatedin_1, v_locatedin_2)) {
			if(!locatedin_bf.insert(v_locatedin_1, v_locatedin_2)) return -1;
			if(!locatedin_fb.insert(v_locatedin_2, v_locatedin_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1603)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(1005)) return -1;
			if(!backtrack_stack.push(999)) return -1;
			if(!backtrack_stack.push(987)) return -1;
			if(!backtrack_stack.push(981)) return -1;
			if(!backtrack_stack.push(969)) return -1;
			if(!backtrack_stack.push(963)) return -1;
			if(!backtrack_stack.push(952)) return -1;
			if(!backtrack_stack.push(946)) return -1;
			if(!backtrack_stack.push(936)) return -1;
			if(!backtrack_stack.push(930)) return -1;
			if(!backtrack_stack.push(920)) return -1;
			if(!backtrack_stack.push(917)) return -1;
			if(!backtrack_stack.push(898)) return -1;
			if(!backtrack_stack.push(897)) return -1;
			if(!backtrack_stack.push(896)) return -1;
			if(!backtrack_stack.push(893)) return -1;
			if(!backtrack_stack.push(726)) return -1;
			if(!backtrack_stack.push(721)) return -1;
			if(!backtrack_stack.push(694)) return -1;
			if(!backtrack_stack.push(693)) return -1;
			if(!backtrack_stack.push(688)) return -1;
			if(!backtrack_stack.push(687)) return -1;
			if(!backtrack_stack.push(636)) return -1;
			if(!backtrack_stack.push(631)) return -1;
			if(!backtrack_stack.push(592)) return -1;
			if(!backtrack_stack.push(591)) return -1;
			if(!backtrack_stack.push(529)) return -1;
			if(!backtrack_stack.push(528)) return -1;
			if(!backtrack_stack.push(513)) return -1;
			if(!backtrack_stack.push(512)) return -1;
			if(!backtrack_stack.push(511)) return -1;
			if(!backtrack_stack.push(506)) return -1;
			if(!backtrack_stack.push(455)) return -1;
			if(!backtrack_stack.push(450)) return -1;
			if(!backtrack_stack.push(441)) return -1;
			if(!backtrack_stack.push(440)) return -1;
			if(!backtrack_stack.push(439)) return -1;
			if(!backtrack_stack.push(434)) return -1;
			if(!backtrack_stack.push(339)) return -1;
			if(!backtrack_stack.push(338)) return -1;
			if(!backtrack_stack.push(249)) return -1;
			if(!backtrack_stack.push(244)) return -1;
			if(!backtrack_stack.push(52)) return -1;
			if(!backtrack_stack.push(51)) return -1;
			if(!backtrack_stack.push(50)) return -1;
			if(!backtrack_stack.push(49)) return -1;
			if(!backtrack_stack.push(48)) return -1;
			goto label_47;
		}
		}
		break;
	case 591: label_591: 
	// using  symbolic fact locatedin(v_locatedin_1,v_locatedin_2) with literal 1 in rule 
	// 308: q1(A) :- locatedin(A,B), q2(B).
	// resulting in symbolic fact q1(v_q1_1)

		if (!q2_set.contains(v_locatedin_2)) continue;
	case 1604:
	// Continuation for label 591
			v_q1_1 = v_locatedin_1;
		if(q1_set.insert(v_q1_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1604)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(589)) return -1;
			if(!backtrack_stack.push(586)) return -1;
			if(!backtrack_stack.push(556)) return -1;
			goto label_527;
		}
		break;
	case 592: label_592: 
	// using  symbolic fact locatedin(v_locatedin_1,v_locatedin_2) with literal 1 in rule 
	// 309: q2(A) :- locatedin(A,B), ot____nom23(B).
	// resulting in symbolic fact q2(v_q2_1)

		if (!ot____nom23_set.contains(v_locatedin_2)) continue;
	case 1605:
	// Continuation for label 592
			v_q2_1 = v_locatedin_1;
		if(q2_set.insert(v_q2_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1605)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(585)) return -1;
			if(!backtrack_stack.push(584)) return -1;
			goto label_557;
		}
		break;
	case 593:
	// applying rule 
	// 102: sancerre(A) :- sancerre_aux(A).
	// resulting in symbolic fact sancerre(v_sancerre_1)
		c102_1_f.open();
	case 1606:
	// Continuation for label 593
		while (c102_1_f.fetch()) {
			v_sancerre_1 = c102_1_f.c1();
		if(sancerre_set.insert(v_sancerre_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1606)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(627)) return -1;
			if(!backtrack_stack.push(623)) return -1;
			if(!backtrack_stack.push(619)) return -1;
			if(!backtrack_stack.push(600)) return -1;
			if(!backtrack_stack.push(599)) return -1;
			if(!backtrack_stack.push(598)) return -1;
			if(!backtrack_stack.push(595)) return -1;
			goto label_594;
		}
		}
		break;
	case 594: label_594: 
	// using  symbolic fact sancerre(v_sancerre_1) with literal 1 in rule 
	// 101: loire(A) :- sancerre(A).
	// resulting in symbolic fact loire(v_loire_1)

		v_loire_1 = v_sancerre_1;
		if(loire_set.insert(v_loire_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(971)) return -1;
		if(!backtrack_stack.push(715)) return -1;
		if(!backtrack_stack.push(714)) return -1;
		if(!backtrack_stack.push(487)) return -1;
		if(!backtrack_stack.push(486)) return -1;
		if(!backtrack_stack.push(484)) return -1;
		if(!backtrack_stack.push(482)) return -1;
		goto label_478;
		}
		break;
	case 595: label_595: 
	// using  symbolic fact sancerre(v_sancerre_1) with literal 1 in rule 
	// 105: q39(A) :- sancerre(A).
	// resulting in symbolic fact q39(v_q39_1)

		//recursive
		if(!int_value_stack.push(v_q39_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2621)) return -1;
		v_q39_1 = v_sancerre_1;
		if(q39_set.insert(v_q39_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(633)) return -1;
		if(!backtrack_stack.push(632)) return -1;
		if(!backtrack_stack.push(630)) return -1;
		goto label_596;
		}
		break;
	case 2621:
	// Label to restore values of label 595
		v_q39_1 = int_value_stack.pop();
		break;
	case 596: label_596: 
	// using  symbolic fact q39(v_q39_1) with literal 1 in rule 
	// 104: q40(A) :- q39(A).
	// resulting in symbolic fact q40(v_q40_1)

		//recursive
		if(!int_value_stack.push(v_q40_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2622)) return -1;
		v_q40_1 = v_q39_1;
		if(q40_set.insert(v_q40_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(629)) return -1;
		if(!backtrack_stack.push(628)) return -1;
		goto label_597;
		}
		break;
	case 2622:
	// Label to restore values of label 596
		v_q40_1 = int_value_stack.pop();
		break;
	case 597: label_597: 
	// using  symbolic fact q40(v_q40_1) with literal 1 in rule 
	// 103: sancerre(A) :- q40(A), loire(A).
	// resulting in symbolic fact sancerre(v_sancerre_1)

		//recursive
		if(!int_value_stack.push(v_sancerre_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2623)) return -1;
		if (!loire_set.contains(v_q40_1)) continue;
	case 1610:
	// Continuation for label 597
			v_sancerre_1 = v_q40_1;
		if(sancerre_set.insert(v_sancerre_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1610)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(627)) return -1;
			if(!backtrack_stack.push(623)) return -1;
			if(!backtrack_stack.push(619)) return -1;
			if(!backtrack_stack.push(600)) return -1;
			if(!backtrack_stack.push(599)) return -1;
			if(!backtrack_stack.push(598)) return -1;
			if(!backtrack_stack.push(595)) return -1;
			goto label_594;
		}
		break;
	case 2623:
	// Label to restore values of label 597
		v_sancerre_1 = int_value_stack.pop();
		break;
	case 598: label_598: 
	// using  symbolic fact sancerre(v_sancerre_1) with literal 1 in rule 
	// 106: sancerre(A) :- sancerre(B), kaon2equal(A,B).
	// resulting in symbolic fact sancerre(v_sancerre_1)

		//recursive
		c106_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_sancerre_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2624)) return -1;
		c106_2_fb.open(v_sancerre_1);
	case 1611:
	// Continuation for label 598
		while (c106_2_fb.fetch()) {
			v_sancerre_1 = c106_2_fb.c2();
		if(sancerre_set.insert(v_sancerre_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1611)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(627)) return -1;
			if(!backtrack_stack.push(623)) return -1;
			if(!backtrack_stack.push(619)) return -1;
			if(!backtrack_stack.push(600)) return -1;
			if(!backtrack_stack.push(599)) return -1;
			if(!backtrack_stack.push(598)) return -1;
			if(!backtrack_stack.push(595)) return -1;
			goto label_594;
		}
		}
		break;
	case 2624:
	// Label to restore values of label 598
		v_sancerre_1 = int_value_stack.pop();
		c106_2_fb.pop(&cursor_stack);
		break;
	case 599: label_599: 
	// using  symbolic fact sancerre(v_sancerre_1) with literal 1 in rule 
	// 136: q20(A) :- sancerre(A).
	// resulting in symbolic fact q20(v_q20_1)

		v_q20_1 = v_sancerre_1;
		if(q20_set.insert(v_q20_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(184)) return -1;
		if(!backtrack_stack.push(183)) return -1;
		goto label_182;
		}
		break;
	case 600: label_600: 
	// using  symbolic fact sancerre(v_sancerre_1) with literal 1 in rule 
	// 341: q44(A) :- sancerre(A).
	// resulting in symbolic fact q44(v_q44_1)

		v_q44_1 = v_sancerre_1;
		if(q44_set.insert(v_q44_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(610)) return -1;
		if(!backtrack_stack.push(608)) return -1;
		if(!backtrack_stack.push(607)) return -1;
		if(!backtrack_stack.push(606)) return -1;
		if(!backtrack_stack.push(605)) return -1;
		goto label_601;
		}
		break;
	case 601: label_601: 
	// using  symbolic fact q44(v_q44_1) with literal 1 in rule 
	// 340: sauvignonblanc(A) :- q44(A), semillonorsauvignonblanc(A).
	// resulting in symbolic fact sauvignonblanc(v_sauvignonblanc_1)

		//recursive
		if(!int_value_stack.push(v_sauvignonblanc_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2627)) return -1;
		if (!semillonorsauvignonblanc_set.contains(v_q44_1)) continue;
	case 1614:
	// Continuation for label 601
			v_sauvignonblanc_1 = v_q44_1;
		if(sauvignonblanc_set.insert(v_sauvignonblanc_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1614)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(614)) return -1;
			if(!backtrack_stack.push(613)) return -1;
			if(!backtrack_stack.push(604)) return -1;
			goto label_602;
		}
		break;
	case 2627:
	// Label to restore values of label 601
		v_sauvignonblanc_1 = int_value_stack.pop();
		break;
	case 602: label_602: 
	// using  symbolic fact sauvignonblanc(v_sauvignonblanc_1) with literal 1 in rule 
	// 338: semillonorsauvignonblanc(A) :- sauvignonblanc(A).
	// resulting in symbolic fact semillonorsauvignonblanc(v_semillonorsauvignonblanc_1)

		//recursive
		if(!int_value_stack.push(v_semillonorsauvignonblanc_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2628)) return -1;
		v_semillonorsauvignonblanc_1 = v_sauvignonblanc_1;
		if(semillonorsauvignonblanc_set.insert(v_semillonorsauvignonblanc_1)) {
		if(!semillonorsauvignonblanc_f.insert(v_semillonorsauvignonblanc_1)) return -1;
		// Labels for next rules
		if(!backtrack_stack.push(817)) return -1;
		if(!backtrack_stack.push(816)) return -1;
		if(!backtrack_stack.push(618)) return -1;
		if(!backtrack_stack.push(603)) return -1;
		if(!backtrack_stack.push(580)) return -1;
		if(!backtrack_stack.push(579)) return -1;
		if(!backtrack_stack.push(576)) return -1;
		goto label_574;
		}
		break;
	case 2628:
	// Label to restore values of label 602
		v_semillonorsauvignonblanc_1 = int_value_stack.pop();
		break;
	case 603: label_603: 
	// using  symbolic fact semillonorsauvignonblanc(v_semillonorsauvignonblanc_1) with literal 2 in rule 
	// 340: sauvignonblanc(A) :- q44(A), semillonorsauvignonblanc(A).
	// resulting in symbolic fact sauvignonblanc(v_sauvignonblanc_1)

		//recursive
		if(!int_value_stack.push(v_sauvignonblanc_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2629)) return -1;
		if (!q44_set.contains(v_semillonorsauvignonblanc_1)) continue;
	case 1616:
	// Continuation for label 603
			v_sauvignonblanc_1 = v_semillonorsauvignonblanc_1;
		if(sauvignonblanc_set.insert(v_sauvignonblanc_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1616)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(614)) return -1;
			if(!backtrack_stack.push(613)) return -1;
			if(!backtrack_stack.push(604)) return -1;
			goto label_602;
		}
		break;
	case 2629:
	// Label to restore values of label 603
		v_sauvignonblanc_1 = int_value_stack.pop();
		break;
	case 604: label_604: 
	// using  symbolic fact sauvignonblanc(v_sauvignonblanc_1) with literal 1 in rule 
	// 342: q44(A) :- sauvignonblanc(A).
	// resulting in symbolic fact q44(v_q44_1)

		//recursive
		if(!int_value_stack.push(v_q44_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2630)) return -1;
		v_q44_1 = v_sauvignonblanc_1;
		if(q44_set.insert(v_q44_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(610)) return -1;
		if(!backtrack_stack.push(608)) return -1;
		if(!backtrack_stack.push(607)) return -1;
		if(!backtrack_stack.push(606)) return -1;
		if(!backtrack_stack.push(605)) return -1;
		goto label_601;
		}
		break;
	case 2630:
	// Label to restore values of label 604
		v_q44_1 = int_value_stack.pop();
		break;
	case 605: label_605: 
	// using  symbolic fact q44(v_q44_1) with literal 1 in rule 
	// 424: wine(A) :- q44(A).
	// resulting in symbolic fact wine(v_wine_1)

		v_wine_1 = v_q44_1;
		if(wine_set.insert(v_wine_1)) {
		if(!wine_f.insert(v_wine_1)) return -1;
		// Labels for next rules
		if(!backtrack_stack.push(1007)) return -1;
		if(!backtrack_stack.push(989)) return -1;
		if(!backtrack_stack.push(954)) return -1;
		if(!backtrack_stack.push(938)) return -1;
		if(!backtrack_stack.push(911)) return -1;
		if(!backtrack_stack.push(903)) return -1;
		if(!backtrack_stack.push(902)) return -1;
		if(!backtrack_stack.push(901)) return -1;
		if(!backtrack_stack.push(849)) return -1;
		if(!backtrack_stack.push(848)) return -1;
		if(!backtrack_stack.push(821)) return -1;
		if(!backtrack_stack.push(820)) return -1;
		if(!backtrack_stack.push(737)) return -1;
		if(!backtrack_stack.push(712)) return -1;
		if(!backtrack_stack.push(674)) return -1;
		if(!backtrack_stack.push(672)) return -1;
		if(!backtrack_stack.push(583)) return -1;
		if(!backtrack_stack.push(575)) return -1;
		if(!backtrack_stack.push(563)) return -1;
		if(!backtrack_stack.push(551)) return -1;
		if(!backtrack_stack.push(549)) return -1;
		if(!backtrack_stack.push(483)) return -1;
		if(!backtrack_stack.push(473)) return -1;
		if(!backtrack_stack.push(470)) return -1;
		if(!backtrack_stack.push(430)) return -1;
		if(!backtrack_stack.push(401)) return -1;
		if(!backtrack_stack.push(399)) return -1;
		if(!backtrack_stack.push(380)) return -1;
		if(!backtrack_stack.push(364)) return -1;
		if(!backtrack_stack.push(363)) return -1;
		if(!backtrack_stack.push(329)) return -1;
		if(!backtrack_stack.push(328)) return -1;
		if(!backtrack_stack.push(327)) return -1;
		if(!backtrack_stack.push(295)) return -1;
		if(!backtrack_stack.push(294)) return -1;
		if(!backtrack_stack.push(216)) return -1;
		if(!backtrack_stack.push(210)) return -1;
		if(!backtrack_stack.push(192)) return -1;
		if(!backtrack_stack.push(180)) return -1;
		if(!backtrack_stack.push(179)) return -1;
		if(!backtrack_stack.push(178)) return -1;
		if(!backtrack_stack.push(154)) return -1;
		if(!backtrack_stack.push(153)) return -1;
		if(!backtrack_stack.push(100)) return -1;
		if(!backtrack_stack.push(76)) return -1;
		if(!backtrack_stack.push(75)) return -1;
		if(!backtrack_stack.push(67)) return -1;
		if(!backtrack_stack.push(63)) return -1;
		if(!backtrack_stack.push(53)) return -1;
		if(!backtrack_stack.push(46)) return -1;
		if(!backtrack_stack.push(19)) return -1;
		goto label_16;
		}
		break;
	case 606: label_606: 
	// using  symbolic fact q44(v_q44_1) with literal 1 in rule 
	// 425: q44(A) :- q44(B), kaon2equal(A,B).
	// resulting in symbolic fact q44(v_q44_1)

		//recursive
		c425_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q44_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2632)) return -1;
		c425_2_fb.open(v_q44_1);
	case 1619:
	// Continuation for label 606
		while (c425_2_fb.fetch()) {
			v_q44_1 = c425_2_fb.c2();
		if(q44_set.insert(v_q44_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1619)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(610)) return -1;
			if(!backtrack_stack.push(608)) return -1;
			if(!backtrack_stack.push(607)) return -1;
			if(!backtrack_stack.push(606)) return -1;
			if(!backtrack_stack.push(605)) return -1;
			goto label_601;
		}
		}
		break;
	case 2632:
	// Label to restore values of label 606
		v_q44_1 = int_value_stack.pop();
		c425_2_fb.pop(&cursor_stack);
		break;
	case 607: label_607: 
	// using  symbolic fact q44(v_q44_1) with literal 1 in rule 
	// 506: q5(A) :- q44(A), semillonorsauvignonblanc(A).
	// resulting in symbolic fact q5(v_q5_1)

		if (!semillonorsauvignonblanc_set.contains(v_q44_1)) continue;
	case 1620:
	// Continuation for label 607
			v_q5_1 = v_q44_1;
		if(q5_set.insert(v_q5_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1620)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(136)) return -1;
			if(!backtrack_stack.push(135)) return -1;
			goto label_131;
		}
		break;
	case 608: label_608: 
	// using  symbolic fact q44(v_q44_1) with literal 1 in rule 
	// 527: madefromgrape(A,A) :- q44(A), kaon2namedobjects(A).
	// resulting in symbolic fact madefromgrape(v_madefromgrape_1,v_madefromgrape_2)

		//recursive
		if(!int_value_stack.push(v_madefromgrape_2)) return -1;
		if(!int_value_stack.push(v_madefromgrape_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2634)) return -1;
		if (!kaon2namedobjects_b.contains(v_q44_1)) continue;
	case 1621:
	// Continuation for label 608
			v_madefromgrape_1 = v_q44_1;
			v_madefromgrape_2 = v_q44_1;
		if(madefromgrape_set.insert(v_madefromgrape_1, v_madefromgrape_2)) {
			if(!madefromgrape_bf.insert(v_madefromgrape_1, v_madefromgrape_2)) return -1;
			if(!madefromgrape_fb.insert(v_madefromgrape_2, v_madefromgrape_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1621)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(860)) return -1;
			if(!backtrack_stack.push(859)) return -1;
			if(!backtrack_stack.push(845)) return -1;
			if(!backtrack_stack.push(844)) return -1;
			if(!backtrack_stack.push(829)) return -1;
			if(!backtrack_stack.push(828)) return -1;
			if(!backtrack_stack.push(813)) return -1;
			if(!backtrack_stack.push(812)) return -1;
			if(!backtrack_stack.push(677)) return -1;
			if(!backtrack_stack.push(676)) return -1;
			if(!backtrack_stack.push(617)) return -1;
			if(!backtrack_stack.push(616)) return -1;
			if(!backtrack_stack.push(609)) return -1;
			if(!backtrack_stack.push(554)) return -1;
			if(!backtrack_stack.push(553)) return -1;
			if(!backtrack_stack.push(476)) return -1;
			if(!backtrack_stack.push(475)) return -1;
			if(!backtrack_stack.push(404)) return -1;
			if(!backtrack_stack.push(403)) return -1;
			if(!backtrack_stack.push(362)) return -1;
			if(!backtrack_stack.push(361)) return -1;
			if(!backtrack_stack.push(343)) return -1;
			if(!backtrack_stack.push(342)) return -1;
			if(!backtrack_stack.push(300)) return -1;
			if(!backtrack_stack.push(170)) return -1;
			if(!backtrack_stack.push(169)) return -1;
			if(!backtrack_stack.push(151)) return -1;
			if(!backtrack_stack.push(150)) return -1;
			if(!backtrack_stack.push(144)) return -1;
			if(!backtrack_stack.push(132)) return -1;
			if(!backtrack_stack.push(71)) return -1;
			if(!backtrack_stack.push(70)) return -1;
			if(!backtrack_stack.push(66)) return -1;
			goto label_64;
		}
		break;
	case 2634:
	// Label to restore values of label 608
		v_madefromgrape_1 = int_value_stack.pop();
		v_madefromgrape_2 = int_value_stack.pop();
		break;
	case 609: label_609: 
	// using  symbolic fact madefromgrape(v_madefromgrape_1,v_madefromgrape_2) with literal 2 in rule 
	// 351: ot____nom6(A) :- q68(B), madefromgrape(B,A).
	// resulting in symbolic fact ot____nom6(v_ot____nom6_1)

		if (!q68_set.contains(v_madefromgrape_1)) continue;
	case 1622:
	// Continuation for label 609
			v_ot____nom6_1 = v_madefromgrape_2;
		if(ot____nom6_set.insert(v_ot____nom6_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1622)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(581)) return -1;
			goto label_578;
		}
		break;
	case 610: label_610: 
	// using  symbolic fact q44(v_q44_1) with literal 1 in rule 
	// 544: ot____nom11(A) :- q44(A), kaon2namedobjects(A).
	// resulting in symbolic fact ot____nom11(v_ot____nom11_1)

		//recursive
		if(!int_value_stack.push(v_ot____nom11_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2636)) return -1;
		if (!kaon2namedobjects_b.contains(v_q44_1)) continue;
	case 1623:
	// Continuation for label 610
			v_ot____nom11_1 = v_q44_1;
		if(ot____nom11_set.insert(v_ot____nom11_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1623)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(615)) return -1;
			if(!backtrack_stack.push(612)) return -1;
			goto label_611;
		}
		break;
	case 2636:
	// Label to restore values of label 610
		v_ot____nom11_1 = int_value_stack.pop();
		break;
	case 611: label_611: 
	// using  symbolic fact ot____nom11(v_ot____nom11_1) with literal 1 in rule 
	// 545: ot____nom11(A) :- ot____nom11(B), kaon2equal(A,B).
	// resulting in symbolic fact ot____nom11(v_ot____nom11_1)

		//recursive
		c545_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_ot____nom11_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2637)) return -1;
		c545_2_fb.open(v_ot____nom11_1);
	case 1624:
	// Continuation for label 611
		while (c545_2_fb.fetch()) {
			v_ot____nom11_1 = c545_2_fb.c2();
		if(ot____nom11_set.insert(v_ot____nom11_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1624)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(615)) return -1;
			if(!backtrack_stack.push(612)) return -1;
			goto label_611;
		}
		}
		break;
	case 2637:
	// Label to restore values of label 611
		v_ot____nom11_1 = int_value_stack.pop();
		c545_2_fb.pop(&cursor_stack);
		break;
	case 612: label_612: 
	// using  symbolic fact ot____nom11(v_ot____nom11_1) with literal 3 in rule 
	// 344: sauvignonblanc(A) :- semillonorsauvignonblanc(A), madefromgrape(A,B), ot____nom11(B).
	// resulting in symbolic fact sauvignonblanc(v_sauvignonblanc_1)

		//recursive
		c344_1_f.push(&cursor_stack);
		if(!int_value_stack.push(v_sauvignonblanc_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2638)) return -1;
		c344_1_f.open();
		while (c344_1_f.fetch()) {
			if (!madefromgrape_set.contains(c344_1_f.c1(), v_ot____nom11_1)) continue;
	case 1625:
	// Continuation for label 612
				v_sauvignonblanc_1 = c344_1_f.c1();
		if(sauvignonblanc_set.insert(v_sauvignonblanc_1)) {
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1625)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(614)) return -1;
				if(!backtrack_stack.push(613)) return -1;
				if(!backtrack_stack.push(604)) return -1;
				goto label_602;
		}
		}
		break;
	case 2638:
	// Label to restore values of label 612
		v_sauvignonblanc_1 = int_value_stack.pop();
		c344_1_f.pop(&cursor_stack);
		break;
	case 613: label_613: 
	// using  symbolic fact sauvignonblanc(v_sauvignonblanc_1) with literal 1 in rule 
	// 343: sauvignonblanc(A) :- sauvignonblanc(B), kaon2equal(A,B).
	// resulting in symbolic fact sauvignonblanc(v_sauvignonblanc_1)

		//recursive
		c343_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_sauvignonblanc_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2639)) return -1;
		c343_2_fb.open(v_sauvignonblanc_1);
	case 1626:
	// Continuation for label 613
		while (c343_2_fb.fetch()) {
			v_sauvignonblanc_1 = c343_2_fb.c2();
		if(sauvignonblanc_set.insert(v_sauvignonblanc_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1626)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(614)) return -1;
			if(!backtrack_stack.push(613)) return -1;
			if(!backtrack_stack.push(604)) return -1;
			goto label_602;
		}
		}
		break;
	case 2639:
	// Label to restore values of label 613
		v_sauvignonblanc_1 = int_value_stack.pop();
		c343_2_fb.pop(&cursor_stack);
		break;
	case 614: label_614: 
	// using  symbolic fact sauvignonblanc(v_sauvignonblanc_1) with literal 1 in rule 
	// 776: q3(A) :- sauvignonblanc(A).
	// resulting in symbolic fact q3(v_q3_1)

		v_q3_1 = v_sauvignonblanc_1;
		if(q3_set.insert(v_q3_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(143)) return -1;
		goto label_123;
		}
		break;
	case 615: label_615: 
	// using  symbolic fact ot____nom11(v_ot____nom11_1) with literal 3 in rule 
	// 542: q5(A) :- semillonorsauvignonblanc(A), madefromgrape(A,B), ot____nom11(B).
	// resulting in symbolic fact q5(v_q5_1)

		c542_1_f.open();
		while (c542_1_f.fetch()) {
			if (!madefromgrape_set.contains(c542_1_f.c1(), v_ot____nom11_1)) continue;
	case 1628:
	// Continuation for label 615
				v_q5_1 = c542_1_f.c1();
		if(q5_set.insert(v_q5_1)) {
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1628)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(136)) return -1;
				if(!backtrack_stack.push(135)) return -1;
				goto label_131;
		}
		}
		break;
	case 616: label_616: 
	// using  symbolic fact madefromgrape(v_madefromgrape_1,v_madefromgrape_2) with literal 2 in rule 
	// 344: sauvignonblanc(A) :- semillonorsauvignonblanc(A), madefromgrape(A,B), ot____nom11(B).
	// resulting in symbolic fact sauvignonblanc(v_sauvignonblanc_1)

		//recursive
		if(!int_value_stack.push(v_sauvignonblanc_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2642)) return -1;
		if (!semillonorsauvignonblanc_set.contains(v_madefromgrape_1)) continue;
			if (!ot____nom11_set.contains(v_madefromgrape_2)) continue;
	case 1629:
	// Continuation for label 616
				v_sauvignonblanc_1 = v_madefromgrape_1;
		if(sauvignonblanc_set.insert(v_sauvignonblanc_1)) {
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1629)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(614)) return -1;
				if(!backtrack_stack.push(613)) return -1;
				if(!backtrack_stack.push(604)) return -1;
				goto label_602;
		}
		break;
	case 2642:
	// Label to restore values of label 616
		v_sauvignonblanc_1 = int_value_stack.pop();
		break;
	case 617: label_617: 
	// using  symbolic fact madefromgrape(v_madefromgrape_1,v_madefromgrape_2) with literal 2 in rule 
	// 542: q5(A) :- semillonorsauvignonblanc(A), madefromgrape(A,B), ot____nom11(B).
	// resulting in symbolic fact q5(v_q5_1)

		if (!semillonorsauvignonblanc_set.contains(v_madefromgrape_1)) continue;
			if (!ot____nom11_set.contains(v_madefromgrape_2)) continue;
	case 1630:
	// Continuation for label 617
				v_q5_1 = v_madefromgrape_1;
		if(q5_set.insert(v_q5_1)) {
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1630)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(136)) return -1;
				if(!backtrack_stack.push(135)) return -1;
				goto label_131;
		}
		break;
	case 618: label_618: 
	// using  symbolic fact semillonorsauvignonblanc(v_semillonorsauvignonblanc_1) with literal 2 in rule 
	// 506: q5(A) :- q44(A), semillonorsauvignonblanc(A).
	// resulting in symbolic fact q5(v_q5_1)

		if (!q44_set.contains(v_semillonorsauvignonblanc_1)) continue;
	case 1631:
	// Continuation for label 618
			v_q5_1 = v_semillonorsauvignonblanc_1;
		if(q5_set.insert(v_q5_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1631)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(136)) return -1;
			if(!backtrack_stack.push(135)) return -1;
			goto label_131;
		}
		break;
	case 619: label_619: 
	// using  symbolic fact sancerre(v_sancerre_1) with literal 1 in rule 
	// 409: q7(A) :- sancerre(A).
	// resulting in symbolic fact q7(v_q7_1)

		v_q7_1 = v_sancerre_1;
		if(q7_set.insert(v_q7_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(622)) return -1;
		if(!backtrack_stack.push(621)) return -1;
		goto label_620;
		}
		break;
	case 620: label_620: 
	// using  symbolic fact q7(v_q7_1) with literal 1 in rule 
	// 406: wine(A) :- q7(A).
	// resulting in symbolic fact wine(v_wine_1)

		v_wine_1 = v_q7_1;
		if(wine_set.insert(v_wine_1)) {
		if(!wine_f.insert(v_wine_1)) return -1;
		// Labels for next rules
		if(!backtrack_stack.push(1007)) return -1;
		if(!backtrack_stack.push(989)) return -1;
		if(!backtrack_stack.push(954)) return -1;
		if(!backtrack_stack.push(938)) return -1;
		if(!backtrack_stack.push(911)) return -1;
		if(!backtrack_stack.push(903)) return -1;
		if(!backtrack_stack.push(902)) return -1;
		if(!backtrack_stack.push(901)) return -1;
		if(!backtrack_stack.push(849)) return -1;
		if(!backtrack_stack.push(848)) return -1;
		if(!backtrack_stack.push(821)) return -1;
		if(!backtrack_stack.push(820)) return -1;
		if(!backtrack_stack.push(737)) return -1;
		if(!backtrack_stack.push(712)) return -1;
		if(!backtrack_stack.push(674)) return -1;
		if(!backtrack_stack.push(672)) return -1;
		if(!backtrack_stack.push(583)) return -1;
		if(!backtrack_stack.push(575)) return -1;
		if(!backtrack_stack.push(563)) return -1;
		if(!backtrack_stack.push(551)) return -1;
		if(!backtrack_stack.push(549)) return -1;
		if(!backtrack_stack.push(483)) return -1;
		if(!backtrack_stack.push(473)) return -1;
		if(!backtrack_stack.push(470)) return -1;
		if(!backtrack_stack.push(430)) return -1;
		if(!backtrack_stack.push(401)) return -1;
		if(!backtrack_stack.push(399)) return -1;
		if(!backtrack_stack.push(380)) return -1;
		if(!backtrack_stack.push(364)) return -1;
		if(!backtrack_stack.push(363)) return -1;
		if(!backtrack_stack.push(329)) return -1;
		if(!backtrack_stack.push(328)) return -1;
		if(!backtrack_stack.push(327)) return -1;
		if(!backtrack_stack.push(295)) return -1;
		if(!backtrack_stack.push(294)) return -1;
		if(!backtrack_stack.push(216)) return -1;
		if(!backtrack_stack.push(210)) return -1;
		if(!backtrack_stack.push(192)) return -1;
		if(!backtrack_stack.push(180)) return -1;
		if(!backtrack_stack.push(179)) return -1;
		if(!backtrack_stack.push(178)) return -1;
		if(!backtrack_stack.push(154)) return -1;
		if(!backtrack_stack.push(153)) return -1;
		if(!backtrack_stack.push(100)) return -1;
		if(!backtrack_stack.push(76)) return -1;
		if(!backtrack_stack.push(75)) return -1;
		if(!backtrack_stack.push(67)) return -1;
		if(!backtrack_stack.push(63)) return -1;
		if(!backtrack_stack.push(53)) return -1;
		if(!backtrack_stack.push(46)) return -1;
		if(!backtrack_stack.push(19)) return -1;
		goto label_16;
		}
		break;
	case 621: label_621: 
	// using  symbolic fact q7(v_q7_1) with literal 1 in rule 
	// 410: q7(A) :- q7(B), kaon2equal(A,B).
	// resulting in symbolic fact q7(v_q7_1)

		//recursive
		c410_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q7_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2647)) return -1;
		c410_2_fb.open(v_q7_1);
	case 1634:
	// Continuation for label 621
		while (c410_2_fb.fetch()) {
			v_q7_1 = c410_2_fb.c2();
		if(q7_set.insert(v_q7_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1634)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(622)) return -1;
			if(!backtrack_stack.push(621)) return -1;
			goto label_620;
		}
		}
		break;
	case 2647:
	// Label to restore values of label 621
		v_q7_1 = int_value_stack.pop();
		c410_2_fb.pop(&cursor_stack);
		break;
	case 622: label_622: 
	// using  symbolic fact q7(v_q7_1) with literal 1 in rule 
	// 620: hasbody(A,A) :- q7(A), kaon2namedobjects(A).
	// resulting in symbolic fact hasbody(v_hasbody_1,v_hasbody_2)

		if (!kaon2namedobjects_b.contains(v_q7_1)) continue;
	case 1635:
	// Continuation for label 622
			v_hasbody_1 = v_q7_1;
			v_hasbody_2 = v_q7_1;
		if(hasbody_set.insert(v_hasbody_1, v_hasbody_2)) {
			if(!hasbody_bf.insert(v_hasbody_1, v_hasbody_2)) return -1;
			if(!hasbody_fb.insert(v_hasbody_2, v_hasbody_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1635)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(429)) return -1;
			if(!backtrack_stack.push(61)) return -1;
			if(!backtrack_stack.push(60)) return -1;
			if(!backtrack_stack.push(59)) return -1;
			if(!backtrack_stack.push(58)) return -1;
			goto label_54;
		}
		break;
	case 623: label_623: 
	// using  symbolic fact sancerre(v_sancerre_1) with literal 1 in rule 
	// 511: q32(A) :- sancerre(A).
	// resulting in symbolic fact q32(v_q32_1)

		v_q32_1 = v_sancerre_1;
		if(q32_set.insert(v_q32_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(626)) return -1;
		if(!backtrack_stack.push(625)) return -1;
		goto label_624;
		}
		break;
	case 624: label_624: 
	// using  symbolic fact q32(v_q32_1) with literal 1 in rule 
	// 510: wine(A) :- q32(A).
	// resulting in symbolic fact wine(v_wine_1)

		v_wine_1 = v_q32_1;
		if(wine_set.insert(v_wine_1)) {
		if(!wine_f.insert(v_wine_1)) return -1;
		// Labels for next rules
		if(!backtrack_stack.push(1007)) return -1;
		if(!backtrack_stack.push(989)) return -1;
		if(!backtrack_stack.push(954)) return -1;
		if(!backtrack_stack.push(938)) return -1;
		if(!backtrack_stack.push(911)) return -1;
		if(!backtrack_stack.push(903)) return -1;
		if(!backtrack_stack.push(902)) return -1;
		if(!backtrack_stack.push(901)) return -1;
		if(!backtrack_stack.push(849)) return -1;
		if(!backtrack_stack.push(848)) return -1;
		if(!backtrack_stack.push(821)) return -1;
		if(!backtrack_stack.push(820)) return -1;
		if(!backtrack_stack.push(737)) return -1;
		if(!backtrack_stack.push(712)) return -1;
		if(!backtrack_stack.push(674)) return -1;
		if(!backtrack_stack.push(672)) return -1;
		if(!backtrack_stack.push(583)) return -1;
		if(!backtrack_stack.push(575)) return -1;
		if(!backtrack_stack.push(563)) return -1;
		if(!backtrack_stack.push(551)) return -1;
		if(!backtrack_stack.push(549)) return -1;
		if(!backtrack_stack.push(483)) return -1;
		if(!backtrack_stack.push(473)) return -1;
		if(!backtrack_stack.push(470)) return -1;
		if(!backtrack_stack.push(430)) return -1;
		if(!backtrack_stack.push(401)) return -1;
		if(!backtrack_stack.push(399)) return -1;
		if(!backtrack_stack.push(380)) return -1;
		if(!backtrack_stack.push(364)) return -1;
		if(!backtrack_stack.push(363)) return -1;
		if(!backtrack_stack.push(329)) return -1;
		if(!backtrack_stack.push(328)) return -1;
		if(!backtrack_stack.push(327)) return -1;
		if(!backtrack_stack.push(295)) return -1;
		if(!backtrack_stack.push(294)) return -1;
		if(!backtrack_stack.push(216)) return -1;
		if(!backtrack_stack.push(210)) return -1;
		if(!backtrack_stack.push(192)) return -1;
		if(!backtrack_stack.push(180)) return -1;
		if(!backtrack_stack.push(179)) return -1;
		if(!backtrack_stack.push(178)) return -1;
		if(!backtrack_stack.push(154)) return -1;
		if(!backtrack_stack.push(153)) return -1;
		if(!backtrack_stack.push(100)) return -1;
		if(!backtrack_stack.push(76)) return -1;
		if(!backtrack_stack.push(75)) return -1;
		if(!backtrack_stack.push(67)) return -1;
		if(!backtrack_stack.push(63)) return -1;
		if(!backtrack_stack.push(53)) return -1;
		if(!backtrack_stack.push(46)) return -1;
		if(!backtrack_stack.push(19)) return -1;
		goto label_16;
		}
		break;
	case 625: label_625: 
	// using  symbolic fact q32(v_q32_1) with literal 1 in rule 
	// 513: q32(A) :- q32(B), kaon2equal(A,B).
	// resulting in symbolic fact q32(v_q32_1)

		//recursive
		c513_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q32_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2651)) return -1;
		c513_2_fb.open(v_q32_1);
	case 1638:
	// Continuation for label 625
		while (c513_2_fb.fetch()) {
			v_q32_1 = c513_2_fb.c2();
		if(q32_set.insert(v_q32_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1638)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(626)) return -1;
			if(!backtrack_stack.push(625)) return -1;
			goto label_624;
		}
		}
		break;
	case 2651:
	// Label to restore values of label 625
		v_q32_1 = int_value_stack.pop();
		c513_2_fb.pop(&cursor_stack);
		break;
	case 626: label_626: 
	// using  symbolic fact q32(v_q32_1) with literal 1 in rule 
	// 592: hassugar(A,A) :- q32(A), kaon2namedobjects(A).
	// resulting in symbolic fact hassugar(v_hassugar_1,v_hassugar_2)

		if (!kaon2namedobjects_b.contains(v_q32_1)) continue;
	case 1639:
	// Continuation for label 626
			v_hassugar_1 = v_q32_1;
			v_hassugar_2 = v_q32_1;
		if(hassugar_set.insert(v_hassugar_1, v_hassugar_2)) {
			if(!hassugar_bf.insert(v_hassugar_1, v_hassugar_2)) return -1;
			if(!hassugar_fb.insert(v_hassugar_2, v_hassugar_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1639)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(755)) return -1;
			if(!backtrack_stack.push(238)) return -1;
			if(!backtrack_stack.push(214)) return -1;
			if(!backtrack_stack.push(27)) return -1;
			if(!backtrack_stack.push(24)) return -1;
			if(!backtrack_stack.push(23)) return -1;
			if(!backtrack_stack.push(22)) return -1;
			goto label_17;
		}
		break;
	case 627: label_627: 
	// using  symbolic fact sancerre(v_sancerre_1) with literal 1 in rule 
	// 781: q3(A) :- sancerre(A).
	// resulting in symbolic fact q3(v_q3_1)

		v_q3_1 = v_sancerre_1;
		if(q3_set.insert(v_q3_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(143)) return -1;
		goto label_123;
		}
		break;
	case 628: label_628: 
	// using  symbolic fact q40(v_q40_1) with literal 1 in rule 
	// 760: q40(A) :- q40(B), kaon2equal(A,B).
	// resulting in symbolic fact q40(v_q40_1)

		//recursive
		c760_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q40_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2654)) return -1;
		c760_2_fb.open(v_q40_1);
	case 1641:
	// Continuation for label 628
		while (c760_2_fb.fetch()) {
			v_q40_1 = c760_2_fb.c2();
		if(q40_set.insert(v_q40_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1641)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(629)) return -1;
			if(!backtrack_stack.push(628)) return -1;
			goto label_597;
		}
		}
		break;
	case 2654:
	// Label to restore values of label 628
		v_q40_1 = int_value_stack.pop();
		c760_2_fb.pop(&cursor_stack);
		break;
	case 629: label_629: 
	// using  symbolic fact q40(v_q40_1) with literal 2 in rule 
	// 107: q39(A) :- locatedin(A,B), q40(B).
	// resulting in symbolic fact q39(v_q39_1)

		//recursive
		c107_1_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q39_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2655)) return -1;
		c107_1_fb.open(v_q40_1);
	case 1642:
	// Continuation for label 629
		while (c107_1_fb.fetch()) {
			v_q39_1 = c107_1_fb.c2();
		if(q39_set.insert(v_q39_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1642)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(633)) return -1;
			if(!backtrack_stack.push(632)) return -1;
			if(!backtrack_stack.push(630)) return -1;
			goto label_596;
		}
		}
		break;
	case 2655:
	// Label to restore values of label 629
		v_q39_1 = int_value_stack.pop();
		c107_1_fb.pop(&cursor_stack);
		break;
	case 630: label_630: 
	// using  symbolic fact q39(v_q39_1) with literal 1 in rule 
	// 207: locatedin(A,A) :- q39(A), kaon2namedobjects(A).
	// resulting in symbolic fact locatedin(v_locatedin_1,v_locatedin_2)

		//recursive
		if(!int_value_stack.push(v_locatedin_2)) return -1;
		if(!int_value_stack.push(v_locatedin_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2656)) return -1;
		if (!kaon2namedobjects_b.contains(v_q39_1)) continue;
	case 1643:
	// Continuation for label 630
			v_locatedin_1 = v_q39_1;
			v_locatedin_2 = v_q39_1;
		if(locatedin_set.insert(v_locatedin_1, v_locatedin_2)) {
			if(!locatedin_bf.insert(v_locatedin_1, v_locatedin_2)) return -1;
			if(!locatedin_fb.insert(v_locatedin_2, v_locatedin_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1643)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(1005)) return -1;
			if(!backtrack_stack.push(999)) return -1;
			if(!backtrack_stack.push(987)) return -1;
			if(!backtrack_stack.push(981)) return -1;
			if(!backtrack_stack.push(969)) return -1;
			if(!backtrack_stack.push(963)) return -1;
			if(!backtrack_stack.push(952)) return -1;
			if(!backtrack_stack.push(946)) return -1;
			if(!backtrack_stack.push(936)) return -1;
			if(!backtrack_stack.push(930)) return -1;
			if(!backtrack_stack.push(920)) return -1;
			if(!backtrack_stack.push(917)) return -1;
			if(!backtrack_stack.push(898)) return -1;
			if(!backtrack_stack.push(897)) return -1;
			if(!backtrack_stack.push(896)) return -1;
			if(!backtrack_stack.push(893)) return -1;
			if(!backtrack_stack.push(726)) return -1;
			if(!backtrack_stack.push(721)) return -1;
			if(!backtrack_stack.push(694)) return -1;
			if(!backtrack_stack.push(693)) return -1;
			if(!backtrack_stack.push(688)) return -1;
			if(!backtrack_stack.push(687)) return -1;
			if(!backtrack_stack.push(636)) return -1;
			if(!backtrack_stack.push(631)) return -1;
			if(!backtrack_stack.push(592)) return -1;
			if(!backtrack_stack.push(591)) return -1;
			if(!backtrack_stack.push(529)) return -1;
			if(!backtrack_stack.push(528)) return -1;
			if(!backtrack_stack.push(513)) return -1;
			if(!backtrack_stack.push(512)) return -1;
			if(!backtrack_stack.push(511)) return -1;
			if(!backtrack_stack.push(506)) return -1;
			if(!backtrack_stack.push(455)) return -1;
			if(!backtrack_stack.push(450)) return -1;
			if(!backtrack_stack.push(441)) return -1;
			if(!backtrack_stack.push(440)) return -1;
			if(!backtrack_stack.push(439)) return -1;
			if(!backtrack_stack.push(434)) return -1;
			if(!backtrack_stack.push(339)) return -1;
			if(!backtrack_stack.push(338)) return -1;
			if(!backtrack_stack.push(249)) return -1;
			if(!backtrack_stack.push(244)) return -1;
			if(!backtrack_stack.push(52)) return -1;
			if(!backtrack_stack.push(51)) return -1;
			if(!backtrack_stack.push(50)) return -1;
			if(!backtrack_stack.push(49)) return -1;
			if(!backtrack_stack.push(48)) return -1;
			goto label_47;
		}
		break;
	case 2656:
	// Label to restore values of label 630
		v_locatedin_1 = int_value_stack.pop();
		v_locatedin_2 = int_value_stack.pop();
		break;
	case 631: label_631: 
	// using  symbolic fact locatedin(v_locatedin_1,v_locatedin_2) with literal 1 in rule 
	// 107: q39(A) :- locatedin(A,B), q40(B).
	// resulting in symbolic fact q39(v_q39_1)

		//recursive
		if(!int_value_stack.push(v_q39_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2657)) return -1;
		if (!q40_set.contains(v_locatedin_2)) continue;
	case 1644:
	// Continuation for label 631
			v_q39_1 = v_locatedin_1;
		if(q39_set.insert(v_q39_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1644)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(633)) return -1;
			if(!backtrack_stack.push(632)) return -1;
			if(!backtrack_stack.push(630)) return -1;
			goto label_596;
		}
		break;
	case 2657:
	// Label to restore values of label 631
		v_q39_1 = int_value_stack.pop();
		break;
	case 632: label_632: 
	// using  symbolic fact q39(v_q39_1) with literal 1 in rule 
	// 208: q39(A) :- q39(B), kaon2equal(A,B).
	// resulting in symbolic fact q39(v_q39_1)

		//recursive
		c208_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q39_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2658)) return -1;
		c208_2_fb.open(v_q39_1);
	case 1645:
	// Continuation for label 632
		while (c208_2_fb.fetch()) {
			v_q39_1 = c208_2_fb.c2();
		if(q39_set.insert(v_q39_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1645)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(633)) return -1;
			if(!backtrack_stack.push(632)) return -1;
			if(!backtrack_stack.push(630)) return -1;
			goto label_596;
		}
		}
		break;
	case 2658:
	// Label to restore values of label 632
		v_q39_1 = int_value_stack.pop();
		c208_2_fb.pop(&cursor_stack);
		break;
	case 633: label_633: 
	// using  symbolic fact q39(v_q39_1) with literal 1 in rule 
	// 758: ot____nom30(A) :- q39(A), kaon2namedobjects(A).
	// resulting in symbolic fact ot____nom30(v_ot____nom30_1)

		//recursive
		if(!int_value_stack.push(v_ot____nom30_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2659)) return -1;
		if (!kaon2namedobjects_b.contains(v_q39_1)) continue;
	case 1646:
	// Continuation for label 633
			v_ot____nom30_1 = v_q39_1;
		if(ot____nom30_set.insert(v_ot____nom30_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1646)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(635)) return -1;
			goto label_634;
		}
		break;
	case 2659:
	// Label to restore values of label 633
		v_ot____nom30_1 = int_value_stack.pop();
		break;
	case 634: label_634: 
	// using  symbolic fact ot____nom30(v_ot____nom30_1) with literal 1 in rule 
	// 759: ot____nom30(A) :- ot____nom30(B), kaon2equal(A,B).
	// resulting in symbolic fact ot____nom30(v_ot____nom30_1)

		//recursive
		c759_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_ot____nom30_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2660)) return -1;
		c759_2_fb.open(v_ot____nom30_1);
	case 1647:
	// Continuation for label 634
		while (c759_2_fb.fetch()) {
			v_ot____nom30_1 = c759_2_fb.c2();
		if(ot____nom30_set.insert(v_ot____nom30_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1647)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(635)) return -1;
			goto label_634;
		}
		}
		break;
	case 2660:
	// Label to restore values of label 634
		v_ot____nom30_1 = int_value_stack.pop();
		c759_2_fb.pop(&cursor_stack);
		break;
	case 635: label_635: 
	// using  symbolic fact ot____nom30(v_ot____nom30_1) with literal 2 in rule 
	// 756: q40(A) :- locatedin(A,B), ot____nom30(B).
	// resulting in symbolic fact q40(v_q40_1)

		//recursive
		c756_1_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q40_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2661)) return -1;
		c756_1_fb.open(v_ot____nom30_1);
	case 1648:
	// Continuation for label 635
		while (c756_1_fb.fetch()) {
			v_q40_1 = c756_1_fb.c2();
		if(q40_set.insert(v_q40_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1648)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(629)) return -1;
			if(!backtrack_stack.push(628)) return -1;
			goto label_597;
		}
		}
		break;
	case 2661:
	// Label to restore values of label 635
		v_q40_1 = int_value_stack.pop();
		c756_1_fb.pop(&cursor_stack);
		break;
	case 636: label_636: 
	// using  symbolic fact locatedin(v_locatedin_1,v_locatedin_2) with literal 1 in rule 
	// 756: q40(A) :- locatedin(A,B), ot____nom30(B).
	// resulting in symbolic fact q40(v_q40_1)

		//recursive
		if(!int_value_stack.push(v_q40_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2662)) return -1;
		if (!ot____nom30_set.contains(v_locatedin_2)) continue;
	case 1649:
	// Continuation for label 636
			v_q40_1 = v_locatedin_1;
		if(q40_set.insert(v_q40_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1649)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(629)) return -1;
			if(!backtrack_stack.push(628)) return -1;
			goto label_597;
		}
		break;
	case 2662:
	// Label to restore values of label 636
		v_q40_1 = int_value_stack.pop();
		break;
	case 637:
	// applying rule 
	// 110: cotesdor(A) :- cotesdor_aux(A).
	// resulting in symbolic fact cotesdor(v_cotesdor_1)
		c110_1_f.open();
	case 1650:
	// Continuation for label 637
		while (c110_1_f.fetch()) {
			v_cotesdor_1 = c110_1_f.c1();
		if(cotesdor_set.insert(v_cotesdor_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1650)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(645)) return -1;
			if(!backtrack_stack.push(644)) return -1;
			if(!backtrack_stack.push(642)) return -1;
			goto label_638;
		}
		}
		break;
	case 638: label_638: 
	// using  symbolic fact cotesdor(v_cotesdor_1) with literal 1 in rule 
	// 109: q37(A) :- cotesdor(A).
	// resulting in symbolic fact q37(v_q37_1)

		//recursive
		if(!int_value_stack.push(v_q37_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2664)) return -1;
		v_q37_1 = v_cotesdor_1;
		if(q37_set.insert(v_q37_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(651)) return -1;
		if(!backtrack_stack.push(648)) return -1;
		if(!backtrack_stack.push(640)) return -1;
		goto label_639;
		}
		break;
	case 2664:
	// Label to restore values of label 638
		v_q37_1 = int_value_stack.pop();
		break;
	case 639: label_639: 
	// using  symbolic fact q37(v_q37_1) with literal 1 in rule 
	// 108: locatedin(A,A) :- q37(A), kaon2namedobjects(A).
	// resulting in symbolic fact locatedin(v_locatedin_1,v_locatedin_2)

		if (!kaon2namedobjects_b.contains(v_q37_1)) continue;
	case 1652:
	// Continuation for label 639
			v_locatedin_1 = v_q37_1;
			v_locatedin_2 = v_q37_1;
		if(locatedin_set.insert(v_locatedin_1, v_locatedin_2)) {
			if(!locatedin_bf.insert(v_locatedin_1, v_locatedin_2)) return -1;
			if(!locatedin_fb.insert(v_locatedin_2, v_locatedin_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1652)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(1005)) return -1;
			if(!backtrack_stack.push(999)) return -1;
			if(!backtrack_stack.push(987)) return -1;
			if(!backtrack_stack.push(981)) return -1;
			if(!backtrack_stack.push(969)) return -1;
			if(!backtrack_stack.push(963)) return -1;
			if(!backtrack_stack.push(952)) return -1;
			if(!backtrack_stack.push(946)) return -1;
			if(!backtrack_stack.push(936)) return -1;
			if(!backtrack_stack.push(930)) return -1;
			if(!backtrack_stack.push(920)) return -1;
			if(!backtrack_stack.push(917)) return -1;
			if(!backtrack_stack.push(898)) return -1;
			if(!backtrack_stack.push(897)) return -1;
			if(!backtrack_stack.push(896)) return -1;
			if(!backtrack_stack.push(893)) return -1;
			if(!backtrack_stack.push(726)) return -1;
			if(!backtrack_stack.push(721)) return -1;
			if(!backtrack_stack.push(694)) return -1;
			if(!backtrack_stack.push(693)) return -1;
			if(!backtrack_stack.push(688)) return -1;
			if(!backtrack_stack.push(687)) return -1;
			if(!backtrack_stack.push(636)) return -1;
			if(!backtrack_stack.push(631)) return -1;
			if(!backtrack_stack.push(592)) return -1;
			if(!backtrack_stack.push(591)) return -1;
			if(!backtrack_stack.push(529)) return -1;
			if(!backtrack_stack.push(528)) return -1;
			if(!backtrack_stack.push(513)) return -1;
			if(!backtrack_stack.push(512)) return -1;
			if(!backtrack_stack.push(511)) return -1;
			if(!backtrack_stack.push(506)) return -1;
			if(!backtrack_stack.push(455)) return -1;
			if(!backtrack_stack.push(450)) return -1;
			if(!backtrack_stack.push(441)) return -1;
			if(!backtrack_stack.push(440)) return -1;
			if(!backtrack_stack.push(439)) return -1;
			if(!backtrack_stack.push(434)) return -1;
			if(!backtrack_stack.push(339)) return -1;
			if(!backtrack_stack.push(338)) return -1;
			if(!backtrack_stack.push(249)) return -1;
			if(!backtrack_stack.push(244)) return -1;
			if(!backtrack_stack.push(52)) return -1;
			if(!backtrack_stack.push(51)) return -1;
			if(!backtrack_stack.push(50)) return -1;
			if(!backtrack_stack.push(49)) return -1;
			if(!backtrack_stack.push(48)) return -1;
			goto label_47;
		}
		break;
	case 640: label_640: 
	// using  symbolic fact q37(v_q37_1) with literal 1 in rule 
	// 112: q38(A) :- q37(A).
	// resulting in symbolic fact q38(v_q38_1)

		//recursive
		if(!int_value_stack.push(v_q38_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2666)) return -1;
		v_q38_1 = v_q37_1;
		if(q38_set.insert(v_q38_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(647)) return -1;
		if(!backtrack_stack.push(646)) return -1;
		goto label_641;
		}
		break;
	case 2666:
	// Label to restore values of label 640
		v_q38_1 = int_value_stack.pop();
		break;
	case 641: label_641: 
	// using  symbolic fact q38(v_q38_1) with literal 1 in rule 
	// 111: cotesdor(A) :- q38(A), redburgundy(A).
	// resulting in symbolic fact cotesdor(v_cotesdor_1)

		//recursive
		if(!int_value_stack.push(v_cotesdor_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2667)) return -1;
		if (!redburgundy_set.contains(v_q38_1)) continue;
	case 1654:
	// Continuation for label 641
			v_cotesdor_1 = v_q38_1;
		if(cotesdor_set.insert(v_cotesdor_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1654)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(645)) return -1;
			if(!backtrack_stack.push(644)) return -1;
			if(!backtrack_stack.push(642)) return -1;
			goto label_638;
		}
		break;
	case 2667:
	// Label to restore values of label 641
		v_cotesdor_1 = int_value_stack.pop();
		break;
	case 642: label_642: 
	// using  symbolic fact cotesdor(v_cotesdor_1) with literal 1 in rule 
	// 189: redburgundy(A) :- cotesdor(A).
	// resulting in symbolic fact redburgundy(v_redburgundy_1)

		//recursive
		if(!int_value_stack.push(v_redburgundy_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2668)) return -1;
		v_redburgundy_1 = v_cotesdor_1;
		if(redburgundy_set.insert(v_redburgundy_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(643)) return -1;
		if(!backtrack_stack.push(405)) return -1;
		if(!backtrack_stack.push(386)) return -1;
		if(!backtrack_stack.push(385)) return -1;
		if(!backtrack_stack.push(383)) return -1;
		goto label_378;
		}
		break;
	case 2668:
	// Label to restore values of label 642
		v_redburgundy_1 = int_value_stack.pop();
		break;
	case 643: label_643: 
	// using  symbolic fact redburgundy(v_redburgundy_1) with literal 2 in rule 
	// 111: cotesdor(A) :- q38(A), redburgundy(A).
	// resulting in symbolic fact cotesdor(v_cotesdor_1)

		//recursive
		if(!int_value_stack.push(v_cotesdor_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2669)) return -1;
		if (!q38_set.contains(v_redburgundy_1)) continue;
	case 1656:
	// Continuation for label 643
			v_cotesdor_1 = v_redburgundy_1;
		if(cotesdor_set.insert(v_cotesdor_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1656)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(645)) return -1;
			if(!backtrack_stack.push(644)) return -1;
			if(!backtrack_stack.push(642)) return -1;
			goto label_638;
		}
		break;
	case 2669:
	// Label to restore values of label 643
		v_cotesdor_1 = int_value_stack.pop();
		break;
	case 644: label_644: 
	// using  symbolic fact cotesdor(v_cotesdor_1) with literal 1 in rule 
	// 190: cotesdor(A) :- cotesdor(B), kaon2equal(A,B).
	// resulting in symbolic fact cotesdor(v_cotesdor_1)

		//recursive
		c190_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_cotesdor_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2670)) return -1;
		c190_2_fb.open(v_cotesdor_1);
	case 1657:
	// Continuation for label 644
		while (c190_2_fb.fetch()) {
			v_cotesdor_1 = c190_2_fb.c2();
		if(cotesdor_set.insert(v_cotesdor_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1657)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(645)) return -1;
			if(!backtrack_stack.push(644)) return -1;
			if(!backtrack_stack.push(642)) return -1;
			goto label_638;
		}
		}
		break;
	case 2670:
	// Label to restore values of label 644
		v_cotesdor_1 = int_value_stack.pop();
		c190_2_fb.pop(&cursor_stack);
		break;
	case 645: label_645: 
	// using  symbolic fact cotesdor(v_cotesdor_1) with literal 1 in rule 
	// 491: q4(A) :- cotesdor(A).
	// resulting in symbolic fact q4(v_q4_1)

		v_q4_1 = v_cotesdor_1;
		if(q4_set.insert(v_q4_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(90)) return -1;
		if(!backtrack_stack.push(89)) return -1;
		goto label_88;
		}
		break;
	case 646: label_646: 
	// using  symbolic fact q38(v_q38_1) with literal 1 in rule 
	// 755: q38(A) :- q38(B), kaon2equal(A,B).
	// resulting in symbolic fact q38(v_q38_1)

		//recursive
		c755_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q38_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2672)) return -1;
		c755_2_fb.open(v_q38_1);
	case 1659:
	// Continuation for label 646
		while (c755_2_fb.fetch()) {
			v_q38_1 = c755_2_fb.c2();
		if(q38_set.insert(v_q38_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1659)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(647)) return -1;
			if(!backtrack_stack.push(646)) return -1;
			goto label_641;
		}
		}
		break;
	case 2672:
	// Label to restore values of label 646
		v_q38_1 = int_value_stack.pop();
		c755_2_fb.pop(&cursor_stack);
		break;
	case 647: label_647: 
	// using  symbolic fact q38(v_q38_1) with literal 2 in rule 
	// 113: q37(A) :- locatedin(A,B), q38(B).
	// resulting in symbolic fact q37(v_q37_1)

		//recursive
		c113_1_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q37_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2673)) return -1;
		c113_1_fb.open(v_q38_1);
	case 1660:
	// Continuation for label 647
		while (c113_1_fb.fetch()) {
			v_q37_1 = c113_1_fb.c2();
		if(q37_set.insert(v_q37_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1660)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(651)) return -1;
			if(!backtrack_stack.push(648)) return -1;
			if(!backtrack_stack.push(640)) return -1;
			goto label_639;
		}
		}
		break;
	case 2673:
	// Label to restore values of label 647
		v_q37_1 = int_value_stack.pop();
		c113_1_fb.pop(&cursor_stack);
		break;
	case 648: label_648: 
	// using  symbolic fact q37(v_q37_1) with literal 1 in rule 
	// 752: ot____nom42(A) :- q37(A), kaon2namedobjects(A).
	// resulting in symbolic fact ot____nom42(v_ot____nom42_1)

		//recursive
		if(!int_value_stack.push(v_ot____nom42_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2674)) return -1;
		if (!kaon2namedobjects_b.contains(v_q37_1)) continue;
	case 1661:
	// Continuation for label 648
			v_ot____nom42_1 = v_q37_1;
		if(ot____nom42_set.insert(v_ot____nom42_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1661)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(650)) return -1;
			goto label_649;
		}
		break;
	case 2674:
	// Label to restore values of label 648
		v_ot____nom42_1 = int_value_stack.pop();
		break;
	case 649: label_649: 
	// using  symbolic fact ot____nom42(v_ot____nom42_1) with literal 1 in rule 
	// 754: ot____nom42(A) :- ot____nom42(B), kaon2equal(A,B).
	// resulting in symbolic fact ot____nom42(v_ot____nom42_1)

		//recursive
		c754_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_ot____nom42_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2675)) return -1;
		c754_2_fb.open(v_ot____nom42_1);
	case 1662:
	// Continuation for label 649
		while (c754_2_fb.fetch()) {
			v_ot____nom42_1 = c754_2_fb.c2();
		if(ot____nom42_set.insert(v_ot____nom42_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1662)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(650)) return -1;
			goto label_649;
		}
		}
		break;
	case 2675:
	// Label to restore values of label 649
		v_ot____nom42_1 = int_value_stack.pop();
		c754_2_fb.pop(&cursor_stack);
		break;
	case 650: label_650: 
	// using  symbolic fact ot____nom42(v_ot____nom42_1) with literal 2 in rule 
	// 750: q38(A) :- locatedin(A,B), ot____nom42(B).
	// resulting in symbolic fact q38(v_q38_1)

		//recursive
		c750_1_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q38_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2676)) return -1;
		c750_1_fb.open(v_ot____nom42_1);
	case 1663:
	// Continuation for label 650
		while (c750_1_fb.fetch()) {
			v_q38_1 = c750_1_fb.c2();
		if(q38_set.insert(v_q38_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1663)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(647)) return -1;
			if(!backtrack_stack.push(646)) return -1;
			goto label_641;
		}
		}
		break;
	case 2676:
	// Label to restore values of label 650
		v_q38_1 = int_value_stack.pop();
		c750_1_fb.pop(&cursor_stack);
		break;
	case 651: label_651: 
	// using  symbolic fact q37(v_q37_1) with literal 1 in rule 
	// 753: q37(A) :- q37(B), kaon2equal(A,B).
	// resulting in symbolic fact q37(v_q37_1)

		//recursive
		c753_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q37_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2677)) return -1;
		c753_2_fb.open(v_q37_1);
	case 1664:
	// Continuation for label 651
		while (c753_2_fb.fetch()) {
			v_q37_1 = c753_2_fb.c2();
		if(q37_set.insert(v_q37_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1664)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(651)) return -1;
			if(!backtrack_stack.push(648)) return -1;
			if(!backtrack_stack.push(640)) return -1;
			goto label_639;
		}
		}
		break;
	case 2677:
	// Label to restore values of label 651
		v_q37_1 = int_value_stack.pop();
		c753_2_fb.pop(&cursor_stack);
		break;
	case 652:
	// applying rule 
	// 128: pauillac(A) :- pauillac_aux(A).
	// resulting in symbolic fact pauillac(v_pauillac_1)
		c128_1_f.open();
	case 1665:
	// Continuation for label 652
		while (c128_1_f.fetch()) {
			v_pauillac_1 = c128_1_f.c1();
		if(pauillac_set.insert(v_pauillac_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1665)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(683)) return -1;
			if(!backtrack_stack.push(682)) return -1;
			if(!backtrack_stack.push(678)) return -1;
			if(!backtrack_stack.push(658)) return -1;
			if(!backtrack_stack.push(657)) return -1;
			if(!backtrack_stack.push(654)) return -1;
			goto label_653;
		}
		}
		break;
	case 653: label_653: 
	// using  symbolic fact pauillac(v_pauillac_1) with literal 1 in rule 
	// 127: medoc(A) :- pauillac(A).
	// resulting in symbolic fact medoc(v_medoc_1)

		v_medoc_1 = v_pauillac_1;
		if(medoc_set.insert(v_medoc_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(534)) return -1;
		if(!backtrack_stack.push(533)) return -1;
		if(!backtrack_stack.push(532)) return -1;
		if(!backtrack_stack.push(531)) return -1;
		if(!backtrack_stack.push(525)) return -1;
		goto label_516;
		}
		break;
	case 654: label_654: 
	// using  symbolic fact pauillac(v_pauillac_1) with literal 1 in rule 
	// 131: q42(A) :- pauillac(A).
	// resulting in symbolic fact q42(v_q42_1)

		//recursive
		if(!int_value_stack.push(v_q42_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2680)) return -1;
		v_q42_1 = v_pauillac_1;
		if(q42_set.insert(v_q42_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(690)) return -1;
		if(!backtrack_stack.push(689)) return -1;
		if(!backtrack_stack.push(686)) return -1;
		goto label_655;
		}
		break;
	case 2680:
	// Label to restore values of label 654
		v_q42_1 = int_value_stack.pop();
		break;
	case 655: label_655: 
	// using  symbolic fact q42(v_q42_1) with literal 1 in rule 
	// 130: q43(A) :- q42(A).
	// resulting in symbolic fact q43(v_q43_1)

		//recursive
		if(!int_value_stack.push(v_q43_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2681)) return -1;
		v_q43_1 = v_q42_1;
		if(q43_set.insert(v_q43_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(685)) return -1;
		if(!backtrack_stack.push(684)) return -1;
		goto label_656;
		}
		break;
	case 2681:
	// Label to restore values of label 655
		v_q43_1 = int_value_stack.pop();
		break;
	case 656: label_656: 
	// using  symbolic fact q43(v_q43_1) with literal 1 in rule 
	// 129: pauillac(A) :- q43(A), medoc(A).
	// resulting in symbolic fact pauillac(v_pauillac_1)

		//recursive
		if(!int_value_stack.push(v_pauillac_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2682)) return -1;
		if (!medoc_set.contains(v_q43_1)) continue;
	case 1669:
	// Continuation for label 656
			v_pauillac_1 = v_q43_1;
		if(pauillac_set.insert(v_pauillac_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1669)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(683)) return -1;
			if(!backtrack_stack.push(682)) return -1;
			if(!backtrack_stack.push(678)) return -1;
			if(!backtrack_stack.push(658)) return -1;
			if(!backtrack_stack.push(657)) return -1;
			if(!backtrack_stack.push(654)) return -1;
			goto label_653;
		}
		break;
	case 2682:
	// Label to restore values of label 656
		v_pauillac_1 = int_value_stack.pop();
		break;
	case 657: label_657: 
	// using  symbolic fact pauillac(v_pauillac_1) with literal 1 in rule 
	// 132: pauillac(A) :- pauillac(B), kaon2equal(A,B).
	// resulting in symbolic fact pauillac(v_pauillac_1)

		//recursive
		c132_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_pauillac_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2683)) return -1;
		c132_2_fb.open(v_pauillac_1);
	case 1670:
	// Continuation for label 657
		while (c132_2_fb.fetch()) {
			v_pauillac_1 = c132_2_fb.c2();
		if(pauillac_set.insert(v_pauillac_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1670)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(683)) return -1;
			if(!backtrack_stack.push(682)) return -1;
			if(!backtrack_stack.push(678)) return -1;
			if(!backtrack_stack.push(658)) return -1;
			if(!backtrack_stack.push(657)) return -1;
			if(!backtrack_stack.push(654)) return -1;
			goto label_653;
		}
		}
		break;
	case 2683:
	// Label to restore values of label 657
		v_pauillac_1 = int_value_stack.pop();
		c132_2_fb.pop(&cursor_stack);
		break;
	case 658: label_658: 
	// using  symbolic fact pauillac(v_pauillac_1) with literal 1 in rule 
	// 246: q69(A) :- pauillac(A).
	// resulting in symbolic fact q69(v_q69_1)

		v_q69_1 = v_pauillac_1;
		if(q69_set.insert(v_q69_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(666)) return -1;
		if(!backtrack_stack.push(665)) return -1;
		if(!backtrack_stack.push(664)) return -1;
		if(!backtrack_stack.push(663)) return -1;
		if(!backtrack_stack.push(662)) return -1;
		goto label_659;
		}
		break;
	case 659: label_659: 
	// using  symbolic fact q69(v_q69_1) with literal 1 in rule 
	// 245: cabernetsauvignon(A) :- q69(A).
	// resulting in symbolic fact cabernetsauvignon(v_cabernetsauvignon_1)

		//recursive
		if(!int_value_stack.push(v_cabernetsauvignon_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2685)) return -1;
		v_cabernetsauvignon_1 = v_q69_1;
		if(cabernetsauvignon_set.insert(v_cabernetsauvignon_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(673)) return -1;
		if(!backtrack_stack.push(671)) return -1;
		if(!backtrack_stack.push(670)) return -1;
		if(!backtrack_stack.push(669)) return -1;
		if(!backtrack_stack.push(661)) return -1;
		goto label_660;
		}
		break;
	case 2685:
	// Label to restore values of label 659
		v_cabernetsauvignon_1 = int_value_stack.pop();
		break;
	case 660: label_660: 
	// using  symbolic fact cabernetsauvignon(v_cabernetsauvignon_1) with literal 1 in rule 
	// 243: q31(A) :- cabernetsauvignon(A).
	// resulting in symbolic fact q31(v_q31_1)

		v_q31_1 = v_cabernetsauvignon_1;
		if(q31_set.insert(v_q31_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(221)) return -1;
		if(!backtrack_stack.push(220)) return -1;
		if(!backtrack_stack.push(215)) return -1;
		if(!backtrack_stack.push(213)) return -1;
		goto label_201;
		}
		break;
	case 661: label_661: 
	// using  symbolic fact cabernetsauvignon(v_cabernetsauvignon_1) with literal 1 in rule 
	// 247: q69(A) :- cabernetsauvignon(A).
	// resulting in symbolic fact q69(v_q69_1)

		//recursive
		if(!int_value_stack.push(v_q69_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2687)) return -1;
		v_q69_1 = v_cabernetsauvignon_1;
		if(q69_set.insert(v_q69_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(666)) return -1;
		if(!backtrack_stack.push(665)) return -1;
		if(!backtrack_stack.push(664)) return -1;
		if(!backtrack_stack.push(663)) return -1;
		if(!backtrack_stack.push(662)) return -1;
		goto label_659;
		}
		break;
	case 2687:
	// Label to restore values of label 661
		v_q69_1 = int_value_stack.pop();
		break;
	case 662: label_662: 
	// using  symbolic fact q69(v_q69_1) with literal 1 in rule 
	// 463: wine(A) :- q69(A).
	// resulting in symbolic fact wine(v_wine_1)

		v_wine_1 = v_q69_1;
		if(wine_set.insert(v_wine_1)) {
		if(!wine_f.insert(v_wine_1)) return -1;
		// Labels for next rules
		if(!backtrack_stack.push(1007)) return -1;
		if(!backtrack_stack.push(989)) return -1;
		if(!backtrack_stack.push(954)) return -1;
		if(!backtrack_stack.push(938)) return -1;
		if(!backtrack_stack.push(911)) return -1;
		if(!backtrack_stack.push(903)) return -1;
		if(!backtrack_stack.push(902)) return -1;
		if(!backtrack_stack.push(901)) return -1;
		if(!backtrack_stack.push(849)) return -1;
		if(!backtrack_stack.push(848)) return -1;
		if(!backtrack_stack.push(821)) return -1;
		if(!backtrack_stack.push(820)) return -1;
		if(!backtrack_stack.push(737)) return -1;
		if(!backtrack_stack.push(712)) return -1;
		if(!backtrack_stack.push(674)) return -1;
		if(!backtrack_stack.push(672)) return -1;
		if(!backtrack_stack.push(583)) return -1;
		if(!backtrack_stack.push(575)) return -1;
		if(!backtrack_stack.push(563)) return -1;
		if(!backtrack_stack.push(551)) return -1;
		if(!backtrack_stack.push(549)) return -1;
		if(!backtrack_stack.push(483)) return -1;
		if(!backtrack_stack.push(473)) return -1;
		if(!backtrack_stack.push(470)) return -1;
		if(!backtrack_stack.push(430)) return -1;
		if(!backtrack_stack.push(401)) return -1;
		if(!backtrack_stack.push(399)) return -1;
		if(!backtrack_stack.push(380)) return -1;
		if(!backtrack_stack.push(364)) return -1;
		if(!backtrack_stack.push(363)) return -1;
		if(!backtrack_stack.push(329)) return -1;
		if(!backtrack_stack.push(328)) return -1;
		if(!backtrack_stack.push(327)) return -1;
		if(!backtrack_stack.push(295)) return -1;
		if(!backtrack_stack.push(294)) return -1;
		if(!backtrack_stack.push(216)) return -1;
		if(!backtrack_stack.push(210)) return -1;
		if(!backtrack_stack.push(192)) return -1;
		if(!backtrack_stack.push(180)) return -1;
		if(!backtrack_stack.push(179)) return -1;
		if(!backtrack_stack.push(178)) return -1;
		if(!backtrack_stack.push(154)) return -1;
		if(!backtrack_stack.push(153)) return -1;
		if(!backtrack_stack.push(100)) return -1;
		if(!backtrack_stack.push(76)) return -1;
		if(!backtrack_stack.push(75)) return -1;
		if(!backtrack_stack.push(67)) return -1;
		if(!backtrack_stack.push(63)) return -1;
		if(!backtrack_stack.push(53)) return -1;
		if(!backtrack_stack.push(46)) return -1;
		if(!backtrack_stack.push(19)) return -1;
		goto label_16;
		}
		break;
	case 663: label_663: 
	// using  symbolic fact q69(v_q69_1) with literal 1 in rule 
	// 465: q69(A) :- q69(B), kaon2equal(A,B).
	// resulting in symbolic fact q69(v_q69_1)

		//recursive
		c465_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q69_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2689)) return -1;
		c465_2_fb.open(v_q69_1);
	case 1676:
	// Continuation for label 663
		while (c465_2_fb.fetch()) {
			v_q69_1 = c465_2_fb.c2();
		if(q69_set.insert(v_q69_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1676)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(666)) return -1;
			if(!backtrack_stack.push(665)) return -1;
			if(!backtrack_stack.push(664)) return -1;
			if(!backtrack_stack.push(663)) return -1;
			if(!backtrack_stack.push(662)) return -1;
			goto label_659;
		}
		}
		break;
	case 2689:
	// Label to restore values of label 663
		v_q69_1 = int_value_stack.pop();
		c465_2_fb.pop(&cursor_stack);
		break;
	case 664: label_664: 
	// using  symbolic fact q69(v_q69_1) with literal 1 in rule 
	// 503: q5(A) :- q69(A).
	// resulting in symbolic fact q5(v_q5_1)

		v_q5_1 = v_q69_1;
		if(q5_set.insert(v_q5_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(136)) return -1;
		if(!backtrack_stack.push(135)) return -1;
		goto label_131;
		}
		break;
	case 665: label_665: 
	// using  symbolic fact q69(v_q69_1) with literal 1 in rule 
	// 526: madefromgrape(A,A) :- q69(A), kaon2namedobjects(A).
	// resulting in symbolic fact madefromgrape(v_madefromgrape_1,v_madefromgrape_2)

		//recursive
		if(!int_value_stack.push(v_madefromgrape_2)) return -1;
		if(!int_value_stack.push(v_madefromgrape_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2691)) return -1;
		if (!kaon2namedobjects_b.contains(v_q69_1)) continue;
	case 1678:
	// Continuation for label 665
			v_madefromgrape_1 = v_q69_1;
			v_madefromgrape_2 = v_q69_1;
		if(madefromgrape_set.insert(v_madefromgrape_1, v_madefromgrape_2)) {
			if(!madefromgrape_bf.insert(v_madefromgrape_1, v_madefromgrape_2)) return -1;
			if(!madefromgrape_fb.insert(v_madefromgrape_2, v_madefromgrape_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1678)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(860)) return -1;
			if(!backtrack_stack.push(859)) return -1;
			if(!backtrack_stack.push(845)) return -1;
			if(!backtrack_stack.push(844)) return -1;
			if(!backtrack_stack.push(829)) return -1;
			if(!backtrack_stack.push(828)) return -1;
			if(!backtrack_stack.push(813)) return -1;
			if(!backtrack_stack.push(812)) return -1;
			if(!backtrack_stack.push(677)) return -1;
			if(!backtrack_stack.push(676)) return -1;
			if(!backtrack_stack.push(617)) return -1;
			if(!backtrack_stack.push(616)) return -1;
			if(!backtrack_stack.push(609)) return -1;
			if(!backtrack_stack.push(554)) return -1;
			if(!backtrack_stack.push(553)) return -1;
			if(!backtrack_stack.push(476)) return -1;
			if(!backtrack_stack.push(475)) return -1;
			if(!backtrack_stack.push(404)) return -1;
			if(!backtrack_stack.push(403)) return -1;
			if(!backtrack_stack.push(362)) return -1;
			if(!backtrack_stack.push(361)) return -1;
			if(!backtrack_stack.push(343)) return -1;
			if(!backtrack_stack.push(342)) return -1;
			if(!backtrack_stack.push(300)) return -1;
			if(!backtrack_stack.push(170)) return -1;
			if(!backtrack_stack.push(169)) return -1;
			if(!backtrack_stack.push(151)) return -1;
			if(!backtrack_stack.push(150)) return -1;
			if(!backtrack_stack.push(144)) return -1;
			if(!backtrack_stack.push(132)) return -1;
			if(!backtrack_stack.push(71)) return -1;
			if(!backtrack_stack.push(70)) return -1;
			if(!backtrack_stack.push(66)) return -1;
			goto label_64;
		}
		break;
	case 2691:
	// Label to restore values of label 665
		v_madefromgrape_1 = int_value_stack.pop();
		v_madefromgrape_2 = int_value_stack.pop();
		break;
	case 666: label_666: 
	// using  symbolic fact q69(v_q69_1) with literal 1 in rule 
	// 580: ot____nom29(A) :- q69(A), kaon2namedobjects(A).
	// resulting in symbolic fact ot____nom29(v_ot____nom29_1)

		//recursive
		if(!int_value_stack.push(v_ot____nom29_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2692)) return -1;
		if (!kaon2namedobjects_b.contains(v_q69_1)) continue;
	case 1679:
	// Continuation for label 666
			v_ot____nom29_1 = v_q69_1;
		if(ot____nom29_set.insert(v_ot____nom29_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1679)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(675)) return -1;
			if(!backtrack_stack.push(668)) return -1;
			goto label_667;
		}
		break;
	case 2692:
	// Label to restore values of label 666
		v_ot____nom29_1 = int_value_stack.pop();
		break;
	case 667: label_667: 
	// using  symbolic fact ot____nom29(v_ot____nom29_1) with literal 1 in rule 
	// 581: ot____nom29(A) :- ot____nom29(B), kaon2equal(A,B).
	// resulting in symbolic fact ot____nom29(v_ot____nom29_1)

		//recursive
		c581_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_ot____nom29_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2693)) return -1;
		c581_2_fb.open(v_ot____nom29_1);
	case 1680:
	// Continuation for label 667
		while (c581_2_fb.fetch()) {
			v_ot____nom29_1 = c581_2_fb.c2();
		if(ot____nom29_set.insert(v_ot____nom29_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1680)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(675)) return -1;
			if(!backtrack_stack.push(668)) return -1;
			goto label_667;
		}
		}
		break;
	case 2693:
	// Label to restore values of label 667
		v_ot____nom29_1 = int_value_stack.pop();
		c581_2_fb.pop(&cursor_stack);
		break;
	case 668: label_668: 
	// using  symbolic fact ot____nom29(v_ot____nom29_1) with literal 3 in rule 
	// 249: cabernetsauvignon(A) :- wine(A), madefromgrape(A,B), ot____nom29(B).
	// resulting in symbolic fact cabernetsauvignon(v_cabernetsauvignon_1)

		//recursive
		c249_1_f.push(&cursor_stack);
		if(!int_value_stack.push(v_cabernetsauvignon_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2694)) return -1;
		c249_1_f.open();
		while (c249_1_f.fetch()) {
			if (!madefromgrape_set.contains(c249_1_f.c1(), v_ot____nom29_1)) continue;
	case 1681:
	// Continuation for label 668
				v_cabernetsauvignon_1 = c249_1_f.c1();
		if(cabernetsauvignon_set.insert(v_cabernetsauvignon_1)) {
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1681)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(673)) return -1;
				if(!backtrack_stack.push(671)) return -1;
				if(!backtrack_stack.push(670)) return -1;
				if(!backtrack_stack.push(669)) return -1;
				if(!backtrack_stack.push(661)) return -1;
				goto label_660;
		}
		}
		break;
	case 2694:
	// Label to restore values of label 668
		v_cabernetsauvignon_1 = int_value_stack.pop();
		c249_1_f.pop(&cursor_stack);
		break;
	case 669: label_669: 
	// using  symbolic fact cabernetsauvignon(v_cabernetsauvignon_1) with literal 1 in rule 
	// 248: cabernetsauvignon(A) :- cabernetsauvignon(B), kaon2equal(A,B).
	// resulting in symbolic fact cabernetsauvignon(v_cabernetsauvignon_1)

		//recursive
		c248_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_cabernetsauvignon_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2695)) return -1;
		c248_2_fb.open(v_cabernetsauvignon_1);
	case 1682:
	// Continuation for label 669
		while (c248_2_fb.fetch()) {
			v_cabernetsauvignon_1 = c248_2_fb.c2();
		if(cabernetsauvignon_set.insert(v_cabernetsauvignon_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1682)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(673)) return -1;
			if(!backtrack_stack.push(671)) return -1;
			if(!backtrack_stack.push(670)) return -1;
			if(!backtrack_stack.push(669)) return -1;
			if(!backtrack_stack.push(661)) return -1;
			goto label_660;
		}
		}
		break;
	case 2695:
	// Label to restore values of label 669
		v_cabernetsauvignon_1 = int_value_stack.pop();
		c248_2_fb.pop(&cursor_stack);
		break;
	case 670: label_670: 
	// using  symbolic fact cabernetsauvignon(v_cabernetsauvignon_1) with literal 1 in rule 
	// 442: q0(A) :- cabernetsauvignon(A).
	// resulting in symbolic fact q0(v_q0_1)

		v_q0_1 = v_cabernetsauvignon_1;
		if(q0_set.insert(v_q0_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(198)) return -1;
		if(!backtrack_stack.push(194)) return -1;
		if(!backtrack_stack.push(189)) return -1;
		if(!backtrack_stack.push(188)) return -1;
		goto label_187;
		}
		break;
	case 671: label_671: 
	// using  symbolic fact cabernetsauvignon(v_cabernetsauvignon_1) with literal 1 in rule 
	// 477: wine(A) :- cabernetsauvignon(A).
	// resulting in symbolic fact wine(v_wine_1)

		//recursive
		if(!int_value_stack.push(v_wine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2697)) return -1;
		v_wine_1 = v_cabernetsauvignon_1;
		if(wine_set.insert(v_wine_1)) {
		if(!wine_f.insert(v_wine_1)) return -1;
		// Labels for next rules
		if(!backtrack_stack.push(1007)) return -1;
		if(!backtrack_stack.push(989)) return -1;
		if(!backtrack_stack.push(954)) return -1;
		if(!backtrack_stack.push(938)) return -1;
		if(!backtrack_stack.push(911)) return -1;
		if(!backtrack_stack.push(903)) return -1;
		if(!backtrack_stack.push(902)) return -1;
		if(!backtrack_stack.push(901)) return -1;
		if(!backtrack_stack.push(849)) return -1;
		if(!backtrack_stack.push(848)) return -1;
		if(!backtrack_stack.push(821)) return -1;
		if(!backtrack_stack.push(820)) return -1;
		if(!backtrack_stack.push(737)) return -1;
		if(!backtrack_stack.push(712)) return -1;
		if(!backtrack_stack.push(674)) return -1;
		if(!backtrack_stack.push(672)) return -1;
		if(!backtrack_stack.push(583)) return -1;
		if(!backtrack_stack.push(575)) return -1;
		if(!backtrack_stack.push(563)) return -1;
		if(!backtrack_stack.push(551)) return -1;
		if(!backtrack_stack.push(549)) return -1;
		if(!backtrack_stack.push(483)) return -1;
		if(!backtrack_stack.push(473)) return -1;
		if(!backtrack_stack.push(470)) return -1;
		if(!backtrack_stack.push(430)) return -1;
		if(!backtrack_stack.push(401)) return -1;
		if(!backtrack_stack.push(399)) return -1;
		if(!backtrack_stack.push(380)) return -1;
		if(!backtrack_stack.push(364)) return -1;
		if(!backtrack_stack.push(363)) return -1;
		if(!backtrack_stack.push(329)) return -1;
		if(!backtrack_stack.push(328)) return -1;
		if(!backtrack_stack.push(327)) return -1;
		if(!backtrack_stack.push(295)) return -1;
		if(!backtrack_stack.push(294)) return -1;
		if(!backtrack_stack.push(216)) return -1;
		if(!backtrack_stack.push(210)) return -1;
		if(!backtrack_stack.push(192)) return -1;
		if(!backtrack_stack.push(180)) return -1;
		if(!backtrack_stack.push(179)) return -1;
		if(!backtrack_stack.push(178)) return -1;
		if(!backtrack_stack.push(154)) return -1;
		if(!backtrack_stack.push(153)) return -1;
		if(!backtrack_stack.push(100)) return -1;
		if(!backtrack_stack.push(76)) return -1;
		if(!backtrack_stack.push(75)) return -1;
		if(!backtrack_stack.push(67)) return -1;
		if(!backtrack_stack.push(63)) return -1;
		if(!backtrack_stack.push(53)) return -1;
		if(!backtrack_stack.push(46)) return -1;
		if(!backtrack_stack.push(19)) return -1;
		goto label_16;
		}
		break;
	case 2697:
	// Label to restore values of label 671
		v_wine_1 = int_value_stack.pop();
		break;
	case 672: label_672: 
	// using  symbolic fact wine(v_wine_1) with literal 1 in rule 
	// 249: cabernetsauvignon(A) :- wine(A), madefromgrape(A,B), ot____nom29(B).
	// resulting in symbolic fact cabernetsauvignon(v_cabernetsauvignon_1)

		//recursive
		c249_2_bf.push(&cursor_stack);
		if(!int_value_stack.push(v_cabernetsauvignon_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2698)) return -1;
		c249_2_bf.open(v_wine_1);
		while (c249_2_bf.fetch()) {
			if (!ot____nom29_set.contains(c249_2_bf.c2())) continue;
	case 1685:
	// Continuation for label 672
				v_cabernetsauvignon_1 = v_wine_1;
		if(cabernetsauvignon_set.insert(v_cabernetsauvignon_1)) {
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1685)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(673)) return -1;
				if(!backtrack_stack.push(671)) return -1;
				if(!backtrack_stack.push(670)) return -1;
				if(!backtrack_stack.push(669)) return -1;
				if(!backtrack_stack.push(661)) return -1;
				goto label_660;
		}
		}
		break;
	case 2698:
	// Label to restore values of label 672
		v_cabernetsauvignon_1 = int_value_stack.pop();
		c249_2_bf.pop(&cursor_stack);
		break;
	case 673: label_673: 
	// using  symbolic fact cabernetsauvignon(v_cabernetsauvignon_1) with literal 1 in rule 
	// 774: q3(A) :- cabernetsauvignon(A).
	// resulting in symbolic fact q3(v_q3_1)

		v_q3_1 = v_cabernetsauvignon_1;
		if(q3_set.insert(v_q3_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(143)) return -1;
		goto label_123;
		}
		break;
	case 674: label_674: 
	// using  symbolic fact wine(v_wine_1) with literal 1 in rule 
	// 578: q5(A) :- wine(A), madefromgrape(A,B), ot____nom29(B).
	// resulting in symbolic fact q5(v_q5_1)

		c578_2_bf.open(v_wine_1);
		while (c578_2_bf.fetch()) {
			if (!ot____nom29_set.contains(c578_2_bf.c2())) continue;
	case 1687:
	// Continuation for label 674
				v_q5_1 = v_wine_1;
		if(q5_set.insert(v_q5_1)) {
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1687)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(136)) return -1;
				if(!backtrack_stack.push(135)) return -1;
				goto label_131;
		}
		}
		break;
	case 675: label_675: 
	// using  symbolic fact ot____nom29(v_ot____nom29_1) with literal 3 in rule 
	// 578: q5(A) :- wine(A), madefromgrape(A,B), ot____nom29(B).
	// resulting in symbolic fact q5(v_q5_1)

		c578_1_f.open();
		while (c578_1_f.fetch()) {
			if (!madefromgrape_set.contains(c578_1_f.c1(), v_ot____nom29_1)) continue;
	case 1688:
	// Continuation for label 675
				v_q5_1 = c578_1_f.c1();
		if(q5_set.insert(v_q5_1)) {
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1688)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(136)) return -1;
				if(!backtrack_stack.push(135)) return -1;
				goto label_131;
		}
		}
		break;
	case 676: label_676: 
	// using  symbolic fact madefromgrape(v_madefromgrape_1,v_madefromgrape_2) with literal 2 in rule 
	// 249: cabernetsauvignon(A) :- wine(A), madefromgrape(A,B), ot____nom29(B).
	// resulting in symbolic fact cabernetsauvignon(v_cabernetsauvignon_1)

		//recursive
		if(!int_value_stack.push(v_cabernetsauvignon_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2702)) return -1;
		if (!wine_set.contains(v_madefromgrape_1)) continue;
			if (!ot____nom29_set.contains(v_madefromgrape_2)) continue;
	case 1689:
	// Continuation for label 676
				v_cabernetsauvignon_1 = v_madefromgrape_1;
		if(cabernetsauvignon_set.insert(v_cabernetsauvignon_1)) {
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1689)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(673)) return -1;
				if(!backtrack_stack.push(671)) return -1;
				if(!backtrack_stack.push(670)) return -1;
				if(!backtrack_stack.push(669)) return -1;
				if(!backtrack_stack.push(661)) return -1;
				goto label_660;
		}
		break;
	case 2702:
	// Label to restore values of label 676
		v_cabernetsauvignon_1 = int_value_stack.pop();
		break;
	case 677: label_677: 
	// using  symbolic fact madefromgrape(v_madefromgrape_1,v_madefromgrape_2) with literal 2 in rule 
	// 578: q5(A) :- wine(A), madefromgrape(A,B), ot____nom29(B).
	// resulting in symbolic fact q5(v_q5_1)

		if (!wine_set.contains(v_madefromgrape_1)) continue;
			if (!ot____nom29_set.contains(v_madefromgrape_2)) continue;
	case 1690:
	// Continuation for label 677
				v_q5_1 = v_madefromgrape_1;
		if(q5_set.insert(v_q5_1)) {
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1690)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(136)) return -1;
				if(!backtrack_stack.push(135)) return -1;
				goto label_131;
		}
		break;
	case 678: label_678: 
	// using  symbolic fact pauillac(v_pauillac_1) with literal 1 in rule 
	// 280: q11(A) :- pauillac(A).
	// resulting in symbolic fact q11(v_q11_1)

		v_q11_1 = v_pauillac_1;
		if(q11_set.insert(v_q11_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(681)) return -1;
		if(!backtrack_stack.push(680)) return -1;
		goto label_679;
		}
		break;
	case 679: label_679: 
	// using  symbolic fact q11(v_q11_1) with literal 1 in rule 
	// 279: wine(A) :- q11(A).
	// resulting in symbolic fact wine(v_wine_1)

		v_wine_1 = v_q11_1;
		if(wine_set.insert(v_wine_1)) {
		if(!wine_f.insert(v_wine_1)) return -1;
		// Labels for next rules
		if(!backtrack_stack.push(1007)) return -1;
		if(!backtrack_stack.push(989)) return -1;
		if(!backtrack_stack.push(954)) return -1;
		if(!backtrack_stack.push(938)) return -1;
		if(!backtrack_stack.push(911)) return -1;
		if(!backtrack_stack.push(903)) return -1;
		if(!backtrack_stack.push(902)) return -1;
		if(!backtrack_stack.push(901)) return -1;
		if(!backtrack_stack.push(849)) return -1;
		if(!backtrack_stack.push(848)) return -1;
		if(!backtrack_stack.push(821)) return -1;
		if(!backtrack_stack.push(820)) return -1;
		if(!backtrack_stack.push(737)) return -1;
		if(!backtrack_stack.push(712)) return -1;
		if(!backtrack_stack.push(674)) return -1;
		if(!backtrack_stack.push(672)) return -1;
		if(!backtrack_stack.push(583)) return -1;
		if(!backtrack_stack.push(575)) return -1;
		if(!backtrack_stack.push(563)) return -1;
		if(!backtrack_stack.push(551)) return -1;
		if(!backtrack_stack.push(549)) return -1;
		if(!backtrack_stack.push(483)) return -1;
		if(!backtrack_stack.push(473)) return -1;
		if(!backtrack_stack.push(470)) return -1;
		if(!backtrack_stack.push(430)) return -1;
		if(!backtrack_stack.push(401)) return -1;
		if(!backtrack_stack.push(399)) return -1;
		if(!backtrack_stack.push(380)) return -1;
		if(!backtrack_stack.push(364)) return -1;
		if(!backtrack_stack.push(363)) return -1;
		if(!backtrack_stack.push(329)) return -1;
		if(!backtrack_stack.push(328)) return -1;
		if(!backtrack_stack.push(327)) return -1;
		if(!backtrack_stack.push(295)) return -1;
		if(!backtrack_stack.push(294)) return -1;
		if(!backtrack_stack.push(216)) return -1;
		if(!backtrack_stack.push(210)) return -1;
		if(!backtrack_stack.push(192)) return -1;
		if(!backtrack_stack.push(180)) return -1;
		if(!backtrack_stack.push(179)) return -1;
		if(!backtrack_stack.push(178)) return -1;
		if(!backtrack_stack.push(154)) return -1;
		if(!backtrack_stack.push(153)) return -1;
		if(!backtrack_stack.push(100)) return -1;
		if(!backtrack_stack.push(76)) return -1;
		if(!backtrack_stack.push(75)) return -1;
		if(!backtrack_stack.push(67)) return -1;
		if(!backtrack_stack.push(63)) return -1;
		if(!backtrack_stack.push(53)) return -1;
		if(!backtrack_stack.push(46)) return -1;
		if(!backtrack_stack.push(19)) return -1;
		goto label_16;
		}
		break;
	case 680: label_680: 
	// using  symbolic fact q11(v_q11_1) with literal 1 in rule 
	// 284: q11(A) :- q11(B), kaon2equal(A,B).
	// resulting in symbolic fact q11(v_q11_1)

		//recursive
		c284_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q11_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2706)) return -1;
		c284_2_fb.open(v_q11_1);
	case 1693:
	// Continuation for label 680
		while (c284_2_fb.fetch()) {
			v_q11_1 = c284_2_fb.c2();
		if(q11_set.insert(v_q11_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1693)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(681)) return -1;
			if(!backtrack_stack.push(680)) return -1;
			goto label_679;
		}
		}
		break;
	case 2706:
	// Label to restore values of label 680
		v_q11_1 = int_value_stack.pop();
		c284_2_fb.pop(&cursor_stack);
		break;
	case 681: label_681: 
	// using  symbolic fact q11(v_q11_1) with literal 1 in rule 
	// 643: hasflavor(A,A) :- q11(A), kaon2namedobjects(A).
	// resulting in symbolic fact hasflavor(v_hasflavor_1,v_hasflavor_2)

		if (!kaon2namedobjects_b.contains(v_q11_1)) continue;
	case 1694:
	// Continuation for label 681
			v_hasflavor_1 = v_q11_1;
			v_hasflavor_2 = v_q11_1;
		if(hasflavor_set.insert(v_hasflavor_1, v_hasflavor_2)) {
			if(!hasflavor_bf.insert(v_hasflavor_1, v_hasflavor_2)) return -1;
			if(!hasflavor_fb.insert(v_hasflavor_2, v_hasflavor_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1694)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(32)) return -1;
			if(!backtrack_stack.push(31)) return -1;
			if(!backtrack_stack.push(30)) return -1;
			if(!backtrack_stack.push(14)) return -1;
			goto label_12;
		}
		break;
	case 682: label_682: 
	// using  symbolic fact pauillac(v_pauillac_1) with literal 1 in rule 
	// 368: q35(A) :- pauillac(A).
	// resulting in symbolic fact q35(v_q35_1)

		v_q35_1 = v_pauillac_1;
		if(q35_set.insert(v_q35_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(425)) return -1;
		if(!backtrack_stack.push(424)) return -1;
		if(!backtrack_stack.push(423)) return -1;
		if(!backtrack_stack.push(422)) return -1;
		goto label_419;
		}
		break;
	case 683: label_683: 
	// using  symbolic fact pauillac(v_pauillac_1) with literal 1 in rule 
	// 766: q3(A) :- pauillac(A).
	// resulting in symbolic fact q3(v_q3_1)

		v_q3_1 = v_pauillac_1;
		if(q3_set.insert(v_q3_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(143)) return -1;
		goto label_123;
		}
		break;
	case 684: label_684: 
	// using  symbolic fact q43(v_q43_1) with literal 1 in rule 
	// 737: q43(A) :- q43(B), kaon2equal(A,B).
	// resulting in symbolic fact q43(v_q43_1)

		//recursive
		c737_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q43_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2710)) return -1;
		c737_2_fb.open(v_q43_1);
	case 1697:
	// Continuation for label 684
		while (c737_2_fb.fetch()) {
			v_q43_1 = c737_2_fb.c2();
		if(q43_set.insert(v_q43_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1697)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(685)) return -1;
			if(!backtrack_stack.push(684)) return -1;
			goto label_656;
		}
		}
		break;
	case 2710:
	// Label to restore values of label 684
		v_q43_1 = int_value_stack.pop();
		c737_2_fb.pop(&cursor_stack);
		break;
	case 685: label_685: 
	// using  symbolic fact q43(v_q43_1) with literal 2 in rule 
	// 133: q42(A) :- locatedin(A,B), q43(B).
	// resulting in symbolic fact q42(v_q42_1)

		//recursive
		c133_1_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q42_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2711)) return -1;
		c133_1_fb.open(v_q43_1);
	case 1698:
	// Continuation for label 685
		while (c133_1_fb.fetch()) {
			v_q42_1 = c133_1_fb.c2();
		if(q42_set.insert(v_q42_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1698)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(690)) return -1;
			if(!backtrack_stack.push(689)) return -1;
			if(!backtrack_stack.push(686)) return -1;
			goto label_655;
		}
		}
		break;
	case 2711:
	// Label to restore values of label 685
		v_q42_1 = int_value_stack.pop();
		c133_1_fb.pop(&cursor_stack);
		break;
	case 686: label_686: 
	// using  symbolic fact q42(v_q42_1) with literal 1 in rule 
	// 155: locatedin(A,A) :- q42(A), kaon2namedobjects(A).
	// resulting in symbolic fact locatedin(v_locatedin_1,v_locatedin_2)

		//recursive
		if(!int_value_stack.push(v_locatedin_2)) return -1;
		if(!int_value_stack.push(v_locatedin_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2712)) return -1;
		if (!kaon2namedobjects_b.contains(v_q42_1)) continue;
	case 1699:
	// Continuation for label 686
			v_locatedin_1 = v_q42_1;
			v_locatedin_2 = v_q42_1;
		if(locatedin_set.insert(v_locatedin_1, v_locatedin_2)) {
			if(!locatedin_bf.insert(v_locatedin_1, v_locatedin_2)) return -1;
			if(!locatedin_fb.insert(v_locatedin_2, v_locatedin_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1699)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(1005)) return -1;
			if(!backtrack_stack.push(999)) return -1;
			if(!backtrack_stack.push(987)) return -1;
			if(!backtrack_stack.push(981)) return -1;
			if(!backtrack_stack.push(969)) return -1;
			if(!backtrack_stack.push(963)) return -1;
			if(!backtrack_stack.push(952)) return -1;
			if(!backtrack_stack.push(946)) return -1;
			if(!backtrack_stack.push(936)) return -1;
			if(!backtrack_stack.push(930)) return -1;
			if(!backtrack_stack.push(920)) return -1;
			if(!backtrack_stack.push(917)) return -1;
			if(!backtrack_stack.push(898)) return -1;
			if(!backtrack_stack.push(897)) return -1;
			if(!backtrack_stack.push(896)) return -1;
			if(!backtrack_stack.push(893)) return -1;
			if(!backtrack_stack.push(726)) return -1;
			if(!backtrack_stack.push(721)) return -1;
			if(!backtrack_stack.push(694)) return -1;
			if(!backtrack_stack.push(693)) return -1;
			if(!backtrack_stack.push(688)) return -1;
			if(!backtrack_stack.push(687)) return -1;
			if(!backtrack_stack.push(636)) return -1;
			if(!backtrack_stack.push(631)) return -1;
			if(!backtrack_stack.push(592)) return -1;
			if(!backtrack_stack.push(591)) return -1;
			if(!backtrack_stack.push(529)) return -1;
			if(!backtrack_stack.push(528)) return -1;
			if(!backtrack_stack.push(513)) return -1;
			if(!backtrack_stack.push(512)) return -1;
			if(!backtrack_stack.push(511)) return -1;
			if(!backtrack_stack.push(506)) return -1;
			if(!backtrack_stack.push(455)) return -1;
			if(!backtrack_stack.push(450)) return -1;
			if(!backtrack_stack.push(441)) return -1;
			if(!backtrack_stack.push(440)) return -1;
			if(!backtrack_stack.push(439)) return -1;
			if(!backtrack_stack.push(434)) return -1;
			if(!backtrack_stack.push(339)) return -1;
			if(!backtrack_stack.push(338)) return -1;
			if(!backtrack_stack.push(249)) return -1;
			if(!backtrack_stack.push(244)) return -1;
			if(!backtrack_stack.push(52)) return -1;
			if(!backtrack_stack.push(51)) return -1;
			if(!backtrack_stack.push(50)) return -1;
			if(!backtrack_stack.push(49)) return -1;
			if(!backtrack_stack.push(48)) return -1;
			goto label_47;
		}
		break;
	case 2712:
	// Label to restore values of label 686
		v_locatedin_1 = int_value_stack.pop();
		v_locatedin_2 = int_value_stack.pop();
		break;
	case 687: label_687: 
	// using  symbolic fact locatedin(v_locatedin_1,v_locatedin_2) with literal 1 in rule 
	// 113: q37(A) :- locatedin(A,B), q38(B).
	// resulting in symbolic fact q37(v_q37_1)

		if (!q38_set.contains(v_locatedin_2)) continue;
	case 1700:
	// Continuation for label 687
			v_q37_1 = v_locatedin_1;
		if(q37_set.insert(v_q37_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1700)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(651)) return -1;
			if(!backtrack_stack.push(648)) return -1;
			if(!backtrack_stack.push(640)) return -1;
			goto label_639;
		}
		break;
	case 688: label_688: 
	// using  symbolic fact locatedin(v_locatedin_1,v_locatedin_2) with literal 1 in rule 
	// 133: q42(A) :- locatedin(A,B), q43(B).
	// resulting in symbolic fact q42(v_q42_1)

		//recursive
		if(!int_value_stack.push(v_q42_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2714)) return -1;
		if (!q43_set.contains(v_locatedin_2)) continue;
	case 1701:
	// Continuation for label 688
			v_q42_1 = v_locatedin_1;
		if(q42_set.insert(v_q42_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1701)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(690)) return -1;
			if(!backtrack_stack.push(689)) return -1;
			if(!backtrack_stack.push(686)) return -1;
			goto label_655;
		}
		break;
	case 2714:
	// Label to restore values of label 688
		v_q42_1 = int_value_stack.pop();
		break;
	case 689: label_689: 
	// using  symbolic fact q42(v_q42_1) with literal 1 in rule 
	// 156: q42(A) :- q42(B), kaon2equal(A,B).
	// resulting in symbolic fact q42(v_q42_1)

		//recursive
		c156_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q42_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2715)) return -1;
		c156_2_fb.open(v_q42_1);
	case 1702:
	// Continuation for label 689
		while (c156_2_fb.fetch()) {
			v_q42_1 = c156_2_fb.c2();
		if(q42_set.insert(v_q42_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1702)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(690)) return -1;
			if(!backtrack_stack.push(689)) return -1;
			if(!backtrack_stack.push(686)) return -1;
			goto label_655;
		}
		}
		break;
	case 2715:
	// Label to restore values of label 689
		v_q42_1 = int_value_stack.pop();
		c156_2_fb.pop(&cursor_stack);
		break;
	case 690: label_690: 
	// using  symbolic fact q42(v_q42_1) with literal 1 in rule 
	// 735: ot____nom46(A) :- q42(A), kaon2namedobjects(A).
	// resulting in symbolic fact ot____nom46(v_ot____nom46_1)

		//recursive
		if(!int_value_stack.push(v_ot____nom46_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2716)) return -1;
		if (!kaon2namedobjects_b.contains(v_q42_1)) continue;
	case 1703:
	// Continuation for label 690
			v_ot____nom46_1 = v_q42_1;
		if(ot____nom46_set.insert(v_ot____nom46_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1703)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(692)) return -1;
			goto label_691;
		}
		break;
	case 2716:
	// Label to restore values of label 690
		v_ot____nom46_1 = int_value_stack.pop();
		break;
	case 691: label_691: 
	// using  symbolic fact ot____nom46(v_ot____nom46_1) with literal 1 in rule 
	// 736: ot____nom46(A) :- ot____nom46(B), kaon2equal(A,B).
	// resulting in symbolic fact ot____nom46(v_ot____nom46_1)

		//recursive
		c736_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_ot____nom46_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2717)) return -1;
		c736_2_fb.open(v_ot____nom46_1);
	case 1704:
	// Continuation for label 691
		while (c736_2_fb.fetch()) {
			v_ot____nom46_1 = c736_2_fb.c2();
		if(ot____nom46_set.insert(v_ot____nom46_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1704)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(692)) return -1;
			goto label_691;
		}
		}
		break;
	case 2717:
	// Label to restore values of label 691
		v_ot____nom46_1 = int_value_stack.pop();
		c736_2_fb.pop(&cursor_stack);
		break;
	case 692: label_692: 
	// using  symbolic fact ot____nom46(v_ot____nom46_1) with literal 2 in rule 
	// 733: q43(A) :- locatedin(A,B), ot____nom46(B).
	// resulting in symbolic fact q43(v_q43_1)

		//recursive
		c733_1_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q43_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2718)) return -1;
		c733_1_fb.open(v_ot____nom46_1);
	case 1705:
	// Continuation for label 692
		while (c733_1_fb.fetch()) {
			v_q43_1 = c733_1_fb.c2();
		if(q43_set.insert(v_q43_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1705)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(685)) return -1;
			if(!backtrack_stack.push(684)) return -1;
			goto label_656;
		}
		}
		break;
	case 2718:
	// Label to restore values of label 692
		v_q43_1 = int_value_stack.pop();
		c733_1_fb.pop(&cursor_stack);
		break;
	case 693: label_693: 
	// using  symbolic fact locatedin(v_locatedin_1,v_locatedin_2) with literal 1 in rule 
	// 733: q43(A) :- locatedin(A,B), ot____nom46(B).
	// resulting in symbolic fact q43(v_q43_1)

		//recursive
		if(!int_value_stack.push(v_q43_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2719)) return -1;
		if (!ot____nom46_set.contains(v_locatedin_2)) continue;
	case 1706:
	// Continuation for label 693
			v_q43_1 = v_locatedin_1;
		if(q43_set.insert(v_q43_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1706)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(685)) return -1;
			if(!backtrack_stack.push(684)) return -1;
			goto label_656;
		}
		break;
	case 2719:
	// Label to restore values of label 693
		v_q43_1 = int_value_stack.pop();
		break;
	case 694: label_694: 
	// using  symbolic fact locatedin(v_locatedin_1,v_locatedin_2) with literal 1 in rule 
	// 750: q38(A) :- locatedin(A,B), ot____nom42(B).
	// resulting in symbolic fact q38(v_q38_1)

		if (!ot____nom42_set.contains(v_locatedin_2)) continue;
	case 1707:
	// Continuation for label 694
			v_q38_1 = v_locatedin_1;
		if(q38_set.insert(v_q38_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1707)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(647)) return -1;
			if(!backtrack_stack.push(646)) return -1;
			goto label_641;
		}
		break;
	case 695:
	// applying rule 
	// 142: anjou(A) :- anjou_aux(A).
	// resulting in symbolic fact anjou(v_anjou_1)
		c142_1_f.open();
	case 1708:
	// Continuation for label 695
		while (c142_1_f.fetch()) {
			v_anjou_1 = c142_1_f.c1();
		if(anjou_set.insert(v_anjou_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1708)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(717)) return -1;
			if(!backtrack_stack.push(716)) return -1;
			if(!backtrack_stack.push(713)) return -1;
			if(!backtrack_stack.push(701)) return -1;
			if(!backtrack_stack.push(700)) return -1;
			if(!backtrack_stack.push(697)) return -1;
			goto label_696;
		}
		}
		break;
	case 696: label_696: 
	// using  symbolic fact anjou(v_anjou_1) with literal 1 in rule 
	// 141: q20(A) :- anjou(A).
	// resulting in symbolic fact q20(v_q20_1)

		v_q20_1 = v_anjou_1;
		if(q20_set.insert(v_q20_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(184)) return -1;
		if(!backtrack_stack.push(183)) return -1;
		goto label_182;
		}
		break;
	case 697: label_697: 
	// using  symbolic fact anjou(v_anjou_1) with literal 1 in rule 
	// 145: q66(A) :- anjou(A).
	// resulting in symbolic fact q66(v_q66_1)

		//recursive
		if(!int_value_stack.push(v_q66_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2723)) return -1;
		v_q66_1 = v_anjou_1;
		if(q66_set.insert(v_q66_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(723)) return -1;
		if(!backtrack_stack.push(722)) return -1;
		if(!backtrack_stack.push(720)) return -1;
		goto label_698;
		}
		break;
	case 2723:
	// Label to restore values of label 697
		v_q66_1 = int_value_stack.pop();
		break;
	case 698: label_698: 
	// using  symbolic fact q66(v_q66_1) with literal 1 in rule 
	// 144: q67(A) :- q66(A).
	// resulting in symbolic fact q67(v_q67_1)

		//recursive
		if(!int_value_stack.push(v_q67_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2724)) return -1;
		v_q67_1 = v_q66_1;
		if(q67_set.insert(v_q67_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(719)) return -1;
		if(!backtrack_stack.push(718)) return -1;
		goto label_699;
		}
		break;
	case 2724:
	// Label to restore values of label 698
		v_q67_1 = int_value_stack.pop();
		break;
	case 699: label_699: 
	// using  symbolic fact q67(v_q67_1) with literal 1 in rule 
	// 143: anjou(A) :- q67(A), loire(A).
	// resulting in symbolic fact anjou(v_anjou_1)

		//recursive
		if(!int_value_stack.push(v_anjou_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2725)) return -1;
		if (!loire_set.contains(v_q67_1)) continue;
	case 1712:
	// Continuation for label 699
			v_anjou_1 = v_q67_1;
		if(anjou_set.insert(v_anjou_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1712)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(717)) return -1;
			if(!backtrack_stack.push(716)) return -1;
			if(!backtrack_stack.push(713)) return -1;
			if(!backtrack_stack.push(701)) return -1;
			if(!backtrack_stack.push(700)) return -1;
			if(!backtrack_stack.push(697)) return -1;
			goto label_696;
		}
		break;
	case 2725:
	// Label to restore values of label 699
		v_anjou_1 = int_value_stack.pop();
		break;
	case 700: label_700: 
	// using  symbolic fact anjou(v_anjou_1) with literal 1 in rule 
	// 146: anjou(A) :- anjou(B), kaon2equal(A,B).
	// resulting in symbolic fact anjou(v_anjou_1)

		//recursive
		c146_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_anjou_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2726)) return -1;
		c146_2_fb.open(v_anjou_1);
	case 1713:
	// Continuation for label 700
		while (c146_2_fb.fetch()) {
			v_anjou_1 = c146_2_fb.c2();
		if(anjou_set.insert(v_anjou_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1713)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(717)) return -1;
			if(!backtrack_stack.push(716)) return -1;
			if(!backtrack_stack.push(713)) return -1;
			if(!backtrack_stack.push(701)) return -1;
			if(!backtrack_stack.push(700)) return -1;
			if(!backtrack_stack.push(697)) return -1;
			goto label_696;
		}
		}
		break;
	case 2726:
	// Label to restore values of label 700
		v_anjou_1 = int_value_stack.pop();
		c146_2_fb.pop(&cursor_stack);
		break;
	case 701: label_701: 
	// using  symbolic fact anjou(v_anjou_1) with literal 1 in rule 
	// 260: q15(A) :- anjou(A).
	// resulting in symbolic fact q15(v_q15_1)

		v_q15_1 = v_anjou_1;
		if(q15_set.insert(v_q15_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(707)) return -1;
		if(!backtrack_stack.push(706)) return -1;
		if(!backtrack_stack.push(705)) return -1;
		if(!backtrack_stack.push(703)) return -1;
		goto label_702;
		}
		break;
	case 702: label_702: 
	// using  symbolic fact q15(v_q15_1) with literal 1 in rule 
	// 259: hascolor(A,A) :- q15(A), kaon2namedobjects(A).
	// resulting in symbolic fact hascolor(v_hascolor_1,v_hascolor_2)

		if (!kaon2namedobjects_b.contains(v_q15_1)) continue;
	case 1715:
	// Continuation for label 702
			v_hascolor_1 = v_q15_1;
			v_hascolor_2 = v_q15_1;
		if(hascolor_set.insert(v_hascolor_1, v_hascolor_2)) {
			if(!hascolor_bf.insert(v_hascolor_1, v_hascolor_2)) return -1;
			if(!hascolor_fb.insert(v_hascolor_2, v_hascolor_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1715)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(286)) return -1;
			if(!backtrack_stack.push(120)) return -1;
			if(!backtrack_stack.push(80)) return -1;
			if(!backtrack_stack.push(79)) return -1;
			if(!backtrack_stack.push(78)) return -1;
			if(!backtrack_stack.push(77)) return -1;
			if(!backtrack_stack.push(74)) return -1;
			goto label_68;
		}
		break;
	case 703: label_703: 
	// using  symbolic fact q15(v_q15_1) with literal 1 in rule 
	// 262: rosewine(A) :- q15(A).
	// resulting in symbolic fact rosewine(v_rosewine_1)

		//recursive
		if(!int_value_stack.push(v_rosewine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2729)) return -1;
		v_rosewine_1 = v_q15_1;
		if(rosewine_set.insert(v_rosewine_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(711)) return -1;
		if(!backtrack_stack.push(710)) return -1;
		goto label_704;
		}
		break;
	case 2729:
	// Label to restore values of label 703
		v_rosewine_1 = int_value_stack.pop();
		break;
	case 704: label_704: 
	// using  symbolic fact rosewine(v_rosewine_1) with literal 1 in rule 
	// 261: q15(A) :- rosewine(A).
	// resulting in symbolic fact q15(v_q15_1)

		//recursive
		if(!int_value_stack.push(v_q15_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2730)) return -1;
		v_q15_1 = v_rosewine_1;
		if(q15_set.insert(v_q15_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(707)) return -1;
		if(!backtrack_stack.push(706)) return -1;
		if(!backtrack_stack.push(705)) return -1;
		if(!backtrack_stack.push(703)) return -1;
		goto label_702;
		}
		break;
	case 2730:
	// Label to restore values of label 704
		v_q15_1 = int_value_stack.pop();
		break;
	case 705: label_705: 
	// using  symbolic fact q15(v_q15_1) with literal 1 in rule 
	// 263: q15(A) :- q15(B), kaon2equal(A,B).
	// resulting in symbolic fact q15(v_q15_1)

		//recursive
		c263_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q15_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2731)) return -1;
		c263_2_fb.open(v_q15_1);
	case 1718:
	// Continuation for label 705
		while (c263_2_fb.fetch()) {
			v_q15_1 = c263_2_fb.c2();
		if(q15_set.insert(v_q15_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1718)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(707)) return -1;
			if(!backtrack_stack.push(706)) return -1;
			if(!backtrack_stack.push(705)) return -1;
			if(!backtrack_stack.push(703)) return -1;
			goto label_702;
		}
		}
		break;
	case 2731:
	// Label to restore values of label 705
		v_q15_1 = int_value_stack.pop();
		c263_2_fb.pop(&cursor_stack);
		break;
	case 706: label_706: 
	// using  symbolic fact q15(v_q15_1) with literal 1 in rule 
	// 285: wine(A) :- q15(A).
	// resulting in symbolic fact wine(v_wine_1)

		v_wine_1 = v_q15_1;
		if(wine_set.insert(v_wine_1)) {
		if(!wine_f.insert(v_wine_1)) return -1;
		// Labels for next rules
		if(!backtrack_stack.push(1007)) return -1;
		if(!backtrack_stack.push(989)) return -1;
		if(!backtrack_stack.push(954)) return -1;
		if(!backtrack_stack.push(938)) return -1;
		if(!backtrack_stack.push(911)) return -1;
		if(!backtrack_stack.push(903)) return -1;
		if(!backtrack_stack.push(902)) return -1;
		if(!backtrack_stack.push(901)) return -1;
		if(!backtrack_stack.push(849)) return -1;
		if(!backtrack_stack.push(848)) return -1;
		if(!backtrack_stack.push(821)) return -1;
		if(!backtrack_stack.push(820)) return -1;
		if(!backtrack_stack.push(737)) return -1;
		if(!backtrack_stack.push(712)) return -1;
		if(!backtrack_stack.push(674)) return -1;
		if(!backtrack_stack.push(672)) return -1;
		if(!backtrack_stack.push(583)) return -1;
		if(!backtrack_stack.push(575)) return -1;
		if(!backtrack_stack.push(563)) return -1;
		if(!backtrack_stack.push(551)) return -1;
		if(!backtrack_stack.push(549)) return -1;
		if(!backtrack_stack.push(483)) return -1;
		if(!backtrack_stack.push(473)) return -1;
		if(!backtrack_stack.push(470)) return -1;
		if(!backtrack_stack.push(430)) return -1;
		if(!backtrack_stack.push(401)) return -1;
		if(!backtrack_stack.push(399)) return -1;
		if(!backtrack_stack.push(380)) return -1;
		if(!backtrack_stack.push(364)) return -1;
		if(!backtrack_stack.push(363)) return -1;
		if(!backtrack_stack.push(329)) return -1;
		if(!backtrack_stack.push(328)) return -1;
		if(!backtrack_stack.push(327)) return -1;
		if(!backtrack_stack.push(295)) return -1;
		if(!backtrack_stack.push(294)) return -1;
		if(!backtrack_stack.push(216)) return -1;
		if(!backtrack_stack.push(210)) return -1;
		if(!backtrack_stack.push(192)) return -1;
		if(!backtrack_stack.push(180)) return -1;
		if(!backtrack_stack.push(179)) return -1;
		if(!backtrack_stack.push(178)) return -1;
		if(!backtrack_stack.push(154)) return -1;
		if(!backtrack_stack.push(153)) return -1;
		if(!backtrack_stack.push(100)) return -1;
		if(!backtrack_stack.push(76)) return -1;
		if(!backtrack_stack.push(75)) return -1;
		if(!backtrack_stack.push(67)) return -1;
		if(!backtrack_stack.push(63)) return -1;
		if(!backtrack_stack.push(53)) return -1;
		if(!backtrack_stack.push(46)) return -1;
		if(!backtrack_stack.push(19)) return -1;
		goto label_16;
		}
		break;
	case 707: label_707: 
	// using  symbolic fact q15(v_q15_1) with literal 1 in rule 
	// 654: ot____nom32(A) :- q15(A), kaon2namedobjects(A).
	// resulting in symbolic fact ot____nom32(v_ot____nom32_1)

		//recursive
		if(!int_value_stack.push(v_ot____nom32_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2733)) return -1;
		if (!kaon2namedobjects_b.contains(v_q15_1)) continue;
	case 1720:
	// Continuation for label 707
			v_ot____nom32_1 = v_q15_1;
		if(ot____nom32_set.insert(v_ot____nom32_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1720)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(709)) return -1;
			goto label_708;
		}
		break;
	case 2733:
	// Label to restore values of label 707
		v_ot____nom32_1 = int_value_stack.pop();
		break;
	case 708: label_708: 
	// using  symbolic fact ot____nom32(v_ot____nom32_1) with literal 1 in rule 
	// 655: ot____nom32(A) :- ot____nom32(B), kaon2equal(A,B).
	// resulting in symbolic fact ot____nom32(v_ot____nom32_1)

		//recursive
		c655_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_ot____nom32_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2734)) return -1;
		c655_2_fb.open(v_ot____nom32_1);
	case 1721:
	// Continuation for label 708
		while (c655_2_fb.fetch()) {
			v_ot____nom32_1 = c655_2_fb.c2();
		if(ot____nom32_set.insert(v_ot____nom32_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1721)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(709)) return -1;
			goto label_708;
		}
		}
		break;
	case 2734:
	// Label to restore values of label 708
		v_ot____nom32_1 = int_value_stack.pop();
		c655_2_fb.pop(&cursor_stack);
		break;
	case 709: label_709: 
	// using  symbolic fact ot____nom32(v_ot____nom32_1) with literal 3 in rule 
	// 265: rosewine(A) :- wine(A), hascolor(A,B), ot____nom32(B).
	// resulting in symbolic fact rosewine(v_rosewine_1)

		//recursive
		c265_1_f.push(&cursor_stack);
		if(!int_value_stack.push(v_rosewine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2735)) return -1;
		c265_1_f.open();
		while (c265_1_f.fetch()) {
			if (!hascolor_set.contains(c265_1_f.c1(), v_ot____nom32_1)) continue;
	case 1722:
	// Continuation for label 709
				v_rosewine_1 = c265_1_f.c1();
		if(rosewine_set.insert(v_rosewine_1)) {
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1722)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(711)) return -1;
				if(!backtrack_stack.push(710)) return -1;
				goto label_704;
		}
		}
		break;
	case 2735:
	// Label to restore values of label 709
		v_rosewine_1 = int_value_stack.pop();
		c265_1_f.pop(&cursor_stack);
		break;
	case 710: label_710: 
	// using  symbolic fact rosewine(v_rosewine_1) with literal 1 in rule 
	// 264: rosewine(A) :- rosewine(B), kaon2equal(A,B).
	// resulting in symbolic fact rosewine(v_rosewine_1)

		//recursive
		c264_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_rosewine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2736)) return -1;
		c264_2_fb.open(v_rosewine_1);
	case 1723:
	// Continuation for label 710
		while (c264_2_fb.fetch()) {
			v_rosewine_1 = c264_2_fb.c2();
		if(rosewine_set.insert(v_rosewine_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1723)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(711)) return -1;
			if(!backtrack_stack.push(710)) return -1;
			goto label_704;
		}
		}
		break;
	case 2736:
	// Label to restore values of label 710
		v_rosewine_1 = int_value_stack.pop();
		c264_2_fb.pop(&cursor_stack);
		break;
	case 711: label_711: 
	// using  symbolic fact rosewine(v_rosewine_1) with literal 1 in rule 
	// 509: wine(A) :- rosewine(A).
	// resulting in symbolic fact wine(v_wine_1)

		//recursive
		if(!int_value_stack.push(v_wine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2737)) return -1;
		v_wine_1 = v_rosewine_1;
		if(wine_set.insert(v_wine_1)) {
		if(!wine_f.insert(v_wine_1)) return -1;
		// Labels for next rules
		if(!backtrack_stack.push(1007)) return -1;
		if(!backtrack_stack.push(989)) return -1;
		if(!backtrack_stack.push(954)) return -1;
		if(!backtrack_stack.push(938)) return -1;
		if(!backtrack_stack.push(911)) return -1;
		if(!backtrack_stack.push(903)) return -1;
		if(!backtrack_stack.push(902)) return -1;
		if(!backtrack_stack.push(901)) return -1;
		if(!backtrack_stack.push(849)) return -1;
		if(!backtrack_stack.push(848)) return -1;
		if(!backtrack_stack.push(821)) return -1;
		if(!backtrack_stack.push(820)) return -1;
		if(!backtrack_stack.push(737)) return -1;
		if(!backtrack_stack.push(712)) return -1;
		if(!backtrack_stack.push(674)) return -1;
		if(!backtrack_stack.push(672)) return -1;
		if(!backtrack_stack.push(583)) return -1;
		if(!backtrack_stack.push(575)) return -1;
		if(!backtrack_stack.push(563)) return -1;
		if(!backtrack_stack.push(551)) return -1;
		if(!backtrack_stack.push(549)) return -1;
		if(!backtrack_stack.push(483)) return -1;
		if(!backtrack_stack.push(473)) return -1;
		if(!backtrack_stack.push(470)) return -1;
		if(!backtrack_stack.push(430)) return -1;
		if(!backtrack_stack.push(401)) return -1;
		if(!backtrack_stack.push(399)) return -1;
		if(!backtrack_stack.push(380)) return -1;
		if(!backtrack_stack.push(364)) return -1;
		if(!backtrack_stack.push(363)) return -1;
		if(!backtrack_stack.push(329)) return -1;
		if(!backtrack_stack.push(328)) return -1;
		if(!backtrack_stack.push(327)) return -1;
		if(!backtrack_stack.push(295)) return -1;
		if(!backtrack_stack.push(294)) return -1;
		if(!backtrack_stack.push(216)) return -1;
		if(!backtrack_stack.push(210)) return -1;
		if(!backtrack_stack.push(192)) return -1;
		if(!backtrack_stack.push(180)) return -1;
		if(!backtrack_stack.push(179)) return -1;
		if(!backtrack_stack.push(178)) return -1;
		if(!backtrack_stack.push(154)) return -1;
		if(!backtrack_stack.push(153)) return -1;
		if(!backtrack_stack.push(100)) return -1;
		if(!backtrack_stack.push(76)) return -1;
		if(!backtrack_stack.push(75)) return -1;
		if(!backtrack_stack.push(67)) return -1;
		if(!backtrack_stack.push(63)) return -1;
		if(!backtrack_stack.push(53)) return -1;
		if(!backtrack_stack.push(46)) return -1;
		if(!backtrack_stack.push(19)) return -1;
		goto label_16;
		}
		break;
	case 2737:
	// Label to restore values of label 711
		v_wine_1 = int_value_stack.pop();
		break;
	case 712: label_712: 
	// using  symbolic fact wine(v_wine_1) with literal 1 in rule 
	// 265: rosewine(A) :- wine(A), hascolor(A,B), ot____nom32(B).
	// resulting in symbolic fact rosewine(v_rosewine_1)

		//recursive
		c265_2_bf.push(&cursor_stack);
		if(!int_value_stack.push(v_rosewine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2738)) return -1;
		c265_2_bf.open(v_wine_1);
		while (c265_2_bf.fetch()) {
			if (!ot____nom32_set.contains(c265_2_bf.c2())) continue;
	case 1725:
	// Continuation for label 712
				v_rosewine_1 = v_wine_1;
		if(rosewine_set.insert(v_rosewine_1)) {
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1725)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(711)) return -1;
				if(!backtrack_stack.push(710)) return -1;
				goto label_704;
		}
		}
		break;
	case 2738:
	// Label to restore values of label 712
		v_rosewine_1 = int_value_stack.pop();
		c265_2_bf.pop(&cursor_stack);
		break;
	case 713: label_713: 
	// using  symbolic fact anjou(v_anjou_1) with literal 1 in rule 
	// 381: loire(A) :- anjou(A).
	// resulting in symbolic fact loire(v_loire_1)

		//recursive
		if(!int_value_stack.push(v_loire_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2739)) return -1;
		v_loire_1 = v_anjou_1;
		if(loire_set.insert(v_loire_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(971)) return -1;
		if(!backtrack_stack.push(715)) return -1;
		if(!backtrack_stack.push(714)) return -1;
		if(!backtrack_stack.push(487)) return -1;
		if(!backtrack_stack.push(486)) return -1;
		if(!backtrack_stack.push(484)) return -1;
		if(!backtrack_stack.push(482)) return -1;
		goto label_478;
		}
		break;
	case 2739:
	// Label to restore values of label 713
		v_loire_1 = int_value_stack.pop();
		break;
	case 714: label_714: 
	// using  symbolic fact loire(v_loire_1) with literal 2 in rule 
	// 103: sancerre(A) :- q40(A), loire(A).
	// resulting in symbolic fact sancerre(v_sancerre_1)

		if (!q40_set.contains(v_loire_1)) continue;
	case 1727:
	// Continuation for label 714
			v_sancerre_1 = v_loire_1;
		if(sancerre_set.insert(v_sancerre_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1727)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(627)) return -1;
			if(!backtrack_stack.push(623)) return -1;
			if(!backtrack_stack.push(619)) return -1;
			if(!backtrack_stack.push(600)) return -1;
			if(!backtrack_stack.push(599)) return -1;
			if(!backtrack_stack.push(598)) return -1;
			if(!backtrack_stack.push(595)) return -1;
			goto label_594;
		}
		break;
	case 715: label_715: 
	// using  symbolic fact loire(v_loire_1) with literal 2 in rule 
	// 143: anjou(A) :- q67(A), loire(A).
	// resulting in symbolic fact anjou(v_anjou_1)

		//recursive
		if(!int_value_stack.push(v_anjou_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2741)) return -1;
		if (!q67_set.contains(v_loire_1)) continue;
	case 1728:
	// Continuation for label 715
			v_anjou_1 = v_loire_1;
		if(anjou_set.insert(v_anjou_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1728)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(717)) return -1;
			if(!backtrack_stack.push(716)) return -1;
			if(!backtrack_stack.push(713)) return -1;
			if(!backtrack_stack.push(701)) return -1;
			if(!backtrack_stack.push(700)) return -1;
			if(!backtrack_stack.push(697)) return -1;
			goto label_696;
		}
		break;
	case 2741:
	// Label to restore values of label 715
		v_anjou_1 = int_value_stack.pop();
		break;
	case 716: label_716: 
	// using  symbolic fact anjou(v_anjou_1) with literal 1 in rule 
	// 404: q45(A) :- anjou(A).
	// resulting in symbolic fact q45(v_q45_1)

		v_q45_1 = v_anjou_1;
		if(q45_set.insert(v_q45_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(240)) return -1;
		if(!backtrack_stack.push(177)) return -1;
		goto label_176;
		}
		break;
	case 717: label_717: 
	// using  symbolic fact anjou(v_anjou_1) with literal 1 in rule 
	// 512: q32(A) :- anjou(A).
	// resulting in symbolic fact q32(v_q32_1)

		v_q32_1 = v_anjou_1;
		if(q32_set.insert(v_q32_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(626)) return -1;
		if(!backtrack_stack.push(625)) return -1;
		goto label_624;
		}
		break;
	case 718: label_718: 
	// using  symbolic fact q67(v_q67_1) with literal 1 in rule 
	// 732: q67(A) :- q67(B), kaon2equal(A,B).
	// resulting in symbolic fact q67(v_q67_1)

		//recursive
		c732_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q67_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2744)) return -1;
		c732_2_fb.open(v_q67_1);
	case 1731:
	// Continuation for label 718
		while (c732_2_fb.fetch()) {
			v_q67_1 = c732_2_fb.c2();
		if(q67_set.insert(v_q67_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1731)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(719)) return -1;
			if(!backtrack_stack.push(718)) return -1;
			goto label_699;
		}
		}
		break;
	case 2744:
	// Label to restore values of label 718
		v_q67_1 = int_value_stack.pop();
		c732_2_fb.pop(&cursor_stack);
		break;
	case 719: label_719: 
	// using  symbolic fact q67(v_q67_1) with literal 2 in rule 
	// 147: q66(A) :- locatedin(A,B), q67(B).
	// resulting in symbolic fact q66(v_q66_1)

		//recursive
		c147_1_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q66_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2745)) return -1;
		c147_1_fb.open(v_q67_1);
	case 1732:
	// Continuation for label 719
		while (c147_1_fb.fetch()) {
			v_q66_1 = c147_1_fb.c2();
		if(q66_set.insert(v_q66_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1732)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(723)) return -1;
			if(!backtrack_stack.push(722)) return -1;
			if(!backtrack_stack.push(720)) return -1;
			goto label_698;
		}
		}
		break;
	case 2745:
	// Label to restore values of label 719
		v_q66_1 = int_value_stack.pop();
		c147_1_fb.pop(&cursor_stack);
		break;
	case 720: label_720: 
	// using  symbolic fact q66(v_q66_1) with literal 1 in rule 
	// 216: locatedin(A,A) :- q66(A), kaon2namedobjects(A).
	// resulting in symbolic fact locatedin(v_locatedin_1,v_locatedin_2)

		//recursive
		if(!int_value_stack.push(v_locatedin_2)) return -1;
		if(!int_value_stack.push(v_locatedin_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2746)) return -1;
		if (!kaon2namedobjects_b.contains(v_q66_1)) continue;
	case 1733:
	// Continuation for label 720
			v_locatedin_1 = v_q66_1;
			v_locatedin_2 = v_q66_1;
		if(locatedin_set.insert(v_locatedin_1, v_locatedin_2)) {
			if(!locatedin_bf.insert(v_locatedin_1, v_locatedin_2)) return -1;
			if(!locatedin_fb.insert(v_locatedin_2, v_locatedin_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1733)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(1005)) return -1;
			if(!backtrack_stack.push(999)) return -1;
			if(!backtrack_stack.push(987)) return -1;
			if(!backtrack_stack.push(981)) return -1;
			if(!backtrack_stack.push(969)) return -1;
			if(!backtrack_stack.push(963)) return -1;
			if(!backtrack_stack.push(952)) return -1;
			if(!backtrack_stack.push(946)) return -1;
			if(!backtrack_stack.push(936)) return -1;
			if(!backtrack_stack.push(930)) return -1;
			if(!backtrack_stack.push(920)) return -1;
			if(!backtrack_stack.push(917)) return -1;
			if(!backtrack_stack.push(898)) return -1;
			if(!backtrack_stack.push(897)) return -1;
			if(!backtrack_stack.push(896)) return -1;
			if(!backtrack_stack.push(893)) return -1;
			if(!backtrack_stack.push(726)) return -1;
			if(!backtrack_stack.push(721)) return -1;
			if(!backtrack_stack.push(694)) return -1;
			if(!backtrack_stack.push(693)) return -1;
			if(!backtrack_stack.push(688)) return -1;
			if(!backtrack_stack.push(687)) return -1;
			if(!backtrack_stack.push(636)) return -1;
			if(!backtrack_stack.push(631)) return -1;
			if(!backtrack_stack.push(592)) return -1;
			if(!backtrack_stack.push(591)) return -1;
			if(!backtrack_stack.push(529)) return -1;
			if(!backtrack_stack.push(528)) return -1;
			if(!backtrack_stack.push(513)) return -1;
			if(!backtrack_stack.push(512)) return -1;
			if(!backtrack_stack.push(511)) return -1;
			if(!backtrack_stack.push(506)) return -1;
			if(!backtrack_stack.push(455)) return -1;
			if(!backtrack_stack.push(450)) return -1;
			if(!backtrack_stack.push(441)) return -1;
			if(!backtrack_stack.push(440)) return -1;
			if(!backtrack_stack.push(439)) return -1;
			if(!backtrack_stack.push(434)) return -1;
			if(!backtrack_stack.push(339)) return -1;
			if(!backtrack_stack.push(338)) return -1;
			if(!backtrack_stack.push(249)) return -1;
			if(!backtrack_stack.push(244)) return -1;
			if(!backtrack_stack.push(52)) return -1;
			if(!backtrack_stack.push(51)) return -1;
			if(!backtrack_stack.push(50)) return -1;
			if(!backtrack_stack.push(49)) return -1;
			if(!backtrack_stack.push(48)) return -1;
			goto label_47;
		}
		break;
	case 2746:
	// Label to restore values of label 720
		v_locatedin_1 = int_value_stack.pop();
		v_locatedin_2 = int_value_stack.pop();
		break;
	case 721: label_721: 
	// using  symbolic fact locatedin(v_locatedin_1,v_locatedin_2) with literal 1 in rule 
	// 147: q66(A) :- locatedin(A,B), q67(B).
	// resulting in symbolic fact q66(v_q66_1)

		//recursive
		if(!int_value_stack.push(v_q66_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2747)) return -1;
		if (!q67_set.contains(v_locatedin_2)) continue;
	case 1734:
	// Continuation for label 721
			v_q66_1 = v_locatedin_1;
		if(q66_set.insert(v_q66_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1734)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(723)) return -1;
			if(!backtrack_stack.push(722)) return -1;
			if(!backtrack_stack.push(720)) return -1;
			goto label_698;
		}
		break;
	case 2747:
	// Label to restore values of label 721
		v_q66_1 = int_value_stack.pop();
		break;
	case 722: label_722: 
	// using  symbolic fact q66(v_q66_1) with literal 1 in rule 
	// 217: q66(A) :- q66(B), kaon2equal(A,B).
	// resulting in symbolic fact q66(v_q66_1)

		//recursive
		c217_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q66_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2748)) return -1;
		c217_2_fb.open(v_q66_1);
	case 1735:
	// Continuation for label 722
		while (c217_2_fb.fetch()) {
			v_q66_1 = c217_2_fb.c2();
		if(q66_set.insert(v_q66_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1735)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(723)) return -1;
			if(!backtrack_stack.push(722)) return -1;
			if(!backtrack_stack.push(720)) return -1;
			goto label_698;
		}
		}
		break;
	case 2748:
	// Label to restore values of label 722
		v_q66_1 = int_value_stack.pop();
		c217_2_fb.pop(&cursor_stack);
		break;
	case 723: label_723: 
	// using  symbolic fact q66(v_q66_1) with literal 1 in rule 
	// 730: ot____nom14(A) :- q66(A), kaon2namedobjects(A).
	// resulting in symbolic fact ot____nom14(v_ot____nom14_1)

		//recursive
		if(!int_value_stack.push(v_ot____nom14_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2749)) return -1;
		if (!kaon2namedobjects_b.contains(v_q66_1)) continue;
	case 1736:
	// Continuation for label 723
			v_ot____nom14_1 = v_q66_1;
		if(ot____nom14_set.insert(v_ot____nom14_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1736)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(725)) return -1;
			goto label_724;
		}
		break;
	case 2749:
	// Label to restore values of label 723
		v_ot____nom14_1 = int_value_stack.pop();
		break;
	case 724: label_724: 
	// using  symbolic fact ot____nom14(v_ot____nom14_1) with literal 1 in rule 
	// 731: ot____nom14(A) :- ot____nom14(B), kaon2equal(A,B).
	// resulting in symbolic fact ot____nom14(v_ot____nom14_1)

		//recursive
		c731_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_ot____nom14_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2750)) return -1;
		c731_2_fb.open(v_ot____nom14_1);
	case 1737:
	// Continuation for label 724
		while (c731_2_fb.fetch()) {
			v_ot____nom14_1 = c731_2_fb.c2();
		if(ot____nom14_set.insert(v_ot____nom14_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1737)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(725)) return -1;
			goto label_724;
		}
		}
		break;
	case 2750:
	// Label to restore values of label 724
		v_ot____nom14_1 = int_value_stack.pop();
		c731_2_fb.pop(&cursor_stack);
		break;
	case 725: label_725: 
	// using  symbolic fact ot____nom14(v_ot____nom14_1) with literal 2 in rule 
	// 728: q67(A) :- locatedin(A,B), ot____nom14(B).
	// resulting in symbolic fact q67(v_q67_1)

		//recursive
		c728_1_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q67_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2751)) return -1;
		c728_1_fb.open(v_ot____nom14_1);
	case 1738:
	// Continuation for label 725
		while (c728_1_fb.fetch()) {
			v_q67_1 = c728_1_fb.c2();
		if(q67_set.insert(v_q67_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1738)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(719)) return -1;
			if(!backtrack_stack.push(718)) return -1;
			goto label_699;
		}
		}
		break;
	case 2751:
	// Label to restore values of label 725
		v_q67_1 = int_value_stack.pop();
		c728_1_fb.pop(&cursor_stack);
		break;
	case 726: label_726: 
	// using  symbolic fact locatedin(v_locatedin_1,v_locatedin_2) with literal 1 in rule 
	// 728: q67(A) :- locatedin(A,B), ot____nom14(B).
	// resulting in symbolic fact q67(v_q67_1)

		//recursive
		if(!int_value_stack.push(v_q67_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2752)) return -1;
		if (!ot____nom14_set.contains(v_locatedin_2)) continue;
	case 1739:
	// Continuation for label 726
			v_q67_1 = v_locatedin_1;
		if(q67_set.insert(v_q67_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1739)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(719)) return -1;
			if(!backtrack_stack.push(718)) return -1;
			goto label_699;
		}
		break;
	case 2752:
	// Label to restore values of label 726
		v_q67_1 = int_value_stack.pop();
		break;
	case 727:
	// applying rule 
	// 166: port(A) :- port_aux(A).
	// resulting in symbolic fact port(v_port_1)
		c166_1_f.open();
	case 1740:
	// Continuation for label 727
		while (c166_1_f.fetch()) {
			v_port_1 = c166_1_f.c1();
		if(port_set.insert(v_port_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1740)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(733)) return -1;
			if(!backtrack_stack.push(732)) return -1;
			if(!backtrack_stack.push(731)) return -1;
			if(!backtrack_stack.push(730)) return -1;
			if(!backtrack_stack.push(729)) return -1;
			goto label_728;
		}
		}
		break;
	case 728: label_728: 
	// using  symbolic fact port(v_port_1) with literal 1 in rule 
	// 165: locatedin(A,A) :- port(A), kaon2namedobjects(A).
	// resulting in symbolic fact locatedin(v_locatedin_1,v_locatedin_2)

		if (!kaon2namedobjects_b.contains(v_port_1)) continue;
	case 1741:
	// Continuation for label 728
			v_locatedin_1 = v_port_1;
			v_locatedin_2 = v_port_1;
		if(locatedin_set.insert(v_locatedin_1, v_locatedin_2)) {
			if(!locatedin_bf.insert(v_locatedin_1, v_locatedin_2)) return -1;
			if(!locatedin_fb.insert(v_locatedin_2, v_locatedin_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1741)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(1005)) return -1;
			if(!backtrack_stack.push(999)) return -1;
			if(!backtrack_stack.push(987)) return -1;
			if(!backtrack_stack.push(981)) return -1;
			if(!backtrack_stack.push(969)) return -1;
			if(!backtrack_stack.push(963)) return -1;
			if(!backtrack_stack.push(952)) return -1;
			if(!backtrack_stack.push(946)) return -1;
			if(!backtrack_stack.push(936)) return -1;
			if(!backtrack_stack.push(930)) return -1;
			if(!backtrack_stack.push(920)) return -1;
			if(!backtrack_stack.push(917)) return -1;
			if(!backtrack_stack.push(898)) return -1;
			if(!backtrack_stack.push(897)) return -1;
			if(!backtrack_stack.push(896)) return -1;
			if(!backtrack_stack.push(893)) return -1;
			if(!backtrack_stack.push(726)) return -1;
			if(!backtrack_stack.push(721)) return -1;
			if(!backtrack_stack.push(694)) return -1;
			if(!backtrack_stack.push(693)) return -1;
			if(!backtrack_stack.push(688)) return -1;
			if(!backtrack_stack.push(687)) return -1;
			if(!backtrack_stack.push(636)) return -1;
			if(!backtrack_stack.push(631)) return -1;
			if(!backtrack_stack.push(592)) return -1;
			if(!backtrack_stack.push(591)) return -1;
			if(!backtrack_stack.push(529)) return -1;
			if(!backtrack_stack.push(528)) return -1;
			if(!backtrack_stack.push(513)) return -1;
			if(!backtrack_stack.push(512)) return -1;
			if(!backtrack_stack.push(511)) return -1;
			if(!backtrack_stack.push(506)) return -1;
			if(!backtrack_stack.push(455)) return -1;
			if(!backtrack_stack.push(450)) return -1;
			if(!backtrack_stack.push(441)) return -1;
			if(!backtrack_stack.push(440)) return -1;
			if(!backtrack_stack.push(439)) return -1;
			if(!backtrack_stack.push(434)) return -1;
			if(!backtrack_stack.push(339)) return -1;
			if(!backtrack_stack.push(338)) return -1;
			if(!backtrack_stack.push(249)) return -1;
			if(!backtrack_stack.push(244)) return -1;
			if(!backtrack_stack.push(52)) return -1;
			if(!backtrack_stack.push(51)) return -1;
			if(!backtrack_stack.push(50)) return -1;
			if(!backtrack_stack.push(49)) return -1;
			if(!backtrack_stack.push(48)) return -1;
			goto label_47;
		}
		break;
	case 729: label_729: 
	// using  symbolic fact port(v_port_1) with literal 1 in rule 
	// 167: port(A) :- port(B), kaon2equal(A,B).
	// resulting in symbolic fact port(v_port_1)

		//recursive
		c167_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_port_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2755)) return -1;
		c167_2_fb.open(v_port_1);
	case 1742:
	// Continuation for label 729
		while (c167_2_fb.fetch()) {
			v_port_1 = c167_2_fb.c2();
		if(port_set.insert(v_port_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1742)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(733)) return -1;
			if(!backtrack_stack.push(732)) return -1;
			if(!backtrack_stack.push(731)) return -1;
			if(!backtrack_stack.push(730)) return -1;
			if(!backtrack_stack.push(729)) return -1;
			goto label_728;
		}
		}
		break;
	case 2755:
	// Label to restore values of label 729
		v_port_1 = int_value_stack.pop();
		c167_2_fb.pop(&cursor_stack);
		break;
	case 730: label_730: 
	// using  symbolic fact port(v_port_1) with literal 1 in rule 
	// 281: q11(A) :- port(A).
	// resulting in symbolic fact q11(v_q11_1)

		v_q11_1 = v_port_1;
		if(q11_set.insert(v_q11_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(681)) return -1;
		if(!backtrack_stack.push(680)) return -1;
		goto label_679;
		}
		break;
	case 731: label_731: 
	// using  symbolic fact port(v_port_1) with literal 1 in rule 
	// 369: q35(A) :- port(A).
	// resulting in symbolic fact q35(v_q35_1)

		v_q35_1 = v_port_1;
		if(q35_set.insert(v_q35_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(425)) return -1;
		if(!backtrack_stack.push(424)) return -1;
		if(!backtrack_stack.push(423)) return -1;
		if(!backtrack_stack.push(422)) return -1;
		goto label_419;
		}
		break;
	case 732: label_732: 
	// using  symbolic fact port(v_port_1) with literal 1 in rule 
	// 435: redwine(A) :- port(A).
	// resulting in symbolic fact redwine(v_redwine_1)

		v_redwine_1 = v_port_1;
		if(redwine_set.insert(v_redwine_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(569)) return -1;
		if(!backtrack_stack.push(384)) return -1;
		if(!backtrack_stack.push(205)) return -1;
		if(!backtrack_stack.push(199)) return -1;
		if(!backtrack_stack.push(193)) return -1;
		goto label_191;
		}
		break;
	case 733: label_733: 
	// using  symbolic fact port(v_port_1) with literal 1 in rule 
	// 480: q70(A) :- port(A).
	// resulting in symbolic fact q70(v_q70_1)

		v_q70_1 = v_port_1;
		if(q70_set.insert(v_q70_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(742)) return -1;
		if(!backtrack_stack.push(741)) return -1;
		if(!backtrack_stack.push(740)) return -1;
		if(!backtrack_stack.push(739)) return -1;
		goto label_734;
		}
		break;
	case 734: label_734: 
	// using  symbolic fact q70(v_q70_1) with literal 1 in rule 
	// 290: q71(A) :- q70(A).
	// resulting in symbolic fact q71(v_q71_1)

		//recursive
		if(!int_value_stack.push(v_q71_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2760)) return -1;
		v_q71_1 = v_q70_1;
		if(q71_set.insert(v_q71_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(754)) return -1;
		if(!backtrack_stack.push(745)) return -1;
		goto label_735;
		}
		break;
	case 2760:
	// Label to restore values of label 734
		v_q71_1 = int_value_stack.pop();
		break;
	case 735: label_735: 
	// using  symbolic fact q71(v_q71_1) with literal 1 in rule 
	// 289: sweetwine(A) :- q71(A), wine(A).
	// resulting in symbolic fact sweetwine(v_sweetwine_1)

		//recursive
		if(!int_value_stack.push(v_sweetwine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2761)) return -1;
		if (!wine_set.contains(v_q71_1)) continue;
	case 1748:
	// Continuation for label 735
			v_sweetwine_1 = v_q71_1;
		if(sweetwine_set.insert(v_sweetwine_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1748)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(756)) return -1;
			if(!backtrack_stack.push(738)) return -1;
			goto label_736;
		}
		break;
	case 2761:
	// Label to restore values of label 735
		v_sweetwine_1 = int_value_stack.pop();
		break;
	case 736: label_736: 
	// using  symbolic fact sweetwine(v_sweetwine_1) with literal 1 in rule 
	// 288: wine(A) :- sweetwine(A).
	// resulting in symbolic fact wine(v_wine_1)

		//recursive
		if(!int_value_stack.push(v_wine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2762)) return -1;
		v_wine_1 = v_sweetwine_1;
		if(wine_set.insert(v_wine_1)) {
		if(!wine_f.insert(v_wine_1)) return -1;
		// Labels for next rules
		if(!backtrack_stack.push(1007)) return -1;
		if(!backtrack_stack.push(989)) return -1;
		if(!backtrack_stack.push(954)) return -1;
		if(!backtrack_stack.push(938)) return -1;
		if(!backtrack_stack.push(911)) return -1;
		if(!backtrack_stack.push(903)) return -1;
		if(!backtrack_stack.push(902)) return -1;
		if(!backtrack_stack.push(901)) return -1;
		if(!backtrack_stack.push(849)) return -1;
		if(!backtrack_stack.push(848)) return -1;
		if(!backtrack_stack.push(821)) return -1;
		if(!backtrack_stack.push(820)) return -1;
		if(!backtrack_stack.push(737)) return -1;
		if(!backtrack_stack.push(712)) return -1;
		if(!backtrack_stack.push(674)) return -1;
		if(!backtrack_stack.push(672)) return -1;
		if(!backtrack_stack.push(583)) return -1;
		if(!backtrack_stack.push(575)) return -1;
		if(!backtrack_stack.push(563)) return -1;
		if(!backtrack_stack.push(551)) return -1;
		if(!backtrack_stack.push(549)) return -1;
		if(!backtrack_stack.push(483)) return -1;
		if(!backtrack_stack.push(473)) return -1;
		if(!backtrack_stack.push(470)) return -1;
		if(!backtrack_stack.push(430)) return -1;
		if(!backtrack_stack.push(401)) return -1;
		if(!backtrack_stack.push(399)) return -1;
		if(!backtrack_stack.push(380)) return -1;
		if(!backtrack_stack.push(364)) return -1;
		if(!backtrack_stack.push(363)) return -1;
		if(!backtrack_stack.push(329)) return -1;
		if(!backtrack_stack.push(328)) return -1;
		if(!backtrack_stack.push(327)) return -1;
		if(!backtrack_stack.push(295)) return -1;
		if(!backtrack_stack.push(294)) return -1;
		if(!backtrack_stack.push(216)) return -1;
		if(!backtrack_stack.push(210)) return -1;
		if(!backtrack_stack.push(192)) return -1;
		if(!backtrack_stack.push(180)) return -1;
		if(!backtrack_stack.push(179)) return -1;
		if(!backtrack_stack.push(178)) return -1;
		if(!backtrack_stack.push(154)) return -1;
		if(!backtrack_stack.push(153)) return -1;
		if(!backtrack_stack.push(100)) return -1;
		if(!backtrack_stack.push(76)) return -1;
		if(!backtrack_stack.push(75)) return -1;
		if(!backtrack_stack.push(67)) return -1;
		if(!backtrack_stack.push(63)) return -1;
		if(!backtrack_stack.push(53)) return -1;
		if(!backtrack_stack.push(46)) return -1;
		if(!backtrack_stack.push(19)) return -1;
		goto label_16;
		}
		break;
	case 2762:
	// Label to restore values of label 736
		v_wine_1 = int_value_stack.pop();
		break;
	case 737: label_737: 
	// using  symbolic fact wine(v_wine_1) with literal 2 in rule 
	// 289: sweetwine(A) :- q71(A), wine(A).
	// resulting in symbolic fact sweetwine(v_sweetwine_1)

		//recursive
		if(!int_value_stack.push(v_sweetwine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2763)) return -1;
		if (!q71_set.contains(v_wine_1)) continue;
	case 1750:
	// Continuation for label 737
			v_sweetwine_1 = v_wine_1;
		if(sweetwine_set.insert(v_sweetwine_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1750)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(756)) return -1;
			if(!backtrack_stack.push(738)) return -1;
			goto label_736;
		}
		break;
	case 2763:
	// Label to restore values of label 737
		v_sweetwine_1 = int_value_stack.pop();
		break;
	case 738: label_738: 
	// using  symbolic fact sweetwine(v_sweetwine_1) with literal 1 in rule 
	// 481: q70(A) :- sweetwine(A).
	// resulting in symbolic fact q70(v_q70_1)

		//recursive
		if(!int_value_stack.push(v_q70_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2764)) return -1;
		v_q70_1 = v_sweetwine_1;
		if(q70_set.insert(v_q70_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(742)) return -1;
		if(!backtrack_stack.push(741)) return -1;
		if(!backtrack_stack.push(740)) return -1;
		if(!backtrack_stack.push(739)) return -1;
		goto label_734;
		}
		break;
	case 2764:
	// Label to restore values of label 738
		v_q70_1 = int_value_stack.pop();
		break;
	case 739: label_739: 
	// using  symbolic fact q70(v_q70_1) with literal 1 in rule 
	// 479: wine(A) :- q70(A).
	// resulting in symbolic fact wine(v_wine_1)

		//recursive
		if(!int_value_stack.push(v_wine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2765)) return -1;
		v_wine_1 = v_q70_1;
		if(wine_set.insert(v_wine_1)) {
		if(!wine_f.insert(v_wine_1)) return -1;
		// Labels for next rules
		if(!backtrack_stack.push(1007)) return -1;
		if(!backtrack_stack.push(989)) return -1;
		if(!backtrack_stack.push(954)) return -1;
		if(!backtrack_stack.push(938)) return -1;
		if(!backtrack_stack.push(911)) return -1;
		if(!backtrack_stack.push(903)) return -1;
		if(!backtrack_stack.push(902)) return -1;
		if(!backtrack_stack.push(901)) return -1;
		if(!backtrack_stack.push(849)) return -1;
		if(!backtrack_stack.push(848)) return -1;
		if(!backtrack_stack.push(821)) return -1;
		if(!backtrack_stack.push(820)) return -1;
		if(!backtrack_stack.push(737)) return -1;
		if(!backtrack_stack.push(712)) return -1;
		if(!backtrack_stack.push(674)) return -1;
		if(!backtrack_stack.push(672)) return -1;
		if(!backtrack_stack.push(583)) return -1;
		if(!backtrack_stack.push(575)) return -1;
		if(!backtrack_stack.push(563)) return -1;
		if(!backtrack_stack.push(551)) return -1;
		if(!backtrack_stack.push(549)) return -1;
		if(!backtrack_stack.push(483)) return -1;
		if(!backtrack_stack.push(473)) return -1;
		if(!backtrack_stack.push(470)) return -1;
		if(!backtrack_stack.push(430)) return -1;
		if(!backtrack_stack.push(401)) return -1;
		if(!backtrack_stack.push(399)) return -1;
		if(!backtrack_stack.push(380)) return -1;
		if(!backtrack_stack.push(364)) return -1;
		if(!backtrack_stack.push(363)) return -1;
		if(!backtrack_stack.push(329)) return -1;
		if(!backtrack_stack.push(328)) return -1;
		if(!backtrack_stack.push(327)) return -1;
		if(!backtrack_stack.push(295)) return -1;
		if(!backtrack_stack.push(294)) return -1;
		if(!backtrack_stack.push(216)) return -1;
		if(!backtrack_stack.push(210)) return -1;
		if(!backtrack_stack.push(192)) return -1;
		if(!backtrack_stack.push(180)) return -1;
		if(!backtrack_stack.push(179)) return -1;
		if(!backtrack_stack.push(178)) return -1;
		if(!backtrack_stack.push(154)) return -1;
		if(!backtrack_stack.push(153)) return -1;
		if(!backtrack_stack.push(100)) return -1;
		if(!backtrack_stack.push(76)) return -1;
		if(!backtrack_stack.push(75)) return -1;
		if(!backtrack_stack.push(67)) return -1;
		if(!backtrack_stack.push(63)) return -1;
		if(!backtrack_stack.push(53)) return -1;
		if(!backtrack_stack.push(46)) return -1;
		if(!backtrack_stack.push(19)) return -1;
		goto label_16;
		}
		break;
	case 2765:
	// Label to restore values of label 739
		v_wine_1 = int_value_stack.pop();
		break;
	case 740: label_740: 
	// using  symbolic fact q70(v_q70_1) with literal 1 in rule 
	// 487: q70(A) :- q70(B), kaon2equal(A,B).
	// resulting in symbolic fact q70(v_q70_1)

		//recursive
		c487_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q70_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2766)) return -1;
		c487_2_fb.open(v_q70_1);
	case 1753:
	// Continuation for label 740
		while (c487_2_fb.fetch()) {
			v_q70_1 = c487_2_fb.c2();
		if(q70_set.insert(v_q70_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1753)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(742)) return -1;
			if(!backtrack_stack.push(741)) return -1;
			if(!backtrack_stack.push(740)) return -1;
			if(!backtrack_stack.push(739)) return -1;
			goto label_734;
		}
		}
		break;
	case 2766:
	// Label to restore values of label 740
		v_q70_1 = int_value_stack.pop();
		c487_2_fb.pop(&cursor_stack);
		break;
	case 741: label_741: 
	// using  symbolic fact q70(v_q70_1) with literal 1 in rule 
	// 591: hassugar(A,A) :- q70(A), kaon2namedobjects(A).
	// resulting in symbolic fact hassugar(v_hassugar_1,v_hassugar_2)

		//recursive
		if(!int_value_stack.push(v_hassugar_2)) return -1;
		if(!int_value_stack.push(v_hassugar_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2767)) return -1;
		if (!kaon2namedobjects_b.contains(v_q70_1)) continue;
	case 1754:
	// Continuation for label 741
			v_hassugar_1 = v_q70_1;
			v_hassugar_2 = v_q70_1;
		if(hassugar_set.insert(v_hassugar_1, v_hassugar_2)) {
			if(!hassugar_bf.insert(v_hassugar_1, v_hassugar_2)) return -1;
			if(!hassugar_fb.insert(v_hassugar_2, v_hassugar_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1754)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(755)) return -1;
			if(!backtrack_stack.push(238)) return -1;
			if(!backtrack_stack.push(214)) return -1;
			if(!backtrack_stack.push(27)) return -1;
			if(!backtrack_stack.push(24)) return -1;
			if(!backtrack_stack.push(23)) return -1;
			if(!backtrack_stack.push(22)) return -1;
			goto label_17;
		}
		break;
	case 2767:
	// Label to restore values of label 741
		v_hassugar_1 = int_value_stack.pop();
		v_hassugar_2 = int_value_stack.pop();
		break;
	case 742: label_742: 
	// using  symbolic fact q70(v_q70_1) with literal 1 in rule 
	// 628: ot____nom1(A) :- q70(A), kaon2namedobjects(A).
	// resulting in symbolic fact ot____nom1(v_ot____nom1_1)

		//recursive
		if(!int_value_stack.push(v_ot____nom1_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2768)) return -1;
		if (!kaon2namedobjects_b.contains(v_q70_1)) continue;
	case 1755:
	// Continuation for label 742
			v_ot____nom1_1 = v_q70_1;
		if(ot____nom1_set.insert(v_ot____nom1_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1755)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(744)) return -1;
			goto label_743;
		}
		break;
	case 2768:
	// Label to restore values of label 742
		v_ot____nom1_1 = int_value_stack.pop();
		break;
	case 743: label_743: 
	// using  symbolic fact ot____nom1(v_ot____nom1_1) with literal 1 in rule 
	// 629: ot____nom1(A) :- ot____nom1(B), kaon2equal(A,B).
	// resulting in symbolic fact ot____nom1(v_ot____nom1_1)

		//recursive
		c629_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_ot____nom1_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2769)) return -1;
		c629_2_fb.open(v_ot____nom1_1);
	case 1756:
	// Continuation for label 743
		while (c629_2_fb.fetch()) {
			v_ot____nom1_1 = c629_2_fb.c2();
		if(ot____nom1_set.insert(v_ot____nom1_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1756)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(744)) return -1;
			goto label_743;
		}
		}
		break;
	case 2769:
	// Label to restore values of label 743
		v_ot____nom1_1 = int_value_stack.pop();
		c629_2_fb.pop(&cursor_stack);
		break;
	case 744: label_744: 
	// using  symbolic fact ot____nom1(v_ot____nom1_1) with literal 2 in rule 
	// 294: q71(A) :- hassugar(A,B), ot____nom1(B).
	// resulting in symbolic fact q71(v_q71_1)

		//recursive
		c294_1_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q71_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2770)) return -1;
		c294_1_fb.open(v_ot____nom1_1);
	case 1757:
	// Continuation for label 744
		while (c294_1_fb.fetch()) {
			v_q71_1 = c294_1_fb.c2();
		if(q71_set.insert(v_q71_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1757)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(754)) return -1;
			if(!backtrack_stack.push(745)) return -1;
			goto label_735;
		}
		}
		break;
	case 2770:
	// Label to restore values of label 744
		v_q71_1 = int_value_stack.pop();
		c294_1_fb.pop(&cursor_stack);
		break;
	case 745: label_745: 
	// using  symbolic fact q71(v_q71_1) with literal 1 in rule 
	// 293: sweetriesling(A) :- q71(A), riesling(A).
	// resulting in symbolic fact sweetriesling(v_sweetriesling_1)

		//recursive
		if(!int_value_stack.push(v_sweetriesling_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2771)) return -1;
		if (!riesling_set.contains(v_q71_1)) continue;
	case 1758:
	// Continuation for label 745
			v_sweetriesling_1 = v_q71_1;
		if(sweetriesling_set.insert(v_sweetriesling_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1758)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(751)) return -1;
			if(!backtrack_stack.push(749)) return -1;
			if(!backtrack_stack.push(748)) return -1;
			if(!backtrack_stack.push(747)) return -1;
			goto label_746;
		}
		break;
	case 2771:
	// Label to restore values of label 745
		v_sweetriesling_1 = int_value_stack.pop();
		break;
	case 746: label_746: 
	// using  symbolic fact sweetriesling(v_sweetriesling_1) with literal 1 in rule 
	// 291: q70(A) :- sweetriesling(A).
	// resulting in symbolic fact q70(v_q70_1)

		//recursive
		if(!int_value_stack.push(v_q70_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2772)) return -1;
		v_q70_1 = v_sweetriesling_1;
		if(q70_set.insert(v_q70_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(742)) return -1;
		if(!backtrack_stack.push(741)) return -1;
		if(!backtrack_stack.push(740)) return -1;
		if(!backtrack_stack.push(739)) return -1;
		goto label_734;
		}
		break;
	case 2772:
	// Label to restore values of label 746
		v_q70_1 = int_value_stack.pop();
		break;
	case 747: label_747: 
	// using  symbolic fact sweetriesling(v_sweetriesling_1) with literal 1 in rule 
	// 370: q35(A) :- sweetriesling(A).
	// resulting in symbolic fact q35(v_q35_1)

		v_q35_1 = v_sweetriesling_1;
		if(q35_set.insert(v_q35_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(425)) return -1;
		if(!backtrack_stack.push(424)) return -1;
		if(!backtrack_stack.push(423)) return -1;
		if(!backtrack_stack.push(422)) return -1;
		goto label_419;
		}
		break;
	case 748: label_748: 
	// using  symbolic fact sweetriesling(v_sweetriesling_1) with literal 1 in rule 
	// 371: sweetriesling(A) :- sweetriesling(B), kaon2equal(A,B).
	// resulting in symbolic fact sweetriesling(v_sweetriesling_1)

		//recursive
		c371_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_sweetriesling_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2774)) return -1;
		c371_2_fb.open(v_sweetriesling_1);
	case 1761:
	// Continuation for label 748
		while (c371_2_fb.fetch()) {
			v_sweetriesling_1 = c371_2_fb.c2();
		if(sweetriesling_set.insert(v_sweetriesling_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1761)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(751)) return -1;
			if(!backtrack_stack.push(749)) return -1;
			if(!backtrack_stack.push(748)) return -1;
			if(!backtrack_stack.push(747)) return -1;
			goto label_746;
		}
		}
		break;
	case 2774:
	// Label to restore values of label 748
		v_sweetriesling_1 = int_value_stack.pop();
		c371_2_fb.pop(&cursor_stack);
		break;
	case 749: label_749: 
	// using  symbolic fact sweetriesling(v_sweetriesling_1) with literal 1 in rule 
	// 429: riesling(A) :- sweetriesling(A).
	// resulting in symbolic fact riesling(v_riesling_1)

		//recursive
		if(!int_value_stack.push(v_riesling_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2775)) return -1;
		v_riesling_1 = v_sweetriesling_1;
		if(riesling_set.insert(v_riesling_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(750)) return -1;
		if(!backtrack_stack.push(310)) return -1;
		if(!backtrack_stack.push(309)) return -1;
		if(!backtrack_stack.push(308)) return -1;
		if(!backtrack_stack.push(305)) return -1;
		if(!backtrack_stack.push(304)) return -1;
		if(!backtrack_stack.push(303)) return -1;
		goto label_302;
		}
		break;
	case 2775:
	// Label to restore values of label 749
		v_riesling_1 = int_value_stack.pop();
		break;
	case 750: label_750: 
	// using  symbolic fact riesling(v_riesling_1) with literal 2 in rule 
	// 293: sweetriesling(A) :- q71(A), riesling(A).
	// resulting in symbolic fact sweetriesling(v_sweetriesling_1)

		//recursive
		if(!int_value_stack.push(v_sweetriesling_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2776)) return -1;
		if (!q71_set.contains(v_riesling_1)) continue;
	case 1763:
	// Continuation for label 750
			v_sweetriesling_1 = v_riesling_1;
		if(sweetriesling_set.insert(v_sweetriesling_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1763)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(751)) return -1;
			if(!backtrack_stack.push(749)) return -1;
			if(!backtrack_stack.push(748)) return -1;
			if(!backtrack_stack.push(747)) return -1;
			goto label_746;
		}
		break;
	case 2776:
	// Label to restore values of label 750
		v_sweetriesling_1 = int_value_stack.pop();
		break;
	case 751: label_751: 
	// using  symbolic fact sweetriesling(v_sweetriesling_1) with literal 1 in rule 
	// 460: dessertwine(A) :- sweetriesling(A).
	// resulting in symbolic fact dessertwine(v_dessertwine_1)

		//recursive
		if(!int_value_stack.push(v_dessertwine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2777)) return -1;
		v_dessertwine_1 = v_sweetriesling_1;
		if(dessertwine_set.insert(v_dessertwine_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(770)) return -1;
		if(!backtrack_stack.push(753)) return -1;
		goto label_752;
		}
		break;
	case 2777:
	// Label to restore values of label 751
		v_dessertwine_1 = int_value_stack.pop();
		break;
	case 752: label_752: 
	// using  symbolic fact dessertwine(v_dessertwine_1) with literal 1 in rule 
	// 458: wine(A) :- dessertwine(A).
	// resulting in symbolic fact wine(v_wine_1)

		//recursive
		if(!int_value_stack.push(v_wine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2778)) return -1;
		v_wine_1 = v_dessertwine_1;
		if(wine_set.insert(v_wine_1)) {
		if(!wine_f.insert(v_wine_1)) return -1;
		// Labels for next rules
		if(!backtrack_stack.push(1007)) return -1;
		if(!backtrack_stack.push(989)) return -1;
		if(!backtrack_stack.push(954)) return -1;
		if(!backtrack_stack.push(938)) return -1;
		if(!backtrack_stack.push(911)) return -1;
		if(!backtrack_stack.push(903)) return -1;
		if(!backtrack_stack.push(902)) return -1;
		if(!backtrack_stack.push(901)) return -1;
		if(!backtrack_stack.push(849)) return -1;
		if(!backtrack_stack.push(848)) return -1;
		if(!backtrack_stack.push(821)) return -1;
		if(!backtrack_stack.push(820)) return -1;
		if(!backtrack_stack.push(737)) return -1;
		if(!backtrack_stack.push(712)) return -1;
		if(!backtrack_stack.push(674)) return -1;
		if(!backtrack_stack.push(672)) return -1;
		if(!backtrack_stack.push(583)) return -1;
		if(!backtrack_stack.push(575)) return -1;
		if(!backtrack_stack.push(563)) return -1;
		if(!backtrack_stack.push(551)) return -1;
		if(!backtrack_stack.push(549)) return -1;
		if(!backtrack_stack.push(483)) return -1;
		if(!backtrack_stack.push(473)) return -1;
		if(!backtrack_stack.push(470)) return -1;
		if(!backtrack_stack.push(430)) return -1;
		if(!backtrack_stack.push(401)) return -1;
		if(!backtrack_stack.push(399)) return -1;
		if(!backtrack_stack.push(380)) return -1;
		if(!backtrack_stack.push(364)) return -1;
		if(!backtrack_stack.push(363)) return -1;
		if(!backtrack_stack.push(329)) return -1;
		if(!backtrack_stack.push(328)) return -1;
		if(!backtrack_stack.push(327)) return -1;
		if(!backtrack_stack.push(295)) return -1;
		if(!backtrack_stack.push(294)) return -1;
		if(!backtrack_stack.push(216)) return -1;
		if(!backtrack_stack.push(210)) return -1;
		if(!backtrack_stack.push(192)) return -1;
		if(!backtrack_stack.push(180)) return -1;
		if(!backtrack_stack.push(179)) return -1;
		if(!backtrack_stack.push(178)) return -1;
		if(!backtrack_stack.push(154)) return -1;
		if(!backtrack_stack.push(153)) return -1;
		if(!backtrack_stack.push(100)) return -1;
		if(!backtrack_stack.push(76)) return -1;
		if(!backtrack_stack.push(75)) return -1;
		if(!backtrack_stack.push(67)) return -1;
		if(!backtrack_stack.push(63)) return -1;
		if(!backtrack_stack.push(53)) return -1;
		if(!backtrack_stack.push(46)) return -1;
		if(!backtrack_stack.push(19)) return -1;
		goto label_16;
		}
		break;
	case 2778:
	// Label to restore values of label 752
		v_wine_1 = int_value_stack.pop();
		break;
	case 753: label_753: 
	// using  symbolic fact dessertwine(v_dessertwine_1) with literal 1 in rule 
	// 462: dessertwine(A) :- dessertwine(B), kaon2equal(A,B).
	// resulting in symbolic fact dessertwine(v_dessertwine_1)

		//recursive
		c462_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_dessertwine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2779)) return -1;
		c462_2_fb.open(v_dessertwine_1);
	case 1766:
	// Continuation for label 753
		while (c462_2_fb.fetch()) {
			v_dessertwine_1 = c462_2_fb.c2();
		if(dessertwine_set.insert(v_dessertwine_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1766)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(770)) return -1;
			if(!backtrack_stack.push(753)) return -1;
			goto label_752;
		}
		}
		break;
	case 2779:
	// Label to restore values of label 753
		v_dessertwine_1 = int_value_stack.pop();
		c462_2_fb.pop(&cursor_stack);
		break;
	case 754: label_754: 
	// using  symbolic fact q71(v_q71_1) with literal 1 in rule 
	// 630: q71(A) :- q71(B), kaon2equal(A,B).
	// resulting in symbolic fact q71(v_q71_1)

		//recursive
		c630_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q71_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2780)) return -1;
		c630_2_fb.open(v_q71_1);
	case 1767:
	// Continuation for label 754
		while (c630_2_fb.fetch()) {
			v_q71_1 = c630_2_fb.c2();
		if(q71_set.insert(v_q71_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1767)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(754)) return -1;
			if(!backtrack_stack.push(745)) return -1;
			goto label_735;
		}
		}
		break;
	case 2780:
	// Label to restore values of label 754
		v_q71_1 = int_value_stack.pop();
		c630_2_fb.pop(&cursor_stack);
		break;
	case 755: label_755: 
	// using  symbolic fact hassugar(v_hassugar_1,v_hassugar_2) with literal 1 in rule 
	// 294: q71(A) :- hassugar(A,B), ot____nom1(B).
	// resulting in symbolic fact q71(v_q71_1)

		//recursive
		if(!int_value_stack.push(v_q71_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2781)) return -1;
		if (!ot____nom1_set.contains(v_hassugar_2)) continue;
	case 1768:
	// Continuation for label 755
			v_q71_1 = v_hassugar_1;
		if(q71_set.insert(v_q71_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1768)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(754)) return -1;
			if(!backtrack_stack.push(745)) return -1;
			goto label_735;
		}
		break;
	case 2781:
	// Label to restore values of label 755
		v_q71_1 = int_value_stack.pop();
		break;
	case 756: label_756: 
	// using  symbolic fact sweetwine(v_sweetwine_1) with literal 1 in rule 
	// 482: sweetwine(A) :- sweetwine(B), kaon2equal(A,B).
	// resulting in symbolic fact sweetwine(v_sweetwine_1)

		//recursive
		c482_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_sweetwine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2782)) return -1;
		c482_2_fb.open(v_sweetwine_1);
	case 1769:
	// Continuation for label 756
		while (c482_2_fb.fetch()) {
			v_sweetwine_1 = c482_2_fb.c2();
		if(sweetwine_set.insert(v_sweetwine_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1769)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(756)) return -1;
			if(!backtrack_stack.push(738)) return -1;
			goto label_736;
		}
		}
		break;
	case 2782:
	// Label to restore values of label 756
		v_sweetwine_1 = int_value_stack.pop();
		c482_2_fb.pop(&cursor_stack);
		break;
	case 757:
	// applying rule 
	// 198: sauternes(A) :- sauternes_aux(A).
	// resulting in symbolic fact sauternes(v_sauternes_1)
		c198_1_f.open();
	case 1770:
	// Continuation for label 757
		while (c198_1_f.fetch()) {
			v_sauternes_1 = c198_1_f.c1();
		if(sauternes_set.insert(v_sauternes_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1770)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(772)) return -1;
			if(!backtrack_stack.push(762)) return -1;
			if(!backtrack_stack.push(761)) return -1;
			if(!backtrack_stack.push(760)) return -1;
			if(!backtrack_stack.push(759)) return -1;
			goto label_758;
		}
		}
		break;
	case 758: label_758: 
	// using  symbolic fact sauternes(v_sauternes_1) with literal 1 in rule 
	// 197: bordeaux(A) :- sauternes(A).
	// resulting in symbolic fact bordeaux(v_bordeaux_1)

		v_bordeaux_1 = v_sauternes_1;
		if(bordeaux_set.insert(v_bordeaux_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(781)) return -1;
		if(!backtrack_stack.push(566)) return -1;
		if(!backtrack_stack.push(565)) return -1;
		if(!backtrack_stack.push(564)) return -1;
		if(!backtrack_stack.push(562)) return -1;
		if(!backtrack_stack.push(558)) return -1;
		goto label_526;
		}
		break;
	case 759: label_759: 
	// using  symbolic fact sauternes(v_sauternes_1) with literal 1 in rule 
	// 199: sauternes(A) :- sauternes(B), kaon2equal(A,B).
	// resulting in symbolic fact sauternes(v_sauternes_1)

		//recursive
		c199_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_sauternes_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2785)) return -1;
		c199_2_fb.open(v_sauternes_1);
	case 1772:
	// Continuation for label 759
		while (c199_2_fb.fetch()) {
			v_sauternes_1 = c199_2_fb.c2();
		if(sauternes_set.insert(v_sauternes_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1772)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(772)) return -1;
			if(!backtrack_stack.push(762)) return -1;
			if(!backtrack_stack.push(761)) return -1;
			if(!backtrack_stack.push(760)) return -1;
			if(!backtrack_stack.push(759)) return -1;
			goto label_758;
		}
		}
		break;
	case 2785:
	// Label to restore values of label 759
		v_sauternes_1 = int_value_stack.pop();
		c199_2_fb.pop(&cursor_stack);
		break;
	case 760: label_760: 
	// using  symbolic fact sauternes(v_sauternes_1) with literal 1 in rule 
	// 215: locatedin(A,A) :- sauternes(A), kaon2namedobjects(A).
	// resulting in symbolic fact locatedin(v_locatedin_1,v_locatedin_2)

		if (!kaon2namedobjects_b.contains(v_sauternes_1)) continue;
	case 1773:
	// Continuation for label 760
			v_locatedin_1 = v_sauternes_1;
			v_locatedin_2 = v_sauternes_1;
		if(locatedin_set.insert(v_locatedin_1, v_locatedin_2)) {
			if(!locatedin_bf.insert(v_locatedin_1, v_locatedin_2)) return -1;
			if(!locatedin_fb.insert(v_locatedin_2, v_locatedin_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1773)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(1005)) return -1;
			if(!backtrack_stack.push(999)) return -1;
			if(!backtrack_stack.push(987)) return -1;
			if(!backtrack_stack.push(981)) return -1;
			if(!backtrack_stack.push(969)) return -1;
			if(!backtrack_stack.push(963)) return -1;
			if(!backtrack_stack.push(952)) return -1;
			if(!backtrack_stack.push(946)) return -1;
			if(!backtrack_stack.push(936)) return -1;
			if(!backtrack_stack.push(930)) return -1;
			if(!backtrack_stack.push(920)) return -1;
			if(!backtrack_stack.push(917)) return -1;
			if(!backtrack_stack.push(898)) return -1;
			if(!backtrack_stack.push(897)) return -1;
			if(!backtrack_stack.push(896)) return -1;
			if(!backtrack_stack.push(893)) return -1;
			if(!backtrack_stack.push(726)) return -1;
			if(!backtrack_stack.push(721)) return -1;
			if(!backtrack_stack.push(694)) return -1;
			if(!backtrack_stack.push(693)) return -1;
			if(!backtrack_stack.push(688)) return -1;
			if(!backtrack_stack.push(687)) return -1;
			if(!backtrack_stack.push(636)) return -1;
			if(!backtrack_stack.push(631)) return -1;
			if(!backtrack_stack.push(592)) return -1;
			if(!backtrack_stack.push(591)) return -1;
			if(!backtrack_stack.push(529)) return -1;
			if(!backtrack_stack.push(528)) return -1;
			if(!backtrack_stack.push(513)) return -1;
			if(!backtrack_stack.push(512)) return -1;
			if(!backtrack_stack.push(511)) return -1;
			if(!backtrack_stack.push(506)) return -1;
			if(!backtrack_stack.push(455)) return -1;
			if(!backtrack_stack.push(450)) return -1;
			if(!backtrack_stack.push(441)) return -1;
			if(!backtrack_stack.push(440)) return -1;
			if(!backtrack_stack.push(439)) return -1;
			if(!backtrack_stack.push(434)) return -1;
			if(!backtrack_stack.push(339)) return -1;
			if(!backtrack_stack.push(338)) return -1;
			if(!backtrack_stack.push(249)) return -1;
			if(!backtrack_stack.push(244)) return -1;
			if(!backtrack_stack.push(52)) return -1;
			if(!backtrack_stack.push(51)) return -1;
			if(!backtrack_stack.push(50)) return -1;
			if(!backtrack_stack.push(49)) return -1;
			if(!backtrack_stack.push(48)) return -1;
			goto label_47;
		}
		break;
	case 761: label_761: 
	// using  symbolic fact sauternes(v_sauternes_1) with literal 1 in rule 
	// 408: q7(A) :- sauternes(A).
	// resulting in symbolic fact q7(v_q7_1)

		v_q7_1 = v_sauternes_1;
		if(q7_set.insert(v_q7_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(622)) return -1;
		if(!backtrack_stack.push(621)) return -1;
		goto label_620;
		}
		break;
	case 762: label_762: 
	// using  symbolic fact sauternes(v_sauternes_1) with literal 1 in rule 
	// 485: lateharvest(A) :- sauternes(A).
	// resulting in symbolic fact lateharvest(v_lateharvest_1)

		v_lateharvest_1 = v_sauternes_1;
		if(lateharvest_set.insert(v_lateharvest_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(766)) return -1;
		if(!backtrack_stack.push(765)) return -1;
		if(!backtrack_stack.push(764)) return -1;
		goto label_763;
		}
		break;
	case 763: label_763: 
	// using  symbolic fact lateharvest(v_lateharvest_1) with literal 1 in rule 
	// 483: q70(A) :- lateharvest(A).
	// resulting in symbolic fact q70(v_q70_1)

		v_q70_1 = v_lateharvest_1;
		if(q70_set.insert(v_q70_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(742)) return -1;
		if(!backtrack_stack.push(741)) return -1;
		if(!backtrack_stack.push(740)) return -1;
		if(!backtrack_stack.push(739)) return -1;
		goto label_734;
		}
		break;
	case 764: label_764: 
	// using  symbolic fact lateharvest(v_lateharvest_1) with literal 1 in rule 
	// 486: lateharvest(A) :- lateharvest(B), kaon2equal(A,B).
	// resulting in symbolic fact lateharvest(v_lateharvest_1)

		//recursive
		c486_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_lateharvest_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2790)) return -1;
		c486_2_fb.open(v_lateharvest_1);
	case 1777:
	// Continuation for label 764
		while (c486_2_fb.fetch()) {
			v_lateharvest_1 = c486_2_fb.c2();
		if(lateharvest_set.insert(v_lateharvest_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1777)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(766)) return -1;
			if(!backtrack_stack.push(765)) return -1;
			if(!backtrack_stack.push(764)) return -1;
			goto label_763;
		}
		}
		break;
	case 2790:
	// Label to restore values of label 764
		v_lateharvest_1 = int_value_stack.pop();
		c486_2_fb.pop(&cursor_stack);
		break;
	case 765: label_765: 
	// using  symbolic fact lateharvest(v_lateharvest_1) with literal 1 in rule 
	// 514: wine(A) :- lateharvest(A).
	// resulting in symbolic fact wine(v_wine_1)

		v_wine_1 = v_lateharvest_1;
		if(wine_set.insert(v_wine_1)) {
		if(!wine_f.insert(v_wine_1)) return -1;
		// Labels for next rules
		if(!backtrack_stack.push(1007)) return -1;
		if(!backtrack_stack.push(989)) return -1;
		if(!backtrack_stack.push(954)) return -1;
		if(!backtrack_stack.push(938)) return -1;
		if(!backtrack_stack.push(911)) return -1;
		if(!backtrack_stack.push(903)) return -1;
		if(!backtrack_stack.push(902)) return -1;
		if(!backtrack_stack.push(901)) return -1;
		if(!backtrack_stack.push(849)) return -1;
		if(!backtrack_stack.push(848)) return -1;
		if(!backtrack_stack.push(821)) return -1;
		if(!backtrack_stack.push(820)) return -1;
		if(!backtrack_stack.push(737)) return -1;
		if(!backtrack_stack.push(712)) return -1;
		if(!backtrack_stack.push(674)) return -1;
		if(!backtrack_stack.push(672)) return -1;
		if(!backtrack_stack.push(583)) return -1;
		if(!backtrack_stack.push(575)) return -1;
		if(!backtrack_stack.push(563)) return -1;
		if(!backtrack_stack.push(551)) return -1;
		if(!backtrack_stack.push(549)) return -1;
		if(!backtrack_stack.push(483)) return -1;
		if(!backtrack_stack.push(473)) return -1;
		if(!backtrack_stack.push(470)) return -1;
		if(!backtrack_stack.push(430)) return -1;
		if(!backtrack_stack.push(401)) return -1;
		if(!backtrack_stack.push(399)) return -1;
		if(!backtrack_stack.push(380)) return -1;
		if(!backtrack_stack.push(364)) return -1;
		if(!backtrack_stack.push(363)) return -1;
		if(!backtrack_stack.push(329)) return -1;
		if(!backtrack_stack.push(328)) return -1;
		if(!backtrack_stack.push(327)) return -1;
		if(!backtrack_stack.push(295)) return -1;
		if(!backtrack_stack.push(294)) return -1;
		if(!backtrack_stack.push(216)) return -1;
		if(!backtrack_stack.push(210)) return -1;
		if(!backtrack_stack.push(192)) return -1;
		if(!backtrack_stack.push(180)) return -1;
		if(!backtrack_stack.push(179)) return -1;
		if(!backtrack_stack.push(178)) return -1;
		if(!backtrack_stack.push(154)) return -1;
		if(!backtrack_stack.push(153)) return -1;
		if(!backtrack_stack.push(100)) return -1;
		if(!backtrack_stack.push(76)) return -1;
		if(!backtrack_stack.push(75)) return -1;
		if(!backtrack_stack.push(67)) return -1;
		if(!backtrack_stack.push(63)) return -1;
		if(!backtrack_stack.push(53)) return -1;
		if(!backtrack_stack.push(46)) return -1;
		if(!backtrack_stack.push(19)) return -1;
		goto label_16;
		}
		break;
	case 766: label_766: 
	// using  symbolic fact lateharvest(v_lateharvest_1) with literal 3 in rule 
	// 257: icewine(A) :- q21(A), dessertwine(A), lateharvest(A).
	// resulting in symbolic fact icewine(v_icewine_1)

		//recursive
		if(!int_value_stack.push(v_icewine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2792)) return -1;
		if (!q21_set.contains(v_lateharvest_1)) continue;
			if (!dessertwine_set.contains(v_lateharvest_1)) continue;
	case 1779:
	// Continuation for label 766
				v_icewine_1 = v_lateharvest_1;
		if(icewine_set.insert(v_icewine_1)) {
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1779)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(771)) return -1;
				if(!backtrack_stack.push(769)) return -1;
				if(!backtrack_stack.push(768)) return -1;
				goto label_767;
		}
		break;
	case 2792:
	// Label to restore values of label 766
		v_icewine_1 = int_value_stack.pop();
		break;
	case 767: label_767: 
	// using  symbolic fact icewine(v_icewine_1) with literal 1 in rule 
	// 254: q12(A) :- icewine(A).
	// resulting in symbolic fact q12(v_q12_1)

		v_q12_1 = v_icewine_1;
		if(q12_set.insert(v_q12_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(116)) return -1;
		if(!backtrack_stack.push(115)) return -1;
		if(!backtrack_stack.push(114)) return -1;
		if(!backtrack_stack.push(99)) return -1;
		goto label_95;
		}
		break;
	case 768: label_768: 
	// using  symbolic fact icewine(v_icewine_1) with literal 1 in rule 
	// 256: icewine(A) :- icewine(B), kaon2equal(A,B).
	// resulting in symbolic fact icewine(v_icewine_1)

		//recursive
		c256_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_icewine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2794)) return -1;
		c256_2_fb.open(v_icewine_1);
	case 1781:
	// Continuation for label 768
		while (c256_2_fb.fetch()) {
			v_icewine_1 = c256_2_fb.c2();
		if(icewine_set.insert(v_icewine_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1781)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(771)) return -1;
			if(!backtrack_stack.push(769)) return -1;
			if(!backtrack_stack.push(768)) return -1;
			goto label_767;
		}
		}
		break;
	case 2794:
	// Label to restore values of label 768
		v_icewine_1 = int_value_stack.pop();
		c256_2_fb.pop(&cursor_stack);
		break;
	case 769: label_769: 
	// using  symbolic fact icewine(v_icewine_1) with literal 1 in rule 
	// 461: dessertwine(A) :- icewine(A).
	// resulting in symbolic fact dessertwine(v_dessertwine_1)

		//recursive
		if(!int_value_stack.push(v_dessertwine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2795)) return -1;
		v_dessertwine_1 = v_icewine_1;
		if(dessertwine_set.insert(v_dessertwine_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(770)) return -1;
		if(!backtrack_stack.push(753)) return -1;
		goto label_752;
		}
		break;
	case 2795:
	// Label to restore values of label 769
		v_dessertwine_1 = int_value_stack.pop();
		break;
	case 770: label_770: 
	// using  symbolic fact dessertwine(v_dessertwine_1) with literal 2 in rule 
	// 257: icewine(A) :- q21(A), dessertwine(A), lateharvest(A).
	// resulting in symbolic fact icewine(v_icewine_1)

		//recursive
		if(!int_value_stack.push(v_icewine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2796)) return -1;
		if (!q21_set.contains(v_dessertwine_1)) continue;
			if (!lateharvest_set.contains(v_dessertwine_1)) continue;
	case 1783:
	// Continuation for label 770
				v_icewine_1 = v_dessertwine_1;
		if(icewine_set.insert(v_icewine_1)) {
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1783)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(771)) return -1;
				if(!backtrack_stack.push(769)) return -1;
				if(!backtrack_stack.push(768)) return -1;
				goto label_767;
		}
		break;
	case 2796:
	// Label to restore values of label 770
		v_icewine_1 = int_value_stack.pop();
		break;
	case 771: label_771: 
	// using  symbolic fact icewine(v_icewine_1) with literal 1 in rule 
	// 484: lateharvest(A) :- icewine(A).
	// resulting in symbolic fact lateharvest(v_lateharvest_1)

		//recursive
		if(!int_value_stack.push(v_lateharvest_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2797)) return -1;
		v_lateharvest_1 = v_icewine_1;
		if(lateharvest_set.insert(v_lateharvest_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(766)) return -1;
		if(!backtrack_stack.push(765)) return -1;
		if(!backtrack_stack.push(764)) return -1;
		goto label_763;
		}
		break;
	case 2797:
	// Label to restore values of label 771
		v_lateharvest_1 = int_value_stack.pop();
		break;
	case 772: label_772: 
	// using  symbolic fact sauternes(v_sauternes_1) with literal 1 in rule 
	// 597: q12(A) :- sauternes(A).
	// resulting in symbolic fact q12(v_q12_1)

		v_q12_1 = v_sauternes_1;
		if(q12_set.insert(v_q12_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(116)) return -1;
		if(!backtrack_stack.push(115)) return -1;
		if(!backtrack_stack.push(114)) return -1;
		if(!backtrack_stack.push(99)) return -1;
		goto label_95;
		}
		break;
	case 773:
	// applying rule 
	// 211: stemilion(A) :- stemilion_aux(A).
	// resulting in symbolic fact stemilion(v_stemilion_1)
		c211_1_f.open();
	case 1786:
	// Continuation for label 773
		while (c211_1_f.fetch()) {
			v_stemilion_1 = c211_1_f.c1();
		if(stemilion_set.insert(v_stemilion_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1786)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(784)) return -1;
			if(!backtrack_stack.push(783)) return -1;
			if(!backtrack_stack.push(782)) return -1;
			if(!backtrack_stack.push(780)) return -1;
			if(!backtrack_stack.push(779)) return -1;
			if(!backtrack_stack.push(778)) return -1;
			goto label_774;
		}
		}
		break;
	case 774: label_774: 
	// using  symbolic fact stemilion(v_stemilion_1) with literal 1 in rule 
	// 210: q64(A) :- stemilion(A).
	// resulting in symbolic fact q64(v_q64_1)

		//recursive
		if(!int_value_stack.push(v_q64_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2800)) return -1;
		v_q64_1 = v_stemilion_1;
		if(q64_set.insert(v_q64_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(790)) return -1;
		if(!backtrack_stack.push(787)) return -1;
		if(!backtrack_stack.push(776)) return -1;
		goto label_775;
		}
		break;
	case 2800:
	// Label to restore values of label 774
		v_q64_1 = int_value_stack.pop();
		break;
	case 775: label_775: 
	// using  symbolic fact q64(v_q64_1) with literal 1 in rule 
	// 209: locatedin(A,A) :- q64(A), kaon2namedobjects(A).
	// resulting in symbolic fact locatedin(v_locatedin_1,v_locatedin_2)

		if (!kaon2namedobjects_b.contains(v_q64_1)) continue;
	case 1788:
	// Continuation for label 775
			v_locatedin_1 = v_q64_1;
			v_locatedin_2 = v_q64_1;
		if(locatedin_set.insert(v_locatedin_1, v_locatedin_2)) {
			if(!locatedin_bf.insert(v_locatedin_1, v_locatedin_2)) return -1;
			if(!locatedin_fb.insert(v_locatedin_2, v_locatedin_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1788)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(1005)) return -1;
			if(!backtrack_stack.push(999)) return -1;
			if(!backtrack_stack.push(987)) return -1;
			if(!backtrack_stack.push(981)) return -1;
			if(!backtrack_stack.push(969)) return -1;
			if(!backtrack_stack.push(963)) return -1;
			if(!backtrack_stack.push(952)) return -1;
			if(!backtrack_stack.push(946)) return -1;
			if(!backtrack_stack.push(936)) return -1;
			if(!backtrack_stack.push(930)) return -1;
			if(!backtrack_stack.push(920)) return -1;
			if(!backtrack_stack.push(917)) return -1;
			if(!backtrack_stack.push(898)) return -1;
			if(!backtrack_stack.push(897)) return -1;
			if(!backtrack_stack.push(896)) return -1;
			if(!backtrack_stack.push(893)) return -1;
			if(!backtrack_stack.push(726)) return -1;
			if(!backtrack_stack.push(721)) return -1;
			if(!backtrack_stack.push(694)) return -1;
			if(!backtrack_stack.push(693)) return -1;
			if(!backtrack_stack.push(688)) return -1;
			if(!backtrack_stack.push(687)) return -1;
			if(!backtrack_stack.push(636)) return -1;
			if(!backtrack_stack.push(631)) return -1;
			if(!backtrack_stack.push(592)) return -1;
			if(!backtrack_stack.push(591)) return -1;
			if(!backtrack_stack.push(529)) return -1;
			if(!backtrack_stack.push(528)) return -1;
			if(!backtrack_stack.push(513)) return -1;
			if(!backtrack_stack.push(512)) return -1;
			if(!backtrack_stack.push(511)) return -1;
			if(!backtrack_stack.push(506)) return -1;
			if(!backtrack_stack.push(455)) return -1;
			if(!backtrack_stack.push(450)) return -1;
			if(!backtrack_stack.push(441)) return -1;
			if(!backtrack_stack.push(440)) return -1;
			if(!backtrack_stack.push(439)) return -1;
			if(!backtrack_stack.push(434)) return -1;
			if(!backtrack_stack.push(339)) return -1;
			if(!backtrack_stack.push(338)) return -1;
			if(!backtrack_stack.push(249)) return -1;
			if(!backtrack_stack.push(244)) return -1;
			if(!backtrack_stack.push(52)) return -1;
			if(!backtrack_stack.push(51)) return -1;
			if(!backtrack_stack.push(50)) return -1;
			if(!backtrack_stack.push(49)) return -1;
			if(!backtrack_stack.push(48)) return -1;
			goto label_47;
		}
		break;
	case 776: label_776: 
	// using  symbolic fact q64(v_q64_1) with literal 1 in rule 
	// 213: q65(A) :- q64(A).
	// resulting in symbolic fact q65(v_q65_1)

		//recursive
		if(!int_value_stack.push(v_q65_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2802)) return -1;
		v_q65_1 = v_q64_1;
		if(q65_set.insert(v_q65_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(786)) return -1;
		if(!backtrack_stack.push(785)) return -1;
		goto label_777;
		}
		break;
	case 2802:
	// Label to restore values of label 776
		v_q65_1 = int_value_stack.pop();
		break;
	case 777: label_777: 
	// using  symbolic fact q65(v_q65_1) with literal 1 in rule 
	// 212: stemilion(A) :- q65(A), bordeaux(A).
	// resulting in symbolic fact stemilion(v_stemilion_1)

		//recursive
		if(!int_value_stack.push(v_stemilion_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2803)) return -1;
		if (!bordeaux_set.contains(v_q65_1)) continue;
	case 1790:
	// Continuation for label 777
			v_stemilion_1 = v_q65_1;
		if(stemilion_set.insert(v_stemilion_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1790)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(784)) return -1;
			if(!backtrack_stack.push(783)) return -1;
			if(!backtrack_stack.push(782)) return -1;
			if(!backtrack_stack.push(780)) return -1;
			if(!backtrack_stack.push(779)) return -1;
			if(!backtrack_stack.push(778)) return -1;
			goto label_774;
		}
		break;
	case 2803:
	// Label to restore values of label 777
		v_stemilion_1 = int_value_stack.pop();
		break;
	case 778: label_778: 
	// using  symbolic fact stemilion(v_stemilion_1) with literal 1 in rule 
	// 282: q11(A) :- stemilion(A).
	// resulting in symbolic fact q11(v_q11_1)

		v_q11_1 = v_stemilion_1;
		if(q11_set.insert(v_q11_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(681)) return -1;
		if(!backtrack_stack.push(680)) return -1;
		goto label_679;
		}
		break;
	case 779: label_779: 
	// using  symbolic fact stemilion(v_stemilion_1) with literal 1 in rule 
	// 283: stemilion(A) :- stemilion(B), kaon2equal(A,B).
	// resulting in symbolic fact stemilion(v_stemilion_1)

		//recursive
		c283_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_stemilion_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2805)) return -1;
		c283_2_fb.open(v_stemilion_1);
	case 1792:
	// Continuation for label 779
		while (c283_2_fb.fetch()) {
			v_stemilion_1 = c283_2_fb.c2();
		if(stemilion_set.insert(v_stemilion_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1792)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(784)) return -1;
			if(!backtrack_stack.push(783)) return -1;
			if(!backtrack_stack.push(782)) return -1;
			if(!backtrack_stack.push(780)) return -1;
			if(!backtrack_stack.push(779)) return -1;
			if(!backtrack_stack.push(778)) return -1;
			goto label_774;
		}
		}
		break;
	case 2805:
	// Label to restore values of label 779
		v_stemilion_1 = int_value_stack.pop();
		c283_2_fb.pop(&cursor_stack);
		break;
	case 780: label_780: 
	// using  symbolic fact stemilion(v_stemilion_1) with literal 1 in rule 
	// 303: bordeaux(A) :- stemilion(A).
	// resulting in symbolic fact bordeaux(v_bordeaux_1)

		//recursive
		if(!int_value_stack.push(v_bordeaux_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2806)) return -1;
		v_bordeaux_1 = v_stemilion_1;
		if(bordeaux_set.insert(v_bordeaux_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(781)) return -1;
		if(!backtrack_stack.push(566)) return -1;
		if(!backtrack_stack.push(565)) return -1;
		if(!backtrack_stack.push(564)) return -1;
		if(!backtrack_stack.push(562)) return -1;
		if(!backtrack_stack.push(558)) return -1;
		goto label_526;
		}
		break;
	case 2806:
	// Label to restore values of label 780
		v_bordeaux_1 = int_value_stack.pop();
		break;
	case 781: label_781: 
	// using  symbolic fact bordeaux(v_bordeaux_1) with literal 2 in rule 
	// 212: stemilion(A) :- q65(A), bordeaux(A).
	// resulting in symbolic fact stemilion(v_stemilion_1)

		//recursive
		if(!int_value_stack.push(v_stemilion_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2807)) return -1;
		if (!q65_set.contains(v_bordeaux_1)) continue;
	case 1794:
	// Continuation for label 781
			v_stemilion_1 = v_bordeaux_1;
		if(stemilion_set.insert(v_stemilion_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1794)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(784)) return -1;
			if(!backtrack_stack.push(783)) return -1;
			if(!backtrack_stack.push(782)) return -1;
			if(!backtrack_stack.push(780)) return -1;
			if(!backtrack_stack.push(779)) return -1;
			if(!backtrack_stack.push(778)) return -1;
			goto label_774;
		}
		break;
	case 2807:
	// Label to restore values of label 781
		v_stemilion_1 = int_value_stack.pop();
		break;
	case 782: label_782: 
	// using  symbolic fact stemilion(v_stemilion_1) with literal 1 in rule 
	// 464: q69(A) :- stemilion(A).
	// resulting in symbolic fact q69(v_q69_1)

		v_q69_1 = v_stemilion_1;
		if(q69_set.insert(v_q69_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(666)) return -1;
		if(!backtrack_stack.push(665)) return -1;
		if(!backtrack_stack.push(664)) return -1;
		if(!backtrack_stack.push(663)) return -1;
		if(!backtrack_stack.push(662)) return -1;
		goto label_659;
		}
		break;
	case 783: label_783: 
	// using  symbolic fact stemilion(v_stemilion_1) with literal 1 in rule 
	// 609: q0(A) :- stemilion(A).
	// resulting in symbolic fact q0(v_q0_1)

		v_q0_1 = v_stemilion_1;
		if(q0_set.insert(v_q0_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(198)) return -1;
		if(!backtrack_stack.push(194)) return -1;
		if(!backtrack_stack.push(189)) return -1;
		if(!backtrack_stack.push(188)) return -1;
		goto label_187;
		}
		break;
	case 784: label_784: 
	// using  symbolic fact stemilion(v_stemilion_1) with literal 1 in rule 
	// 779: q3(A) :- stemilion(A).
	// resulting in symbolic fact q3(v_q3_1)

		v_q3_1 = v_stemilion_1;
		if(q3_set.insert(v_q3_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(143)) return -1;
		goto label_123;
		}
		break;
	case 785: label_785: 
	// using  symbolic fact q65(v_q65_1) with literal 1 in rule 
	// 227: q65(A) :- q65(B), kaon2equal(A,B).
	// resulting in symbolic fact q65(v_q65_1)

		//recursive
		c227_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q65_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2811)) return -1;
		c227_2_fb.open(v_q65_1);
	case 1798:
	// Continuation for label 785
		while (c227_2_fb.fetch()) {
			v_q65_1 = c227_2_fb.c2();
		if(q65_set.insert(v_q65_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1798)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(786)) return -1;
			if(!backtrack_stack.push(785)) return -1;
			goto label_777;
		}
		}
		break;
	case 2811:
	// Label to restore values of label 785
		v_q65_1 = int_value_stack.pop();
		c227_2_fb.pop(&cursor_stack);
		break;
	case 786: label_786: 
	// using  symbolic fact q65(v_q65_1) with literal 2 in rule 
	// 214: q64(A) :- locatedin(A,B), q65(B).
	// resulting in symbolic fact q64(v_q64_1)

		//recursive
		c214_1_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q64_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2812)) return -1;
		c214_1_fb.open(v_q65_1);
	case 1799:
	// Continuation for label 786
		while (c214_1_fb.fetch()) {
			v_q64_1 = c214_1_fb.c2();
		if(q64_set.insert(v_q64_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1799)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(790)) return -1;
			if(!backtrack_stack.push(787)) return -1;
			if(!backtrack_stack.push(776)) return -1;
			goto label_775;
		}
		}
		break;
	case 2812:
	// Label to restore values of label 786
		v_q64_1 = int_value_stack.pop();
		c214_1_fb.pop(&cursor_stack);
		break;
	case 787: label_787: 
	// using  symbolic fact q64(v_q64_1) with literal 1 in rule 
	// 224: ot____nom36(A) :- q64(A), kaon2namedobjects(A).
	// resulting in symbolic fact ot____nom36(v_ot____nom36_1)

		//recursive
		if(!int_value_stack.push(v_ot____nom36_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2813)) return -1;
		if (!kaon2namedobjects_b.contains(v_q64_1)) continue;
	case 1800:
	// Continuation for label 787
			v_ot____nom36_1 = v_q64_1;
		if(ot____nom36_set.insert(v_ot____nom36_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1800)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(789)) return -1;
			goto label_788;
		}
		break;
	case 2813:
	// Label to restore values of label 787
		v_ot____nom36_1 = int_value_stack.pop();
		break;
	case 788: label_788: 
	// using  symbolic fact ot____nom36(v_ot____nom36_1) with literal 1 in rule 
	// 226: ot____nom36(A) :- ot____nom36(B), kaon2equal(A,B).
	// resulting in symbolic fact ot____nom36(v_ot____nom36_1)

		//recursive
		c226_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_ot____nom36_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2814)) return -1;
		c226_2_fb.open(v_ot____nom36_1);
	case 1801:
	// Continuation for label 788
		while (c226_2_fb.fetch()) {
			v_ot____nom36_1 = c226_2_fb.c2();
		if(ot____nom36_set.insert(v_ot____nom36_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1801)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(789)) return -1;
			goto label_788;
		}
		}
		break;
	case 2814:
	// Label to restore values of label 788
		v_ot____nom36_1 = int_value_stack.pop();
		c226_2_fb.pop(&cursor_stack);
		break;
	case 789: label_789: 
	// using  symbolic fact ot____nom36(v_ot____nom36_1) with literal 2 in rule 
	// 222: q65(A) :- locatedin(A,B), ot____nom36(B).
	// resulting in symbolic fact q65(v_q65_1)

		//recursive
		c222_1_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q65_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2815)) return -1;
		c222_1_fb.open(v_ot____nom36_1);
	case 1802:
	// Continuation for label 789
		while (c222_1_fb.fetch()) {
			v_q65_1 = c222_1_fb.c2();
		if(q65_set.insert(v_q65_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1802)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(786)) return -1;
			if(!backtrack_stack.push(785)) return -1;
			goto label_777;
		}
		}
		break;
	case 2815:
	// Label to restore values of label 789
		v_q65_1 = int_value_stack.pop();
		c222_1_fb.pop(&cursor_stack);
		break;
	case 790: label_790: 
	// using  symbolic fact q64(v_q64_1) with literal 1 in rule 
	// 225: q64(A) :- q64(B), kaon2equal(A,B).
	// resulting in symbolic fact q64(v_q64_1)

		//recursive
		c225_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q64_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2816)) return -1;
		c225_2_fb.open(v_q64_1);
	case 1803:
	// Continuation for label 790
		while (c225_2_fb.fetch()) {
			v_q64_1 = c225_2_fb.c2();
		if(q64_set.insert(v_q64_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1803)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(790)) return -1;
			if(!backtrack_stack.push(787)) return -1;
			if(!backtrack_stack.push(776)) return -1;
			goto label_775;
		}
		}
		break;
	case 2816:
	// Label to restore values of label 790
		v_q64_1 = int_value_stack.pop();
		c225_2_fb.pop(&cursor_stack);
		break;
	case 791:
	// applying rule 
	// 223: ot____nom36(A) :- ot____nom36_aux(A).
	// resulting in symbolic fact ot____nom36(v_ot____nom36_1)
		c223_1_f.open();
	case 1804:
	// Continuation for label 791
		while (c223_1_f.fetch()) {
			v_ot____nom36_1 = c223_1_f.c1();
		if(ot____nom36_set.insert(v_ot____nom36_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1804)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(789)) return -1;
			goto label_788;
		}
		}
		break;
	case 792:
	// applying rule 
	// 244: cabernetsauvignon(A) :- cabernetsauvignon_aux(A).
	// resulting in symbolic fact cabernetsauvignon(v_cabernetsauvignon_1)
		c244_1_f.open();
	case 1805:
	// Continuation for label 792
		while (c244_1_f.fetch()) {
			v_cabernetsauvignon_1 = c244_1_f.c1();
		if(cabernetsauvignon_set.insert(v_cabernetsauvignon_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1805)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(673)) return -1;
			if(!backtrack_stack.push(671)) return -1;
			if(!backtrack_stack.push(670)) return -1;
			if(!backtrack_stack.push(669)) return -1;
			if(!backtrack_stack.push(661)) return -1;
			goto label_660;
		}
		}
		break;
	case 793:
	// applying rule 
	// 255: icewine(A) :- icewine_aux(A).
	// resulting in symbolic fact icewine(v_icewine_1)
		c255_1_f.open();
	case 1806:
	// Continuation for label 793
		while (c255_1_f.fetch()) {
			v_icewine_1 = c255_1_f.c1();
		if(icewine_set.insert(v_icewine_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1806)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(771)) return -1;
			if(!backtrack_stack.push(769)) return -1;
			if(!backtrack_stack.push(768)) return -1;
			goto label_767;
		}
		}
		break;
	case 794:
	// applying rule 
	// 273: merlot(A) :- merlot_aux(A).
	// resulting in symbolic fact merlot(v_merlot_1)
		c273_1_f.open();
	case 1807:
	// Continuation for label 794
		while (c273_1_f.fetch()) {
			v_merlot_1 = c273_1_f.c1();
		if(merlot_set.insert(v_merlot_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1807)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(550)) return -1;
			if(!backtrack_stack.push(548)) return -1;
			if(!backtrack_stack.push(547)) return -1;
			if(!backtrack_stack.push(546)) return -1;
			if(!backtrack_stack.push(538)) return -1;
			goto label_537;
		}
		}
		break;
	case 795:
	// applying rule 
	// 292: sweetriesling(A) :- sweetriesling_aux(A).
	// resulting in symbolic fact sweetriesling(v_sweetriesling_1)
		c292_1_f.open();
	case 1808:
	// Continuation for label 795
		while (c292_1_f.fetch()) {
			v_sweetriesling_1 = c292_1_f.c1();
		if(sweetriesling_set.insert(v_sweetriesling_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1808)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(751)) return -1;
			if(!backtrack_stack.push(749)) return -1;
			if(!backtrack_stack.push(748)) return -1;
			if(!backtrack_stack.push(747)) return -1;
			goto label_746;
		}
		}
		break;
	case 796:
	// applying rule 
	// 296: whitewine(A) :- whitewine_aux(A).
	// resulting in symbolic fact whitewine(v_whitewine_1)
		c296_1_f.open();
	case 1809:
	// Continuation for label 796
		while (c296_1_f.fetch()) {
			v_whitewine_1 = c296_1_f.c1();
		if(whitewine_set.insert(v_whitewine_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1809)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(560)) return -1;
			if(!backtrack_stack.push(491)) return -1;
			if(!backtrack_stack.push(374)) return -1;
			if(!backtrack_stack.push(235)) return -1;
			if(!backtrack_stack.push(113)) return -1;
			if(!backtrack_stack.push(112)) return -1;
			if(!backtrack_stack.push(104)) return -1;
			if(!backtrack_stack.push(103)) return -1;
			if(!backtrack_stack.push(101)) return -1;
			if(!backtrack_stack.push(98)) return -1;
			goto label_97;
		}
		}
		break;
	case 797:
	// applying rule 
	// 310: ot____nom23(A) :- ot____nom23_aux(A).
	// resulting in symbolic fact ot____nom23(v_ot____nom23_1)
		c310_1_f.open();
	case 1810:
	// Continuation for label 797
		while (c310_1_f.fetch()) {
			v_ot____nom23_1 = c310_1_f.c1();
		if(ot____nom23_set.insert(v_ot____nom23_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1810)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(588)) return -1;
			goto label_587;
		}
		}
		break;
	case 798:
	// applying rule 
	// 322: pinotnoir(A) :- pinotnoir_aux(A).
	// resulting in symbolic fact pinotnoir(v_pinotnoir_1)
		c322_1_f.open();
	case 1811:
	// Continuation for label 798
		while (c322_1_f.fetch()) {
			v_pinotnoir_1 = c322_1_f.c1();
		if(pinotnoir_set.insert(v_pinotnoir_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1811)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(400)) return -1;
			if(!backtrack_stack.push(398)) return -1;
			if(!backtrack_stack.push(397)) return -1;
			if(!backtrack_stack.push(389)) return -1;
			goto label_388;
		}
		}
		break;
	case 799:
	// applying rule 
	// 330: cabernetfranc(A) :- cabernetfranc_aux(A).
	// resulting in symbolic fact cabernetfranc(v_cabernetfranc_1)
		c330_1_f.open();
	case 1812:
	// Continuation for label 799
		while (c330_1_f.fetch()) {
			v_cabernetfranc_1 = c330_1_f.c1();
		if(cabernetfranc_set.insert(v_cabernetfranc_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1812)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(810)) return -1;
			if(!backtrack_stack.push(809)) return -1;
			if(!backtrack_stack.push(806)) return -1;
			if(!backtrack_stack.push(805)) return -1;
			if(!backtrack_stack.push(804)) return -1;
			if(!backtrack_stack.push(803)) return -1;
			if(!backtrack_stack.push(802)) return -1;
			if(!backtrack_stack.push(801)) return -1;
			goto label_800;
		}
		}
		break;
	case 800: label_800: 
	// using  symbolic fact cabernetfranc(v_cabernetfranc_1) with literal 1 in rule 
	// 329: wine(A) :- cabernetfranc(A).
	// resulting in symbolic fact wine(v_wine_1)

		v_wine_1 = v_cabernetfranc_1;
		if(wine_set.insert(v_wine_1)) {
		if(!wine_f.insert(v_wine_1)) return -1;
		// Labels for next rules
		if(!backtrack_stack.push(1007)) return -1;
		if(!backtrack_stack.push(989)) return -1;
		if(!backtrack_stack.push(954)) return -1;
		if(!backtrack_stack.push(938)) return -1;
		if(!backtrack_stack.push(911)) return -1;
		if(!backtrack_stack.push(903)) return -1;
		if(!backtrack_stack.push(902)) return -1;
		if(!backtrack_stack.push(901)) return -1;
		if(!backtrack_stack.push(849)) return -1;
		if(!backtrack_stack.push(848)) return -1;
		if(!backtrack_stack.push(821)) return -1;
		if(!backtrack_stack.push(820)) return -1;
		if(!backtrack_stack.push(737)) return -1;
		if(!backtrack_stack.push(712)) return -1;
		if(!backtrack_stack.push(674)) return -1;
		if(!backtrack_stack.push(672)) return -1;
		if(!backtrack_stack.push(583)) return -1;
		if(!backtrack_stack.push(575)) return -1;
		if(!backtrack_stack.push(563)) return -1;
		if(!backtrack_stack.push(551)) return -1;
		if(!backtrack_stack.push(549)) return -1;
		if(!backtrack_stack.push(483)) return -1;
		if(!backtrack_stack.push(473)) return -1;
		if(!backtrack_stack.push(470)) return -1;
		if(!backtrack_stack.push(430)) return -1;
		if(!backtrack_stack.push(401)) return -1;
		if(!backtrack_stack.push(399)) return -1;
		if(!backtrack_stack.push(380)) return -1;
		if(!backtrack_stack.push(364)) return -1;
		if(!backtrack_stack.push(363)) return -1;
		if(!backtrack_stack.push(329)) return -1;
		if(!backtrack_stack.push(328)) return -1;
		if(!backtrack_stack.push(327)) return -1;
		if(!backtrack_stack.push(295)) return -1;
		if(!backtrack_stack.push(294)) return -1;
		if(!backtrack_stack.push(216)) return -1;
		if(!backtrack_stack.push(210)) return -1;
		if(!backtrack_stack.push(192)) return -1;
		if(!backtrack_stack.push(180)) return -1;
		if(!backtrack_stack.push(179)) return -1;
		if(!backtrack_stack.push(178)) return -1;
		if(!backtrack_stack.push(154)) return -1;
		if(!backtrack_stack.push(153)) return -1;
		if(!backtrack_stack.push(100)) return -1;
		if(!backtrack_stack.push(76)) return -1;
		if(!backtrack_stack.push(75)) return -1;
		if(!backtrack_stack.push(67)) return -1;
		if(!backtrack_stack.push(63)) return -1;
		if(!backtrack_stack.push(53)) return -1;
		if(!backtrack_stack.push(46)) return -1;
		if(!backtrack_stack.push(19)) return -1;
		goto label_16;
		}
		break;
	case 801: label_801: 
	// using  symbolic fact cabernetfranc(v_cabernetfranc_1) with literal 1 in rule 
	// 331: cabernetfranc(A) :- cabernetfranc(B), kaon2equal(A,B).
	// resulting in symbolic fact cabernetfranc(v_cabernetfranc_1)

		//recursive
		c331_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_cabernetfranc_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2827)) return -1;
		c331_2_fb.open(v_cabernetfranc_1);
	case 1814:
	// Continuation for label 801
		while (c331_2_fb.fetch()) {
			v_cabernetfranc_1 = c331_2_fb.c2();
		if(cabernetfranc_set.insert(v_cabernetfranc_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1814)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(810)) return -1;
			if(!backtrack_stack.push(809)) return -1;
			if(!backtrack_stack.push(806)) return -1;
			if(!backtrack_stack.push(805)) return -1;
			if(!backtrack_stack.push(804)) return -1;
			if(!backtrack_stack.push(803)) return -1;
			if(!backtrack_stack.push(802)) return -1;
			if(!backtrack_stack.push(801)) return -1;
			goto label_800;
		}
		}
		break;
	case 2827:
	// Label to restore values of label 801
		v_cabernetfranc_1 = int_value_stack.pop();
		c331_2_fb.pop(&cursor_stack);
		break;
	case 802: label_802: 
	// using  symbolic fact cabernetfranc(v_cabernetfranc_1) with literal 1 in rule 
	// 407: q7(A) :- cabernetfranc(A).
	// resulting in symbolic fact q7(v_q7_1)

		v_q7_1 = v_cabernetfranc_1;
		if(q7_set.insert(v_q7_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(622)) return -1;
		if(!backtrack_stack.push(621)) return -1;
		goto label_620;
		}
		break;
	case 803: label_803: 
	// using  symbolic fact cabernetfranc(v_cabernetfranc_1) with literal 1 in rule 
	// 439: q0(A) :- cabernetfranc(A).
	// resulting in symbolic fact q0(v_q0_1)

		v_q0_1 = v_cabernetfranc_1;
		if(q0_set.insert(v_q0_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(198)) return -1;
		if(!backtrack_stack.push(194)) return -1;
		if(!backtrack_stack.push(189)) return -1;
		if(!backtrack_stack.push(188)) return -1;
		goto label_187;
		}
		break;
	case 804: label_804: 
	// using  symbolic fact cabernetfranc(v_cabernetfranc_1) with literal 1 in rule 
	// 490: q4(A) :- cabernetfranc(A).
	// resulting in symbolic fact q4(v_q4_1)

		v_q4_1 = v_cabernetfranc_1;
		if(q4_set.insert(v_q4_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(90)) return -1;
		if(!backtrack_stack.push(89)) return -1;
		goto label_88;
		}
		break;
	case 805: label_805: 
	// using  symbolic fact cabernetfranc(v_cabernetfranc_1) with literal 1 in rule 
	// 525: madefromgrape(A,A) :- cabernetfranc(A), kaon2namedobjects(A).
	// resulting in symbolic fact madefromgrape(v_madefromgrape_1,v_madefromgrape_2)

		//recursive
		if(!int_value_stack.push(v_madefromgrape_2)) return -1;
		if(!int_value_stack.push(v_madefromgrape_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2831)) return -1;
		if (!kaon2namedobjects_b.contains(v_cabernetfranc_1)) continue;
	case 1818:
	// Continuation for label 805
			v_madefromgrape_1 = v_cabernetfranc_1;
			v_madefromgrape_2 = v_cabernetfranc_1;
		if(madefromgrape_set.insert(v_madefromgrape_1, v_madefromgrape_2)) {
			if(!madefromgrape_bf.insert(v_madefromgrape_1, v_madefromgrape_2)) return -1;
			if(!madefromgrape_fb.insert(v_madefromgrape_2, v_madefromgrape_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1818)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(860)) return -1;
			if(!backtrack_stack.push(859)) return -1;
			if(!backtrack_stack.push(845)) return -1;
			if(!backtrack_stack.push(844)) return -1;
			if(!backtrack_stack.push(829)) return -1;
			if(!backtrack_stack.push(828)) return -1;
			if(!backtrack_stack.push(813)) return -1;
			if(!backtrack_stack.push(812)) return -1;
			if(!backtrack_stack.push(677)) return -1;
			if(!backtrack_stack.push(676)) return -1;
			if(!backtrack_stack.push(617)) return -1;
			if(!backtrack_stack.push(616)) return -1;
			if(!backtrack_stack.push(609)) return -1;
			if(!backtrack_stack.push(554)) return -1;
			if(!backtrack_stack.push(553)) return -1;
			if(!backtrack_stack.push(476)) return -1;
			if(!backtrack_stack.push(475)) return -1;
			if(!backtrack_stack.push(404)) return -1;
			if(!backtrack_stack.push(403)) return -1;
			if(!backtrack_stack.push(362)) return -1;
			if(!backtrack_stack.push(361)) return -1;
			if(!backtrack_stack.push(343)) return -1;
			if(!backtrack_stack.push(342)) return -1;
			if(!backtrack_stack.push(300)) return -1;
			if(!backtrack_stack.push(170)) return -1;
			if(!backtrack_stack.push(169)) return -1;
			if(!backtrack_stack.push(151)) return -1;
			if(!backtrack_stack.push(150)) return -1;
			if(!backtrack_stack.push(144)) return -1;
			if(!backtrack_stack.push(132)) return -1;
			if(!backtrack_stack.push(71)) return -1;
			if(!backtrack_stack.push(70)) return -1;
			if(!backtrack_stack.push(66)) return -1;
			goto label_64;
		}
		break;
	case 2831:
	// Label to restore values of label 805
		v_madefromgrape_1 = int_value_stack.pop();
		v_madefromgrape_2 = int_value_stack.pop();
		break;
	case 806: label_806: 
	// using  symbolic fact cabernetfranc(v_cabernetfranc_1) with literal 1 in rule 
	// 532: ot____nom3(A) :- cabernetfranc(A), kaon2namedobjects(A).
	// resulting in symbolic fact ot____nom3(v_ot____nom3_1)

		//recursive
		if(!int_value_stack.push(v_ot____nom3_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2832)) return -1;
		if (!kaon2namedobjects_b.contains(v_cabernetfranc_1)) continue;
	case 1819:
	// Continuation for label 806
			v_ot____nom3_1 = v_cabernetfranc_1;
		if(ot____nom3_set.insert(v_ot____nom3_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1819)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(811)) return -1;
			if(!backtrack_stack.push(808)) return -1;
			goto label_807;
		}
		break;
	case 2832:
	// Label to restore values of label 806
		v_ot____nom3_1 = int_value_stack.pop();
		break;
	case 807: label_807: 
	// using  symbolic fact ot____nom3(v_ot____nom3_1) with literal 1 in rule 
	// 533: ot____nom3(A) :- ot____nom3(B), kaon2equal(A,B).
	// resulting in symbolic fact ot____nom3(v_ot____nom3_1)

		//recursive
		c533_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_ot____nom3_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2833)) return -1;
		c533_2_fb.open(v_ot____nom3_1);
	case 1820:
	// Continuation for label 807
		while (c533_2_fb.fetch()) {
			v_ot____nom3_1 = c533_2_fb.c2();
		if(ot____nom3_set.insert(v_ot____nom3_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1820)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(811)) return -1;
			if(!backtrack_stack.push(808)) return -1;
			goto label_807;
		}
		}
		break;
	case 2833:
	// Label to restore values of label 807
		v_ot____nom3_1 = int_value_stack.pop();
		c533_2_fb.pop(&cursor_stack);
		break;
	case 808: label_808: 
	// using  symbolic fact ot____nom3(v_ot____nom3_1) with literal 3 in rule 
	// 332: cabernetfranc(A) :- wine(A), madefromgrape(A,B), ot____nom3(B).
	// resulting in symbolic fact cabernetfranc(v_cabernetfranc_1)

		//recursive
		c332_1_f.push(&cursor_stack);
		if(!int_value_stack.push(v_cabernetfranc_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2834)) return -1;
		c332_1_f.open();
		while (c332_1_f.fetch()) {
			if (!madefromgrape_set.contains(c332_1_f.c1(), v_ot____nom3_1)) continue;
	case 1821:
	// Continuation for label 808
				v_cabernetfranc_1 = c332_1_f.c1();
		if(cabernetfranc_set.insert(v_cabernetfranc_1)) {
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1821)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(810)) return -1;
				if(!backtrack_stack.push(809)) return -1;
				if(!backtrack_stack.push(806)) return -1;
				if(!backtrack_stack.push(805)) return -1;
				if(!backtrack_stack.push(804)) return -1;
				if(!backtrack_stack.push(803)) return -1;
				if(!backtrack_stack.push(802)) return -1;
				if(!backtrack_stack.push(801)) return -1;
				goto label_800;
		}
		}
		break;
	case 2834:
	// Label to restore values of label 808
		v_cabernetfranc_1 = int_value_stack.pop();
		c332_1_f.pop(&cursor_stack);
		break;
	case 809: label_809: 
	// using  symbolic fact cabernetfranc(v_cabernetfranc_1) with literal 1 in rule 
	// 636: q31(A) :- cabernetfranc(A).
	// resulting in symbolic fact q31(v_q31_1)

		v_q31_1 = v_cabernetfranc_1;
		if(q31_set.insert(v_q31_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(221)) return -1;
		if(!backtrack_stack.push(220)) return -1;
		if(!backtrack_stack.push(215)) return -1;
		if(!backtrack_stack.push(213)) return -1;
		goto label_201;
		}
		break;
	case 810: label_810: 
	// using  symbolic fact cabernetfranc(v_cabernetfranc_1) with literal 1 in rule 
	// 777: q3(A) :- cabernetfranc(A).
	// resulting in symbolic fact q3(v_q3_1)

		v_q3_1 = v_cabernetfranc_1;
		if(q3_set.insert(v_q3_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(143)) return -1;
		goto label_123;
		}
		break;
	case 811: label_811: 
	// using  symbolic fact ot____nom3(v_ot____nom3_1) with literal 3 in rule 
	// 508: q5(A) :- wine(A), madefromgrape(A,B), ot____nom3(B).
	// resulting in symbolic fact q5(v_q5_1)

		c508_1_f.open();
		while (c508_1_f.fetch()) {
			if (!madefromgrape_set.contains(c508_1_f.c1(), v_ot____nom3_1)) continue;
	case 1824:
	// Continuation for label 811
				v_q5_1 = c508_1_f.c1();
		if(q5_set.insert(v_q5_1)) {
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1824)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(136)) return -1;
				if(!backtrack_stack.push(135)) return -1;
				goto label_131;
		}
		}
		break;
	case 812: label_812: 
	// using  symbolic fact madefromgrape(v_madefromgrape_1,v_madefromgrape_2) with literal 2 in rule 
	// 332: cabernetfranc(A) :- wine(A), madefromgrape(A,B), ot____nom3(B).
	// resulting in symbolic fact cabernetfranc(v_cabernetfranc_1)

		//recursive
		if(!int_value_stack.push(v_cabernetfranc_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2838)) return -1;
		if (!wine_set.contains(v_madefromgrape_1)) continue;
			if (!ot____nom3_set.contains(v_madefromgrape_2)) continue;
	case 1825:
	// Continuation for label 812
				v_cabernetfranc_1 = v_madefromgrape_1;
		if(cabernetfranc_set.insert(v_cabernetfranc_1)) {
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1825)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(810)) return -1;
				if(!backtrack_stack.push(809)) return -1;
				if(!backtrack_stack.push(806)) return -1;
				if(!backtrack_stack.push(805)) return -1;
				if(!backtrack_stack.push(804)) return -1;
				if(!backtrack_stack.push(803)) return -1;
				if(!backtrack_stack.push(802)) return -1;
				if(!backtrack_stack.push(801)) return -1;
				goto label_800;
		}
		break;
	case 2838:
	// Label to restore values of label 812
		v_cabernetfranc_1 = int_value_stack.pop();
		break;
	case 813: label_813: 
	// using  symbolic fact madefromgrape(v_madefromgrape_1,v_madefromgrape_2) with literal 2 in rule 
	// 508: q5(A) :- wine(A), madefromgrape(A,B), ot____nom3(B).
	// resulting in symbolic fact q5(v_q5_1)

		if (!wine_set.contains(v_madefromgrape_1)) continue;
			if (!ot____nom3_set.contains(v_madefromgrape_2)) continue;
	case 1826:
	// Continuation for label 813
				v_q5_1 = v_madefromgrape_1;
		if(q5_set.insert(v_q5_1)) {
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1826)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(136)) return -1;
				if(!backtrack_stack.push(135)) return -1;
				goto label_131;
		}
		break;
	case 814:
	// applying rule 
	// 335: semillon(A) :- semillon_aux(A).
	// resulting in symbolic fact semillon(v_semillon_1)
		c335_1_f.open();
	case 1827:
	// Continuation for label 814
		while (c335_1_f.fetch()) {
			v_semillon_1 = c335_1_f.c1();
		if(semillon_set.insert(v_semillon_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1827)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(826)) return -1;
			if(!backtrack_stack.push(823)) return -1;
			if(!backtrack_stack.push(822)) return -1;
			if(!backtrack_stack.push(819)) return -1;
			if(!backtrack_stack.push(818)) return -1;
			goto label_815;
		}
		}
		break;
	case 815: label_815: 
	// using  symbolic fact semillon(v_semillon_1) with literal 1 in rule 
	// 334: semillonorsauvignonblanc(A) :- semillon(A).
	// resulting in symbolic fact semillonorsauvignonblanc(v_semillonorsauvignonblanc_1)

		v_semillonorsauvignonblanc_1 = v_semillon_1;
		if(semillonorsauvignonblanc_set.insert(v_semillonorsauvignonblanc_1)) {
		if(!semillonorsauvignonblanc_f.insert(v_semillonorsauvignonblanc_1)) return -1;
		// Labels for next rules
		if(!backtrack_stack.push(817)) return -1;
		if(!backtrack_stack.push(816)) return -1;
		if(!backtrack_stack.push(618)) return -1;
		if(!backtrack_stack.push(603)) return -1;
		if(!backtrack_stack.push(580)) return -1;
		if(!backtrack_stack.push(579)) return -1;
		if(!backtrack_stack.push(576)) return -1;
		goto label_574;
		}
		break;
	case 816: label_816: 
	// using  symbolic fact semillonorsauvignonblanc(v_semillonorsauvignonblanc_1) with literal 1 in rule 
	// 344: sauvignonblanc(A) :- semillonorsauvignonblanc(A), madefromgrape(A,B), ot____nom11(B).
	// resulting in symbolic fact sauvignonblanc(v_sauvignonblanc_1)

		c344_2_bf.open(v_semillonorsauvignonblanc_1);
		while (c344_2_bf.fetch()) {
			if (!ot____nom11_set.contains(c344_2_bf.c2())) continue;
	case 1829:
	// Continuation for label 816
				v_sauvignonblanc_1 = v_semillonorsauvignonblanc_1;
		if(sauvignonblanc_set.insert(v_sauvignonblanc_1)) {
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1829)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(614)) return -1;
				if(!backtrack_stack.push(613)) return -1;
				if(!backtrack_stack.push(604)) return -1;
				goto label_602;
		}
		}
		break;
	case 817: label_817: 
	// using  symbolic fact semillonorsauvignonblanc(v_semillonorsauvignonblanc_1) with literal 1 in rule 
	// 542: q5(A) :- semillonorsauvignonblanc(A), madefromgrape(A,B), ot____nom11(B).
	// resulting in symbolic fact q5(v_q5_1)

		c542_2_bf.open(v_semillonorsauvignonblanc_1);
		while (c542_2_bf.fetch()) {
			if (!ot____nom11_set.contains(c542_2_bf.c2())) continue;
	case 1830:
	// Continuation for label 817
				v_q5_1 = v_semillonorsauvignonblanc_1;
		if(q5_set.insert(v_q5_1)) {
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1830)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(136)) return -1;
				if(!backtrack_stack.push(135)) return -1;
				goto label_131;
		}
		}
		break;
	case 818: label_818: 
	// using  symbolic fact semillon(v_semillon_1) with literal 1 in rule 
	// 336: semillon(A) :- semillon(B), kaon2equal(A,B).
	// resulting in symbolic fact semillon(v_semillon_1)

		//recursive
		c336_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_semillon_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2844)) return -1;
		c336_2_fb.open(v_semillon_1);
	case 1831:
	// Continuation for label 818
		while (c336_2_fb.fetch()) {
			v_semillon_1 = c336_2_fb.c2();
		if(semillon_set.insert(v_semillon_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1831)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(826)) return -1;
			if(!backtrack_stack.push(823)) return -1;
			if(!backtrack_stack.push(822)) return -1;
			if(!backtrack_stack.push(819)) return -1;
			if(!backtrack_stack.push(818)) return -1;
			goto label_815;
		}
		}
		break;
	case 2844:
	// Label to restore values of label 818
		v_semillon_1 = int_value_stack.pop();
		c336_2_fb.pop(&cursor_stack);
		break;
	case 819: label_819: 
	// using  symbolic fact semillon(v_semillon_1) with literal 1 in rule 
	// 496: wine(A) :- semillon(A).
	// resulting in symbolic fact wine(v_wine_1)

		v_wine_1 = v_semillon_1;
		if(wine_set.insert(v_wine_1)) {
		if(!wine_f.insert(v_wine_1)) return -1;
		// Labels for next rules
		if(!backtrack_stack.push(1007)) return -1;
		if(!backtrack_stack.push(989)) return -1;
		if(!backtrack_stack.push(954)) return -1;
		if(!backtrack_stack.push(938)) return -1;
		if(!backtrack_stack.push(911)) return -1;
		if(!backtrack_stack.push(903)) return -1;
		if(!backtrack_stack.push(902)) return -1;
		if(!backtrack_stack.push(901)) return -1;
		if(!backtrack_stack.push(849)) return -1;
		if(!backtrack_stack.push(848)) return -1;
		if(!backtrack_stack.push(821)) return -1;
		if(!backtrack_stack.push(820)) return -1;
		if(!backtrack_stack.push(737)) return -1;
		if(!backtrack_stack.push(712)) return -1;
		if(!backtrack_stack.push(674)) return -1;
		if(!backtrack_stack.push(672)) return -1;
		if(!backtrack_stack.push(583)) return -1;
		if(!backtrack_stack.push(575)) return -1;
		if(!backtrack_stack.push(563)) return -1;
		if(!backtrack_stack.push(551)) return -1;
		if(!backtrack_stack.push(549)) return -1;
		if(!backtrack_stack.push(483)) return -1;
		if(!backtrack_stack.push(473)) return -1;
		if(!backtrack_stack.push(470)) return -1;
		if(!backtrack_stack.push(430)) return -1;
		if(!backtrack_stack.push(401)) return -1;
		if(!backtrack_stack.push(399)) return -1;
		if(!backtrack_stack.push(380)) return -1;
		if(!backtrack_stack.push(364)) return -1;
		if(!backtrack_stack.push(363)) return -1;
		if(!backtrack_stack.push(329)) return -1;
		if(!backtrack_stack.push(328)) return -1;
		if(!backtrack_stack.push(327)) return -1;
		if(!backtrack_stack.push(295)) return -1;
		if(!backtrack_stack.push(294)) return -1;
		if(!backtrack_stack.push(216)) return -1;
		if(!backtrack_stack.push(210)) return -1;
		if(!backtrack_stack.push(192)) return -1;
		if(!backtrack_stack.push(180)) return -1;
		if(!backtrack_stack.push(179)) return -1;
		if(!backtrack_stack.push(178)) return -1;
		if(!backtrack_stack.push(154)) return -1;
		if(!backtrack_stack.push(153)) return -1;
		if(!backtrack_stack.push(100)) return -1;
		if(!backtrack_stack.push(76)) return -1;
		if(!backtrack_stack.push(75)) return -1;
		if(!backtrack_stack.push(67)) return -1;
		if(!backtrack_stack.push(63)) return -1;
		if(!backtrack_stack.push(53)) return -1;
		if(!backtrack_stack.push(46)) return -1;
		if(!backtrack_stack.push(19)) return -1;
		goto label_16;
		}
		break;
	case 820: label_820: 
	// using  symbolic fact wine(v_wine_1) with literal 1 in rule 
	// 332: cabernetfranc(A) :- wine(A), madefromgrape(A,B), ot____nom3(B).
	// resulting in symbolic fact cabernetfranc(v_cabernetfranc_1)

		c332_2_bf.open(v_wine_1);
		while (c332_2_bf.fetch()) {
			if (!ot____nom3_set.contains(c332_2_bf.c2())) continue;
	case 1833:
	// Continuation for label 820
				v_cabernetfranc_1 = v_wine_1;
		if(cabernetfranc_set.insert(v_cabernetfranc_1)) {
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1833)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(810)) return -1;
				if(!backtrack_stack.push(809)) return -1;
				if(!backtrack_stack.push(806)) return -1;
				if(!backtrack_stack.push(805)) return -1;
				if(!backtrack_stack.push(804)) return -1;
				if(!backtrack_stack.push(803)) return -1;
				if(!backtrack_stack.push(802)) return -1;
				if(!backtrack_stack.push(801)) return -1;
				goto label_800;
		}
		}
		break;
	case 821: label_821: 
	// using  symbolic fact wine(v_wine_1) with literal 1 in rule 
	// 508: q5(A) :- wine(A), madefromgrape(A,B), ot____nom3(B).
	// resulting in symbolic fact q5(v_q5_1)

		c508_2_bf.open(v_wine_1);
		while (c508_2_bf.fetch()) {
			if (!ot____nom3_set.contains(c508_2_bf.c2())) continue;
	case 1834:
	// Continuation for label 821
				v_q5_1 = v_wine_1;
		if(q5_set.insert(v_q5_1)) {
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1834)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(136)) return -1;
				if(!backtrack_stack.push(135)) return -1;
				goto label_131;
		}
		}
		break;
	case 822: label_822: 
	// using  symbolic fact semillon(v_semillon_1) with literal 1 in rule 
	// 517: madefromgrape(A,A) :- semillon(A), kaon2namedobjects(A).
	// resulting in symbolic fact madefromgrape(v_madefromgrape_1,v_madefromgrape_2)

		//recursive
		if(!int_value_stack.push(v_madefromgrape_2)) return -1;
		if(!int_value_stack.push(v_madefromgrape_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2848)) return -1;
		if (!kaon2namedobjects_b.contains(v_semillon_1)) continue;
	case 1835:
	// Continuation for label 822
			v_madefromgrape_1 = v_semillon_1;
			v_madefromgrape_2 = v_semillon_1;
		if(madefromgrape_set.insert(v_madefromgrape_1, v_madefromgrape_2)) {
			if(!madefromgrape_bf.insert(v_madefromgrape_1, v_madefromgrape_2)) return -1;
			if(!madefromgrape_fb.insert(v_madefromgrape_2, v_madefromgrape_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1835)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(860)) return -1;
			if(!backtrack_stack.push(859)) return -1;
			if(!backtrack_stack.push(845)) return -1;
			if(!backtrack_stack.push(844)) return -1;
			if(!backtrack_stack.push(829)) return -1;
			if(!backtrack_stack.push(828)) return -1;
			if(!backtrack_stack.push(813)) return -1;
			if(!backtrack_stack.push(812)) return -1;
			if(!backtrack_stack.push(677)) return -1;
			if(!backtrack_stack.push(676)) return -1;
			if(!backtrack_stack.push(617)) return -1;
			if(!backtrack_stack.push(616)) return -1;
			if(!backtrack_stack.push(609)) return -1;
			if(!backtrack_stack.push(554)) return -1;
			if(!backtrack_stack.push(553)) return -1;
			if(!backtrack_stack.push(476)) return -1;
			if(!backtrack_stack.push(475)) return -1;
			if(!backtrack_stack.push(404)) return -1;
			if(!backtrack_stack.push(403)) return -1;
			if(!backtrack_stack.push(362)) return -1;
			if(!backtrack_stack.push(361)) return -1;
			if(!backtrack_stack.push(343)) return -1;
			if(!backtrack_stack.push(342)) return -1;
			if(!backtrack_stack.push(300)) return -1;
			if(!backtrack_stack.push(170)) return -1;
			if(!backtrack_stack.push(169)) return -1;
			if(!backtrack_stack.push(151)) return -1;
			if(!backtrack_stack.push(150)) return -1;
			if(!backtrack_stack.push(144)) return -1;
			if(!backtrack_stack.push(132)) return -1;
			if(!backtrack_stack.push(71)) return -1;
			if(!backtrack_stack.push(70)) return -1;
			if(!backtrack_stack.push(66)) return -1;
			goto label_64;
		}
		break;
	case 2848:
	// Label to restore values of label 822
		v_madefromgrape_1 = int_value_stack.pop();
		v_madefromgrape_2 = int_value_stack.pop();
		break;
	case 823: label_823: 
	// using  symbolic fact semillon(v_semillon_1) with literal 1 in rule 
	// 568: ot____nom27(A) :- semillon(A), kaon2namedobjects(A).
	// resulting in symbolic fact ot____nom27(v_ot____nom27_1)

		//recursive
		if(!int_value_stack.push(v_ot____nom27_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2849)) return -1;
		if (!kaon2namedobjects_b.contains(v_semillon_1)) continue;
	case 1836:
	// Continuation for label 823
			v_ot____nom27_1 = v_semillon_1;
		if(ot____nom27_set.insert(v_ot____nom27_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1836)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(827)) return -1;
			if(!backtrack_stack.push(825)) return -1;
			goto label_824;
		}
		break;
	case 2849:
	// Label to restore values of label 823
		v_ot____nom27_1 = int_value_stack.pop();
		break;
	case 824: label_824: 
	// using  symbolic fact ot____nom27(v_ot____nom27_1) with literal 1 in rule 
	// 569: ot____nom27(A) :- ot____nom27(B), kaon2equal(A,B).
	// resulting in symbolic fact ot____nom27(v_ot____nom27_1)

		//recursive
		c569_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_ot____nom27_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2850)) return -1;
		c569_2_fb.open(v_ot____nom27_1);
	case 1837:
	// Continuation for label 824
		while (c569_2_fb.fetch()) {
			v_ot____nom27_1 = c569_2_fb.c2();
		if(ot____nom27_set.insert(v_ot____nom27_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1837)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(827)) return -1;
			if(!backtrack_stack.push(825)) return -1;
			goto label_824;
		}
		}
		break;
	case 2850:
	// Label to restore values of label 824
		v_ot____nom27_1 = int_value_stack.pop();
		c569_2_fb.pop(&cursor_stack);
		break;
	case 825: label_825: 
	// using  symbolic fact ot____nom27(v_ot____nom27_1) with literal 3 in rule 
	// 337: semillon(A) :- semillonorsauvignonblanc(A), madefromgrape(A,B), ot____nom27(B).
	// resulting in symbolic fact semillon(v_semillon_1)

		//recursive
		c337_1_f.push(&cursor_stack);
		if(!int_value_stack.push(v_semillon_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2851)) return -1;
		c337_1_f.open();
		while (c337_1_f.fetch()) {
			if (!madefromgrape_set.contains(c337_1_f.c1(), v_ot____nom27_1)) continue;
	case 1838:
	// Continuation for label 825
				v_semillon_1 = c337_1_f.c1();
		if(semillon_set.insert(v_semillon_1)) {
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1838)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(826)) return -1;
				if(!backtrack_stack.push(823)) return -1;
				if(!backtrack_stack.push(822)) return -1;
				if(!backtrack_stack.push(819)) return -1;
				if(!backtrack_stack.push(818)) return -1;
				goto label_815;
		}
		}
		break;
	case 2851:
	// Label to restore values of label 825
		v_semillon_1 = int_value_stack.pop();
		c337_1_f.pop(&cursor_stack);
		break;
	case 826: label_826: 
	// using  symbolic fact semillon(v_semillon_1) with literal 1 in rule 
	// 771: q3(A) :- semillon(A).
	// resulting in symbolic fact q3(v_q3_1)

		v_q3_1 = v_semillon_1;
		if(q3_set.insert(v_q3_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(143)) return -1;
		goto label_123;
		}
		break;
	case 827: label_827: 
	// using  symbolic fact ot____nom27(v_ot____nom27_1) with literal 3 in rule 
	// 566: q5(A) :- semillonorsauvignonblanc(A), madefromgrape(A,B), ot____nom27(B).
	// resulting in symbolic fact q5(v_q5_1)

		c566_1_f.open();
		while (c566_1_f.fetch()) {
			if (!madefromgrape_set.contains(c566_1_f.c1(), v_ot____nom27_1)) continue;
	case 1840:
	// Continuation for label 827
				v_q5_1 = c566_1_f.c1();
		if(q5_set.insert(v_q5_1)) {
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1840)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(136)) return -1;
				if(!backtrack_stack.push(135)) return -1;
				goto label_131;
		}
		}
		break;
	case 828: label_828: 
	// using  symbolic fact madefromgrape(v_madefromgrape_1,v_madefromgrape_2) with literal 2 in rule 
	// 337: semillon(A) :- semillonorsauvignonblanc(A), madefromgrape(A,B), ot____nom27(B).
	// resulting in symbolic fact semillon(v_semillon_1)

		//recursive
		if(!int_value_stack.push(v_semillon_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2854)) return -1;
		if (!semillonorsauvignonblanc_set.contains(v_madefromgrape_1)) continue;
			if (!ot____nom27_set.contains(v_madefromgrape_2)) continue;
	case 1841:
	// Continuation for label 828
				v_semillon_1 = v_madefromgrape_1;
		if(semillon_set.insert(v_semillon_1)) {
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1841)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(826)) return -1;
				if(!backtrack_stack.push(823)) return -1;
				if(!backtrack_stack.push(822)) return -1;
				if(!backtrack_stack.push(819)) return -1;
				if(!backtrack_stack.push(818)) return -1;
				goto label_815;
		}
		break;
	case 2854:
	// Label to restore values of label 828
		v_semillon_1 = int_value_stack.pop();
		break;
	case 829: label_829: 
	// using  symbolic fact madefromgrape(v_madefromgrape_1,v_madefromgrape_2) with literal 2 in rule 
	// 566: q5(A) :- semillonorsauvignonblanc(A), madefromgrape(A,B), ot____nom27(B).
	// resulting in symbolic fact q5(v_q5_1)

		if (!semillonorsauvignonblanc_set.contains(v_madefromgrape_1)) continue;
			if (!ot____nom27_set.contains(v_madefromgrape_2)) continue;
	case 1842:
	// Continuation for label 829
				v_q5_1 = v_madefromgrape_1;
		if(q5_set.insert(v_q5_1)) {
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1842)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(136)) return -1;
				if(!backtrack_stack.push(135)) return -1;
				goto label_131;
		}
		break;
	case 830:
	// applying rule 
	// 339: sauvignonblanc(A) :- sauvignonblanc_aux(A).
	// resulting in symbolic fact sauvignonblanc(v_sauvignonblanc_1)
		c339_1_f.open();
	case 1843:
	// Continuation for label 830
		while (c339_1_f.fetch()) {
			v_sauvignonblanc_1 = c339_1_f.c1();
		if(sauvignonblanc_set.insert(v_sauvignonblanc_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1843)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(614)) return -1;
			if(!backtrack_stack.push(613)) return -1;
			if(!backtrack_stack.push(604)) return -1;
			goto label_602;
		}
		}
		break;
	case 831:
	// applying rule 
	// 350: ot____nom6(A) :- ot____nom6_aux(A).
	// resulting in symbolic fact ot____nom6(v_ot____nom6_1)
		c350_1_f.open();
	case 1844:
	// Continuation for label 831
		while (c350_1_f.fetch()) {
			v_ot____nom6_1 = c350_1_f.c1();
		if(ot____nom6_set.insert(v_ot____nom6_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1844)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(581)) return -1;
			goto label_578;
		}
		}
		break;
	case 832:
	// applying rule 
	// 387: ot____nom19(A) :- ot____nom19_aux(A).
	// resulting in symbolic fact ot____nom19(v_ot____nom19_1)
		c387_1_f.open();
	case 1845:
	// Continuation for label 832
		while (c387_1_f.fetch()) {
			v_ot____nom19_1 = c387_1_f.c1();
		if(ot____nom19_set.insert(v_ot____nom19_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1845)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(497)) return -1;
			goto label_496;
		}
		}
		break;
	case 833:
	// applying rule 
	// 393: petitesyrah(A) :- petitesyrah_aux(A).
	// resulting in symbolic fact petitesyrah(v_petitesyrah_1)
		c393_1_f.open();
	case 1846:
	// Continuation for label 833
		while (c393_1_f.fetch()) {
			v_petitesyrah_1 = c393_1_f.c1();
		if(petitesyrah_set.insert(v_petitesyrah_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1846)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(842)) return -1;
			if(!backtrack_stack.push(841)) return -1;
			if(!backtrack_stack.push(840)) return -1;
			if(!backtrack_stack.push(837)) return -1;
			if(!backtrack_stack.push(836)) return -1;
			if(!backtrack_stack.push(835)) return -1;
			goto label_834;
		}
		}
		break;
	case 834: label_834: 
	// using  symbolic fact petitesyrah(v_petitesyrah_1) with literal 1 in rule 
	// 392: wine(A) :- petitesyrah(A).
	// resulting in symbolic fact wine(v_wine_1)

		v_wine_1 = v_petitesyrah_1;
		if(wine_set.insert(v_wine_1)) {
		if(!wine_f.insert(v_wine_1)) return -1;
		// Labels for next rules
		if(!backtrack_stack.push(1007)) return -1;
		if(!backtrack_stack.push(989)) return -1;
		if(!backtrack_stack.push(954)) return -1;
		if(!backtrack_stack.push(938)) return -1;
		if(!backtrack_stack.push(911)) return -1;
		if(!backtrack_stack.push(903)) return -1;
		if(!backtrack_stack.push(902)) return -1;
		if(!backtrack_stack.push(901)) return -1;
		if(!backtrack_stack.push(849)) return -1;
		if(!backtrack_stack.push(848)) return -1;
		if(!backtrack_stack.push(821)) return -1;
		if(!backtrack_stack.push(820)) return -1;
		if(!backtrack_stack.push(737)) return -1;
		if(!backtrack_stack.push(712)) return -1;
		if(!backtrack_stack.push(674)) return -1;
		if(!backtrack_stack.push(672)) return -1;
		if(!backtrack_stack.push(583)) return -1;
		if(!backtrack_stack.push(575)) return -1;
		if(!backtrack_stack.push(563)) return -1;
		if(!backtrack_stack.push(551)) return -1;
		if(!backtrack_stack.push(549)) return -1;
		if(!backtrack_stack.push(483)) return -1;
		if(!backtrack_stack.push(473)) return -1;
		if(!backtrack_stack.push(470)) return -1;
		if(!backtrack_stack.push(430)) return -1;
		if(!backtrack_stack.push(401)) return -1;
		if(!backtrack_stack.push(399)) return -1;
		if(!backtrack_stack.push(380)) return -1;
		if(!backtrack_stack.push(364)) return -1;
		if(!backtrack_stack.push(363)) return -1;
		if(!backtrack_stack.push(329)) return -1;
		if(!backtrack_stack.push(328)) return -1;
		if(!backtrack_stack.push(327)) return -1;
		if(!backtrack_stack.push(295)) return -1;
		if(!backtrack_stack.push(294)) return -1;
		if(!backtrack_stack.push(216)) return -1;
		if(!backtrack_stack.push(210)) return -1;
		if(!backtrack_stack.push(192)) return -1;
		if(!backtrack_stack.push(180)) return -1;
		if(!backtrack_stack.push(179)) return -1;
		if(!backtrack_stack.push(178)) return -1;
		if(!backtrack_stack.push(154)) return -1;
		if(!backtrack_stack.push(153)) return -1;
		if(!backtrack_stack.push(100)) return -1;
		if(!backtrack_stack.push(76)) return -1;
		if(!backtrack_stack.push(75)) return -1;
		if(!backtrack_stack.push(67)) return -1;
		if(!backtrack_stack.push(63)) return -1;
		if(!backtrack_stack.push(53)) return -1;
		if(!backtrack_stack.push(46)) return -1;
		if(!backtrack_stack.push(19)) return -1;
		goto label_16;
		}
		break;
	case 835: label_835: 
	// using  symbolic fact petitesyrah(v_petitesyrah_1) with literal 1 in rule 
	// 394: petitesyrah(A) :- petitesyrah(B), kaon2equal(A,B).
	// resulting in symbolic fact petitesyrah(v_petitesyrah_1)

		//recursive
		c394_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_petitesyrah_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2861)) return -1;
		c394_2_fb.open(v_petitesyrah_1);
	case 1848:
	// Continuation for label 835
		while (c394_2_fb.fetch()) {
			v_petitesyrah_1 = c394_2_fb.c2();
		if(petitesyrah_set.insert(v_petitesyrah_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1848)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(842)) return -1;
			if(!backtrack_stack.push(841)) return -1;
			if(!backtrack_stack.push(840)) return -1;
			if(!backtrack_stack.push(837)) return -1;
			if(!backtrack_stack.push(836)) return -1;
			if(!backtrack_stack.push(835)) return -1;
			goto label_834;
		}
		}
		break;
	case 2861:
	// Label to restore values of label 835
		v_petitesyrah_1 = int_value_stack.pop();
		c394_2_fb.pop(&cursor_stack);
		break;
	case 836: label_836: 
	// using  symbolic fact petitesyrah(v_petitesyrah_1) with literal 1 in rule 
	// 522: madefromgrape(A,A) :- petitesyrah(A), kaon2namedobjects(A).
	// resulting in symbolic fact madefromgrape(v_madefromgrape_1,v_madefromgrape_2)

		//recursive
		if(!int_value_stack.push(v_madefromgrape_2)) return -1;
		if(!int_value_stack.push(v_madefromgrape_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2862)) return -1;
		if (!kaon2namedobjects_b.contains(v_petitesyrah_1)) continue;
	case 1849:
	// Continuation for label 836
			v_madefromgrape_1 = v_petitesyrah_1;
			v_madefromgrape_2 = v_petitesyrah_1;
		if(madefromgrape_set.insert(v_madefromgrape_1, v_madefromgrape_2)) {
			if(!madefromgrape_bf.insert(v_madefromgrape_1, v_madefromgrape_2)) return -1;
			if(!madefromgrape_fb.insert(v_madefromgrape_2, v_madefromgrape_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1849)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(860)) return -1;
			if(!backtrack_stack.push(859)) return -1;
			if(!backtrack_stack.push(845)) return -1;
			if(!backtrack_stack.push(844)) return -1;
			if(!backtrack_stack.push(829)) return -1;
			if(!backtrack_stack.push(828)) return -1;
			if(!backtrack_stack.push(813)) return -1;
			if(!backtrack_stack.push(812)) return -1;
			if(!backtrack_stack.push(677)) return -1;
			if(!backtrack_stack.push(676)) return -1;
			if(!backtrack_stack.push(617)) return -1;
			if(!backtrack_stack.push(616)) return -1;
			if(!backtrack_stack.push(609)) return -1;
			if(!backtrack_stack.push(554)) return -1;
			if(!backtrack_stack.push(553)) return -1;
			if(!backtrack_stack.push(476)) return -1;
			if(!backtrack_stack.push(475)) return -1;
			if(!backtrack_stack.push(404)) return -1;
			if(!backtrack_stack.push(403)) return -1;
			if(!backtrack_stack.push(362)) return -1;
			if(!backtrack_stack.push(361)) return -1;
			if(!backtrack_stack.push(343)) return -1;
			if(!backtrack_stack.push(342)) return -1;
			if(!backtrack_stack.push(300)) return -1;
			if(!backtrack_stack.push(170)) return -1;
			if(!backtrack_stack.push(169)) return -1;
			if(!backtrack_stack.push(151)) return -1;
			if(!backtrack_stack.push(150)) return -1;
			if(!backtrack_stack.push(144)) return -1;
			if(!backtrack_stack.push(132)) return -1;
			if(!backtrack_stack.push(71)) return -1;
			if(!backtrack_stack.push(70)) return -1;
			if(!backtrack_stack.push(66)) return -1;
			goto label_64;
		}
		break;
	case 2862:
	// Label to restore values of label 836
		v_madefromgrape_1 = int_value_stack.pop();
		v_madefromgrape_2 = int_value_stack.pop();
		break;
	case 837: label_837: 
	// using  symbolic fact petitesyrah(v_petitesyrah_1) with literal 1 in rule 
	// 572: ot____nom25(A) :- petitesyrah(A), kaon2namedobjects(A).
	// resulting in symbolic fact ot____nom25(v_ot____nom25_1)

		//recursive
		if(!int_value_stack.push(v_ot____nom25_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2863)) return -1;
		if (!kaon2namedobjects_b.contains(v_petitesyrah_1)) continue;
	case 1850:
	// Continuation for label 837
			v_ot____nom25_1 = v_petitesyrah_1;
		if(ot____nom25_set.insert(v_ot____nom25_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1850)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(843)) return -1;
			if(!backtrack_stack.push(839)) return -1;
			goto label_838;
		}
		break;
	case 2863:
	// Label to restore values of label 837
		v_ot____nom25_1 = int_value_stack.pop();
		break;
	case 838: label_838: 
	// using  symbolic fact ot____nom25(v_ot____nom25_1) with literal 1 in rule 
	// 573: ot____nom25(A) :- ot____nom25(B), kaon2equal(A,B).
	// resulting in symbolic fact ot____nom25(v_ot____nom25_1)

		//recursive
		c573_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_ot____nom25_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2864)) return -1;
		c573_2_fb.open(v_ot____nom25_1);
	case 1851:
	// Continuation for label 838
		while (c573_2_fb.fetch()) {
			v_ot____nom25_1 = c573_2_fb.c2();
		if(ot____nom25_set.insert(v_ot____nom25_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1851)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(843)) return -1;
			if(!backtrack_stack.push(839)) return -1;
			goto label_838;
		}
		}
		break;
	case 2864:
	// Label to restore values of label 838
		v_ot____nom25_1 = int_value_stack.pop();
		c573_2_fb.pop(&cursor_stack);
		break;
	case 839: label_839: 
	// using  symbolic fact ot____nom25(v_ot____nom25_1) with literal 3 in rule 
	// 395: petitesyrah(A) :- wine(A), madefromgrape(A,B), ot____nom25(B).
	// resulting in symbolic fact petitesyrah(v_petitesyrah_1)

		//recursive
		c395_1_f.push(&cursor_stack);
		if(!int_value_stack.push(v_petitesyrah_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2865)) return -1;
		c395_1_f.open();
		while (c395_1_f.fetch()) {
			if (!madefromgrape_set.contains(c395_1_f.c1(), v_ot____nom25_1)) continue;
	case 1852:
	// Continuation for label 839
				v_petitesyrah_1 = c395_1_f.c1();
		if(petitesyrah_set.insert(v_petitesyrah_1)) {
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1852)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(842)) return -1;
				if(!backtrack_stack.push(841)) return -1;
				if(!backtrack_stack.push(840)) return -1;
				if(!backtrack_stack.push(837)) return -1;
				if(!backtrack_stack.push(836)) return -1;
				if(!backtrack_stack.push(835)) return -1;
				goto label_834;
		}
		}
		break;
	case 2865:
	// Label to restore values of label 839
		v_petitesyrah_1 = int_value_stack.pop();
		c395_1_f.pop(&cursor_stack);
		break;
	case 840: label_840: 
	// using  symbolic fact petitesyrah(v_petitesyrah_1) with literal 1 in rule 
	// 605: q0(A) :- petitesyrah(A).
	// resulting in symbolic fact q0(v_q0_1)

		v_q0_1 = v_petitesyrah_1;
		if(q0_set.insert(v_q0_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(198)) return -1;
		if(!backtrack_stack.push(194)) return -1;
		if(!backtrack_stack.push(189)) return -1;
		if(!backtrack_stack.push(188)) return -1;
		goto label_187;
		}
		break;
	case 841: label_841: 
	// using  symbolic fact petitesyrah(v_petitesyrah_1) with literal 1 in rule 
	// 639: q31(A) :- petitesyrah(A).
	// resulting in symbolic fact q31(v_q31_1)

		v_q31_1 = v_petitesyrah_1;
		if(q31_set.insert(v_q31_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(221)) return -1;
		if(!backtrack_stack.push(220)) return -1;
		if(!backtrack_stack.push(215)) return -1;
		if(!backtrack_stack.push(213)) return -1;
		goto label_201;
		}
		break;
	case 842: label_842: 
	// using  symbolic fact petitesyrah(v_petitesyrah_1) with literal 1 in rule 
	// 772: q3(A) :- petitesyrah(A).
	// resulting in symbolic fact q3(v_q3_1)

		v_q3_1 = v_petitesyrah_1;
		if(q3_set.insert(v_q3_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(143)) return -1;
		goto label_123;
		}
		break;
	case 843: label_843: 
	// using  symbolic fact ot____nom25(v_ot____nom25_1) with literal 3 in rule 
	// 570: q5(A) :- wine(A), madefromgrape(A,B), ot____nom25(B).
	// resulting in symbolic fact q5(v_q5_1)

		c570_1_f.open();
		while (c570_1_f.fetch()) {
			if (!madefromgrape_set.contains(c570_1_f.c1(), v_ot____nom25_1)) continue;
	case 1856:
	// Continuation for label 843
				v_q5_1 = c570_1_f.c1();
		if(q5_set.insert(v_q5_1)) {
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1856)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(136)) return -1;
				if(!backtrack_stack.push(135)) return -1;
				goto label_131;
		}
		}
		break;
	case 844: label_844: 
	// using  symbolic fact madefromgrape(v_madefromgrape_1,v_madefromgrape_2) with literal 2 in rule 
	// 395: petitesyrah(A) :- wine(A), madefromgrape(A,B), ot____nom25(B).
	// resulting in symbolic fact petitesyrah(v_petitesyrah_1)

		//recursive
		if(!int_value_stack.push(v_petitesyrah_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2870)) return -1;
		if (!wine_set.contains(v_madefromgrape_1)) continue;
			if (!ot____nom25_set.contains(v_madefromgrape_2)) continue;
	case 1857:
	// Continuation for label 844
				v_petitesyrah_1 = v_madefromgrape_1;
		if(petitesyrah_set.insert(v_petitesyrah_1)) {
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1857)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(842)) return -1;
				if(!backtrack_stack.push(841)) return -1;
				if(!backtrack_stack.push(840)) return -1;
				if(!backtrack_stack.push(837)) return -1;
				if(!backtrack_stack.push(836)) return -1;
				if(!backtrack_stack.push(835)) return -1;
				goto label_834;
		}
		break;
	case 2870:
	// Label to restore values of label 844
		v_petitesyrah_1 = int_value_stack.pop();
		break;
	case 845: label_845: 
	// using  symbolic fact madefromgrape(v_madefromgrape_1,v_madefromgrape_2) with literal 2 in rule 
	// 570: q5(A) :- wine(A), madefromgrape(A,B), ot____nom25(B).
	// resulting in symbolic fact q5(v_q5_1)

		if (!wine_set.contains(v_madefromgrape_1)) continue;
			if (!ot____nom25_set.contains(v_madefromgrape_2)) continue;
	case 1858:
	// Continuation for label 845
				v_q5_1 = v_madefromgrape_1;
		if(q5_set.insert(v_q5_1)) {
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1858)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(136)) return -1;
				if(!backtrack_stack.push(135)) return -1;
				goto label_131;
		}
		break;
	case 846:
	// applying rule 
	// 419: zinfandel(A) :- zinfandel_aux(A).
	// resulting in symbolic fact zinfandel(v_zinfandel_1)
		c419_1_f.open();
	case 1859:
	// Continuation for label 846
		while (c419_1_f.fetch()) {
			v_zinfandel_1 = c419_1_f.c1();
		if(zinfandel_set.insert(v_zinfandel_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1859)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(857)) return -1;
			if(!backtrack_stack.push(856)) return -1;
			if(!backtrack_stack.push(853)) return -1;
			if(!backtrack_stack.push(852)) return -1;
			if(!backtrack_stack.push(851)) return -1;
			if(!backtrack_stack.push(850)) return -1;
			goto label_847;
		}
		}
		break;
	case 847: label_847: 
	// using  symbolic fact zinfandel(v_zinfandel_1) with literal 1 in rule 
	// 418: wine(A) :- zinfandel(A).
	// resulting in symbolic fact wine(v_wine_1)

		v_wine_1 = v_zinfandel_1;
		if(wine_set.insert(v_wine_1)) {
		if(!wine_f.insert(v_wine_1)) return -1;
		// Labels for next rules
		if(!backtrack_stack.push(1007)) return -1;
		if(!backtrack_stack.push(989)) return -1;
		if(!backtrack_stack.push(954)) return -1;
		if(!backtrack_stack.push(938)) return -1;
		if(!backtrack_stack.push(911)) return -1;
		if(!backtrack_stack.push(903)) return -1;
		if(!backtrack_stack.push(902)) return -1;
		if(!backtrack_stack.push(901)) return -1;
		if(!backtrack_stack.push(849)) return -1;
		if(!backtrack_stack.push(848)) return -1;
		if(!backtrack_stack.push(821)) return -1;
		if(!backtrack_stack.push(820)) return -1;
		if(!backtrack_stack.push(737)) return -1;
		if(!backtrack_stack.push(712)) return -1;
		if(!backtrack_stack.push(674)) return -1;
		if(!backtrack_stack.push(672)) return -1;
		if(!backtrack_stack.push(583)) return -1;
		if(!backtrack_stack.push(575)) return -1;
		if(!backtrack_stack.push(563)) return -1;
		if(!backtrack_stack.push(551)) return -1;
		if(!backtrack_stack.push(549)) return -1;
		if(!backtrack_stack.push(483)) return -1;
		if(!backtrack_stack.push(473)) return -1;
		if(!backtrack_stack.push(470)) return -1;
		if(!backtrack_stack.push(430)) return -1;
		if(!backtrack_stack.push(401)) return -1;
		if(!backtrack_stack.push(399)) return -1;
		if(!backtrack_stack.push(380)) return -1;
		if(!backtrack_stack.push(364)) return -1;
		if(!backtrack_stack.push(363)) return -1;
		if(!backtrack_stack.push(329)) return -1;
		if(!backtrack_stack.push(328)) return -1;
		if(!backtrack_stack.push(327)) return -1;
		if(!backtrack_stack.push(295)) return -1;
		if(!backtrack_stack.push(294)) return -1;
		if(!backtrack_stack.push(216)) return -1;
		if(!backtrack_stack.push(210)) return -1;
		if(!backtrack_stack.push(192)) return -1;
		if(!backtrack_stack.push(180)) return -1;
		if(!backtrack_stack.push(179)) return -1;
		if(!backtrack_stack.push(178)) return -1;
		if(!backtrack_stack.push(154)) return -1;
		if(!backtrack_stack.push(153)) return -1;
		if(!backtrack_stack.push(100)) return -1;
		if(!backtrack_stack.push(76)) return -1;
		if(!backtrack_stack.push(75)) return -1;
		if(!backtrack_stack.push(67)) return -1;
		if(!backtrack_stack.push(63)) return -1;
		if(!backtrack_stack.push(53)) return -1;
		if(!backtrack_stack.push(46)) return -1;
		if(!backtrack_stack.push(19)) return -1;
		goto label_16;
		}
		break;
	case 848: label_848: 
	// using  symbolic fact wine(v_wine_1) with literal 1 in rule 
	// 395: petitesyrah(A) :- wine(A), madefromgrape(A,B), ot____nom25(B).
	// resulting in symbolic fact petitesyrah(v_petitesyrah_1)

		c395_2_bf.open(v_wine_1);
		while (c395_2_bf.fetch()) {
			if (!ot____nom25_set.contains(c395_2_bf.c2())) continue;
	case 1861:
	// Continuation for label 848
				v_petitesyrah_1 = v_wine_1;
		if(petitesyrah_set.insert(v_petitesyrah_1)) {
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1861)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(842)) return -1;
				if(!backtrack_stack.push(841)) return -1;
				if(!backtrack_stack.push(840)) return -1;
				if(!backtrack_stack.push(837)) return -1;
				if(!backtrack_stack.push(836)) return -1;
				if(!backtrack_stack.push(835)) return -1;
				goto label_834;
		}
		}
		break;
	case 849: label_849: 
	// using  symbolic fact wine(v_wine_1) with literal 1 in rule 
	// 570: q5(A) :- wine(A), madefromgrape(A,B), ot____nom25(B).
	// resulting in symbolic fact q5(v_q5_1)

		c570_2_bf.open(v_wine_1);
		while (c570_2_bf.fetch()) {
			if (!ot____nom25_set.contains(c570_2_bf.c2())) continue;
	case 1862:
	// Continuation for label 849
				v_q5_1 = v_wine_1;
		if(q5_set.insert(v_q5_1)) {
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1862)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(136)) return -1;
				if(!backtrack_stack.push(135)) return -1;
				goto label_131;
		}
		}
		break;
	case 850: label_850: 
	// using  symbolic fact zinfandel(v_zinfandel_1) with literal 1 in rule 
	// 420: zinfandel(A) :- zinfandel(B), kaon2equal(A,B).
	// resulting in symbolic fact zinfandel(v_zinfandel_1)

		//recursive
		c420_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_zinfandel_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2876)) return -1;
		c420_2_fb.open(v_zinfandel_1);
	case 1863:
	// Continuation for label 850
		while (c420_2_fb.fetch()) {
			v_zinfandel_1 = c420_2_fb.c2();
		if(zinfandel_set.insert(v_zinfandel_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1863)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(857)) return -1;
			if(!backtrack_stack.push(856)) return -1;
			if(!backtrack_stack.push(853)) return -1;
			if(!backtrack_stack.push(852)) return -1;
			if(!backtrack_stack.push(851)) return -1;
			if(!backtrack_stack.push(850)) return -1;
			goto label_847;
		}
		}
		break;
	case 2876:
	// Label to restore values of label 850
		v_zinfandel_1 = int_value_stack.pop();
		c420_2_fb.pop(&cursor_stack);
		break;
	case 851: label_851: 
	// using  symbolic fact zinfandel(v_zinfandel_1) with literal 1 in rule 
	// 440: q0(A) :- zinfandel(A).
	// resulting in symbolic fact q0(v_q0_1)

		v_q0_1 = v_zinfandel_1;
		if(q0_set.insert(v_q0_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(198)) return -1;
		if(!backtrack_stack.push(194)) return -1;
		if(!backtrack_stack.push(189)) return -1;
		if(!backtrack_stack.push(188)) return -1;
		goto label_187;
		}
		break;
	case 852: label_852: 
	// using  symbolic fact zinfandel(v_zinfandel_1) with literal 1 in rule 
	// 520: madefromgrape(A,A) :- zinfandel(A), kaon2namedobjects(A).
	// resulting in symbolic fact madefromgrape(v_madefromgrape_1,v_madefromgrape_2)

		//recursive
		if(!int_value_stack.push(v_madefromgrape_2)) return -1;
		if(!int_value_stack.push(v_madefromgrape_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2878)) return -1;
		if (!kaon2namedobjects_b.contains(v_zinfandel_1)) continue;
	case 1865:
	// Continuation for label 852
			v_madefromgrape_1 = v_zinfandel_1;
			v_madefromgrape_2 = v_zinfandel_1;
		if(madefromgrape_set.insert(v_madefromgrape_1, v_madefromgrape_2)) {
			if(!madefromgrape_bf.insert(v_madefromgrape_1, v_madefromgrape_2)) return -1;
			if(!madefromgrape_fb.insert(v_madefromgrape_2, v_madefromgrape_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1865)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(860)) return -1;
			if(!backtrack_stack.push(859)) return -1;
			if(!backtrack_stack.push(845)) return -1;
			if(!backtrack_stack.push(844)) return -1;
			if(!backtrack_stack.push(829)) return -1;
			if(!backtrack_stack.push(828)) return -1;
			if(!backtrack_stack.push(813)) return -1;
			if(!backtrack_stack.push(812)) return -1;
			if(!backtrack_stack.push(677)) return -1;
			if(!backtrack_stack.push(676)) return -1;
			if(!backtrack_stack.push(617)) return -1;
			if(!backtrack_stack.push(616)) return -1;
			if(!backtrack_stack.push(609)) return -1;
			if(!backtrack_stack.push(554)) return -1;
			if(!backtrack_stack.push(553)) return -1;
			if(!backtrack_stack.push(476)) return -1;
			if(!backtrack_stack.push(475)) return -1;
			if(!backtrack_stack.push(404)) return -1;
			if(!backtrack_stack.push(403)) return -1;
			if(!backtrack_stack.push(362)) return -1;
			if(!backtrack_stack.push(361)) return -1;
			if(!backtrack_stack.push(343)) return -1;
			if(!backtrack_stack.push(342)) return -1;
			if(!backtrack_stack.push(300)) return -1;
			if(!backtrack_stack.push(170)) return -1;
			if(!backtrack_stack.push(169)) return -1;
			if(!backtrack_stack.push(151)) return -1;
			if(!backtrack_stack.push(150)) return -1;
			if(!backtrack_stack.push(144)) return -1;
			if(!backtrack_stack.push(132)) return -1;
			if(!backtrack_stack.push(71)) return -1;
			if(!backtrack_stack.push(70)) return -1;
			if(!backtrack_stack.push(66)) return -1;
			goto label_64;
		}
		break;
	case 2878:
	// Label to restore values of label 852
		v_madefromgrape_1 = int_value_stack.pop();
		v_madefromgrape_2 = int_value_stack.pop();
		break;
	case 853: label_853: 
	// using  symbolic fact zinfandel(v_zinfandel_1) with literal 1 in rule 
	// 556: ot____nom20(A) :- zinfandel(A), kaon2namedobjects(A).
	// resulting in symbolic fact ot____nom20(v_ot____nom20_1)

		//recursive
		if(!int_value_stack.push(v_ot____nom20_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2879)) return -1;
		if (!kaon2namedobjects_b.contains(v_zinfandel_1)) continue;
	case 1866:
	// Continuation for label 853
			v_ot____nom20_1 = v_zinfandel_1;
		if(ot____nom20_set.insert(v_ot____nom20_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1866)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(858)) return -1;
			if(!backtrack_stack.push(855)) return -1;
			goto label_854;
		}
		break;
	case 2879:
	// Label to restore values of label 853
		v_ot____nom20_1 = int_value_stack.pop();
		break;
	case 854: label_854: 
	// using  symbolic fact ot____nom20(v_ot____nom20_1) with literal 1 in rule 
	// 557: ot____nom20(A) :- ot____nom20(B), kaon2equal(A,B).
	// resulting in symbolic fact ot____nom20(v_ot____nom20_1)

		//recursive
		c557_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_ot____nom20_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2880)) return -1;
		c557_2_fb.open(v_ot____nom20_1);
	case 1867:
	// Continuation for label 854
		while (c557_2_fb.fetch()) {
			v_ot____nom20_1 = c557_2_fb.c2();
		if(ot____nom20_set.insert(v_ot____nom20_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1867)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(858)) return -1;
			if(!backtrack_stack.push(855)) return -1;
			goto label_854;
		}
		}
		break;
	case 2880:
	// Label to restore values of label 854
		v_ot____nom20_1 = int_value_stack.pop();
		c557_2_fb.pop(&cursor_stack);
		break;
	case 855: label_855: 
	// using  symbolic fact ot____nom20(v_ot____nom20_1) with literal 3 in rule 
	// 421: zinfandel(A) :- wine(A), madefromgrape(A,B), ot____nom20(B).
	// resulting in symbolic fact zinfandel(v_zinfandel_1)

		//recursive
		c421_1_f.push(&cursor_stack);
		if(!int_value_stack.push(v_zinfandel_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2881)) return -1;
		c421_1_f.open();
		while (c421_1_f.fetch()) {
			if (!madefromgrape_set.contains(c421_1_f.c1(), v_ot____nom20_1)) continue;
	case 1868:
	// Continuation for label 855
				v_zinfandel_1 = c421_1_f.c1();
		if(zinfandel_set.insert(v_zinfandel_1)) {
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1868)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(857)) return -1;
				if(!backtrack_stack.push(856)) return -1;
				if(!backtrack_stack.push(853)) return -1;
				if(!backtrack_stack.push(852)) return -1;
				if(!backtrack_stack.push(851)) return -1;
				if(!backtrack_stack.push(850)) return -1;
				goto label_847;
		}
		}
		break;
	case 2881:
	// Label to restore values of label 855
		v_zinfandel_1 = int_value_stack.pop();
		c421_1_f.pop(&cursor_stack);
		break;
	case 856: label_856: 
	// using  symbolic fact zinfandel(v_zinfandel_1) with literal 1 in rule 
	// 635: q31(A) :- zinfandel(A).
	// resulting in symbolic fact q31(v_q31_1)

		v_q31_1 = v_zinfandel_1;
		if(q31_set.insert(v_q31_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(221)) return -1;
		if(!backtrack_stack.push(220)) return -1;
		if(!backtrack_stack.push(215)) return -1;
		if(!backtrack_stack.push(213)) return -1;
		goto label_201;
		}
		break;
	case 857: label_857: 
	// using  symbolic fact zinfandel(v_zinfandel_1) with literal 1 in rule 
	// 767: q3(A) :- zinfandel(A).
	// resulting in symbolic fact q3(v_q3_1)

		v_q3_1 = v_zinfandel_1;
		if(q3_set.insert(v_q3_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(143)) return -1;
		goto label_123;
		}
		break;
	case 858: label_858: 
	// using  symbolic fact ot____nom20(v_ot____nom20_1) with literal 3 in rule 
	// 554: q5(A) :- wine(A), madefromgrape(A,B), ot____nom20(B).
	// resulting in symbolic fact q5(v_q5_1)

		c554_1_f.open();
		while (c554_1_f.fetch()) {
			if (!madefromgrape_set.contains(c554_1_f.c1(), v_ot____nom20_1)) continue;
	case 1871:
	// Continuation for label 858
				v_q5_1 = c554_1_f.c1();
		if(q5_set.insert(v_q5_1)) {
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1871)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(136)) return -1;
				if(!backtrack_stack.push(135)) return -1;
				goto label_131;
		}
		}
		break;
	case 859: label_859: 
	// using  symbolic fact madefromgrape(v_madefromgrape_1,v_madefromgrape_2) with literal 2 in rule 
	// 421: zinfandel(A) :- wine(A), madefromgrape(A,B), ot____nom20(B).
	// resulting in symbolic fact zinfandel(v_zinfandel_1)

		//recursive
		if(!int_value_stack.push(v_zinfandel_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2885)) return -1;
		if (!wine_set.contains(v_madefromgrape_1)) continue;
			if (!ot____nom20_set.contains(v_madefromgrape_2)) continue;
	case 1872:
	// Continuation for label 859
				v_zinfandel_1 = v_madefromgrape_1;
		if(zinfandel_set.insert(v_zinfandel_1)) {
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1872)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(857)) return -1;
				if(!backtrack_stack.push(856)) return -1;
				if(!backtrack_stack.push(853)) return -1;
				if(!backtrack_stack.push(852)) return -1;
				if(!backtrack_stack.push(851)) return -1;
				if(!backtrack_stack.push(850)) return -1;
				goto label_847;
		}
		break;
	case 2885:
	// Label to restore values of label 859
		v_zinfandel_1 = int_value_stack.pop();
		break;
	case 860: label_860: 
	// using  symbolic fact madefromgrape(v_madefromgrape_1,v_madefromgrape_2) with literal 2 in rule 
	// 554: q5(A) :- wine(A), madefromgrape(A,B), ot____nom20(B).
	// resulting in symbolic fact q5(v_q5_1)

		if (!wine_set.contains(v_madefromgrape_1)) continue;
			if (!ot____nom20_set.contains(v_madefromgrape_2)) continue;
	case 1873:
	// Continuation for label 860
				v_q5_1 = v_madefromgrape_1;
		if(q5_set.insert(v_q5_1)) {
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1873)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(136)) return -1;
				if(!backtrack_stack.push(135)) return -1;
				goto label_131;
		}
		break;
	case 861:
	// applying rule 
	// 444: redtablewine(A) :- redtablewine_aux(A).
	// resulting in symbolic fact redtablewine(v_redtablewine_1)
		c444_1_f.open();
	case 1874:
	// Continuation for label 861
		while (c444_1_f.fetch()) {
			v_redtablewine_1 = c444_1_f.c1();
		if(redtablewine_set.insert(v_redtablewine_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1874)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(230)) return -1;
			if(!backtrack_stack.push(229)) return -1;
			goto label_228;
		}
		}
		break;
	case 862:
	// applying rule 
	// 459: dessertwine(A) :- dessertwine_aux(A).
	// resulting in symbolic fact dessertwine(v_dessertwine_1)
		c459_1_f.open();
	case 1875:
	// Continuation for label 862
		while (c459_1_f.fetch()) {
			v_dessertwine_1 = c459_1_f.c1();
		if(dessertwine_set.insert(v_dessertwine_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1875)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(770)) return -1;
			if(!backtrack_stack.push(753)) return -1;
			goto label_752;
		}
		}
		break;
	case 863:
	// applying rule 
	// 531: ot____nom3(A) :- ot____nom3_aux(A).
	// resulting in symbolic fact ot____nom3(v_ot____nom3_1)
		c531_1_f.open();
	case 1876:
	// Continuation for label 863
		while (c531_1_f.fetch()) {
			v_ot____nom3_1 = c531_1_f.c1();
		if(ot____nom3_set.insert(v_ot____nom3_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1876)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(811)) return -1;
			if(!backtrack_stack.push(808)) return -1;
			goto label_807;
		}
		}
		break;
	case 864:
	// applying rule 
	// 535: ot____nom43(A) :- ot____nom43_aux(A).
	// resulting in symbolic fact ot____nom43(v_ot____nom43_1)
		c535_1_f.open();
	case 1877:
	// Continuation for label 864
		while (c535_1_f.fetch()) {
			v_ot____nom43_1 = c535_1_f.c1();
		if(ot____nom43_set.insert(v_ot____nom43_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1877)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(552)) return -1;
			if(!backtrack_stack.push(545)) return -1;
			goto label_544;
		}
		}
		break;
	case 865:
	// applying rule 
	// 539: ot____nom24(A) :- ot____nom24_aux(A).
	// resulting in symbolic fact ot____nom24(v_ot____nom24_1)
		c539_1_f.open();
	case 1878:
	// Continuation for label 865
		while (c539_1_f.fetch()) {
			v_ot____nom24_1 = c539_1_f.c1();
		if(ot____nom24_set.insert(v_ot____nom24_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1878)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(360)) return -1;
			if(!backtrack_stack.push(359)) return -1;
			goto label_358;
		}
		}
		break;
	case 866:
	// applying rule 
	// 543: ot____nom11(A) :- ot____nom11_aux(A).
	// resulting in symbolic fact ot____nom11(v_ot____nom11_1)
		c543_1_f.open();
	case 1879:
	// Continuation for label 866
		while (c543_1_f.fetch()) {
			v_ot____nom11_1 = c543_1_f.c1();
		if(ot____nom11_set.insert(v_ot____nom11_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1879)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(615)) return -1;
			if(!backtrack_stack.push(612)) return -1;
			goto label_611;
		}
		}
		break;
	case 867:
	// applying rule 
	// 547: ot____nom34(A) :- ot____nom34_aux(A).
	// resulting in symbolic fact ot____nom34(v_ot____nom34_1)
		c547_1_f.open();
	case 1880:
	// Continuation for label 867
		while (c547_1_f.fetch()) {
			v_ot____nom34_1 = c547_1_f.c1();
		if(ot____nom34_set.insert(v_ot____nom34_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1880)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(149)) return -1;
			if(!backtrack_stack.push(148)) return -1;
			goto label_147;
		}
		}
		break;
	case 868:
	// applying rule 
	// 551: ot____nom37(A) :- ot____nom37_aux(A).
	// resulting in symbolic fact ot____nom37(v_ot____nom37_1)
		c551_1_f.open();
	case 1881:
	// Continuation for label 868
		while (c551_1_f.fetch()) {
			v_ot____nom37_1 = c551_1_f.c1();
		if(ot____nom37_set.insert(v_ot____nom37_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1881)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(316)) return -1;
			if(!backtrack_stack.push(307)) return -1;
			goto label_306;
		}
		}
		break;
	case 869:
	// applying rule 
	// 555: ot____nom20(A) :- ot____nom20_aux(A).
	// resulting in symbolic fact ot____nom20(v_ot____nom20_1)
		c555_1_f.open();
	case 1882:
	// Continuation for label 869
		while (c555_1_f.fetch()) {
			v_ot____nom20_1 = c555_1_f.c1();
		if(ot____nom20_set.insert(v_ot____nom20_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1882)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(858)) return -1;
			if(!backtrack_stack.push(855)) return -1;
			goto label_854;
		}
		}
		break;
	case 870:
	// applying rule 
	// 559: ot____nom33(A) :- ot____nom33_aux(A).
	// resulting in symbolic fact ot____nom33(v_ot____nom33_1)
		c559_1_f.open();
	case 1883:
	// Continuation for label 870
		while (c559_1_f.fetch()) {
			v_ot____nom33_1 = c559_1_f.c1();
		if(ot____nom33_set.insert(v_ot____nom33_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1883)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(166)) return -1;
			if(!backtrack_stack.push(165)) return -1;
			goto label_164;
		}
		}
		break;
	case 871:
	// applying rule 
	// 563: ot____nom39(A) :- ot____nom39_aux(A).
	// resulting in symbolic fact ot____nom39(v_ot____nom39_1)
		c563_1_f.open();
	case 1884:
	// Continuation for label 871
		while (c563_1_f.fetch()) {
			v_ot____nom39_1 = c563_1_f.c1();
		if(ot____nom39_set.insert(v_ot____nom39_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1884)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(402)) return -1;
			if(!backtrack_stack.push(396)) return -1;
			goto label_395;
		}
		}
		break;
	case 872:
	// applying rule 
	// 567: ot____nom27(A) :- ot____nom27_aux(A).
	// resulting in symbolic fact ot____nom27(v_ot____nom27_1)
		c567_1_f.open();
	case 1885:
	// Continuation for label 872
		while (c567_1_f.fetch()) {
			v_ot____nom27_1 = c567_1_f.c1();
		if(ot____nom27_set.insert(v_ot____nom27_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1885)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(827)) return -1;
			if(!backtrack_stack.push(825)) return -1;
			goto label_824;
		}
		}
		break;
	case 873:
	// applying rule 
	// 571: ot____nom25(A) :- ot____nom25_aux(A).
	// resulting in symbolic fact ot____nom25(v_ot____nom25_1)
		c571_1_f.open();
	case 1886:
	// Continuation for label 873
		while (c571_1_f.fetch()) {
			v_ot____nom25_1 = c571_1_f.c1();
		if(ot____nom25_set.insert(v_ot____nom25_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1886)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(843)) return -1;
			if(!backtrack_stack.push(839)) return -1;
			goto label_838;
		}
		}
		break;
	case 874:
	// applying rule 
	// 575: ot____nom21(A) :- ot____nom21_aux(A).
	// resulting in symbolic fact ot____nom21(v_ot____nom21_1)
		c575_1_f.open();
	case 1887:
	// Continuation for label 874
		while (c575_1_f.fetch()) {
			v_ot____nom21_1 = c575_1_f.c1();
		if(ot____nom21_set.insert(v_ot____nom21_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1887)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(474)) return -1;
			if(!backtrack_stack.push(467)) return -1;
			goto label_466;
		}
		}
		break;
	case 875:
	// applying rule 
	// 579: ot____nom29(A) :- ot____nom29_aux(A).
	// resulting in symbolic fact ot____nom29(v_ot____nom29_1)
		c579_1_f.open();
	case 1888:
	// Continuation for label 875
		while (c579_1_f.fetch()) {
			v_ot____nom29_1 = c579_1_f.c1();
		if(ot____nom29_set.insert(v_ot____nom29_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1888)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(675)) return -1;
			if(!backtrack_stack.push(668)) return -1;
			goto label_667;
		}
		}
		break;
	case 876:
	// applying rule 
	// 586: ot____nom7(A) :- ot____nom7_aux(A).
	// resulting in symbolic fact ot____nom7(v_ot____nom7_1)
		c586_1_f.open();
	case 1889:
	// Continuation for label 876
		while (c586_1_f.fetch()) {
			v_ot____nom7_1 = c586_1_f.c1();
		if(ot____nom7_set.insert(v_ot____nom7_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1889)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(110)) return -1;
			goto label_109;
		}
		}
		break;
	case 877:
	// applying rule 
	// 603: ot____nom28(A) :- ot____nom28_aux(A).
	// resulting in symbolic fact ot____nom28(v_ot____nom28_1)
		c603_1_f.open();
	case 1890:
	// Continuation for label 877
		while (c603_1_f.fetch()) {
			v_ot____nom28_1 = c603_1_f.c1();
		if(ot____nom28_set.insert(v_ot____nom28_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1890)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(196)) return -1;
			goto label_195;
		}
		}
		break;
	case 878:
	// applying rule 
	// 623: ot____nom45(A) :- ot____nom45_aux(A).
	// resulting in symbolic fact ot____nom45(v_ot____nom45_1)
		c623_1_f.open();
	case 1891:
	// Continuation for label 878
		while (c623_1_f.fetch()) {
			v_ot____nom45_1 = c623_1_f.c1();
		if(ot____nom45_set.insert(v_ot____nom45_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1891)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(427)) return -1;
			goto label_426;
		}
		}
		break;
	case 879:
	// applying rule 
	// 627: ot____nom1(A) :- ot____nom1_aux(A).
	// resulting in symbolic fact ot____nom1(v_ot____nom1_1)
		c627_1_f.open();
	case 1892:
	// Continuation for label 879
		while (c627_1_f.fetch()) {
			v_ot____nom1_1 = c627_1_f.c1();
		if(ot____nom1_set.insert(v_ot____nom1_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1892)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(744)) return -1;
			goto label_743;
		}
		}
		break;
	case 880:
	// applying rule 
	// 653: ot____nom32(A) :- ot____nom32_aux(A).
	// resulting in symbolic fact ot____nom32(v_ot____nom32_1)
		c653_1_f.open();
	case 1893:
	// Continuation for label 880
		while (c653_1_f.fetch()) {
			v_ot____nom32_1 = c653_1_f.c1();
		if(ot____nom32_set.insert(v_ot____nom32_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1893)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(709)) return -1;
			goto label_708;
		}
		}
		break;
	case 881:
	// applying rule 
	// 656: ot____nom5(A) :- ot____nom5_aux(A).
	// resulting in symbolic fact ot____nom5(v_ot____nom5_1)
		c656_1_f.open();
	case 1894:
	// Continuation for label 881
		while (c656_1_f.fetch()) {
			v_ot____nom5_1 = c656_1_f.c1();
		if(ot____nom5_set.insert(v_ot____nom5_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1894)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(118)) return -1;
			goto label_117;
		}
		}
		break;
	case 882:
	// applying rule 
	// 659: ot____nom12(A) :- ot____nom12_aux(A).
	// resulting in symbolic fact ot____nom12(v_ot____nom12_1)
		c659_1_f.open();
	case 1895:
	// Continuation for label 882
		while (c659_1_f.fetch()) {
			v_ot____nom12_1 = c659_1_f.c1();
		if(ot____nom12_set.insert(v_ot____nom12_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1895)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(223)) return -1;
			goto label_222;
		}
		}
		break;
	case 883:
	// applying rule 
	// 663: ot____nom15(A) :- ot____nom15_aux(A).
	// resulting in symbolic fact ot____nom15(v_ot____nom15_1)
		c663_1_f.open();
	case 1896:
	// Continuation for label 883
		while (c663_1_f.fetch()) {
			v_ot____nom15_1 = c663_1_f.c1();
		if(ot____nom15_set.insert(v_ot____nom15_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1896)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(885)) return -1;
			goto label_884;
		}
		}
		break;
	case 884: label_884: 
	// using  symbolic fact ot____nom15(v_ot____nom15_1) with literal 1 in rule 
	// 666: ot____nom15(A) :- ot____nom15(B), kaon2equal(A,B).
	// resulting in symbolic fact ot____nom15(v_ot____nom15_1)

		//recursive
		c666_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_ot____nom15_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2910)) return -1;
		c666_2_fb.open(v_ot____nom15_1);
	case 1897:
	// Continuation for label 884
		while (c666_2_fb.fetch()) {
			v_ot____nom15_1 = c666_2_fb.c2();
		if(ot____nom15_set.insert(v_ot____nom15_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1897)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(885)) return -1;
			goto label_884;
		}
		}
		break;
	case 2910:
	// Label to restore values of label 884
		v_ot____nom15_1 = int_value_stack.pop();
		c666_2_fb.pop(&cursor_stack);
		break;
	case 885: label_885: 
	// using  symbolic fact ot____nom15(v_ot____nom15_1) with literal 2 in rule 
	// 662: q48(A) :- locatedin(A,B), ot____nom15(B).
	// resulting in symbolic fact q48(v_q48_1)

		//recursive
		c662_1_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q48_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2911)) return -1;
		c662_1_fb.open(v_ot____nom15_1);
	case 1898:
	// Continuation for label 885
		while (c662_1_fb.fetch()) {
			v_q48_1 = c662_1_fb.c2();
		if(q48_set.insert(v_q48_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1898)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(891)) return -1;
			if(!backtrack_stack.push(890)) return -1;
			goto label_886;
		}
		}
		break;
	case 2911:
	// Label to restore values of label 885
		v_q48_1 = int_value_stack.pop();
		c662_1_fb.pop(&cursor_stack);
		break;
	case 886: label_886: 
	// using  symbolic fact q48(v_q48_1) with literal 1 in rule 
	// 2: californiawine(A) :- q48(A), wine(A).
	// resulting in symbolic fact californiawine(v_californiawine_1)

		//recursive
		if(!int_value_stack.push(v_californiawine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2912)) return -1;
		if (!wine_set.contains(v_q48_1)) continue;
	case 1899:
	// Continuation for label 886
			v_californiawine_1 = v_q48_1;
		if(californiawine_set.insert(v_californiawine_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1899)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(900)) return -1;
			if(!backtrack_stack.push(899)) return -1;
			if(!backtrack_stack.push(888)) return -1;
			goto label_887;
		}
		break;
	case 2912:
	// Label to restore values of label 886
		v_californiawine_1 = int_value_stack.pop();
		break;
	case 887: label_887: 
	// using  symbolic fact californiawine(v_californiawine_1) with literal 1 in rule 
	// 1: answer :- californiawine(A).
	// resulting in symbolic fact answer

		num_results++;
		if(v_californiawine_1 < 0)
			return -1;
		out_c::answer_s(num_results, dom.get(v_californiawine_1));
		//out_c::verbose_msg("Answer ");
		//out_c::verbose_long(num_results);
		//out_c::verbose_msg(": '");
		//out_c::verbose_msg(dom.get(v_californiawine_1));
		//out_c::verbose_msg("' (Code: ");
		//out_c::verbose_int(v_californiawine_1);
		//out_c::verbose_msg(")\n");
		break;
	case 888: label_888: 
	// using  symbolic fact californiawine(v_californiawine_1) with literal 1 in rule 
	// 4: q47(A) :- californiawine(A).
	// resulting in symbolic fact q47(v_q47_1)

		//recursive
		if(!int_value_stack.push(v_q47_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2914)) return -1;
		v_q47_1 = v_californiawine_1;
		if(q47_set.insert(v_q47_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(895)) return -1;
		if(!backtrack_stack.push(894)) return -1;
		if(!backtrack_stack.push(892)) return -1;
		goto label_889;
		}
		break;
	case 2914:
	// Label to restore values of label 888
		v_q47_1 = int_value_stack.pop();
		break;
	case 889: label_889: 
	// using  symbolic fact q47(v_q47_1) with literal 1 in rule 
	// 3: q48(A) :- q47(A).
	// resulting in symbolic fact q48(v_q48_1)

		//recursive
		if(!int_value_stack.push(v_q48_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2915)) return -1;
		v_q48_1 = v_q47_1;
		if(q48_set.insert(v_q48_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(891)) return -1;
		if(!backtrack_stack.push(890)) return -1;
		goto label_886;
		}
		break;
	case 2915:
	// Label to restore values of label 889
		v_q48_1 = int_value_stack.pop();
		break;
	case 890: label_890: 
	// using  symbolic fact q48(v_q48_1) with literal 1 in rule 
	// 667: q48(A) :- q48(B), kaon2equal(A,B).
	// resulting in symbolic fact q48(v_q48_1)

		//recursive
		c667_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q48_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2916)) return -1;
		c667_2_fb.open(v_q48_1);
	case 1903:
	// Continuation for label 890
		while (c667_2_fb.fetch()) {
			v_q48_1 = c667_2_fb.c2();
		if(q48_set.insert(v_q48_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1903)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(891)) return -1;
			if(!backtrack_stack.push(890)) return -1;
			goto label_886;
		}
		}
		break;
	case 2916:
	// Label to restore values of label 890
		v_q48_1 = int_value_stack.pop();
		c667_2_fb.pop(&cursor_stack);
		break;
	case 891: label_891: 
	// using  symbolic fact q48(v_q48_1) with literal 2 in rule 
	// 206: q47(A) :- locatedin(A,B), q48(B).
	// resulting in symbolic fact q47(v_q47_1)

		//recursive
		c206_1_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q47_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2917)) return -1;
		c206_1_fb.open(v_q48_1);
	case 1904:
	// Continuation for label 891
		while (c206_1_fb.fetch()) {
			v_q47_1 = c206_1_fb.c2();
		if(q47_set.insert(v_q47_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1904)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(895)) return -1;
			if(!backtrack_stack.push(894)) return -1;
			if(!backtrack_stack.push(892)) return -1;
			goto label_889;
		}
		}
		break;
	case 2917:
	// Label to restore values of label 891
		v_q47_1 = int_value_stack.pop();
		c206_1_fb.pop(&cursor_stack);
		break;
	case 892: label_892: 
	// using  symbolic fact q47(v_q47_1) with literal 1 in rule 
	// 205: locatedin(A,A) :- q47(A), kaon2namedobjects(A).
	// resulting in symbolic fact locatedin(v_locatedin_1,v_locatedin_2)

		//recursive
		if(!int_value_stack.push(v_locatedin_2)) return -1;
		if(!int_value_stack.push(v_locatedin_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2918)) return -1;
		if (!kaon2namedobjects_b.contains(v_q47_1)) continue;
	case 1905:
	// Continuation for label 892
			v_locatedin_1 = v_q47_1;
			v_locatedin_2 = v_q47_1;
		if(locatedin_set.insert(v_locatedin_1, v_locatedin_2)) {
			if(!locatedin_bf.insert(v_locatedin_1, v_locatedin_2)) return -1;
			if(!locatedin_fb.insert(v_locatedin_2, v_locatedin_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1905)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(1005)) return -1;
			if(!backtrack_stack.push(999)) return -1;
			if(!backtrack_stack.push(987)) return -1;
			if(!backtrack_stack.push(981)) return -1;
			if(!backtrack_stack.push(969)) return -1;
			if(!backtrack_stack.push(963)) return -1;
			if(!backtrack_stack.push(952)) return -1;
			if(!backtrack_stack.push(946)) return -1;
			if(!backtrack_stack.push(936)) return -1;
			if(!backtrack_stack.push(930)) return -1;
			if(!backtrack_stack.push(920)) return -1;
			if(!backtrack_stack.push(917)) return -1;
			if(!backtrack_stack.push(898)) return -1;
			if(!backtrack_stack.push(897)) return -1;
			if(!backtrack_stack.push(896)) return -1;
			if(!backtrack_stack.push(893)) return -1;
			if(!backtrack_stack.push(726)) return -1;
			if(!backtrack_stack.push(721)) return -1;
			if(!backtrack_stack.push(694)) return -1;
			if(!backtrack_stack.push(693)) return -1;
			if(!backtrack_stack.push(688)) return -1;
			if(!backtrack_stack.push(687)) return -1;
			if(!backtrack_stack.push(636)) return -1;
			if(!backtrack_stack.push(631)) return -1;
			if(!backtrack_stack.push(592)) return -1;
			if(!backtrack_stack.push(591)) return -1;
			if(!backtrack_stack.push(529)) return -1;
			if(!backtrack_stack.push(528)) return -1;
			if(!backtrack_stack.push(513)) return -1;
			if(!backtrack_stack.push(512)) return -1;
			if(!backtrack_stack.push(511)) return -1;
			if(!backtrack_stack.push(506)) return -1;
			if(!backtrack_stack.push(455)) return -1;
			if(!backtrack_stack.push(450)) return -1;
			if(!backtrack_stack.push(441)) return -1;
			if(!backtrack_stack.push(440)) return -1;
			if(!backtrack_stack.push(439)) return -1;
			if(!backtrack_stack.push(434)) return -1;
			if(!backtrack_stack.push(339)) return -1;
			if(!backtrack_stack.push(338)) return -1;
			if(!backtrack_stack.push(249)) return -1;
			if(!backtrack_stack.push(244)) return -1;
			if(!backtrack_stack.push(52)) return -1;
			if(!backtrack_stack.push(51)) return -1;
			if(!backtrack_stack.push(50)) return -1;
			if(!backtrack_stack.push(49)) return -1;
			if(!backtrack_stack.push(48)) return -1;
			goto label_47;
		}
		break;
	case 2918:
	// Label to restore values of label 892
		v_locatedin_1 = int_value_stack.pop();
		v_locatedin_2 = int_value_stack.pop();
		break;
	case 893: label_893: 
	// using  symbolic fact locatedin(v_locatedin_1,v_locatedin_2) with literal 1 in rule 
	// 206: q47(A) :- locatedin(A,B), q48(B).
	// resulting in symbolic fact q47(v_q47_1)

		//recursive
		if(!int_value_stack.push(v_q47_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2919)) return -1;
		if (!q48_set.contains(v_locatedin_2)) continue;
	case 1906:
	// Continuation for label 893
			v_q47_1 = v_locatedin_1;
		if(q47_set.insert(v_q47_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1906)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(895)) return -1;
			if(!backtrack_stack.push(894)) return -1;
			if(!backtrack_stack.push(892)) return -1;
			goto label_889;
		}
		break;
	case 2919:
	// Label to restore values of label 893
		v_q47_1 = int_value_stack.pop();
		break;
	case 894: label_894: 
	// using  symbolic fact q47(v_q47_1) with literal 1 in rule 
	// 664: ot____nom15(A) :- q47(A), kaon2namedobjects(A).
	// resulting in symbolic fact ot____nom15(v_ot____nom15_1)

		//recursive
		if(!int_value_stack.push(v_ot____nom15_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2920)) return -1;
		if (!kaon2namedobjects_b.contains(v_q47_1)) continue;
	case 1907:
	// Continuation for label 894
			v_ot____nom15_1 = v_q47_1;
		if(ot____nom15_set.insert(v_ot____nom15_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1907)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(885)) return -1;
			goto label_884;
		}
		break;
	case 2920:
	// Label to restore values of label 894
		v_ot____nom15_1 = int_value_stack.pop();
		break;
	case 895: label_895: 
	// using  symbolic fact q47(v_q47_1) with literal 1 in rule 
	// 665: q47(A) :- q47(B), kaon2equal(A,B).
	// resulting in symbolic fact q47(v_q47_1)

		//recursive
		c665_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q47_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2921)) return -1;
		c665_2_fb.open(v_q47_1);
	case 1908:
	// Continuation for label 895
		while (c665_2_fb.fetch()) {
			v_q47_1 = c665_2_fb.c2();
		if(q47_set.insert(v_q47_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1908)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(895)) return -1;
			if(!backtrack_stack.push(894)) return -1;
			if(!backtrack_stack.push(892)) return -1;
			goto label_889;
		}
		}
		break;
	case 2921:
	// Label to restore values of label 895
		v_q47_1 = int_value_stack.pop();
		c665_2_fb.pop(&cursor_stack);
		break;
	case 896: label_896: 
	// using  symbolic fact locatedin(v_locatedin_1,v_locatedin_2) with literal 1 in rule 
	// 214: q64(A) :- locatedin(A,B), q65(B).
	// resulting in symbolic fact q64(v_q64_1)

		if (!q65_set.contains(v_locatedin_2)) continue;
	case 1909:
	// Continuation for label 896
			v_q64_1 = v_locatedin_1;
		if(q64_set.insert(v_q64_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1909)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(790)) return -1;
			if(!backtrack_stack.push(787)) return -1;
			if(!backtrack_stack.push(776)) return -1;
			goto label_775;
		}
		break;
	case 897: label_897: 
	// using  symbolic fact locatedin(v_locatedin_1,v_locatedin_2) with literal 1 in rule 
	// 222: q65(A) :- locatedin(A,B), ot____nom36(B).
	// resulting in symbolic fact q65(v_q65_1)

		if (!ot____nom36_set.contains(v_locatedin_2)) continue;
	case 1910:
	// Continuation for label 897
			v_q65_1 = v_locatedin_1;
		if(q65_set.insert(v_q65_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1910)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(786)) return -1;
			if(!backtrack_stack.push(785)) return -1;
			goto label_777;
		}
		break;
	case 898: label_898: 
	// using  symbolic fact locatedin(v_locatedin_1,v_locatedin_2) with literal 1 in rule 
	// 662: q48(A) :- locatedin(A,B), ot____nom15(B).
	// resulting in symbolic fact q48(v_q48_1)

		//recursive
		if(!int_value_stack.push(v_q48_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2924)) return -1;
		if (!ot____nom15_set.contains(v_locatedin_2)) continue;
	case 1911:
	// Continuation for label 898
			v_q48_1 = v_locatedin_1;
		if(q48_set.insert(v_q48_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1911)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(891)) return -1;
			if(!backtrack_stack.push(890)) return -1;
			goto label_886;
		}
		break;
	case 2924:
	// Label to restore values of label 898
		v_q48_1 = int_value_stack.pop();
		break;
	case 899: label_899: 
	// using  symbolic fact californiawine(v_californiawine_1) with literal 1 in rule 
	// 5: californiawine(A) :- californiawine(B), kaon2equal(A,B).
	// resulting in symbolic fact californiawine(v_californiawine_1)

		//recursive
		c5_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_californiawine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2925)) return -1;
		c5_2_fb.open(v_californiawine_1);
	case 1912:
	// Continuation for label 899
		while (c5_2_fb.fetch()) {
			v_californiawine_1 = c5_2_fb.c2();
		if(californiawine_set.insert(v_californiawine_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1912)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(900)) return -1;
			if(!backtrack_stack.push(899)) return -1;
			if(!backtrack_stack.push(888)) return -1;
			goto label_887;
		}
		}
		break;
	case 2925:
	// Label to restore values of label 899
		v_californiawine_1 = int_value_stack.pop();
		c5_2_fb.pop(&cursor_stack);
		break;
	case 900: label_900: 
	// using  symbolic fact californiawine(v_californiawine_1) with literal 1 in rule 
	// 422: wine(A) :- californiawine(A).
	// resulting in symbolic fact wine(v_wine_1)

		//recursive
		if(!int_value_stack.push(v_wine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2926)) return -1;
		v_wine_1 = v_californiawine_1;
		if(wine_set.insert(v_wine_1)) {
		if(!wine_f.insert(v_wine_1)) return -1;
		// Labels for next rules
		if(!backtrack_stack.push(1007)) return -1;
		if(!backtrack_stack.push(989)) return -1;
		if(!backtrack_stack.push(954)) return -1;
		if(!backtrack_stack.push(938)) return -1;
		if(!backtrack_stack.push(911)) return -1;
		if(!backtrack_stack.push(903)) return -1;
		if(!backtrack_stack.push(902)) return -1;
		if(!backtrack_stack.push(901)) return -1;
		if(!backtrack_stack.push(849)) return -1;
		if(!backtrack_stack.push(848)) return -1;
		if(!backtrack_stack.push(821)) return -1;
		if(!backtrack_stack.push(820)) return -1;
		if(!backtrack_stack.push(737)) return -1;
		if(!backtrack_stack.push(712)) return -1;
		if(!backtrack_stack.push(674)) return -1;
		if(!backtrack_stack.push(672)) return -1;
		if(!backtrack_stack.push(583)) return -1;
		if(!backtrack_stack.push(575)) return -1;
		if(!backtrack_stack.push(563)) return -1;
		if(!backtrack_stack.push(551)) return -1;
		if(!backtrack_stack.push(549)) return -1;
		if(!backtrack_stack.push(483)) return -1;
		if(!backtrack_stack.push(473)) return -1;
		if(!backtrack_stack.push(470)) return -1;
		if(!backtrack_stack.push(430)) return -1;
		if(!backtrack_stack.push(401)) return -1;
		if(!backtrack_stack.push(399)) return -1;
		if(!backtrack_stack.push(380)) return -1;
		if(!backtrack_stack.push(364)) return -1;
		if(!backtrack_stack.push(363)) return -1;
		if(!backtrack_stack.push(329)) return -1;
		if(!backtrack_stack.push(328)) return -1;
		if(!backtrack_stack.push(327)) return -1;
		if(!backtrack_stack.push(295)) return -1;
		if(!backtrack_stack.push(294)) return -1;
		if(!backtrack_stack.push(216)) return -1;
		if(!backtrack_stack.push(210)) return -1;
		if(!backtrack_stack.push(192)) return -1;
		if(!backtrack_stack.push(180)) return -1;
		if(!backtrack_stack.push(179)) return -1;
		if(!backtrack_stack.push(178)) return -1;
		if(!backtrack_stack.push(154)) return -1;
		if(!backtrack_stack.push(153)) return -1;
		if(!backtrack_stack.push(100)) return -1;
		if(!backtrack_stack.push(76)) return -1;
		if(!backtrack_stack.push(75)) return -1;
		if(!backtrack_stack.push(67)) return -1;
		if(!backtrack_stack.push(63)) return -1;
		if(!backtrack_stack.push(53)) return -1;
		if(!backtrack_stack.push(46)) return -1;
		if(!backtrack_stack.push(19)) return -1;
		goto label_16;
		}
		break;
	case 2926:
	// Label to restore values of label 900
		v_wine_1 = int_value_stack.pop();
		break;
	case 901: label_901: 
	// using  symbolic fact wine(v_wine_1) with literal 1 in rule 
	// 421: zinfandel(A) :- wine(A), madefromgrape(A,B), ot____nom20(B).
	// resulting in symbolic fact zinfandel(v_zinfandel_1)

		c421_2_bf.open(v_wine_1);
		while (c421_2_bf.fetch()) {
			if (!ot____nom20_set.contains(c421_2_bf.c2())) continue;
	case 1914:
	// Continuation for label 901
				v_zinfandel_1 = v_wine_1;
		if(zinfandel_set.insert(v_zinfandel_1)) {
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1914)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(857)) return -1;
				if(!backtrack_stack.push(856)) return -1;
				if(!backtrack_stack.push(853)) return -1;
				if(!backtrack_stack.push(852)) return -1;
				if(!backtrack_stack.push(851)) return -1;
				if(!backtrack_stack.push(850)) return -1;
				goto label_847;
		}
		}
		break;
	case 902: label_902: 
	// using  symbolic fact wine(v_wine_1) with literal 1 in rule 
	// 554: q5(A) :- wine(A), madefromgrape(A,B), ot____nom20(B).
	// resulting in symbolic fact q5(v_q5_1)

		c554_2_bf.open(v_wine_1);
		while (c554_2_bf.fetch()) {
			if (!ot____nom20_set.contains(c554_2_bf.c2())) continue;
	case 1915:
	// Continuation for label 902
				v_q5_1 = v_wine_1;
		if(q5_set.insert(v_q5_1)) {
				// Push backtrack label to continuation
				if(!backtrack_stack.push(1915)) return -1;
				// Labels for next rules
				if(!backtrack_stack.push(136)) return -1;
				if(!backtrack_stack.push(135)) return -1;
				goto label_131;
		}
		}
		break;
	case 903: label_903: 
	// using  symbolic fact wine(v_wine_1) with literal 2 in rule 
	// 2: californiawine(A) :- q48(A), wine(A).
	// resulting in symbolic fact californiawine(v_californiawine_1)

		//recursive
		if(!int_value_stack.push(v_californiawine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2929)) return -1;
		if (!q48_set.contains(v_wine_1)) continue;
	case 1916:
	// Continuation for label 903
			v_californiawine_1 = v_wine_1;
		if(californiawine_set.insert(v_californiawine_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1916)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(900)) return -1;
			if(!backtrack_stack.push(899)) return -1;
			if(!backtrack_stack.push(888)) return -1;
			goto label_887;
		}
		break;
	case 2929:
	// Label to restore values of label 903
		v_californiawine_1 = int_value_stack.pop();
		break;
	case 904:
	// applying rule 
	// 669: ot____nom2(A) :- ot____nom2_aux(A).
	// resulting in symbolic fact ot____nom2(v_ot____nom2_1)
		c669_1_f.open();
	case 1917:
	// Continuation for label 904
		while (c669_1_f.fetch()) {
			v_ot____nom2_1 = c669_1_f.c1();
		if(ot____nom2_set.insert(v_ot____nom2_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1917)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(523)) return -1;
			goto label_522;
		}
		}
		break;
	case 905:
	// applying rule 
	// 675: ot____nom44(A) :- ot____nom44_aux(A).
	// resulting in symbolic fact ot____nom44(v_ot____nom44_1)
		c675_1_f.open();
	case 1918:
	// Continuation for label 905
		while (c675_1_f.fetch()) {
			v_ot____nom44_1 = c675_1_f.c1();
		if(ot____nom44_set.insert(v_ot____nom44_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1918)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(410)) return -1;
			goto label_409;
		}
		}
		break;
	case 906:
	// applying rule 
	// 681: ot____nom10(A) :- ot____nom10_aux(A).
	// resulting in symbolic fact ot____nom10(v_ot____nom10_1)
		c681_1_f.open();
	case 1919:
	// Continuation for label 906
		while (c681_1_f.fetch()) {
			v_ot____nom10_1 = c681_1_f.c1();
		if(ot____nom10_set.insert(v_ot____nom10_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1919)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(908)) return -1;
			goto label_907;
		}
		}
		break;
	case 907: label_907: 
	// using  symbolic fact ot____nom10(v_ot____nom10_1) with literal 1 in rule 
	// 684: ot____nom10(A) :- ot____nom10(B), kaon2equal(A,B).
	// resulting in symbolic fact ot____nom10(v_ot____nom10_1)

		//recursive
		c684_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_ot____nom10_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2933)) return -1;
		c684_2_fb.open(v_ot____nom10_1);
	case 1920:
	// Continuation for label 907
		while (c684_2_fb.fetch()) {
			v_ot____nom10_1 = c684_2_fb.c2();
		if(ot____nom10_set.insert(v_ot____nom10_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1920)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(908)) return -1;
			goto label_907;
		}
		}
		break;
	case 2933:
	// Label to restore values of label 907
		v_ot____nom10_1 = int_value_stack.pop();
		c684_2_fb.pop(&cursor_stack);
		break;
	case 908: label_908: 
	// using  symbolic fact ot____nom10(v_ot____nom10_1) with literal 2 in rule 
	// 680: q74(A) :- locatedin(A,B), ot____nom10(B).
	// resulting in symbolic fact q74(v_q74_1)

		//recursive
		c680_1_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q74_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2934)) return -1;
		c680_1_fb.open(v_ot____nom10_1);
	case 1921:
	// Continuation for label 908
		while (c680_1_fb.fetch()) {
			v_q74_1 = c680_1_fb.c2();
		if(q74_set.insert(v_q74_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1921)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(915)) return -1;
			if(!backtrack_stack.push(914)) return -1;
			goto label_909;
		}
		}
		break;
	case 2934:
	// Label to restore values of label 908
		v_q74_1 = int_value_stack.pop();
		c680_1_fb.pop(&cursor_stack);
		break;
	case 909: label_909: 
	// using  symbolic fact q74(v_q74_1) with literal 1 in rule 
	// 30: texaswine(A) :- q74(A), wine(A).
	// resulting in symbolic fact texaswine(v_texaswine_1)

		//recursive
		if(!int_value_stack.push(v_texaswine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2935)) return -1;
		if (!wine_set.contains(v_q74_1)) continue;
	case 1922:
	// Continuation for label 909
			v_texaswine_1 = v_q74_1;
		if(texaswine_set.insert(v_texaswine_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1922)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(921)) return -1;
			if(!backtrack_stack.push(912)) return -1;
			goto label_910;
		}
		break;
	case 2935:
	// Label to restore values of label 909
		v_texaswine_1 = int_value_stack.pop();
		break;
	case 910: label_910: 
	// using  symbolic fact texaswine(v_texaswine_1) with literal 1 in rule 
	// 29: wine(A) :- texaswine(A).
	// resulting in symbolic fact wine(v_wine_1)

		//recursive
		if(!int_value_stack.push(v_wine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2936)) return -1;
		v_wine_1 = v_texaswine_1;
		if(wine_set.insert(v_wine_1)) {
		if(!wine_f.insert(v_wine_1)) return -1;
		// Labels for next rules
		if(!backtrack_stack.push(1007)) return -1;
		if(!backtrack_stack.push(989)) return -1;
		if(!backtrack_stack.push(954)) return -1;
		if(!backtrack_stack.push(938)) return -1;
		if(!backtrack_stack.push(911)) return -1;
		if(!backtrack_stack.push(903)) return -1;
		if(!backtrack_stack.push(902)) return -1;
		if(!backtrack_stack.push(901)) return -1;
		if(!backtrack_stack.push(849)) return -1;
		if(!backtrack_stack.push(848)) return -1;
		if(!backtrack_stack.push(821)) return -1;
		if(!backtrack_stack.push(820)) return -1;
		if(!backtrack_stack.push(737)) return -1;
		if(!backtrack_stack.push(712)) return -1;
		if(!backtrack_stack.push(674)) return -1;
		if(!backtrack_stack.push(672)) return -1;
		if(!backtrack_stack.push(583)) return -1;
		if(!backtrack_stack.push(575)) return -1;
		if(!backtrack_stack.push(563)) return -1;
		if(!backtrack_stack.push(551)) return -1;
		if(!backtrack_stack.push(549)) return -1;
		if(!backtrack_stack.push(483)) return -1;
		if(!backtrack_stack.push(473)) return -1;
		if(!backtrack_stack.push(470)) return -1;
		if(!backtrack_stack.push(430)) return -1;
		if(!backtrack_stack.push(401)) return -1;
		if(!backtrack_stack.push(399)) return -1;
		if(!backtrack_stack.push(380)) return -1;
		if(!backtrack_stack.push(364)) return -1;
		if(!backtrack_stack.push(363)) return -1;
		if(!backtrack_stack.push(329)) return -1;
		if(!backtrack_stack.push(328)) return -1;
		if(!backtrack_stack.push(327)) return -1;
		if(!backtrack_stack.push(295)) return -1;
		if(!backtrack_stack.push(294)) return -1;
		if(!backtrack_stack.push(216)) return -1;
		if(!backtrack_stack.push(210)) return -1;
		if(!backtrack_stack.push(192)) return -1;
		if(!backtrack_stack.push(180)) return -1;
		if(!backtrack_stack.push(179)) return -1;
		if(!backtrack_stack.push(178)) return -1;
		if(!backtrack_stack.push(154)) return -1;
		if(!backtrack_stack.push(153)) return -1;
		if(!backtrack_stack.push(100)) return -1;
		if(!backtrack_stack.push(76)) return -1;
		if(!backtrack_stack.push(75)) return -1;
		if(!backtrack_stack.push(67)) return -1;
		if(!backtrack_stack.push(63)) return -1;
		if(!backtrack_stack.push(53)) return -1;
		if(!backtrack_stack.push(46)) return -1;
		if(!backtrack_stack.push(19)) return -1;
		goto label_16;
		}
		break;
	case 2936:
	// Label to restore values of label 910
		v_wine_1 = int_value_stack.pop();
		break;
	case 911: label_911: 
	// using  symbolic fact wine(v_wine_1) with literal 2 in rule 
	// 30: texaswine(A) :- q74(A), wine(A).
	// resulting in symbolic fact texaswine(v_texaswine_1)

		//recursive
		if(!int_value_stack.push(v_texaswine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2937)) return -1;
		if (!q74_set.contains(v_wine_1)) continue;
	case 1924:
	// Continuation for label 911
			v_texaswine_1 = v_wine_1;
		if(texaswine_set.insert(v_texaswine_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1924)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(921)) return -1;
			if(!backtrack_stack.push(912)) return -1;
			goto label_910;
		}
		break;
	case 2937:
	// Label to restore values of label 911
		v_texaswine_1 = int_value_stack.pop();
		break;
	case 912: label_912: 
	// using  symbolic fact texaswine(v_texaswine_1) with literal 1 in rule 
	// 32: q73(A) :- texaswine(A).
	// resulting in symbolic fact q73(v_q73_1)

		//recursive
		if(!int_value_stack.push(v_q73_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2938)) return -1;
		v_q73_1 = v_texaswine_1;
		if(q73_set.insert(v_q73_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(919)) return -1;
		if(!backtrack_stack.push(918)) return -1;
		if(!backtrack_stack.push(916)) return -1;
		goto label_913;
		}
		break;
	case 2938:
	// Label to restore values of label 912
		v_q73_1 = int_value_stack.pop();
		break;
	case 913: label_913: 
	// using  symbolic fact q73(v_q73_1) with literal 1 in rule 
	// 31: q74(A) :- q73(A).
	// resulting in symbolic fact q74(v_q74_1)

		//recursive
		if(!int_value_stack.push(v_q74_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2939)) return -1;
		v_q74_1 = v_q73_1;
		if(q74_set.insert(v_q74_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(915)) return -1;
		if(!backtrack_stack.push(914)) return -1;
		goto label_909;
		}
		break;
	case 2939:
	// Label to restore values of label 913
		v_q74_1 = int_value_stack.pop();
		break;
	case 914: label_914: 
	// using  symbolic fact q74(v_q74_1) with literal 1 in rule 
	// 685: q74(A) :- q74(B), kaon2equal(A,B).
	// resulting in symbolic fact q74(v_q74_1)

		//recursive
		c685_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q74_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2940)) return -1;
		c685_2_fb.open(v_q74_1);
	case 1927:
	// Continuation for label 914
		while (c685_2_fb.fetch()) {
			v_q74_1 = c685_2_fb.c2();
		if(q74_set.insert(v_q74_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1927)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(915)) return -1;
			if(!backtrack_stack.push(914)) return -1;
			goto label_909;
		}
		}
		break;
	case 2940:
	// Label to restore values of label 914
		v_q74_1 = int_value_stack.pop();
		c685_2_fb.pop(&cursor_stack);
		break;
	case 915: label_915: 
	// using  symbolic fact q74(v_q74_1) with literal 2 in rule 
	// 183: q73(A) :- locatedin(A,B), q74(B).
	// resulting in symbolic fact q73(v_q73_1)

		//recursive
		c183_1_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q73_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2941)) return -1;
		c183_1_fb.open(v_q74_1);
	case 1928:
	// Continuation for label 915
		while (c183_1_fb.fetch()) {
			v_q73_1 = c183_1_fb.c2();
		if(q73_set.insert(v_q73_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1928)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(919)) return -1;
			if(!backtrack_stack.push(918)) return -1;
			if(!backtrack_stack.push(916)) return -1;
			goto label_913;
		}
		}
		break;
	case 2941:
	// Label to restore values of label 915
		v_q73_1 = int_value_stack.pop();
		c183_1_fb.pop(&cursor_stack);
		break;
	case 916: label_916: 
	// using  symbolic fact q73(v_q73_1) with literal 1 in rule 
	// 182: locatedin(A,A) :- q73(A), kaon2namedobjects(A).
	// resulting in symbolic fact locatedin(v_locatedin_1,v_locatedin_2)

		//recursive
		if(!int_value_stack.push(v_locatedin_2)) return -1;
		if(!int_value_stack.push(v_locatedin_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2942)) return -1;
		if (!kaon2namedobjects_b.contains(v_q73_1)) continue;
	case 1929:
	// Continuation for label 916
			v_locatedin_1 = v_q73_1;
			v_locatedin_2 = v_q73_1;
		if(locatedin_set.insert(v_locatedin_1, v_locatedin_2)) {
			if(!locatedin_bf.insert(v_locatedin_1, v_locatedin_2)) return -1;
			if(!locatedin_fb.insert(v_locatedin_2, v_locatedin_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1929)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(1005)) return -1;
			if(!backtrack_stack.push(999)) return -1;
			if(!backtrack_stack.push(987)) return -1;
			if(!backtrack_stack.push(981)) return -1;
			if(!backtrack_stack.push(969)) return -1;
			if(!backtrack_stack.push(963)) return -1;
			if(!backtrack_stack.push(952)) return -1;
			if(!backtrack_stack.push(946)) return -1;
			if(!backtrack_stack.push(936)) return -1;
			if(!backtrack_stack.push(930)) return -1;
			if(!backtrack_stack.push(920)) return -1;
			if(!backtrack_stack.push(917)) return -1;
			if(!backtrack_stack.push(898)) return -1;
			if(!backtrack_stack.push(897)) return -1;
			if(!backtrack_stack.push(896)) return -1;
			if(!backtrack_stack.push(893)) return -1;
			if(!backtrack_stack.push(726)) return -1;
			if(!backtrack_stack.push(721)) return -1;
			if(!backtrack_stack.push(694)) return -1;
			if(!backtrack_stack.push(693)) return -1;
			if(!backtrack_stack.push(688)) return -1;
			if(!backtrack_stack.push(687)) return -1;
			if(!backtrack_stack.push(636)) return -1;
			if(!backtrack_stack.push(631)) return -1;
			if(!backtrack_stack.push(592)) return -1;
			if(!backtrack_stack.push(591)) return -1;
			if(!backtrack_stack.push(529)) return -1;
			if(!backtrack_stack.push(528)) return -1;
			if(!backtrack_stack.push(513)) return -1;
			if(!backtrack_stack.push(512)) return -1;
			if(!backtrack_stack.push(511)) return -1;
			if(!backtrack_stack.push(506)) return -1;
			if(!backtrack_stack.push(455)) return -1;
			if(!backtrack_stack.push(450)) return -1;
			if(!backtrack_stack.push(441)) return -1;
			if(!backtrack_stack.push(440)) return -1;
			if(!backtrack_stack.push(439)) return -1;
			if(!backtrack_stack.push(434)) return -1;
			if(!backtrack_stack.push(339)) return -1;
			if(!backtrack_stack.push(338)) return -1;
			if(!backtrack_stack.push(249)) return -1;
			if(!backtrack_stack.push(244)) return -1;
			if(!backtrack_stack.push(52)) return -1;
			if(!backtrack_stack.push(51)) return -1;
			if(!backtrack_stack.push(50)) return -1;
			if(!backtrack_stack.push(49)) return -1;
			if(!backtrack_stack.push(48)) return -1;
			goto label_47;
		}
		break;
	case 2942:
	// Label to restore values of label 916
		v_locatedin_1 = int_value_stack.pop();
		v_locatedin_2 = int_value_stack.pop();
		break;
	case 917: label_917: 
	// using  symbolic fact locatedin(v_locatedin_1,v_locatedin_2) with literal 1 in rule 
	// 183: q73(A) :- locatedin(A,B), q74(B).
	// resulting in symbolic fact q73(v_q73_1)

		//recursive
		if(!int_value_stack.push(v_q73_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2943)) return -1;
		if (!q74_set.contains(v_locatedin_2)) continue;
	case 1930:
	// Continuation for label 917
			v_q73_1 = v_locatedin_1;
		if(q73_set.insert(v_q73_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1930)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(919)) return -1;
			if(!backtrack_stack.push(918)) return -1;
			if(!backtrack_stack.push(916)) return -1;
			goto label_913;
		}
		break;
	case 2943:
	// Label to restore values of label 917
		v_q73_1 = int_value_stack.pop();
		break;
	case 918: label_918: 
	// using  symbolic fact q73(v_q73_1) with literal 1 in rule 
	// 682: ot____nom10(A) :- q73(A), kaon2namedobjects(A).
	// resulting in symbolic fact ot____nom10(v_ot____nom10_1)

		//recursive
		if(!int_value_stack.push(v_ot____nom10_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2944)) return -1;
		if (!kaon2namedobjects_b.contains(v_q73_1)) continue;
	case 1931:
	// Continuation for label 918
			v_ot____nom10_1 = v_q73_1;
		if(ot____nom10_set.insert(v_ot____nom10_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1931)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(908)) return -1;
			goto label_907;
		}
		break;
	case 2944:
	// Label to restore values of label 918
		v_ot____nom10_1 = int_value_stack.pop();
		break;
	case 919: label_919: 
	// using  symbolic fact q73(v_q73_1) with literal 1 in rule 
	// 683: q73(A) :- q73(B), kaon2equal(A,B).
	// resulting in symbolic fact q73(v_q73_1)

		//recursive
		c683_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q73_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2945)) return -1;
		c683_2_fb.open(v_q73_1);
	case 1932:
	// Continuation for label 919
		while (c683_2_fb.fetch()) {
			v_q73_1 = c683_2_fb.c2();
		if(q73_set.insert(v_q73_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1932)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(919)) return -1;
			if(!backtrack_stack.push(918)) return -1;
			if(!backtrack_stack.push(916)) return -1;
			goto label_913;
		}
		}
		break;
	case 2945:
	// Label to restore values of label 919
		v_q73_1 = int_value_stack.pop();
		c683_2_fb.pop(&cursor_stack);
		break;
	case 920: label_920: 
	// using  symbolic fact locatedin(v_locatedin_1,v_locatedin_2) with literal 1 in rule 
	// 680: q74(A) :- locatedin(A,B), ot____nom10(B).
	// resulting in symbolic fact q74(v_q74_1)

		//recursive
		if(!int_value_stack.push(v_q74_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2946)) return -1;
		if (!ot____nom10_set.contains(v_locatedin_2)) continue;
	case 1933:
	// Continuation for label 920
			v_q74_1 = v_locatedin_1;
		if(q74_set.insert(v_q74_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1933)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(915)) return -1;
			if(!backtrack_stack.push(914)) return -1;
			goto label_909;
		}
		break;
	case 2946:
	// Label to restore values of label 920
		v_q74_1 = int_value_stack.pop();
		break;
	case 921: label_921: 
	// using  symbolic fact texaswine(v_texaswine_1) with literal 1 in rule 
	// 33: texaswine(A) :- texaswine(B), kaon2equal(A,B).
	// resulting in symbolic fact texaswine(v_texaswine_1)

		//recursive
		c33_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_texaswine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2947)) return -1;
		c33_2_fb.open(v_texaswine_1);
	case 1934:
	// Continuation for label 921
		while (c33_2_fb.fetch()) {
			v_texaswine_1 = c33_2_fb.c2();
		if(texaswine_set.insert(v_texaswine_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1934)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(921)) return -1;
			if(!backtrack_stack.push(912)) return -1;
			goto label_910;
		}
		}
		break;
	case 2947:
	// Label to restore values of label 921
		v_texaswine_1 = int_value_stack.pop();
		c33_2_fb.pop(&cursor_stack);
		break;
	case 922:
	// applying rule 
	// 687: ot____nom31(A) :- ot____nom31_aux(A).
	// resulting in symbolic fact ot____nom31(v_ot____nom31_1)
		c687_1_f.open();
	case 1935:
	// Continuation for label 922
		while (c687_1_f.fetch()) {
			v_ot____nom31_1 = c687_1_f.c1();
		if(ot____nom31_set.insert(v_ot____nom31_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1935)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(437)) return -1;
			goto label_436;
		}
		}
		break;
	case 923:
	// applying rule 
	// 693: ot____nom16(A) :- ot____nom16_aux(A).
	// resulting in symbolic fact ot____nom16(v_ot____nom16_1)
		c693_1_f.open();
	case 1936:
	// Continuation for label 923
		while (c693_1_f.fetch()) {
			v_ot____nom16_1 = c693_1_f.c1();
		if(ot____nom16_set.insert(v_ot____nom16_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1936)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(453)) return -1;
			goto label_452;
		}
		}
		break;
	case 924:
	// applying rule 
	// 699: ot____nom41(A) :- ot____nom41_aux(A).
	// resulting in symbolic fact ot____nom41(v_ot____nom41_1)
		c699_1_f.open();
	case 1937:
	// Continuation for label 924
		while (c699_1_f.fetch()) {
			v_ot____nom41_1 = c699_1_f.c1();
		if(ot____nom41_set.insert(v_ot____nom41_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1937)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(926)) return -1;
			goto label_925;
		}
		}
		break;
	case 925: label_925: 
	// using  symbolic fact ot____nom41(v_ot____nom41_1) with literal 1 in rule 
	// 702: ot____nom41(A) :- ot____nom41(B), kaon2equal(A,B).
	// resulting in symbolic fact ot____nom41(v_ot____nom41_1)

		//recursive
		c702_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_ot____nom41_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2951)) return -1;
		c702_2_fb.open(v_ot____nom41_1);
	case 1938:
	// Continuation for label 925
		while (c702_2_fb.fetch()) {
			v_ot____nom41_1 = c702_2_fb.c2();
		if(ot____nom41_set.insert(v_ot____nom41_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1938)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(926)) return -1;
			goto label_925;
		}
		}
		break;
	case 2951:
	// Label to restore values of label 925
		v_ot____nom41_1 = int_value_stack.pop();
		c702_2_fb.pop(&cursor_stack);
		break;
	case 926: label_926: 
	// using  symbolic fact ot____nom41(v_ot____nom41_1) with literal 2 in rule 
	// 698: q52(A) :- locatedin(A,B), ot____nom41(B).
	// resulting in symbolic fact q52(v_q52_1)

		//recursive
		c698_1_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q52_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2952)) return -1;
		c698_1_fb.open(v_ot____nom41_1);
	case 1939:
	// Continuation for label 926
		while (c698_1_fb.fetch()) {
			v_q52_1 = c698_1_fb.c2();
		if(q52_set.insert(v_q52_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1939)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(933)) return -1;
			if(!backtrack_stack.push(932)) return -1;
			goto label_927;
		}
		}
		break;
	case 2952:
	// Label to restore values of label 926
		v_q52_1 = int_value_stack.pop();
		c698_1_fb.pop(&cursor_stack);
		break;
	case 927: label_927: 
	// using  symbolic fact q52(v_q52_1) with literal 1 in rule 
	// 175: frenchwine(A) :- q52(A), wine(A).
	// resulting in symbolic fact frenchwine(v_frenchwine_1)

		//recursive
		if(!int_value_stack.push(v_frenchwine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2953)) return -1;
		if (!wine_set.contains(v_q52_1)) continue;
	case 1940:
	// Continuation for label 927
			v_frenchwine_1 = v_q52_1;
		if(frenchwine_set.insert(v_frenchwine_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1940)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(939)) return -1;
			if(!backtrack_stack.push(937)) return -1;
			goto label_928;
		}
		break;
	case 2953:
	// Label to restore values of label 927
		v_frenchwine_1 = int_value_stack.pop();
		break;
	case 928: label_928: 
	// using  symbolic fact frenchwine(v_frenchwine_1) with literal 1 in rule 
	// 174: q51(A) :- frenchwine(A).
	// resulting in symbolic fact q51(v_q51_1)

		//recursive
		if(!int_value_stack.push(v_q51_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2954)) return -1;
		v_q51_1 = v_frenchwine_1;
		if(q51_set.insert(v_q51_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(935)) return -1;
		if(!backtrack_stack.push(934)) return -1;
		if(!backtrack_stack.push(931)) return -1;
		goto label_929;
		}
		break;
	case 2954:
	// Label to restore values of label 928
		v_q51_1 = int_value_stack.pop();
		break;
	case 929: label_929: 
	// using  symbolic fact q51(v_q51_1) with literal 1 in rule 
	// 173: locatedin(A,A) :- q51(A), kaon2namedobjects(A).
	// resulting in symbolic fact locatedin(v_locatedin_1,v_locatedin_2)

		//recursive
		if(!int_value_stack.push(v_locatedin_2)) return -1;
		if(!int_value_stack.push(v_locatedin_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2955)) return -1;
		if (!kaon2namedobjects_b.contains(v_q51_1)) continue;
	case 1942:
	// Continuation for label 929
			v_locatedin_1 = v_q51_1;
			v_locatedin_2 = v_q51_1;
		if(locatedin_set.insert(v_locatedin_1, v_locatedin_2)) {
			if(!locatedin_bf.insert(v_locatedin_1, v_locatedin_2)) return -1;
			if(!locatedin_fb.insert(v_locatedin_2, v_locatedin_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1942)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(1005)) return -1;
			if(!backtrack_stack.push(999)) return -1;
			if(!backtrack_stack.push(987)) return -1;
			if(!backtrack_stack.push(981)) return -1;
			if(!backtrack_stack.push(969)) return -1;
			if(!backtrack_stack.push(963)) return -1;
			if(!backtrack_stack.push(952)) return -1;
			if(!backtrack_stack.push(946)) return -1;
			if(!backtrack_stack.push(936)) return -1;
			if(!backtrack_stack.push(930)) return -1;
			if(!backtrack_stack.push(920)) return -1;
			if(!backtrack_stack.push(917)) return -1;
			if(!backtrack_stack.push(898)) return -1;
			if(!backtrack_stack.push(897)) return -1;
			if(!backtrack_stack.push(896)) return -1;
			if(!backtrack_stack.push(893)) return -1;
			if(!backtrack_stack.push(726)) return -1;
			if(!backtrack_stack.push(721)) return -1;
			if(!backtrack_stack.push(694)) return -1;
			if(!backtrack_stack.push(693)) return -1;
			if(!backtrack_stack.push(688)) return -1;
			if(!backtrack_stack.push(687)) return -1;
			if(!backtrack_stack.push(636)) return -1;
			if(!backtrack_stack.push(631)) return -1;
			if(!backtrack_stack.push(592)) return -1;
			if(!backtrack_stack.push(591)) return -1;
			if(!backtrack_stack.push(529)) return -1;
			if(!backtrack_stack.push(528)) return -1;
			if(!backtrack_stack.push(513)) return -1;
			if(!backtrack_stack.push(512)) return -1;
			if(!backtrack_stack.push(511)) return -1;
			if(!backtrack_stack.push(506)) return -1;
			if(!backtrack_stack.push(455)) return -1;
			if(!backtrack_stack.push(450)) return -1;
			if(!backtrack_stack.push(441)) return -1;
			if(!backtrack_stack.push(440)) return -1;
			if(!backtrack_stack.push(439)) return -1;
			if(!backtrack_stack.push(434)) return -1;
			if(!backtrack_stack.push(339)) return -1;
			if(!backtrack_stack.push(338)) return -1;
			if(!backtrack_stack.push(249)) return -1;
			if(!backtrack_stack.push(244)) return -1;
			if(!backtrack_stack.push(52)) return -1;
			if(!backtrack_stack.push(51)) return -1;
			if(!backtrack_stack.push(50)) return -1;
			if(!backtrack_stack.push(49)) return -1;
			if(!backtrack_stack.push(48)) return -1;
			goto label_47;
		}
		break;
	case 2955:
	// Label to restore values of label 929
		v_locatedin_1 = int_value_stack.pop();
		v_locatedin_2 = int_value_stack.pop();
		break;
	case 930: label_930: 
	// using  symbolic fact locatedin(v_locatedin_1,v_locatedin_2) with literal 1 in rule 
	// 177: q51(A) :- locatedin(A,B), q52(B).
	// resulting in symbolic fact q51(v_q51_1)

		//recursive
		if(!int_value_stack.push(v_q51_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2956)) return -1;
		if (!q52_set.contains(v_locatedin_2)) continue;
	case 1943:
	// Continuation for label 930
			v_q51_1 = v_locatedin_1;
		if(q51_set.insert(v_q51_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1943)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(935)) return -1;
			if(!backtrack_stack.push(934)) return -1;
			if(!backtrack_stack.push(931)) return -1;
			goto label_929;
		}
		break;
	case 2956:
	// Label to restore values of label 930
		v_q51_1 = int_value_stack.pop();
		break;
	case 931: label_931: 
	// using  symbolic fact q51(v_q51_1) with literal 1 in rule 
	// 176: q52(A) :- q51(A).
	// resulting in symbolic fact q52(v_q52_1)

		//recursive
		if(!int_value_stack.push(v_q52_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2957)) return -1;
		v_q52_1 = v_q51_1;
		if(q52_set.insert(v_q52_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(933)) return -1;
		if(!backtrack_stack.push(932)) return -1;
		goto label_927;
		}
		break;
	case 2957:
	// Label to restore values of label 931
		v_q52_1 = int_value_stack.pop();
		break;
	case 932: label_932: 
	// using  symbolic fact q52(v_q52_1) with literal 1 in rule 
	// 703: q52(A) :- q52(B), kaon2equal(A,B).
	// resulting in symbolic fact q52(v_q52_1)

		//recursive
		c703_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q52_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2958)) return -1;
		c703_2_fb.open(v_q52_1);
	case 1945:
	// Continuation for label 932
		while (c703_2_fb.fetch()) {
			v_q52_1 = c703_2_fb.c2();
		if(q52_set.insert(v_q52_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1945)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(933)) return -1;
			if(!backtrack_stack.push(932)) return -1;
			goto label_927;
		}
		}
		break;
	case 2958:
	// Label to restore values of label 932
		v_q52_1 = int_value_stack.pop();
		c703_2_fb.pop(&cursor_stack);
		break;
	case 933: label_933: 
	// using  symbolic fact q52(v_q52_1) with literal 2 in rule 
	// 177: q51(A) :- locatedin(A,B), q52(B).
	// resulting in symbolic fact q51(v_q51_1)

		//recursive
		c177_1_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q51_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2959)) return -1;
		c177_1_fb.open(v_q52_1);
	case 1946:
	// Continuation for label 933
		while (c177_1_fb.fetch()) {
			v_q51_1 = c177_1_fb.c2();
		if(q51_set.insert(v_q51_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1946)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(935)) return -1;
			if(!backtrack_stack.push(934)) return -1;
			if(!backtrack_stack.push(931)) return -1;
			goto label_929;
		}
		}
		break;
	case 2959:
	// Label to restore values of label 933
		v_q51_1 = int_value_stack.pop();
		c177_1_fb.pop(&cursor_stack);
		break;
	case 934: label_934: 
	// using  symbolic fact q51(v_q51_1) with literal 1 in rule 
	// 700: ot____nom41(A) :- q51(A), kaon2namedobjects(A).
	// resulting in symbolic fact ot____nom41(v_ot____nom41_1)

		//recursive
		if(!int_value_stack.push(v_ot____nom41_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2960)) return -1;
		if (!kaon2namedobjects_b.contains(v_q51_1)) continue;
	case 1947:
	// Continuation for label 934
			v_ot____nom41_1 = v_q51_1;
		if(ot____nom41_set.insert(v_ot____nom41_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1947)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(926)) return -1;
			goto label_925;
		}
		break;
	case 2960:
	// Label to restore values of label 934
		v_ot____nom41_1 = int_value_stack.pop();
		break;
	case 935: label_935: 
	// using  symbolic fact q51(v_q51_1) with literal 1 in rule 
	// 701: q51(A) :- q51(B), kaon2equal(A,B).
	// resulting in symbolic fact q51(v_q51_1)

		//recursive
		c701_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q51_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2961)) return -1;
		c701_2_fb.open(v_q51_1);
	case 1948:
	// Continuation for label 935
		while (c701_2_fb.fetch()) {
			v_q51_1 = c701_2_fb.c2();
		if(q51_set.insert(v_q51_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1948)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(935)) return -1;
			if(!backtrack_stack.push(934)) return -1;
			if(!backtrack_stack.push(931)) return -1;
			goto label_929;
		}
		}
		break;
	case 2961:
	// Label to restore values of label 935
		v_q51_1 = int_value_stack.pop();
		c701_2_fb.pop(&cursor_stack);
		break;
	case 936: label_936: 
	// using  symbolic fact locatedin(v_locatedin_1,v_locatedin_2) with literal 1 in rule 
	// 698: q52(A) :- locatedin(A,B), ot____nom41(B).
	// resulting in symbolic fact q52(v_q52_1)

		//recursive
		if(!int_value_stack.push(v_q52_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2962)) return -1;
		if (!ot____nom41_set.contains(v_locatedin_2)) continue;
	case 1949:
	// Continuation for label 936
			v_q52_1 = v_locatedin_1;
		if(q52_set.insert(v_q52_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1949)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(933)) return -1;
			if(!backtrack_stack.push(932)) return -1;
			goto label_927;
		}
		break;
	case 2962:
	// Label to restore values of label 936
		v_q52_1 = int_value_stack.pop();
		break;
	case 937: label_937: 
	// using  symbolic fact frenchwine(v_frenchwine_1) with literal 1 in rule 
	// 415: wine(A) :- frenchwine(A).
	// resulting in symbolic fact wine(v_wine_1)

		//recursive
		if(!int_value_stack.push(v_wine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2963)) return -1;
		v_wine_1 = v_frenchwine_1;
		if(wine_set.insert(v_wine_1)) {
		if(!wine_f.insert(v_wine_1)) return -1;
		// Labels for next rules
		if(!backtrack_stack.push(1007)) return -1;
		if(!backtrack_stack.push(989)) return -1;
		if(!backtrack_stack.push(954)) return -1;
		if(!backtrack_stack.push(938)) return -1;
		if(!backtrack_stack.push(911)) return -1;
		if(!backtrack_stack.push(903)) return -1;
		if(!backtrack_stack.push(902)) return -1;
		if(!backtrack_stack.push(901)) return -1;
		if(!backtrack_stack.push(849)) return -1;
		if(!backtrack_stack.push(848)) return -1;
		if(!backtrack_stack.push(821)) return -1;
		if(!backtrack_stack.push(820)) return -1;
		if(!backtrack_stack.push(737)) return -1;
		if(!backtrack_stack.push(712)) return -1;
		if(!backtrack_stack.push(674)) return -1;
		if(!backtrack_stack.push(672)) return -1;
		if(!backtrack_stack.push(583)) return -1;
		if(!backtrack_stack.push(575)) return -1;
		if(!backtrack_stack.push(563)) return -1;
		if(!backtrack_stack.push(551)) return -1;
		if(!backtrack_stack.push(549)) return -1;
		if(!backtrack_stack.push(483)) return -1;
		if(!backtrack_stack.push(473)) return -1;
		if(!backtrack_stack.push(470)) return -1;
		if(!backtrack_stack.push(430)) return -1;
		if(!backtrack_stack.push(401)) return -1;
		if(!backtrack_stack.push(399)) return -1;
		if(!backtrack_stack.push(380)) return -1;
		if(!backtrack_stack.push(364)) return -1;
		if(!backtrack_stack.push(363)) return -1;
		if(!backtrack_stack.push(329)) return -1;
		if(!backtrack_stack.push(328)) return -1;
		if(!backtrack_stack.push(327)) return -1;
		if(!backtrack_stack.push(295)) return -1;
		if(!backtrack_stack.push(294)) return -1;
		if(!backtrack_stack.push(216)) return -1;
		if(!backtrack_stack.push(210)) return -1;
		if(!backtrack_stack.push(192)) return -1;
		if(!backtrack_stack.push(180)) return -1;
		if(!backtrack_stack.push(179)) return -1;
		if(!backtrack_stack.push(178)) return -1;
		if(!backtrack_stack.push(154)) return -1;
		if(!backtrack_stack.push(153)) return -1;
		if(!backtrack_stack.push(100)) return -1;
		if(!backtrack_stack.push(76)) return -1;
		if(!backtrack_stack.push(75)) return -1;
		if(!backtrack_stack.push(67)) return -1;
		if(!backtrack_stack.push(63)) return -1;
		if(!backtrack_stack.push(53)) return -1;
		if(!backtrack_stack.push(46)) return -1;
		if(!backtrack_stack.push(19)) return -1;
		goto label_16;
		}
		break;
	case 2963:
	// Label to restore values of label 937
		v_wine_1 = int_value_stack.pop();
		break;
	case 938: label_938: 
	// using  symbolic fact wine(v_wine_1) with literal 2 in rule 
	// 175: frenchwine(A) :- q52(A), wine(A).
	// resulting in symbolic fact frenchwine(v_frenchwine_1)

		//recursive
		if(!int_value_stack.push(v_frenchwine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2964)) return -1;
		if (!q52_set.contains(v_wine_1)) continue;
	case 1951:
	// Continuation for label 938
			v_frenchwine_1 = v_wine_1;
		if(frenchwine_set.insert(v_frenchwine_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1951)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(939)) return -1;
			if(!backtrack_stack.push(937)) return -1;
			goto label_928;
		}
		break;
	case 2964:
	// Label to restore values of label 938
		v_frenchwine_1 = int_value_stack.pop();
		break;
	case 939: label_939: 
	// using  symbolic fact frenchwine(v_frenchwine_1) with literal 1 in rule 
	// 416: frenchwine(A) :- frenchwine(B), kaon2equal(A,B).
	// resulting in symbolic fact frenchwine(v_frenchwine_1)

		//recursive
		c416_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_frenchwine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2965)) return -1;
		c416_2_fb.open(v_frenchwine_1);
	case 1952:
	// Continuation for label 939
		while (c416_2_fb.fetch()) {
			v_frenchwine_1 = c416_2_fb.c2();
		if(frenchwine_set.insert(v_frenchwine_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1952)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(939)) return -1;
			if(!backtrack_stack.push(937)) return -1;
			goto label_928;
		}
		}
		break;
	case 2965:
	// Label to restore values of label 939
		v_frenchwine_1 = int_value_stack.pop();
		c416_2_fb.pop(&cursor_stack);
		break;
	case 940:
	// applying rule 
	// 705: ot____nom38(A) :- ot____nom38_aux(A).
	// resulting in symbolic fact ot____nom38(v_ot____nom38_1)
		c705_1_f.open();
	case 1953:
	// Continuation for label 940
		while (c705_1_f.fetch()) {
			v_ot____nom38_1 = c705_1_f.c1();
		if(ot____nom38_set.insert(v_ot____nom38_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1953)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(942)) return -1;
			goto label_941;
		}
		}
		break;
	case 941: label_941: 
	// using  symbolic fact ot____nom38(v_ot____nom38_1) with literal 1 in rule 
	// 708: ot____nom38(A) :- ot____nom38(B), kaon2equal(A,B).
	// resulting in symbolic fact ot____nom38(v_ot____nom38_1)

		//recursive
		c708_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_ot____nom38_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2967)) return -1;
		c708_2_fb.open(v_ot____nom38_1);
	case 1954:
	// Continuation for label 941
		while (c708_2_fb.fetch()) {
			v_ot____nom38_1 = c708_2_fb.c2();
		if(ot____nom38_set.insert(v_ot____nom38_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1954)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(942)) return -1;
			goto label_941;
		}
		}
		break;
	case 2967:
	// Label to restore values of label 941
		v_ot____nom38_1 = int_value_stack.pop();
		c708_2_fb.pop(&cursor_stack);
		break;
	case 942: label_942: 
	// using  symbolic fact ot____nom38(v_ot____nom38_1) with literal 2 in rule 
	// 704: q27(A) :- locatedin(A,B), ot____nom38(B).
	// resulting in symbolic fact q27(v_q27_1)

		//recursive
		c704_1_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q27_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2968)) return -1;
		c704_1_fb.open(v_ot____nom38_1);
	case 1955:
	// Continuation for label 942
		while (c704_1_fb.fetch()) {
			v_q27_1 = c704_1_fb.c2();
		if(q27_set.insert(v_q27_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1955)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(949)) return -1;
			if(!backtrack_stack.push(948)) return -1;
			goto label_943;
		}
		}
		break;
	case 2968:
	// Label to restore values of label 942
		v_q27_1 = int_value_stack.pop();
		c704_1_fb.pop(&cursor_stack);
		break;
	case 943: label_943: 
	// using  symbolic fact q27(v_q27_1) with literal 1 in rule 
	// 170: americanwine(A) :- q27(A), wine(A).
	// resulting in symbolic fact americanwine(v_americanwine_1)

		//recursive
		if(!int_value_stack.push(v_americanwine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2969)) return -1;
		if (!wine_set.contains(v_q27_1)) continue;
	case 1956:
	// Continuation for label 943
			v_americanwine_1 = v_q27_1;
		if(americanwine_set.insert(v_americanwine_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1956)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(955)) return -1;
			if(!backtrack_stack.push(953)) return -1;
			goto label_944;
		}
		break;
	case 2969:
	// Label to restore values of label 943
		v_americanwine_1 = int_value_stack.pop();
		break;
	case 944: label_944: 
	// using  symbolic fact americanwine(v_americanwine_1) with literal 1 in rule 
	// 169: q26(A) :- americanwine(A).
	// resulting in symbolic fact q26(v_q26_1)

		//recursive
		if(!int_value_stack.push(v_q26_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2970)) return -1;
		v_q26_1 = v_americanwine_1;
		if(q26_set.insert(v_q26_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(951)) return -1;
		if(!backtrack_stack.push(950)) return -1;
		if(!backtrack_stack.push(947)) return -1;
		goto label_945;
		}
		break;
	case 2970:
	// Label to restore values of label 944
		v_q26_1 = int_value_stack.pop();
		break;
	case 945: label_945: 
	// using  symbolic fact q26(v_q26_1) with literal 1 in rule 
	// 168: locatedin(A,A) :- q26(A), kaon2namedobjects(A).
	// resulting in symbolic fact locatedin(v_locatedin_1,v_locatedin_2)

		//recursive
		if(!int_value_stack.push(v_locatedin_2)) return -1;
		if(!int_value_stack.push(v_locatedin_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2971)) return -1;
		if (!kaon2namedobjects_b.contains(v_q26_1)) continue;
	case 1958:
	// Continuation for label 945
			v_locatedin_1 = v_q26_1;
			v_locatedin_2 = v_q26_1;
		if(locatedin_set.insert(v_locatedin_1, v_locatedin_2)) {
			if(!locatedin_bf.insert(v_locatedin_1, v_locatedin_2)) return -1;
			if(!locatedin_fb.insert(v_locatedin_2, v_locatedin_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1958)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(1005)) return -1;
			if(!backtrack_stack.push(999)) return -1;
			if(!backtrack_stack.push(987)) return -1;
			if(!backtrack_stack.push(981)) return -1;
			if(!backtrack_stack.push(969)) return -1;
			if(!backtrack_stack.push(963)) return -1;
			if(!backtrack_stack.push(952)) return -1;
			if(!backtrack_stack.push(946)) return -1;
			if(!backtrack_stack.push(936)) return -1;
			if(!backtrack_stack.push(930)) return -1;
			if(!backtrack_stack.push(920)) return -1;
			if(!backtrack_stack.push(917)) return -1;
			if(!backtrack_stack.push(898)) return -1;
			if(!backtrack_stack.push(897)) return -1;
			if(!backtrack_stack.push(896)) return -1;
			if(!backtrack_stack.push(893)) return -1;
			if(!backtrack_stack.push(726)) return -1;
			if(!backtrack_stack.push(721)) return -1;
			if(!backtrack_stack.push(694)) return -1;
			if(!backtrack_stack.push(693)) return -1;
			if(!backtrack_stack.push(688)) return -1;
			if(!backtrack_stack.push(687)) return -1;
			if(!backtrack_stack.push(636)) return -1;
			if(!backtrack_stack.push(631)) return -1;
			if(!backtrack_stack.push(592)) return -1;
			if(!backtrack_stack.push(591)) return -1;
			if(!backtrack_stack.push(529)) return -1;
			if(!backtrack_stack.push(528)) return -1;
			if(!backtrack_stack.push(513)) return -1;
			if(!backtrack_stack.push(512)) return -1;
			if(!backtrack_stack.push(511)) return -1;
			if(!backtrack_stack.push(506)) return -1;
			if(!backtrack_stack.push(455)) return -1;
			if(!backtrack_stack.push(450)) return -1;
			if(!backtrack_stack.push(441)) return -1;
			if(!backtrack_stack.push(440)) return -1;
			if(!backtrack_stack.push(439)) return -1;
			if(!backtrack_stack.push(434)) return -1;
			if(!backtrack_stack.push(339)) return -1;
			if(!backtrack_stack.push(338)) return -1;
			if(!backtrack_stack.push(249)) return -1;
			if(!backtrack_stack.push(244)) return -1;
			if(!backtrack_stack.push(52)) return -1;
			if(!backtrack_stack.push(51)) return -1;
			if(!backtrack_stack.push(50)) return -1;
			if(!backtrack_stack.push(49)) return -1;
			if(!backtrack_stack.push(48)) return -1;
			goto label_47;
		}
		break;
	case 2971:
	// Label to restore values of label 945
		v_locatedin_1 = int_value_stack.pop();
		v_locatedin_2 = int_value_stack.pop();
		break;
	case 946: label_946: 
	// using  symbolic fact locatedin(v_locatedin_1,v_locatedin_2) with literal 1 in rule 
	// 172: q26(A) :- locatedin(A,B), q27(B).
	// resulting in symbolic fact q26(v_q26_1)

		//recursive
		if(!int_value_stack.push(v_q26_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2972)) return -1;
		if (!q27_set.contains(v_locatedin_2)) continue;
	case 1959:
	// Continuation for label 946
			v_q26_1 = v_locatedin_1;
		if(q26_set.insert(v_q26_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1959)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(951)) return -1;
			if(!backtrack_stack.push(950)) return -1;
			if(!backtrack_stack.push(947)) return -1;
			goto label_945;
		}
		break;
	case 2972:
	// Label to restore values of label 946
		v_q26_1 = int_value_stack.pop();
		break;
	case 947: label_947: 
	// using  symbolic fact q26(v_q26_1) with literal 1 in rule 
	// 171: q27(A) :- q26(A).
	// resulting in symbolic fact q27(v_q27_1)

		//recursive
		if(!int_value_stack.push(v_q27_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2973)) return -1;
		v_q27_1 = v_q26_1;
		if(q27_set.insert(v_q27_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(949)) return -1;
		if(!backtrack_stack.push(948)) return -1;
		goto label_943;
		}
		break;
	case 2973:
	// Label to restore values of label 947
		v_q27_1 = int_value_stack.pop();
		break;
	case 948: label_948: 
	// using  symbolic fact q27(v_q27_1) with literal 1 in rule 
	// 709: q27(A) :- q27(B), kaon2equal(A,B).
	// resulting in symbolic fact q27(v_q27_1)

		//recursive
		c709_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q27_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2974)) return -1;
		c709_2_fb.open(v_q27_1);
	case 1961:
	// Continuation for label 948
		while (c709_2_fb.fetch()) {
			v_q27_1 = c709_2_fb.c2();
		if(q27_set.insert(v_q27_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1961)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(949)) return -1;
			if(!backtrack_stack.push(948)) return -1;
			goto label_943;
		}
		}
		break;
	case 2974:
	// Label to restore values of label 948
		v_q27_1 = int_value_stack.pop();
		c709_2_fb.pop(&cursor_stack);
		break;
	case 949: label_949: 
	// using  symbolic fact q27(v_q27_1) with literal 2 in rule 
	// 172: q26(A) :- locatedin(A,B), q27(B).
	// resulting in symbolic fact q26(v_q26_1)

		//recursive
		c172_1_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q26_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2975)) return -1;
		c172_1_fb.open(v_q27_1);
	case 1962:
	// Continuation for label 949
		while (c172_1_fb.fetch()) {
			v_q26_1 = c172_1_fb.c2();
		if(q26_set.insert(v_q26_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1962)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(951)) return -1;
			if(!backtrack_stack.push(950)) return -1;
			if(!backtrack_stack.push(947)) return -1;
			goto label_945;
		}
		}
		break;
	case 2975:
	// Label to restore values of label 949
		v_q26_1 = int_value_stack.pop();
		c172_1_fb.pop(&cursor_stack);
		break;
	case 950: label_950: 
	// using  symbolic fact q26(v_q26_1) with literal 1 in rule 
	// 706: ot____nom38(A) :- q26(A), kaon2namedobjects(A).
	// resulting in symbolic fact ot____nom38(v_ot____nom38_1)

		//recursive
		if(!int_value_stack.push(v_ot____nom38_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2976)) return -1;
		if (!kaon2namedobjects_b.contains(v_q26_1)) continue;
	case 1963:
	// Continuation for label 950
			v_ot____nom38_1 = v_q26_1;
		if(ot____nom38_set.insert(v_ot____nom38_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1963)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(942)) return -1;
			goto label_941;
		}
		break;
	case 2976:
	// Label to restore values of label 950
		v_ot____nom38_1 = int_value_stack.pop();
		break;
	case 951: label_951: 
	// using  symbolic fact q26(v_q26_1) with literal 1 in rule 
	// 707: q26(A) :- q26(B), kaon2equal(A,B).
	// resulting in symbolic fact q26(v_q26_1)

		//recursive
		c707_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q26_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2977)) return -1;
		c707_2_fb.open(v_q26_1);
	case 1964:
	// Continuation for label 951
		while (c707_2_fb.fetch()) {
			v_q26_1 = c707_2_fb.c2();
		if(q26_set.insert(v_q26_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1964)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(951)) return -1;
			if(!backtrack_stack.push(950)) return -1;
			if(!backtrack_stack.push(947)) return -1;
			goto label_945;
		}
		}
		break;
	case 2977:
	// Label to restore values of label 951
		v_q26_1 = int_value_stack.pop();
		c707_2_fb.pop(&cursor_stack);
		break;
	case 952: label_952: 
	// using  symbolic fact locatedin(v_locatedin_1,v_locatedin_2) with literal 1 in rule 
	// 704: q27(A) :- locatedin(A,B), ot____nom38(B).
	// resulting in symbolic fact q27(v_q27_1)

		//recursive
		if(!int_value_stack.push(v_q27_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2978)) return -1;
		if (!ot____nom38_set.contains(v_locatedin_2)) continue;
	case 1965:
	// Continuation for label 952
			v_q27_1 = v_locatedin_1;
		if(q27_set.insert(v_q27_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1965)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(949)) return -1;
			if(!backtrack_stack.push(948)) return -1;
			goto label_943;
		}
		break;
	case 2978:
	// Label to restore values of label 952
		v_q27_1 = int_value_stack.pop();
		break;
	case 953: label_953: 
	// using  symbolic fact americanwine(v_americanwine_1) with literal 1 in rule 
	// 315: wine(A) :- americanwine(A).
	// resulting in symbolic fact wine(v_wine_1)

		//recursive
		if(!int_value_stack.push(v_wine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2979)) return -1;
		v_wine_1 = v_americanwine_1;
		if(wine_set.insert(v_wine_1)) {
		if(!wine_f.insert(v_wine_1)) return -1;
		// Labels for next rules
		if(!backtrack_stack.push(1007)) return -1;
		if(!backtrack_stack.push(989)) return -1;
		if(!backtrack_stack.push(954)) return -1;
		if(!backtrack_stack.push(938)) return -1;
		if(!backtrack_stack.push(911)) return -1;
		if(!backtrack_stack.push(903)) return -1;
		if(!backtrack_stack.push(902)) return -1;
		if(!backtrack_stack.push(901)) return -1;
		if(!backtrack_stack.push(849)) return -1;
		if(!backtrack_stack.push(848)) return -1;
		if(!backtrack_stack.push(821)) return -1;
		if(!backtrack_stack.push(820)) return -1;
		if(!backtrack_stack.push(737)) return -1;
		if(!backtrack_stack.push(712)) return -1;
		if(!backtrack_stack.push(674)) return -1;
		if(!backtrack_stack.push(672)) return -1;
		if(!backtrack_stack.push(583)) return -1;
		if(!backtrack_stack.push(575)) return -1;
		if(!backtrack_stack.push(563)) return -1;
		if(!backtrack_stack.push(551)) return -1;
		if(!backtrack_stack.push(549)) return -1;
		if(!backtrack_stack.push(483)) return -1;
		if(!backtrack_stack.push(473)) return -1;
		if(!backtrack_stack.push(470)) return -1;
		if(!backtrack_stack.push(430)) return -1;
		if(!backtrack_stack.push(401)) return -1;
		if(!backtrack_stack.push(399)) return -1;
		if(!backtrack_stack.push(380)) return -1;
		if(!backtrack_stack.push(364)) return -1;
		if(!backtrack_stack.push(363)) return -1;
		if(!backtrack_stack.push(329)) return -1;
		if(!backtrack_stack.push(328)) return -1;
		if(!backtrack_stack.push(327)) return -1;
		if(!backtrack_stack.push(295)) return -1;
		if(!backtrack_stack.push(294)) return -1;
		if(!backtrack_stack.push(216)) return -1;
		if(!backtrack_stack.push(210)) return -1;
		if(!backtrack_stack.push(192)) return -1;
		if(!backtrack_stack.push(180)) return -1;
		if(!backtrack_stack.push(179)) return -1;
		if(!backtrack_stack.push(178)) return -1;
		if(!backtrack_stack.push(154)) return -1;
		if(!backtrack_stack.push(153)) return -1;
		if(!backtrack_stack.push(100)) return -1;
		if(!backtrack_stack.push(76)) return -1;
		if(!backtrack_stack.push(75)) return -1;
		if(!backtrack_stack.push(67)) return -1;
		if(!backtrack_stack.push(63)) return -1;
		if(!backtrack_stack.push(53)) return -1;
		if(!backtrack_stack.push(46)) return -1;
		if(!backtrack_stack.push(19)) return -1;
		goto label_16;
		}
		break;
	case 2979:
	// Label to restore values of label 953
		v_wine_1 = int_value_stack.pop();
		break;
	case 954: label_954: 
	// using  symbolic fact wine(v_wine_1) with literal 2 in rule 
	// 170: americanwine(A) :- q27(A), wine(A).
	// resulting in symbolic fact americanwine(v_americanwine_1)

		//recursive
		if(!int_value_stack.push(v_americanwine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2980)) return -1;
		if (!q27_set.contains(v_wine_1)) continue;
	case 1967:
	// Continuation for label 954
			v_americanwine_1 = v_wine_1;
		if(americanwine_set.insert(v_americanwine_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1967)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(955)) return -1;
			if(!backtrack_stack.push(953)) return -1;
			goto label_944;
		}
		break;
	case 2980:
	// Label to restore values of label 954
		v_americanwine_1 = int_value_stack.pop();
		break;
	case 955: label_955: 
	// using  symbolic fact americanwine(v_americanwine_1) with literal 1 in rule 
	// 316: americanwine(A) :- americanwine(B), kaon2equal(A,B).
	// resulting in symbolic fact americanwine(v_americanwine_1)

		//recursive
		c316_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_americanwine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2981)) return -1;
		c316_2_fb.open(v_americanwine_1);
	case 1968:
	// Continuation for label 955
		while (c316_2_fb.fetch()) {
			v_americanwine_1 = c316_2_fb.c2();
		if(americanwine_set.insert(v_americanwine_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1968)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(955)) return -1;
			if(!backtrack_stack.push(953)) return -1;
			goto label_944;
		}
		}
		break;
	case 2981:
	// Label to restore values of label 955
		v_americanwine_1 = int_value_stack.pop();
		c316_2_fb.pop(&cursor_stack);
		break;
	case 956:
	// applying rule 
	// 711: ot____nom26(A) :- ot____nom26_aux(A).
	// resulting in symbolic fact ot____nom26(v_ot____nom26_1)
		c711_1_f.open();
	case 1969:
	// Continuation for label 956
		while (c711_1_f.fetch()) {
			v_ot____nom26_1 = c711_1_f.c1();
		if(ot____nom26_set.insert(v_ot____nom26_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1969)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(247)) return -1;
			goto label_246;
		}
		}
		break;
	case 957:
	// applying rule 
	// 717: ot____nom4(A) :- ot____nom4_aux(A).
	// resulting in symbolic fact ot____nom4(v_ot____nom4_1)
		c717_1_f.open();
	case 1970:
	// Continuation for label 957
		while (c717_1_f.fetch()) {
			v_ot____nom4_1 = c717_1_f.c1();
		if(ot____nom4_set.insert(v_ot____nom4_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1970)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(959)) return -1;
			goto label_958;
		}
		}
		break;
	case 958: label_958: 
	// using  symbolic fact ot____nom4(v_ot____nom4_1) with literal 1 in rule 
	// 720: ot____nom4(A) :- ot____nom4(B), kaon2equal(A,B).
	// resulting in symbolic fact ot____nom4(v_ot____nom4_1)

		//recursive
		c720_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_ot____nom4_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2984)) return -1;
		c720_2_fb.open(v_ot____nom4_1);
	case 1971:
	// Continuation for label 958
		while (c720_2_fb.fetch()) {
			v_ot____nom4_1 = c720_2_fb.c2();
		if(ot____nom4_set.insert(v_ot____nom4_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1971)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(959)) return -1;
			goto label_958;
		}
		}
		break;
	case 2984:
	// Label to restore values of label 958
		v_ot____nom4_1 = int_value_stack.pop();
		c720_2_fb.pop(&cursor_stack);
		break;
	case 959: label_959: 
	// using  symbolic fact ot____nom4(v_ot____nom4_1) with literal 2 in rule 
	// 716: q10(A) :- locatedin(A,B), ot____nom4(B).
	// resulting in symbolic fact q10(v_q10_1)

		//recursive
		c716_1_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q10_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2985)) return -1;
		c716_1_fb.open(v_ot____nom4_1);
	case 1972:
	// Continuation for label 959
		while (c716_1_fb.fetch()) {
			v_q10_1 = c716_1_fb.c2();
		if(q10_set.insert(v_q10_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1972)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(966)) return -1;
			if(!backtrack_stack.push(965)) return -1;
			goto label_960;
		}
		}
		break;
	case 2985:
	// Label to restore values of label 959
		v_q10_1 = int_value_stack.pop();
		c716_1_fb.pop(&cursor_stack);
		break;
	case 960: label_960: 
	// using  symbolic fact q10(v_q10_1) with literal 1 in rule 
	// 159: tours(A) :- q10(A), loire(A).
	// resulting in symbolic fact tours(v_tours_1)

		//recursive
		if(!int_value_stack.push(v_tours_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2986)) return -1;
		if (!loire_set.contains(v_q10_1)) continue;
	case 1973:
	// Continuation for label 960
			v_tours_1 = v_q10_1;
		if(tours_set.insert(v_tours_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1973)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(974)) return -1;
			if(!backtrack_stack.push(973)) return -1;
			if(!backtrack_stack.push(972)) return -1;
			if(!backtrack_stack.push(970)) return -1;
			goto label_961;
		}
		break;
	case 2986:
	// Label to restore values of label 960
		v_tours_1 = int_value_stack.pop();
		break;
	case 961: label_961: 
	// using  symbolic fact tours(v_tours_1) with literal 1 in rule 
	// 158: q9(A) :- tours(A).
	// resulting in symbolic fact q9(v_q9_1)

		//recursive
		if(!int_value_stack.push(v_q9_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2987)) return -1;
		v_q9_1 = v_tours_1;
		if(q9_set.insert(v_q9_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(968)) return -1;
		if(!backtrack_stack.push(967)) return -1;
		if(!backtrack_stack.push(964)) return -1;
		goto label_962;
		}
		break;
	case 2987:
	// Label to restore values of label 961
		v_q9_1 = int_value_stack.pop();
		break;
	case 962: label_962: 
	// using  symbolic fact q9(v_q9_1) with literal 1 in rule 
	// 157: locatedin(A,A) :- q9(A), kaon2namedobjects(A).
	// resulting in symbolic fact locatedin(v_locatedin_1,v_locatedin_2)

		//recursive
		if(!int_value_stack.push(v_locatedin_2)) return -1;
		if(!int_value_stack.push(v_locatedin_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2988)) return -1;
		if (!kaon2namedobjects_b.contains(v_q9_1)) continue;
	case 1975:
	// Continuation for label 962
			v_locatedin_1 = v_q9_1;
			v_locatedin_2 = v_q9_1;
		if(locatedin_set.insert(v_locatedin_1, v_locatedin_2)) {
			if(!locatedin_bf.insert(v_locatedin_1, v_locatedin_2)) return -1;
			if(!locatedin_fb.insert(v_locatedin_2, v_locatedin_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1975)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(1005)) return -1;
			if(!backtrack_stack.push(999)) return -1;
			if(!backtrack_stack.push(987)) return -1;
			if(!backtrack_stack.push(981)) return -1;
			if(!backtrack_stack.push(969)) return -1;
			if(!backtrack_stack.push(963)) return -1;
			if(!backtrack_stack.push(952)) return -1;
			if(!backtrack_stack.push(946)) return -1;
			if(!backtrack_stack.push(936)) return -1;
			if(!backtrack_stack.push(930)) return -1;
			if(!backtrack_stack.push(920)) return -1;
			if(!backtrack_stack.push(917)) return -1;
			if(!backtrack_stack.push(898)) return -1;
			if(!backtrack_stack.push(897)) return -1;
			if(!backtrack_stack.push(896)) return -1;
			if(!backtrack_stack.push(893)) return -1;
			if(!backtrack_stack.push(726)) return -1;
			if(!backtrack_stack.push(721)) return -1;
			if(!backtrack_stack.push(694)) return -1;
			if(!backtrack_stack.push(693)) return -1;
			if(!backtrack_stack.push(688)) return -1;
			if(!backtrack_stack.push(687)) return -1;
			if(!backtrack_stack.push(636)) return -1;
			if(!backtrack_stack.push(631)) return -1;
			if(!backtrack_stack.push(592)) return -1;
			if(!backtrack_stack.push(591)) return -1;
			if(!backtrack_stack.push(529)) return -1;
			if(!backtrack_stack.push(528)) return -1;
			if(!backtrack_stack.push(513)) return -1;
			if(!backtrack_stack.push(512)) return -1;
			if(!backtrack_stack.push(511)) return -1;
			if(!backtrack_stack.push(506)) return -1;
			if(!backtrack_stack.push(455)) return -1;
			if(!backtrack_stack.push(450)) return -1;
			if(!backtrack_stack.push(441)) return -1;
			if(!backtrack_stack.push(440)) return -1;
			if(!backtrack_stack.push(439)) return -1;
			if(!backtrack_stack.push(434)) return -1;
			if(!backtrack_stack.push(339)) return -1;
			if(!backtrack_stack.push(338)) return -1;
			if(!backtrack_stack.push(249)) return -1;
			if(!backtrack_stack.push(244)) return -1;
			if(!backtrack_stack.push(52)) return -1;
			if(!backtrack_stack.push(51)) return -1;
			if(!backtrack_stack.push(50)) return -1;
			if(!backtrack_stack.push(49)) return -1;
			if(!backtrack_stack.push(48)) return -1;
			goto label_47;
		}
		break;
	case 2988:
	// Label to restore values of label 962
		v_locatedin_1 = int_value_stack.pop();
		v_locatedin_2 = int_value_stack.pop();
		break;
	case 963: label_963: 
	// using  symbolic fact locatedin(v_locatedin_1,v_locatedin_2) with literal 1 in rule 
	// 161: q9(A) :- locatedin(A,B), q10(B).
	// resulting in symbolic fact q9(v_q9_1)

		//recursive
		if(!int_value_stack.push(v_q9_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2989)) return -1;
		if (!q10_set.contains(v_locatedin_2)) continue;
	case 1976:
	// Continuation for label 963
			v_q9_1 = v_locatedin_1;
		if(q9_set.insert(v_q9_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1976)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(968)) return -1;
			if(!backtrack_stack.push(967)) return -1;
			if(!backtrack_stack.push(964)) return -1;
			goto label_962;
		}
		break;
	case 2989:
	// Label to restore values of label 963
		v_q9_1 = int_value_stack.pop();
		break;
	case 964: label_964: 
	// using  symbolic fact q9(v_q9_1) with literal 1 in rule 
	// 160: q10(A) :- q9(A).
	// resulting in symbolic fact q10(v_q10_1)

		//recursive
		if(!int_value_stack.push(v_q10_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2990)) return -1;
		v_q10_1 = v_q9_1;
		if(q10_set.insert(v_q10_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(966)) return -1;
		if(!backtrack_stack.push(965)) return -1;
		goto label_960;
		}
		break;
	case 2990:
	// Label to restore values of label 964
		v_q10_1 = int_value_stack.pop();
		break;
	case 965: label_965: 
	// using  symbolic fact q10(v_q10_1) with literal 1 in rule 
	// 721: q10(A) :- q10(B), kaon2equal(A,B).
	// resulting in symbolic fact q10(v_q10_1)

		//recursive
		c721_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q10_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2991)) return -1;
		c721_2_fb.open(v_q10_1);
	case 1978:
	// Continuation for label 965
		while (c721_2_fb.fetch()) {
			v_q10_1 = c721_2_fb.c2();
		if(q10_set.insert(v_q10_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1978)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(966)) return -1;
			if(!backtrack_stack.push(965)) return -1;
			goto label_960;
		}
		}
		break;
	case 2991:
	// Label to restore values of label 965
		v_q10_1 = int_value_stack.pop();
		c721_2_fb.pop(&cursor_stack);
		break;
	case 966: label_966: 
	// using  symbolic fact q10(v_q10_1) with literal 2 in rule 
	// 161: q9(A) :- locatedin(A,B), q10(B).
	// resulting in symbolic fact q9(v_q9_1)

		//recursive
		c161_1_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q9_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2992)) return -1;
		c161_1_fb.open(v_q10_1);
	case 1979:
	// Continuation for label 966
		while (c161_1_fb.fetch()) {
			v_q9_1 = c161_1_fb.c2();
		if(q9_set.insert(v_q9_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1979)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(968)) return -1;
			if(!backtrack_stack.push(967)) return -1;
			if(!backtrack_stack.push(964)) return -1;
			goto label_962;
		}
		}
		break;
	case 2992:
	// Label to restore values of label 966
		v_q9_1 = int_value_stack.pop();
		c161_1_fb.pop(&cursor_stack);
		break;
	case 967: label_967: 
	// using  symbolic fact q9(v_q9_1) with literal 1 in rule 
	// 718: ot____nom4(A) :- q9(A), kaon2namedobjects(A).
	// resulting in symbolic fact ot____nom4(v_ot____nom4_1)

		//recursive
		if(!int_value_stack.push(v_ot____nom4_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2993)) return -1;
		if (!kaon2namedobjects_b.contains(v_q9_1)) continue;
	case 1980:
	// Continuation for label 967
			v_ot____nom4_1 = v_q9_1;
		if(ot____nom4_set.insert(v_ot____nom4_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1980)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(959)) return -1;
			goto label_958;
		}
		break;
	case 2993:
	// Label to restore values of label 967
		v_ot____nom4_1 = int_value_stack.pop();
		break;
	case 968: label_968: 
	// using  symbolic fact q9(v_q9_1) with literal 1 in rule 
	// 719: q9(A) :- q9(B), kaon2equal(A,B).
	// resulting in symbolic fact q9(v_q9_1)

		//recursive
		c719_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q9_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2994)) return -1;
		c719_2_fb.open(v_q9_1);
	case 1981:
	// Continuation for label 968
		while (c719_2_fb.fetch()) {
			v_q9_1 = c719_2_fb.c2();
		if(q9_set.insert(v_q9_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1981)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(968)) return -1;
			if(!backtrack_stack.push(967)) return -1;
			if(!backtrack_stack.push(964)) return -1;
			goto label_962;
		}
		}
		break;
	case 2994:
	// Label to restore values of label 968
		v_q9_1 = int_value_stack.pop();
		c719_2_fb.pop(&cursor_stack);
		break;
	case 969: label_969: 
	// using  symbolic fact locatedin(v_locatedin_1,v_locatedin_2) with literal 1 in rule 
	// 716: q10(A) :- locatedin(A,B), ot____nom4(B).
	// resulting in symbolic fact q10(v_q10_1)

		//recursive
		if(!int_value_stack.push(v_q10_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2995)) return -1;
		if (!ot____nom4_set.contains(v_locatedin_2)) continue;
	case 1982:
	// Continuation for label 969
			v_q10_1 = v_locatedin_1;
		if(q10_set.insert(v_q10_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1982)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(966)) return -1;
			if(!backtrack_stack.push(965)) return -1;
			goto label_960;
		}
		break;
	case 2995:
	// Label to restore values of label 969
		v_q10_1 = int_value_stack.pop();
		break;
	case 970: label_970: 
	// using  symbolic fact tours(v_tours_1) with literal 1 in rule 
	// 377: loire(A) :- tours(A).
	// resulting in symbolic fact loire(v_loire_1)

		//recursive
		if(!int_value_stack.push(v_loire_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2996)) return -1;
		v_loire_1 = v_tours_1;
		if(loire_set.insert(v_loire_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(971)) return -1;
		if(!backtrack_stack.push(715)) return -1;
		if(!backtrack_stack.push(714)) return -1;
		if(!backtrack_stack.push(487)) return -1;
		if(!backtrack_stack.push(486)) return -1;
		if(!backtrack_stack.push(484)) return -1;
		if(!backtrack_stack.push(482)) return -1;
		goto label_478;
		}
		break;
	case 2996:
	// Label to restore values of label 970
		v_loire_1 = int_value_stack.pop();
		break;
	case 971: label_971: 
	// using  symbolic fact loire(v_loire_1) with literal 2 in rule 
	// 159: tours(A) :- q10(A), loire(A).
	// resulting in symbolic fact tours(v_tours_1)

		//recursive
		if(!int_value_stack.push(v_tours_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2997)) return -1;
		if (!q10_set.contains(v_loire_1)) continue;
	case 1984:
	// Continuation for label 971
			v_tours_1 = v_loire_1;
		if(tours_set.insert(v_tours_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1984)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(974)) return -1;
			if(!backtrack_stack.push(973)) return -1;
			if(!backtrack_stack.push(972)) return -1;
			if(!backtrack_stack.push(970)) return -1;
			goto label_961;
		}
		break;
	case 2997:
	// Label to restore values of label 971
		v_tours_1 = int_value_stack.pop();
		break;
	case 972: label_972: 
	// using  symbolic fact tours(v_tours_1) with literal 1 in rule 
	// 378: tours(A) :- tours(B), kaon2equal(A,B).
	// resulting in symbolic fact tours(v_tours_1)

		//recursive
		c378_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_tours_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(2998)) return -1;
		c378_2_fb.open(v_tours_1);
	case 1985:
	// Continuation for label 972
		while (c378_2_fb.fetch()) {
			v_tours_1 = c378_2_fb.c2();
		if(tours_set.insert(v_tours_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1985)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(974)) return -1;
			if(!backtrack_stack.push(973)) return -1;
			if(!backtrack_stack.push(972)) return -1;
			if(!backtrack_stack.push(970)) return -1;
			goto label_961;
		}
		}
		break;
	case 2998:
	// Label to restore values of label 972
		v_tours_1 = int_value_stack.pop();
		c378_2_fb.pop(&cursor_stack);
		break;
	case 973: label_973: 
	// using  symbolic fact tours(v_tours_1) with literal 1 in rule 
	// 467: q6(A) :- tours(A).
	// resulting in symbolic fact q6(v_q6_1)

		v_q6_1 = v_tours_1;
		if(q6_set.insert(v_q6_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(146)) return -1;
		if(!backtrack_stack.push(145)) return -1;
		if(!backtrack_stack.push(130)) return -1;
		if(!backtrack_stack.push(129)) return -1;
		if(!backtrack_stack.push(128)) return -1;
		goto label_92;
		}
		break;
	case 974: label_974: 
	// using  symbolic fact tours(v_tours_1) with literal 1 in rule 
	// 769: q3(A) :- tours(A).
	// resulting in symbolic fact q3(v_q3_1)

		v_q3_1 = v_tours_1;
		if(q3_set.insert(v_q3_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(143)) return -1;
		goto label_123;
		}
		break;
	case 975:
	// applying rule 
	// 723: ot____nom17(A) :- ot____nom17_aux(A).
	// resulting in symbolic fact ot____nom17(v_ot____nom17_1)
		c723_1_f.open();
	case 1988:
	// Continuation for label 975
		while (c723_1_f.fetch()) {
			v_ot____nom17_1 = c723_1_f.c1();
		if(ot____nom17_set.insert(v_ot____nom17_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1988)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(977)) return -1;
			goto label_976;
		}
		}
		break;
	case 976: label_976: 
	// using  symbolic fact ot____nom17(v_ot____nom17_1) with literal 1 in rule 
	// 726: ot____nom17(A) :- ot____nom17(B), kaon2equal(A,B).
	// resulting in symbolic fact ot____nom17(v_ot____nom17_1)

		//recursive
		c726_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_ot____nom17_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(3002)) return -1;
		c726_2_fb.open(v_ot____nom17_1);
	case 1989:
	// Continuation for label 976
		while (c726_2_fb.fetch()) {
			v_ot____nom17_1 = c726_2_fb.c2();
		if(ot____nom17_set.insert(v_ot____nom17_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1989)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(977)) return -1;
			goto label_976;
		}
		}
		break;
	case 3002:
	// Label to restore values of label 976
		v_ot____nom17_1 = int_value_stack.pop();
		c726_2_fb.pop(&cursor_stack);
		break;
	case 977: label_977: 
	// using  symbolic fact ot____nom17(v_ot____nom17_1) with literal 2 in rule 
	// 722: q50(A) :- locatedin(A,B), ot____nom17(B).
	// resulting in symbolic fact q50(v_q50_1)

		//recursive
		c722_1_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q50_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(3003)) return -1;
		c722_1_fb.open(v_ot____nom17_1);
	case 1990:
	// Continuation for label 977
		while (c722_1_fb.fetch()) {
			v_q50_1 = c722_1_fb.c2();
		if(q50_set.insert(v_q50_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1990)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(984)) return -1;
			if(!backtrack_stack.push(983)) return -1;
			goto label_978;
		}
		}
		break;
	case 3003:
	// Label to restore values of label 977
		v_q50_1 = int_value_stack.pop();
		c722_1_fb.pop(&cursor_stack);
		break;
	case 978: label_978: 
	// using  symbolic fact q50(v_q50_1) with literal 1 in rule 
	// 152: germanwine(A) :- q50(A), wine(A).
	// resulting in symbolic fact germanwine(v_germanwine_1)

		//recursive
		if(!int_value_stack.push(v_germanwine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(3004)) return -1;
		if (!wine_set.contains(v_q50_1)) continue;
	case 1991:
	// Continuation for label 978
			v_germanwine_1 = v_q50_1;
		if(germanwine_set.insert(v_germanwine_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1991)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(990)) return -1;
			if(!backtrack_stack.push(988)) return -1;
			goto label_979;
		}
		break;
	case 3004:
	// Label to restore values of label 978
		v_germanwine_1 = int_value_stack.pop();
		break;
	case 979: label_979: 
	// using  symbolic fact germanwine(v_germanwine_1) with literal 1 in rule 
	// 151: q49(A) :- germanwine(A).
	// resulting in symbolic fact q49(v_q49_1)

		//recursive
		if(!int_value_stack.push(v_q49_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(3005)) return -1;
		v_q49_1 = v_germanwine_1;
		if(q49_set.insert(v_q49_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(986)) return -1;
		if(!backtrack_stack.push(985)) return -1;
		if(!backtrack_stack.push(982)) return -1;
		goto label_980;
		}
		break;
	case 3005:
	// Label to restore values of label 979
		v_q49_1 = int_value_stack.pop();
		break;
	case 980: label_980: 
	// using  symbolic fact q49(v_q49_1) with literal 1 in rule 
	// 150: locatedin(A,A) :- q49(A), kaon2namedobjects(A).
	// resulting in symbolic fact locatedin(v_locatedin_1,v_locatedin_2)

		//recursive
		if(!int_value_stack.push(v_locatedin_2)) return -1;
		if(!int_value_stack.push(v_locatedin_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(3006)) return -1;
		if (!kaon2namedobjects_b.contains(v_q49_1)) continue;
	case 1993:
	// Continuation for label 980
			v_locatedin_1 = v_q49_1;
			v_locatedin_2 = v_q49_1;
		if(locatedin_set.insert(v_locatedin_1, v_locatedin_2)) {
			if(!locatedin_bf.insert(v_locatedin_1, v_locatedin_2)) return -1;
			if(!locatedin_fb.insert(v_locatedin_2, v_locatedin_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1993)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(1005)) return -1;
			if(!backtrack_stack.push(999)) return -1;
			if(!backtrack_stack.push(987)) return -1;
			if(!backtrack_stack.push(981)) return -1;
			if(!backtrack_stack.push(969)) return -1;
			if(!backtrack_stack.push(963)) return -1;
			if(!backtrack_stack.push(952)) return -1;
			if(!backtrack_stack.push(946)) return -1;
			if(!backtrack_stack.push(936)) return -1;
			if(!backtrack_stack.push(930)) return -1;
			if(!backtrack_stack.push(920)) return -1;
			if(!backtrack_stack.push(917)) return -1;
			if(!backtrack_stack.push(898)) return -1;
			if(!backtrack_stack.push(897)) return -1;
			if(!backtrack_stack.push(896)) return -1;
			if(!backtrack_stack.push(893)) return -1;
			if(!backtrack_stack.push(726)) return -1;
			if(!backtrack_stack.push(721)) return -1;
			if(!backtrack_stack.push(694)) return -1;
			if(!backtrack_stack.push(693)) return -1;
			if(!backtrack_stack.push(688)) return -1;
			if(!backtrack_stack.push(687)) return -1;
			if(!backtrack_stack.push(636)) return -1;
			if(!backtrack_stack.push(631)) return -1;
			if(!backtrack_stack.push(592)) return -1;
			if(!backtrack_stack.push(591)) return -1;
			if(!backtrack_stack.push(529)) return -1;
			if(!backtrack_stack.push(528)) return -1;
			if(!backtrack_stack.push(513)) return -1;
			if(!backtrack_stack.push(512)) return -1;
			if(!backtrack_stack.push(511)) return -1;
			if(!backtrack_stack.push(506)) return -1;
			if(!backtrack_stack.push(455)) return -1;
			if(!backtrack_stack.push(450)) return -1;
			if(!backtrack_stack.push(441)) return -1;
			if(!backtrack_stack.push(440)) return -1;
			if(!backtrack_stack.push(439)) return -1;
			if(!backtrack_stack.push(434)) return -1;
			if(!backtrack_stack.push(339)) return -1;
			if(!backtrack_stack.push(338)) return -1;
			if(!backtrack_stack.push(249)) return -1;
			if(!backtrack_stack.push(244)) return -1;
			if(!backtrack_stack.push(52)) return -1;
			if(!backtrack_stack.push(51)) return -1;
			if(!backtrack_stack.push(50)) return -1;
			if(!backtrack_stack.push(49)) return -1;
			if(!backtrack_stack.push(48)) return -1;
			goto label_47;
		}
		break;
	case 3006:
	// Label to restore values of label 980
		v_locatedin_1 = int_value_stack.pop();
		v_locatedin_2 = int_value_stack.pop();
		break;
	case 981: label_981: 
	// using  symbolic fact locatedin(v_locatedin_1,v_locatedin_2) with literal 1 in rule 
	// 154: q49(A) :- locatedin(A,B), q50(B).
	// resulting in symbolic fact q49(v_q49_1)

		//recursive
		if(!int_value_stack.push(v_q49_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(3007)) return -1;
		if (!q50_set.contains(v_locatedin_2)) continue;
	case 1994:
	// Continuation for label 981
			v_q49_1 = v_locatedin_1;
		if(q49_set.insert(v_q49_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1994)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(986)) return -1;
			if(!backtrack_stack.push(985)) return -1;
			if(!backtrack_stack.push(982)) return -1;
			goto label_980;
		}
		break;
	case 3007:
	// Label to restore values of label 981
		v_q49_1 = int_value_stack.pop();
		break;
	case 982: label_982: 
	// using  symbolic fact q49(v_q49_1) with literal 1 in rule 
	// 153: q50(A) :- q49(A).
	// resulting in symbolic fact q50(v_q50_1)

		//recursive
		if(!int_value_stack.push(v_q50_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(3008)) return -1;
		v_q50_1 = v_q49_1;
		if(q50_set.insert(v_q50_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(984)) return -1;
		if(!backtrack_stack.push(983)) return -1;
		goto label_978;
		}
		break;
	case 3008:
	// Label to restore values of label 982
		v_q50_1 = int_value_stack.pop();
		break;
	case 983: label_983: 
	// using  symbolic fact q50(v_q50_1) with literal 1 in rule 
	// 727: q50(A) :- q50(B), kaon2equal(A,B).
	// resulting in symbolic fact q50(v_q50_1)

		//recursive
		c727_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q50_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(3009)) return -1;
		c727_2_fb.open(v_q50_1);
	case 1996:
	// Continuation for label 983
		while (c727_2_fb.fetch()) {
			v_q50_1 = c727_2_fb.c2();
		if(q50_set.insert(v_q50_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1996)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(984)) return -1;
			if(!backtrack_stack.push(983)) return -1;
			goto label_978;
		}
		}
		break;
	case 3009:
	// Label to restore values of label 983
		v_q50_1 = int_value_stack.pop();
		c727_2_fb.pop(&cursor_stack);
		break;
	case 984: label_984: 
	// using  symbolic fact q50(v_q50_1) with literal 2 in rule 
	// 154: q49(A) :- locatedin(A,B), q50(B).
	// resulting in symbolic fact q49(v_q49_1)

		//recursive
		c154_1_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q49_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(3010)) return -1;
		c154_1_fb.open(v_q50_1);
	case 1997:
	// Continuation for label 984
		while (c154_1_fb.fetch()) {
			v_q49_1 = c154_1_fb.c2();
		if(q49_set.insert(v_q49_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1997)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(986)) return -1;
			if(!backtrack_stack.push(985)) return -1;
			if(!backtrack_stack.push(982)) return -1;
			goto label_980;
		}
		}
		break;
	case 3010:
	// Label to restore values of label 984
		v_q49_1 = int_value_stack.pop();
		c154_1_fb.pop(&cursor_stack);
		break;
	case 985: label_985: 
	// using  symbolic fact q49(v_q49_1) with literal 1 in rule 
	// 724: ot____nom17(A) :- q49(A), kaon2namedobjects(A).
	// resulting in symbolic fact ot____nom17(v_ot____nom17_1)

		//recursive
		if(!int_value_stack.push(v_ot____nom17_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(3011)) return -1;
		if (!kaon2namedobjects_b.contains(v_q49_1)) continue;
	case 1998:
	// Continuation for label 985
			v_ot____nom17_1 = v_q49_1;
		if(ot____nom17_set.insert(v_ot____nom17_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1998)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(977)) return -1;
			goto label_976;
		}
		break;
	case 3011:
	// Label to restore values of label 985
		v_ot____nom17_1 = int_value_stack.pop();
		break;
	case 986: label_986: 
	// using  symbolic fact q49(v_q49_1) with literal 1 in rule 
	// 725: q49(A) :- q49(B), kaon2equal(A,B).
	// resulting in symbolic fact q49(v_q49_1)

		//recursive
		c725_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q49_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(3012)) return -1;
		c725_2_fb.open(v_q49_1);
	case 1999:
	// Continuation for label 986
		while (c725_2_fb.fetch()) {
			v_q49_1 = c725_2_fb.c2();
		if(q49_set.insert(v_q49_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(1999)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(986)) return -1;
			if(!backtrack_stack.push(985)) return -1;
			if(!backtrack_stack.push(982)) return -1;
			goto label_980;
		}
		}
		break;
	case 3012:
	// Label to restore values of label 986
		v_q49_1 = int_value_stack.pop();
		c725_2_fb.pop(&cursor_stack);
		break;
	case 987: label_987: 
	// using  symbolic fact locatedin(v_locatedin_1,v_locatedin_2) with literal 1 in rule 
	// 722: q50(A) :- locatedin(A,B), ot____nom17(B).
	// resulting in symbolic fact q50(v_q50_1)

		//recursive
		if(!int_value_stack.push(v_q50_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(3013)) return -1;
		if (!ot____nom17_set.contains(v_locatedin_2)) continue;
	case 2000:
	// Continuation for label 987
			v_q50_1 = v_locatedin_1;
		if(q50_set.insert(v_q50_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(2000)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(984)) return -1;
			if(!backtrack_stack.push(983)) return -1;
			goto label_978;
		}
		break;
	case 3013:
	// Label to restore values of label 987
		v_q50_1 = int_value_stack.pop();
		break;
	case 988: label_988: 
	// using  symbolic fact germanwine(v_germanwine_1) with literal 1 in rule 
	// 473: wine(A) :- germanwine(A).
	// resulting in symbolic fact wine(v_wine_1)

		//recursive
		if(!int_value_stack.push(v_wine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(3014)) return -1;
		v_wine_1 = v_germanwine_1;
		if(wine_set.insert(v_wine_1)) {
		if(!wine_f.insert(v_wine_1)) return -1;
		// Labels for next rules
		if(!backtrack_stack.push(1007)) return -1;
		if(!backtrack_stack.push(989)) return -1;
		if(!backtrack_stack.push(954)) return -1;
		if(!backtrack_stack.push(938)) return -1;
		if(!backtrack_stack.push(911)) return -1;
		if(!backtrack_stack.push(903)) return -1;
		if(!backtrack_stack.push(902)) return -1;
		if(!backtrack_stack.push(901)) return -1;
		if(!backtrack_stack.push(849)) return -1;
		if(!backtrack_stack.push(848)) return -1;
		if(!backtrack_stack.push(821)) return -1;
		if(!backtrack_stack.push(820)) return -1;
		if(!backtrack_stack.push(737)) return -1;
		if(!backtrack_stack.push(712)) return -1;
		if(!backtrack_stack.push(674)) return -1;
		if(!backtrack_stack.push(672)) return -1;
		if(!backtrack_stack.push(583)) return -1;
		if(!backtrack_stack.push(575)) return -1;
		if(!backtrack_stack.push(563)) return -1;
		if(!backtrack_stack.push(551)) return -1;
		if(!backtrack_stack.push(549)) return -1;
		if(!backtrack_stack.push(483)) return -1;
		if(!backtrack_stack.push(473)) return -1;
		if(!backtrack_stack.push(470)) return -1;
		if(!backtrack_stack.push(430)) return -1;
		if(!backtrack_stack.push(401)) return -1;
		if(!backtrack_stack.push(399)) return -1;
		if(!backtrack_stack.push(380)) return -1;
		if(!backtrack_stack.push(364)) return -1;
		if(!backtrack_stack.push(363)) return -1;
		if(!backtrack_stack.push(329)) return -1;
		if(!backtrack_stack.push(328)) return -1;
		if(!backtrack_stack.push(327)) return -1;
		if(!backtrack_stack.push(295)) return -1;
		if(!backtrack_stack.push(294)) return -1;
		if(!backtrack_stack.push(216)) return -1;
		if(!backtrack_stack.push(210)) return -1;
		if(!backtrack_stack.push(192)) return -1;
		if(!backtrack_stack.push(180)) return -1;
		if(!backtrack_stack.push(179)) return -1;
		if(!backtrack_stack.push(178)) return -1;
		if(!backtrack_stack.push(154)) return -1;
		if(!backtrack_stack.push(153)) return -1;
		if(!backtrack_stack.push(100)) return -1;
		if(!backtrack_stack.push(76)) return -1;
		if(!backtrack_stack.push(75)) return -1;
		if(!backtrack_stack.push(67)) return -1;
		if(!backtrack_stack.push(63)) return -1;
		if(!backtrack_stack.push(53)) return -1;
		if(!backtrack_stack.push(46)) return -1;
		if(!backtrack_stack.push(19)) return -1;
		goto label_16;
		}
		break;
	case 3014:
	// Label to restore values of label 988
		v_wine_1 = int_value_stack.pop();
		break;
	case 989: label_989: 
	// using  symbolic fact wine(v_wine_1) with literal 2 in rule 
	// 152: germanwine(A) :- q50(A), wine(A).
	// resulting in symbolic fact germanwine(v_germanwine_1)

		//recursive
		if(!int_value_stack.push(v_germanwine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(3015)) return -1;
		if (!q50_set.contains(v_wine_1)) continue;
	case 2002:
	// Continuation for label 989
			v_germanwine_1 = v_wine_1;
		if(germanwine_set.insert(v_germanwine_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(2002)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(990)) return -1;
			if(!backtrack_stack.push(988)) return -1;
			goto label_979;
		}
		break;
	case 3015:
	// Label to restore values of label 989
		v_germanwine_1 = int_value_stack.pop();
		break;
	case 990: label_990: 
	// using  symbolic fact germanwine(v_germanwine_1) with literal 1 in rule 
	// 474: germanwine(A) :- germanwine(B), kaon2equal(A,B).
	// resulting in symbolic fact germanwine(v_germanwine_1)

		//recursive
		c474_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_germanwine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(3016)) return -1;
		c474_2_fb.open(v_germanwine_1);
	case 2003:
	// Continuation for label 990
		while (c474_2_fb.fetch()) {
			v_germanwine_1 = c474_2_fb.c2();
		if(germanwine_set.insert(v_germanwine_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(2003)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(990)) return -1;
			if(!backtrack_stack.push(988)) return -1;
			goto label_979;
		}
		}
		break;
	case 3016:
	// Label to restore values of label 990
		v_germanwine_1 = int_value_stack.pop();
		c474_2_fb.pop(&cursor_stack);
		break;
	case 991:
	// applying rule 
	// 729: ot____nom14(A) :- ot____nom14_aux(A).
	// resulting in symbolic fact ot____nom14(v_ot____nom14_1)
		c729_1_f.open();
	case 2004:
	// Continuation for label 991
		while (c729_1_f.fetch()) {
			v_ot____nom14_1 = c729_1_f.c1();
		if(ot____nom14_set.insert(v_ot____nom14_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(2004)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(725)) return -1;
			goto label_724;
		}
		}
		break;
	case 992:
	// applying rule 
	// 734: ot____nom46(A) :- ot____nom46_aux(A).
	// resulting in symbolic fact ot____nom46(v_ot____nom46_1)
		c734_1_f.open();
	case 2005:
	// Continuation for label 992
		while (c734_1_f.fetch()) {
			v_ot____nom46_1 = c734_1_f.c1();
		if(ot____nom46_set.insert(v_ot____nom46_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(2005)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(692)) return -1;
			goto label_691;
		}
		}
		break;
	case 993:
	// applying rule 
	// 739: ot____nom13(A) :- ot____nom13_aux(A).
	// resulting in symbolic fact ot____nom13(v_ot____nom13_1)
		c739_1_f.open();
	case 2006:
	// Continuation for label 993
		while (c739_1_f.fetch()) {
			v_ot____nom13_1 = c739_1_f.c1();
		if(ot____nom13_set.insert(v_ot____nom13_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(2006)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(995)) return -1;
			goto label_994;
		}
		}
		break;
	case 994: label_994: 
	// using  symbolic fact ot____nom13(v_ot____nom13_1) with literal 1 in rule 
	// 742: ot____nom13(A) :- ot____nom13(B), kaon2equal(A,B).
	// resulting in symbolic fact ot____nom13(v_ot____nom13_1)

		//recursive
		c742_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_ot____nom13_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(3020)) return -1;
		c742_2_fb.open(v_ot____nom13_1);
	case 2007:
	// Continuation for label 994
		while (c742_2_fb.fetch()) {
			v_ot____nom13_1 = c742_2_fb.c2();
		if(ot____nom13_set.insert(v_ot____nom13_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(2007)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(995)) return -1;
			goto label_994;
		}
		}
		break;
	case 3020:
	// Label to restore values of label 994
		v_ot____nom13_1 = int_value_stack.pop();
		c742_2_fb.pop(&cursor_stack);
		break;
	case 995: label_995: 
	// using  symbolic fact ot____nom13(v_ot____nom13_1) with literal 2 in rule 
	// 738: q60(A) :- locatedin(A,B), ot____nom13(B).
	// resulting in symbolic fact q60(v_q60_1)

		//recursive
		c738_1_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q60_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(3021)) return -1;
		c738_1_fb.open(v_ot____nom13_1);
	case 2008:
	// Continuation for label 995
		while (c738_1_fb.fetch()) {
			v_q60_1 = c738_1_fb.c2();
		if(q60_set.insert(v_q60_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(2008)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(1002)) return -1;
			if(!backtrack_stack.push(1001)) return -1;
			goto label_996;
		}
		}
		break;
	case 3021:
	// Label to restore values of label 995
		v_q60_1 = int_value_stack.pop();
		c738_1_fb.pop(&cursor_stack);
		break;
	case 996: label_996: 
	// using  symbolic fact q60(v_q60_1) with literal 1 in rule 
	// 122: alsatianwine(A) :- q60(A), wine(A).
	// resulting in symbolic fact alsatianwine(v_alsatianwine_1)

		//recursive
		if(!int_value_stack.push(v_alsatianwine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(3022)) return -1;
		if (!wine_set.contains(v_q60_1)) continue;
	case 2009:
	// Continuation for label 996
			v_alsatianwine_1 = v_q60_1;
		if(alsatianwine_set.insert(v_alsatianwine_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(2009)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(1008)) return -1;
			if(!backtrack_stack.push(1006)) return -1;
			goto label_997;
		}
		break;
	case 3022:
	// Label to restore values of label 996
		v_alsatianwine_1 = int_value_stack.pop();
		break;
	case 997: label_997: 
	// using  symbolic fact alsatianwine(v_alsatianwine_1) with literal 1 in rule 
	// 121: q59(A) :- alsatianwine(A).
	// resulting in symbolic fact q59(v_q59_1)

		//recursive
		if(!int_value_stack.push(v_q59_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(3023)) return -1;
		v_q59_1 = v_alsatianwine_1;
		if(q59_set.insert(v_q59_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(1004)) return -1;
		if(!backtrack_stack.push(1003)) return -1;
		if(!backtrack_stack.push(1000)) return -1;
		goto label_998;
		}
		break;
	case 3023:
	// Label to restore values of label 997
		v_q59_1 = int_value_stack.pop();
		break;
	case 998: label_998: 
	// using  symbolic fact q59(v_q59_1) with literal 1 in rule 
	// 120: locatedin(A,A) :- q59(A), kaon2namedobjects(A).
	// resulting in symbolic fact locatedin(v_locatedin_1,v_locatedin_2)

		//recursive
		if(!int_value_stack.push(v_locatedin_2)) return -1;
		if(!int_value_stack.push(v_locatedin_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(3024)) return -1;
		if (!kaon2namedobjects_b.contains(v_q59_1)) continue;
	case 2011:
	// Continuation for label 998
			v_locatedin_1 = v_q59_1;
			v_locatedin_2 = v_q59_1;
		if(locatedin_set.insert(v_locatedin_1, v_locatedin_2)) {
			if(!locatedin_bf.insert(v_locatedin_1, v_locatedin_2)) return -1;
			if(!locatedin_fb.insert(v_locatedin_2, v_locatedin_1)) return -1;
			// Push backtrack label to continuation
			if(!backtrack_stack.push(2011)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(1005)) return -1;
			if(!backtrack_stack.push(999)) return -1;
			if(!backtrack_stack.push(987)) return -1;
			if(!backtrack_stack.push(981)) return -1;
			if(!backtrack_stack.push(969)) return -1;
			if(!backtrack_stack.push(963)) return -1;
			if(!backtrack_stack.push(952)) return -1;
			if(!backtrack_stack.push(946)) return -1;
			if(!backtrack_stack.push(936)) return -1;
			if(!backtrack_stack.push(930)) return -1;
			if(!backtrack_stack.push(920)) return -1;
			if(!backtrack_stack.push(917)) return -1;
			if(!backtrack_stack.push(898)) return -1;
			if(!backtrack_stack.push(897)) return -1;
			if(!backtrack_stack.push(896)) return -1;
			if(!backtrack_stack.push(893)) return -1;
			if(!backtrack_stack.push(726)) return -1;
			if(!backtrack_stack.push(721)) return -1;
			if(!backtrack_stack.push(694)) return -1;
			if(!backtrack_stack.push(693)) return -1;
			if(!backtrack_stack.push(688)) return -1;
			if(!backtrack_stack.push(687)) return -1;
			if(!backtrack_stack.push(636)) return -1;
			if(!backtrack_stack.push(631)) return -1;
			if(!backtrack_stack.push(592)) return -1;
			if(!backtrack_stack.push(591)) return -1;
			if(!backtrack_stack.push(529)) return -1;
			if(!backtrack_stack.push(528)) return -1;
			if(!backtrack_stack.push(513)) return -1;
			if(!backtrack_stack.push(512)) return -1;
			if(!backtrack_stack.push(511)) return -1;
			if(!backtrack_stack.push(506)) return -1;
			if(!backtrack_stack.push(455)) return -1;
			if(!backtrack_stack.push(450)) return -1;
			if(!backtrack_stack.push(441)) return -1;
			if(!backtrack_stack.push(440)) return -1;
			if(!backtrack_stack.push(439)) return -1;
			if(!backtrack_stack.push(434)) return -1;
			if(!backtrack_stack.push(339)) return -1;
			if(!backtrack_stack.push(338)) return -1;
			if(!backtrack_stack.push(249)) return -1;
			if(!backtrack_stack.push(244)) return -1;
			if(!backtrack_stack.push(52)) return -1;
			if(!backtrack_stack.push(51)) return -1;
			if(!backtrack_stack.push(50)) return -1;
			if(!backtrack_stack.push(49)) return -1;
			if(!backtrack_stack.push(48)) return -1;
			goto label_47;
		}
		break;
	case 3024:
	// Label to restore values of label 998
		v_locatedin_1 = int_value_stack.pop();
		v_locatedin_2 = int_value_stack.pop();
		break;
	case 999: label_999: 
	// using  symbolic fact locatedin(v_locatedin_1,v_locatedin_2) with literal 1 in rule 
	// 124: q59(A) :- locatedin(A,B), q60(B).
	// resulting in symbolic fact q59(v_q59_1)

		//recursive
		if(!int_value_stack.push(v_q59_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(3025)) return -1;
		if (!q60_set.contains(v_locatedin_2)) continue;
	case 2012:
	// Continuation for label 999
			v_q59_1 = v_locatedin_1;
		if(q59_set.insert(v_q59_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(2012)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(1004)) return -1;
			if(!backtrack_stack.push(1003)) return -1;
			if(!backtrack_stack.push(1000)) return -1;
			goto label_998;
		}
		break;
	case 3025:
	// Label to restore values of label 999
		v_q59_1 = int_value_stack.pop();
		break;
	case 1000: label_1000: 
	// using  symbolic fact q59(v_q59_1) with literal 1 in rule 
	// 123: q60(A) :- q59(A).
	// resulting in symbolic fact q60(v_q60_1)

		//recursive
		if(!int_value_stack.push(v_q60_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(3026)) return -1;
		v_q60_1 = v_q59_1;
		if(q60_set.insert(v_q60_1)) {
		// Labels for next rules
		if(!backtrack_stack.push(1002)) return -1;
		if(!backtrack_stack.push(1001)) return -1;
		goto label_996;
		}
		break;
	case 3026:
	// Label to restore values of label 1000
		v_q60_1 = int_value_stack.pop();
		break;
	case 1001: label_1001: 
	// using  symbolic fact q60(v_q60_1) with literal 1 in rule 
	// 743: q60(A) :- q60(B), kaon2equal(A,B).
	// resulting in symbolic fact q60(v_q60_1)

		//recursive
		c743_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q60_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(3027)) return -1;
		c743_2_fb.open(v_q60_1);
	case 2014:
	// Continuation for label 1001
		while (c743_2_fb.fetch()) {
			v_q60_1 = c743_2_fb.c2();
		if(q60_set.insert(v_q60_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(2014)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(1002)) return -1;
			if(!backtrack_stack.push(1001)) return -1;
			goto label_996;
		}
		}
		break;
	case 3027:
	// Label to restore values of label 1001
		v_q60_1 = int_value_stack.pop();
		c743_2_fb.pop(&cursor_stack);
		break;
	case 1002: label_1002: 
	// using  symbolic fact q60(v_q60_1) with literal 2 in rule 
	// 124: q59(A) :- locatedin(A,B), q60(B).
	// resulting in symbolic fact q59(v_q59_1)

		//recursive
		c124_1_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q59_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(3028)) return -1;
		c124_1_fb.open(v_q60_1);
	case 2015:
	// Continuation for label 1002
		while (c124_1_fb.fetch()) {
			v_q59_1 = c124_1_fb.c2();
		if(q59_set.insert(v_q59_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(2015)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(1004)) return -1;
			if(!backtrack_stack.push(1003)) return -1;
			if(!backtrack_stack.push(1000)) return -1;
			goto label_998;
		}
		}
		break;
	case 3028:
	// Label to restore values of label 1002
		v_q59_1 = int_value_stack.pop();
		c124_1_fb.pop(&cursor_stack);
		break;
	case 1003: label_1003: 
	// using  symbolic fact q59(v_q59_1) with literal 1 in rule 
	// 740: ot____nom13(A) :- q59(A), kaon2namedobjects(A).
	// resulting in symbolic fact ot____nom13(v_ot____nom13_1)

		//recursive
		if(!int_value_stack.push(v_ot____nom13_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(3029)) return -1;
		if (!kaon2namedobjects_b.contains(v_q59_1)) continue;
	case 2016:
	// Continuation for label 1003
			v_ot____nom13_1 = v_q59_1;
		if(ot____nom13_set.insert(v_ot____nom13_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(2016)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(995)) return -1;
			goto label_994;
		}
		break;
	case 3029:
	// Label to restore values of label 1003
		v_ot____nom13_1 = int_value_stack.pop();
		break;
	case 1004: label_1004: 
	// using  symbolic fact q59(v_q59_1) with literal 1 in rule 
	// 741: q59(A) :- q59(B), kaon2equal(A,B).
	// resulting in symbolic fact q59(v_q59_1)

		//recursive
		c741_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_q59_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(3030)) return -1;
		c741_2_fb.open(v_q59_1);
	case 2017:
	// Continuation for label 1004
		while (c741_2_fb.fetch()) {
			v_q59_1 = c741_2_fb.c2();
		if(q59_set.insert(v_q59_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(2017)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(1004)) return -1;
			if(!backtrack_stack.push(1003)) return -1;
			if(!backtrack_stack.push(1000)) return -1;
			goto label_998;
		}
		}
		break;
	case 3030:
	// Label to restore values of label 1004
		v_q59_1 = int_value_stack.pop();
		c741_2_fb.pop(&cursor_stack);
		break;
	case 1005: label_1005: 
	// using  symbolic fact locatedin(v_locatedin_1,v_locatedin_2) with literal 1 in rule 
	// 738: q60(A) :- locatedin(A,B), ot____nom13(B).
	// resulting in symbolic fact q60(v_q60_1)

		//recursive
		if(!int_value_stack.push(v_q60_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(3031)) return -1;
		if (!ot____nom13_set.contains(v_locatedin_2)) continue;
	case 2018:
	// Continuation for label 1005
			v_q60_1 = v_locatedin_1;
		if(q60_set.insert(v_q60_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(2018)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(1002)) return -1;
			if(!backtrack_stack.push(1001)) return -1;
			goto label_996;
		}
		break;
	case 3031:
	// Label to restore values of label 1005
		v_q60_1 = int_value_stack.pop();
		break;
	case 1006: label_1006: 
	// using  symbolic fact alsatianwine(v_alsatianwine_1) with literal 1 in rule 
	// 317: wine(A) :- alsatianwine(A).
	// resulting in symbolic fact wine(v_wine_1)

		//recursive
		if(!int_value_stack.push(v_wine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(3032)) return -1;
		v_wine_1 = v_alsatianwine_1;
		if(wine_set.insert(v_wine_1)) {
		if(!wine_f.insert(v_wine_1)) return -1;
		// Labels for next rules
		if(!backtrack_stack.push(1007)) return -1;
		if(!backtrack_stack.push(989)) return -1;
		if(!backtrack_stack.push(954)) return -1;
		if(!backtrack_stack.push(938)) return -1;
		if(!backtrack_stack.push(911)) return -1;
		if(!backtrack_stack.push(903)) return -1;
		if(!backtrack_stack.push(902)) return -1;
		if(!backtrack_stack.push(901)) return -1;
		if(!backtrack_stack.push(849)) return -1;
		if(!backtrack_stack.push(848)) return -1;
		if(!backtrack_stack.push(821)) return -1;
		if(!backtrack_stack.push(820)) return -1;
		if(!backtrack_stack.push(737)) return -1;
		if(!backtrack_stack.push(712)) return -1;
		if(!backtrack_stack.push(674)) return -1;
		if(!backtrack_stack.push(672)) return -1;
		if(!backtrack_stack.push(583)) return -1;
		if(!backtrack_stack.push(575)) return -1;
		if(!backtrack_stack.push(563)) return -1;
		if(!backtrack_stack.push(551)) return -1;
		if(!backtrack_stack.push(549)) return -1;
		if(!backtrack_stack.push(483)) return -1;
		if(!backtrack_stack.push(473)) return -1;
		if(!backtrack_stack.push(470)) return -1;
		if(!backtrack_stack.push(430)) return -1;
		if(!backtrack_stack.push(401)) return -1;
		if(!backtrack_stack.push(399)) return -1;
		if(!backtrack_stack.push(380)) return -1;
		if(!backtrack_stack.push(364)) return -1;
		if(!backtrack_stack.push(363)) return -1;
		if(!backtrack_stack.push(329)) return -1;
		if(!backtrack_stack.push(328)) return -1;
		if(!backtrack_stack.push(327)) return -1;
		if(!backtrack_stack.push(295)) return -1;
		if(!backtrack_stack.push(294)) return -1;
		if(!backtrack_stack.push(216)) return -1;
		if(!backtrack_stack.push(210)) return -1;
		if(!backtrack_stack.push(192)) return -1;
		if(!backtrack_stack.push(180)) return -1;
		if(!backtrack_stack.push(179)) return -1;
		if(!backtrack_stack.push(178)) return -1;
		if(!backtrack_stack.push(154)) return -1;
		if(!backtrack_stack.push(153)) return -1;
		if(!backtrack_stack.push(100)) return -1;
		if(!backtrack_stack.push(76)) return -1;
		if(!backtrack_stack.push(75)) return -1;
		if(!backtrack_stack.push(67)) return -1;
		if(!backtrack_stack.push(63)) return -1;
		if(!backtrack_stack.push(53)) return -1;
		if(!backtrack_stack.push(46)) return -1;
		if(!backtrack_stack.push(19)) return -1;
		goto label_16;
		}
		break;
	case 3032:
	// Label to restore values of label 1006
		v_wine_1 = int_value_stack.pop();
		break;
	case 1007: label_1007: 
	// using  symbolic fact wine(v_wine_1) with literal 2 in rule 
	// 122: alsatianwine(A) :- q60(A), wine(A).
	// resulting in symbolic fact alsatianwine(v_alsatianwine_1)

		//recursive
		if(!int_value_stack.push(v_alsatianwine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(3033)) return -1;
		if (!q60_set.contains(v_wine_1)) continue;
	case 2020:
	// Continuation for label 1007
			v_alsatianwine_1 = v_wine_1;
		if(alsatianwine_set.insert(v_alsatianwine_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(2020)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(1008)) return -1;
			if(!backtrack_stack.push(1006)) return -1;
			goto label_997;
		}
		break;
	case 3033:
	// Label to restore values of label 1007
		v_alsatianwine_1 = int_value_stack.pop();
		break;
	case 1008: label_1008: 
	// using  symbolic fact alsatianwine(v_alsatianwine_1) with literal 1 in rule 
	// 318: alsatianwine(A) :- alsatianwine(B), kaon2equal(A,B).
	// resulting in symbolic fact alsatianwine(v_alsatianwine_1)

		//recursive
		c318_2_fb.push(&cursor_stack);
		if(!int_value_stack.push(v_alsatianwine_1)) return -1;
		// Push backtrack label to restore values
		if(!backtrack_stack.push(3034)) return -1;
		c318_2_fb.open(v_alsatianwine_1);
	case 2021:
	// Continuation for label 1008
		while (c318_2_fb.fetch()) {
			v_alsatianwine_1 = c318_2_fb.c2();
		if(alsatianwine_set.insert(v_alsatianwine_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(2021)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(1008)) return -1;
			if(!backtrack_stack.push(1006)) return -1;
			goto label_997;
		}
		}
		break;
	case 3034:
	// Label to restore values of label 1008
		v_alsatianwine_1 = int_value_stack.pop();
		c318_2_fb.pop(&cursor_stack);
		break;
	case 1009:
	// applying rule 
	// 745: ot____nom47(A) :- ot____nom47_aux(A).
	// resulting in symbolic fact ot____nom47(v_ot____nom47_1)
		c745_1_f.open();
	case 2022:
	// Continuation for label 1009
		while (c745_1_f.fetch()) {
			v_ot____nom47_1 = c745_1_f.c1();
		if(ot____nom47_set.insert(v_ot____nom47_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(2022)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(335)) return -1;
			goto label_334;
		}
		}
		break;
	case 1010:
	// applying rule 
	// 751: ot____nom42(A) :- ot____nom42_aux(A).
	// resulting in symbolic fact ot____nom42(v_ot____nom42_1)
		c751_1_f.open();
	case 2023:
	// Continuation for label 1010
		while (c751_1_f.fetch()) {
			v_ot____nom42_1 = c751_1_f.c1();
		if(ot____nom42_set.insert(v_ot____nom42_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(2023)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(650)) return -1;
			goto label_649;
		}
		}
		break;
	case 1011:
	// applying rule 
	// 757: ot____nom30(A) :- ot____nom30_aux(A).
	// resulting in symbolic fact ot____nom30(v_ot____nom30_1)
		c757_1_f.open();
	case 2024:
	// Continuation for label 1011
		while (c757_1_f.fetch()) {
			v_ot____nom30_1 = c757_1_f.c1();
		if(ot____nom30_set.insert(v_ot____nom30_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(2024)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(635)) return -1;
			goto label_634;
		}
		}
		break;
	case 1012:
	// applying rule 
	// 762: ot____nom22(A) :- ot____nom22_aux(A).
	// resulting in symbolic fact ot____nom22(v_ot____nom22_1)
		c762_1_f.open();
	case 2025:
	// Continuation for label 1012
		while (c762_1_f.fetch()) {
			v_ot____nom22_1 = c762_1_f.c1();
		if(ot____nom22_set.insert(v_ot____nom22_1)) {
			// Push backtrack label to continuation
			if(!backtrack_stack.push(2025)) return -1;
			// Labels for next rules
			if(!backtrack_stack.push(510)) return -1;
			goto label_509;
		}
		}
		break;
	}
}

return num_results;
// #endif
//END_GENERATED_CODE

	// Return number of result tuples:
	return 0;
}


//-----------------------------------------------------------------------------
// file: List of data files.
//-----------------------------------------------------------------------------

str_t bench_wine_c::file(int i) const
{
	switch(i) {
		case 0:
			return VER_FILE_WINE;
		default:
			return STR_NULL;
	}
}


//-----------------------------------------------------------------------------
// result: List of correct results of eval method.
//-----------------------------------------------------------------------------

long bench_wine_c::result(int i) const
{
	switch(i) {
		case 0: return -1;
		default: return -1;
	}
}


//=============================================================================
// Debugging Support:
//=============================================================================


//-----------------------------------------------------------------------------
// check: Check the integrity of this object, return error message or NULL.
//-----------------------------------------------------------------------------

#ifdef	VER_DEBUG
str_t	bench_wine_c::check() const
{
	// Check magic number:
	if(Magic != BENCH_WINE_MAGIC)
		return "wrong magic number";

	// Check sub-objects (domain):
	CHECK_SUBOBJ(dom, "bench_wine_c::check");

	// Check sub-objects (relations for EDB predicates):
	CHECK_SUBOBJ(kaon2namedobjects_f, "bench_wine_c::check");
	CHECK_SUBOBJ(hasflavor_aux_ff, "bench_wine_c::check");
	CHECK_SUBOBJ(kaon2namedobjects_b, "bench_wine_c::check");
	CHECK_SUBOBJ(cheninblanc_aux_f, "bench_wine_c::check");
	CHECK_SUBOBJ(beaujolais_aux_f, "bench_wine_c::check");
	CHECK_SUBOBJ(hassugar_aux_ff, "bench_wine_c::check");
	CHECK_SUBOBJ(hasvintageyear_aux_ff, "bench_wine_c::check");
	CHECK_SUBOBJ(hascolor_aux_ff, "bench_wine_c::check");
	CHECK_SUBOBJ(meritage_aux_f, "bench_wine_c::check");
	CHECK_SUBOBJ(ot____nom8_aux_f, "bench_wine_c::check");
	CHECK_SUBOBJ(madefromgrape_aux_ff, "bench_wine_c::check");
	CHECK_SUBOBJ(riesling_aux_f, "bench_wine_c::check");
	CHECK_SUBOBJ(dryriesling_aux_f, "bench_wine_c::check");
	CHECK_SUBOBJ(chianti_aux_f, "bench_wine_c::check");
	CHECK_SUBOBJ(hasmaker_aux_ff, "bench_wine_c::check");
	CHECK_SUBOBJ(chardonnay_aux_f, "bench_wine_c::check");
	CHECK_SUBOBJ(whiteburgundy_aux_f, "bench_wine_c::check");
	CHECK_SUBOBJ(meursault_aux_f, "bench_wine_c::check");
	CHECK_SUBOBJ(hasbody_aux_ff, "bench_wine_c::check");
	CHECK_SUBOBJ(muscadet_aux_f, "bench_wine_c::check");
	CHECK_SUBOBJ(margaux_aux_f, "bench_wine_c::check");
	CHECK_SUBOBJ(locatedin_aux_ff, "bench_wine_c::check");
	CHECK_SUBOBJ(sancerre_aux_f, "bench_wine_c::check");
	CHECK_SUBOBJ(cotesdor_aux_f, "bench_wine_c::check");
	CHECK_SUBOBJ(pauillac_aux_f, "bench_wine_c::check");
	CHECK_SUBOBJ(anjou_aux_f, "bench_wine_c::check");
	CHECK_SUBOBJ(port_aux_f, "bench_wine_c::check");
	CHECK_SUBOBJ(sauternes_aux_f, "bench_wine_c::check");
	CHECK_SUBOBJ(stemilion_aux_f, "bench_wine_c::check");
	CHECK_SUBOBJ(ot____nom36_aux_f, "bench_wine_c::check");
	CHECK_SUBOBJ(cabernetsauvignon_aux_f, "bench_wine_c::check");
	CHECK_SUBOBJ(icewine_aux_f, "bench_wine_c::check");
	CHECK_SUBOBJ(merlot_aux_f, "bench_wine_c::check");
	CHECK_SUBOBJ(sweetriesling_aux_f, "bench_wine_c::check");
	CHECK_SUBOBJ(whitewine_aux_f, "bench_wine_c::check");
	CHECK_SUBOBJ(ot____nom23_aux_f, "bench_wine_c::check");
	CHECK_SUBOBJ(pinotnoir_aux_f, "bench_wine_c::check");
	CHECK_SUBOBJ(cabernetfranc_aux_f, "bench_wine_c::check");
	CHECK_SUBOBJ(semillon_aux_f, "bench_wine_c::check");
	CHECK_SUBOBJ(sauvignonblanc_aux_f, "bench_wine_c::check");
	CHECK_SUBOBJ(ot____nom6_aux_f, "bench_wine_c::check");
	CHECK_SUBOBJ(ot____nom19_aux_f, "bench_wine_c::check");
	CHECK_SUBOBJ(petitesyrah_aux_f, "bench_wine_c::check");
	CHECK_SUBOBJ(zinfandel_aux_f, "bench_wine_c::check");
	CHECK_SUBOBJ(redtablewine_aux_f, "bench_wine_c::check");
	CHECK_SUBOBJ(dessertwine_aux_f, "bench_wine_c::check");
	CHECK_SUBOBJ(ot____nom3_aux_f, "bench_wine_c::check");
	CHECK_SUBOBJ(ot____nom43_aux_f, "bench_wine_c::check");
	CHECK_SUBOBJ(ot____nom24_aux_f, "bench_wine_c::check");
	CHECK_SUBOBJ(ot____nom11_aux_f, "bench_wine_c::check");
	CHECK_SUBOBJ(ot____nom34_aux_f, "bench_wine_c::check");
	CHECK_SUBOBJ(ot____nom37_aux_f, "bench_wine_c::check");
	CHECK_SUBOBJ(ot____nom20_aux_f, "bench_wine_c::check");
	CHECK_SUBOBJ(ot____nom33_aux_f, "bench_wine_c::check");
	CHECK_SUBOBJ(ot____nom39_aux_f, "bench_wine_c::check");
	CHECK_SUBOBJ(ot____nom27_aux_f, "bench_wine_c::check");
	CHECK_SUBOBJ(ot____nom25_aux_f, "bench_wine_c::check");
	CHECK_SUBOBJ(ot____nom21_aux_f, "bench_wine_c::check");
	CHECK_SUBOBJ(ot____nom29_aux_f, "bench_wine_c::check");
	CHECK_SUBOBJ(ot____nom7_aux_f, "bench_wine_c::check");
	CHECK_SUBOBJ(ot____nom28_aux_f, "bench_wine_c::check");
	CHECK_SUBOBJ(ot____nom45_aux_f, "bench_wine_c::check");
	CHECK_SUBOBJ(ot____nom1_aux_f, "bench_wine_c::check");
	CHECK_SUBOBJ(ot____nom32_aux_f, "bench_wine_c::check");
	CHECK_SUBOBJ(ot____nom5_aux_f, "bench_wine_c::check");
	CHECK_SUBOBJ(ot____nom12_aux_f, "bench_wine_c::check");
	CHECK_SUBOBJ(ot____nom15_aux_f, "bench_wine_c::check");
	CHECK_SUBOBJ(ot____nom2_aux_f, "bench_wine_c::check");
	CHECK_SUBOBJ(ot____nom44_aux_f, "bench_wine_c::check");
	CHECK_SUBOBJ(ot____nom10_aux_f, "bench_wine_c::check");
	CHECK_SUBOBJ(ot____nom31_aux_f, "bench_wine_c::check");
	CHECK_SUBOBJ(ot____nom16_aux_f, "bench_wine_c::check");
	CHECK_SUBOBJ(ot____nom41_aux_f, "bench_wine_c::check");
	CHECK_SUBOBJ(ot____nom38_aux_f, "bench_wine_c::check");
	CHECK_SUBOBJ(ot____nom26_aux_f, "bench_wine_c::check");
	CHECK_SUBOBJ(ot____nom4_aux_f, "bench_wine_c::check");
	CHECK_SUBOBJ(ot____nom17_aux_f, "bench_wine_c::check");
	CHECK_SUBOBJ(ot____nom14_aux_f, "bench_wine_c::check");
	CHECK_SUBOBJ(ot____nom46_aux_f, "bench_wine_c::check");
	CHECK_SUBOBJ(ot____nom13_aux_f, "bench_wine_c::check");
	CHECK_SUBOBJ(ot____nom47_aux_f, "bench_wine_c::check");
	CHECK_SUBOBJ(ot____nom42_aux_f, "bench_wine_c::check");
	CHECK_SUBOBJ(ot____nom30_aux_f, "bench_wine_c::check");
	CHECK_SUBOBJ(ot____nom22_aux_f, "bench_wine_c::check");

	// Ok:
	return 0;
}

#endif


//-----------------------------------------------------------------------------
// dump: Show data structure.
//-----------------------------------------------------------------------------

#ifdef	VER_DUMP
void bench_wine_c::dump(str_t headline) const
{
	// Headline:
	if(headline == STR_NULL)
		headline = "Evaluation of Wine Ontology Example";
	err_c::dump_begin(headline);

	// Basic information:
	err_c::dump_str("ObjectSize:  ");
	err_c::dump_uint(static_cast<unsigned int>(sizeof(bench_wine_c)));
	err_c::dump_str(" Bytes");
	err_c::dump_nl();

	err_c::dump_nl();

	// Show entire hash table with chains of entries:
	err_c::dump_str("String Table (Domain):");
	err_c::dump_nl();
	for(int i = 0; i < dom.num_strings(); i++) {
		err_c::dump_str("  ID ");
		err_c::dump_str(str_fill(i, 3));
		err_c::dump_int(i);
		err_c::dump_str(": ");
		str_t const_name = dom.get(i);
		if(const_name == STR_NULL)
			err_c::dump_str("*** NULL ***");
		else {
			err_c::dump_str("'");
			err_c::dump_str(const_name);
			err_c::dump_str("'");
		}
		err_c::dump_nl();
	}
	err_c::dump_nl();

	// Show data for relation kaon2namedobjects_f:
	err_c::dump_str("Relation kaon2namedobjects_f:");
	err_c::dump_nl();
	cur_1_c kaon2namedobjects_c(&kaon2namedobjects_f);
	kaon2namedobjects_c.open();
	int i = 1;
	while(kaon2namedobjects_c.fetch()) {
		err_c::dump_str("  ROW ");
		err_c::dump_str(str_fill(i, 3));
		err_c::dump_int(i);
		err_c::dump_str(": (");
		int c1 = kaon2namedobjects_c.c1();
		str_t const_name = dom.get(c1);
		if(const_name == STR_NULL)
			err_c::dump_str("*** NULL ***");
		else {
			err_c::dump_str("'");
			err_c::dump_str(const_name);
			err_c::dump_str("'");
		}
		err_c::dump_str(")");
		err_c::dump_nl();
		i++;
	}
	kaon2namedobjects_c.close();
	err_c::dump_nl();

	// Done:
	err_c::dump_end();
}

// End of #ifdef VER_DUMP:
#endif

// End of #ifdef VER_WINE_ENABLED:
#endif

